# -*- coding: utf-8 -*-
# pylint: disable=bare-except,chained-comparison,invalid-name,len-as-condition,line-too-long,protected-access,redefined-outer-name,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unneeded-not,unused-import
u"""
Interfaces for the NAG Mark 27.0 `dot` Chapter.

``dot`` - Inner Products

Base interfaces.

We do not intend for these lower-level base interfaces to be called
directly from user code. Please refer to the ``naginterfaces.library``
subpackage instead.

This module is concerned with the calculation of innerproducts required
by other functions within the NAG Library.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/x03/x03intro.html
"""

# NAG Copyright 2017-2019.

import collections as _collections
import sys as _sys

from ..base import utils

def real_prec(a, isizea, b, isizeb, n, istepa, istepb, c1, c2, sw):
    u"""
    Real inner product added to initial value, basic/additional
    precision.

    For full information please refer to the NAG Library document for
    x03aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/x03/x03aaf.html

    Parameters
    ----------
    a : float, shape (`isizea`)
        The elements of the first vector.

        The ith vector element is stored in the array element
        `a`[(i-1)*`istepa`].

        In your function from which ``real_prec`` is called, `a` can be
        part of a multidimensional array and the actual argument must be
        the array element containing the first vector element.

    isizea : int
        The upper bound for `isizea` is found by multiplying together
        the dimensions of `a` as declared in your function from which
        ``real_prec`` is called, subtracting the starting position and
        adding 1.

    b : float, shape (`isizeb`)
        The elements of the second vector.

        The ith vector element is stored in the array element
        `b`[(i-1)*`istepb`].

        In your function from which ``real_prec`` is called, `b` can be
        part of a multidimensional array and the actual argument must be
        the array element containing the first vector element.

    isizeb : int
        The upper bound for `isizeb` is found by multiplying together
        the dimensions of `b` as declared in your function from which
        ``real_prec`` is called, subtracting the starting position and
        adding 1.

    n : int
        n, the number of elements in the scalar product.

    istepa : int
        The step length between elements of the first vector in array
        `a`.

    istepb : int
        The step length between elements of the second vector in array
        `b`.

    c1 : float
        `c1` and `c2` must specify the initial value c: c = `c1`+`c2`.
        Normally, if c is in additional precision, `c1` specifies the
        most significant part and `c2` the least significant part; if c
        is in basic precision, then `c1` specifies c and `c2` must have
        the value 0.0. Both `c1` and `c2` must be defined on entry.

    c2 : float
        `c1` and `c2` must specify the initial value c: c = `c1`+`c2`.
        Normally, if c is in additional precision, `c1` specifies the
        most significant part and `c2` the least significant part; if c
        is in basic precision, then `c1` specifies c and `c2` must have
        the value 0.0. Both `c1` and `c2` must be defined on entry.

    sw : bool
        The precision to be used in the calculation.

        `sw` = True

          additional precision.

        `sw` = False

          basic precision.

    Returns
    -------
    d1 : float
        The result d.

        If the calculation is in additional precision (`sw` = True),

        - `d1` = d rounded to basic precision;

        - `d2` = d-`d1`,

        thus `d1` holds the correctly rounded basic precision result and
        the sum `d1`+`d2` gives the result in additional precision. `d2`
        may have the opposite sign to `d1`.

        If the calculation is in basic precision (`sw` = False),

        - `d1` = d;

        - `d2` = 0.0.

    d2 : float
        The result d.

        If the calculation is in additional precision (`sw` = True),

        - `d1` = d rounded to basic precision;

        - `d2` = d-`d1`,

        thus `d1` holds the correctly rounded basic precision result and
        the sum `d1`+`d2` gives the result in additional precision. `d2`
        may have the opposite sign to `d1`.

        If the calculation is in basic precision (`sw` = False),

        - `d1` = d;

        - `d2` = 0.0.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `istepb` = *<value>*.

            Constraint: `istepb` > 0.

        (`errno` 1)
            On entry, `istepa` = *<value>*.

            Constraint: `istepa` > 0.

        (`errno` 2)
            On entry, `isizeb` = *<value>*, `n` = *<value>* and `istepb`
            = *<value>*.

            Constraint: `isizeb` >= (`n`-1)*`istepb`+1.

        (`errno` 2)
            On entry, `isizea` = *<value>*, `n` = *<value>* and `istepa`
            = *<value>*.

            Constraint: `isizea` >= (`n`-1)*`istepa`+1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.dot.real_prec'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('x03aaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, a[isizea].
        utils._BYREF_INT, # Input, isizea.
        utils._EngineFloat64ArrayType.c_type, # Input, b[isizeb].
        utils._BYREF_INT, # Input, isizeb.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, istepa.
        utils._BYREF_INT, # Input, istepb.
        utils._BYREF_FLOAT64, # Input, c1.
        utils._BYREF_FLOAT64, # Input, c2.
        utils._BYREF_FLOAT64, # Output, d1.
        utils._BYREF_FLOAT64, # Output, d2.
        utils._BYREF_LOGICAL, # Input, sw.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _isizea = utils._EngineIntScalarType(
        isizea, {'fun_name': fun_name, 'entity': 'isizea'},
    ).to_ctypes()
    _isizea_py = utils._EngineIntScalarType.to_py(_isizea)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_isizea_py,),
    ).to_ctypes()
    _isizeb = utils._EngineIntScalarType(
        isizeb, {'fun_name': fun_name, 'entity': 'isizeb'},
    ).to_ctypes()
    _isizeb_py = utils._EngineIntScalarType.to_py(_isizeb)
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_isizeb_py,),
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _istepa = utils._EngineIntScalarType(
        istepa, {'fun_name': fun_name, 'entity': 'istepa'},
    ).to_ctypes()
    _istepb = utils._EngineIntScalarType(
        istepb, {'fun_name': fun_name, 'entity': 'istepb'},
    ).to_ctypes()
    _c1 = utils._EngineFloat64ScalarType(
        c1, {'fun_name': fun_name, 'entity': 'c1'},
    ).to_ctypes()
    _c2 = utils._EngineFloat64ScalarType(
        c2, {'fun_name': fun_name, 'entity': 'c2'},
    ).to_ctypes()
    _d1 = utils._EngineFloat64ScalarType.empty_ctypes()
    _d2 = utils._EngineFloat64ScalarType.empty_ctypes()
    _sw = utils._EngineLogicalScalarType(
        sw, {'fun_name': fun_name, 'entity': 'sw'},
    ).to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _a,
        _isizea,
        _b,
        _isizeb,
        _n,
        _istepa,
        _istepb,
        _c1,
        _c2,
        _d1,
        _d2,
        _sw,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, istepb = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: istepb > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, istepa = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: istepa > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, isizeb = {:s}, n = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'istepb = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: isizeb >= (n-1)*istepb+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, isizea = {:s}, n = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'istepa = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: isizea >= (n-1)*istepa+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _d1_py = utils._EngineFloat64ScalarType.to_py(_d1)
        _d2_py = utils._EngineFloat64ScalarType.to_py(_d2)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['d1', 'd2'],
        )(
            _d1_py,
            _d2_py,
        )
    _err_class.manage()
    return return_data

def complex_prec(a, isizea, b, isizeb, n, istepa, istepb, cx, sw):
    u"""
    Complex inner product added to initial value, basic/additional
    precision.

    For full information please refer to the NAG Library document for
    x03ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/x03/x03abf.html

    Parameters
    ----------
    a : complex, shape (`isizea`)
        The elements of the first vector.

        The ith vector element is stored in the array element
        `a`[(i-1)*`istepa`].

        In your function from which ``complex_prec`` is called, `a` can
        be part of a multidimensional array and the actual argument must
        be the array element containing the first vector element.

    isizea : int
        The upper bound for `isizea` is found by multiplying together
        the dimensions of `a` as declared in your function from which
        ``complex_prec`` is called, subtracting the starting position
        and adding 1.

    b : complex, shape (`isizeb`)
        The elements of the second vector.

        The ith vector element is stored in the array element
        `b`[(i-1)*`istepb`].

        In your function from which ``complex_prec`` is called, `b` can
        be part of a multidimensional array and the actual argument must
        be the array element containing the first vector element.

    isizeb : int
        The upper bound for `isizeb` is found by multiplying together
        the dimensions of `b` as declared in your function from which
        ``complex_prec`` is called, subtracting the starting position
        and adding 1.

    n : int
        n, the number of elements in the scalar product.

    istepa : int
        The step length between elements of the first vector in array
        `a`.

    istepb : int
        The step length between elements of the second vector in array
        `b`.

    cx : complex
        The initial value c.

    sw : bool
        The precision to be used in the calculation.

        `sw` = True

          additional precision.

        `sw` = False

          basic precision.

    Returns
    -------
    dx : complex
        The result d.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `istepb` = *<value>*.

            Constraint: `istepb` > 0.

        (`errno` 1)
            On entry, `istepa` = *<value>*.

            Constraint: `istepa` > 0.

        (`errno` 2)
            On entry, `isizeb` = *<value>*, `n` = *<value>* and `istepb`
            = *<value>*.

            Constraint: `isizeb` >= (`n`-1)*`istepb`+1.

        (`errno` 2)
            On entry, `isizea` = *<value>*, `n` = *<value>* and `istepa`
            = *<value>*.

            Constraint: `isizea` >= (`n`-1)*`istepa`+1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.dot.complex_prec'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('x03abft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._EngineComplex128ArrayType.c_type, # Input, a[isizea].
        utils._BYREF_INT, # Input, isizea.
        utils._EngineComplex128ArrayType.c_type, # Input, b[isizeb].
        utils._BYREF_INT, # Input, isizeb.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, istepa.
        utils._BYREF_INT, # Input, istepb.
        utils._BYREF_COMPLEX128, # Input, cx.
        utils._BYREF_COMPLEX128, # Output, dx.
        utils._BYREF_LOGICAL, # Input, sw.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _isizea = utils._EngineIntScalarType(
        isizea, {'fun_name': fun_name, 'entity': 'isizea'},
    ).to_ctypes()
    _isizea_py = utils._EngineIntScalarType.to_py(_isizea)
    _a = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_isizea_py,),
    ).to_ctypes()
    _isizeb = utils._EngineIntScalarType(
        isizeb, {'fun_name': fun_name, 'entity': 'isizeb'},
    ).to_ctypes()
    _isizeb_py = utils._EngineIntScalarType.to_py(_isizeb)
    _b = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_isizeb_py,),
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _istepa = utils._EngineIntScalarType(
        istepa, {'fun_name': fun_name, 'entity': 'istepa'},
    ).to_ctypes()
    _istepb = utils._EngineIntScalarType(
        istepb, {'fun_name': fun_name, 'entity': 'istepb'},
    ).to_ctypes()
    _cx = utils._EngineComplex128ScalarType(
        cx, {'fun_name': fun_name, 'entity': 'cx'},
    ).to_ctypes()
    _dx = utils._EngineComplex128ScalarType.empty_ctypes()
    _sw = utils._EngineLogicalScalarType(
        sw, {'fun_name': fun_name, 'entity': 'sw'},
    ).to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _a,
        _isizea,
        _b,
        _isizeb,
        _n,
        _istepa,
        _istepb,
        _cx,
        _dx,
        _sw,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, istepb = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: istepb > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, istepa = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: istepa > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, isizeb = {:s}, n = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'istepb = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: isizeb >= (n-1)*istepb+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, isizea = {:s}, n = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'istepa = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: isizea >= (n-1)*istepa+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _dx_py = utils._EngineComplex128ScalarType.to_py(_dx)
        return_data = _dx_py
    _err_class.manage()
    return return_data
