# -*- coding: utf-8 -*-
# pylint: disable=bare-except,chained-comparison,invalid-name,len-as-condition,line-too-long,protected-access,redefined-outer-name,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unneeded-not,unused-import
u"""
Interfaces for the NAG Mark 27.0 `surviv` Chapter.

``surviv`` - Survival Analysis

Base interfaces.

We do not intend for these lower-level base interfaces to be called
directly from user code. Please refer to the ``naginterfaces.library``
subpackage instead.

This module is concerned with statistical techniques used in the
analysis of survival/reliability/failure time data.

Other modules contain functions which are also used to analyse this type
of data. Submodule ``correg`` contains generalized linear models,
submodule ``univar`` contains functions to fit distribution models, and
submodule ``nonpar`` contains rank based methods.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g12/g12intro.html
"""

# NAG Copyright 2017-2019.

import collections as _collections
import sys as _sys

from ..base import utils

def kaplanmeier(n, t, ic, freq, ifreq, tp, p, psig):
    u"""
    Computes Kaplan--Meier (product-limit) estimates of survival
    probabilities.

    For full information please refer to the NAG Library document for
    g12aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g12/g12aaf.html

    Parameters
    ----------
    n : int
        The number of failure and censored times given in `t`.

    t : float, shape (`n`)
        The failure and censored times; these need not be ordered.

    ic : int, shape (`n`)
        `ic`[i-1] contains the censoring code of the ith observation,
        for i = 1,2,...,`n`.

        `ic`[i-1] = 0

          The ith observation is a failure time.

        `ic`[i-1] = 1

          The ith observation is right-censored.

    freq : str, length 1
        Indicates whether frequencies are provided for each time point.

        `freq` = 'F'

          Frequencies are provided for each failure and censored time.

        `freq` = 'S'

          The failure and censored times are considered as single
          observations, i.e., a frequency of 1 is assumed.

    ifreq : int, shape (:)
        Note: the required length for this argument is determined as
        follows: if `freq`='F': `n`; if `freq`='S': 1; otherwise: 0.

        If `freq` = 'F', `ifreq`[i] must contain the frequency of the
        ith observation.

        If `ifreq` = 'S', a frequency of 1 is assumed and `ifreq` is not
        referenced.

    tp : float, shape (`n`), modified in place
        On exit: `tp`[i-1] contains the ith ordered distinct failure
        time, t_i, for i = 1,2,...,n_d.

    p : float, shape (`n`), modified in place
        On exit: `p`[i-1] contains the Kaplan--Meier estimate of the
        survival probability, S^(t), for time `tp`[i-1], for i =
        1,2,...,n_d.

    psig : float, shape (`n`), modified in place
        On exit: `psig`[i-1] contains an estimate of the standard
        deviation of `p`[i-1], for i = 1,2,...,n_d.

    Returns
    -------
    nd : int
        The number of distinct failure times, n_d.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 2.

        (`errno` 2)
            On entry, `freq` = *<value>*.

            Constraint: `freq` = 'F' or 'S'.

        (`errno` 3)
            On entry, i = *<value>* and `ic`[i-1] = *<value>*.

            Constraint: `ic`[i-1] = 0 or 1.

        (`errno` 4)
            On entry, i = *<value>* and `ifreq`[i-1] = *<value>*.

            Constraint: `ifreq`[i-1] >= 0.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.surviv.kaplanmeier'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g12aaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, t[n].
        utils._EngineIntArrayType.c_type, # Input, ic[n].
        utils._BYREF_STRING, # Input, freq(len=1).
        utils._EngineIntArrayType.c_type, # Input, ifreq[*].
        utils._BYREF_INT, # Output, nd.
        utils._EngineFloat64ArrayType.c_type, # Output, tp[n].
        utils._EngineFloat64ArrayType.c_type, # Output, p[n].
        utils._EngineFloat64ArrayType.c_type, # Output, psig[n].
        utils._EngineIntArrayType.c_type, # Workspace, iwk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of freq.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _t = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ic = utils._EngineIntArrayType(
        ic, {'fun_name': fun_name, 'entity': 'ic'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _freq = utils._EngineCharScalarType(
        freq, {'fun_name': fun_name, 'entity': 'freq'}, exp_chrlen=1,
    ).to_ctypes()
    _freq_py = utils._EngineCharScalarType.to_py(_freq)
    if _freq_py in ('F', 'f'):
        _ifreq_dim1 = _n_py
    elif _freq_py in ('S', 's'):
        _ifreq_dim1 = 1
    else:
        _ifreq_dim1 = 0
    exp_shape_ifreq = (_ifreq_dim1,)
    _ifreq = utils._EngineIntArrayType(
        ifreq, {'fun_name': fun_name, 'entity': 'ifreq'},
        exp_shape=exp_shape_ifreq,
    ).to_ctypes()
    _nd = utils._EngineIntScalarType.empty_ctypes()
    _tp_cls = utils._EngineFloat64ArrayType(
        tp, {'fun_name': fun_name, 'entity': 'tp'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _tp = _tp_cls.to_ctypes(can_reference_contents=False)
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _psig_cls = utils._EngineFloat64ArrayType(
        psig, {'fun_name': fun_name, 'entity': 'psig'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _psig = _psig_cls.to_ctypes(can_reference_contents=False)
    _iwk = utils._EngineIntArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _t,
        _ic,
        _freq,
        _ifreq,
        _nd,
        _tp,
        _p,
        _psig,
        _iwk,
        _errbuf,
        _ierr,
        _freq._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, freq = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: freq = \'F\' or \'S\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and ic[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ic[i-1] = 0 or 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and ifreq[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ifreq[i-1] >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _tp_cls.refresh_py(_tp)
        _p_cls.refresh_py(_p)
        _psig_cls.refresh_py(_psig)
        _nd_py = utils._EngineIntScalarType.to_py(_nd)
        return_data = _nd_py
    _err_class.manage()
    return return_data

def logrank(n, t, ic, grp, ngrp, ifreq, wt, obsd, expt, di, ni, ldn):
    u"""
    Computes rank statistics for comparing survival curves.

    For full information please refer to the NAG Library document for
    g12ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g12/g12abf.html

    Parameters
    ----------
    n : int
        n, the number of failure and censored times.

    t : float, shape (`n`)
        The observed failure and censored times; these need not be
        ordered.

    ic : int, shape (`n`)
        `ic`[i-1] contains the censoring code of the ith observation,
        for i = 1,2,...,`n`.

        `ic`[i-1] = 0

          the ith observation is a failure time.

        `ic`[i-1] = 1

          the ith observation is right-censored.

    grp : int, shape (`n`)
        `grp`[i-1] contains a flag indicating which group the ith
        observation belongs in, for i = 1,2,...,`n`.

    ngrp : int
        g, the number of groups.

    ifreq : None or int, shape (:)
        Note: the required length for this argument is determined as
        follows: if `ifreq` is not None: `n`; otherwise: 0.

        Optionally, the frequency (number of observations) that each
        entry in `t` corresponds to. If `ifreq` is None then each entry
        in `t` is assumed to correspond to a single observation, i.e., a
        frequency of 1 is assumed.

    wt : None or float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `wt` is not None: `ldn`; otherwise: 0.

        Optionally, the n_d weights, w_i, where n_d is the number of
        distinct failure times. If `wt` is None then w_i = 1 for all i.

    obsd : float, shape (`ngrp`), modified in place
        On exit: O_i, the observed number of failures in each group.

    expt : float, shape (`ngrp`), modified in place
        On exit: E_i, the expected number of failures in each group.

    di : int, shape (`ldn`), modified in place
        On exit: the first `nd` elements of `di` contain d_i, the number
        of failures, across all groups, at time t_i.

    ni : int, shape (`ldn`), modified in place
        On exit: the first `nd` elements of `ni` contain n_i, the size
        of the risk set, across all groups, at time t_i.

    ldn : int
        The size of arrays `di` and `ni`. As n_d <= n, if n_d is not
        known a priori then a value of `n` can safely be used for `ldn`.

    Returns
    -------
    ts : float
        T, the test statistic.

    df : int
        nu, the degrees of freedom.

    p : float
        P(X >= T), when X~chi_nu^2, i.e., the probability associated
        with `ts`.

    nd : int
        n_d, the number of distinct failure times.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 2.

        (`errno` 2)
            On entry, all the times in `t` are the same.

        (`errno` 3)
            On entry, `ic`[*<value>*] = *<value>*.

            Constraint: `ic`[i-1] = 0 or 1.

        (`errno` 4)
            On entry, `grp`[*<value>*] = *<value>* and `ngrp` =
            *<value>*.

            Constraint: 1 <= `grp`[i-1] <= `ngrp`.

        (`errno` 5)
            On entry, `ngrp` = *<value>* and `n` = *<value>*.

            Constraint: 2 <= `ngrp` <= `n`.

        (`errno` 6)
            On entry, lfreq = *<value>* and `n` = *<value>*.

            Constraint: lfreq = 0 or lfreq >= `n`.

        (`errno` 7)
            On entry, `ifreq`[*<value>*] = *<value>*.

            Constraint: `ifreq`[i-1] >= 0.

        (`errno` 8)
            On entry, lwt = *<value>* and `ldn` = *<value>*.

            Constraint: lwt = 0 or lwt >= `ldn`.

        (`errno` 9)
            On entry, `wt`[*<value>*] = *<value>*.

            Constraint: `wt`[i-1] >= 0.0.

        (`errno` 11)
            The degrees of freedom are zero.

        (`errno` 18)
            On entry, `ldn` = *<value>*.

            Constraint: `ldn` >= *<value>*.

        (`errno` 31)
            On entry, all observations are censored.

        (`errno` 41)
            On entry, group *<value>* has no observations.
    """

    excs = []
    fun_name = 'naginterfaces.base.surviv.logrank'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g12abft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, t[n].
        utils._EngineIntArrayType.c_type, # Input, ic[n].
        utils._EngineIntArrayType.c_type, # Input, grp[n].
        utils._BYREF_INT, # Input, ngrp.
        utils._BYREF_INT, # Input, lfreq.
        utils._EngineIntArrayType.c_type, # Input, ifreq[lfreq].
        utils._BYREF_INT, # Input, lwt.
        utils._EngineFloat64ArrayType.c_type, # Input, wt[lwt].
        utils._BYREF_FLOAT64, # Output, ts.
        utils._BYREF_INT, # Output, df.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineFloat64ArrayType.c_type, # Output, obsd[ngrp].
        utils._EngineFloat64ArrayType.c_type, # Output, expt[ngrp].
        utils._BYREF_INT, # Output, nd.
        utils._EngineIntArrayType.c_type, # Output, di[ldn].
        utils._EngineIntArrayType.c_type, # Output, ni[ldn].
        utils._BYREF_INT, # Input, ldn.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _t = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ic = utils._EngineIntArrayType(
        ic, {'fun_name': fun_name, 'entity': 'ic'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _grp = utils._EngineIntArrayType(
        grp, {'fun_name': fun_name, 'entity': 'grp'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ngrp = utils._EngineIntScalarType(
        ngrp, {'fun_name': fun_name, 'entity': 'ngrp'},
    ).to_ctypes()
    _lfreq_py = utils._EngineIntArrayType.get_shape(ifreq, {'fun_name': fun_name, 'entity': 'ifreq'}, exp_rank=1, allow_none=True)[0]
    _lfreq = utils._EngineIntScalarType(
        _lfreq_py, {'fun_name': fun_name, 'entity': '_lfreq'},
    ).to_ctypes()
    if ifreq is not None:
        _ifreq_dim1 = _n_py
    else:
        _ifreq_dim1 = 0
    exp_shape_ifreq = (_ifreq_dim1,)
    _ifreq = utils._EngineIntArrayType(
        ifreq, {'fun_name': fun_name, 'entity': 'ifreq'},
        exp_shape=exp_shape_ifreq,
        allow_none=True,
    ).to_ctypes()
    _ldn = utils._EngineIntScalarType(
        ldn, {'fun_name': fun_name, 'entity': 'ldn'},
    ).to_ctypes()
    _lwt_py = utils._EngineFloat64ArrayType.get_shape(wt, {'fun_name': fun_name, 'entity': 'wt'}, exp_rank=1, allow_none=True)[0]
    _lwt = utils._EngineIntScalarType(
        _lwt_py, {'fun_name': fun_name, 'entity': '_lwt'},
    ).to_ctypes()
    _ldn_py = utils._EngineIntScalarType.to_py(_ldn)
    if wt is not None:
        _wt_dim1 = _ldn_py
    else:
        _wt_dim1 = 0
    exp_shape_wt = (_wt_dim1,)
    _wt = utils._EngineFloat64ArrayType(
        wt, {'fun_name': fun_name, 'entity': 'wt'},
        exp_shape=exp_shape_wt,
        allow_none=True,
    ).to_ctypes()
    _ts = utils._EngineFloat64ScalarType.empty_ctypes()
    _df = utils._EngineIntScalarType.empty_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _ngrp_py = utils._EngineIntScalarType.to_py(_ngrp)
    _obsd_cls = utils._EngineFloat64ArrayType(
        obsd, {'fun_name': fun_name, 'entity': 'obsd'},
        exp_shape=(_ngrp_py,),
        check_contents=False,
    )
    _obsd = _obsd_cls.to_ctypes(can_reference_contents=False)
    _expt_cls = utils._EngineFloat64ArrayType(
        expt, {'fun_name': fun_name, 'entity': 'expt'},
        exp_shape=(_ngrp_py,),
        check_contents=False,
    )
    _expt = _expt_cls.to_ctypes(can_reference_contents=False)
    _nd = utils._EngineIntScalarType.empty_ctypes()
    _di_cls = utils._EngineIntArrayType(
        di, {'fun_name': fun_name, 'entity': 'di'},
        exp_shape=(_ldn_py,),
        check_contents=False,
    )
    _di = _di_cls.to_ctypes(can_reference_contents=False)
    _ni_cls = utils._EngineIntArrayType(
        ni, {'fun_name': fun_name, 'entity': 'ni'},
        exp_shape=(_ldn_py,),
        check_contents=False,
    )
    _ni = _ni_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _t,
        _ic,
        _grp,
        _ngrp,
        _lfreq,
        _ifreq,
        _lwt,
        _wt,
        _ts,
        _df,
        _p,
        _obsd,
        _expt,
        _nd,
        _di,
        _ni,
        _ldn,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, all the times in t are the same.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ic[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ic[i-1] = 0 or 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, grp[{:s}] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'ngrp = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: 1 <= grp[i-1] <= ngrp.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ngrp = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: 2 <= ngrp <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lfreq = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: lfreq = 0 or lfreq >= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ifreq[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ifreq[i-1] >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lwt = {:s} and ldn = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: lwt = 0 or lwt >= ldn.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, wt[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: wt[i-1] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 11:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The degrees of freedom are zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 18 and _ifmt == 18:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ldn = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ldn >= {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 31 and _ifmt == 31:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, all observations are censored.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 41 and _ifmt == 41:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, group {:s} has no observations.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _obsd_cls.refresh_py(_obsd)
        _expt_cls.refresh_py(_expt)
        _di_cls.refresh_py(_di)
        _ni_cls.refresh_py(_ni)
        _ts_py = utils._EngineFloat64ScalarType.to_py(_ts)
        _df_py = utils._EngineIntScalarType.to_py(_df)
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        _nd_py = utils._EngineIntScalarType.to_py(_nd)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ts', 'df', 'p', 'nd'],
        )(
            _ts_py,
            _df_py,
            _p_py,
            _nd_py,
        )
    _err_class.manage()
    return return_data

def coxmodel(n, m, ns, z, isz, ip, t, ic, omega, isi, b, se, sc, cov, res, tp, sur, ndmax, tol, maxit, iprint, io_manager=None):
    u"""
    Fits Cox's proportional hazard model.

    For full information please refer to the NAG Library document for
    g12ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g12/g12baf.html

    Parameters
    ----------
    n : int
        n, the number of data points.

    m : int
        The number of covariates in array `z`.

    ns : int
        The number of strata. If `ns` > 0 then the stratum for each
        observation must be supplied in `isi`.

    z : float, shape (`n`, `m`)
        The ith row must contain the covariates which are associated
        with the ith failure time given in `t`.

    isz : int, shape (`m`)
        Indicates which subset of covariates is to be included in the
        model.

        `isz`[j-1] >= 1

          The jth covariate is included in the model.

        `isz`[j-1] = 0

          The jth covariate is excluded from the model and not
          referenced.

    ip : int
        The number of covariates included in the model as indicated by
        `isz`.

    t : float, shape (`n`)
        The vector of n failure censoring times.

    ic : int, shape (`n`)
        The status of the individual at time t given in `t`.

        `ic`[i-1] = 0

          The ith individual has failed at time `t`[i-1].

        `ic`[i-1] = 1

          The ith individual has been censored at time `t`[i-1].

    omega : None or float, shape (`n`)
        If `omega` is not None, the offset, omega_i, for i =
        1,2,...,`n`. Otherwise `omega` is not referenced.

    isi : int, shape (:)
        Note: the required length for this argument is determined as
        follows: if `ns`>0: `n`; otherwise: 1.

        If `ns` > 0, the stratum indicators which also allow data points
        to be excluded from the analysis.

        If `ns` = 0, `isi` is not referenced.

        `isi`[i-1] = k

          The ith data point is in the kth stratum, where k =
          1,2,...,`ns`.

        `isi`[i-1] = 0

          The ith data point is omitted from the analysis.

    b : float, shape (`ip`), modified in place
        On entry: initial estimates of the covariate coefficient
        parameters beta. `b`[j-1] must contain the initial estimate of
        the coefficient of the covariate in `z` corresponding to the jth
        nonzero value of `isz`.

        On exit: `b`[j-1] contains the estimate beta^_i, the coefficient
        of the covariate stored in the ith column of `z` where i is the
        jth nonzero value in the array `isz`.

    se : float, shape (`ip`), modified in place
        On exit: `se`[j-1] is the asymptotic standard error of the
        estimate contained in `b`[j-1] and score function in `sc`[j-1],
        for j = 1,2,...,`ip`.

    sc : float, shape (`ip`), modified in place
        On exit: `sc`[j-1] is the value of the score function,
        U_j(beta), for the estimate contained in `b`[j-1].

    cov : float, shape (`ip`*(`ip`+1)/2), modified in place
        On exit: the variance-covariance matrix of the parameter
        estimates in `b` stored in packed form by column, i.e., the
        covariance between the parameter estimates given in `b`[i-1] and
        `b`[j-1], j >= i, is stored in `cov`[j(j-1)/2+i-1].

    res : float, shape (`n`), modified in place
        On exit: the residuals, r(t_l), for l = 1,2,...,`n`.

    tp : float, shape (`ndmax`), modified in place
        On exit: `tp`[i-1] contains the ith distinct failure time, for i
        = 1,2,...,`nd`.

    sur : float, shape (`ndmax`, max(`ns`, 1)), modified in place
        On exit: if `ns` = 0, `sur`[i-1,0] contains the estimated
        survival function for the ith distinct failure time.

        If `ns` > 0, `sur`[i-1,k-1] contains the estimated survival
        function for the ith distinct failure time in the kth stratum.

    ndmax : int
        The dimension of the array `tp`.

        The first dimension of the array `sur`.

    tol : float
        Indicates the accuracy required for the estimation. Convergence
        is assumed when the decrease in deviance is less than
        `tol`*(1.0+CurrentDeviance). This corresponds approximately to
        an absolute precision if the deviance is small and a relative
        precision if the deviance is large.

    maxit : int
        The maximum number of iterations to be used for computing the
        estimates. If `maxit` is set to 0 then the standard errors,
        score functions, variance-covariance matrix and the survival
        function are computed for the input value of beta in `b` but
        beta is not updated.

    iprint : int
        Indicates if the printing of information on the iterations is
        required.

        `iprint` <= 0

          No printing.

        `iprint` >= 1

          The deviance and the current estimates are printed every
          `iprint` iterations. When printing occurs the output is
          directed to the file object associated with the advisory I/O
          unit (see :class:`~naginterfaces.base.utils.FileObjManager`).

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    dev : float
        The deviance, that is -2*(maximized log marginal likelihood).

    nd : int
        The number of distinct failure times.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ip` = *<value>*.

            Constraint: `ip` >= 1.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` >= 10*machine precision.

        (`errno` 1)
            On entry, `maxit` = *<value>*.

            Constraint: `maxit` >= 0.

        (`errno` 1)
            On entry, `ns` = *<value>*.

            Constraint: `ns` >= 0.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 2.

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 2)
            On entry, there are not `ip` values of `isz` > 0.

        (`errno` 2)
            On entry, i = *<value>* and `isz`[i-1] = *<value>*.

            Constraint: `isz`[i-1] >= 0.

        (`errno` 2)
            On entry too few observations included in model.

        (`errno` 2)
            On entry, i = *<value>*, `isi`[i-1] = *<value>* and `ns` =
            *<value>*.

            Constraint: 0 <= `isi`[i-1] <= `ns`.

        (`errno` 2)
            On entry, i = *<value>* and `ic`[i-1] = *<value>*.

            Constraint: `ic`[i-1] = 0 or 1.

        (`errno` 2)
            On entry, `ndmax` = *<value>* and minimum value for `ndmax`
            = *<value>*.

            Constraint: `ndmax` >= number of distinct failure times.

        (`errno` 2)
            All observations are censored.

        (`errno` 3)
            The matrix of second partial derivative is singular.

        (`errno` 4)
            Overflow has been detected in the calculations.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 5)
            Convergence not achieved in *<value>* iterations.

        (`errno` 6)
            Too many step halvings required.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.

    See Also
    --------
    :meth:`naginterfaces.library.examples.surviv.coxmodel_ex.main`
    """

    excs = []
    fun_name = 'naginterfaces.base.surviv.coxmodel'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g12baft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_STRING, # Input, offset(len=1).
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, ns.
        utils._EngineFloat64ArrayType.c_type, # Input, z[ldz, m].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineIntArrayType.c_type, # Input, isz[m].
        utils._BYREF_INT, # Input, ip.
        utils._EngineFloat64ArrayType.c_type, # Input, t[n].
        utils._EngineIntArrayType.c_type, # Input, ic[n].
        utils._EngineFloat64ArrayType.c_type, # Input, omega[n].
        utils._EngineIntArrayType.c_type, # Input, isi[*].
        utils._BYREF_FLOAT64, # Output, dev.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ip].
        utils._EngineFloat64ArrayType.c_type, # Output, se[ip].
        utils._EngineFloat64ArrayType.c_type, # Output, sc[ip].
        utils._EngineFloat64ArrayType.c_type, # Output, cov[ip*(ip+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Output, res[n].
        utils._BYREF_INT, # Output, nd.
        utils._EngineFloat64ArrayType.c_type, # Output, tp[ndmax].
        utils._EngineFloat64ArrayType.c_type, # Output, sur[ndmax, max(ns, 1)].
        utils._BYREF_INT, # Input, ndmax.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._BYREF_INT, # Input, maxit.
        utils._BYREF_INT, # Input, iprint.
        utils._BYREF_INT, # Input, nout.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wk.
        utils._EngineIntArrayType.c_type, # Workspace, iwk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of offset.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'), (sur, 'sur')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    if omega is None:
        _offset = utils._EngineCharScalarType(
            'N', {'fun_name': fun_name, 'entity': '_offset'}, exp_chrlen=1,
        ).to_ctypes()
    else:
        _offset = utils._EngineCharScalarType(
            'Y', {'fun_name': fun_name, 'entity': '_offset'}, exp_chrlen=1,
        ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _ns = utils._EngineIntScalarType(
        ns, {'fun_name': fun_name, 'entity': 'ns'},
    ).to_ctypes()
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _z = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _isz = utils._EngineIntArrayType(
        isz, {'fun_name': fun_name, 'entity': 'isz'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _ip = utils._EngineIntScalarType(
        ip, {'fun_name': fun_name, 'entity': 'ip'},
    ).to_ctypes()
    _t = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ic = utils._EngineIntArrayType(
        ic, {'fun_name': fun_name, 'entity': 'ic'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _omega = utils._EngineFloat64ArrayType(
        omega, {'fun_name': fun_name, 'entity': 'omega'},
        exp_shape=(_n_py,),
        allow_none=True,
    ).to_ctypes()
    _ns_py = utils._EngineIntScalarType.to_py(_ns)
    if _ns_py > 0:
        _isi_dim1 = _n_py
    else:
        _isi_dim1 = 1
    exp_shape_isi = (_isi_dim1,)
    _isi = utils._EngineIntArrayType(
        isi, {'fun_name': fun_name, 'entity': 'isi'},
        exp_shape=exp_shape_isi,
    ).to_ctypes()
    _dev = utils._EngineFloat64ScalarType.empty_ctypes()
    _ip_py = utils._EngineIntScalarType.to_py(_ip)
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_ip_py,),
    )
    _b = _b_cls.to_ctypes()
    _se_cls = utils._EngineFloat64ArrayType(
        se, {'fun_name': fun_name, 'entity': 'se'},
        exp_shape=(_ip_py,),
        check_contents=False,
    )
    _se = _se_cls.to_ctypes(can_reference_contents=False)
    _sc_cls = utils._EngineFloat64ArrayType(
        sc, {'fun_name': fun_name, 'entity': 'sc'},
        exp_shape=(_ip_py,),
        check_contents=False,
    )
    _sc = _sc_cls.to_ctypes(can_reference_contents=False)
    _cov_cls = utils._EngineFloat64ArrayType(
        cov, {'fun_name': fun_name, 'entity': 'cov'},
        exp_shape=(_ip_py*(_ip_py+1)//2,),
        check_contents=False,
    )
    _cov = _cov_cls.to_ctypes(can_reference_contents=False)
    _res_cls = utils._EngineFloat64ArrayType(
        res, {'fun_name': fun_name, 'entity': 'res'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _res = _res_cls.to_ctypes(can_reference_contents=False)
    _nd = utils._EngineIntScalarType.empty_ctypes()
    _ndmax = utils._EngineIntScalarType(
        ndmax, {'fun_name': fun_name, 'entity': 'ndmax'},
    ).to_ctypes()
    _ndmax_py = utils._EngineIntScalarType.to_py(_ndmax)
    _tp_cls = utils._EngineFloat64ArrayType(
        tp, {'fun_name': fun_name, 'entity': 'tp'},
        exp_shape=(_ndmax_py,),
        check_contents=False,
    )
    _tp = _tp_cls.to_ctypes(can_reference_contents=False)
    _sur_cls = utils._EngineFloat64ArrayType(
        sur, {'fun_name': fun_name, 'entity': 'sur'},
        exp_shape=(_ndmax_py, max(_ns_py, 1)),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _sur = _sur_cls.to_ctypes(can_reference_contents=False)
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _maxit = utils._EngineIntScalarType(
        maxit, {'fun_name': fun_name, 'entity': 'maxit'},
    ).to_ctypes()
    _iprint = utils._EngineIntScalarType(
        iprint, {'fun_name': fun_name, 'entity': 'iprint'},
    ).to_ctypes()
    _nout = utils._EngineIntScalarType.c_type(utils.FileObjManager.advunit)
    _wk = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwk = utils._EngineIntArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _offset,
        _n,
        _m,
        _ns,
        _z,
        _ldz,
        _isz,
        _ip,
        _t,
        _ic,
        _omega,
        _isi,
        _dev,
        _b,
        _se,
        _sc,
        _cov,
        _res,
        _nd,
        _tp,
        _sur,
        _ndmax,
        _tol,
        _maxit,
        _iprint,
        _nout,
        _wk,
        _iwk,
        _errbuf,
        _ierr,
        _offset._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ip = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ip >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tol >= 10*machine precision.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, maxit = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: maxit >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ns = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ns >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, there are not ip values of isz > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and isz[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: isz[i-1] >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry too few observations included in model.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, isi[i-1] = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and ns = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: 0 <= isi[i-1] <= ns.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and ic[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ic[i-1] = 0 or 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ndmax = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'minimum value for ndmax = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ndmax >= number of distinct failure times.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'All observations are censored.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The matrix of second partial derivative is singular.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Overflow has been detected in the calculations.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99987:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Convergence not achieved in {:s} iterations.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99983:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Too many step halvings required.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _b_cls.refresh_py(_b)
        _se_cls.refresh_py(_se)
        _sc_cls.refresh_py(_sc)
        _cov_cls.refresh_py(_cov)
        _res_cls.refresh_py(_res)
        _tp_cls.refresh_py(_tp)
        _sur_cls.refresh_py(_sur)
        _dev_py = utils._EngineFloat64ScalarType.to_py(_dev)
        _nd_py = utils._EngineIntScalarType.to_py(_nd)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['dev', 'nd'],
        )(
            _dev_py,
            _nd_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def coxmodel_risksets(n, m, ns, z, isz, ip, t, ic, isi, ixs, x, mxn, fid, tp, irs):
    u"""
    Creates the risk sets associated with the Cox proportional hazards
    model for fixed covariates.

    For full information please refer to the NAG Library document for
    g12za

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g12/g12zaf.html

    Parameters
    ----------
    n : int
        n, the number of data points.

    m : int
        The number of covariates in array `z`.

    ns : int
        The number of strata. If `ns` > 0 then the stratum for each
        observation must be supplied in `isi`.

    z : float, shape (`n`, `m`)
        The ith row must contain the covariates which are associated
        with the ith failure time given in `t`.

    isz : int, shape (`m`)
        Indicates which subset of covariates are to be included in the
        model.

        `isz`[j-1] >= 1

          The jth covariate is included in the model.

        `isz`[j-1] = 0

          The jth covariate is excluded from the model and not
          referenced.

    ip : int
        p, the number of covariates included in the model as indicated
        by `isz`.

    t : float, shape (`n`)
        The vector of n failure censoring times.

    ic : int, shape (`n`)
        The status of the individual at time t given in `t`.

        `ic`[i-1] = 0

          Indicates that the ith individual has failed at time `t`[i-1].

        `ic`[i-1] = 1

          Indicates that the ith individual has been censored at time
          `t`[i-1].

    isi : int, shape (:)
        Note: the required length for this argument is determined as
        follows: if `ns`>0: `n`; otherwise: 1.

        If `ns` > 0, the stratum indicators which also allow data points
        to be excluded from the analysis.

        If `ns` = 0, `isi` is not referenced.

        `isi`[i] = k

          Indicates that the ith data point is in the kth stratum, where
          k = 1,2,...,`ns`.

        `isi`[i] = 0

          Indicates that the ith data point is omitted from the
          analysis.

    ixs : int, shape (`mxn`), modified in place
        On exit: the factor giving the risk sets/strata for the data in
        `x` and `fid`.

        If `ns` = 0 or 1, `ixs`[i-1] = l for members of the lth risk
        set.

        If `ns` > 1, `ixs`[i-1] = (j-1)*`nd`+l for the observations in
        the lth risk set for the jth strata.

    x : float, shape (`mxn`, `ip`), modified in place
        On exit: the first `num` rows contain the values of the
        covariates for the members of the risk sets.

    mxn : int
        The first dimension of the array `x`.

        The dimension of the arrays `ixs` and `fid`.

    fid : int, shape (`mxn`), modified in place
        On exit: indicates if the member of the risk set given in `x`
        failed.

        `fid`[i-1] = 1 if the member of the risk set failed at the time
        defining the risk set and `fid`[i-1] = 0 otherwise.

    tp : float, shape (`n`), modified in place
        On exit: `tp`[i-1] contains the ith distinct failure time, for i
        = 1,2,...,`nd`.

    irs : int, shape (`n`), modified in place
        On exit: indicates rows in `x` and elements in `ixs` and `fid`
        corresponding to the risk sets. The first risk set corresponding
        to failure time `tp`[0] is given by rows 1 to `irs`[0]. The lth
        risk set is given by rows `irs`[l-2]+1 to `irs`[l-1], for l =
        1,2,...,`nd`.

    Returns
    -------
    num : int
        The number of values in the combined risk sets.

    nxs : int
        The number of levels for the risk sets/strata factor given in
        `ixs`.

    nd : int
        The number of distinct failure times, i.e., the number of risk
        sets.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ns` = *<value>*.

            Constraint: `ns` >= 0.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 2.

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 2)
            On entry, i = *<value>*, `isi`[i-1] = *<value>* and `ns` =
            *<value>*.

            Constraint: 0 <= `isi`[i-1] <= `ns`.

        (`errno` 2)
            On entry, i = *<value>* and `ic`[i-1] = *<value>*.

            Constraint: `ic`[i-1] = 0 or 1.

        (`errno` 2)
            On entry, there are not `ip` values of `isz` > 0.

        (`errno` 2)
            On entry, i = *<value>* and `isz`[i-1] = *<value>*.

            Constraint: `isz`[j-1] >= 0.

        (`errno` 3)
            On entry, `mxn` = *<value>* and minimum value for `mxn` =
            *<value>*.

            Constraint: `mxn` must be sufficiently large for the arrays
            to contain the expanded risk set.
    """

    excs = []
    fun_name = 'naginterfaces.base.surviv.coxmodel_risksets'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g12zaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, ns.
        utils._EngineFloat64ArrayType.c_type, # Input, z[ldz, m].
        utils._BYREF_INT, # Input, ldz.
        utils._EngineIntArrayType.c_type, # Input, isz[m].
        utils._BYREF_INT, # Input, ip.
        utils._EngineFloat64ArrayType.c_type, # Input, t[n].
        utils._EngineIntArrayType.c_type, # Input, ic[n].
        utils._EngineIntArrayType.c_type, # Input, isi[*].
        utils._BYREF_INT, # Output, num.
        utils._EngineIntArrayType.c_type, # Output, ixs[mxn].
        utils._BYREF_INT, # Output, nxs.
        utils._EngineFloat64ArrayType.c_type, # Output, x[mxn, ip].
        utils._BYREF_INT, # Input, mxn.
        utils._EngineIntArrayType.c_type, # Output, fid[mxn].
        utils._BYREF_INT, # Output, nd.
        utils._EngineFloat64ArrayType.c_type, # Output, tp[n].
        utils._EngineIntArrayType.c_type, # Output, irs[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((z, 'z'), (x, 'x')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _ns = utils._EngineIntScalarType(
        ns, {'fun_name': fun_name, 'entity': 'ns'},
    ).to_ctypes()
    _ldz_py = utils._EngineFloat64ArrayType.get_nd(z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldz = utils._EngineIntScalarType(
        _ldz_py, {'fun_name': fun_name, 'entity': '_ldz'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _z = utils._EngineFloat64ArrayType(
        z, {'fun_name': fun_name, 'entity': 'z'},
        exp_shape=(_n_py, _m_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _isz = utils._EngineIntArrayType(
        isz, {'fun_name': fun_name, 'entity': 'isz'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _ip = utils._EngineIntScalarType(
        ip, {'fun_name': fun_name, 'entity': 'ip'},
    ).to_ctypes()
    _t = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ic = utils._EngineIntArrayType(
        ic, {'fun_name': fun_name, 'entity': 'ic'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ns_py = utils._EngineIntScalarType.to_py(_ns)
    if _ns_py > 0:
        _isi_dim1 = _n_py
    else:
        _isi_dim1 = 1
    exp_shape_isi = (_isi_dim1,)
    _isi = utils._EngineIntArrayType(
        isi, {'fun_name': fun_name, 'entity': 'isi'},
        exp_shape=exp_shape_isi,
    ).to_ctypes()
    _num = utils._EngineIntScalarType.empty_ctypes()
    _mxn = utils._EngineIntScalarType(
        mxn, {'fun_name': fun_name, 'entity': 'mxn'},
    ).to_ctypes()
    _mxn_py = utils._EngineIntScalarType.to_py(_mxn)
    _ixs_cls = utils._EngineIntArrayType(
        ixs, {'fun_name': fun_name, 'entity': 'ixs'},
        exp_shape=(_mxn_py,),
        check_contents=False,
    )
    _ixs = _ixs_cls.to_ctypes(can_reference_contents=False)
    _nxs = utils._EngineIntScalarType.empty_ctypes()
    _ip_py = utils._EngineIntScalarType.to_py(_ip)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_mxn_py, _ip_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _fid_cls = utils._EngineIntArrayType(
        fid, {'fun_name': fun_name, 'entity': 'fid'},
        exp_shape=(_mxn_py,),
        check_contents=False,
    )
    _fid = _fid_cls.to_ctypes(can_reference_contents=False)
    _nd = utils._EngineIntScalarType.empty_ctypes()
    _tp_cls = utils._EngineFloat64ArrayType(
        tp, {'fun_name': fun_name, 'entity': 'tp'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _tp = _tp_cls.to_ctypes(can_reference_contents=False)
    _irs_cls = utils._EngineIntArrayType(
        irs, {'fun_name': fun_name, 'entity': 'irs'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _irs = _irs_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _m,
        _ns,
        _z,
        _ldz,
        _isz,
        _ip,
        _t,
        _ic,
        _isi,
        _num,
        _ixs,
        _nxs,
        _x,
        _mxn,
        _fid,
        _nd,
        _tp,
        _irs,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ns = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ns >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, isi[i-1] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'ns = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: 0 <= isi[i-1] <= ns.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and ic[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ic[i-1] = 0 or 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, there are not ip values of isz > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and isz[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: isz[j-1] >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mxn = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'minimum value for mxn = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mxn must be sufficiently large for',
                    'the arrays to contain the expanded risk set.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ixs_cls.refresh_py(_ixs)
        _x_cls.refresh_py(_x)
        _fid_cls.refresh_py(_fid)
        _tp_cls.refresh_py(_tp)
        _irs_cls.refresh_py(_irs)
        _num_py = utils._EngineIntScalarType.to_py(_num)
        _nxs_py = utils._EngineIntScalarType.to_py(_nxs)
        _nd_py = utils._EngineIntScalarType.to_py(_nd)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['num', 'nxs', 'nd'],
        )(
            _num_py,
            _nxs_py,
            _nd_py,
        )
    _err_class.manage()
    return return_data
