# -*- coding: utf-8 -*-
# pylint: disable=chained-comparison,invalid-name,line-too-long,protected-access,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unused-import
u"""
Interfaces for the NAG Mark 27.0 `numdiff` Chapter.

``numdiff`` - Numerical Differentiation

This module is concerned with calculating approximations to derivatives
of a function f.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d04/d04intro.html
"""

# NAG Copyright 2017-2019.

import collections as _collections

from ..base import utils as _b_utils
from ..library import _utils

def fwd(xval, nder, hbase, f, data=None):
    u"""
    Numerical differentiation, derivatives up to order 14, function of
    one real variable.

    For full information please refer to the NAG Library document for
    d04aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d04/d04aaf.html

    Parameters
    ----------
    xval : float
        The point at which the derivatives are required, x_0.

    nder : int
        Must be set so that its absolute value is the highest order
        derivative required.

        `nder` > 0

          All derivatives up to order min(`nder`, 14) are calculated.

        `nder` < 0 and `nder` is even

          Only even order derivatives up to order min(-`nder`, 14) are
          calculated.

        `nder` < 0 and `nder` is odd

          Only odd order derivatives up to order min(-`nder`, 13) are
          calculated.

    hbase : float
        The initial step length which may be positive or negative. For
        advice on the choice of `hbase` see [Further Comments].

    f : callable retval = f(x, data=None)
        `f` must evaluate the function f(x) at a specified point.

        Parameters
        ~~~~~~~~~~
        x : float
            The value of the argument x.

            If you have equally spaced tabular data, the following
            information may be useful:

            - in any call of ``fwd`` the only values of x for which f(x)
              will be required are x = `xval` and x =
              `xval`+/-(2*j-1)`hbase`, for j = 1,2,...,10; and

            - f(x_0) is always computed, but it is disregarded when only
              odd order derivatives are required.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of f(x) at the specified point.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    der : float, ndarray, shape (14)
        `der`[j-1] contains an approximation to the jth derivative of
        f(x) at x = `xval`, so long as the jth derivative is one of
        those requested by you when specifying `nder`. For other values
        of j, `der`[j-1] is unused.

    erest : float, ndarray, shape (14)
        An estimate of the absolute error in the corresponding result
        `der`[j-1] so long as the jth derivative is one of those
        requested by you when specifying `nder`. The sign of
        `erest`[j-1] is positive unless the result `der`[j-1] is
        questionable. It is set negative when |`der`[j-1]| <
        |`erest`[j-1]| or when for some other reason there is doubt
        about the validity of the result `der`[j-1] (see [Errors]). For
        other values of j, `erest`[j-1] is unused.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nder` = 0.

            Constraint: `nder` != 0.

        (`errno` 1)
            On entry, `hbase` = 0.0.

            Constraint: `hbase` != 0.0.
    """

    from ..base.numdiff import fwd as b_fwd

    def cb_f(x, data=None):
        """Wrapping user's f. Called as base f."""
        if data is not None:
            _retval_f = f(
                x,
                data,
            )
        else:
            _retval_f = f(
                x,
            )
        return _retval_f

    _der = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (14,),
    )
    _erest = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (14,),
    )
    fun_name = 'naginterfaces.library.numdiff.fwd'

    _b_utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = cb_f
    b_fwd(
        xval,
        nder,
        hbase,
        _der,
        _erest,
        _f,
        data,
    )
    _der_py = _der
    _erest_py = _erest
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['der', 'erest'],
    )(
        _der_py,
        _erest_py,
    )
    return _return_data

def rcomm(xval, fval):
    u"""
    Numerical differentiation, user-supplied function values,
    derivatives up to order 14, derivatives with respect to one real
    variable.

    For full information please refer to the NAG Library document for
    d04ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d04/d04baf.html

    Parameters
    ----------
    xval : float, array-like, shape (21)
        The abscissae at which the function has been evaluated, as
        described in [Description]. These can be generated by calling
        ``sample``. The order of the abscissae is irrelevant.

    fval : float, array-like, shape (21)
        `fval`[j-1] must contain the function value at `xval`[j-1], for
        j = 1,2,...,21.

    Returns
    -------
    der : float, ndarray, shape (14)
        The 14 derivative estimates.

    erest : float, ndarray, shape (14)
        The 14 error estimates for the derivatives.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The derived h is below tolerance.

            Derived h > *<value>* is required. Derived h = *<value>*.

        (`errno` 1)
            On entry, the values of `xval` are not correctly spaced.

            Derived h = *<value>*.
    """

    from ..base.numdiff import rcomm as b_rcomm

    fun_name = 'naginterfaces.library.numdiff.rcomm'

    _xval = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'xval'},
        xval,
        copy=False,
    )
    _fval = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'fval'},
        fval,
        copy=False,
    )
    _der = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (14,),
    )
    _erest = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (14,),
    )
    b_rcomm(
        _xval,
        _fval,
        _der,
        _erest,
    )
    _der_py = _der
    _erest_py = _erest
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['der', 'erest'],
    )(
        _der_py,
        _erest_py,
    )
    return _return_data

def sample(x_0, hbase):
    u"""
    Generates sample points for function evaluations by ``rcomm``.

    For full information please refer to the NAG Library document for
    d04bb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d04/d04bbf.html

    Parameters
    ----------
    x_0 : float
        The abscissa x_0 at which derivatives are required.

    hbase : float
        The chosen step size h. If h < 10 epsilon, where epsilon =
        machine.precision(), the default h = epsilon^(1/4) will be used.

    Returns
    -------
    xval : float, ndarray, shape (21)
        The abscissae for passing to ``rcomm``.
    """

    from ..base.numdiff import sample as b_sample

    _xval = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (21,),
    )
    b_sample(
        x_0,
        hbase,
        _xval,
    )
    _xval_py = _xval
    return _xval_py
