# -*- coding: utf-8 -*-
# pylint: disable=chained-comparison,invalid-name,line-too-long,protected-access,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unused-import
u"""
Interfaces for the NAG Mark 27.0 `rnla` Chapter.

``rnla`` - Randomized Numerical Linear Algebra

This module covers linear algebra functions that make use of random
projections to reduce problem dimension.
This area is referred to as RNLA (Randomized Numerical Linear Algebra).
The functions can be split into the following categories:

- building blocks that are intended to be used as components in RNLA
  algorithms written by you;

- RNLA algorithms for matrix factorization.

It is envisaged that users of the higher-level functions, such as matrix
factorization, will have some background in linear algebra.
A common use case would be that you have tried solving your problem
using a deterministic linear algebra function, e.g., an LAPACK function
from submodule ``lapackeig``, and are in need of a function that is more
computationally efficient.

Users of the building block functions would be expected to have some
familiarity with the RNLA literature, i.e., a higher level of expertise.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f10/f10intro.html
"""

# NAG Copyright 2017-2019.

import collections as _collections

from ..base import utils as _b_utils
from ..library import _utils

def svd_rowext_real(jobu, jobvt, a, k, statecomm, rtol_abs=-1.0, rtol_rel=-1.0):
    u"""
    Computes the singular value decomposition of a real matrix,
    optionally computing the left and/or right singular vectors.

    For full information please refer to the NAG Library document for
    f10ca

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f10/f10caf.html

    Parameters
    ----------
    jobu : str, length 1
        Specifies options for computing part of or none of the matrix U.

        `jobu` = 'S'

          The first k columns of U (the left singular vectors) are
          returned in the array `u`.

        `jobu` = 'N'

          No columns of U (no left singular vectors) are computed.

    jobvt : str, length 1
        Specifies options for computing part of or none of the matrix
        V^T.

        `jobvt` = 'S'

          The first k rows of V^T (the right singular vectors) are
          returned in the array `vt`.

        `jobvt` = 'N'

          No rows of V^T (no right singular vectors) are computed.

    a : float, array-like, shape (m, n)
        The m by n matrix A.

    k : int
        k, number of columns in random projection, Y = A Omega.

    statecomm : dict, RNG communication object, modified in place
        RNG communication structure.

        This argument must have been initialized by a prior call to
        ``rand.init_repeat`` or ``rand.init_nonrepeat``.

    rtol_abs : float, optional
        The absolute tolerance, epsilon_a used in defining the threshold
        on estimating the rank of A. If `rtol_abs` < 0.0 then 0.0 is
        used unless `rtol_rel` <= 0.0 in which case sqrt(machine
        precision) is used.

    rtol_rel : float, optional
        The relative tolerance, epsilon_r used in defining the threshold
        on estimating the rank of A. If `rtol_rel` < 0.0 then 0.0 is
        used unless `rtol_abs` <= 0.0 in which case sqrt(machine
        precision) is used.

    Returns
    -------
    s : float, ndarray, shape (`r`)
        The `r` largest singular values of A in descending order.

    u : None or float, ndarray, shape (:, :)
        If `jobu` = 'S', `u` contains the first `r` columns of U (the
        left singular vectors, stored column-wise).

    vt : None or float, ndarray, shape (:, :)
        If `jobvt` = 'S', `vt` contains the first `r` rows of V^T (the
        right singular vectors).

    r : int
        r, contains estimated rank of array `a`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `jobu` = *<value>*.

            Constraint: `jobu` = 'S' or 'N'.

        (`errno` 2)
            On entry, `jobvt` = *<value>*.

            Constraint: `jobvt` = 'S' or 'N'.

        (`errno` 3)
            On entry, m = *<value>*.

            Constraint: m > 0.

        (`errno` 4)
            On entry, n = *<value>*.

            Constraint: n > 0.

        (`errno` 7)
            On entry, `k` = *<value>*.

            Constraint: 0 < `k` <= min(m, n).

        (`errno` 8)
            On entry, `statecomm`['state'] vector has been corrupted or
            not initialized.

        (`errno` 9)
            On entry, `jobu` = *<value>* and ldu = *<value>*.

            Constraint: if `jobu` = 'S', ldu >= m.

        (`errno` 10)
            On entry, `jobvt` = *<value>* and ldvt = *<value>*.

            Constraint: if `jobvt` = 'S', ldvt >= `k`.

        (`errno` 21)
            On exit, `r` = `k`, the rank of A may be larger than `r`.

            Increase `k` to obtain a more accurate rank estimate.

            Smallest diagonal element of R, from QR of Y, =*<value>*.

            Tolerance used to determine rank =*<value>*.

        (`errno` 22)
            A has effective rank of zero.

            First diagonal element of R, from QR of Y, =*<value>*.

            Tolerance used to determine rank =*<value>*.
    """

    from ..base.rnla import svd_rowext_real as b_svd_rowext_real

    fun_name = 'naginterfaces.library.rnla.svd_rowext_real'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'),),
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _k = _b_utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = _b_utils._EngineIntScalarType.to_py(_k)
    _s = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_k_py,),
    )
    _jobu = _b_utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = _b_utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py in ('S', 's'):
        if _jobu_py in ('S', 's'):
            _u_dim1 = _m_py
        else:
            _u_dim1 = 0
        if _jobu_py in ('S', 's'):
            _u_dim2 = _k_py
        else:
            _u_dim2 = 0
        exp_shape_u = (_u_dim1, _u_dim2)
        _u = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            exp_shape_u,
            sorder=_m_sorder,
        )
    else:
        _u = None
    _jobvt = _b_utils._EngineCharScalarType(
        jobvt, {'fun_name': fun_name, 'entity': 'jobvt'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvt_py = _b_utils._EngineCharScalarType.to_py(_jobvt)
    if _jobvt_py in ('S', 's'):
        if _jobvt_py in ('S', 's'):
            _vt_dim1 = _k_py
        else:
            _vt_dim1 = 0
        if _jobvt_py in ('S', 's'):
            _vt_dim2 = _n_py
        else:
            _vt_dim2 = 1
        exp_shape_vt = (_vt_dim1, _vt_dim2)
        _vt = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            exp_shape_vt,
            sorder=_m_sorder,
        )
    else:
        _vt = None
    _return_scalar = b_svd_rowext_real(
        jobu,
        jobvt,
        _m,
        _n,
        _a,
        _k,
        rtol_abs,
        rtol_rel,
        statecomm,
        _s,
        _u,
        _vt,
    )
    _r_py = _return_scalar
    _s_py = _s[:_r_py]
    if _jobu_py in ('S', 's'):
        _u_dim2 = _r_py
    else:
        _u_dim2 = 0
    if _jobu_py in ('S', 's'):
        _u_py = _u[:, :_u_dim2]
    else:
        _u_py = None
    if _jobvt_py in ('S', 's'):
        _vt_dim1 = _r_py
    else:
        _vt_dim1 = 0
    if _jobvt_py in ('S', 's'):
        _vt_py = _vt[:_vt_dim1, :]
    else:
        _vt_py = None
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['s', 'u', 'vt', 'r'],
    )(
        _s_py,
        _u_py,
        _vt_py,
        _r_py,
    )
    return _return_data

def randproj_dct_real(trans, a, k, statecomm):
    u"""
    Computes a fast random projection of a real matrix using a discrete
    cosine transform.

    For full information please refer to the NAG Library document for
    f10da

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f10/f10daf.html

    Parameters
    ----------
    trans : str, length 1
        Specifies whether the operation pre-multiplies A or
        post-multiplies A.

        `trans` = 'N'

          Random projection is done by post-multiplication, Y = A Omega.

        `trans` = 'T'

          Random projection is done by pre-multiplication, Y = Omega A.

    a : float, array-like, shape (m, n)
        The m by n matrix A.

    k : int
        k, number of columns in the random projection, Y = A Omega.

    statecomm : dict, RNG communication object, modified in place
        RNG communication structure.

        This argument must have been initialized by a prior call to
        ``rand.init_repeat`` or ``rand.init_nonrepeat``.

    Returns
    -------
    a : float, ndarray, shape (:, :)
        If `trans` = 'N', the m by k random projection of A.

        If `trans` = 'T', the k by n random projection of A.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `trans` = *<value>*.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m > 0.

        (`errno` 3)
            On entry, n = *<value>*.

            Constraint: n > 0.

        (`errno` 4)
            On entry, `k` = *<value>* and `trans` = *<value>*.

            Constraint: if `trans` = 'N', 0 < `k` <= n, if `trans` =
            'T', 0 < `k` <= m.

        (`errno` 6)
            On entry, `statecomm`['state'] vector has been corrupted or
            not initialized.
    """

    from ..base.rnla import randproj_dct_real as b_randproj_dct_real

    fun_name = 'naginterfaces.library.rnla.randproj_dct_real'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    b_randproj_dct_real(
        trans,
        _m,
        _n,
        _a,
        k,
        statecomm,
    )
    _trans = _b_utils._EngineCharScalarType(
        trans, {'fun_name': fun_name, 'entity': 'trans'}, exp_chrlen=1,
    ).to_ctypes()
    _trans_py = _b_utils._EngineCharScalarType.to_py(_trans)
    _k = _b_utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = _b_utils._EngineIntScalarType.to_py(_k)
    if _trans_py in ('N', 'n'):
        _a_dim1 = _m_py
    elif _trans_py in ('T', 't'):
        _a_dim1 = _k_py
    else:
        _a_dim1 = 0
    if _trans_py in ('N', 'n'):
        _a_dim2 = _k_py
    elif _trans_py in ('T', 't'):
        _a_dim2 = _n_py
    else:
        _a_dim2 = 0
    _a_py = _a[:_a_dim1, :_a_dim2]
    return _a_py
