# -*- coding: utf-8 -*-
# pylint: disable=chained-comparison,invalid-name,line-too-long,protected-access,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unused-import
u"""
Interfaces for the NAG Mark 27.0 `roots` Chapter.

``roots`` - Roots of One or More Transcendental Equations

This module is concerned with the calculation of zeros of continuous
functions of one or more variables.
The majority of problems considered are for real-valued functions of
real variables, in which case complex equations must be expressed in
terms of the equivalent larger system of real equations.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05intro.html

See Also
--------
``naginterfaces.library.examples.roots`` :
    This subpackage contains examples for the ``roots`` module.
    See also the :ref:`library_roots_ex` subsection.
"""

# NAG Copyright 2017-2019.

from __future__ import absolute_import

import collections as _collections

from math import sqrt

from ..base import utils as _b_utils
from ..library import machine
from ..library import _utils

def contfn_brent_interval(x, f, h=0.1, eps=1000.0*machine.precision(), eta=0.0, data=None):
    u"""
    Zero of continuous function, Brent algorithm, from a given starting
    value, binary search for interval.

    For full information please refer to the NAG Library document for
    c05au

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05auf.html

    Parameters
    ----------
    x : float
        An initial approximation to the zero.

    f : callable retval = f(x, data=None)
        `f` must evaluate the function f whose zero is to be determined.

        Parameters
        ~~~~~~~~~~
        x : float
            The point at which the function must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of f at the specified point.

    h : float, optional
        A step length for use in the binary search for an interval
        containing the zero. The maximum interval searched is
        [`x`-256.0*`h`,`x`+256.0*`h`].

    eps : float, optional
        The termination tolerance on x (see [Description]).

    eta : float, optional
        A value such that if |f(x)| <= `eta`, x is accepted as the zero.
        `eta` may be specified as 0.0 (see [Accuracy]).

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    x : float
        If the function exits successfully or `errno` = 4, this argument
        is the final approximation to the zero.

        If `errno` = 3, this argument is likely to be a pole of f(x).

        Otherwise, this argument contains no useful information.

    a : float
        The lower bound of the interval resulting from the binary search

    b : float
        The upper bound of the interval resulting from the binary search

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `eps` = *<value>*.

            Constraint: `eps` > 0.0.

        (`errno` 1)
            On entry, `x` = *<value>* and `h` = *<value>*.

            Constraint: `x`+`h` != `x` (to machine accuracy).

        (`errno` 2)
            An interval containing the zero could not be found.
            Increasing `h` and calling ``contfn_brent_interval`` again
            will increase the range searched for the zero. Decreasing
            `h` and calling ``contfn_brent_interval`` again will refine
            the mesh used in the search for the zero.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            Solution may be a pole rather than a zero.

        (`errno` 4)
            The tolerance `eps` has been set too small for the problem
            being solved. However, the value `x` returned is a good
            approximation to the zero. `eps` = *<value>*.

    See Also
    --------
    :meth:`naginterfaces.library.examples.roots.contfn_brent_interval_ex.main`
    """

    from ..base.roots import contfn_brent_interval as b_contfn_brent_interval

    def cb_f(x, data=None):
        """Wrapping user's f. Called as base f."""
        if data is not None:
            _retval_f = f(
                x,
                data,
            )
        else:
            _retval_f = f(
                x,
            )
        return _retval_f

    fun_name = 'naginterfaces.library.roots.contfn_brent_interval'

    _b_utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = cb_f
    try:
        _return_scalars = b_contfn_brent_interval(
            x,
            h,
            eps,
            eta,
            _f,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _x_py = _return_scalars.x
    _a_py = _return_scalars.a
    _b_py = _return_scalars.b
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'a', 'b'],
    )(
        _x_py,
        _a_py,
        _b_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def contfn_interval_rcomm(x, fx, h, boundl, boundu, y, comm, ind):
    u"""
    Binary search for interval containing zero of continuous function
    (reverse communication).

    For full information please refer to the NAG Library document for
    c05av

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05avf.html

    Parameters
    ----------
    x : float
        On initial entry: the best available approximation to the zero.

    fx : float
        On initial entry: if `ind` = 1, `fx` need not be set.

        If `ind` = -1, `fx` must contain f(`x`) for the initial value of
        `x`.

        On intermediate entry: must contain f(`x`) for the current value
        of `x`.

    h : float
        On initial entry: a basic step size which is used in the binary
        search for an interval containing a zero. The basic step sizes
        `h`,0.1*`h`, 0.01*`h` and 0.001*`h` are used in turn when
        searching for the zero.

    boundl : float
        The lower bound for the interval of search for the zero.

    boundu : float
        The upper bound for the interval of search for the zero.

    y : float
        On initial entry: need not be set.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    ind : int
        On initial entry: must be set to 1 or -1.

        `ind` = 1

          `fx` need not be set.

        `ind` = -1

          `fx` must contain f(`x`).

    Returns
    -------
    x : float
        On intermediate exit: contains the point at which f must be
        evaluated before re-entry to the function.

        On final exit: contains one end of an interval containing the
        zero, the other end being in `y`, unless an error has occurred.
        If `errno` = 4, this argument and `y` are the end points of the
        largest interval searched. If a zero is located exactly, its
        value is returned in this argument (and in `y`).

    h : float
        On final exit: is undefined.

    y : float
        On final exit: contains the closest point found to the final
        value of `x`, such that f(`x`)*f(`y`) <= 0.0. If a value `x` is
        found such that f(`x`) = 0, `y` = `x`. On final exit with
        `errno` = 4, `x` and this argument are the end points of the
        largest interval searched.

    c1 : float
        On final exit: if the function exits successfully or `errno` =
        4, `c1` is set to f(`y`).

    ind : int
        On intermediate exit: contains 2 or 3. The calling program must
        evaluate f at `x`, storing the result in `fx`, and re-enter
        ``contfn_interval_rcomm`` with all other arguments unchanged.

        On final exit: contains 0.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `boundl` = *<value>* and `boundu` = *<value>*.

            Constraint: `boundl` < `boundu`.

        (`errno` 1)
            On entry, `x` = *<value>*, `boundl` = *<value>* and `boundu`
            = *<value>*.

            Constraint: `boundl` <= `x` <= `boundu`.

        (`errno` 1)
            On entry, `x` = *<value>*, `h` = *<value>*, `boundl` =
            *<value>* and `boundu` = *<value>*.

            Constraint: either `x`+`h` or `x`-`h` must lie inside the
            closed interval [`boundl`,`boundu`].

        (`errno` 2)
            On entry, `x` = *<value>* and `h` = *<value>*.

            Constraint: `h` must be sufficiently large that `x`+`h` !=
            `x` on the computer.

        (`errno` 3)
            On entry, `ind` = *<value>*.

            Constraint: `ind` = -1, 1, 2 or 3.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            An interval containing the zero could not be found. Try
            restarting with modified `x` and `h`.
    """

    from ..base.roots import contfn_interval_rcomm as b_contfn_interval_rcomm

    try:
        _return_scalars = b_contfn_interval_rcomm(
            x,
            fx,
            h,
            boundl,
            boundu,
            y,
            comm,
            ind,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _x_py = _return_scalars.x
    _h_py = _return_scalars.h
    _y_py = _return_scalars.y
    _ind_py = _return_scalars.ind
    _c1_py = _return_scalars.c1
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'h', 'y', 'c1', 'ind'],
    )(
        _x_py,
        _h_py,
        _y_py,
        _c1_py,
        _ind_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def contfn_cntin(x, f, eps=1000.0*machine.precision(), eta=0.0, nfmax=1000, data=None):
    u"""
    Zero of continuous function, continuation method, from a given
    starting value.

    For full information please refer to the NAG Library document for
    c05aw

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05awf.html

    Parameters
    ----------
    x : float
        An initial approximation to the zero.

    f : callable retval = f(x, data=None)
        `f` must evaluate the function f whose zero is to be determined.

        Parameters
        ~~~~~~~~~~
        x : float
            The point at which the function must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of f evaluated at `x`.

    eps : float, optional
        An absolute tolerance to control the accuracy to which the zero
        is determined. In general, the smaller the value of `eps` the
        more accurate `x` will be as an approximation to alpha. Indeed,
        for very small positive values of `eps`, it is likely that the
        final approximation will satisfy |`x`-alpha| < `eps`. You are
        advised to call the function with more than one value for `eps`
        to check the accuracy obtained.

    eta : float, optional
        A value such that if |f(x)| < `eta`, x is accepted as the zero.
        `eta` may be specified as 0.0 (see [Accuracy]).

    nfmax : int, optional
        The maximum permitted number of calls to `f` from
        ``contfn_cntin``. If `f` is inexpensive to evaluate, `nfmax`
        should be given a large value (say > 1000).

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    x : float
        The final approximation to the zero, unless `errno` = 1, 2 or 5,
        in which case it contains no useful information.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nfmax` = *<value>*.

            Constraint: `nfmax` > 0.

        (`errno` 1)
            On entry, `eps` = *<value>*.

            Constraint: `eps` > 0.0.

        (`errno` 2)
            Internal scale factor invalid for this problem. Consider
            using ``contfn_cntin_rcomm`` instead and setting scal.

        (`errno` 3)
            Either `f` has no zero near `x` or too much accuracy has
            been requested. Check the coding of `f` or increase `eps`.

        (`errno` 4)
            More than `nfmax` calls have been made to `f`.

        (`errno` 5)
            A serious error occurred in an internal call to an auxiliary
            function.
    """

    from ..base.roots import contfn_cntin as b_contfn_cntin

    def cb_f(x, data=None):
        """Wrapping user's f. Called as base f."""
        if data is not None:
            _retval_f = f(
                x,
                data,
            )
        else:
            _retval_f = f(
                x,
            )
        return _retval_f

    fun_name = 'naginterfaces.library.roots.contfn_cntin'

    _b_utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = cb_f
    _return_scalar = b_contfn_cntin(
        x,
        eps,
        eta,
        _f,
        nfmax,
        data,
    )
    _x_py = _return_scalar
    return _x_py

def contfn_cntin_rcomm(x, fx, tol, ir, comm, ind, scal=sqrt(machine.precision())):
    u"""
    Zero of continuous function, continuation method, from a given
    starting value (reverse communication).

    For full information please refer to the NAG Library document for
    c05ax

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05axf.html

    Parameters
    ----------
    x : float
        On initial entry: an initial approximation to the zero.

    fx : float
        On initial entry: if `ind` = 1, `fx` need not be set.

        If `ind` = -1, `fx` must contain f(`x`) for the initial value of
        `x`.

        On intermediate entry: must contain f(`x`) for the current value
        of `x`.

    tol : float
        On initial entry: a value that controls the accuracy to which
        the zero is determined. `tol` is used in determining the
        convergence of the secant iteration used at each stage of the
        continuation process. It is used directly when solving the last
        problem (theta_m = 0 in [Description]), and sqrt(`tol`) is used
        for the problem defined by theta_r, r < m. Convergence to the
        accuracy specified by `tol` is not guaranteed, and so you are
        recommended to find the zero using at least two values for `tol`
        to check the accuracy obtained.

    ir : int
        On initial entry: indicates the type of error test required, as
        follows. Solving the problem defined by theta_r, 1 <= r <= m,
        involves computing a sequence of secant iterates
        x_r^0,x_r^1,.... This sequence will be considered to have
        converged only if:

        for `ir` = 0,

            |x_r^(i+1)-x_r^(i)| <= eps*max(1.0, |x_r^(i)|),

        for `ir` = 1,

            |x_r^(i+1)-x_r^(i)| <= eps,

        for `ir` = 2,

            |x_r^(i+1)-x_r^(i)| <= eps*|x_r^(i)|,

        for some i > 1; here eps is either `tol` or sqrt(`tol`) as
        discussed above.

        Note that there are other subsidiary conditions (not given here)
        which must also be satisfied before the secant iteration is
        considered to have converged.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    ind : int
        On initial entry: must be set to 1 or -1.

        `ind` = 1

          `fx` need not be set.

        `ind` = -1

          `fx` must contain f(`x`).

    scal : float, optional
        On initial entry: a factor for use in determining a significant
        approximation to the derivative of f(x) at x = x_0, the initial
        value. A number of difference approximations to f'(x_0) are
        calculated using

            f'(x_0)~(f(x_0+h)-f(x_0))/h

        where |h| < |`scal`| and h has the same sign as `scal`. A
        significance (cancellation) check is made on each difference
        approximation and the approximation is rejected if
        insignificant.

    Returns
    -------
    x : float
        On intermediate exit: the point at which f must be evaluated
        before re-entry to the function.

        On final exit: the final approximation to the zero.

    ind : int
        On intermediate exit: contains 2, 3 or 4. The calling program
        must evaluate f at `x`, storing the result in `fx`, and re-enter
        ``contfn_cntin_rcomm`` with all other arguments unchanged.

        On final exit: contains 0.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` > 0.0.

        (`errno` 1)
            On entry, `ir` = *<value>*.

            Constraint: `ir` = 0, 1 or 2.

        (`errno` 2)
            On initial entry, `ind` = *<value>*.

            Constraint: `ind` = -1 or 1.

        (`errno` 2)
            On intermediate entry, `ind` = *<value>*.

            Constraint: `ind` = 2, 3 or 4.

        (`errno` 3)
            On entry, `scal` = *<value>*.

            Constraint: `x`+`scal` != `x` (to machine accuracy).

        (`errno` 3)
            Significant derivatives of f cannot be computed. This can
            happen when f is almost constant and nonzero, for any value
            of `scal`.

        (`errno` 4)
            Current problem in the continuation sequence cannot be
            solved. Perhaps the original problem had no solution or the
            continuation path passes through a set of insoluble
            problems: consider refining the initial approximation to the
            zero. Alternatively, `tol` is too small, and the accuracy
            requirement is too stringent, or too large and the initial
            approximation too poor.

        (`errno` 5)
            Continuation away from the initial point is not possible.
            This error exit will usually occur if the problem has not
            been properly posed or the error requirement is extremely
            stringent.

        (`errno` 6)
            Final problem (with theta_m = 0) cannot be solved. It is
            likely that too much accuracy has been requested, or that
            the zero is at alpha = 0 and `ir` = 2.
    """

    from ..base.roots import contfn_cntin_rcomm as b_contfn_cntin_rcomm

    _return_scalars = b_contfn_cntin_rcomm(
        x,
        fx,
        tol,
        ir,
        scal,
        comm,
        ind,
    )
    _x_py = _return_scalars.x
    _ind_py = _return_scalars.ind
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'ind'],
    )(
        _x_py,
        _ind_py,
    )
    return _return_data

def contfn_brent(a, b, f, eps=1000.0*machine.precision(), eta=0.0, data=None):
    u"""
    Zero of continuous function in a given interval, Brent algorithm.

    For full information please refer to the NAG Library document for
    c05ay

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05ayf.html

    Parameters
    ----------
    a : float
        a, the lower bound of the interval.

    b : float
        b, the upper bound of the interval.

    f : callable retval = f(x, data=None)
        `f` must evaluate the function f whose zero is to be determined.

        Parameters
        ~~~~~~~~~~
        x : float
            The point at which the function must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of f evaluated at `x`.

    eps : float, optional
        The termination tolerance on x (see [Description]).

    eta : float, optional
        A value such that if |f(x)| <= `eta`, x is accepted as the zero.
        `eta` may be specified as 0.0 (see [Accuracy]).

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    x : float
        If the function exits successfully or `errno` = 2, this argument
        is the final approximation to the zero. If `errno` = 3, this
        argument is likely to be a pole of f(x). Otherwise, this
        argument contains no useful information.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `eps` = *<value>*.

            Constraint: `eps` > 0.0.

        (`errno` 1)
            On entry, `a` = *<value>* and `b` = *<value>*.

            Constraint: `a` != `b`.

        (`errno` 1)
            On entry, `f`(`a`) and `f`(`b`) have the same sign with
            neither equalling 0.0: `f`(`a`) = *<value>* and `f`(`b`) =
            *<value>*.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            No further improvement in the solution is possible. `eps` is
            too small: `eps` = *<value>*. The final value of `x`
            returned is an accurate approximation to the zero.

        (`errno` 3)
            The function values in the interval [`a`,`b`] might contain
            a pole rather than a zero. Reducing `eps` may help in
            distinguishing between a pole and a zero.
    """

    from ..base.roots import contfn_brent as b_contfn_brent

    def cb_f(x, data=None):
        """Wrapping user's f. Called as base f."""
        if data is not None:
            _retval_f = f(
                x,
                data,
            )
        else:
            _retval_f = f(
                x,
            )
        return _retval_f

    fun_name = 'naginterfaces.library.roots.contfn_brent'

    _b_utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = cb_f
    try:
        _return_scalar = b_contfn_brent(
            a,
            b,
            eps,
            eta,
            _f,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _x_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x'],
    )(
        _x_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _x_py

def contfn_brent_rcomm(x, y, fx, tolx, c, ind, ir=0):
    u"""
    Zero of continuous function in a given interval, Brent algorithm
    (reverse communication).

    For full information please refer to the NAG Library document for
    c05az

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05azf.html

    Parameters
    ----------
    x : float
        On initial entry: `x` and `y` must define an initial interval
        [a,b] containing the zero, such that f(`x`)*f(`y`) <= 0.0. It is
        not necessary that `x` < `y`. Otherwise, `x` and `y` must be the
        same entities output by a previous call to
        ``contfn_brent_rcomm``.

    y : float
        On initial entry: `x` and `y` must define an initial interval
        [a,b] containing the zero, such that f(`x`)*f(`y`) <= 0.0. It is
        not necessary that `x` < `y`. Otherwise, `x` and `y` must be the
        same entities output by a previous call to
        ``contfn_brent_rcomm``.

    fx : float
        On initial entry: if `ind` = 1, `fx` need not be set.

        If `ind` = -1, `fx` must contain f(`x`) for the initial value of
        `x`.

        On intermediate entry: must contain f(`x`) for the current value
        of `x`.

    tolx : float
        On initial entry: the accuracy to which the zero is required.
        The type of error test is specified by `ir`.

    c : float, ndarray, shape (17), modified in place
        On initial entry: if `ind` = 1, no elements of `c` need be set.

        If `ind` = -1, `c`[0] must contain f(`y`), other elements of `c`
        need not be set.

        On final exit: is undefined.

    ind : int
        On initial entry: must be set to 1 or -1.

        `ind` = 1

          `fx` and `c`[0] need not be set.

        `ind` = -1

          `fx` and `c`[0] must contain f(`x`) and f(`y`) respectively.

    ir : int, optional
        On initial entry: indicates the type of error test.

        `ir` = 0

          The test is: |`x`-`y`| <= 2.0*`tolx`*max(1.0, |`x`|).

        `ir` = 1

          The test is: |`x`-`y`| <= 2.0*`tolx`.

        `ir` = 2

          The test is: |`x`-`y`| <= 2.0*`tolx`*|`x`|.

    Returns
    -------
    x : float
        On intermediate exit: `x` contains the point at which f must be
        evaluated before re-entry to the function. `y` has no external
        significance but must be communicated back to the next call of
        ``contfn_brent_rcomm`` as argument `y`.

        On final exit: `x` and `y` define a smaller interval containing
        the zero, such that f(`x`)*f(`y`) <= 0.0, and |`x`-`y`|
        satisfies the accuracy specified by `tolx` and `ir`, unless an
        error has occurred. If `errno` = 4, `x` and `y` generally
        contain very good approximations to a pole; if `errno` = 5, `x`
        and `y` generally contain very good approximations to the zero
        (see [Errors]). If a point `x` is found such that f(`x`) = 0.0,
        on final exit `x` = `y` (in this case there is no guarantee that
        `x` is a simple zero). In all cases, the value returned in `x`
        is the better approximation to the zero.

    y : float
        On intermediate exit: `x` contains the point at which f must be
        evaluated before re-entry to the function. `y` has no external
        significance but must be communicated back to the next call of
        ``contfn_brent_rcomm`` as argument `y`.

        On final exit: `x` and `y` define a smaller interval containing
        the zero, such that f(`x`)*f(`y`) <= 0.0, and |`x`-`y`|
        satisfies the accuracy specified by `tolx` and `ir`, unless an
        error has occurred. If `errno` = 4, `x` and `y` generally
        contain very good approximations to a pole; if `errno` = 5, `x`
        and `y` generally contain very good approximations to the zero
        (see [Errors]). If a point `x` is found such that f(`x`) = 0.0,
        on final exit `x` = `y` (in this case there is no guarantee that
        `x` is a simple zero). In all cases, the value returned in `x`
        is the better approximation to the zero.

    ind : int
        On intermediate exit: contains 2, 3 or 4. The calling program
        must evaluate f at `x`, storing the result in `fx`, and re-enter
        ``contfn_brent_rcomm`` with all other arguments unchanged.

        On final exit: contains 0.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, f(`x`) and f(`y`) have the same sign with neither
            equalling 0.0: f(`x`) = *<value>* and f(`y`) = *<value>*.

        (`errno` 2)
            On entry, `ind` = *<value>*.

            Constraint: `ind` = -1, 1, 2, 3 or 4.

        (`errno` 3)
            On entry, `tolx` = *<value>*.

            Constraint: `tolx` > 0.0.

        (`errno` 3)
            On entry, `ir` = *<value>*.

            Constraint: `ir` = 0, 1 or 2.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            The final interval may contain a pole rather than a zero.
            Note that this error exit is not completely reliable: it may
            be taken in extreme cases when [`x`,`y`] contains a zero, or
            it may not be taken when [`x`,`y`] contains a pole. Both
            these cases occur most frequently when `tolx` is large.

        (`errno` 5)
            The tolerance `tolx` has been set too small for the problem
            being solved. However, the values `x` and `y` returned may
            well be good approximations to the zero. `tolx` = *<value>*.
    """

    from ..base.roots import contfn_brent_rcomm as b_contfn_brent_rcomm

    fun_name = 'naginterfaces.library.roots.contfn_brent_rcomm'

    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
    )
    try:
        _return_scalars = b_contfn_brent_rcomm(
            x,
            y,
            fx,
            tolx,
            ir,
            c,
            ind,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _x_py = _return_scalars.x
    _y_py = _return_scalars.y
    _ind_py = _return_scalars.ind
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'y', 'ind'],
    )(
        _x_py,
        _y_py,
        _ind_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def lambertw_real(x, branch, offset):
    u"""
    Real values of Lambert's W function, W(x).

    For full information please refer to the NAG Library document for
    c05ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05baf.html

    Parameters
    ----------
    x : float
        If `offset` = True, `x` is the offset Delta x from -exp(-1) of
        the intended argument to W; that is, W(beta) is computed, where
        beta = -exp(-1)+Delta x.

        If `offset` = False, `x` is the argument x of the function; that
        is, W(beta) is computed, where beta = x.

    branch : int
        The real branch required.

        `branch` = 0

          The branch W_0 is selected.

        `branch` = -1

          The branch W_{-1} is selected.

    offset : bool
        Controls whether or not `x` is being specified as an offset from
        -exp(-1).

    Returns
    -------
    lambw : float
        The real values of Lambert's W function W(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `branch` = *<value>*.

            Constraint: `branch` = 0 or -1.

        (`errno` 1)
            On entry, `offset` = True and `x` = *<value>*.

            Constraint: if `offset` = True then `x` >= 0.0.

        (`errno` 1)
            On entry, `offset` = False and `x` = *<value>*.

            Constraint: if `offset` = False then `x` >= -exp(-1.0).

        (`errno` 1)
            On entry, `branch` = -1, `offset` = True and `x` =
            *<value>*.

            Constraint: if `branch` = -1 and `offset` = True then `x` <
            exp(-1.0).

        (`errno` 1)
            On entry, `branch` = -1, `offset` = False and `x` =
            *<value>*.

            Constraint: if `branch` = -1 and `offset` = False then `x` <
            0.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            For the given offset `x`, W is negligibly different from -1:
            `x` = *<value>*.

        (`errno` 2)
            `x` is close to -exp(-1). Enter `x` as an offset to -exp(-1)
            for greater accuracy: `x` = *<value>*.
    """

    from ..base.roots import lambertw_real as b_lambertw_real

    try:
        _return_scalar = b_lambertw_real(
            x,
            branch,
            offset,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['lambw'],
    )(
        _return_scalar,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_scalar

def lambertw_complex(branch, offset, z):
    u"""
    Values of Lambert's W function, W(z).

    For full information please refer to the NAG Library document for
    c05bb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05bbf.html

    Parameters
    ----------
    branch : int
        The branch required.

    offset : bool
        Controls whether or not `z` is being specified as an offset from
        -exp(-1).

    z : complex
        If `offset` = True, `z` is the offset Delta z from -exp(-1) of
        the intended argument to W; that is, W(beta) is computed, where
        beta = -exp(-1)+Delta z.

        If `offset` = False, `z` is the argument z of the function; that
        is, W(beta) is computed, where beta = z.

    Returns
    -------
    w : complex
        The value W(beta): see also the description of `z`.

    resid : float
        The residual |W(beta)exp(W(beta))-beta|: see also the
        description of `z`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            For the given offset `z`, W is negligibly different from -1:
            Re(`z`) = *<value>* and Im(`z`) = *<value>*.

        (`errno` 1)
            `z` is close to -exp(-1). Enter `z` as an offset to -exp(-1)
            for greater accuracy: Re(`z`) = *<value>* and Im(`z`) =
            *<value>*.

        (`errno` 2)
            The iterative procedure used internally did not converge in
            *<value>* iterations. Check the value of `resid` for the
            accuracy of `w`.
    """

    from ..base.roots import lambertw_complex as b_lambertw_complex

    try:
        _return_scalars = b_lambertw_complex(
            branch,
            offset,
            z,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _w_py = _return_scalars.w
    _resid_py = _return_scalars.resid
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['w', 'resid'],
    )(
        _w_py,
        _resid_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def sys_func_aa(fcn, x, atol=sqrt(machine.precision()), rtol=sqrt(machine.precision()), m=0, cndtol=0.0, astart=0, data=None):
    u"""
    Solution of a system of nonlinear equations using Anderson
    acceleration.

    For full information please refer to the NAG Library document for
    c05mb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05mbf.html

    Parameters
    ----------
    fcn : callable fvec = fcn(x, data=None)
        `fcn` must return the values of the functions f_k at a point x.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (n)
            The components of the point x at which the functions must be
            evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        fvec : float, array-like, shape (n)
            The function values f_k(x).

    x : float, array-like, shape (n)
        An initial guess at the solution vector, x^_0.

    atol : float, optional
        The absolute convergence criterion; see `rtol`.

    rtol : float, optional
        The relative convergence criterion. At each iteration
        ||f(x^_i)|| is computed. The iteration is deemed to have
        converged if ||f(x^_i)|| <= max(`atol`, `rtol`*||f(x^_0)||).

    m : int, optional
        m, the number of previous iterates to use in Anderson
        acceleration. If m = 0, Anderson acceleration is not used.

    cndtol : float, optional
        The maximum allowable condition number for the triangular QR
        factor generated during Anderson acceleration. At each
        iteration, if the condition number exceeds `cndtol`, columns are
        deleted until it is sufficiently small.

        If `cndtol` = 0.0, no condition number tests are performed.

    astart : int, optional
        The number of iterations by which to delay the start of Anderson
        acceleration.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    x : float, ndarray, shape (n)
        The final estimate of the solution vector.

    fvec : float, ndarray, shape (n)
        The function values at the final point, `x`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n > 0.

        (`errno` 2)
            On entry, `atol` = *<value>*.

            Constraint: `atol` >= 0.0.

        (`errno` 3)
            On entry, `rtol` = *<value>*.

            Constraint: `rtol` >= 0.0.

        (`errno` 4)
            On entry, `m` = *<value>* and n = *<value>*.

            Constraint: 0 <= `m` <= n.

        (`errno` 5)
            On entry, `cndtol` = *<value>*.

            Constraint: `cndtol` >= 0.0.

        (`errno` 6)
            On entry, `astart` = *<value>*.

            Constraint: `astart` >= 0.

        (`errno` 7)
            An error occurred in evaluating the QR decomposition during
            Anderson acceleration. This may be due to slow convergence
            of the iteration. Try setting the value of `cndtol`. If
            condition number tests are already performed, try decreasing
            `cndtol`.

        (`errno` 11)
            The iteration has diverged and subsequent iterates are too
            large to be computed in floating-point arithmetic.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 8)
            The iteration is not making good progress. This failure exit
            may indicate that the system does not have a zero, or that
            the solution is very close to the origin. Rerunning
            ``sys_func_aa`` from a different starting point may avoid
            the region of difficulty.

        (`errno` 9)
            There have been at least 200*(n+1) calls to `fcn`. Consider
            restarting the calculation from the point held in `x`.

    NagCallbackTerminateWarning
        (`errno` 10)
            Termination requested in `fcn`.
    """

    from ..base.roots import sys_func_aa as b_sys_func_aa

    fun_name = 'naginterfaces.library.roots.sys_func_aa'

    cb_fcn_name = fun_name + ':fcn'

    def cb_fcn(n, x, fvec, data=None):
        """Wrapping user's fcn. Called as base fcn."""
        if data is not None:
            _fvec_fcn = fcn(
                x,
                data,
            )
        else:
            _fvec_fcn = fcn(
                x,
            )
        fvec[:] = _utils._to_ndarray(
            {'fun_name': cb_fcn_name, 'returned': 'fvec'},
            _fvec_fcn,
            copy=False,
            exp_shape=(n,),
            exp_class=_b_utils._EngineFloat64ArrayType,
        )

    _b_utils._check_callable(fcn, {'fun_name': fun_name, 'entity': 'fcn'})
    _fcn = cb_fcn
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _fvec = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_sys_func_aa(
            _fcn,
            _n,
            _x,
            _fvec,
            atol,
            rtol,
            m,
            cndtol,
            astart,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _x_py = _x
    _fvec_py = _fvec
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'fvec'],
    )(
        _x_py,
        _fvec_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def sys_func_aa_rcomm(irevcm, x, fvec, comm, atol=sqrt(machine.precision()), rtol=sqrt(machine.precision()), m=0, cndtol=0.0, astart=0):
    u"""
    Solution of a system of nonlinear equations using Anderson
    acceleration (reverse communication).

    For full information please refer to the NAG Library document for
    c05md

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05mdf.html

    Parameters
    ----------
    irevcm : int
        On initial entry: must have the value 0.

    x : float, ndarray, shape (n), modified in place
        On initial entry: an initial guess at the solution vector, x^_0.

        On intermediate exit: contains the current point.

        On final exit: the final estimate of the solution vector.

    fvec : float, ndarray, shape (n), modified in place
        On initial entry: need not be set.

        On intermediate entry: if `irevcm` = 1, `fvec` must not be
        changed.

        If `irevcm` = 2, `fvec` must be set to the values of the
        functions computed at the current point `x`, f(x^_i).

        On final exit: the function values at the final point, `x`.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    atol : float, optional
        On initial entry: the absolute convergence criterion.

    rtol : float, optional
        On initial entry: the relative convergence criterion. At each
        iteration ||f(x^_i)|| is computed. The iteration is deemed to
        have converged if ||f(x^_i)|| <= max(`atol`,
        `rtol`*||f(x^_0)||).

    m : int, optional
        On initial entry: m, the number of previous iterates to use in
        Anderson acceleration. If m = 0, Anderson acceleration is not
        used.

    cndtol : float, optional
        On initial entry: the maximum allowable condition number for the
        triangular QR factor generated during Anderson acceleration. At
        each iteration, if the condition number exceeds `cndtol`,
        columns are deleted until it is sufficiently small.

        If `cndtol` = 0.0, no condition number tests are performed.

    astart : int, optional
        On initial entry: the number of iterations by which to delay the
        start of Anderson acceleration.

    Returns
    -------
    irevcm : int
        On intermediate exit: specifies what action you must take before
        re-entering ``sys_func_aa_rcomm`` with `irevcm` set to this
        value. The value of this argument should be interpreted as
        follows:

        `irevcm` = 1

          Indicates the start of a new iteration. No action is required
          by you, but `x` and `fvec` are available for printing, and a
          limit on the number of iterations can be applied.

        `irevcm` = 2

          Indicates that before re-entry to ``sys_func_aa_rcomm``,
          `fvec` must contain the function values f(x^_i).

        On final exit: `irevcm` = 0 and the algorithm has terminated.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On initial entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 0.

        (`errno` 1)
            On intermediate entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 1 or 2.

        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n > 0.

        (`errno` 3)
            On entry, `atol` = *<value>*.

            Constraint: `atol` >= 0.0.

        (`errno` 4)
            On entry, `rtol` = *<value>*.

            Constraint: `rtol` >= 0.0.

        (`errno` 5)
            On entry, `m` = *<value>* and n = *<value>*.

            Constraint: 0 <= `m` <= n.

        (`errno` 6)
            On entry, `cndtol` = *<value>*.

            Constraint: `cndtol` >= 0.0.

        (`errno` 7)
            On entry, `astart` = *<value>*.

            Constraint: `astart` >= 0.

        (`errno` 8)
            An error occurred in evaluating the QR decomposition during
            Anderson acceleration. This may be due to slow convergence
            of the iteration. Try setting the value of `cndtol`. If
            condition number tests are already performed, try decreasing
            `cndtol`.

        (`errno` 9)
            The iteration is not making good progress, as measured by
            the reduction in the norm of f(x) in the last *<value>*
            iterations.

        (`errno` 10)
            The iteration has diverged and subsequent iterates are too
            large to be computed in floating-point arithmetic.
    """

    from ..base.roots import sys_func_aa_rcomm as b_sys_func_aa_rcomm

    fun_name = 'naginterfaces.library.roots.sys_func_aa_rcomm'

    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'fvec'},
        fvec,
    )
    _return_scalar = b_sys_func_aa_rcomm(
        irevcm,
        _n,
        x,
        fvec,
        atol,
        rtol,
        m,
        cndtol,
        astart,
        comm,
    )
    _irevcm_py = _return_scalar
    return _irevcm_py

def sys_func_easy(fcn, x, xtol=sqrt(machine.precision()), data=None):
    u"""
    Solution of a system of nonlinear equations using function values
    only (easy-to-use).

    For full information please refer to the NAG Library document for
    c05qb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05qbf.html

    Parameters
    ----------
    fcn : callable fvec = fcn(x, data=None)
        `fcn` must return the values of the functions f_i at a point x.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (n)
            The components of the point x at which the functions must be
            evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        fvec : float, array-like, shape (n)
            The function values f_i(x).

    x : float, array-like, shape (n)
        An initial guess at the solution vector.

    xtol : float, optional
        The accuracy in `x` to which the solution is required.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    x : float, ndarray, shape (n)
        The final estimate of the solution vector.

    fvec : float, ndarray, shape (n)
        The function values at the final point returned in `x`.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, n = *<value>*.

            Constraint: n > 0.

        (`errno` 12)
            On entry, `xtol` = *<value>*.

            Constraint: `xtol` >= 0.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            There have been at least 200*(n+1) calls to `fcn`. Consider
            restarting the calculation from the point held in `x`.

        (`errno` 3)
            No further improvement in the solution is possible. `xtol`
            is too small: `xtol` = *<value>*.

        (`errno` 4)
            The iteration is not making good progress. This failure exit
            may indicate that the system does not have a zero, or that
            the solution is very close to the origin (see [Accuracy]).
            Otherwise, rerunning ``sys_func_easy`` from a different
            starting point may avoid the region of difficulty.

    NagCallbackTerminateWarning
        (`errno` 5)
            Termination requested in `fcn`.
    """

    from ..base.roots import sys_func_easy as b_sys_func_easy

    fun_name = 'naginterfaces.library.roots.sys_func_easy'

    cb_fcn_name = fun_name + ':fcn'

    def cb_fcn(n, x, fvec, data=None):
        """Wrapping user's fcn. Called as base fcn."""
        if data is not None:
            _fvec_fcn = fcn(
                x,
                data,
            )
        else:
            _fvec_fcn = fcn(
                x,
            )
        fvec[:] = _utils._to_ndarray(
            {'fun_name': cb_fcn_name, 'returned': 'fvec'},
            _fvec_fcn,
            copy=False,
            exp_shape=(n,),
            exp_class=_b_utils._EngineFloat64ArrayType,
        )

    _b_utils._check_callable(fcn, {'fun_name': fun_name, 'entity': 'fcn'})
    _fcn = cb_fcn
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _fvec = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_sys_func_easy(
            _fcn,
            _n,
            _x,
            _fvec,
            xtol,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _x_py = _x
    _fvec_py = _fvec
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'fvec'],
    )(
        _x_py,
        _fvec_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def sys_func_expert(fcnval, x, ml, mu, mode, diag, nprint, fcnmon=None, xtol=sqrt(machine.precision()), maxfev=None, epsfcn=0.0, factor=100.0, data=None):
    u"""
    Solution of a system of nonlinear equations using function values
    only (comprehensive).

    For full information please refer to the NAG Library document for
    c05qc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05qcf.html

    Parameters
    ----------
    fcnval : callable fvec = fcnval(x, data=None)
        `fcnval` must return the values of the functions f_i at a point
        x.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (n)
            The components of the point x at which the functions must be
            evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        fvec : float, array-like, shape (n)
            `fvec` must contain the function values f_i(x).

    x : float, array-like, shape (n)
        An initial guess at the solution vector.

    ml : int
        The number of subdiagonals within the band of the Jacobian
        matrix. (If the Jacobian is not banded, or you are unsure, set
        `ml` = n-1.)

    mu : int
        The number of superdiagonals within the band of the Jacobian
        matrix. (If the Jacobian is not banded, or you are unsure, set
        `mu` = n-1.)

    mode : int
        Indicates whether or not you have provided scaling factors in
        `diag`.

        If `mode` = 2, the scaling must have been specified in `diag`.

        Otherwise, if `mode` = 1, the variables will be scaled
        internally.

    diag : float, array-like, shape (n)
        If `mode` = 2, `diag` must contain multiplicative scale factors
        for the variables.

        If `mode` = 1, `diag` need not be set.

    nprint : int
        Indicates whether (and how often) calls to `fcnmon` are to be
        made.

        `nprint` <= 0

          No calls are made.

        `nprint` > 0

          `fcnmon` is called at the beginning of the first iteration,
          every `nprint` iterations thereafter and immediately before
          the return from ``sys_func_expert``.

    fcnmon : None or callable fcnmon(x, fvec, data=None), optional
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `fcnmon` may be used to monitor the progress of the algorithm.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (n)
            The components of the current evaluation point x.

        fvec : float, ndarray, shape (n)
            This argument contains the function values f_i(x).

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    xtol : float, optional
        The accuracy in `x` to which the solution is required.

    maxfev : int, optional
        Note: if this argument is None then a default value will be
        used, determined as follows: 200*(n+1).

        The maximum number of calls to `fcnval`. ``sys_func_expert``
        will exit with `errno` = 2, if, at the end of an iteration, the
        number of calls to `fcnval` exceeds `maxfev`.

    epsfcn : float, optional
        A rough estimate of the largest relative error in the functions.
        It is used in determining a suitable step for a forward
        difference approximation to the Jacobian. If `epsfcn` is less
        than machine precision (returned by ``machine.precision``) then
        machine precision is used. Consequently a value of 0.0 will
        often be suitable.

    factor : float, optional
        A quantity to be used in determining the initial step bound. In
        most cases, `factor` should lie between 0.1 and 100.0. (The step
        bound is `factor`*||`diag`*`x`||_2 if this is nonzero; otherwise
        the bound is `factor`.)

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    x : float, ndarray, shape (n)
        The final estimate of the solution vector.

    fvec : float, ndarray, shape (n)
        The function values at the final point returned in `x`.

    diag : float, ndarray, shape (n)
        The scale factors actually used (computed internally if `mode` =
        1).

    nfev : int
        The number of calls made to `fcnval`.

    fjac : float, ndarray, shape (n, n)
        The orthogonal matrix Q produced by the QR factorization of the
        final approximate Jacobian.

    r : float, ndarray, shape (n*(n+1)/2)
        The upper triangular matrix R produced by the QR factorization
        of the final approximate Jacobian, stored row-wise.

    qtf : float, ndarray, shape (n)
        The vector Q^Tf.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, n = *<value>*.

            Constraint: n > 0.

        (`errno` 12)
            On entry, `xtol` = *<value>*.

            Constraint: `xtol` >= 0.0.

        (`errno` 13)
            On entry, `mode` = *<value>*.

            Constraint: `mode` = 1 or 2.

        (`errno` 14)
            On entry, `factor` = *<value>*.

            Constraint: `factor` > 0.0.

        (`errno` 15)
            On entry, `mode` = 2 and `diag` contained a non-positive
            element.

        (`errno` 16)
            On entry, `ml` = *<value>*.

            Constraint: `ml` >= 0.

        (`errno` 17)
            On entry, `mu` = *<value>*.

            Constraint: `mu` >= 0.

        (`errno` 18)
            On entry, `maxfev` = *<value>*.

            Constraint: `maxfev` > 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            There have been at least `maxfev` calls to `fcnval`:
            `maxfev` = *<value>*. Consider restarting the calculation
            from the final point held in `x`.

        (`errno` 3)
            No further improvement in the solution is possible. `xtol`
            is too small: `xtol` = *<value>*.

        (`errno` 4)
            The iteration is not making good progress, as measured by
            the improvement from the last *<value>* Jacobian
            evaluations.

        (`errno` 5)
            The iteration is not making good progress, as measured by
            the improvement from the last *<value>* iterations.

    NagCallbackTerminateWarning
        (`errno` 6)
            Termination requested in `fcnval` or `fcnmon`.
    """

    from ..base.roots import sys_func_expert as b_sys_func_expert

    fun_name = 'naginterfaces.library.roots.sys_func_expert'

    cb_fcnval_name = fun_name + ':fcnval'

    def cb_fcnval(n, x, fvec, data=None):
        """Wrapping user's fcnval. Called as base fcnval."""
        if data is not None:
            _fvec_fcnval = fcnval(
                x,
                data,
            )
        else:
            _fvec_fcnval = fcnval(
                x,
            )
        fvec[:] = _utils._to_ndarray(
            {'fun_name': cb_fcnval_name, 'returned': 'fvec'},
            _fvec_fcnval,
            copy=False,
            exp_shape=(n,),
            exp_class=_b_utils._EngineFloat64ArrayType,
        )

    def cb_fcnmon(n, x, fvec, data=None):
        """Wrapping user's fcnmon. Called as base fcnmon."""
        if data is not None:
            fcnmon(
                x,
                fvec,
                data,
            )
        else:
            fcnmon(
                x,
                fvec,
            )

    _b_utils._check_callable(fcnval, {'fun_name': fun_name, 'entity': 'fcnval'})
    _fcnval = cb_fcnval
    _b_utils._check_callable(fcnmon, {'fun_name': fun_name, 'entity': 'fcnmon'}, allow_none=True)
    _fcnmon = None if fcnmon is None else cb_fcnmon
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _fvec = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _maxfev = 200*(_n_py+1) if maxfev is None else maxfev
    _diag = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'diag'},
        diag,
        copy=True,
    )
    _fjac = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _r = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py*(_n_py+1)//2,),
    )
    _qtf = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalar = b_sys_func_expert(
            _fcnval,
            _fcnmon,
            _n,
            _x,
            _fvec,
            xtol,
            _maxfev,
            ml,
            mu,
            epsfcn,
            mode,
            _diag,
            factor,
            nprint,
            _fjac,
            _r,
            _qtf,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _x_py = _x
    _diag_py = _diag
    _fvec_py = _fvec
    _nfev_py = _return_scalar
    _fjac_py = _fjac
    _r_py = _r
    _qtf_py = _qtf
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'fvec', 'diag', 'nfev', 'fjac', 'r', 'qtf'],
    )(
        _x_py,
        _fvec_py,
        _diag_py,
        _nfev_py,
        _fjac_py,
        _r_py,
        _qtf_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def sys_func_rcomm(irevcm, x, fvec, ml, mu, mode, diag, fjac, r, qtf, comm, xtol=sqrt(machine.precision()), epsfcn=0.0, factor=100.0):
    u"""
    Solution of a system of nonlinear equations using function values
    only (reverse communication).

    For full information please refer to the NAG Library document for
    c05qd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05qdf.html

    Parameters
    ----------
    irevcm : int
        On initial entry: must have the value 0.

    x : float, ndarray, shape (n), modified in place
        On initial entry: an initial guess at the solution vector.

        On intermediate exit: contains the current point.

        On final exit: the final estimate of the solution vector.

    fvec : float, ndarray, shape (n), modified in place
        On initial entry: need not be set.

        On intermediate entry: if `irevcm` = 1, `fvec` must not be
        changed.

        If `irevcm` = 2, `fvec` must be set to the values of the
        functions computed at the current point `x`.

        On final exit: the function values at the final point, `x`.

    ml : int
        On initial entry: the number of subdiagonals within the band of
        the Jacobian matrix. (If the Jacobian is not banded, or you are
        unsure, set `ml` = n-1.)

    mu : int
        On initial entry: the number of superdiagonals within the band
        of the Jacobian matrix. (If the Jacobian is not banded, or you
        are unsure, set `mu` = n-1.)

    mode : int
        On initial entry: indicates whether or not you have provided
        scaling factors in `diag`.

        If `mode` = 2, the scaling must have been supplied in `diag`.

        Otherwise, if `mode` = 1, the variables will be scaled
        internally.

    diag : float, ndarray, shape (n), modified in place
        On entry: if `mode` = 2, `diag` must contain multiplicative
        scale factors for the variables.

        If `mode` = 1, `diag` need not be set.

        On exit: the scale factors actually used (computed internally if
        `mode` = 1).

    fjac : float, ndarray, shape (n, n), modified in place
        On initial entry: need not be set.

        On intermediate exit: must not be changed.

        On final exit: the orthogonal matrix Q produced by the QR
        factorization of the final approximate Jacobian.

    r : float, ndarray, shape (n*(n+1)/2), modified in place
        On initial entry: need not be set.

        On intermediate exit: must not be changed.

        On final exit: the upper triangular matrix R produced by the QR
        factorization of the final approximate Jacobian, stored
        row-wise.

    qtf : float, ndarray, shape (n), modified in place
        On initial entry: need not be set.

        On intermediate exit: must not be changed.

        On final exit: the vector Q^Tf.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    xtol : float, optional
        On initial entry: the accuracy in `x` to which the solution is
        required.

    epsfcn : float, optional
        On initial entry: the order of the largest relative error in the
        functions. It is used in determining a suitable step for a
        forward difference approximation to the Jacobian. If `epsfcn` is
        less than machine precision (returned by ``machine.precision``)
        then machine precision is used. Consequently a value of 0.0 will
        often be suitable.

    factor : float, optional
        On initial entry: a quantity to be used in determining the
        initial step bound. In most cases, `factor` should lie between
        0.1 and 100.0. (The step bound is `factor`*||`diag`*`x`||_2 if
        this is nonzero; otherwise the bound is `factor`.)

    Returns
    -------
    irevcm : int
        On intermediate exit: specifies what action you must take before
        re-entering ``sys_func_rcomm`` with `irevcm` set to this value.
        The value of this argument should be interpreted as follows:

        `irevcm` = 1

          Indicates the start of a new iteration. No action is required
          by you, but `x` and `fvec` are available for printing.

        `irevcm` = 2

          Indicates that before re-entry to ``sys_func_rcomm``, `fvec`
          must contain the function values f_i(x).

        On final exit: `irevcm` = 0 and the algorithm has terminated.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 0, 1 or 2.

        (`errno` 11)
            On entry, n = *<value>*.

            Constraint: n > 0.

        (`errno` 12)
            On entry, `xtol` = *<value>*.

            Constraint: `xtol` >= 0.0.

        (`errno` 13)
            On entry, `mode` = *<value>*.

            Constraint: `mode` = 1 or 2.

        (`errno` 14)
            On entry, `factor` = *<value>*.

            Constraint: `factor` > 0.0.

        (`errno` 15)
            On entry, `mode` = 2 and `diag` contained a non-positive
            element.

        (`errno` 16)
            On entry, `ml` = *<value>*.

            Constraint: `ml` >= 0.

        (`errno` 17)
            On entry, `mu` = *<value>*.

            Constraint: `mu` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            No further improvement in the solution is possible. `xtol`
            is too small: `xtol` = *<value>*.

        (`errno` 4)
            The iteration is not making good progress, as measured by
            the improvement from the last *<value>* Jacobian
            evaluations.

        (`errno` 5)
            The iteration is not making good progress, as measured by
            the improvement from the last *<value>* iterations.

    See Also
    --------
    :meth:`naginterfaces.library.examples.roots.sys_func_rcomm_ex.main`
    """

    from ..base.roots import sys_func_rcomm as b_sys_func_rcomm

    fun_name = 'naginterfaces.library.roots.sys_func_rcomm'

    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'fvec'},
        fvec,
    )
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'diag'},
        diag,
    )
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'fjac'},
        fjac,
    )
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'r'},
        r,
    )
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'qtf'},
        qtf,
    )
    try:
        _return_scalar = b_sys_func_rcomm(
            irevcm,
            _n,
            x,
            fvec,
            xtol,
            ml,
            mu,
            epsfcn,
            mode,
            diag,
            factor,
            fjac,
            r,
            qtf,
            comm,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _irevcm_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['irevcm'],
    )(
        _irevcm_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _irevcm_py

def sparsys_func_easy(fcn, x, xtol=sqrt(machine.precision()), nnz=None, comm=None, data=None):
    u"""
    Solution of a sparse system of nonlinear equations using function
    values only (easy-to-use).

    For full information please refer to the NAG Library document for
    c05qs

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05qsf.html

    Parameters
    ----------
    fcn : callable fcn(indf, x, fvec, data=None)
        `fcn` must return the values of the functions f_i at a point x.

        Parameters
        ~~~~~~~~~~
        indf : int, ndarray, shape (lindf)
            Note: this argument represents array indices; the values
            supplied will be base-1.

            This argument specifies the indices i for which values of
            f_i(x) must be computed. The indices are specified in
            strictly ascending order.

        x : float, ndarray, shape (n)
            The components of the point x at which the functions must be
            evaluated. `x`[i-1] contains the coordinate x_i.

        fvec : float, ndarray, shape (n), to be modified in place
            On exit: `fvec`[i-1] must contain the function values
            f_i(x), for all indices i in `indf`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    x : float, array-like, shape (n)
        An initial guess at the solution vector. `x`[i-1] must contain
        the coordinate x_i.

    xtol : float, optional
        The accuracy in `x` to which the solution is required.

    nnz : int, optional
        Note: if this argument is None then a default value will be
        used, determined as follows: n*n.

        If supplied `nnz` should be an upper estimate of the number of
        nonzero entries found in the Jacobian.

        The size of the data that is self communicated to this routine
        via `comm` is a function of `nnz`.

        A suitable overestimate for `nnz` is n^2, but this may be
        suboptimal when n is large.

        If an existing sparsity pattern for the problem is communicated
        via `comm`, `nnz` will be ignored.

        The value that was supplied in the call to ``sparsys_func_easy``
        which computed that sparsity pattern will be extracted from
        `comm` and used instead.

    comm : None or dict, communication object, optional, modified in
    place
        Optionally, on entry: an existing sparsity pattern for the
        Jacobian, computed by a subsequent call. Otherwise, supplying an
        uninitialized object indicates it is the first time
        ``sparsys_func_easy`` is called for this specific problem.

        On exit, if not None on entry: will be populated with the
        sparsity pattern of the Jacobian. On subsequent calls, the same
        communication object can be passed in again if the problem has a
        Jacobian of the same sparsity pattern.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    x : float, ndarray, shape (n)
        The final estimate of the solution vector.

    fvec : float, ndarray, shape (n)
        The function values at the final point returned in `x`.
        `fvec`[i-1] contains the function values f_i.

    actual_nnz : int
        The number of nonzero entries found in the Jacobian.

    Raises
    ------
    NagValueError
        (`errno` 6)
            On entry, the supplied `nnz` = *<value>* was too small for
            this problem.

        (`errno` 7)
            On entry, the supplied `nnz` = *<value>* was too small for
            this problem.

        (`errno` 9)
            An internal error has occurred. Code = *<value>*.

        (`errno` 11)
            On entry, n = *<value>*.

            Constraint: n > 0.

        (`errno` 12)
            On entry, `xtol` = *<value>*.

            Constraint: `xtol` >= 0.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            There have been at least 200*(n+1) calls to `fcn`. Consider
            restarting the calculation from the point held in `x` and
            supplying the populated sparsity pattern from `comm`.

        (`errno` 3)
            No further improvement in the solution is possible. `xtol`
            is too small: `xtol` = *<value>*.

        (`errno` 4)
            The iteration is not making good progress. This failure exit
            may indicate that the system does not have a zero, or that
            the solution is very close to the origin (see [Accuracy]).
            Otherwise, rerunning ``sparsys_func_easy`` from a different
            starting point may avoid the region of difficulty. The
            condition number of the Jacobian is *<value>*.

    NagCallbackTerminateWarning
        (`errno` 5)
            Termination requested in `fcn`.
    """

    from ..base.roots import sparsys_func_easy as b_sparsys_func_easy

    def cb_fcn(n, lindf, indf, x, fvec, data=None):
        """Wrapping user's fcn. Called as base fcn."""
        if data is not None:
            fcn(
                indf,
                x,
                fvec,
                data,
            )
        else:
            fcn(
                indf,
                x,
                fvec,
            )

    fun_name = 'naginterfaces.library.roots.sparsys_func_easy'

    _b_utils._check_callable(fcn, {'fun_name': fun_name, 'entity': 'fcn'})
    _fcn = cb_fcn
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _fvec = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _nnz = _n_py*_n_py if nnz is None else nnz
    try:
        _return_scalar = b_sparsys_func_easy(
            _fcn,
            _n,
            _x,
            _fvec,
            xtol,
            _nnz,
            comm,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _x_py = _x
    _fvec_py = _fvec
    _actual_nnz_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'fvec', 'actual_nnz'],
    )(
        _x_py,
        _fvec_py,
        _actual_nnz_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def sys_deriv_easy(fcnval, fcnjac, x, xtol=sqrt(machine.precision()), data=None, spiked_sorder='C'):
    u"""
    Solution of a system of nonlinear equations using first derivatives
    (easy-to-use).

    For full information please refer to the NAG Library document for
    c05rb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05rbf.html

    Parameters
    ----------
    fcnval : callable fvec = fcnval(x, data=None)
        `fcnval` must return the values of the functions f_i at a point
        x.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (n)
            The components of the point x at which the functions must be
            evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        fvec : float, array-like, shape (n)
            `fvec` must contain the function values f_i(x).

    fcnjac : callable fjac = fcnjac(x, data=None)
        `fcnjac` must return the Jacobian at x.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (n)
            The components of the point x at which the Jacobian must be
            evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        fjac : float, array-like, shape (n, n)
            `fjac`[i-1,j-1] must contain the value of df_i/dx_j at the
            point x, for j = 1,2,...,n, for i = 1,2,...,n.

    x : float, array-like, shape (n)
        An initial guess at the solution vector.

    xtol : float, optional
        The accuracy in `x` to which the solution is required.

    data : arbitrary, optional
        User-communication data for callback functions.

    spiked_sorder : str, optional
        If `fjac` in `fcnjac` is spiked (i.e., has unit extent in all
        but one dimension, or has size 1) and is treated as contiguous
        in both the Fortran and the C sense by NumPy, `spiked_sorder`
        selects the storage order to associate with it in the NAG
        Engine:

        spiked_sorder = 'C'
          row-major storage will be used;

        spiked_sorder = 'F'
          column-major storage will be used.

    Returns
    -------
    x : float, ndarray, shape (n)
        The final estimate of the solution vector.

    fvec : float, ndarray, shape (n)
        The function values at the final point returned in `x`.

    fjac : float, ndarray, shape (n, n)
        The orthogonal matrix Q produced by the QR factorization of the
        final approximate Jacobian.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, n = *<value>*.

            Constraint: n > 0.

        (`errno` 12)
            On entry, `xtol` = *<value>*.

            Constraint: `xtol` >= 0.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            There have been at least 100*(n+1) calls to `fcnval` and
            `fcnjac` combined. Consider restarting the calculation from
            the point held in `x`.

        (`errno` 3)
            No further improvement in the solution is possible. `xtol`
            is too small: `xtol` = *<value>*.

        (`errno` 4)
            The iteration is not making good progress. This failure exit
            may indicate that the system does not have a zero, or that
            the solution is very close to the origin (see [Accuracy]).
            Otherwise, rerunning ``sys_deriv_easy`` from a different
            starting point may avoid the region of difficulty.

    NagCallbackTerminateWarning
        (`errno` 5)
            Termination requested in `fcnval` or `fcnjac`.
    """

    from ..base.roots import sys_deriv_easy as b_sys_deriv_easy

    fun_name = 'naginterfaces.library.roots.sys_deriv_easy'

    cb_fcnval_name = fun_name + ':fcnval'

    def cb_fcnval(n, x, fvec, data=None):
        """Wrapping user's fcnval. Called as base fcnval."""
        if data is not None:
            _fvec_fcnval = fcnval(
                x,
                data,
            )
        else:
            _fvec_fcnval = fcnval(
                x,
            )
        fvec[:] = _utils._to_ndarray(
            {'fun_name': cb_fcnval_name, 'returned': 'fvec'},
            _fvec_fcnval,
            copy=False,
            exp_shape=(n,),
            exp_class=_b_utils._EngineFloat64ArrayType,
        )

    cb_fcnjac_name = fun_name + ':fcnjac'

    def cb_fcnjac(n, x, fjac, data=None):
        """Wrapping user's fcnjac. Called as base fcnjac."""
        if data is not None:
            _fjac_fcnjac = fcnjac(
                x,
                data,
            )
        else:
            _fjac_fcnjac = fcnjac(
                x,
            )
        _fjac_fcnjac_np = _utils._to_ndarray(
            {'fun_name': cb_fcnjac_name, 'returned': 'fjac'},
            _fjac_fcnjac,
            copy=False,
            exp_shape=(n, n),
            exp_class=_b_utils._EngineFloat64ArrayType,
        )
        _utils._check_callback_sorder(
            {'fun_name': cb_fcnjac_name},
            ((_fjac_fcnjac_np, 'fjac'),),
            wrapper_sorder=_m_sorder,
            spiked_sorder=spiked_sorder,
        )
        fjac[:, :] = _fjac_fcnjac_np

    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        (),
        spiked_sorder=spiked_sorder,
    )
    _b_utils._check_callable(fcnval, {'fun_name': fun_name, 'entity': 'fcnval'})
    _fcnval = cb_fcnval
    _b_utils._check_callable(fcnjac, {'fun_name': fun_name, 'entity': 'fcnjac'})
    _fcnjac = cb_fcnjac
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _fvec = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _fjac = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    try:
        b_sys_deriv_easy(
            _fcnval,
            _fcnjac,
            _n,
            _x,
            _fvec,
            _fjac,
            xtol,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _x_py = _x
    _fvec_py = _fvec
    _fjac_py = _fjac
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'fvec', 'fjac'],
    )(
        _x_py,
        _fvec_py,
        _fjac_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def sys_deriv_expert(fcnval, fcnjac, fcnmon, x, mode, diag, nprint, xtol=sqrt(machine.precision()), maxfev=None, factor=100.0, data=None, spiked_sorder='C'):
    u"""
    Solution of a system of nonlinear equations using first derivatives
    (comprehensive).

    For full information please refer to the NAG Library document for
    c05rc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05rcf.html

    Parameters
    ----------
    fcnval : callable fvec = fcnval(x, data=None)
        `fcnval` must return the values of the functions f_i at a point
        x.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (n)
            The components of the point x at which the functions must be
            evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        fvec : float, array-like, shape (n)
            `fvec` must contain the function values f_i(x).

    fcnjac : callable fjac = fcnjac(x, data=None)
        `fcnjac` must return the Jacobian at x.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (n)
            The components of the point x at which the Jacobian must be
            evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        fjac : float, array-like, shape (n, n)
            `fjac`[i-1,j-1] must contain the value of df_i/dx_j at the
            point x, for j = 1,2,...,n, for i = 1,2,...,n.

    fcnmon : callable fcnmon(x, fvec, fjac, data=None)
        `fcnmon` may be used to monitor the progress of the algorithm.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (n)
            The components of the current evaluation point x.

        fvec : float, ndarray, shape (n)
            This argument contains the function values f_i(x).

        fjac : float, ndarray, shape (n, n)
            `fjac`[i-1,j-1] contains the value of df_i/dx_j at the point
            x, for j = 1,2,...,n, for i = 1,2,...,n.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    x : float, array-like, shape (n)
        An initial guess at the solution vector.

    mode : int
        Indicates whether or not you have provided scaling factors in
        `diag`.

        If `mode` = 2, the scaling must have been specified in `diag`.

        Otherwise, if `mode` = 1, the variables will be scaled
        internally.

    diag : float, array-like, shape (n)
        If `mode` = 2, `diag` must contain multiplicative scale factors
        for the variables.

        If `mode` = 1, `diag` need not be set.

    nprint : int
        Indicates whether (and how often) calls to `fcnmon` are to be
        made.

        `nprint` <= 0

          No calls are made.

        `nprint` > 0

          `fcnmon` is called at the beginning of the first iteration,
          every `nprint` iterations thereafter and immediately before
          the return from ``sys_deriv_expert``.

    xtol : float, optional
        The accuracy in `x` to which the solution is required.

    maxfev : int, optional
        Note: if this argument is None then a default value will be
        used, determined as follows: 100*(n+1).

        The maximum number of calls to `fcnval` and `fcnjac` combined.
        ``sys_deriv_expert`` will exit with `errno` = 2, if, at the end
        of an iteration, the number of calls to `fcnval` and `fcnjac`
        combined exceeds `maxfev`.

    factor : float, optional
        A quantity to be used in determining the initial step bound. In
        most cases, `factor` should lie between 0.1 and 100.0. (The step
        bound is `factor`*||`diag`*`x`||_2 if this is nonzero; otherwise
        the bound is `factor`.)

    data : arbitrary, optional
        User-communication data for callback functions.

    spiked_sorder : str, optional
        If `fjac` in `fcnjac` is spiked (i.e., has unit extent in all
        but one dimension, or has size 1) and is treated as contiguous
        in both the Fortran and the C sense by NumPy, `spiked_sorder`
        selects the storage order to associate with it in the NAG
        Engine:

        spiked_sorder = 'C'
          row-major storage will be used;

        spiked_sorder = 'F'
          column-major storage will be used.

    Returns
    -------
    x : float, ndarray, shape (n)
        The final estimate of the solution vector.

    fvec : float, ndarray, shape (n)
        The function values at the final point returned in `x`.

    fjac : float, ndarray, shape (n, n)
        The orthogonal matrix Q produced by the QR factorization of the
        final approximate Jacobian.

    diag : float, ndarray, shape (n)
        The scale factors actually used (computed internally if `mode` =
        1).

    nfev : int
        The number of calls made to `fcnval` and `fcnjac` combined to
        evaluate the functions.

    njev : int
        The number of calls made to `fcnjac`.

    r : float, ndarray, shape (n*(n+1)/2)
        The upper triangular matrix R produced by the QR factorization
        of the final approximate Jacobian, stored row-wise.

    qtf : float, ndarray, shape (n)
        The vector Q^Tf.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, n = *<value>*.

            Constraint: n > 0.

        (`errno` 12)
            On entry, `xtol` = *<value>*.

            Constraint: `xtol` >= 0.0.

        (`errno` 13)
            On entry, `mode` = *<value>*.

            Constraint: `mode` = 1 or 2.

        (`errno` 14)
            On entry, `factor` = *<value>*.

            Constraint: `factor` > 0.0.

        (`errno` 15)
            On entry, `mode` = 2 and `diag` contained a non-positive
            element.

        (`errno` 18)
            On entry, `maxfev` = *<value>*.

            Constraint: `maxfev` > 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            There have been at least `maxfev` calls to `fcnval` and
            `fcnjac` combined: `maxfev` = *<value>*. Consider restarting
            the calculation from the final point held in `x`.

        (`errno` 3)
            No further improvement in the solution is possible. `xtol`
            is too small: `xtol` = *<value>*.

        (`errno` 4)
            The iteration is not making good progress, as measured by
            the improvement from the last *<value>* Jacobian
            evaluations. This failure exit may indicate that the system
            does not have a zero, or that the solution is very close to
            the origin (see [Accuracy]). Otherwise, rerunning
            ``sys_deriv_expert`` from a different starting point may
            avoid the region of difficulty.

        (`errno` 5)
            The iteration is not making good progress, as measured by
            the improvement from the last *<value>* iterations. This
            failure exit may indicate that the system does not have a
            zero, or that the solution is very close to the origin (see
            [Accuracy]). Otherwise, rerunning ``sys_deriv_expert`` from
            a different starting point may avoid the region of
            difficulty.

    NagCallbackTerminateWarning
        (`errno` 6)
            Termination requested in `fcnval`, `fcnjac` or `fcnmon`.
    """

    from ..base.roots import sys_deriv_expert as b_sys_deriv_expert

    fun_name = 'naginterfaces.library.roots.sys_deriv_expert'

    cb_fcnval_name = fun_name + ':fcnval'

    def cb_fcnval(n, x, fvec, data=None):
        """Wrapping user's fcnval. Called as base fcnval."""
        if data is not None:
            _fvec_fcnval = fcnval(
                x,
                data,
            )
        else:
            _fvec_fcnval = fcnval(
                x,
            )
        fvec[:] = _utils._to_ndarray(
            {'fun_name': cb_fcnval_name, 'returned': 'fvec'},
            _fvec_fcnval,
            copy=False,
            exp_shape=(n,),
            exp_class=_b_utils._EngineFloat64ArrayType,
        )

    cb_fcnjac_name = fun_name + ':fcnjac'

    def cb_fcnjac(n, x, fjac, data=None):
        """Wrapping user's fcnjac. Called as base fcnjac."""
        if data is not None:
            _fjac_fcnjac = fcnjac(
                x,
                data,
            )
        else:
            _fjac_fcnjac = fcnjac(
                x,
            )
        _fjac_fcnjac_np = _utils._to_ndarray(
            {'fun_name': cb_fcnjac_name, 'returned': 'fjac'},
            _fjac_fcnjac,
            copy=False,
            exp_shape=(n, n),
            exp_class=_b_utils._EngineFloat64ArrayType,
        )
        _utils._check_callback_sorder(
            {'fun_name': cb_fcnjac_name},
            ((_fjac_fcnjac_np, 'fjac'),),
            wrapper_sorder=_m_sorder,
            spiked_sorder=spiked_sorder,
        )
        fjac[:, :] = _fjac_fcnjac_np

    def cb_fcnmon(n, x, fvec, fjac, data=None):
        """Wrapping user's fcnmon. Called as base fcnmon."""
        if data is not None:
            fcnmon(
                x,
                fvec,
                fjac,
                data,
            )
        else:
            fcnmon(
                x,
                fvec,
                fjac,
            )

    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        (),
        spiked_sorder=spiked_sorder,
    )
    _b_utils._check_callable(fcnval, {'fun_name': fun_name, 'entity': 'fcnval'})
    _fcnval = cb_fcnval
    _b_utils._check_callable(fcnjac, {'fun_name': fun_name, 'entity': 'fcnjac'})
    _fcnjac = cb_fcnjac
    _b_utils._check_callable(fcnmon, {'fun_name': fun_name, 'entity': 'fcnmon'})
    _fcnmon = cb_fcnmon
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _fvec = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _fjac = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _maxfev = 100*(_n_py+1) if maxfev is None else maxfev
    _diag = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'diag'},
        diag,
        copy=True,
    )
    _r = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py*(_n_py+1)//2,),
    )
    _qtf = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalars = b_sys_deriv_expert(
            _fcnval,
            _fcnjac,
            _fcnmon,
            _n,
            _x,
            _fvec,
            _fjac,
            xtol,
            _maxfev,
            mode,
            _diag,
            factor,
            nprint,
            _r,
            _qtf,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _x_py = _x
    _diag_py = _diag
    _fvec_py = _fvec
    _fjac_py = _fjac
    _nfev_py = _return_scalars.nfev
    _njev_py = _return_scalars.njev
    _r_py = _r
    _qtf_py = _qtf
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'fvec', 'fjac', 'diag', 'nfev', 'njev', 'r', 'qtf'],
    )(
        _x_py,
        _fvec_py,
        _fjac_py,
        _diag_py,
        _nfev_py,
        _njev_py,
        _r_py,
        _qtf_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def sys_deriv_rcomm(irevcm, x, fvec, fjac, mode, diag, r, qtf, comm, xtol=sqrt(machine.precision()), factor=100.0):
    u"""
    Solution of a system of nonlinear equations using first derivatives
    (reverse communication).

    For full information please refer to the NAG Library document for
    c05rd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05rdf.html

    Parameters
    ----------
    irevcm : int
        On initial entry: must have the value 0.

    x : float, ndarray, shape (n), modified in place
        On initial entry: an initial guess at the solution vector.

        On intermediate exit: contains the current point.

        On final exit: the final estimate of the solution vector.

    fvec : float, ndarray, shape (n), modified in place
        On initial entry: need not be set.

        On intermediate entry: if `irevcm` != 2, `fvec` must not be
        changed.

        If `irevcm` = 2, `fvec` must be set to the values of the
        functions computed at the current point `x`.

        On final exit: the function values at the final point, `x`.

    fjac : float, ndarray, shape (n, n), modified in place
        On initial entry: need not be set.

        On intermediate entry: if `irevcm` != 3, `fjac` must not be
        changed.

        If `irevcm` = 3, `fjac`[i-1,j-1] must contain the value of
        df_i/dx_j at the point x, for j = 1,2,...,n, for i = 1,2,...,n.

        On final exit: the orthogonal matrix Q produced by the QR
        factorization of the final approximate Jacobian.

    mode : int
        On initial entry: indicates whether or not you have provided
        scaling factors in `diag`.

        If `mode` = 2, the scaling must have been supplied in `diag`.

        Otherwise, if `mode` = 1, the variables will be scaled
        internally.

    diag : float, ndarray, shape (n), modified in place
        On initial entry: if `mode` = 2, `diag` must contain
        multiplicative scale factors for the variables.

        If `mode` = 1, `diag` need not be set.

        On intermediate exit: `diag` must not be changed.

        On final exit: the scale factors actually used (computed
        internally if `mode` = 1).

    r : float, ndarray, shape (n*(n+1)/2), modified in place
        On initial entry: need not be set.

        On intermediate exit: must not be changed.

        On final exit: the upper triangular matrix R produced by the QR
        factorization of the final approximate Jacobian, stored
        row-wise.

    qtf : float, ndarray, shape (n), modified in place
        On initial entry: need not be set.

        On intermediate exit: must not be changed.

        On final exit: the vector Q^Tf.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    xtol : float, optional
        On initial entry: the accuracy in `x` to which the solution is
        required.

    factor : float, optional
        On initial entry: a quantity to be used in determining the
        initial step bound. In most cases, `factor` should lie between
        0.1 and 100.0. (The step bound is `factor`*||`diag`*`x`||_2 if
        this is nonzero; otherwise the bound is `factor`.)

    Returns
    -------
    irevcm : int
        On intermediate exit: specifies what action you must take before
        re-entering ``sys_deriv_rcomm`` with `irevcm` set to this value.
        The value of this argument should be interpreted as follows:

        `irevcm` = 1

          Indicates the start of a new iteration. No action is required
          by you, but `x` and `fvec` are available for printing.

        `irevcm` = 2

          Indicates that before re-entry to ``sys_deriv_rcomm``, `fvec`
          must contain the function values f_i(x).

        `irevcm` = 3

          Indicates that before re-entry to ``sys_deriv_rcomm``,
          `fjac`[i-1,j-1] must contain the value of df_i/dx_j at the
          point x, for j = 1,2,...,n, for i = 1,2,...,n.

        On final exit: `irevcm` = 0 and the algorithm has terminated.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 0, 1, 2 or 3.

        (`errno` 11)
            On entry, n = *<value>*.

            Constraint: n > 0.

        (`errno` 12)
            On entry, `xtol` = *<value>*.

            Constraint: `xtol` >= 0.0.

        (`errno` 13)
            On entry, `mode` = *<value>*.

            Constraint: `mode` = 1 or 2.

        (`errno` 14)
            On entry, `factor` = *<value>*.

            Constraint: `factor` > 0.0.

        (`errno` 15)
            On entry, `mode` = 2 and `diag` contained a non-positive
            element.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            No further improvement in the solution is possible. `xtol`
            is too small: `xtol` = *<value>*.

        (`errno` 4)
            The iteration is not making good progress, as measured by
            the improvement from the last *<value>* Jacobian
            evaluations. This failure exit may indicate that the system
            does not have a zero, or that the solution is very close to
            the origin (see [Accuracy]). Otherwise, rerunning
            ``sys_deriv_rcomm`` from a different starting point may
            avoid the region of difficulty.

        (`errno` 5)
            The iteration is not making good progress, as measured by
            the improvement from the last *<value>* iterations. This
            failure exit may indicate that the system does not have a
            zero, or that the solution is very close to the origin (see
            [Accuracy]). Otherwise, rerunning ``sys_deriv_rcomm`` from a
            different starting point may avoid the region of difficulty.
    """

    from ..base.roots import sys_deriv_rcomm as b_sys_deriv_rcomm

    fun_name = 'naginterfaces.library.roots.sys_deriv_rcomm'

    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'fvec'},
        fvec,
    )
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'fjac'},
        fjac,
    )
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'diag'},
        diag,
    )
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'r'},
        r,
    )
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'qtf'},
        qtf,
    )
    try:
        _return_scalar = b_sys_deriv_rcomm(
            irevcm,
            _n,
            x,
            fvec,
            fjac,
            xtol,
            mode,
            diag,
            factor,
            r,
            qtf,
            comm,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _irevcm_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['irevcm'],
    )(
        _irevcm_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _irevcm_py

def sys_deriv_check(mode, x, fvec, fjac, fvecp):
    u"""
    Check user's function for calculating first derivatives of a set of
    nonlinear functions of several variables.

    For full information please refer to the NAG Library document for
    c05zd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05zdf.html

    Parameters
    ----------
    mode : int
        The value 1 on the first call and the value 2 on the second call
        of ``sys_deriv_check``.

    x : float, array-like, shape (n)
        The components of a point x, at which the consistency check is
        to be made. (See [Accuracy].)

    fvec : float, array-like, shape (m)
        If `mode` = 2, `fvec` must contain the value of the functions
        evaluated at x. If `mode` = 1, `fvec` is not referenced.

    fjac : float, array-like, shape (m, n)
        If `mode` = 2, `fjac` must contain the value of df_i/dx_j at the
        point x, for j = 1,2,...,n, for i = 1,2,...,m. If `mode` = 1,
        `fjac` is not referenced.

    fvecp : float, array-like, shape (m)
        If `mode` = 2, `fvecp` must contain the value of the functions
        evaluated at `xp` (as output by a preceding call to
        ``sys_deriv_check`` with `mode` = 1). If `mode` = 1, `fvecp` is
        not referenced.

    Returns
    -------
    xp : float, ndarray, shape (n)
        If `mode` = 1, this argument is set to a point neighbouring `x`.
        If `mode` = 2, this argument is undefined.

    err : float, ndarray, shape (m)
        If `mode` = 2, this argument contains measures of correctness of
        the respective gradients. If `mode` = 1, this argument is
        undefined. If there is no loss of significance (see [Accuracy]),
        if `err`[i-1] is 1.0 the ith user-supplied gradient df_i/dx_j,
        for j = 1,2,...,n is correct, whilst if `err`[i-1] is 0.0 the
        ith gradient is incorrect. For values of `err`[i-1] between 0.0
        and 1.0 the categorisation is less certain. In general, a value
        of `err`[i-1] > 0.5 indicates that the ith gradient is probably
        correct.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `mode` = *<value>*.

            Constraint: `mode` = 1 or 2.

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 3)
            On entry, n = *<value>*.

            Constraint: n >= 1.
    """

    from ..base.roots import sys_deriv_check as b_sys_deriv_check

    fun_name = 'naginterfaces.library.roots.sys_deriv_check'

    _fvec = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'fvec'},
        fvec,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_fvec, {'fun_name': fun_name, 'entity': 'fvec'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _fjac = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'fjac'},
        fjac,
        copy=False,
    )
    _xp = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _fvecp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'fvecp'},
        fvecp,
        copy=False,
    )
    _err = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    b_sys_deriv_check(
        mode,
        _m,
        _n,
        _x,
        _fvec,
        _fjac,
        _xp,
        _fvecp,
        _err,
    )
    _xp_py = _xp
    _err_py = _err
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['xp', 'err'],
    )(
        _xp_py,
        _err_py,
    )
    return _return_data
