# -*- coding: utf-8 -*-
# pylint: disable=chained-comparison,invalid-name,line-too-long,protected-access,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unused-import
u"""
Interfaces for the NAG Mark 27.0 `sparseig` Chapter.

``sparseig`` - Large Scale Eigenproblems

This module provides functions for computing some eigenvalues and
eigenvectors of large-scale (sparse) standard and generalized eigenvalue
problems.
It provides functions for:

- solution of symmetric eigenvalue problems;

- solution of nonsymmetric eigenvalue problems;

- solution of generalized symmetric-definite eigenvalue problems;

- solution of generalized nonsymmetric eigenvalue problems;

- partial singular value decomposition.

Functions are provided for both real and complex data.

The functions in this module have all been derived from the ARPACK
software suite (see ARPACK Users' Guide), a collection of Fortran 77
functions designed to solve large scale eigenvalue problems.
The interfaces provided in this module have been chosen to combine ease
of use with the flexibility of the original ARPACK software.
The underlying iterative methods and algorithms remain essentially the
same as those in ARPACK and are described fully in ARPACK Users' Guide.

The algorithms used are based upon an algorithmic variant of the Arnoldi
process called the Implicitly Restarted Arnoldi Method.
For symmetric matrices, this reduces to a variant of the Lanczos process
called the Implicitly Restarted Lanczos Method.
These variants may be viewed as a synthesis of the Arnoldi/Lanczos
process with the Implicitly Shifted QR technique that is suitable for
large scale problems.
For many standard problems, a matrix factorization is not required.
Only the action of the matrix on a vector is needed.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12intro.html
"""

# NAG Copyright 2017-2019.

import collections as _collections

from ..base import utils as _b_utils
from ..library import _utils

def real_init(n, nev, ncv):
    u"""
    Initialization routine for (``real_iter``) computing selected
    eigenvalues and, optionally, eigenvectors of a real nonsymmetric
    sparse (standard or generalized) eigenproblem.

    For full information please refer to the NAG Library document for
    f12aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12aaf.html

    Parameters
    ----------
    n : int
        The order of the matrix A (and the order of the matrix B for the
        generalized problem) that defines the eigenvalue problem.

    nev : int
        The number of eigenvalues to be computed.

    ncv : int
        The number of Arnoldi basis vectors to use during the
        computation.

        At present there is no a priori analysis to guide the selection
        of `ncv` relative to `nev`.

        However, it is recommended that `ncv` >= 2*`nev`+1.

        If many problems of the same type are to be solved, you should
        experiment with increasing `ncv` while keeping `nev` fixed for a
        given test problem.

        This will usually decrease the required number of matrix-vector
        operations but it also increases the work and storage required
        to maintain the orthogonal basis vectors.

        The optimal 'cross-over' with respect to CPU time is problem
        dependent and must be determined empirically.

    Returns
    -------
    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 2)
            On entry, `nev` = *<value>*.

            Constraint: `nev` > 0.

        (`errno` 3)
            On entry, `ncv` = *<value>*, `nev` = *<value>* and `n` =
            *<value>*.

            Constraint: `ncv` > `nev`+1 and `ncv` <= `n`.

        (`errno` 4)
            The length of the integer array `comm`['icomm'] is too small
            licomm = *<value>*, but must be at least *<value>*.

        (`errno` 5)
            On entry, lcomm = *<value>*, `n` = *<value>* and `ncv` =
            *<value>*.

            Constraint: lcomm >= 3*`n`+3*`ncv`*`ncv`+6*`ncv`+60.
    """

    from ..base.sparseig import real_init as b_real_init

    _return_scalar = b_real_init(
        n,
        nev,
        ncv,
    )
    _comm_py = _return_scalar
    return _comm_py

def real_iter(irevcm, resid, v, x, mx, comm, io_manager=None):
    u"""
    Selected eigenvalues and, optionally, eigenvectors of a real
    nonsymmetric sparse eigenproblem, reverse communication.

    For full information please refer to the NAG Library document for
    f12ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12abf.html

    Parameters
    ----------
    irevcm : int
        On initial entry: `irevcm` = 0, otherwise an error condition
        will be raised.

        On intermediate entry: must be unchanged from its previous exit
        value. Changing `irevcm` to any other value between calls will
        result in an error.

    resid : float, ndarray, shape (n), modified in place
        On initial entry: need not be set unless the option 'Initial
        Residual' has been set in a prior call to ``real_option`` in
        which case `resid` should contain an initial residual vector,
        possibly from a previous run.

        On intermediate entry: must be unchanged from its previous exit.
        Changing `resid` to any other value between calls may result in
        an error exit.

        On intermediate exit: contains the current residual vector.

        On final exit: contains the final residual vector.

    v : float, ndarray, shape (n, ncv), modified in place
        On initial entry: need not be set.

        On intermediate entry: must be unchanged from its previous exit.

        On intermediate exit: contains the current set of Arnoldi basis
        vectors.

        On final exit: contains the final set of Arnoldi basis vectors.

    x : float, ndarray, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `comm`['icomm'][(int(`comm`['comm'][0])+24)-1]=0:
        `comm`['icomm'][(int(`comm`['comm'][0])+1)-1]; otherwise: 1.

        On initial entry: need not be set, it is used as a convenient
        mechanism for accessing elements of `comm`['comm'].

        On intermediate entry: if 'Pointers' = YES, `x` need not be set.

        If 'Pointers' = NO, `x` must contain the result of y = OPx when
        `irevcm` returns the value -1 or +1.

        It must return the real parts of the computed shifts when
        `irevcm` returns the value 3.

        On intermediate exit: if 'Pointers' = YES, `x` is not
        referenced.

        If 'Pointers' = NO, `x` contains the vector x when `irevcm`
        returns the value -1 or +1.

        On final exit: does not contain useful data.

    mx : float, ndarray, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `comm`['icomm'][(int(`comm`['comm'][0])+24)-1]=0:
        `comm`['icomm'][(int(`comm`['comm'][0])+1)-1]; otherwise: 1.

        On initial entry: need not be set, it is used as a convenient
        mechanism for accessing elements of `comm`['comm'].

        On intermediate entry: if 'Pointers' = YES, `mx` need not be
        set.

        If 'Pointers' = NO, `mx` must contain the result of y = Bx when
        `irevcm` returns the value 2.

        It must return the imaginary parts of the computed shifts when
        `irevcm` returns the value 3.

        On intermediate exit: if 'Pointers' = YES, `mx` is not
        referenced.

        If 'Pointers' = NO, `mx` contains the vector Bx when `irevcm`
        returns the value +1.

        On final exit: does not contain any useful data.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    irevcm : int
        On intermediate exit: has the following meanings.

        `irevcm` = -1

          The calling program must compute the matrix-vector product y =
          OPx, where x is stored in `x` (by default) or in the array
          `comm`['comm'] (starting from the location given by the first
          element of `comm`['icomm']) when the option 'Pointers' = 'YES'
          is set in a prior call to ``real_option``. The result y is
          returned in `x` (by default) or in the array `comm`['comm']
          (starting from the location given by the second element of
          `comm`['icomm']) when the option 'Pointers' = 'YES' is set. If
          B is not symmetric semidefinite then the precomputed values in
          `mx` should not be used (see the explanation under `irevcm` =
          2).

        `irevcm` = 1

          The calling program must compute the matrix-vector product y =
          OPx. This is similar to the case `irevcm` = -1 except that the
          result of the matrix-vector product Bx (as required in some
          computational modes) has already been computed and is
          available in `mx` (by default) or in the array `comm`['comm']
          (starting from the location given by the third element of
          `comm`['icomm']) when the option 'Pointers' = 'YES' is set.

        `irevcm` = 2

          The calling program must compute the matrix-vector product y =
          Bx, where x is stored as described in the case `irevcm` = -1
          and y is returned in the location described by the case
          `irevcm` = 1. This computation is requested when solving the
          'Generalized' problem using either 'Shifted Inverse
          Imaginary', 'Shifted Inverse Real'; in these cases B is used
          as an inner-product space and requires that B be symmetric
          semidefinite. If neither A nor B is symmetric semidefinite
          then the problem should be reformulated in a 'Standard' form.

        `irevcm` = 3

          Compute the `nshift` real and imaginary parts of the shifts
          where the real parts are to be returned in the first `nshift`
          locations of the array `x` and the imaginary parts are to be
          returned in the first `nshift` locations of the array `mx`.
          Only complex conjugate pairs of shifts may be applied and the
          pairs must be placed in consecutive locations. This value of
          this argument will only arise if the option 'Supplied Shifts'
          is set in a prior call to ``real_option`` which is intended
          for experienced users only; the default and recommended option
          is to use exact shifts (see ARPACK Users' Guide for details).

        `irevcm` = 4

          Monitoring step: a call to ``real_monit`` can now be made to
          return the number of Arnoldi iterations, the number of
          converged Ritz values, their real and imaginary parts, and the
          corresponding Ritz estimates.

        On final exit: `irevcm` = 5: ``real_iter`` has completed its
        tasks. The value of errno determines whether the iteration has
        been successfully completed, or whether errors have been
        detected. On successful completion ``real_proc`` must be called
        to return the requested eigenvalues and eigenvectors (and/or
        Schur vectors).

    nshift : int
        On intermediate exit: if the option 'Supplied Shifts' is set and
        `irevcm` returns a value of 3, this argument returns the number
        of complex shifts required.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The maximum number of iterations <= 0, the option 'Iteration
            Limit' has been set to *<value>*.

        (`errno` 2)
            The options 'Generalized' and 'Regular' are incompatible.

        (`errno` 5)
            No shifts could be applied during a cycle of the implicitly
            restarted Arnoldi iteration.

        (`errno` 6)
            Could not build an Arnoldi factorization. The size of the
            current Arnoldi factorization = *<value>*.

        (`errno` 7)
            Error in internal call to compute eigenvalues and
            corresponding error bounds of the current upper Hessenberg
            matrix. Please contact NAG.

        (`errno` 8)
            Either the initialization function has not been called prior
            to the first call of this function or a communication array
            has become corrupted.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            The option 'Initial Residual' was selected but the starting
            vector held in `resid` is zero.

        (`errno` 4)
            The maximum number of iterations has been reached. The
            maximum number of iterations = *<value>*. The number of
            converged eigenvalues = *<value>*. See the function document
            for further details.
    """

    from ..base.sparseig import real_iter as b_real_iter

    fun_name = 'naginterfaces.library.sparseig.real_iter'

    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'resid'},
        resid,
    )
    _b_utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('comm', 'icomm'),
    )
    if int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_n_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False)),), validate=False)
    else:
        _vl_f12aaf_n_py = 0
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
    )
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
    )
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'mx'},
        mx,
    )
    try:
        _return_scalars = b_real_iter(
            irevcm,
            resid,
            v,
            x,
            mx,
            comm,
            io_manager,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _irevcm_py = _return_scalars.irevcm
    _nshift_py = _return_scalars.nshift
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['irevcm', 'nshift'],
    )(
        _irevcm_py,
        _nshift_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def real_proc(sigmar, sigmai, resid, v, comm, io_manager=None):
    u"""
    Selected eigenvalues and, optionally, eigenvectors of a real
    nonsymmetric sparse eigenproblem, postprocessing for ``real_iter``.

    For full information please refer to the NAG Library document for
    f12ac

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12acf.html

    Parameters
    ----------
    sigmar : float
        If one of the 'Shifted Inverse Real' modes have been selected
        then `sigmar` contains the real part of the shift used;
        otherwise `sigmar` is not referenced.

    sigmai : float
        If one of the 'Shifted Inverse Real' modes have been selected
        then `sigmai` contains the imaginary part of the shift used;
        otherwise `sigmai` is not referenced.

    resid : float, array-like, shape (n)
        Must not be modified following a call to ``real_iter`` since it
        contains data required by ``real_proc``.

    v : float, array-like, shape (n, ncv)
        The ncv columns of `v` contain the Arnoldi basis vectors for OP
        as constructed by ``real_iter``.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nconv : int
        The number of converged eigenvalues as found by ``real_iter``.

    dr : float, ndarray, shape (nev+1)
        The first `nconv` locations of the array this argument contain
        the real parts of the converged approximate eigenvalues.

    di : float, ndarray, shape (nev+1)
        The first `nconv` locations of the array this argument contain
        the imaginary parts of the converged approximate eigenvalues.

    z : float, ndarray, shape (n, :)
        If the default option 'Vectors' = 'RITZ' (see ``real_option``)
        has been selected then this argument contains the final set of
        eigenvectors corresponding to the eigenvalues held in `dr` and
        `di`. The complex eigenvector associated with the eigenvalue
        with positive imaginary part is stored in two consecutive
        columns. The first column holds the real part of the eigenvector
        and the second column holds the imaginary part. The eigenvector
        associated with the eigenvalue with negative imaginary part is
        simply the complex conjugate of the eigenvector associated with
        the positive imaginary part.

    v : float, ndarray, shape (n, ncv)
        If the option 'Vectors' = 'SCHUR' has been set, or the option
        'Vectors' = 'RITZ' has been set and a separate array `z` has
        been passed (i.e., `z` does not equal this argument), then the
        first `nconv` columns of this argument will contain approximate
        Schur vectors that span the desired invariant subspace.

    Raises
    ------
    NagValueError
        (`errno` 3)
            The number of eigenvalues found to sufficient accuracy, as
            communicated through the argument `comm`['icomm'], is zero.
            See the function document for further details.

        (`errno` 4)
            Got a different count of the number of converged Ritz values
            than the value passed to it through the argument
            `comm`['icomm']: number counted = *<value>*, number expected
            = *<value>*.

        (`errno` 5)
            During calculation of a real Schur form, there was a failure
            to compute *<value>* eigenvalues in a total of *<value>*
            iterations.

        (`errno` 6)
            The computed Schur form could not be reordered by an
            internal call. This function returned with errno =
            *<value>*. Please contact NAG.

        (`errno` 7)
            In calculating eigenvectors, an internal call returned with
            an error. Please contact NAG.

        (`errno` 8)
            Either the solver function has not been called prior to the
            call of this function or a communication array has become
            corrupted.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            On entry, 'Vectors' = 'SELECT', but this is not yet
            implemented.
    """

    from ..base.sparseig import real_proc as b_real_proc

    fun_name = 'naginterfaces.library.sparseig.real_proc'

    _v = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_v, 'v'),),
    )
    _b_utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('comm', 'icomm'),
    )
    if int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_nev_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+1,), validate=False)
    else:
        _vl_f12aaf_nev_py = 0
    _dr = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_f12aaf_nev_py+1,),
    )
    _di = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_f12aaf_nev_py+1,),
    )
    if int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_n_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False)),), validate=False)
    else:
        _vl_f12aaf_n_py = 0
    if int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        if _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+7)-1,), validate=False) == 1:
            _z_dim2 = _vl_f12aaf_nev_py+1
        else:
            _z_dim2 = 1
    else:
        _z_dim2 = 0
    exp_shape_z = (_vl_f12aaf_n_py, _z_dim2)
    _z = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_m_sorder,
    )
    _resid = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'resid'},
        resid,
        copy=False,
    )
    try:
        _return_scalar = b_real_proc(
            _dr,
            _di,
            _z,
            sigmar,
            sigmai,
            _resid,
            _v,
            comm,
            io_manager,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _v_py = _v
    _nconv_py = _return_scalar
    _dr_py = _dr
    _di_py = _di
    _z_py = _z
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['nconv', 'dr', 'di', 'z', 'v'],
    )(
        _nconv_py,
        _dr_py,
        _di_py,
        _z_py,
        _v_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def real_option(optstr, comm, io_manager=None):
    u"""
    Set a single option from a string
    (``real_iter``/``real_proc``/``real_band_solve``).

    Note: this function uses optional algorithmic parameters, see also:
    ``real_iter``, ``real_proc``, ``real_init``.

    For full information please refer to the NAG Library document for
    f12ad

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12adf.html

    Parameters
    ----------
    optstr : str
        A single valid option string (as described in [Description] and
        [Optional Parameters]).

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Other Parameters
    ----------------
    'Advisory' : int
        Default = advisory message unit number

        The destination for advisory messages.

    'Defaults' : valueless
        This special keyword may be used to reset all options to their
        default values.

    'Exact Shifts' : valueless
        Default

        During the Arnoldi iterative process, shifts are applied
        internally as part of the implicit restarting scheme.
        The shift strategy used by default and selected by the 'Exact
        Shifts' is strongly recommended over the alternative 'Supplied
        Shifts' (see ARPACK Users' Guide for details of shift
        strategies).

        If 'Exact Shifts' are used then these are computed internally by
        the algorithm in the implicit restarting scheme.

        If 'Supplied Shifts' are used then, during the Arnoldi iterative
        process, you must supply shifts through array arguments of when
        returns with irevcm = 3; the real and imaginary parts of the
        shifts are supplied in x and mx respectively (or in comm when
        the option 'Pointers' = 'YES' is set). This option should only
        be used if you are an experienced user since this requires some
        algorithmic knowledge and because more operations are usually
        required than for the implicit shift scheme.
        Details on the use of explicit shifts and further references on
        shift strategies are available in ARPACK Users' Guide.

    'Supplied Shifts' : valueless
        During the Arnoldi iterative process, shifts are applied
        internally as part of the implicit restarting scheme.
        The shift strategy used by default and selected by the 'Exact
        Shifts' is strongly recommended over the alternative 'Supplied
        Shifts' (see ARPACK Users' Guide for details of shift
        strategies).

        If 'Exact Shifts' are used then these are computed internally by
        the algorithm in the implicit restarting scheme.

        If 'Supplied Shifts' are used then, during the Arnoldi iterative
        process, you must supply shifts through array arguments of when
        returns with irevcm = 3; the real and imaginary parts of the
        shifts are supplied in x and mx respectively (or in comm when
        the option 'Pointers' = 'YES' is set). This option should only
        be used if you are an experienced user since this requires some
        algorithmic knowledge and because more operations are usually
        required than for the implicit shift scheme.
        Details on the use of explicit shifts and further references on
        shift strategies are available in ARPACK Users' Guide.

    'Iteration Limit' : int
        Default = 300

        The limit on the number of Arnoldi iterations that can be
        performed before exits.
        If not all requested eigenvalues have converged to within
        'Tolerance' and the number of Arnoldi iterations has reached
        this limit then exits with an error; can still be called
        subsequently to return the number of converged eigenvalues, the
        converged eigenvalues and, if requested, the corresponding
        eigenvectors.

    'Largest Magnitude' : valueless
        Default

        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for to compute the eigenvalues of largest
        magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part,'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Largest Imaginary' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for to compute the eigenvalues of largest
        magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part,'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Largest Real' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for to compute the eigenvalues of largest
        magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part,'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Smallest Imaginary' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for to compute the eigenvalues of largest
        magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part,'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Smallest Magnitude' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for to compute the eigenvalues of largest
        magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part,'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Smallest Real' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for to compute the eigenvalues of largest
        magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part,'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Nolist' : valueless
        Default

        Option 'List' enables printing of each option specification as
        it is supplied. 'Nolist' suppresses this printing.

    'List' : valueless
        Option 'List' enables printing of each option specification as
        it is supplied. 'Nolist' suppresses this printing.

    'Monitoring' : int
        Default = -1

        If i > 0, monitoring information is output to unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`)
        i during the solution of each problem; this may be the same as
        the 'Advisory' unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`).
        The type of information produced is dependent on the value of
        'Print Level', see the description of the option 'Print Level'
        for details of the information produced.
        Please see the ``FileObjManager`` method
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`
        to associate a file with a given unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`).

    'Pointers' : valueless
        Default = NO

        During the iterative process and reverse communication calls to,
        required data can be communicated to and from in one of two
        ways.
        When 'Pointers' = 'NO' is selected (the default) then the array
        arguments x and mx are used to supply you with required data and
        used to return computed values back to.
        For example, when irevcm = 1, returns the vector x in x and the
        matrix-vector product Bx in mx and expects the result or the
        linear operation OP(x) to be returned in x.

        If 'Pointers' = 'YES' is selected then the data is passed
        through sections of the array argument comm.
        The section corresponding to x when 'Pointers' = 'NO' begins at
        a location given by the first element of icomm; similarly the
        section corresponding to mx begins at a location given by the
        second element of icomm.
        This option allows to perform fewer copy operations on each
        intermediate exit and entry, but can also lead to less elegant
        code in the calling program.

    'Print Level' : int
        Default = 0

        This controls the amount of printing produced by ``real_option``
        as follows.

        [table omitted]

    'Random Residual' : valueless
        Default

        To begin the Arnoldi iterative process, requires an initial
        residual vector.
        By default provides its own random initial residual vector; this
        option can also be set using option 'Random Residual'.
        Alternatively, you can supply an initial residual vector
        (perhaps from a previous computation) to through the array
        argument resid; this option can be set using option 'Initial
        Residual'.

    'Initial Residual' : valueless
        To begin the Arnoldi iterative process, requires an initial
        residual vector.
        By default provides its own random initial residual vector; this
        option can also be set using option 'Random Residual'.
        Alternatively, you can supply an initial residual vector
        (perhaps from a previous computation) to through the array
        argument resid; this option can be set using option 'Initial
        Residual'.

    'Regular' : valueless
        Default

        These options define the computational mode which in turn
        defines the form of operation OP(x) to be performed when returns
        with irevcm = -1 or 1 and the matrix-vector product Bx when
        returns with irevcm = 2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

    'Regular Inverse' : valueless
        These options define the computational mode which in turn
        defines the form of operation OP(x) to be performed when returns
        with irevcm = -1 or 1 and the matrix-vector product Bx when
        returns with irevcm = 2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

    'Shifted Inverse Imaginary' : valueless
        These options define the computational mode which in turn
        defines the form of operation OP(x) to be performed when returns
        with irevcm = -1 or 1 and the matrix-vector product Bx when
        returns with irevcm = 2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

    'Shifted Inverse Real' : valueless
        These options define the computational mode which in turn
        defines the form of operation OP(x) to be performed when returns
        with irevcm = -1 or 1 and the matrix-vector product Bx when
        returns with irevcm = 2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

    'Standard' : valueless
        Default

        The problem to be solved is either a standard eigenvalue
        problem, Ax = lambda x, or a generalized eigenvalue problem, Ax
        = lambda Bx.
        The option 'Standard' should be used when a standard eigenvalue
        problem is being solved and the option 'Generalized' should be
        used when a generalized eigenvalue problem is being solved.

    'Generalized' : valueless
        The problem to be solved is either a standard eigenvalue
        problem, Ax = lambda x, or a generalized eigenvalue problem, Ax
        = lambda Bx.
        The option 'Standard' should be used when a standard eigenvalue
        problem is being solved and the option 'Generalized' should be
        used when a generalized eigenvalue problem is being solved.

    'Tolerance' : float
        Default = epsilon

        An approximate eigenvalue has deemed to have converged when the
        corresponding Ritz estimate is within 'Tolerance' relative to
        the magnitude of the eigenvalue.

    'Vectors' : valueless
        Default = RITZ

        The function can optionally compute the Schur vectors and/or the
        eigenvectors corresponding to the converged eigenvalues.
        To turn off computation of any vectors the option 'Vectors' =
        'NONE' should be set.
        To compute only the Schur vectors (at very little extra cost),
        the option 'Vectors' = 'SCHUR' should be set and these will be
        returned in the array argument v of.
        To compute the eigenvectors (Ritz vectors) ­corresponding to the
        eigenvalue estimates, the option 'Vectors' = 'RITZ' should be
        set and these will be returned in the array argument z of, if z
        is set equal to v (as in [Example]) then the Schur vectors in v
        are overwritten by the eigenvectors computed by.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Ambiguous keyword: *<value>*

        (`errno` 2)
            Keyword not recognized: *<value>*

        (`errno` 3)
            Second keyword not recognized: *<value>*

        (`errno` 4)
            Either the initialization function has not been called prior
            to the call of this function or a communication array has
            become corrupted.
    """

    from ..base.sparseig import real_option as b_real_option

    b_real_option(
        optstr,
        comm,
        io_manager,
    )

def real_monit(comm):
    u"""
    Provides monitoring information for ``real_iter``.

    For full information please refer to the NAG Library document for
    f12ae

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12aef.html

    Parameters
    ----------
    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_init``.

    Returns
    -------
    niter : int
        The number of the current Arnoldi iteration.

    nconv : int
        The number of converged eigenvalues so far.

    ritzr : float, ndarray, shape (ncv)
        The first `nconv` locations of the array this argument contain
        the real parts of the converged approximate eigenvalues.

    ritzi : float, ndarray, shape (ncv)
        The first `nconv` locations of the array this argument contain
        the imaginary parts of the converged approximate eigenvalues.

    rzest : float, ndarray, shape (ncv)
        The first `nconv` locations of the array this argument contain
        the Ritz estimates (error bounds) on the converged approximate
        eigenvalues.
    """

    from ..base.sparseig import real_monit as b_real_monit

    fun_name = 'naginterfaces.library.sparseig.real_monit'

    _b_utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm', 'comm'),
    )
    if int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aaf_ncv_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+2,), validate=False)
    else:
        _vl_f12aaf_ncv_py = 0
    _ritzr = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_f12aaf_ncv_py,),
    )
    _ritzi = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_f12aaf_ncv_py,),
    )
    _rzest = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_f12aaf_ncv_py,),
    )
    _return_scalars = b_real_monit(
        _ritzr,
        _ritzi,
        _rzest,
        comm,
    )
    _niter_py = _return_scalars.niter
    _nconv_py = _return_scalars.nconv
    _ritzr_py = _ritzr
    _ritzi_py = _ritzi
    _rzest_py = _rzest
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['niter', 'nconv', 'ritzr', 'ritzi', 'rzest'],
    )(
        _niter_py,
        _nconv_py,
        _ritzr_py,
        _ritzi_py,
        _rzest_py,
    )
    return _return_data

def real_band_init(n, nev, ncv):
    u"""
    Initialization routine for (``real_band_solve``) computing selected
    eigenvalues and, optionally, eigenvectors of a real nonsymmetric
    banded (standard or generalized) eigenproblem.

    For full information please refer to the NAG Library document for
    f12af

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12aff.html

    Parameters
    ----------
    n : int
        The order of the matrix A (and the order of the matrix B for the
        generalized problem) that defines the eigenvalue problem.

    nev : int
        The number of eigenvalues to be computed.

    ncv : int
        The number of Lanczos basis vectors to use during the
        computation.

        At present there is no a priori analysis to guide the selection
        of `ncv` relative to `nev`.

        However, it is recommended that `ncv` >= 2*`nev`+1.

        If many problems of the same type are to be solved, you should
        experiment with increasing `ncv` while keeping `nev` fixed for a
        given test problem.

        This will usually decrease the required number of matrix-vector
        operations but it also increases the work and storage required
        to maintain the orthogonal basis vectors.

        The optimal 'cross-over' with respect to CPU time is problem
        dependent and must be determined empirically.

    Returns
    -------
    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 2)
            On entry, `nev` = *<value>*.

            Constraint: `nev` > 0.

        (`errno` 3)
            On entry, `ncv` = *<value>*, `nev` = *<value>* and `n` =
            *<value>*.

            Constraint: `ncv` > `nev`+1 and `ncv` <= `n`.

        (`errno` 4)
            The length of the integer array `comm`['icomm'] is too small
            licomm = *<value>*, but must be at least *<value>*.

        (`errno` 5)
            The length of the integer array `comm`['comm'] is too small
            lcomm = *<value>*, but must be at least *<value>*.
    """

    from ..base.sparseig import real_band_init as b_real_band_init

    _return_scalar = b_real_band_init(
        n,
        nev,
        ncv,
    )
    _comm_py = _return_scalar
    return _comm_py

def real_band_solve(kl, ku, ab, mb, sigmar, sigmai, resid, comm, io_manager=None):
    u"""
    Selected eigenvalues and, optionally, eigenvectors of a real
    nonsymmetric banded eigenproblem, driver.

    For full information please refer to the NAG Library document for
    f12ag

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12agf.html

    Parameters
    ----------
    kl : int
        The number of subdiagonals of the matrices A and B.

    ku : int
        The number of superdiagonals of the matrices A and B.

    ab : float, array-like, shape (2*`kl`+`ku`+1, n)
        Must contain the matrix A in LAPACK banded storage format for
        nonsymmetric matrices (see [the F07 Introduction]).

    mb : float, array-like, shape (2*`kl`+`ku`+1, n)
        Must contain the matrix B in LAPACK banded storage format for
        nonsymmetric matrices (see [the F07 Introduction]).

    sigmar : float
        If one of the 'Shifted Inverse Real' modes (see ``real_option``)
        have been selected then `sigmar` must contain the real part of
        the shift used; otherwise `sigmar` is not referenced. [the F12
        Introduction] describes the use of shift and inverse
        transformations.

    sigmai : float
        If one of the 'Shifted Inverse Real' modes (see ``real_option``)
        have been selected then `sigmai` must contain the imaginary part
        of the shift used; otherwise `sigmai` is not referenced. [the
        F12 Introduction] describes the use of shift and inverse
        transformations.

    resid : float, array-like, shape (n)
        Need not be set unless the option 'Initial Residual' has been
        set in a prior call to ``real_option`` in which case `resid`
        must contain an initial residual vector.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_band_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nconv : int
        The number of converged eigenvalues.

    dr : float, ndarray, shape (nev+1)
        The first `nconv` locations of the array this argument contain
        the real parts of the converged approximate eigenvalues. The
        number of eigenvalues returned may be one more than the number
        requested by nev since complex values occur as conjugate pairs
        and the second in the pair can be returned in position nev+1 of
        the array.

    di : float, ndarray, shape (nev+1)
        The first `nconv` locations of the array this argument contain
        the imaginary parts of the converged approximate eigenvalues.
        The number of eigenvalues returned may be one more than the
        number requested by nev since complex values occur as conjugate
        pairs and the second in the pair can be returned in position
        nev+1 of the array.

    z : float, ndarray, shape (n, :)
        If the default option 'Vectors' = Ritz has been selected then
        this argument contains the final set of eigenvectors
        corresponding to the eigenvalues held in `dr` and `di`. The
        complex eigenvector associated with the eigenvalue with positive
        imaginary part is stored in two consecutive columns. The first
        column holds the real part of the eigenvector and the second
        column holds the imaginary part. The eigenvector associated with
        the eigenvalue with negative imaginary part is simply the
        complex conjugate of the eigenvector associated with the
        positive imaginary part.

    resid : float, ndarray, shape (n)
        Contains the final residual vector.

    v : float, ndarray, shape (n, ncv)
        If the option 'Vectors' (see ``real_option``) has been set to
        Schur or Ritz then the first `nconv`*n elements of this argument
        will contain approximate Schur vectors that span the desired
        invariant subspace.

        The ith Schur vector is stored in the ith column of `v`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `kl` = *<value>*.

            Constraint: `kl` >= 0.

        (`errno` 2)
            On entry, `ku` = *<value>*.

            Constraint: `ku` >= 0.

        (`errno` 3)
            On entry, ldab = *<value>*, 2*`kl`+`ku`+1 = *<value>*.

            Constraint: ldab >= 2*`kl`+`ku`+1.

        (`errno` 4)
            The option 'Shifted Inverse Imaginary' has been selected and
            `sigmai` = zero on entry; `sigmai` must be nonzero for this
            mode of operation.

        (`errno` 5)
            The maximum number of iterations <= 0, the option 'Iteration
            Limit' has been set to *<value>*.

        (`errno` 6)
            The options 'Generalized' and 'Regular' are incompatible.

        (`errno` 7)
            The option 'Initial Residual' was selected but the starting
            vector held in `resid` is zero.

        (`errno` 8)
            Either the initialization function has not been called prior
            to the first call of this function or a communication array
            has become corrupted.

        (`errno` 10)
            On entry, 'Vectors' = Select, but this is not yet
            implemented.

        (`errno` 11)
            The number of eigenvalues found to sufficient accuracy is
            zero.

        (`errno` 12)
            Could not build an Arnoldi factorization. The size of the
            current Arnoldi factorization = *<value>*.

        (`errno` 13)
            Error in internal call to compute eigenvalues and
            corresponding error bounds of the current upper Hessenberg
            matrix. Please contact NAG.

        (`errno` 14)
            During calculation of a real Schur form, there was a failure
            to compute a number of eigenvalues. Please contact NAG.

        (`errno` 15)
            The computed Schur form could not be reordered by an
            internal call. Please contact NAG.

        (`errno` 16)
            Error in internal call to compute eigenvectors. Please
            contact NAG.

        (`errno` 17)
            Failure during internal factorization of real banded matrix.
            Please contact NAG.

        (`errno` 18)
            Failure during internal solution of real banded matrix.
            Please contact NAG.

        (`errno` 19)
            Failure during internal factorization of complex banded
            matrix. Please contact NAG.

        (`errno` 20)
            Failure during internal solution of complex banded matrix.
            Please contact NAG.

        (`errno` 21)
            The maximum number of iterations has been reached. The
            maximum number of iterations = *<value>*. The number of
            converged eigenvalues = *<value>*.

        (`errno` 22)
            No shifts could be applied during a cycle of the implicitly
            restarted Lanczos iteration.

        (`errno` 23)
            Overflow occurred during transformation of Ritz values to
            those of the original problem.
    """

    from ..base.sparseig import real_band_solve as b_real_band_solve

    fun_name = 'naginterfaces.library.sparseig.real_band_solve'

    _ab = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ab'},
        ab,
        copy=False,
    )
    _mb = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'mb'},
        mb,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_ab, 'ab'), (_mb, 'mb')),
    )
    _b_utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('comm', 'icomm'),
    )
    if int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aff_nev_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+1,), validate=False)
    else:
        _vl_f12aff_nev_py = 0
    _dr = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_f12aff_nev_py+1,),
    )
    _di = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_f12aff_nev_py+1,),
    )
    if int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aff_n_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False)),), validate=False)
    else:
        _vl_f12aff_n_py = 0
    if _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+7)-1,), validate=False) == 1:
        _z_dim2 = _vl_f12aff_nev_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_vl_f12aff_n_py, _z_dim2)
    _z = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_m_sorder,
    )
    _resid = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'resid'},
        resid,
        copy=True,
    )
    if int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12aff_ncv_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+2,), validate=False)
    else:
        _vl_f12aff_ncv_py = 0
    _v = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_f12aff_n_py, _vl_f12aff_ncv_py),
        sorder=_m_sorder,
    )
    _return_scalar = b_real_band_solve(
        kl,
        ku,
        _ab,
        _mb,
        sigmar,
        sigmai,
        _dr,
        _di,
        _z,
        _resid,
        _v,
        comm,
        io_manager,
    )
    _resid_py = _resid
    _nconv_py = _return_scalar
    _dr_py = _dr
    _di_py = _di
    _z_py = _z
    _v_py = _v
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['nconv', 'dr', 'di', 'z', 'resid', 'v'],
    )(
        _nconv_py,
        _dr_py,
        _di_py,
        _z_py,
        _resid_py,
        _v_py,
    )
    return _return_data

def complex_init(n, nev, ncv):
    u"""
    Initialization routine for (``complex_iter``) computing selected
    eigenvalues and, optionally, eigenvectors of a complex sparse
    (standard or generalized) eigenproblem.

    For full information please refer to the NAG Library document for
    f12an

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12anf.html

    Parameters
    ----------
    n : int
        The order of the matrix A (and the order of the matrix B for the
        generalized problem) that defines the eigenvalue problem.

    nev : int
        The number of eigenvalues to be computed.

    ncv : int
        The number of Arnoldi basis vectors to use during the
        computation.

        At present there is no a priori analysis to guide the selection
        of `ncv` relative to `nev`.

        However, it is recommended that `ncv` >= 2*`nev`+1.

        If many problems of the same type are to be solved, you should
        experiment with increasing `ncv` while keeping `nev` fixed for a
        given test problem.

        This will usually decrease the required number of matrix-vector
        operations but it also increases the work and storage required
        to maintain the orthogonal basis vectors.

        The optimal 'cross-over' with respect to CPU time is problem
        dependent and must be determined empirically.

    Returns
    -------
    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 2)
            On entry, `nev` = *<value>*.

            Constraint: `nev` > 0.

        (`errno` 3)
            On entry, `ncv` = *<value>*, `nev` = *<value>* and `n` =
            *<value>*.

            Constraint: `ncv` >= `nev`+1 and `ncv` <= `n`.

        (`errno` 4)
            The length of the integer array `comm`['icomm'] is too small
            licomm = *<value>*, but must be at least *<value>*.

        (`errno` 5)
            On entry, lcomm = *<value>*, `n` = *<value>* and `ncv` =
            *<value>*.

            Constraint: lcomm >= 3*`n`+3*`ncv`*`ncv`+5*`ncv`+60.
    """

    from ..base.sparseig import complex_init as b_complex_init

    _return_scalar = b_complex_init(
        n,
        nev,
        ncv,
    )
    _comm_py = _return_scalar
    return _comm_py

def complex_iter(irevcm, resid, v, x, mx, comm, io_manager=None):
    u"""
    Selected eigenvalues and, optionally, eigenvectors of a complex
    sparse eigenproblem, reverse communication.

    For full information please refer to the NAG Library document for
    f12ap

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12apf.html

    Parameters
    ----------
    irevcm : int
        On initial entry: `irevcm` = 0, otherwise an error condition
        will be raised.

        On intermediate entry: must be unchanged from its previous exit
        value. Changing `irevcm` to any other value between calls will
        result in an error.

    resid : complex, ndarray, shape (n), modified in place
        On initial entry: need not be set unless the option 'Initial
        Residual' has been set in a prior call to ``complex_option`` in
        which case `resid` should contain an initial residual vector,
        possibly from a previous run.

        On intermediate entry: must be unchanged from its previous exit.
        Changing `resid` to any other value between calls may result in
        an error exit.

        On intermediate exit: contains the current residual vector.

        On final exit: contains the final residual vector.

    v : complex, ndarray, shape (n, ncv), modified in place
        On initial entry: need not be set.

        On intermediate entry: must be unchanged from its previous exit.

        On intermediate exit: contains the current set of Arnoldi basis
        vectors.

        On final exit: contains the final set of Arnoldi basis vectors.

    x : complex, ndarray, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if
        `comm`['icomm'][(int(Re(`comm`['comm'][0]))+24)-1]=0:
        `comm`['icomm'][(int(Re(`comm`['comm'][0]))+1)-1]; otherwise: 1.

        On initial entry: need not be set, it is used as a convenient
        mechanism for accessing elements of `comm`['comm'].

        On intermediate entry: if 'Pointers' = YES, `x` need not be set.

        If 'Pointers' = NO, `x` must contain the result of y = OPx when
        `irevcm` returns the value -1 or +1.

        It must return the computed shifts when `irevcm` returns the
        value 3.

        On intermediate exit: if 'Pointers' = YES, `x` is not
        referenced.

        If 'Pointers' = NO, `x` contains the vector x when `irevcm`
        returns the value -1 or +1.

        On final exit: does not contain useful data.

    mx : complex, ndarray, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if
        `comm`['icomm'][(int(Re(`comm`['comm'][0]))+24)-1]=0:
        `comm`['icomm'][(int(Re(`comm`['comm'][0]))+1)-1]; otherwise: 1.

        On initial entry: need not be set, it is used as a convenient
        mechanism for accessing elements of `comm`['comm'].

        On intermediate entry: if 'Pointers' = YES, `mx` need not be
        set.

        If 'Pointers' = NO, `mx` must contain the result of y = Bx when
        `irevcm` returns the value 2.

        On intermediate exit: if 'Pointers' = YES, `mx` is not
        referenced.

        If 'Pointers' = NO, `mx` contains the vector Bx when `irevcm`
        returns the value +1.

        On final exit: does not contain any useful data.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``complex_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    irevcm : int
        On intermediate exit: has the following meanings.

        `irevcm` = -1

          The calling program must compute the matrix-vector product y =
          OPx, where x is stored in `x` (by default) or in the array
          `comm`['comm'] (starting from the location given by the first
          element of `comm`['icomm']) when the option 'Pointers' = 'YES'
          is set in a prior call to ``complex_option``. The result y is
          returned in `x` (by default) or in the array `comm`['comm']
          (starting from the location given by the second element of
          `comm`['icomm']) when the option 'Pointers' = 'YES' is set.

        `irevcm` = 1

          The calling program must compute the matrix-vector product y =
          OPx. This is similar to the case `irevcm` = -1 except that the
          result of the matrix-vector product Bx (as required in some
          computational modes) has already been computed and is
          available in `mx` (by default) or in the array `comm`['comm']
          (starting from the location given by the third element of
          `comm`['icomm']) when the option 'Pointers' = 'YES' is set.

        `irevcm` = 2

          The calling program must compute the matrix-vector product y =
          Bx, where x is stored in `x` and y is returned in `mx` (by
          default) or in the array `comm`['comm'] (starting from the
          location given by the second element of `comm`['icomm']) when
          the option 'Pointers' = 'YES' is set.

        `irevcm` = 3

          Compute the `nshift` complex shifts. This value of this
          argument will only arise if the option 'Supplied Shifts' is
          set in a prior call to ``complex_option`` which is intended
          for experienced users only; the default and recommended option
          is to use exact shifts (see ARPACK Users' Guide for details).

        `irevcm` = 4

          Monitoring step: a call to ``complex_monit`` can now be made
          to return the number of Arnoldi iterations, the number of
          converged Ritz values, the array of converged values, and the
          corresponding Ritz estimates.

        On final exit: `irevcm` = 5: ``complex_iter`` has completed its
        tasks. The value of errno determines whether the iteration has
        been successfully completed, or whether errors have been
        detected. On successful completion ``complex_proc`` must be
        called to return the requested eigenvalues and eigenvectors
        (and/or Schur vectors).

    nshift : int
        On intermediate exit: if the option 'Supplied Shifts' is set and
        `irevcm` returns a value of 3, this argument returns the number
        of complex shifts required.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The maximum number of iterations <= 0, the option 'Iteration
            Limit' has been set to *<value>*.

        (`errno` 2)
            The options 'Generalized' and 'Regular' are incompatible.

        (`errno` 5)
            No shifts could be applied during a cycle of the implicitly
            restarted Arnoldi iteration.

        (`errno` 6)
            Could not build an Arnoldi factorization. The size of the
            current Arnoldi factorization = *<value>*.

        (`errno` 7)
            Error in internal call to compute eigenvalues and
            corresponding error bounds of the current upper Hessenberg
            matrix. Please contact NAG.

        (`errno` 8)
            Either the initialization function has not been called prior
            to the first call of this function or a communication array
            has become corrupted.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            The option 'Initial Residual' was selected but the starting
            vector held in `resid` is zero.

        (`errno` 4)
            The maximum number of iterations has been reached. The
            maximum number of iterations = *<value>*. The number of
            converged eigenvalues = *<value>*. See the function document
            for further details.
    """

    from ..base.sparseig import complex_iter as b_complex_iter

    fun_name = 'naginterfaces.library.sparseig.complex_iter'

    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'resid'},
        resid,
    )
    _b_utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('comm', 'icomm'),
    )
    if int(_b_utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(_b_utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_n_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(_b_utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real),), validate=False)
    else:
        _vl_f12anf_n_py = 0
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
    )
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
    )
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'mx'},
        mx,
    )
    try:
        _return_scalars = b_complex_iter(
            irevcm,
            resid,
            v,
            x,
            mx,
            comm,
            io_manager,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _irevcm_py = _return_scalars.irevcm
    _nshift_py = _return_scalars.nshift
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['irevcm', 'nshift'],
    )(
        _irevcm_py,
        _nshift_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def complex_proc(sigma, resid, v, comm, io_manager=None):
    u"""
    Selected eigenvalues and, optionally, eigenvectors of a complex
    sparse eigenproblem, postprocessing for ``complex_iter``.

    For full information please refer to the NAG Library document for
    f12aq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12aqf.html

    Parameters
    ----------
    sigma : complex
        If one of the 'Shifted Inverse' (see ``complex_option``) modes
        has been selected then `sigma` contains the shift used;
        otherwise `sigma` is not referenced.

    resid : complex, array-like, shape (n)
        Must not be modified following a call to ``complex_iter`` since
        it contains data required by ``complex_proc``.

    v : complex, array-like, shape (n, ncv)
        The ncv columns of `v` contain the Arnoldi basis vectors for OP
        as constructed by ``complex_iter``.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``complex_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nconv : int
        The number of converged eigenvalues as found by
        ``complex_option``.

    d : complex, ndarray, shape (ncv)
        The first `nconv` locations of the array this argument contain
        the converged approximate eigenvalues.

    z : complex, ndarray, shape (n, :)
        If the default option 'Vectors' = 'RITZ' (see ``real_option``)
        has been selected then this argument contains the final set of
        eigenvectors corresponding to the eigenvalues held in `d`. The
        complex eigenvector associated with an eigenvalue is stored in
        the corresponding of this argument.

    v : complex, ndarray, shape (n, ncv)
        If the option 'Vectors' = 'SCHUR' or 'RITZ' has been set and a
        separate array `z` has been passed (i.e., `z` does not equal
        this argument), then the first `nconv` columns of this argument
        will contain approximate Schur vectors that span the desired
        invariant subspace.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, 'Vectors' = Select, but this is not yet
            implemented.

        (`errno` 3)
            The number of eigenvalues found to sufficient accuracy, as
            communicated through the argument `comm`['icomm'], is zero.

        (`errno` 4)
            Got a different count of the number of converged Ritz values
            than the value passed to it through the argument
            `comm`['icomm']: number counted = *<value>*, number expected
            = *<value>*.

        (`errno` 5)
            During calculation of a Schur form, there was a failure to
            compute *<value>* eigenvalues in a total of *<value>*
            iterations.

        (`errno` 6)
            The computed Schur form could not be reordered by an
            internal call. This function returned with errno =
            *<value>*. Please contact NAG.

        (`errno` 7)
            In calculating eigenvectors, an internal call returned with
            an error. The function returned with errno = *<value>*.
            Please contact NAG.
    """

    from ..base.sparseig import complex_proc as b_complex_proc

    fun_name = 'naginterfaces.library.sparseig.complex_proc'

    _v = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_v, 'v'),),
    )
    _b_utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('comm', 'icomm'),
    )
    if int(_b_utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(_b_utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_ncv_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(_b_utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+2,), validate=False)
    else:
        _vl_f12anf_ncv_py = 0
    _d = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_vl_f12anf_ncv_py,),
    )
    if int(_b_utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(_b_utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_n_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(_b_utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real),), validate=False)
    else:
        _vl_f12anf_n_py = 0
    if _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(_b_utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+7)-1,), validate=False) == 1:
        _z_dim2 = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(_b_utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+2)-1,), validate=False)
    else:
        _z_dim2 = 1
    exp_shape_z = (_vl_f12anf_n_py, _z_dim2)
    _z = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_m_sorder,
    )
    _resid = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'resid'},
        resid,
        copy=False,
    )
    _return_scalar = b_complex_proc(
        _d,
        _z,
        sigma,
        _resid,
        _v,
        comm,
        io_manager,
    )
    _v_py = _v
    _nconv_py = _return_scalar
    _d_py = _d
    _z_py = _z
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['nconv', 'd', 'z', 'v'],
    )(
        _nconv_py,
        _d_py,
        _z_py,
        _v_py,
    )
    return _return_data

def complex_option(optstr, comm, io_manager=None):
    u"""
    Set a single option from a string
    (``complex_iter``/``complex_proc``).

    Note: this function uses optional algorithmic parameters, see also:
    ``complex_iter``, ``complex_proc``, ``complex_band_solve``,
    ``complex_band_init``, ``complex_init``.

    For full information please refer to the NAG Library document for
    f12ar

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12arf.html

    Parameters
    ----------
    optstr : str
        A single valid option string (as described in [Description] and
        [Optional Parameters]).

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``complex_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Other Parameters
    ----------------
    'Advisory' : int
        Default = advisory message unit number

        The destination for advisory messages.

    'Defaults' : valueless
        This special keyword may be used to reset all options to their
        default values.

    'Exact Shifts' : valueless
        Default

        During the Arnoldi iterative process, shifts are applied as part
        of the implicit restarting scheme.
        The shift strategy used by default and selected by the option
        'Exact Shifts' is strongly recommended over the alternative
        'Supplied Shifts' and will always be used by
        ``complex_band_solve``.

        If 'Exact Shifts' are used then these are computed internally by
        the algorithm in the implicit restarting scheme.
        This strategy is generally effective and cheaper to apply in
        terms of number of operations than using explicit shifts.

        If 'Supplied Shifts' are used then, during the Arnoldi iterative
        process, you must supply shifts through array arguments of when
        returns with irevcm = 3; the complex shifts are returned in x
        (or in comm when the option 'Pointers' = 'YES' is set). This
        option should only be used if you are an experienced user since
        this requires some algorithmic knowledge and because more
        operations are usually required than for the implicit shift
        scheme.
        Details on the use of explicit shifts and further references on
        shift strategies are available in ARPACK Users' Guide.

    'Supplied Shifts' : valueless
        During the Arnoldi iterative process, shifts are applied as part
        of the implicit restarting scheme.
        The shift strategy used by default and selected by the option
        'Exact Shifts' is strongly recommended over the alternative
        'Supplied Shifts' and will always be used by
        ``complex_band_solve``.

        If 'Exact Shifts' are used then these are computed internally by
        the algorithm in the implicit restarting scheme.
        This strategy is generally effective and cheaper to apply in
        terms of number of operations than using explicit shifts.

        If 'Supplied Shifts' are used then, during the Arnoldi iterative
        process, you must supply shifts through array arguments of when
        returns with irevcm = 3; the complex shifts are returned in x
        (or in comm when the option 'Pointers' = 'YES' is set). This
        option should only be used if you are an experienced user since
        this requires some algorithmic knowledge and because more
        operations are usually required than for the implicit shift
        scheme.
        Details on the use of explicit shifts and further references on
        shift strategies are available in ARPACK Users' Guide.

    'Iteration Limit' : int
        Default = 300

        The limit on the number of Arnoldi iterations that can be
        performed before ``complex_iter`` or ``complex_band_solve``
        exits.
        If not all requested eigenvalues have converged to within
        'Tolerance' and the number of Arnoldi iterations has reached
        this limit then ``complex_iter`` or ``complex_band_solve`` exits
        with an error; ``complex_band_solve`` returns the number of
        converged eigenvalues, the converged eigenvalues and, if
        requested, the corresponding eigenvectors, while
        ``complex_proc`` can be called subsequent to ``complex_iter`` to
        do the same.

    'Largest Magnitude' : valueless
        Default

        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``complex_iter`` or ``complex_band_solve`` to
        compute the eigenvalues of largest magnitude using 'Largest
        Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Largest Imaginary' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``complex_iter`` or ``complex_band_solve`` to
        compute the eigenvalues of largest magnitude using 'Largest
        Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Largest Real' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``complex_iter`` or ``complex_band_solve`` to
        compute the eigenvalues of largest magnitude using 'Largest
        Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Smallest Imaginary' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``complex_iter`` or ``complex_band_solve`` to
        compute the eigenvalues of largest magnitude using 'Largest
        Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Smallest Magnitude' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``complex_iter`` or ``complex_band_solve`` to
        compute the eigenvalues of largest magnitude using 'Largest
        Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Smallest Real' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``complex_iter`` or ``complex_band_solve`` to
        compute the eigenvalues of largest magnitude using 'Largest
        Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Nolist' : valueless
        Default

        Option 'List' enables printing of each option specification as
        it is supplied. 'Nolist' suppresses this printing.

    'List' : valueless
        Option 'List' enables printing of each option specification as
        it is supplied. 'Nolist' suppresses this printing.

    'Monitoring' : int
        Default = -1

        If i > 0, monitoring information is output to unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`)
        i during the solution of each problem; this may be the same as
        the 'Advisory' unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`).
        The type of information produced is dependent on the value of
        'Print Level', see the description of the option 'Print Level'
        for details of the information produced.
        Please see the ``FileObjManager`` method
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`
        to associate a file with a given unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`).

    'Pointers' : valueless
        Default = 'NO'

        During the iterative process and reverse communication calls to
        ``complex_iter``, required data can be communicated to and from
        ``complex_iter`` in one of two ways.
        When 'Pointers' = 'NO' is selected (the default) then the array
        arguments x and mx are used to supply you with required data and
        used to return computed values back to ``complex_iter``.
        For example, when irevcm = 1, ``complex_iter`` returns the
        vector x in x and the matrix-vector product Bx in mx and expects
        the result or the linear operation OP(x) to be returned in x.

        If 'Pointers' = 'YES' is selected then the data is passed
        through sections of the array argument comm.
        The section corresponding to x when 'Pointers' = 'NO' begins at
        a location given by the first element of icomm; similarly the
        section corresponding to mx begins at a location given by the
        second element of icomm.
        This option allows ``complex_iter`` to perform fewer copy
        operations on each intermediate exit and entry, but can also
        lead to less elegant code in the calling program.

        This option has no affect on ``complex_band_solve`` which sets
        'Pointers' = 'YES' internally.

    'Print Level' : int
        Default = 0

        This controls the amount of printing produced by
        ``complex_option`` as follows.

        [table omitted]

    'Random Residual' : valueless
        Default

        To begin the Arnoldi iterative process, ``complex_iter`` and
        ``complex_band_solve`` requires an initial residual vector.
        By default ``complex_iter`` and ``complex_band_solve`` provides
        its own random initial residual vector; this option can also be
        set using option 'Random Residual'.
        Alternatively, you can supply an initial residual vector
        (perhaps from a previous computation) to ``complex_iter`` and
        ``complex_band_solve`` through the array argument resid; this
        option can be set using option 'Initial Residual'.

    'Initial Residual' : valueless
        To begin the Arnoldi iterative process, ``complex_iter`` and
        ``complex_band_solve`` requires an initial residual vector.
        By default ``complex_iter`` and ``complex_band_solve`` provides
        its own random initial residual vector; this option can also be
        set using option 'Random Residual'.
        Alternatively, you can supply an initial residual vector
        (perhaps from a previous computation) to ``complex_iter`` and
        ``complex_band_solve`` through the array argument resid; this
        option can be set using option 'Initial Residual'.

    'Regular' : valueless
        Default

        These options define the computational mode which in turn
        defines the form of operation OP(x) to be performed by
        ``complex_band_solve`` or when ``complex_iter`` returns with
        irevcm = -1 or 1 and the matrix-vector product Bx when
        ``complex_iter`` returns with irevcm = -2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

    'Regular Inverse' : valueless
        These options define the computational mode which in turn
        defines the form of operation OP(x) to be performed by
        ``complex_band_solve`` or when ``complex_iter`` returns with
        irevcm = -1 or 1 and the matrix-vector product Bx when
        ``complex_iter`` returns with irevcm = -2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

    'Shifted Inverse' : valueless
        These options define the computational mode which in turn
        defines the form of operation OP(x) to be performed by
        ``complex_band_solve`` or when ``complex_iter`` returns with
        irevcm = -1 or 1 and the matrix-vector product Bx when
        ``complex_iter`` returns with irevcm = -2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

    'Standard' : valueless
        Default

        The problem to be solved is either a standard eigenvalue
        problem, Ax = lambda x, or a generalized eigenvalue problem, Ax
        = lambda Bx.
        The option 'Standard' should be used when a standard eigenvalue
        problem is being solved and the option 'Generalized' should be
        used when a generalized eigenvalue problem is being solved.

    'Generalized' : valueless
        The problem to be solved is either a standard eigenvalue
        problem, Ax = lambda x, or a generalized eigenvalue problem, Ax
        = lambda Bx.
        The option 'Standard' should be used when a standard eigenvalue
        problem is being solved and the option 'Generalized' should be
        used when a generalized eigenvalue problem is being solved.

    'Tolerance' : float
        Default = epsilon

        An approximate eigenvalue has deemed to have converged when the
        corresponding Ritz estimate is within 'Tolerance' relative to
        the magnitude of the eigenvalue.

    'Vectors' : valueless
        Default = 'RITZ'

        The function ``complex_proc`` or ``complex_band_solve`` can
        optionally compute the Schur vectors and/or the eigenvectors
        corresponding to the converged eigenvalues.
        To turn off computation of any vectors the option 'Vectors' =
        'NONE' should be set.
        To compute only the Schur vectors (at very little extra cost),
        the option 'Vectors' = 'SCHUR' should be set and these will be
        returned in the array argument v of ``complex_proc`` or
        ``complex_band_solve``.
        To compute the eigenvectors (Ritz vectors) ­corresponding to the
        eigenvalue estimates, the option 'Vectors' = 'RITZ' should be
        set and these will be returned in the array argument z of
        ``complex_proc`` or ``complex_band_solve``, if z is set equal to
        v (as in [Example]) then the Schur vectors in v are overwritten
        by the eigenvectors computed by ``complex_proc`` or
        ``complex_band_solve``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Ambiguous keyword: *<value>*

        (`errno` 2)
            Keyword not recognized: *<value>*

        (`errno` 3)
            Second keyword not recognized: *<value>*

        (`errno` 4)
            Either the initialization function has not been called prior
            to the call of this function or a communication array has
            become corrupted.
    """

    from ..base.sparseig import complex_option as b_complex_option

    b_complex_option(
        optstr,
        comm,
        io_manager,
    )

def complex_monit(comm):
    u"""
    Provides monitoring information for ``complex_iter``.

    For full information please refer to the NAG Library document for
    f12as

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12asf.html

    Parameters
    ----------
    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``complex_init``.

    Returns
    -------
    niter : int
        The number of the current Arnoldi iteration.

    nconv : int
        The number of converged eigenvalues so far.

    ritz : complex, ndarray, shape (ncv)
        The first `nconv` locations of the array this argument contain
        the converged approximate eigenvalues.

    rzest : complex, ndarray, shape (ncv)
        The first `nconv` locations of the array this argument contain
        the complex Ritz estimates on the converged approximate
        eigenvalues.
    """

    from ..base.sparseig import complex_monit as b_complex_monit

    fun_name = 'naginterfaces.library.sparseig.complex_monit'

    _b_utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm', 'comm'),
    )
    if int(_b_utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(_b_utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12anf_ncv_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (int(_b_utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+2,), validate=False)
    else:
        _vl_f12anf_ncv_py = 0
    _ritz = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_vl_f12anf_ncv_py,),
    )
    _rzest = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_vl_f12anf_ncv_py,),
    )
    _return_scalars = b_complex_monit(
        _ritz,
        _rzest,
        comm,
    )
    _niter_py = _return_scalars.niter
    _nconv_py = _return_scalars.nconv
    _ritz_py = _ritz
    _rzest_py = _rzest
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['niter', 'nconv', 'ritz', 'rzest'],
    )(
        _niter_py,
        _nconv_py,
        _ritz_py,
        _rzest_py,
    )
    return _return_data

def complex_band_init(n, nev, ncv):
    u"""
    Initialization function for ``complex_band_solve`` computing
    selected eigenvalues and, optionally, eigenvectors of a complex
    banded (standard or generalized) eigenproblem.

    For full information please refer to the NAG Library document for
    f12at

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12atf.html

    Parameters
    ----------
    n : int
        The order of the matrix A (and the order of the matrix B for the
        generalized problem) that defines the eigenvalue problem.

    nev : int
        The number of eigenvalues to be computed.

    ncv : int
        The number of Lanczos basis vectors to use during the
        computation.

        At present there is no a priori analysis to guide the selection
        of `ncv` relative to `nev`.

        However, it is recommended that `ncv` >= 2*`nev`+1.

        If many problems of the same type are to be solved, you should
        experiment with increasing `ncv` while keeping `nev` fixed for a
        given test problem.

        This will usually decrease the required number of matrix-vector
        operations but it also increases the work and storage required
        to maintain the orthogonal basis vectors.

        The optimal 'cross-over' with respect to CPU time is problem
        dependent and must be determined empirically.

    Returns
    -------
    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 2)
            On entry, `nev` = *<value>*.

            Constraint: `nev` > 0.

        (`errno` 3)
            On entry, `ncv` = *<value>*, `nev` = *<value>* and `n` =
            *<value>*.

            Constraint: `ncv` > `nev`+1 and `ncv` <= `n`.

        (`errno` 4)
            The length of the integer array `comm`['icomm'] is too small
            licomm = *<value>*, but must be at least *<value>*.

        (`errno` 5)
            The length of the complex array `comm`['comm'] is too small
            lcomm = *<value>*, but must be at least *<value>*.
    """

    from ..base.sparseig import complex_band_init as b_complex_band_init

    _return_scalar = b_complex_band_init(
        n,
        nev,
        ncv,
    )
    _comm_py = _return_scalar
    return _comm_py

def complex_band_solve(kl, ku, ab, mb, sigma, resid, comm, io_manager=None):
    u"""
    Selected eigenvalues and, optionally, eigenvectors of complex
    non-Hermitian banded eigenproblem, driver.

    For full information please refer to the NAG Library document for
    f12au

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12auf.html

    Parameters
    ----------
    kl : int
        The number of subdiagonals of the matrices A and B.

    ku : int
        The number of superdiagonals of the matrices A and B.

    ab : complex, array-like, shape (2*`kl`+`ku`+1, n)
        Must contain the matrix A in LAPACK banded storage format for
        non-Hermitian matrices (see [the F07 Introduction]).

    mb : complex, array-like, shape (2*`kl`+`ku`+1, n)
        Must contain the matrix B in LAPACK banded storage format for
        non-Hermitian matrices (see [the F07 Introduction]).

    sigma : complex
        If the 'Shifted Inverse' mode (see ``complex_option``) has been
        selected then `sigma` must contain the shift used; otherwise
        `sigma` is not referenced. [the F12 Introduction] describes the
        use of shift and invert transformations.

    resid : complex, array-like, shape (n)
        Need not be set unless the option 'Initial Residual' has been
        set in a prior call to ``complex_option`` in which case `resid`
        must contain an initial residual vector.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``complex_band_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nconv : int
        The number of converged eigenvalues.

    d : complex, ndarray, shape (nev)
        The first `nconv` locations of the array this argument contain
        the converged approximate eigenvalues.

    z : None or complex, ndarray, shape (n, :)
        If the default option 'Vectors' = 'RITZ' (see
        ``complex_option``) has been selected then this argument
        contains the final set of eigenvectors corresponding to the
        eigenvalues held in `d`, otherwise this argument is not
        referenced. The complex eigenvector associated with an
        eigenvalue `d`[j] is stored in the corresponding array section
        of this argument, namely `z`[i-1,j-1], for j = 1,2,...,`nconv`,
        for i = 1,2,...,n.

    resid : complex, ndarray, shape (n)
        Contains the final residual vector. This can be used as the
        starting residual to improve convergence on the solution of a
        closely related eigenproblem. This has no relation to the error
        residual Ax-lambda x or Ax-lambda Bx.

    v : complex, ndarray, shape (n, ncv)
        If the option 'Vectors' = 'SCHUR' or 'RITZ' (see
        ``complex_option``) has been set and a separate array `z` has
        been passed (i.e., `z` does not equal this argument), then the
        first `nconv` columns of this argument will contain approximate
        Schur vectors that span the desired invariant subspace.

        The jth Schur vector is stored in the ith column of `v`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `kl` = *<value>*.

            Constraint: `kl` >= 0.

        (`errno` 2)
            On entry, `ku` = *<value>*.

            Constraint: `ku` >= 0.

        (`errno` 3)
            On entry, ldab = *<value>*, 2*`kl`+`ku`+1 = *<value>*.

            Constraint: ldab >= 2*`kl`+`ku`+1.

        (`errno` 5)
            The maximum number of iterations <= 0, the option 'Iteration
            Limit' has been set to *<value>*.

        (`errno` 6)
            The options 'Generalized' and 'Regular' are incompatible.

        (`errno` 7)
            The option 'Initial Residual' was selected but the starting
            vector held in `resid` is zero.

        (`errno` 8)
            Either the initialization function has not been called prior
            to the first call of this function or a communication array
            has become corrupted.

        (`errno` 10)
            On entry, 'Vectors' = Select, but this is not yet
            implemented.

        (`errno` 11)
            The number of eigenvalues found to sufficient accuracy is
            zero.

        (`errno` 12)
            Could not build an Arnoldi factorization. The size of the
            current Arnoldi factorization = *<value>*.

        (`errno` 13)
            Error in internal call to compute eigenvalues and
            corresponding error bounds of the current upper Hessenberg
            matrix. Please contact NAG.

        (`errno` 14)
            During calculation of a Schur form, there was a failure to
            compute a number of eigenvalues Please contact NAG.

        (`errno` 15)
            The computed Schur form could not be reordered by an
            internal call. Please contact NAG.

        (`errno` 16)
            Error in internal call to compute eigenvectors. Please
            contact NAG.

        (`errno` 17)
            Failure during internal factorization of real banded matrix.
            Please contact NAG.

        (`errno` 18)
            Failure during internal solution of real banded matrix.
            Please contact NAG.

        (`errno` 19)
            Failure during internal factorization of complex banded
            matrix. Please contact NAG.

        (`errno` 20)
            Failure during internal solution of complex banded matrix.
            Please contact NAG.

        (`errno` 21)
            The maximum number of iterations has been reached. The
            maximum number of iterations = *<value>*. The number of
            converged eigenvalues = *<value>*.

        (`errno` 22)
            No shifts could be applied during a cycle of the implicitly
            restarted Arnoldi iteration.

        (`errno` 23)
            Overflow occurred during transformation of Ritz values to
            those of the original problem.
    """

    from ..base.sparseig import complex_band_solve as b_complex_band_solve

    fun_name = 'naginterfaces.library.sparseig.complex_band_solve'

    _ab = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ab'},
        ab,
        copy=False,
    )
    _mb = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'mb'},
        mb,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_ab, 'ab'), (_mb, 'mb')),
    )
    _b_utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('comm', 'icomm'),
    )
    if int(_b_utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(_b_utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12atf_nev_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (21,), validate=False)
    else:
        _vl_f12atf_nev_py = 0
    _d = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_vl_f12atf_nev_py,),
    )
    if int(_b_utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(_b_utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12atf_n_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (20,), validate=False)
    else:
        _vl_f12atf_n_py = 0
    if _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(_b_utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+7)-1,), validate=False) == 1:
        if _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(_b_utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+7)-1,), validate=False) == 1:
            _z_dim2 = _vl_f12atf_nev_py
        else:
            _z_dim2 = 1
        exp_shape_z = (_vl_f12atf_n_py, _z_dim2)
        _z = _b_utils._EngineComplex128ArrayType.empty_ndarray(
            exp_shape_z,
            sorder=_m_sorder,
        )
    else:
        _z = None
    _resid = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'resid'},
        resid,
        copy=True,
    )
    if int(_b_utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(_b_utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12atf_ncv_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (22,), validate=False)
    else:
        _vl_f12atf_ncv_py = 0
    _v = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_vl_f12atf_n_py, _vl_f12atf_ncv_py),
        sorder=_m_sorder,
    )
    _return_scalar = b_complex_band_solve(
        kl,
        ku,
        _ab,
        _mb,
        sigma,
        _d,
        _z,
        _resid,
        _v,
        comm,
        io_manager,
    )
    _resid_py = _resid
    _nconv_py = _return_scalar
    _d_py = _d
    if _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(_b_utils._EngineComplex128ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real)+7)-1,), validate=False) == 1:
        _z_py = _z
    else:
        _z_py = None
    _v_py = _v
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['nconv', 'd', 'z', 'resid', 'v'],
    )(
        _nconv_py,
        _d_py,
        _z_py,
        _resid_py,
        _v_py,
    )
    return _return_data

def real_symm_init(n, nev, ncv):
    u"""
    Initialization routine for (``real_symm_iter``) computing selected
    eigenvalues and, optionally, eigenvectors of a real symmetric sparse
    (standard or generalized) eigenproblem.

    For full information please refer to the NAG Library document for
    f12fa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12faf.html

    Parameters
    ----------
    n : int
        The order of the matrix A (and the order of the matrix B for the
        generalized problem) that defines the eigenvalue problem.

    nev : int
        The number of eigenvalues to be computed.

    ncv : int
        The number of Lanczos basis vectors to use during the
        computation.

        At present there is no a priori analysis to guide the selection
        of `ncv` relative to `nev`.

        However, it is recommended that `ncv` >= 2*`nev`+1.

        If many problems of the same type are to be solved, you should
        experiment with increasing `ncv` while keeping `nev` fixed for a
        given test problem.

        This will usually decrease the required number of matrix-vector
        operations but it also increases the work and storage required
        to maintain the orthogonal basis vectors.

        The optimal 'cross-over' with respect to CPU time is problem
        dependent and must be determined empirically.

    Returns
    -------
    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 2)
            On entry, `nev` = *<value>*.

            Constraint: `nev` > 0.

        (`errno` 3)
            On entry, `ncv` = *<value>*, `nev` = *<value>* and `n` =
            *<value>*.

            Constraint: `ncv` > `nev` and `ncv` <= `n`.

        (`errno` 4)
            The length of the integer array `comm`['icomm'] is too small
            licomm = *<value>*, but must be at least *<value>*.

        (`errno` 5)
            On entry, lcomm = *<value>*, `n` = *<value>* and `ncv` =
            *<value>*.

            Constraint: lcomm >= 3*`n`+`ncv`*`ncv`+8*`ncv`+60.
    """

    from ..base.sparseig import real_symm_init as b_real_symm_init

    _return_scalar = b_real_symm_init(
        n,
        nev,
        ncv,
    )
    _comm_py = _return_scalar
    return _comm_py

def real_symm_iter(irevcm, resid, v, x, mx, comm, io_manager=None):
    u"""
    Selected eigenvalues and, optionally, eigenvectors of a real
    symmetric sparse eigenproblem, reverse communication.

    For full information please refer to the NAG Library document for
    f12fb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12fbf.html

    Parameters
    ----------
    irevcm : int
        On initial entry: `irevcm` = 0, otherwise an error condition
        will be raised.

        On intermediate entry: must be unchanged from its previous exit
        value. Changing `irevcm` to any other value between calls will
        result in an error.

    resid : float, ndarray, shape (n), modified in place
        On initial entry: need not be set unless the option 'Initial
        Residual' has been set in a prior call to ``real_symm_option``
        in which case `resid` should contain an initial residual vector,
        possibly from a previous run.

        On intermediate entry: must be unchanged from its previous exit.
        Changing `resid` to any other value between calls may result in
        an error exit.

        On intermediate exit: contains the current residual vector.

        On final exit: contains the final residual vector.

    v : float, ndarray, shape (n, ncv), modified in place
        On initial entry: need not be set.

        On intermediate entry: must be unchanged from its previous exit.

        On intermediate exit: contains the current set of Arnoldi basis
        vectors.

        On final exit: contains the final set of Arnoldi basis vectors.

    x : float, ndarray, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if int(`comm`['comm'][0])>=1 and
        int(`comm`['comm'][0])<=1000: if
        `comm`['icomm'][(int(`comm`['comm'][0])+24)-1]=0:
        `comm`['icomm'][(int(`comm`['comm'][0])+1)-1]; otherwise: 1;
        otherwise: 0.

        On initial entry: need not be set, it is used as a convenient
        mechanism for accessing elements of `comm`['comm'].

        On intermediate entry: if 'Pointers' = YES, `x` need not be set.

        If 'Pointers' = NO, `x` must contain the result of y = OPx when
        `irevcm` returns the value -1 or +1.

        It must return the real parts of the computed shifts when
        `irevcm` returns the value 3.

        On intermediate exit: if 'Pointers' = YES, `x` is not
        referenced.

        If 'Pointers' = NO, `x` contains the vector x when `irevcm`
        returns the value -1 or +1.

        On final exit: does not contain useful data.

    mx : float, ndarray, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if int(`comm`['comm'][0])>=1 and
        int(`comm`['comm'][0])<=1000: if
        `comm`['icomm'][(int(`comm`['comm'][0])+24)-1]=0:
        `comm`['icomm'][(int(`comm`['comm'][0])+1)-1]; otherwise: 1;
        otherwise: 0.

        On initial entry: need not be set, it is used as a convenient
        mechanism for accessing elements of `comm`['comm'].

        On intermediate entry: if 'Pointers' = YES, `mx` need not be
        set.

        If 'Pointers' = NO, `mx` must contain the result of y = Bx when
        `irevcm` returns the value 2.

        It must return the imaginary parts of the computed shifts when
        `irevcm` returns the value 3.

        On intermediate exit: if 'Pointers' = YES, `mx` is not
        referenced.

        If 'Pointers' = NO, `mx` contains the vector Bx when `irevcm`
        returns the value +1.

        On final exit: does not contain any useful data.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_symm_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    irevcm : int
        On intermediate exit: has the following meanings.

        `irevcm` = -1

          The calling program must compute the matrix-vector product y =
          OPx, where x is stored in `x` (by default) or in the array
          `comm`['comm'] (starting from the location given by the first
          element of `comm`['icomm']) when the option 'Pointers' = 'YES'
          is set in a prior call to ``real_symm_option``. The result y
          is returned in `x` (by default) or in the array `comm`['comm']
          (starting from the location given by the second element of
          `comm`['icomm']) when the option 'Pointers' = 'YES' is set.

        `irevcm` = 1

          The calling program must compute the matrix-vector product y =
          OPx. This is similar to the case `irevcm` = -1 except that the
          result of the matrix-vector product Bx (as required in some
          computational modes) has already been computed and is
          available in `mx` (by default) or in the array `comm`['comm']
          (starting from the location given by the third element of
          `comm`['icomm']) when the option 'Pointers' = 'YES' is set.

        `irevcm` = 2

          The calling program must compute the matrix-vector product y =
          Bx, where x is stored in `x` and y is returned in `mx` (by
          default) or in the array `comm`['comm'] (starting from the
          location given by the second element of `comm`['icomm']) when
          the option 'Pointers' = 'YES' is set.

        `irevcm` = 3

          Compute the `nshift` real and imaginary parts of the shifts
          where the real parts are to be returned in the first `nshift`
          locations of the array `x` and the imaginary parts are to be
          returned in the first `nshift` locations of the array `mx`.
          Only complex conjugate pairs of shifts may be applied and the
          pairs must be placed in consecutive locations. This value of
          this argument will only arise if the option 'Supplied Shifts'
          is set in a prior call to ``real_symm_option`` which is
          intended for experienced users only; the default and
          recommended option is to use exact shifts (see ARPACK Users'
          Guide for details and guidance on the choice of shift
          strategies).

        `irevcm` = 4

          Monitoring step: a call to ``real_symm_monit`` can now be made
          to return the number of Arnoldi iterations, the number of
          converged Ritz values, their real and imaginary parts, and the
          corresponding Ritz estimates.

        On final exit: `irevcm` = 5: ``real_symm_iter`` has completed
        its tasks. The value of errno determines whether the iteration
        has been successfully completed, or whether errors have been
        detected. On successful completion ``real_symm_proc`` must be
        called to return the requested eigenvalues and eigenvectors
        (and/or Schur vectors).

    nshift : int
        On intermediate exit: if the option 'Supplied Shifts' is set and
        `irevcm` returns a value of 3, this argument returns the number
        of complex shifts required.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The maximum number of iterations <= 0, the option 'Iteration
            Limit' has been set to *<value>*.

        (`errno` 2)
            The options 'Generalized' and 'Regular' are incompatible.

        (`errno` 3)
            Eigenvalues from both ends of the spectrum were requested,
            but the number of eigenvalues (see nev in
            ``real_symm_init``) requested is one.

        (`errno` 4)
            The option 'Initial Residual' was selected but the starting
            vector held in `resid` is zero.

        (`errno` 6)
            No shifts could be applied during a cycle of the implicitly
            restarted Lanczos iteration.

        (`errno` 7)
            Could not build a Lanczos factorization. The size of the
            current Lanczos factorization = *<value>*.

        (`errno` 8)
            Error in internal call to compute eigenvalues and
            corresponding error bounds of the current upper Hessenberg
            matrix. Please contact NAG.

        (`errno` 9)
            Either the function was called without an initial call to
            the setup function or the communication arrays have become
            corrupted.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 5)
            The maximum number of iterations has been reached. The
            maximum number of iterations = *<value>*. The number of
            converged eigenvalues = *<value>*. See the function document
            for further details.
    """

    from ..base.sparseig import real_symm_iter as b_real_symm_iter

    fun_name = 'naginterfaces.library.sparseig.real_symm_iter'

    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'resid'},
        resid,
    )
    _b_utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('comm', 'icomm'),
    )
    if int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12faf_n_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (20,), validate=False)
    else:
        _vl_f12faf_n_py = 0
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
    )
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
    )
    _utils._check_is_ndarray(
        {'fun_name': fun_name, 'entity': 'mx'},
        mx,
    )
    try:
        _return_scalars = b_real_symm_iter(
            irevcm,
            resid,
            v,
            x,
            mx,
            comm,
            io_manager,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _irevcm_py = _return_scalars.irevcm
    _nshift_py = _return_scalars.nshift
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['irevcm', 'nshift'],
    )(
        _irevcm_py,
        _nshift_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def real_symm_proc(sigma, resid, v, comm, io_manager=None):
    u"""
    Selected eigenvalues and, optionally, eigenvectors of a real
    symmetric sparse eigenproblem, postprocessing for
    ``real_symm_iter``.

    For full information please refer to the NAG Library document for
    f12fc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12fcf.html

    Parameters
    ----------
    sigma : float
        If one of the 'Shifted Inverse' (see ``real_symm_option``) modes
        has been selected then `sigma` contains the real shift used;
        otherwise `sigma` is not referenced.

    resid : float, array-like, shape (n)
        Must not be modified following a call to ``real_symm_iter``
        since it contains data required by ``real_symm_proc``.

    v : float, array-like, shape (n, ncv)
        The ncv columns of `v` contain the Lanczos basis vectors for OP
        as constructed by ``real_symm_iter``.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_symm_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nconv : int
        The number of converged eigenvalues as found by
        ``real_symm_iter``.

    d : float, ndarray, shape (ncv)
        The first `nconv` locations of the array this argument contain
        the converged approximate eigenvalues.

    z : float, ndarray, shape (n, :)
        If the default option 'Vectors' = 'RITZ' (see
        ``real_symm_option``) has been selected then this argument
        contains the final set of eigenvectors corresponding to the
        eigenvalues held in `d`. The real eigenvector associated with an
        eigenvalue is stored in the corresponding of this argument.

    v : float, ndarray, shape (n, ncv)
        If the option 'Vectors' = 'SCHUR' has been set, or the option
        'Vectors' = 'RITZ' has been set and a separate array `z` has
        been passed (i.e., `z` does not equal this argument), then the
        first `nconv` columns of this argument will contain approximate
        Schur vectors that span the desired invariant subspace.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, 'Vectors' = 'SELECT', but this is not yet
            implemented.

        (`errno` 3)
            The number of eigenvalues found to sufficient accuracy, as
            communicated through the argument `comm`['icomm'], is zero.

        (`errno` 4)
            Got a different count of the number of converged Ritz values
            than the value passed to it through the argument
            `comm`['icomm']: number counted = *<value>*, number expected
            = *<value>*.

        (`errno` 5)
            During calculation of a tridiagonal form, there was a
            failure to compute *<value>* eigenvalues in a total of
            *<value>* iterations.

        (`errno` 8)
            Either the function was called out of sequence (following an
            initial call to the setup function and following completion
            of calls to the reverse communication function) or the
            communication arrays have become corrupted.
    """

    from ..base.sparseig import real_symm_proc as b_real_symm_proc

    fun_name = 'naginterfaces.library.sparseig.real_symm_proc'

    _v = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_v, 'v'),),
    )
    _b_utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('comm', 'icomm'),
    )
    if int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12faf_ncv_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (22,), validate=False)
    else:
        _vl_f12faf_ncv_py = 0
    _d = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_f12faf_ncv_py,),
    )
    if int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12faf_n_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (20,), validate=False)
    else:
        _vl_f12faf_n_py = 0
    if int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False)) >= 1 and int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False)) <= 1000:
        if _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+7)-1,), validate=False) == 1:
            _z_dim2 = _vl_f12faf_ncv_py
        else:
            _z_dim2 = 1
    else:
        _z_dim2 = 1
    exp_shape_z = (_vl_f12faf_n_py, _z_dim2)
    _z = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_m_sorder,
    )
    _resid = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'resid'},
        resid,
        copy=False,
    )
    _return_scalar = b_real_symm_proc(
        _d,
        _z,
        sigma,
        _resid,
        _v,
        comm,
        io_manager,
    )
    _v_py = _v
    _nconv_py = _return_scalar
    _d_py = _d
    _z_py = _z
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['nconv', 'd', 'z', 'v'],
    )(
        _nconv_py,
        _d_py,
        _z_py,
        _v_py,
    )
    return _return_data

def real_symm_option(optstr, comm, io_manager=None):
    u"""
    Set a single option from a string
    (``real_symm_iter``/``real_symm_proc``/``real_symm_band_solve``).

    Note: this function uses optional algorithmic parameters, see also:
    ``real_symm_iter``, ``real_symm_proc``, ``real_symm_init``.

    For full information please refer to the NAG Library document for
    f12fd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12fdf.html

    Parameters
    ----------
    optstr : str
        A single valid option string (as described in [Description] and
        [Optional Parameters]).

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_symm_init``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Other Parameters
    ----------------
    'Advisory' : int
        Default = advisory message unit number

        The destination for advisory messages.

    'Defaults' : valueless
        This special keyword may be used to reset all options to their
        default values.

    'Exact Shifts' : valueless
        Default

        During the Lanczos iterative process, shifts are applied
        internally as part of the implicit restarting scheme.
        The shift strategy used by default and selected by the 'Exact
        Shifts' is strongly recommended over the alternative 'Supplied
        Shifts' (see ARPACK Users' Guide for details of shift
        strategies).

        If 'Exact Shifts' are used then these are computed internally by
        the algorithm in the implicit restarting scheme.

        If 'Supplied Shifts' are used then, during the Lanczos iterative
        process, you must supply shifts through array arguments of when
        returns with irevcm = 3; the real and imaginary parts of the
        shifts are returned in x and mx respectively (or in comm when
        the option 'Pointers' = 'YES' is set). This option should only
        be used if you are an experienced user since this requires some
        algorithmic knowledge and because more operations are usually
        required than for the implicit shift scheme.
        Details on the use of explicit shifts and further references on
        shift strategies are available in ARPACK Users' Guide.

    'Supplied Shifts' : valueless
        During the Lanczos iterative process, shifts are applied
        internally as part of the implicit restarting scheme.
        The shift strategy used by default and selected by the 'Exact
        Shifts' is strongly recommended over the alternative 'Supplied
        Shifts' (see ARPACK Users' Guide for details of shift
        strategies).

        If 'Exact Shifts' are used then these are computed internally by
        the algorithm in the implicit restarting scheme.

        If 'Supplied Shifts' are used then, during the Lanczos iterative
        process, you must supply shifts through array arguments of when
        returns with irevcm = 3; the real and imaginary parts of the
        shifts are returned in x and mx respectively (or in comm when
        the option 'Pointers' = 'YES' is set). This option should only
        be used if you are an experienced user since this requires some
        algorithmic knowledge and because more operations are usually
        required than for the implicit shift scheme.
        Details on the use of explicit shifts and further references on
        shift strategies are available in ARPACK Users' Guide.

    'Iteration Limit' : int
        Default = 300

        The limit on the number of Lanczos iterations that can be
        performed before ``real_symm_iter`` exits.
        If not all requested eigenvalues have converged to within
        'Tolerance' and the number of Lanczos iterations has reached
        this limit then ``real_symm_iter`` exits with an error;
        ``real_symm_proc`` can still be called subsequently to return
        the number of converged eigenvalues, the converged eigenvalues
        and, if requested, the corresponding eigenvectors.

    'Largest Magnitude' : valueless
        Default

        The Lanczos iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``real_symm_iter`` to compute the eigenvalues
        of largest magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Algebraic' part, 'Smallest Magnitude', or 'Smallest Algebraic'
        part; or eigenvalues which are from 'Both Ends' of the algebraic
        spectrum.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Both Ends' : valueless
        The Lanczos iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``real_symm_iter`` to compute the eigenvalues
        of largest magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Algebraic' part, 'Smallest Magnitude', or 'Smallest Algebraic'
        part; or eigenvalues which are from 'Both Ends' of the algebraic
        spectrum.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Largest Algebraic' : valueless
        The Lanczos iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``real_symm_iter`` to compute the eigenvalues
        of largest magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Algebraic' part, 'Smallest Magnitude', or 'Smallest Algebraic'
        part; or eigenvalues which are from 'Both Ends' of the algebraic
        spectrum.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Smallest Algebraic' : valueless
        The Lanczos iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``real_symm_iter`` to compute the eigenvalues
        of largest magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Algebraic' part, 'Smallest Magnitude', or 'Smallest Algebraic'
        part; or eigenvalues which are from 'Both Ends' of the algebraic
        spectrum.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Smallest Magnitude' : valueless
        The Lanczos iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``real_symm_iter`` to compute the eigenvalues
        of largest magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Algebraic' part, 'Smallest Magnitude', or 'Smallest Algebraic'
        part; or eigenvalues which are from 'Both Ends' of the algebraic
        spectrum.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP (and B for 'Generalized' problems), the linear
        operator determined by the computational mode and problem type.

    'Nolist' : valueless
        Default

        Option 'List' enables printing of each option specification as
        it is supplied. 'Nolist' suppresses this printing.

    'List' : valueless
        Option 'List' enables printing of each option specification as
        it is supplied. 'Nolist' suppresses this printing.

    'Monitoring' : int
        Default = -1

        If i > 0, monitoring information is output to unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`)
        i during the solution of each problem; this may be the same as
        the 'Advisory' unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`).
        The type of information produced is dependent on the value of
        'Print Level', see the description of the option 'Print Level'
        for details of the information produced.
        Please see the ``FileObjManager`` method
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`
        to associate a file with a given unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`).

    'Pointers' : valueless
        Default = NO

        During the iterative process and reverse communication calls to,
        required data can be communicated to and from in one of two
        ways.
        When 'Pointers' = 'NO' is selected (the default) then the array
        arguments x and mx are used to supply you with required data and
        used to return computed values back to.
        For example, when irevcm = 1, returns the vector x in x and the
        matrix-vector product Bx in mx and expects the result or the
        linear operation OP(x) to be returned in x.

        If 'Pointers' = 'YES' is selected then the data is passed
        through sections of the array argument comm.
        The section corresponding to x when 'Pointers' = 'NO' begins at
        a location given by the first element of icomm; similarly the
        section corresponding to mx begins at a location given by the
        second element of icomm.
        This option allows to perform fewer copy operations on each
        intermediate exit and entry, but can also lead to less elegant
        code in the calling program.

    'Print Level' : int
        Default = 0

        This controls the amount of printing produced by
        ``real_symm_option`` as follows.

        [table omitted]

        Note that setting 'Print Level' >= 30 can result in very lengthy
        'Monitoring' output.

        Note that setting 'Print Level' >= 30 can result in very lengthy
        'Monitoring' output.

    'Random Residual' : valueless
        Default

        To begin the Lanczos iterative process, requires an initial
        residual vector.
        By default provides its own random initial residual vector; this
        option can also be set using option 'Random Residual'.
        Alternatively, you can supply an initial residual vector
        (perhaps from a previous computation) to through the array
        argument resid; this option can be set using option 'Initial
        Residual'.

    'Initial Residual' : valueless
        To begin the Lanczos iterative process, requires an initial
        residual vector.
        By default provides its own random initial residual vector; this
        option can also be set using option 'Random Residual'.
        Alternatively, you can supply an initial residual vector
        (perhaps from a previous computation) to through the array
        argument resid; this option can be set using option 'Initial
        Residual'.

    'Regular' : valueless
        Default

        These options define the computational mode which in turn
        defines the form of operation OP(x) to be performed when returns
        with irevcm = -1 or 1 and the matrix-vector product Bx when
        returns with irevcm = 2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

    'Regular Inverse' : valueless
        These options define the computational mode which in turn
        defines the form of operation OP(x) to be performed when returns
        with irevcm = -1 or 1 and the matrix-vector product Bx when
        returns with irevcm = 2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

    'Shifted Inverse' : valueless
        These options define the computational mode which in turn
        defines the form of operation OP(x) to be performed when returns
        with irevcm = -1 or 1 and the matrix-vector product Bx when
        returns with irevcm = 2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

    'Buckling' : valueless
        These options define the computational mode which in turn
        defines the form of operation OP(x) to be performed when returns
        with irevcm = -1 or 1 and the matrix-vector product Bx when
        returns with irevcm = 2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

    'Cayley' : valueless
        These options define the computational mode which in turn
        defines the form of operation OP(x) to be performed when returns
        with irevcm = -1 or 1 and the matrix-vector product Bx when
        returns with irevcm = 2.

        Given a 'Standard' eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

        Given a 'Generalized' eigenvalue problem in the form Ax = lambda
        Bx then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

    'Standard' : valueless
        Default

        The problem to be solved is either a standard eigenvalue
        problem, Ax = lambda x, or a generalized eigenvalue problem, Ax
        = lambda Bx.
        The option 'Standard' should be used when a standard eigenvalue
        problem is being solved and the option 'Generalized' should be
        used when a generalized eigenvalue problem is being solved.

    'Generalized' : valueless
        The problem to be solved is either a standard eigenvalue
        problem, Ax = lambda x, or a generalized eigenvalue problem, Ax
        = lambda Bx.
        The option 'Standard' should be used when a standard eigenvalue
        problem is being solved and the option 'Generalized' should be
        used when a generalized eigenvalue problem is being solved.

    'Tolerance' : float
        Default = epsilon

        An approximate eigenvalue has deemed to have converged when the
        corresponding Ritz estimate is within 'Tolerance' relative to
        the magnitude of the eigenvalue.

    'Vectors' : valueless
        Default = RITZ

        The function can optionally compute the Schur vectors and/or the
        eigenvectors corresponding to the converged eigenvalues.
        To turn off computation of any vectors the option 'Vectors' =
        'NONE' should be set.
        To compute only the Schur vectors (at very little extra cost),
        the option 'Vectors' = 'SCHUR' should be set and these will be
        returned in the array argument v of.
        To compute the eigenvectors (Ritz vectors) ­corresponding to the
        eigenvalue estimates, the option 'Vectors' = 'RITZ' should be
        set and these will be returned in the array argument z of, if z
        is set equal to v (as in [Example]) then the Schur vectors in v
        are overwritten by the eigenvectors computed by.

    Raises
    ------
    NagValueError
        (`errno` 1)
            Ambiguous keyword: *<value>*

        (`errno` 2)
            Keyword not recognized: *<value>*

        (`errno` 3)
            Second keyword not recognized: *<value>*
    """

    from ..base.sparseig import real_symm_option as b_real_symm_option

    b_real_symm_option(
        optstr,
        comm,
        io_manager,
    )

def real_symm_monit(comm):
    u"""
    Provides monitoring information for ``real_symm_iter``.

    For full information please refer to the NAG Library document for
    f12fe

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12fef.html

    Parameters
    ----------
    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_symm_init``.

    Returns
    -------
    niter : int
        The number of the current Arnoldi iteration.

    nconv : int
        The number of converged eigenvalues so far.

    ritz : float, ndarray, shape (ncv)
        The first `nconv` locations of the array this argument contain
        the real converged approximate eigenvalues.

    rzest : float, ndarray, shape (ncv)
        The first `nconv` locations of the array this argument contain
        the Ritz estimates (error bounds) on the real `nconv` converged
        approximate eigenvalues.
    """

    from ..base.sparseig import real_symm_monit as b_real_symm_monit

    fun_name = 'naginterfaces.library.sparseig.real_symm_monit'

    _b_utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm', 'comm'),
    )
    if int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12faf_ncv_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (22,), validate=False)
    else:
        _vl_f12faf_ncv_py = 0
    _ritz = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_f12faf_ncv_py,),
    )
    _rzest = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_f12faf_ncv_py,),
    )
    _return_scalars = b_real_symm_monit(
        _ritz,
        _rzest,
        comm,
    )
    _niter_py = _return_scalars.niter
    _nconv_py = _return_scalars.nconv
    _ritz_py = _ritz
    _rzest_py = _rzest
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['niter', 'nconv', 'ritz', 'rzest'],
    )(
        _niter_py,
        _nconv_py,
        _ritz_py,
        _rzest_py,
    )
    return _return_data

def real_symm_band_init(n, nev, ncv):
    u"""
    Initialization routine for (``real_symm_band_solve``) computing
    selected eigenvalues and, optionally, eigenvectors of a real
    symmetric banded (standard or generalized) eigenproblem.

    For full information please refer to the NAG Library document for
    f12ff

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12fff.html

    Parameters
    ----------
    n : int
        The order of the matrix A (and the order of the matrix B for the
        generalized problem) that defines the eigenvalue problem.

    nev : int
        The number of eigenvalues to be computed.

    ncv : int
        The number of Lanczos basis vectors to use during the
        computation.

        At present there is no a priori analysis to guide the selection
        of `ncv` relative to `nev`.

        However, it is recommended that `ncv` >= 2*`nev`+1.

        If many problems of the same type are to be solved, you should
        experiment with increasing `ncv` while keeping `nev` fixed for a
        given test problem.

        This will usually decrease the required number of matrix-vector
        operations but it also increases the work and storage required
        to maintain the orthogonal basis vectors.

        The optimal 'cross-over' with respect to CPU time is problem
        dependent and must be determined empirically.

    Returns
    -------
    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 2)
            On entry, `nev` = *<value>*.

            Constraint: `nev` > 0.

        (`errno` 3)
            On entry, `ncv` = *<value>*, `nev` = *<value>* and `n` =
            *<value>*.

            Constraint: `ncv` > `nev`+1 and `ncv` <= `n`.

        (`errno` 4)
            The length of the integer array `comm`['icomm'] is too small
            licomm = *<value>*, but must be at least *<value>*.

        (`errno` 5)
            The length of the integer array `comm`['comm'] is too small
            lcomm = *<value>*, but must be at least *<value>*.
    """

    from ..base.sparseig import real_symm_band_init as b_real_symm_band_init

    _return_scalar = b_real_symm_band_init(
        n,
        nev,
        ncv,
    )
    _comm_py = _return_scalar
    return _comm_py

def real_symm_band_solve(kl, ku, ab, mb, sigma, resid, comm, io_manager=None):
    u"""
    Selected eigenvalues and, optionally, eigenvectors of a real
    symmetric banded eigenproblem, driver.

    For full information please refer to the NAG Library document for
    f12fg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f12/f12fgf.html

    Parameters
    ----------
    kl : int
        The number of subdiagonals of the matrices A and B.

    ku : int
        The number of superdiagonals of the matrices A and B. Since A
        and B are symmetric, the normal case is `ku` = `kl`.

    ab : float, array-like, shape (2*`kl`+`ku`+1, n)
        Must contain the matrix A in LAPACK banded storage format for
        nonsymmetric matrices (see [the F07 Introduction]).

    mb : float, array-like, shape (2*`kl`+`ku`+1, n)
        Must contain the matrix B in LAPACK banded storage format for
        nonsymmetric matrices (see [the F07 Introduction]).

    sigma : float
        If one of the 'Shifted Inverse' (see ``real_symm_option``) modes
        has been selected then `sigma` contains the real shift used;
        otherwise `sigma` is not referenced.

    resid : float, array-like, shape (n)
        Need not be set unless the option 'Initial Residual' has been
        set in a prior call to ``real_symm_option`` in which case
        `resid` must contain an initial residual vector.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``real_symm_option``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nconv : int
        The number of converged eigenvalues.

    d : float, ndarray, shape (ncv)
        The first `nconv` locations of the array this argument contain
        the converged approximate eigenvalues.

    z : float, ndarray, shape (n, :)
        If the default option 'Vectors' = 'RITZ' (see
        ``real_symm_option``) has been selected then this argument
        contains the final set of eigenvectors corresponding to the
        eigenvalues held in `d`. The real eigenvector associated with
        eigenvalue i-1, for i = 1,2,...,`nconv`, is stored in the ith
        column of `z`.

    resid : float, ndarray, shape (n)
        Contains the final residual vector.

    v : float, ndarray, shape (n, ncv)
        If the option 'Vectors' (see ``real_symm_option``) has been set
        to Schur or Ritz and a separate array `z` has been passed then
        the first `nconv`*n elements of this argument will contain
        approximate Schur vectors that span the desired invariant
        subspace.

        The jth Schur vector is stored in the ith column of `v`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `kl` = *<value>*.

            Constraint: `kl` >= 0.

        (`errno` 2)
            On entry, `ku` = *<value>*.

            Constraint: `ku` >= 0.

        (`errno` 3)
            On entry, ldab = *<value>*, 2*`kl`+`ku`+1 = *<value>*.

            Constraint: ldab >= 2*`kl`+`ku`+1.

        (`errno` 4)
            The maximum number of iterations <= 0, the option 'Iteration
            Limit' has been set to *<value>*.

        (`errno` 5)
            The options 'Generalized' and 'Regular' are incompatible.

        (`errno` 6)
            Eigenvalues from both ends of the spectrum were requested,
            but the number of eigenvalues (nev in
            ``real_symm_band_init``) requested is one.

        (`errno` 7)
            The option 'Initial Residual' was selected but the starting
            vector held in `resid` is zero.

        (`errno` 9)
            On entry, 'Vectors' = Select, but this is not yet
            implemented.

        (`errno` 10)
            The number of eigenvalues found to sufficient accuracy is
            zero.

        (`errno` 11)
            Could not build a Lanczos factorization. The size of the
            current Lanczos factorization = *<value>*.

        (`errno` 12)
            Error in internal call to compute eigenvalues and
            corresponding error bounds of the current upper Hessenberg
            matrix. Please contact NAG.

        (`errno` 13)
            During calculation of a tridiagonal form, there was a
            failure to compute *<value>* eigenvalues in a total of
            *<value>* iterations.

        (`errno` 14)
            Failure during internal factorization of banded matrix.
            Please contact NAG.

        (`errno` 15)
            Failure during internal solution of banded system. Please
            contact NAG.

        (`errno` 17)
            No shifts could be applied during a cycle of the implicitly
            restarted Lanczos iteration.

        (`errno` 18)
            Either an initial call to the setup function has not been
            made or the communication arrays have become corrupted.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 16)
            The maximum number of iterations has been reached: there
            have been *<value>* iterations. There are *<value>*
            converged eigenvalues.
    """

    from ..base.sparseig import real_symm_band_solve as b_real_symm_band_solve

    fun_name = 'naginterfaces.library.sparseig.real_symm_band_solve'

    _ab = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ab'},
        ab,
        copy=False,
    )
    _mb = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'mb'},
        mb,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_ab, 'ab'), (_mb, 'mb')),
    )
    _b_utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('comm', 'icomm'),
    )
    if int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12fff_ncv_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (22,), validate=False)
    else:
        _vl_f12fff_ncv_py = 0
    _d = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_f12fff_ncv_py,),
    )
    if int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12fff_n_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (20,), validate=False)
    else:
        _vl_f12fff_n_py = 0
    if int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) >= 1 and int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False).real) <= 1000:
        _vl_f12fff_nev_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (21,), validate=False)
    else:
        _vl_f12fff_nev_py = 0
    if int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False)) >= 1 and int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False)) <= 1000:
        if _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((int(_b_utils._EngineFloat64ArrayType.get_element(comm['comm'], {'fun_name': fun_name, 'entity': 'comm[\'comm\']'}, (0,), validate=False))+7)-1,), validate=False) == 1:
            _z_dim2 = _vl_f12fff_nev_py+1
        else:
            _z_dim2 = 1
    else:
        _z_dim2 = 1
    exp_shape_z = (_vl_f12fff_n_py, _z_dim2)
    _z = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_m_sorder,
    )
    _resid = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'resid'},
        resid,
        copy=True,
    )
    _v = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_f12fff_n_py, _vl_f12fff_ncv_py),
        sorder=_m_sorder,
    )
    try:
        _return_scalar = b_real_symm_band_solve(
            kl,
            ku,
            _ab,
            _mb,
            sigma,
            _d,
            _z,
            _resid,
            _v,
            comm,
            io_manager,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _resid_py = _resid
    _nconv_py = _return_scalar
    _d_py = _d
    _z_py = _z
    _v_py = _v
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['nconv', 'd', 'z', 'resid', 'v'],
    )(
        _nconv_py,
        _d_py,
        _z_py,
        _resid_py,
        _v_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data
