"""
Deprecated wrappers for the NAG C Library (Mark 26.0) e01 Chapter.

e01 - Interpolation.

https://www.nag.com/numeric/CL/nagdoc_cl26/html/E01/e01conts.html

NAG Copyright 2017.
"""

from ctypes import c_double
from ctypes import POINTER
from .util import get_library
from .util import Integer
from .util import NagError
from .util import Nag_Spline
from .util import Nag_2dSpline

_library_cl = get_library()

e01aac_ctypes = _library_cl.e01aac
e01aac_ctypes.restype = None
e01aac_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), c_double, POINTER(NagError)]


def e01aac(n, a, b, c, x, fail):
    """
    Interpolated values, Aitken's technique, unequally spaced data, one
    variable.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e01/e01aac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, c
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, c
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('e01aac', a, 'a')
    _b = nag_double_type_check_and_cast('e01aac', b, 'b')
    _c = nag_double_type_check_and_cast('e01aac', c, 'c')
    e01aac_ctypes(n, _a, _b, _c, x, fail)

e01abc_ctypes = _library_cl.e01abc
e01abc_ctypes.restype = None
e01abc_ctypes.argtypes = [Integer, c_double, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def e01abc(n, p, a, g, fail):
    """
    Interpolated values, Everett's formula, equally spaced data, one
    variable.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e01/e01abc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, g
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, g
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('e01abc', a, 'a')
    _g = nag_double_type_check_and_cast('e01abc', g, 'g')
    e01abc_ctypes(n, p, _a, _g, fail)

e01aec_ctypes = _library_cl.e01aec
e01aec_ctypes.restype = None
e01aec_ctypes.argtypes = [Integer, c_double, c_double, POINTER(c_double), POINTER(c_double), POINTER(Integer), Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def e01aec(m, xmin, xmax, x, y, p, itmin, itmax, a, perf, numiter, fail):
    """
    Interpolating functions, polynomial interpolant, data may include
    derivative values, one variable.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e01/e01aec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, p, a, perf, numiter
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, a, perf
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        p, numiter
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('e01aec', x, 'x')
    _y = nag_double_type_check_and_cast('e01aec', y, 'y')
    _p = nag_integer_type_check_and_cast('e01aec', p, 'p')
    _a = nag_double_type_check_and_cast('e01aec', a, 'a')
    _perf = nag_double_type_check_and_cast('e01aec', perf, 'perf')
    _numiter = nag_integer_type_check_and_cast('e01aec', numiter, 'numiter')
    e01aec_ctypes(m, xmin, xmax, _x, _y, _p, itmin, itmax, _a, _perf, _numiter, fail)

e01bac_ctypes = _library_cl.e01bac
e01bac_ctypes.restype = None
e01bac_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(Nag_Spline), POINTER(NagError)]


def e01bac(m, x, y, spline, fail):
    """
    Interpolating function, cubic spline interpolant, one variable.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e01/e01bac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('e01bac', x, 'x')
    _y = nag_double_type_check_and_cast('e01bac', y, 'y')
    e01bac_ctypes(m, _x, _y, spline, fail)

e01bec_ctypes = _library_cl.e01bec
e01bec_ctypes.restype = None
e01bec_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def e01bec(n, x, f, d, fail):
    """
    Interpolating function, monotonicity-preserving, piecewise cubic
    Hermite, one variable.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e01/e01bec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, d
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f, d
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('e01bec', x, 'x')
    _f = nag_double_type_check_and_cast('e01bec', f, 'f')
    _d = nag_double_type_check_and_cast('e01bec', d, 'd')
    e01bec_ctypes(n, _x, _f, _d, fail)

e01bfc_ctypes = _library_cl.e01bfc
e01bfc_ctypes.restype = None
e01bfc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def e01bfc(n, x, f, d, m, px, pf, fail):
    """
    Evaluation of interpolant computed by e01bec, function only.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e01/e01bfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, d, px, pf
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f, d, px, pf
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('e01bfc', x, 'x')
    _f = nag_double_type_check_and_cast('e01bfc', f, 'f')
    _d = nag_double_type_check_and_cast('e01bfc', d, 'd')
    _px = nag_double_type_check_and_cast('e01bfc', px, 'px')
    _pf = nag_double_type_check_and_cast('e01bfc', pf, 'pf')
    e01bfc_ctypes(n, _x, _f, _d, m, _px, _pf, fail)

e01bgc_ctypes = _library_cl.e01bgc
e01bgc_ctypes.restype = None
e01bgc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def e01bgc(n, x, f, d, m, px, pf, pd, fail):
    """
    Evaluation of interpolant computed by e01bec, function and first
    derivative.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e01/e01bgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, d, px, pf, pd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f, d, px, pf, pd
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('e01bgc', x, 'x')
    _f = nag_double_type_check_and_cast('e01bgc', f, 'f')
    _d = nag_double_type_check_and_cast('e01bgc', d, 'd')
    _px = nag_double_type_check_and_cast('e01bgc', px, 'px')
    _pf = nag_double_type_check_and_cast('e01bgc', pf, 'pf')
    _pd = nag_double_type_check_and_cast('e01bgc', pd, 'pd')
    e01bgc_ctypes(n, _x, _f, _d, m, _px, _pf, _pd, fail)

e01bhc_ctypes = _library_cl.e01bhc
e01bhc_ctypes.restype = None
e01bhc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), c_double, c_double, POINTER(c_double), POINTER(NagError)]


def e01bhc(n, x, f, d, a, b, integral, fail):
    """
    Evaluation of interpolant computed by e01bec, definite integral.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e01/e01bhc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, d, integral
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f, d, integral
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('e01bhc', x, 'x')
    _f = nag_double_type_check_and_cast('e01bhc', f, 'f')
    _d = nag_double_type_check_and_cast('e01bhc', d, 'd')
    _integral = nag_double_type_check_and_cast('e01bhc', integral, 'integral')
    e01bhc_ctypes(n, _x, _f, _d, a, b, _integral, fail)

e01dac_ctypes = _library_cl.e01dac
e01dac_ctypes.restype = None
e01dac_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Nag_2dSpline), POINTER(NagError)]


def e01dac(mx, my, x, y, f, spline, fail):
    """
    Interpolating function, bicubic spline interpolant, two variables.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e01/e01dac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, f
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, f
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('e01dac', x, 'x')
    _y = nag_double_type_check_and_cast('e01dac', y, 'y')
    _f = nag_double_type_check_and_cast('e01dac', f, 'f')
    e01dac_ctypes(mx, my, _x, _y, _f, spline, fail)

e01eac_ctypes = _library_cl.e01eac
e01eac_ctypes.restype = None
e01eac_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def e01eac(n, x, y, triang, fail):
    """
    Triangulation of two-dimensional scattered grid, method of Renka and
    Cline.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e01/e01eac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, triang
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        triang
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('e01eac', x, 'x')
    _y = nag_double_type_check_and_cast('e01eac', y, 'y')
    _triang = nag_integer_type_check_and_cast('e01eac', triang, 'triang')
    e01eac_ctypes(n, _x, _y, _triang, fail)

nag_2d_triangulate = e01eac

e01ebc_ctypes = _library_cl.e01ebc
e01ebc_ctypes.restype = None
e01ebc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def e01ebc(m, n, x, y, f, triang, px, py, pf, fail):
    """
    Barycentric interpolation on function values provided on a two-
    dimensional scattered grid.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e01/e01ebc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, f, triang, px, py, pf
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, f, px, py, pf
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        triang
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('e01ebc', x, 'x')
    _y = nag_double_type_check_and_cast('e01ebc', y, 'y')
    _f = nag_double_type_check_and_cast('e01ebc', f, 'f')
    _px = nag_double_type_check_and_cast('e01ebc', px, 'px')
    _py = nag_double_type_check_and_cast('e01ebc', py, 'py')
    _pf = nag_double_type_check_and_cast('e01ebc', pf, 'pf')
    _triang = nag_integer_type_check_and_cast('e01ebc', triang, 'triang')
    e01ebc_ctypes(m, n, _x, _y, _f, _triang, _px, _py, _pf, fail)

nag_2d_triang_bary_eval = e01ebc

e01rac_ctypes = _library_cl.e01rac
e01rac_ctypes.restype = None
e01rac_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def e01rac(n, x, f, m, a, u, fail):
    """
    Interpolating functions, rational interpolant, one variable.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e01/e01rac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, m, a, u
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f, a, u
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('e01rac', x, 'x')
    _f = nag_double_type_check_and_cast('e01rac', f, 'f')
    _m = nag_integer_type_check_and_cast('e01rac', m, 'm')
    _a = nag_double_type_check_and_cast('e01rac', a, 'a')
    _u = nag_double_type_check_and_cast('e01rac', u, 'u')
    e01rac_ctypes(n, _x, _f, _m, _a, _u, fail)

e01rbc_ctypes = _library_cl.e01rbc
e01rbc_ctypes.restype = None
e01rbc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), c_double, POINTER(c_double), POINTER(NagError)]


def e01rbc(m, a, u, x, f, fail):
    """
    Interpolated values, evaluate rational interpolant computed by e01rac,
    one variable.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e01/e01rbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, u, f
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, u, f
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('e01rbc', a, 'a')
    _u = nag_double_type_check_and_cast('e01rbc', u, 'u')
    _f = nag_double_type_check_and_cast('e01rbc', f, 'f')
    e01rbc_ctypes(m, _a, _u, x, _f, fail)

e01sgc_ctypes = _library_cl.e01sgc
e01sgc_ctypes.restype = None
e01sgc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, Integer, POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def e01sgc(m, x, y, f, nw, nq, iq, rq, fail):
    """
    Interpolating functions, modified Shepard's method, two variables.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e01/e01sgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, f, iq, rq
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, f, rq
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iq
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('e01sgc', x, 'x')
    _y = nag_double_type_check_and_cast('e01sgc', y, 'y')
    _f = nag_double_type_check_and_cast('e01sgc', f, 'f')
    _iq = nag_integer_type_check_and_cast('e01sgc', iq, 'iq')
    _rq = nag_double_type_check_and_cast('e01sgc', rq, 'rq')
    e01sgc_ctypes(m, _x, _y, _f, nw, nq, _iq, _rq, fail)

e01shc_ctypes = _library_cl.e01shc
e01shc_ctypes.restype = None
e01shc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def e01shc(m, x, y, f, iq, rq, n, u, v, q, qx, qy, fail):
    """
    Interpolated values, evaluate interpolant computed by e01sgc, function
    and first derivatives, two variables.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e01/e01shc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, f, iq, rq, u, v, q, qx, qy
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, f, rq, u, v, q, qx, qy
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iq
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('e01shc', x, 'x')
    _y = nag_double_type_check_and_cast('e01shc', y, 'y')
    _f = nag_double_type_check_and_cast('e01shc', f, 'f')
    _iq = nag_integer_type_check_and_cast('e01shc', iq, 'iq')
    _rq = nag_double_type_check_and_cast('e01shc', rq, 'rq')
    _u = nag_double_type_check_and_cast('e01shc', u, 'u')
    _v = nag_double_type_check_and_cast('e01shc', v, 'v')
    _q = nag_double_type_check_and_cast('e01shc', q, 'q')
    _qx = nag_double_type_check_and_cast('e01shc', qx, 'qx')
    _qy = nag_double_type_check_and_cast('e01shc', qy, 'qy')
    e01shc_ctypes(m, _x, _y, _f, _iq, _rq, n, _u, _v, _q, _qx, _qy, fail)

e01sjc_ctypes = _library_cl.e01sjc
e01sjc_ctypes.restype = None
e01sjc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def e01sjc(m, x, y, f, triang, grads, fail):
    """
    A function to generate a two-dimensional surface interpolating a set of
    data points, using the method of Renka and Cline.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e01/e01sjc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, f, triang, grads
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, f, grads
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        triang
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('e01sjc', x, 'x')
    _y = nag_double_type_check_and_cast('e01sjc', y, 'y')
    _f = nag_double_type_check_and_cast('e01sjc', f, 'f')
    _triang = nag_integer_type_check_and_cast('e01sjc', triang, 'triang')
    _grads = nag_double_type_check_and_cast('e01sjc', grads, 'grads')
    e01sjc_ctypes(m, _x, _y, _f, _triang, _grads, fail)

e01skc_ctypes = _library_cl.e01skc
e01skc_ctypes.restype = None
e01skc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), c_double, c_double, POINTER(c_double), POINTER(NagError)]


def e01skc(m, x, y, f, triang, grads, px, py, pf, fail):
    """
    A function to evaluate, at a set of points, the two-dimensional
    interpolant function generated by e01sjc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e01/e01skc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, f, triang, grads, pf
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, f, grads, pf
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        triang
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('e01skc', x, 'x')
    _y = nag_double_type_check_and_cast('e01skc', y, 'y')
    _f = nag_double_type_check_and_cast('e01skc', f, 'f')
    _triang = nag_integer_type_check_and_cast('e01skc', triang, 'triang')
    _grads = nag_double_type_check_and_cast('e01skc', grads, 'grads')
    _pf = nag_double_type_check_and_cast('e01skc', pf, 'pf')
    e01skc_ctypes(m, _x, _y, _f, _triang, _grads, px, py, _pf, fail)

e01tgc_ctypes = _library_cl.e01tgc
e01tgc_ctypes.restype = None
e01tgc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, Integer, POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def e01tgc(m, x, y, z, f, nw, nq, iq, rq, fail):
    """
    Interpolating functions, modified Shepard's method, three variables.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e01/e01tgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, z, f, iq, rq
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, z, f, rq
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iq
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('e01tgc', x, 'x')
    _y = nag_double_type_check_and_cast('e01tgc', y, 'y')
    _z = nag_double_type_check_and_cast('e01tgc', z, 'z')
    _f = nag_double_type_check_and_cast('e01tgc', f, 'f')
    _iq = nag_integer_type_check_and_cast('e01tgc', iq, 'iq')
    _rq = nag_double_type_check_and_cast('e01tgc', rq, 'rq')
    e01tgc_ctypes(m, _x, _y, _z, _f, nw, nq, _iq, _rq, fail)

e01thc_ctypes = _library_cl.e01thc
e01thc_ctypes.restype = None
e01thc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def e01thc(m, x, y, z, f, iq, rq, n, u, v, w, q, qx, qy, qz, fail):
    """
    Interpolated values, evaluate interpolant computed by e01tgc, function
    and first derivatives, three variables.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e01/e01thc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, z, f, iq, rq, u, v, w, q, qx, qy, qz
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, z, f, rq, u, v, w, q, qx, qy, qz
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iq
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('e01thc', x, 'x')
    _y = nag_double_type_check_and_cast('e01thc', y, 'y')
    _z = nag_double_type_check_and_cast('e01thc', z, 'z')
    _f = nag_double_type_check_and_cast('e01thc', f, 'f')
    _iq = nag_integer_type_check_and_cast('e01thc', iq, 'iq')
    _rq = nag_double_type_check_and_cast('e01thc', rq, 'rq')
    _u = nag_double_type_check_and_cast('e01thc', u, 'u')
    _v = nag_double_type_check_and_cast('e01thc', v, 'v')
    _w = nag_double_type_check_and_cast('e01thc', w, 'w')
    _q = nag_double_type_check_and_cast('e01thc', q, 'q')
    _qx = nag_double_type_check_and_cast('e01thc', qx, 'qx')
    _qy = nag_double_type_check_and_cast('e01thc', qy, 'qy')
    _qz = nag_double_type_check_and_cast('e01thc', qz, 'qz')
    e01thc_ctypes(m, _x, _y, _z, _f, _iq, _rq, n, _u, _v, _w, _q, _qx, _qy, _qz, fail)

e01tkc_ctypes = _library_cl.e01tkc
e01tkc_ctypes.restype = None
e01tkc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), Integer, Integer, POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def e01tkc(m, x, f, nw, nq, iq, rq, fail):
    """
    Interpolating functions, modified Shepard's method, four variables.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e01/e01tkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, iq, rq
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f, rq
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iq
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('e01tkc', x, 'x')
    _f = nag_double_type_check_and_cast('e01tkc', f, 'f')
    _iq = nag_integer_type_check_and_cast('e01tkc', iq, 'iq')
    _rq = nag_double_type_check_and_cast('e01tkc', rq, 'rq')
    e01tkc_ctypes(m, _x, _f, nw, nq, _iq, _rq, fail)

e01tlc_ctypes = _library_cl.e01tlc
e01tlc_ctypes.restype = None
e01tlc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def e01tlc(m, x, f, iq, rq, n, xe, q, qx, fail):
    """
    Interpolated values, evaluate interpolant computed by e01tkc, function
    and first derivatives, four variables.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e01/e01tlc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, iq, rq, xe, q, qx
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f, rq, xe, q, qx
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iq
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('e01tlc', x, 'x')
    _f = nag_double_type_check_and_cast('e01tlc', f, 'f')
    _iq = nag_integer_type_check_and_cast('e01tlc', iq, 'iq')
    _rq = nag_double_type_check_and_cast('e01tlc', rq, 'rq')
    _xe = nag_double_type_check_and_cast('e01tlc', xe, 'xe')
    _q = nag_double_type_check_and_cast('e01tlc', q, 'q')
    _qx = nag_double_type_check_and_cast('e01tlc', qx, 'qx')
    e01tlc_ctypes(m, _x, _f, _iq, _rq, n, _xe, _q, _qx, fail)

e01tmc_ctypes = _library_cl.e01tmc
e01tmc_ctypes.restype = None
e01tmc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), Integer, Integer, POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def e01tmc(m, x, f, nw, nq, iq, rq, fail):
    """
    Interpolating functions, modified Shepard's method, five variables.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e01/e01tmc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, iq, rq
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f, rq
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iq
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('e01tmc', x, 'x')
    _f = nag_double_type_check_and_cast('e01tmc', f, 'f')
    _iq = nag_integer_type_check_and_cast('e01tmc', iq, 'iq')
    _rq = nag_double_type_check_and_cast('e01tmc', rq, 'rq')
    e01tmc_ctypes(m, _x, _f, nw, nq, _iq, _rq, fail)

e01tnc_ctypes = _library_cl.e01tnc
e01tnc_ctypes.restype = None
e01tnc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def e01tnc(m, x, f, iq, rq, n, xe, q, qx, fail):
    """
    Interpolated values, evaluate interpolant computed by e01tmc, function
    and first derivatives, five variables.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e01/e01tnc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, iq, rq, xe, q, qx
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f, rq, xe, q, qx
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iq
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('e01tnc', x, 'x')
    _f = nag_double_type_check_and_cast('e01tnc', f, 'f')
    _iq = nag_integer_type_check_and_cast('e01tnc', iq, 'iq')
    _rq = nag_double_type_check_and_cast('e01tnc', rq, 'rq')
    _xe = nag_double_type_check_and_cast('e01tnc', xe, 'xe')
    _q = nag_double_type_check_and_cast('e01tnc', q, 'q')
    _qx = nag_double_type_check_and_cast('e01tnc', qx, 'qx')
    e01tnc_ctypes(m, _x, _f, _iq, _rq, n, _xe, _q, _qx, fail)

e01zmc_ctypes = _library_cl.e01zmc
e01zmc_ctypes.restype = None
e01zmc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(c_double), Integer, Integer, POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def e01zmc(d, m, x, f, nw, nq, iq, rq, fail):
    """
    Interpolating function, modified Shepard's method, d dimensions.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e01/e01zmc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, iq, rq
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f, rq
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iq
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('e01zmc', x, 'x')
    _f = nag_double_type_check_and_cast('e01zmc', f, 'f')
    _iq = nag_integer_type_check_and_cast('e01zmc', iq, 'iq')
    _rq = nag_double_type_check_and_cast('e01zmc', rq, 'rq')
    e01zmc_ctypes(d, m, _x, _f, nw, nq, _iq, _rq, fail)

e01znc_ctypes = _library_cl.e01znc
e01znc_ctypes.restype = None
e01znc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def e01znc(d, m, x, f, iq, rq, n, xe, q, qx, fail):
    """
    Interpolated values, evaluate interpolant computed by e01zmc, function
    and first derivatives, d dimensions.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e01/e01znc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, f, iq, rq, xe, q, qx
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, f, rq, xe, q, qx
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iq
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('e01znc', x, 'x')
    _f = nag_double_type_check_and_cast('e01znc', f, 'f')
    _iq = nag_integer_type_check_and_cast('e01znc', iq, 'iq')
    _rq = nag_double_type_check_and_cast('e01znc', rq, 'rq')
    _xe = nag_double_type_check_and_cast('e01znc', xe, 'xe')
    _q = nag_double_type_check_and_cast('e01znc', q, 'q')
    _qx = nag_double_type_check_and_cast('e01znc', qx, 'qx')
    e01znc_ctypes(d, m, _x, _f, _iq, _rq, n, _xe, _q, _qx, fail)

nag_1d_aitken_interp = e01aac
nag_1d_everett_interp = e01abc
nag_1d_cheb_interp = e01aec
nag_1d_spline_interpolant = e01bac
nag_monotonic_interpolant = e01bec
nag_monotonic_evaluate = e01bfc
nag_monotonic_deriv = e01bgc
nag_monotonic_intg = e01bhc
nag_2d_spline_interpolant = e01dac
nag_1d_ratnl_interp = e01rac
nag_1d_ratnl_eval = e01rbc
nag_2d_shep_interp = e01sgc
nag_2d_shep_eval = e01shc
nag_2d_triang_interp = e01sjc
nag_2d_triang_eval = e01skc
nag_3d_shep_interp = e01tgc
nag_3d_shep_eval = e01thc
nag_4d_shep_interp = e01tkc
nag_4d_shep_eval = e01tlc
nag_5d_shep_interp = e01tmc
nag_5d_shep_eval = e01tnc
nag_nd_shep_interp = e01zmc
nag_nd_shep_eval = e01znc
__all__ = ['e01aac', 'e01abc', 'e01aec', 'e01bac', 'e01bec', 'e01bfc',
           'e01bgc', 'e01bhc', 'e01dac', 'e01rac',
           'e01rbc', 'e01sgc', 'e01shc', 'e01sjc', 'e01skc', 'e01tgc',
           'e01thc', 'e01tkc', 'e01tlc', 'e01tmc', 'e01tnc', 'e01zmc',
           'e01znc', 'nag_1d_aitken_interp', 'nag_1d_cheb_interp',
           'nag_1d_everett_interp', 'nag_1d_ratnl_eval',
           'nag_1d_ratnl_interp', 'nag_1d_spline_interpolant',
           'nag_2d_shep_eval', 'nag_2d_shep_interp',
           'nag_2d_spline_interpolant', 'nag_2d_triang_eval',
           'nag_2d_triang_interp',
           'nag_3d_shep_eval',
           'nag_3d_shep_interp', 'nag_4d_shep_eval',
           'nag_4d_shep_interp', 'nag_5d_shep_eval',
           'nag_5d_shep_interp', 'nag_monotonic_deriv',
           'nag_monotonic_evaluate', 'nag_monotonic_interpolant',
           'nag_monotonic_intg', 'nag_nd_shep_eval',
           'nag_nd_shep_interp']
