"""
Deprecated wrappers for the NAG C Library (Mark 26.0) e05 Chapter.

e05 - Global Optimization of a Function.

https://www.nag.com/numeric/CL/nagdoc_cl26/html/E05/e05conts.html

NAG Copyright 2017.
"""

from ctypes import c_double
from ctypes import POINTER
from ctypes import c_ulong
from ctypes import c_char_p
from .util import get_library
from .util import Integer
from .util import Nag_E05State
from .util import NagError
from .util import NAG_E05JBC_OBJFUN
from .util import Nag_BoundType
from .util import Nag_MCSInitMethod
from .util import NAG_E05JBC_MON
from .util import Nag_Comm
from .util import Nag_FileID
from .util import NAG_E05SAC_OBJFUN
from .util import NAG_E05SAC_MONMOD
from .util import NAG_E05SBC_OBJFUN
from .util import NAG_E05SBC_CONFUN
from .util import NAG_E05SBC_MONMOD
from .util import NAG_E05UCC_CONFUN
from .util import NAG_E05UCC_OBJFUN
from .util import NAG_E05UCC_START
from .util import Nag_Boolean
from .util import NAG_E05USC_CONFUN
from .util import NAG_E05USC_OBJFUN
from .util import NAG_E05USC_START
from .util import Nag_VariableType


_library_cl = get_library()

STRING = c_char_p

size_t = c_ulong

e05jac_ctypes = _library_cl.e05jac
e05jac_ctypes.restype = None
e05jac_ctypes.argtypes = [Integer, POINTER(Nag_E05State), POINTER(NagError)]


def e05jac(nr, state, fail):
    """
    Initialization function for e05jbc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e05/e05jac.html
    """
    e05jac_ctypes(nr, state, fail)

e05jbc_ctypes = _library_cl.e05jbc
e05jbc_ctypes.restype = None
e05jbc_ctypes.argtypes = [Integer, NAG_E05JBC_OBJFUN, Nag_BoundType, Nag_MCSInitMethod, POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(Integer), POINTER(Integer), NAG_E05JBC_MON, POINTER(c_double), POINTER(c_double), POINTER(Nag_E05State), POINTER(Nag_Comm), POINTER(NagError)]


def e05jbc(n, objfun, bound, initmethod, bl, bu, sdlist, _list, numpts, initpt, monit, x, obj, state, comm, fail):
    """
    Global optimization by multi-level coordinate search, simple bounds,
    using function values only.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e05/e05jbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        bl, bu, _list, numpts, initpt, x, obj
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        bl, bu, _list, x, obj
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        numpts, initpt
    """
    from ctypes import cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _bl = nag_double_type_check_and_cast('e05jbc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('e05jbc', bu, 'bu')
    __list = nag_double_type_check_and_cast('e05jbc', _list, '_list')
    _numpts = nag_integer_type_check_and_cast('e05jbc', numpts, 'numpts')
    _initpt = nag_integer_type_check_and_cast('e05jbc', initpt, 'initpt')
    if monit is None:
        monit = cast(None, NAG_E05JBC_MON)
    _x = nag_double_type_check_and_cast('e05jbc', x, 'x')
    _obj = nag_double_type_check_and_cast('e05jbc', obj, 'obj')
    e05jbc_ctypes(n, objfun, bound, initmethod, _bl, _bu, sdlist, __list, _numpts, _initpt, monit, _x, _obj, state, comm, fail)

e05jcc_ctypes = _library_cl.e05jcc
e05jcc_ctypes.restype = None
e05jcc_ctypes.argtypes = [Nag_FileID, POINTER(Nag_E05State), POINTER(NagError)]


def e05jcc(fileid, state, fail):
    """
    Supply optional argument values for e05jbc from external file.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e05/e05jcc.html
    """
    e05jcc_ctypes(fileid, state, fail)

e05jdc_ctypes = _library_cl.e05jdc
e05jdc_ctypes.restype = None
e05jdc_ctypes.argtypes = [STRING, POINTER(Nag_E05State), POINTER(NagError)]


def e05jdc(optstr, state, fail):
    """
    Set a single optional argument for e05jbc from a character string.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e05/e05jdc.html
    """
    e05jdc_ctypes(optstr, state, fail)

e05jec_ctypes = _library_cl.e05jec
e05jec_ctypes.restype = None
e05jec_ctypes.argtypes = [STRING, STRING, POINTER(Nag_E05State), POINTER(NagError)]


def e05jec(optstr, cvalue, state, fail):
    """
    Set a single optional argument for e05jbc from an 'ON'/'OFF'-valued
    character argument.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e05/e05jec.html
    """
    e05jec_ctypes(optstr, cvalue, state, fail)

e05jfc_ctypes = _library_cl.e05jfc
e05jfc_ctypes.restype = None
e05jfc_ctypes.argtypes = [STRING, Integer, POINTER(Nag_E05State), POINTER(NagError)]


def e05jfc(optstr, ivalue, state, fail):
    """
    Set a single optional argument for e05jbc from an integer argument.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e05/e05jfc.html
    """
    e05jfc_ctypes(optstr, ivalue, state, fail)

e05jgc_ctypes = _library_cl.e05jgc
e05jgc_ctypes.restype = None
e05jgc_ctypes.argtypes = [STRING, c_double, POINTER(Nag_E05State), POINTER(NagError)]


def e05jgc(optstr, rvalue, state, fail):
    """
    Set a single optional argument for e05jbc from a real argument.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e05/e05jgc.html
    """
    e05jgc_ctypes(optstr, rvalue, state, fail)

e05jhc_ctypes = _library_cl.e05jhc
e05jhc_ctypes.restype = Integer
e05jhc_ctypes.argtypes = [STRING, POINTER(Nag_E05State), POINTER(NagError)]


def e05jhc(optstr, state, fail):
    """
    Determine whether an optional argument for e05jbc has been set by you or
    not.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e05/e05jhc.html
    """
    return_value = e05jhc_ctypes(optstr, state, fail)
    return return_value

e05jkc_ctypes = _library_cl.e05jkc
e05jkc_ctypes.restype = None
e05jkc_ctypes.argtypes = [STRING, POINTER(Integer), POINTER(Nag_E05State), POINTER(NagError)]


def e05jkc(optstr, ivalue, state, fail):
    """
    Get the setting of an integer valued optional argument of e05jbc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e05/e05jkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ivalue
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalue
    """
    from .util import nag_integer_type_check_and_cast
    _ivalue = nag_integer_type_check_and_cast('e05jkc', ivalue, 'ivalue')
    e05jkc_ctypes(optstr, _ivalue, state, fail)

e05jlc_ctypes = _library_cl.e05jlc
e05jlc_ctypes.restype = None
e05jlc_ctypes.argtypes = [STRING, POINTER(c_double), POINTER(Nag_E05State), POINTER(NagError)]


def e05jlc(optstr, rvalue, state, fail):
    """
    Get the setting of a real valued optional argument of e05jbc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e05/e05jlc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        rvalue
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rvalue
    """
    from .util import nag_double_type_check_and_cast
    _rvalue = nag_double_type_check_and_cast('e05jlc', rvalue, 'rvalue')
    e05jlc_ctypes(optstr, _rvalue, state, fail)

e05sac_ctypes = _library_cl.e05sac
e05sac_ctypes.restype = None
e05sac_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), NAG_E05SAC_OBJFUN, NAG_E05SAC_MONMOD, POINTER(Integer), POINTER(c_double), POINTER(Nag_Comm), POINTER(Integer), POINTER(Integer), POINTER(NagError)]


def e05sac(ndim, npar, xb, fb, bl, bu, objfun, monmod, iopts, opts, comm, itt, inform, fail):
    """
    Global optimization using particle swarm algorithm (PSO), bound
    constraints only.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e05/e05sac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        xb, fb, bl, bu, iopts, opts, itt, inform
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        xb, fb, bl, bu, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iopts, itt, inform
    """
    from ctypes import cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _xb = nag_double_type_check_and_cast('e05sac', xb, 'xb')
    _fb = nag_double_type_check_and_cast('e05sac', fb, 'fb')
    _bl = nag_double_type_check_and_cast('e05sac', bl, 'bl')
    _bu = nag_double_type_check_and_cast('e05sac', bu, 'bu')
    if monmod is None:
        monmod = cast(None, NAG_E05SAC_MONMOD)
    _iopts = nag_integer_type_check_and_cast('e05sac', iopts, 'iopts')
    _opts = nag_double_type_check_and_cast('e05sac', opts, 'opts')
    _itt = nag_integer_type_check_and_cast('e05sac', itt, 'itt')
    _inform = nag_integer_type_check_and_cast('e05sac', inform, 'inform')
    e05sac_ctypes(ndim, npar, _xb, _fb, _bl, _bu, objfun, monmod, _iopts, _opts, comm, _itt, _inform, fail)

e05sbc_ctypes = _library_cl.e05sbc
e05sbc_ctypes.restype = None
e05sbc_ctypes.argtypes = [Integer, Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), NAG_E05SBC_OBJFUN, NAG_E05SBC_CONFUN, NAG_E05SBC_MONMOD, POINTER(Integer), POINTER(c_double), POINTER(Nag_Comm), POINTER(Integer), POINTER(Integer), POINTER(NagError)]


def e05sbc(ndim, ncon, npar, xb, fb, cb, bl, bu, xbest, fbest, cbest, objfun, confun, monmod, iopts, opts, comm, itt, inform, fail):
    """
    Global optimization using particle swarm algorithm (PSO), comprehensive.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e05/e05sbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        xb, fb, cb, bl, bu, xbest, fbest, cbest, iopts, opts, itt, inform
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        xb, fb, cb, bl, bu, xbest, fbest, cbest, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iopts, itt, inform
    """
    from ctypes import cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _xb = nag_double_type_check_and_cast('e05sbc', xb, 'xb')
    _fb = nag_double_type_check_and_cast('e05sbc', fb, 'fb')
    _cb = nag_double_type_check_and_cast('e05sbc', cb, 'cb')
    _bl = nag_double_type_check_and_cast('e05sbc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('e05sbc', bu, 'bu')
    _xbest = nag_double_type_check_and_cast('e05sbc', xbest, 'xbest')
    _fbest = nag_double_type_check_and_cast('e05sbc', fbest, 'fbest')
    _cbest = nag_double_type_check_and_cast('e05sbc', cbest, 'cbest')
    if confun is None:
        confun = cast(None, NAG_E05SBC_CONFUN)
    if monmod is None:
        monmod = cast(None, NAG_E05SBC_MONMOD)
    _iopts = nag_integer_type_check_and_cast('e05sbc', iopts, 'iopts')
    _opts = nag_double_type_check_and_cast('e05sbc', opts, 'opts')
    _itt = nag_integer_type_check_and_cast('e05sbc', itt, 'itt')
    _inform = nag_integer_type_check_and_cast('e05sbc', inform, 'inform')
    e05sbc_ctypes(ndim, ncon, npar, _xb, _fb, _cb, _bl, _bu, _xbest, _fbest, _cbest, objfun, confun, monmod, _iopts, _opts, comm, _itt, _inform, fail)

e05ucc_ctypes = _library_cl.e05ucc
e05ucc_ctypes.restype = None
e05ucc_ctypes.argtypes = [Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), NAG_E05UCC_CONFUN, NAG_E05UCC_OBJFUN, Integer, POINTER(c_double), Integer, NAG_E05UCC_START, Nag_Boolean, Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), Integer, Integer, POINTER(c_double), Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), Integer, POINTER(Integer), POINTER(c_double), POINTER(Nag_Comm), POINTER(Integer), POINTER(NagError)]


def e05ucc(n, nclin, ncnln, a, tda, bl, bu, confun, objfun, npts, x, ldx, start, repeat, nb, objf, objgrd, ldobjgrd, _iter, c, ldc, cjac, ldcjac, sdcjac, r, ldr, sdr, clamda, ldclamda, istate, ldistate, iopts, opts, comm, info, fail):
    """
    Global optimization using multi-start, nonlinear constraints.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e05/e05ucc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, bl, bu, x, objf, objgrd, _iter, c, cjac, r, clamda, istate, iopts, opts, info
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, bl, bu, x, objf, objgrd, c, cjac, r, clamda, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        _iter, istate, iopts, info
    """
    from ctypes import cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('e05ucc', a, 'a')
    _bl = nag_double_type_check_and_cast('e05ucc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('e05ucc', bu, 'bu')
    if confun is None:
        confun = cast(None, NAG_E05UCC_CONFUN)
    _x = nag_double_type_check_and_cast('e05ucc', x, 'x')
    if start is None:
        start = cast(None, NAG_E05UCC_START)
    _objf = nag_double_type_check_and_cast('e05ucc', objf, 'objf')
    _objgrd = nag_double_type_check_and_cast('e05ucc', objgrd, 'objgrd')
    __iter = nag_integer_type_check_and_cast('e05ucc', _iter, '_iter')
    _c = nag_double_type_check_and_cast('e05ucc', c, 'c')
    _cjac = nag_double_type_check_and_cast('e05ucc', cjac, 'cjac')
    _r = nag_double_type_check_and_cast('e05ucc', r, 'r')
    _clamda = nag_double_type_check_and_cast('e05ucc', clamda, 'clamda')
    _istate = nag_integer_type_check_and_cast('e05ucc', istate, 'istate')
    _iopts = nag_integer_type_check_and_cast('e05ucc', iopts, 'iopts')
    _opts = nag_double_type_check_and_cast('e05ucc', opts, 'opts')
    _info = nag_integer_type_check_and_cast('e05ucc', info, 'info')
    e05ucc_ctypes(n, nclin, ncnln, _a, tda, _bl, _bu, confun, objfun, npts, _x, ldx, start, repeat, nb, _objf, _objgrd, ldobjgrd, __iter, _c, ldc, _cjac, ldcjac, sdcjac, _r, ldr, sdr, _clamda, ldclamda, _istate, ldistate, _iopts, _opts, comm, _info, fail)

e05usc_ctypes = _library_cl.e05usc
e05usc_ctypes.restype = None
e05usc_ctypes.argtypes = [Integer, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), NAG_E05USC_CONFUN, NAG_E05USC_OBJFUN, Integer, POINTER(c_double), Integer, NAG_E05USC_START, Nag_Boolean, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, Integer, POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), Integer, POINTER(Integer), POINTER(c_double), POINTER(Nag_Comm), POINTER(Integer), POINTER(NagError)]


def e05usc(m, n, nclin, ncnln, a, tda, bl, bu, y, confun, objfun, npts, x, ldx, start, repeat, nb, objf, f, fjac, tdfjac, sdfjac, _iter, c, ldc, cjac, ldcjac, sdcjac, clamda, ldclamda, istate, ldistate, iopts, opts, comm, info, fail):
    """
    Global optimization of a sum of squares problem using multi-start,
    nonlinear constraints.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e05/e05usc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, bl, bu, y, x, objf, f, fjac, _iter, c, cjac, clamda, istate, iopts, opts, info
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, bl, bu, y, x, objf, f, fjac, c, cjac, clamda, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        _iter, istate, iopts, info
    """
    from ctypes import cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('e05usc', a, 'a')
    _bl = nag_double_type_check_and_cast('e05usc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('e05usc', bu, 'bu')
    _y = nag_double_type_check_and_cast('e05usc', y, 'y')
    if confun is None:
        confun = cast(None, NAG_E05USC_CONFUN)
    if start is None:
        start = cast(None, NAG_E05USC_START)
    _x = nag_double_type_check_and_cast('e05usc', x, 'x')
    _objf = nag_double_type_check_and_cast('e05usc', objf, 'objf')
    _f = nag_double_type_check_and_cast('e05usc', f, 'f')
    _fjac = nag_double_type_check_and_cast('e05usc', fjac, 'fjac')
    __iter = nag_integer_type_check_and_cast('e05usc', _iter, '_iter')
    _c = nag_double_type_check_and_cast('e05usc', c, 'c')
    _cjac = nag_double_type_check_and_cast('e05usc', cjac, 'cjac')
    _clamda = nag_double_type_check_and_cast('e05usc', clamda, 'clamda')
    _istate = nag_integer_type_check_and_cast('e05usc', istate, 'istate')
    _iopts = nag_integer_type_check_and_cast('e05usc', iopts, 'iopts')
    _opts = nag_double_type_check_and_cast('e05usc', opts, 'opts')
    _info = nag_integer_type_check_and_cast('e05usc', info, 'info')
    e05usc_ctypes(m, n, nclin, ncnln, _a, tda, _bl, _bu, _y, confun, objfun, npts, _x, ldx, start, repeat, nb, _objf, _f, _fjac, tdfjac, sdfjac, __iter, _c, ldc, _cjac, ldcjac, sdcjac, _clamda, ldclamda, _istate, ldistate, _iopts, _opts, comm, _info, fail)

e05zkc_ctypes = _library_cl.e05zkc
e05zkc_ctypes.restype = None
e05zkc_ctypes.argtypes = [STRING, POINTER(Integer), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def e05zkc(optstr, iopts, liopts, opts, lopts, fail):
    """
    Option setting routine for e05sac, e05sbc, e05ucc and e05usc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e05/e05zkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        iopts, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        opts
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iopts
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _iopts = nag_integer_type_check_and_cast('e05zkc', iopts, 'iopts')
    _opts = nag_double_type_check_and_cast('e05zkc', opts, 'opts')
    e05zkc_ctypes(optstr, _iopts, liopts, _opts, lopts, fail)

e05zlc_ctypes = _library_cl.e05zlc
e05zlc_ctypes.restype = None
e05zlc_ctypes.argtypes = [STRING, POINTER(Integer), POINTER(c_double), STRING, Integer, POINTER(Nag_VariableType), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def e05zlc(optstr, ivalue, rvalue, cvalue, lcvalue, optype, iopts, opts, fail):
    """
    Option getting routine for e05sac, e05sbc, e05ucc and e05usc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/e05/e05zlc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ivalue, rvalue, iopts, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rvalue, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalue, iopts
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _ivalue = nag_integer_type_check_and_cast('e05zlc', ivalue, 'ivalue')
    _rvalue = nag_double_type_check_and_cast('e05zlc', rvalue, 'rvalue')
    _iopts = nag_integer_type_check_and_cast('e05zlc', iopts, 'iopts')
    _opts = nag_double_type_check_and_cast('e05zlc', opts, 'opts')
    e05zlc_ctypes(optstr, _ivalue, _rvalue, cvalue, lcvalue, optype, _iopts, _opts, fail)

nag_glopt_bnd_mcs_init = e05jac
nag_glopt_bnd_mcs_solve = e05jbc
nag_glopt_bnd_mcs_optset_file = e05jcc
nag_glopt_bnd_mcs_optset_string = e05jdc
nag_glopt_bnd_mcs_optset_char = e05jec
nag_glopt_bnd_mcs_optset_int = e05jfc
nag_glopt_bnd_mcs_optset_real = e05jgc
nag_glopt_bnd_mcs_option_check = e05jhc
nag_glopt_bnd_mcs_optget_int = e05jkc
nag_glopt_bnd_mcs_optget_real = e05jlc
nag_glopt_bnd_pso = e05sac
nag_glopt_nlp_pso = e05sbc
nag_glopt_nlp_multistart_sqp = e05ucc
nag_glopt_nlp_multistart_sqp_lsq = e05usc
nag_glopt_opt_set = e05zkc
nag_glopt_opt_get = e05zlc
__all__ = ['e05jac', 'e05jbc', 'e05jcc', 'e05jdc', 'e05jec', 'e05jfc',
           'e05jgc', 'e05jhc', 'e05jkc', 'e05jlc', 'e05sac', 'e05sbc',
           'e05ucc', 'e05usc', 'e05zkc', 'e05zlc',
           'nag_glopt_bnd_mcs_init', 'nag_glopt_bnd_mcs_optget_int',
           'nag_glopt_bnd_mcs_optget_real',
           'nag_glopt_bnd_mcs_option_check',
           'nag_glopt_bnd_mcs_optset_char',
           'nag_glopt_bnd_mcs_optset_file',
           'nag_glopt_bnd_mcs_optset_int',
           'nag_glopt_bnd_mcs_optset_real',
           'nag_glopt_bnd_mcs_optset_string',
           'nag_glopt_bnd_mcs_solve', 'nag_glopt_bnd_pso',
           'nag_glopt_nlp_multistart_sqp',
           'nag_glopt_nlp_multistart_sqp_lsq', 'nag_glopt_nlp_pso',
           'nag_glopt_opt_get', 'nag_glopt_opt_set']
