#!/usr/bin/env python
"""
Integration tests for wrapper functionality in the blgm submodule.

Public use of this module is not supported by NAG.
"""

# NAG Copyright 2018.

# pylint: disable=invalid-name

import unittest

import numpy as np

from naginterfaces.base import utils as b_utils
from test_functionality_core import IntegrationTest

class TestG22y(IntegrationTest):
    """
    Abstract suite of integration tests for the g22y wrappers.
    """

    # pylint: disable=too-few-public-methods

    # Obs/Var, C-ordered dat matrix:
    dat_ovc = [
        [3, 1, -2.4],
        [3, 3, 0.2],
        [1, 3, -1.4],
        [2, 1, -5.4],
        [3, 3, 0.2],
        [3, 2, 1.4],
        [1, 2, 6.8],
        [1, 2, 6.7],
        [1, 1, 5.3],
        [2, 3, -1.3],
        [3, 2, -3.6],
        [3, 2, -0.7],
        [1, 1, 5.7],
        [3, 3, 2.3],
        [1, 2, 3.3],
        [2, 3, -0.5],
        [1, 1, -2.6],
        [1, 2, 3.7],
        [1, 2, 0.9],
        [3, 1, -1.1],
        [2, 2, 2.1],
        [1, 3, 4.6],
        [2, 3, 4.6],
        [1, 2, 5.1],
        [1, 3, 0.9],
    ]

    def set_up_good_call(self):
        """
        Some initialized problem structure.
        """
        # pylint: disable=attribute-defined-outside-init
        from naginterfaces.library import blgm
        self.optset = blgm.optset
        self.hform = b_utils.Handle()
        blgm.lm_formula(self.hform, '(F2 + Con + F1)^2')
        self.optset(self.hform, 'Explicit Mean = Yes')
        self.hddesc = b_utils.Handle()
        self.nobs = len(self.dat_ovc)
        levels = [3, 3, 1]
        vnames = ['F1', 'F2', 'Con']
        blgm.lm_describe_data(self.hddesc, self.nobs, levels, vnames=vnames)
        self.hxdesc = b_utils.Handle()
        self.empty_x = np.empty((0, 0))
        self.xsize = [-42, -42]

class TestG22ybBase(TestG22y, unittest.TestCase):
    """
    Concrete base tests.
    """

    def setUp(self):
        from naginterfaces.base import blgm as chapter
        TestG22y.set_up(self)
        self.fun = chapter.lm_describe_data

    def test_no_vnames(self):
        """
        Using default vnames (None).
        """
        hddesc = b_utils.Handle()
        self.fun(hddesc, 25, 3, [3, 3, 1], 0, None)

    def test_bad_vnames(self):
        """
        Non-None vnames required.
        """
        hddesc = b_utils.Handle()
        self.my_assert_raises_regex(
            b_utils.NagTypeError,
            'must not be None',
            self.fun,
            hddesc, 25, 3, [3, 3, 1], 1, None,
        )

class TestG22ybNp(TestG22y, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import blgm as chapter
        TestG22y.set_up(self)
        self.fun = chapter.lm_describe_data

    def test_no_vnames(self):
        """
        Using default vnames.
        """
        hddesc = b_utils.Handle()
        self.fun(hddesc, 25, [3, 3, 1])

class TestG22ycBase(TestG22y, unittest.TestCase):
    """
    Concrete base tests.
    """

    def setUp(self):
        from naginterfaces.base import blgm as chapter
        TestG22y.set_up(self)
        self.fun = chapter.lm_design_matrix

    def test_good_xsize_ovc(self):
        """
        Correct returned xsize, Obs/Var + C-ordered.
        """
        import warnings

        self.set_up_good_call()
        self.optset(self.hform, 'Storage Order = ObsVar')
        warnings.simplefilter('error', b_utils.NagAlgorithmicWarning)
        self.fun(
            self.hform, self.hddesc, np.array(self.dat_ovc), self.hxdesc,
            self.empty_x, self.xsize,
        )
        warnings.simplefilter('always', b_utils.NagAlgorithmicWarning)
        self.assertEqual(
            self.xsize,
            [self.nobs, 14],
        )

    def test_good_xsize_ovf(self):
        """
        Correct returned xsize, Obs/Var + F-ordered.
        """
        import warnings

        self.set_up_good_call()
        self.optset(self.hform, 'Storage Order = ObsVar')
        warnings.simplefilter('error', b_utils.NagAlgorithmicWarning)
        self.fun(
            self.hform, self.hddesc, np.array(self.dat_ovc, order='F'),
            self.hxdesc,
            self.empty_x, self.xsize,
        )
        warnings.simplefilter('always', b_utils.NagAlgorithmicWarning)
        self.assertEqual(
            self.xsize,
            [self.nobs, 14],
        )

    def test_good_xsize_voc(self):
        """
        Correct returned xsize, Var/Obs + C-ordered.
        """
        import warnings

        self.set_up_good_call()
        self.optset(self.hddesc, 'Storage Order = VarObs')
        warnings.simplefilter('error', b_utils.NagAlgorithmicWarning)
        dat_voc = np.transpose(self.dat_ovc)
        self.fun(
            self.hform, self.hddesc, dat_voc, self.hxdesc,
            self.empty_x, self.xsize,
        )
        warnings.simplefilter('always', b_utils.NagAlgorithmicWarning)
        self.assertEqual(
            self.xsize,
            [self.nobs, 14],
        )

    def test_good_xsize_vof(self):
        """
        Correct returned xsize, Var/Obs + F-ordered.
        """
        import warnings

        self.set_up_good_call()
        self.optset(self.hddesc, 'Storage Order = VarObs')
        warnings.simplefilter('error', b_utils.NagAlgorithmicWarning)
        dat_vof = np.array(np.transpose(self.dat_ovc), order='F')
        self.fun(
            self.hform, self.hddesc, dat_vof, self.hxdesc,
            self.empty_x, self.xsize,
        )
        warnings.simplefilter('always', b_utils.NagAlgorithmicWarning)
        self.assertEqual(
            self.xsize,
            [self.nobs, 14],
        )

class TestG22ycNp(TestG22y, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import blgm as chapter
        TestG22y.set_up(self)
        self.fun = chapter.lm_design_matrix

    def test_bad_hform(self):
        """
        hform not initialized.
        """
        hform = b_utils.Handle()
        hddesc = b_utils.Handle()
        hxdesc = b_utils.Handle()
        self.my_assert_raises_regex(
            b_utils.NagValueError,
            "hform.*corrupt",
            self.fun,
            hform, hddesc, [[], []], hxdesc,
        )

    def test_warn_as_exc(self):
        """
        Query mode works correctly when warnings promoted to exceptions.
        """
        import warnings

        warnings.simplefilter('error', b_utils.NagAlgorithmicWarning)
        self.set_up_good_call()
        self.fun(self.hform, self.hddesc, self.dat_ovc, self.hxdesc)
        warnings.simplefilter('always', b_utils.NagAlgorithmicWarning)

class TestG22ydNp(TestG22y, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import blgm as chapter
        TestG22y.set_up(self)
        self.set_up_good_call()
        self.x = chapter.lm_design_matrix(
            self.hform, self.hddesc, self.dat_ovc, self.hxdesc,
        )
        self.fun = chapter.lm_submodel

    def test_good_plabs(self):
        """
        Good call w.r.t. returned plabs.
        """
        import warnings

        m_x = self.x.shape[1]
        # There should be no warning about plab's chrlen being too short:
        warnings.simplefilter('error', b_utils.NagAlgorithmicWarning)
        soln = self.fun('S', self.hform, self.hxdesc, m_x, m_x+1, 0)
        warnings.simplefilter('always', b_utils.NagAlgorithmicWarning)
        np.testing.assert_equal(
            soln.plab,
            [
                'Intercept', 'F2_2', 'F2_3', 'CON', 'F1_2', 'F1_3', 'F2_2.CON',
                'F2_3.CON', 'F2_2.F1_2', 'F2_2.F1_3', 'F2_3.F1_2', 'F2_3.F1_3',
                'CON.F1_2', 'CON.F1_3',
            ]
        )

if __name__ == '__main__': # pragma: no cover
    unittest.main()
