#!/usr/bin/env python
"""
Integration tests for wrapper functionality in the rand submodule.

Public use of this module is not supported by NAG.
"""

# NAG Copyright 2018-2019.

# pylint: disable=invalid-name

import unittest

import numpy as np

from naginterfaces.base import utils as b_utils
from test_functionality_core import IntegrationTest

class TestG05kf(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the g05kf wrapper.
    """

class TestG05kfBase(TestG05kf, unittest.TestCase):
    """
    Concrete base tests.
    """

    def setUp(self):
        from naginterfaces.base import rand as chapter
        TestG05kf.set_up(self)
        self.fun = chapter.init_repeat

    def test_good(self):
        """
        Good call.
        """
        statecomm = self.fun(genid=3, subid=0, seed=[42], lseed=1)
        self.assertEqual(len(statecomm['state']), 1260)

class TestG05kfNp(TestG05kf, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import rand as chapter
        TestG05kf.set_up(self)
        self.fun = chapter.init_repeat

    def test_good(self):
        """
        Good call.
        """
        statecomm = self.fun(genid=3, seed=[42])
        self.assertEqual(statecomm['state'][0], 1260)

class TestG05kg(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the g05kg wrapper.
    """

class TestG05kgNp(TestG05kg, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import rand as chapter
        TestG05kg.set_up(self)
        self.fun = chapter.init_nonrepeat

    def test_good(self):
        """
        Good call.
        """
        statecomm = self.fun(genid=3)
        self.assertEqual(statecomm['state'][0], 1260)

class TestG05nd(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the g05nd wrapper.
    """

class TestG05ndNp(TestG05nd, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import rand as chapter
        TestG05nd.set_up(self)
        self.fun = chapter.sample
        self.statecomm = chapter.init_repeat(genid=1, seed=[1762543])
        self.n = 8

    def test_good_ipop(self):
        """
        Good call, ipop supplied.
        """
        for m in range(1, 4+1):
            isampl = self.fun(m, self.statecomm, ipop=list(range(1, self.n+1)))
        np.testing.assert_equal(
            isampl,
            [2, 6, 7, 8],
        )

    def test_good_noipop(self):
        """
        Good call, ipop inferred.
        """
        for m in range(1, 4+1):
            isampl = self.fun(m, self.statecomm, n=self.n)
        np.testing.assert_equal(
            isampl,
            [2, 6, 7, 8],
        )

    def test_bad(self):
        """
        Bad call, ipop and n omitted.
        """
        self.my_assert_raises_regex(
            b_utils.NagTypeError,
            'ipop.*must not be None when n is None',
            self.fun,
            1, self.statecomm,
        )

class TestG05py(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the g05py wrapper.
    """

class TestG05pyNp(TestG05py, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import rand as chapter
        TestG05py.set_up(self)
        self.init = chapter.init_repeat
        self.fun = chapter.matrix_corr

    def test_good(self):
        """
        Good call.
        """
        statecomm = self.init(genid=1, seed=[1762543])
        d = [0.7, 0.9, 1.4]
        np.testing.assert_allclose(
            self.fun(d, statecomm),
            [
                [1., -0.2549414, -0.1004264],
                [-0.2549414, 1., 0.23434893],
                [-0.1004264, 0.23434893, 1.],
            ],
            )

class TestG05rd(IntegrationTest):
    """
    Abstract suite of integration tests for the g05rd wrapper.
    """

    # pylint: disable=too-few-public-methods

    def default_args(self):
        """
        A default argument set.
        """
        # pylint: disable=attribute-defined-outside-init
        from naginterfaces.library import rand as chapter
        n = 10
        c = np.array([
            [1.69, 0.39, -1.86, 0.07],
            [0., 98.01, -7.07, -0.71],
            [0., 0., 11.56, 0.03],
            [0., 0., 0., 0.01],
        ])
        comm = {}
        self.args = [
            2, # mode, 0
            n, # n, 1
            c.shape[0], # m, 2
            c, # c, 3
            comm, # comm, 4
            chapter.init_repeat(
                genid=1, subid=1, seed=[1762543],
            ), # statecomm, 5
            np.empty((n, c.shape[0])), # x, 6
        ]
        self.result = np.array([
            [0.6364, 0.0517, 0.4137, 0.8817],
            [0.1065, 0.2461, 0.7993, 0.3806],
            [0.7460, 0.6313, 0.2708, 0.5421],
            [0.7983, 0.0564, 0.6868, 0.9234],
            [0.1046, 0.5790, 0.8533, 0.2208],
            [0.4925, 0.2784, 0.3513, 0.5158],
            [0.3843, 0.2349, 0.9472, 0.7801],
            [0.7871, 0.9941, 0.9403, 0.2044],
            [0.4982, 0.9015, 0.7176, 0.2914],
            [0.6717, 0.5359, 0.5961, 0.4487],
        ])

class TestG05rdBase(TestG05rd, unittest.TestCase):
    """
    Concrete base tests.
    """

    def setUp(self):
        from naginterfaces.base import rand as chapter
        TestG05rd.set_up(self)
        self.fun = chapter.copula_normal
        self.default_args()

    def test_good_setup_gen(self):
        """
        Good call.
        """
        self.fun(*self.args)
        np.testing.assert_allclose(
            self.args[-1],
            self.result,
            rtol=1e-3,
        )

    def test_good_setup_then_gen(self):
        """
        Good call.
        """
        self.args[0] = 0
        self.args[-1] = np.empty((self.args[1], 0))
        self.fun(*self.args)
        self.args[0] = 1
        self.args[-1] = np.empty((self.args[1], self.args[2]))
        self.fun(*self.args)
        np.testing.assert_allclose(
            self.args[-1],
            self.result,
            rtol=1e-3,
        )

class TestG05td(IntegrationTest):
    """
    Abstract suite of integration tests for the g05td wrapper.
    """

    # pylint: disable=too-few-public-methods

    def default_args(self):
        """
        A default argument set.
        """
        # pylint: disable=attribute-defined-outside-init
        from naginterfaces.library import rand as chapter
        p = np.array([
            0.01, 0.02, 0.04, 0.08, 0.2, 0.3, 0.2, 0.08, 0.04, 0.02, 0.01,
        ])
        n = 20
        comm = {}
        self.args = [
            2, # mode, 0
            n, # n, 1
            p, # p, 2
            len(p), # np, 3
            -5, # ip, 4
            1, # itype, 5
            comm, # comm, 6
            chapter.init_repeat(
                genid=1, subid=1, seed=[1762543],
            ), # statecomm, 7
            np.empty(n, dtype=int), # x, 8
        ]
        self.result = [
            0, -2, 1, 1, -2, 0, 0, 1, 0, 1,
            -3, -1, 0, -3, 0, -1, -1, 5, 2, 0,
        ]

class TestG05tdBase(TestG05td, unittest.TestCase):
    """
    Concrete base tests.
    """

    def setUp(self):
        from naginterfaces.base import rand as chapter
        TestG05td.set_up(self)
        self.fun = chapter.int_general
        self.default_args()

    def test_good_setup_gen(self):
        """
        Good call.
        """
        self.fun(*self.args)
        np.testing.assert_equal(
            self.args[8],
            self.result,
        )

    def test_good_setup_then_gen(self):
        """
        Good call.
        """
        self.args[0] = 0
        self.fun(*self.args)
        self.args[0] = 1
        self.fun(*self.args)
        np.testing.assert_equal(
            self.args[8],
            self.result,
        )

    def test_good_setup_no_ref(self):
        """
        Good call.
        """
        self.args[0] = 3
        self.args[6] = None
        self.fun(*self.args)
        np.testing.assert_equal(
            self.args[8],
            self.result,
        )
        self.assertIsNone(self.args[6])

if __name__ == '__main__': # pragma: no cover
    unittest.main()
