#!/usr/bin/env python
"""
Integration tests for wrapper functionality in the sparse submodule.

Public use of this module is not supported by NAG.
"""

# NAG Copyright 2018-2019.

# pylint: disable=invalid-name

import unittest

import numpy as np

from naginterfaces.base import utils as b_utils
from test_functionality_core import IntegrationTest

class TestF11bd(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the f11bd wrapper.
    """

class TestF11bdNp(TestF11bd, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import sparse as chapter
        TestF11bd.set_up(self)
        self.fun = chapter.real_gen_basic_setup

    def test_bad_method(self):
        """
        Bad method supplied.
        """
        self.my_assert_raises_regex(
            b_utils.NagValueError,
            'Constraint: method',
            self.fun,
            'Spam', 'P', 8, 1, 1.e-8, 20, 0., 0., 0,
        )

    def test_good(self):
        """
        Good call.
        """
        comm = self.fun(
            'BiCGSTAB', 'P', 8, 1, 1.e-8, 20, 0., 0., 0,
        )
        self.assertEqual(
            len(comm['work']), 151,
        )

class TestF11dg(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the f11dg wrapper.
    """

class TestF11dgBase(TestF11dg, unittest.TestCase):
    """
    Concrete base tests.
    """

    def setUp(self):
        from naginterfaces.base import sparse as chapter
        TestF11dg.set_up(self)
        self.fun = chapter.real_symm_basic_setup

    def test_bad_sigtol(self):
        """
        Bad sigtol supplied.

        Coverage for: STR_FMT_FLOAT outputting.
        """
        self.my_assert_raises_regex(
            b_utils.NagValueError,
            'sigtol = *4.20',
            self.fun,
            'CG', 'N', 'S', '1', 'N', 1, 1, 0., 1, 0., -42., 42., 1, 0, 120,
        )

class TestF11ye(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the f11ye wrapper.
    """

class TestF11yeNp(TestF11ye, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import sparse as chapter
        TestF11ye.set_up(self)
        self.fun = chapter.sym_rcm

    def test_good(self):
        """
        Good call.
        """
        irowix = [
            2, 5, 6, 1, 3, 11, 2, 4, 16,
            3, 5, 21, 1, 4, 26, 1, 7, 10,
            6, 8, 30, 7, 9, 42, 8, 10, 38,
            6, 9, 12, 2, 12, 15, 10, 11, 13,
            12, 14, 37, 13, 15, 33, 11, 14, 17,
            3, 17, 20, 15, 16, 18, 17, 19, 32,
            18, 20, 53, 16, 19, 22, 4, 22, 25,
            20, 21, 23, 22, 24, 52, 23, 25, 48,
            21, 24, 27, 5, 27, 30, 25, 26, 28,
            27, 29, 47, 28, 30, 43, 7, 26, 29,
            32, 35, 54, 18, 31, 33, 14, 32, 34,
            33, 35, 36, 31, 34, 56, 34, 37, 40,
            13, 36, 38, 9, 37, 39, 38, 40, 41,
            36, 39, 57, 39, 42, 45, 8, 41, 43,
            29, 42, 44, 43, 45, 46, 41, 44, 58,
            44, 47, 50, 28, 46, 48, 24, 47, 49,
            48, 50, 51, 46, 49, 59, 49, 52, 55,
            23, 51, 53, 19, 52, 54, 31, 53, 55,
            51, 54, 60, 35, 57, 60, 40, 56, 58,
            45, 57, 59, 50, 58, 60, 55, 56, 59,
        ]
        icolzp = [
            1, 4, 7, 10, 13, 16,
            19, 22, 25, 28, 31, 34,
            37, 40, 43, 46, 49, 52,
            55, 58, 61, 64, 67, 70,
            73, 76, 79, 82, 85, 88,
            91, 94, 97, 100, 103, 106,
            109, 112, 115, 118, 121, 124,
            127, 130, 133, 136, 139, 142,
            145, 148, 151, 154, 157, 160,
            163, 166, 169, 172, 175, 178,
            181,
        ]
        lopts = [False]*5
        np.testing.assert_equal(
            self.fun(icolzp, irowix, lopts).perm,
            [
                1, 5, 2, 6, 4, 3, 26, 7, 30, 11, 12, 10, 21, 16, 27, 25, 8, 29,
                17, 15, 13, 9, 22, 20, 28, 24, 42, 43, 18, 14, 37, 38, 23, 19,
                47, 48, 41, 44, 32, 33, 36, 39, 52, 53, 46, 49, 45, 31, 34, 40,
                51, 54, 50, 58, 35, 57, 55, 59, 56, 60,
            ],
        )

class TestF11zc(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the f11zc wrapper.
    """

class TestF11zcNp(TestF11zc, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    # pylint: disable=too-many-instance-attributes

    def setUp(self):
        from naginterfaces.library import sparse as chapter
        TestF11zc.set_up(self)
        self.fun = chapter.real_rect_sort
        self.m = 3
        self.n = 4
        self.a = [1., 2., 3., 4., 5., 0.]
        self.irow = [3, 2, 2, 1, 3, 1]
        self.icol = [3, 1, 4, 2, 3, 1]
        self.dup = 'S'
        self.zer = 'R'
        self.exp_a = [2., 4., 6., 3.]
        self.exp_irow = [2, 1, 3, 2]
        self.exp_icol = [1, 2, 3, 4]
        self.exp_istc = [1, 2, 3, 4, 5]

    def test_good_defaults(self):
        """
        Good call.
        """
        ret = self.fun(self.m, self.a, self.irow, n=self.n, icol=self.icol)
        np.testing.assert_array_equal(ret.a, self.exp_a)
        np.testing.assert_array_equal(ret.irow, self.exp_irow)
        np.testing.assert_array_equal(ret.icol, self.exp_icol)
        np.testing.assert_array_equal(ret.istc, self.exp_istc)

    def test_good_supplied(self):
        """
        Good call.
        """
        ret = self.fun(
            self.m, self.a, self.irow,
            n=self.n, icol=self.icol, dup=self.dup, zer=self.zer,
        )
        np.testing.assert_array_equal(ret.a, self.exp_a)
        np.testing.assert_array_equal(ret.irow, self.exp_irow)
        np.testing.assert_array_equal(ret.icol, self.exp_icol)
        np.testing.assert_array_equal(ret.istc, self.exp_istc)

    def test_good_ccs(self):
        """
        Good call.
        """
        a = [4., 6., 1., -1., 1., 1., 2., 2.]
        irow = [3, 1, 1, 2, 3, 2, 1, 3]
        istc = [1, 3, 5, 8, 9]
        ret = self.fun(self.m, a, irow, istc=istc)
        np.testing.assert_array_equal(
            ret.a, [6., 4., 1., -1., 2., 1., 1., 2.],
        )
        np.testing.assert_array_equal(
            ret.irow, [1, 3, 1, 2, 1, 2, 3, 3],
        )
        np.testing.assert_array_equal(
            ret.icol, [1, 1, 2, 2, 3, 3, 3, 4],
        )
        np.testing.assert_array_equal(
            ret.istc, istc,
        )

    def test_bad_n(self):
        """
        Bad call, n not supplied.
        """
        self.my_assert_raises_regex(
            b_utils.NagTypeError,
            'argument n. must not be None when istc is None',
            self.fun,
            self.m, self.a, self.irow, icol=self.icol,
        )

    def test_bad_col_omitted(self):
        """
        Bad call, all column data omitted.
        """
        self.my_assert_raises_regex(
            b_utils.NagValueError,
            'Exactly one of icol and istc must be provided',
            self.fun,
            self.m, self.a, self.irow,
        )

    def test_bad_col_both(self):
        """
        Bad call, all column data supplied.
        """
        self.my_assert_raises_regex(
            b_utils.NagValueError,
            'Exactly one of icol and istc must be provided',
            self.fun,
            self.m, self.a, self.irow,
            icol=self.icol, istc=[42],
        )

if __name__ == '__main__': # pragma: no cover
    unittest.main()
