# -*- coding: utf-8 -*-
# pylint: disable=bare-except,chained-comparison,invalid-name,len-as-condition,line-too-long,protected-access,redefined-outer-name,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unneeded-not,unused-import
u"""
Interfaces for the NAG Mark 27.0 `eigen` Chapter.

``eigen`` - Eigenvalues and Eigenvectors

Base interfaces.

We do not intend for these lower-level base interfaces to be called
directly from user code. Please refer to the ``naginterfaces.library``
subpackage instead.

This module provides functions for various types of matrix eigenvalue
problem:

- standard eigenvalue problems (finding eigenvalues and eigenvectors of
  a square matrix A);

- singular value problems (finding singular values and singular vectors
  of a rectangular matrix A);

- generalized eigenvalue problems (finding eigenvalues and eigenvectors
  of a matrix pencil A-lambda B).

- quadratic eigenvalue problems (finding eigenvalues and eigenvectors of
  the quadratic lambda^2A+lambda B+C).

Functions are provided for both real and complex data.

The majority of functions for these problems can be found in submodule
``lapackeig`` which contains software derived from LAPACK.
However, you should read [the F02 Introduction] before turning to
submodule ``lapackeig``, especially if you are a new user. Submodule
``sparseig`` contains functions for large sparse eigenvalue problems,
although one such function is also available in this module.

Submodule ``eigen`` and submodule ``lapackeig`` contain Black Box (or
Driver) functions that enable many problems to be solved by a call to a
single function, and the decision trees in [Decision Trees] direct you
to the most appropriate functions in submodule ``eigen`` and submodule
``lapackeig``.
The submodule ``eigen`` functions call functions in submodule
``lapacklin`` and submodule ``lapackeig`` wherever possible to perform
the computations, and there are pointers in [Decision Trees] to the
relevant decision trees in submodule ``lapackeig``.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f02/f02intro.html
"""

# NAG Copyright 2017-2019.

import collections as _collections
import sys as _sys

from ..base import utils

def real_gen_eigsys(crit, n, a, wl, wu, mest, wr, wi, vr, vi):
    u"""
    Selected eigenvalues and eigenvectors of real nonsymmetric matrix
    (Black Box).

    For full information please refer to the NAG Library document for
    f02ec

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f02/f02ecf.html

    Parameters
    ----------
    crit : str, length 1
        Indicates the criterion for selecting eigenvalues.

        `crit` = 'M'

          Eigenvalues are selected according to their moduli: w_l <=
          |lambda_i| <= w_u.

        `crit` = 'R'

          Eigenvalues are selected according to their real parts: w_l <=
          Re(lambda_i) <= w_u.

    n : int
        n, the order of the matrix A.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n general matrix A.

        On exit: contains the Hessenberg form of the balanced input
        matrix A' (see [Further Comments]).

    wl : float
        w_l and w_u, the lower and upper bounds on the criterion for the
        selected eigenvalues (see `crit`).

    wu : float
        w_l and w_u, the lower and upper bounds on the criterion for the
        selected eigenvalues (see `crit`).

    mest : int
        `mest` must be an upper bound on m, the number of eigenvalues
        and eigenvectors selected. No eigenvectors are computed if
        `mest` < m.

    wr : float, shape (`n`), modified in place
        On exit: the first `m` elements of `wr` and `wi` hold the real
        and imaginary parts, respectively, of the selected eigenvalues;
        elements `m`+1 to `n` contain the other eigenvalues. Complex
        conjugate pairs of eigenvalues are stored in consecutive
        elements of the arrays, with the eigenvalue having positive
        imaginary part first. See also [Further Comments].

    wi : float, shape (`n`), modified in place
        On exit: the first `m` elements of `wr` and `wi` hold the real
        and imaginary parts, respectively, of the selected eigenvalues;
        elements `m`+1 to `n` contain the other eigenvalues. Complex
        conjugate pairs of eigenvalues are stored in consecutive
        elements of the arrays, with the eigenvalue having positive
        imaginary part first. See also [Further Comments].

    vr : float, shape (`n`, `mest`), modified in place
        On exit: contains the real parts of the selected eigenvectors,
        with the ith column holding the real part of the eigenvector
        associated with the eigenvalue lambda_i (stored in `wr`[i-1] and
        `wi`[i-1]).

    vi : float, shape (`n`, `mest`), modified in place
        On exit: contains the imaginary parts of the selected
        eigenvectors, with the ith column holding the imaginary part of
        the eigenvector associated with the eigenvalue lambda_i (stored
        in `wr`[i-1] and `wi`[i-1]).

    Returns
    -------
    m : int
        m, the number of eigenvalues actually selected.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `mest` = *<value>*.

            Constraint: `mest` >= 1.

        (`errno` 1)
            On entry, `wu` = *<value>* and `wl` = *<value>*.

            Constraint: `wu` > `wl`.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 1)
            On entry, `crit` = *<value>*.

            Constraint: `crit` = 'M' or 'R'.

        (`errno` 2)
            The QR algorithm failed to converge: only *<value>*
            eigenvalues have been computed; no eigenvectors have been
            computed.

        (`errno` 3)
            There are more than `mest` eigenvalues in the specified
            range. `m` (number of eigenvalues in range) = *<value>* and
            `mest` = *<value>*. No eigenvectors have been computed.
            Rerun with second dimension of `vr` and `vi` = `mest` >=
            `m`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            Inverse iteration failed to compute all the specified
            eigenvectors. The number of eigenvectors which failed to
            converge is *<value>*. The corresponding columns of `vr` and
            `vi` are set to zero.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.eigen.real_gen_eigsys'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f02ecft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, crit(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Input, wl.
        utils._BYREF_FLOAT64, # Input, wu.
        utils._BYREF_INT, # Input, mest.
        utils._BYREF_INT, # Output, m.
        utils._EngineFloat64ArrayType.c_type, # Output, wr[n].
        utils._EngineFloat64ArrayType.c_type, # Output, wi[n].
        utils._EngineFloat64ArrayType.c_type, # Output, vr[ldvr, mest].
        utils._BYREF_INT, # Input, ldvr.
        utils._EngineFloat64ArrayType.c_type, # Output, vi[ldvi, mest].
        utils._BYREF_INT, # Input, ldvi.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._EngineLogicalArrayType.c_type, # Workspace, bwork.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of crit.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (vr, 'vr'), (vi, 'vi')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _crit = utils._EngineCharScalarType(
        crit, {'fun_name': fun_name, 'entity': 'crit'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _wl = utils._EngineFloat64ScalarType(
        wl, {'fun_name': fun_name, 'entity': 'wl'},
    ).to_ctypes()
    _wu = utils._EngineFloat64ScalarType(
        wu, {'fun_name': fun_name, 'entity': 'wu'},
    ).to_ctypes()
    _mest = utils._EngineIntScalarType(
        mest, {'fun_name': fun_name, 'entity': 'mest'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType.empty_ctypes()
    _wr_cls = utils._EngineFloat64ArrayType(
        wr, {'fun_name': fun_name, 'entity': 'wr'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _wr = _wr_cls.to_ctypes(can_reference_contents=False)
    _wi_cls = utils._EngineFloat64ArrayType(
        wi, {'fun_name': fun_name, 'entity': 'wi'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _wi = _wi_cls.to_ctypes(can_reference_contents=False)
    _ldvr = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(vr, {'fun_name': fun_name, 'entity': 'vr'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldvr'},
    ).to_ctypes()
    _mest_py = utils._EngineIntScalarType.to_py(_mest)
    _vr_cls = utils._EngineFloat64ArrayType(
        vr, {'fun_name': fun_name, 'entity': 'vr'},
        exp_shape=(_n_py, _mest_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vr = _vr_cls.to_ctypes(can_reference_contents=False)
    _ldvi = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(vi, {'fun_name': fun_name, 'entity': 'vi'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldvi'},
    ).to_ctypes()
    _vi_cls = utils._EngineFloat64ArrayType(
        vi, {'fun_name': fun_name, 'entity': 'vi'},
        exp_shape=(_n_py, _mest_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _vi = _vi_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _bwork = utils._EngineLogicalArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _crit,
        _n,
        _a,
        _lda,
        _wl,
        _wu,
        _mest,
        _m,
        _wr,
        _wi,
        _vr,
        _ldvr,
        _vi,
        _ldvi,
        _work,
        _lwork,
        _iwork,
        _bwork,
        _errbuf,
        _ierr,
        _crit._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mest = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mest >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, wu = {:s} and wl = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: wu > wl.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, crit = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: crit = \'M\' or \'R\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QR algorithm failed to converge:',
                    'only {:s} eigenvalues have been computed;'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'no eigenvectors have been computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'There are more than mest eigenvalues in the specified range.',
                    'm (number of eigenvalues in range) = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'mest = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'No eigenvectors have been computed.',
                    'Rerun with second dimension of vr and vi = mest >= m.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99989:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Inverse iteration failed to compute all the specified eigenvectors.',
                    'The number of eigenvectors which failed to converge is {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'The corresponding columns of vr and vi are set to zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _wr_cls.refresh_py(_wr)
        _wi_cls.refresh_py(_wi)
        _vr_cls.refresh_py(_vr)
        _vi_cls.refresh_py(_vi)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['m'],
            )(
                _m_py,
            )
    _err_class.manage()
    return return_data

def real_gen_sparse_arnoldi(n, nnz, a, icolzp, irowix, nev, ncv, sigma, monit, option, w, v, resid, data=None, io_manager=None):
    u"""
    Selected eigenvalues and eigenvectors of a real sparse general
    matrix.

    Note: this function uses optional algorithmic parameters, see also:
    ``sparseig.real_init``, ``sparseig.real_iter``,
    ``sparseig.real_proc``, ``sparseig.real_option``,
    ``sparseig.real_monit``.

    For full information please refer to the NAG Library document for
    f02ek

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f02/f02ekf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    nnz : int
        The number of nonzero elements of the matrix A and, if a nonzero
        shifted inverse is to be applied, all diagonal elements. Each
        nonzero is counted once in the latter case.

    a : float, shape (`nnz`), modified in place
        On entry: the array of nonzero elements (and diagonal elements
        if a nonzero inverse shift is to be applied) of the n by n
        general matrix A.

        On exit: if a nonzero shifted inverse is to be applied then the
        diagonal elements of A have the shift value, as supplied in
        `sigma`, subtracted.

    icolzp : int, shape (`n`+1)
        `icolzp`[i-1] contains the index in `a` of the start of column
        i, for i = 1,2,...,n; `icolzp`[`n`] must contain the value
        `nnz`+1. Thus the number of nonzero elements in column i of A is
        `icolzp`[i]-`icolzp`[i-1]; when shifts are applied this includes
        diagonal elements irrespective of value. See [the F11
        Introduction].

    irowix : int, shape (`nnz`)
        `irowix`[i-1] contains the row index for each entry in `a`. See
        [the F11 Introduction].

    nev : int
        The number of eigenvalues to be computed.

    ncv : int
        The number of Arnoldi basis vectors to use during the
        computation.

        At present there is no a priori analysis to guide the selection
        of `ncv` relative to `nev`.

        However, it is recommended that `ncv` >= 2*`nev`+1.

        If many problems of the same type are to be solved, you should
        experiment with increasing `ncv` while keeping `nev` fixed for a
        given test problem.

        This will usually decrease the required number of matrix-vector
        operations but it also increases the work and storage required
        to maintain the orthogonal basis vectors.

        The optimal 'cross-over' with respect to CPU time is problem
        dependent and must be determined empirically.

    sigma : float
        If the 'Shifted Inverse Real' mode has been selected then
        `sigma` contains the real shift used; otherwise `sigma` is not
        referenced. This mode can be selected by setting the appropriate
        options in the user-supplied function `option`.

    monit : None or callable monit(ncv, niter, nconv, w, rzest,
        data=None)
        `monit` is used to monitor the progress of
        ``real_gen_sparse_arnoldi``. `monit` may be None if no
        monitoring is actually required. `monit` is called after the
        solution of each eigenvalue sub-problem and also just prior to
        return from ``real_gen_sparse_arnoldi``.

        Parameters
        ~~~~~~~~~~
        ncv : int
            The dimension of the arrays `w` and `rzest`. The number of
            Arnoldi basis vectors used during the computation.

        niter : int
            The number of the current Arnoldi iteration.

        nconv : int
            The number of converged eigenvalues so far.

        w : complex, ndarray, shape (`ncv`)
            The first `nconv` elements of this argument contain the
            converged approximate eigenvalues.

        rzest : float, ndarray, shape (`ncv`)
            The first `nconv` elements of this argument contain the Ritz
            estimates (error bounds) on the converged approximate
            eigenvalues.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    option : None or callable option(comm, data=None)
        You can supply non-default options to the Arnoldi eigensolver by
        repeated calls to ``sparseig.real_option`` from within `option`.
        (Please note that it is only necessary to call
        ``sparseig.real_option``; no call to ``sparseig.real_init`` is
        required from within `option`.) For example, you can set the
        mode to 'Shifted Inverse Real', you can increase the 'Iteration
        Limit' beyond its default and you can print varying levels of
        detail on the iterative process using 'Print Level'.

        Parameters
        ~~~~~~~~~~
        comm : dict, communication object, modifiable in place
            Communication structure.

            This argument has been initialized to hold the default
            option set.

            If you wish to supply non-default options to the Arnoldi
            eigensolver, this argument may be passed as argument comm in
            a call to ``sparseig.real_option``.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    w : complex, shape (`ncv`), modified in place
        On exit: the first `nconv` elements contain the converged
        approximations to the selected eigenvalues. Since complex
        conjugate pairs of eigenvalues appear together, it is possible
        (given an odd number of converged real eigenvalues) for
        ``real_gen_sparse_arnoldi`` to return one more eigenvalue than
        requested.

    v : float, shape (`n`, `ncv`), modified in place
        On exit: contains the eigenvectors associated with the
        eigenvalue lambda_i, for i = 1,2,...,`nconv` (stored in `w`).
        For a real eigenvalue, lambda_j, the corresponding eigenvector
        is real and is stored in `v`[i-1,j-1], for i = 1,2,...,n. For
        complex conjugate pairs of eigenvalues, w_{j+1} = w_j_, the real
        and imaginary parts of the corresponding eigenvectors are
        stored, respectively, in `v`[i-1,j-1] and `v`[i-1,j], for i =
        1,2,...,n. The imaginary parts stored are for the first of the
        conjugate pair of eigenvectors; the other eigenvector in the
        pair is obtained by negating these imaginary parts.

    resid : float, shape (`nev`+1), modified in place
        On exit: the residual ||Aw_i-lambda_i w_i||_2 for the estimates
        to the eigenpair lambda_i and w_i is returned in `resid`[i-1],
        for i = 1,2,...,`nconv`.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nconv : int
        The number of converged approximations to the selected
        eigenvalues. On successful exit, this will normally be either
        `nev` or `nev`+1 depending on the number of complex conjugate
        pairs of eigenvalues returned.

    Other Parameters
    ----------------
    'Advisory' : int
        Default = 0

        If the option 'List' is set then option specifications are
        listed in a 'List' file by setting the option to a file
        identification (unit) number associated with advisory messages
        (see :class:`~naginterfaces.base.utils.FileObjManager` and the
        ``FileObjManager`` method
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`).

    'Defaults' : valueless
        This special keyword may be used to reset all options to their
        default values.

    'Iteration Limit' : int
        Default = 300

        The limit on the number of Arnoldi iterations that can be
        performed before ``real_gen_sparse_arnoldi`` exits with `errno`
        = 24.

    'Largest Magnitude' : valueless
        Default

        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is to compute the eigenvalues of largest magnitude
        using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP the linear operator determined by the
        computational mode and problem type.

    'Largest Imaginary' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is to compute the eigenvalues of largest magnitude
        using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP the linear operator determined by the
        computational mode and problem type.

    'Largest Real' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is to compute the eigenvalues of largest magnitude
        using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP the linear operator determined by the
        computational mode and problem type.

    'Smallest Imaginary' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is to compute the eigenvalues of largest magnitude
        using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP the linear operator determined by the
        computational mode and problem type.

    'Smallest Magnitude' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is to compute the eigenvalues of largest magnitude
        using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP the linear operator determined by the
        computational mode and problem type.

    'Smallest Real' : valueless
        The Arnoldi iterative method converges on a number of
        eigenvalues with given properties.
        The default is to compute the eigenvalues of largest magnitude
        using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Real' part, 'Largest Imaginary' part, 'Smallest Magnitude',
        'Smallest Real' part or 'Smallest Imaginary' part.

        Note that these options select the eigenvalue properties for
        eigenvalues of OP the linear operator determined by the
        computational mode and problem type.

    'Nolist' : valueless
        Default

        Option 'List' enables printing of each option specification as
        it is supplied. 'Nolist' suppresses this printing.

    'List' : valueless
        Option 'List' enables printing of each option specification as
        it is supplied. 'Nolist' suppresses this printing.

    'Monitoring' : int
        Default = -1

        If i > 0, monitoring information is output to unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`)
        i during the solution of each problem; this may be the same as
        the 'Advisory' unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`).
        The type of information produced is dependent on the value of
        'Print Level', see the description of the option 'Print Level'
        for details of the information produced.
        Please see the ``FileObjManager`` method
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`
        to associate a file with a given unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`).

    'Print Level' : int
        Default = 0

        This controls the amount of printing produced by
        ``real_gen_sparse_arnoldi`` as follows.

        [table omitted]

    'Regular' : valueless
        Default

        These options define the computational mode which in turn
        defines the form of operation OP(x) to be performed.

        Given a standard eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

    'Shifted Inverse Real' : valueless
        These options define the computational mode which in turn
        defines the form of operation OP(x) to be performed.

        Given a standard eigenvalue problem in the form Ax = lambda x
        then the following modes are available with the appropriate
        operator OP(x).

        [table omitted]

    'Tolerance' : float
        Default = epsilon

        An approximate eigenvalue has deemed to have converged when the
        corresponding Ritz estimate is within 'Tolerance' relative to
        the magnitude of the eigenvalue.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 2)
            On entry, `nnz` = *<value>*.

            Constraint: `nnz` > 0.

        (`errno` 2)
            On entry, `nnz` = *<value>* and `n` = *<value>*.

            Constraint: `nnz` <= `n`*`n`.

        (`errno` 3)
            On entry, in shifted inverse mode, the jth diagonal element
            of A is not defined, for j = *<value>*.

        (`errno` 4)
            On entry, `icolzp`[0] = *<value>*.

            Constraint: `icolzp`[0] = 1.

        (`errno` 4)
            On entry, for i = *<value>*, `icolzp`[i-1] = *<value>* and
            `icolzp`[i] = *<value>*.

            Constraint: `icolzp`[i-1] < `icolzp`[i].

        (`errno` 4)
            On entry, `icolzp`[`n`] = *<value>* and `nnz` = *<value>*.

            Constraint: `icolzp`[`n`] = `nnz`+1.

        (`errno` 5)
            On entry, in specification of column *<value>*, and for j =
            *<value>*, `irowix`[j-1] = *<value>* and `irowix`[j] =
            *<value>*.

            Constraint: `irowix`[j-1] < `irowix`[j].

        (`errno` 6)
            On entry, `nev` = *<value>*.

            Constraint: `nev` > 0.

        (`errno` 7)
            On entry, `ncv` = *<value>* and `nev` = *<value>*.

            Constraint: `ncv` > `nev`+1.

        (`errno` 7)
            On entry, `ncv` = *<value>* and `n` = *<value>*.

            Constraint: `ncv` <= `n`.

        (`errno` 8)
            On entry, the matrix A-sigma*I is numerically singular and
            could not be inverted. Try perturbing the value of sigma.

        (`errno` 8)
            On entry, the matrix A-sigma*I is nearly numerically
            singular and could not be inverted. Try perturbing the value
            of sigma. Norm of matrix = *<value>*, Reciprocal condition
            number = *<value>*.

        (`errno` 21)
            The maximum number of iterations <= 0, the option 'Iteration
            Limit' has been set to *<value>*.

        (`errno` 22)
            An unexpected internal error occurred when solving an
            eigenproblem.

            This error should not occur. Please contact NAG.

        (`errno` 23)
            An unexpected internal error occurred when solving an
            eigenproblem.

            This error should not occur. Please contact NAG.

        (`errno` 24)
            The maximum number of iterations has been reached.

            The maximum number of iterations = *<value>*.

            The number of converged eigenvalues = *<value>*.

            See the function document for further details.

        (`errno` 25)
            No shifts could be applied during a cycle of the implicitly
            restarted Arnoldi iteration.

        (`errno` 26)
            Could not build an Arnoldi factorization. The size of the
            current Arnoldi factorization = *<value>*.

        (`errno` 27)
            Error in internal call to compute eigenvalues and
            corresponding error bounds of the current upper Hessenberg
            matrix.

            Please contact NAG.

        (`errno` 32)
            An unexpected internal error occurred when postprocessing an
            eigenproblem.

            This error should not occur. Please contact NAG.

        (`errno` 33)
            The number of eigenvalues found to sufficient accuracy is
            zero.

        (`errno` 34)
            Internal inconsistency in the number of converged Ritz
            values. Number counted = *<value>*, number expected =
            *<value>*.

        (`errno` 35)
            During calculation of a real Schur form, there was a failure
            to compute *<value>* eigenvalues in a total of *<value>*
            iterations.

        (`errno` 36)
            The computed Schur form could not be reordered by an
            internal call.

            This routine returned with errno = *<value>*.

            Please contact NAG.

        (`errno` 37)
            In calculating eigenvectors, an internal call returned with
            an error.

            Please contact NAG.

    Warns
    -----
    NagCallbackTerminateWarning
        (`errno` 9)
            User requested termination in `monit`, `istat` = *<value>*.

        (`errno` 10)
            User requested termination in `option`, `istat` = *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.eigen.real_gen_sparse_arnoldi'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    monith = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ncv.
        utils._BYREF_INT, # Input, niter.
        utils._BYREF_INT, # Input, nconv.
        utils._EngineComplex128ArrayType.c_type, # Input, w[ncv].
        utils._EngineFloat64ArrayType.c_type, # Input, rzest[ncv].
        utils._BYREF_INT, # Input/Output, istat.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_monith(_monit, en_data, ncv, niter, nconv, w, rzest, istat, _iuser, _ruser):
        """Helper for monit."""
        if monit is None:
            return
        try:
            _ncv_monith = utils._EngineIntScalarType.to_py(ncv.contents)
            _niter_monith = utils._EngineIntScalarType.to_py(niter.contents)
            _nconv_monith = utils._EngineIntScalarType.to_py(nconv.contents)
            _w_monith = utils._EngineComplex128ArrayType.from_pointer(
                w,
                (_ncv_monith,),
            )
            _rzest_monith = utils._EngineFloat64ArrayType.from_pointer(
                rzest,
                (_ncv_monith,),
            )
            if data is not None:
                monit(
                    _ncv_monith,
                    _niter_monith,
                    _nconv_monith,
                    _w_monith,
                    _rzest_monith,
                    data,
                )
            else:
                monit(
                    _ncv_monith,
                    _niter_monith,
                    _nconv_monith,
                    _w_monith,
                    _rzest_monith,
                )
            utils._unlink_pointer(w)
            utils._unlink_pointer(rzest)
        except:
            excs.append(_sys.exc_info())
            istat.contents.value = -1
            if excs[-1][0] is not utils.UserCallbackTerminate:
                en_data.contents.hlperr = utils._IERR_HELPER

    optionh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, comm[lcomm].
        utils._BYREF_INT, # Input, lcomm.
        utils._BYREF_INT, # Input/Output, istat.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_optionh(_option, en_data, icomm, licomm, comm, lcomm, istat, _iuser, _ruser):
        """Helper for option."""
        if option is None:
            return
        try:
            _comm_optionh = {'icomm': icomm, 'comm': comm}
            if data is not None:
                option(
                    _comm_optionh,
                    data,
                )
            else:
                option(
                    _comm_optionh,
                )
        except:
            excs.append(_sys.exc_info())
            istat.contents.value = -1
            if excs[-1][0] is not utils.UserCallbackTerminate:
                en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('f02ekft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nnz.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[nnz].
        utils._EngineIntArrayType.c_type, # Input, icolzp[n+1].
        utils._EngineIntArrayType.c_type, # Input, irowix[nnz].
        utils._BYREF_INT, # Input, nev.
        utils._BYREF_INT, # Input, ncv.
        utils._BYREF_FLOAT64, # Input, sigma.
        utils._UNUSED_FUNCTYPE,
        monith,
        utils._UNUSED_FUNCTYPE,
        optionh,
        utils._BYREF_INT, # Output, nconv.
        utils._EngineComplex128ArrayType.c_type, # Output, w[ncv].
        utils._EngineFloat64ArrayType.c_type, # Output, v[ldv, ncv].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineFloat64ArrayType.c_type, # Output, resid[nev+1].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_STRING, # Input, routine_name(len=6).
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of routine_name.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((v, 'v'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nnz = utils._EngineIntScalarType(
        nnz, {'fun_name': fun_name, 'entity': 'nnz'},
    ).to_ctypes()
    _nnz_py = utils._EngineIntScalarType.to_py(_nnz)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_nnz_py,),
    )
    _a = _a_cls.to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _icolzp = utils._EngineIntArrayType(
        icolzp, {'fun_name': fun_name, 'entity': 'icolzp'},
        exp_shape=(_n_py+1,),
    ).to_ctypes()
    _irowix = utils._EngineIntArrayType(
        irowix, {'fun_name': fun_name, 'entity': 'irowix'},
        exp_shape=(_nnz_py,),
    ).to_ctypes()
    _nev = utils._EngineIntScalarType(
        nev, {'fun_name': fun_name, 'entity': 'nev'},
    ).to_ctypes()
    _ncv = utils._EngineIntScalarType(
        ncv, {'fun_name': fun_name, 'entity': 'ncv'},
    ).to_ctypes()
    _sigma = utils._EngineFloat64ScalarType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
    ).to_ctypes()
    utils._check_callable(monit, {'fun_name': fun_name, 'entity': 'monit'}, allow_none=True)
    _monit = utils._UNUSED_CALLBACK
    _monith = utils._cast_callback(
        monith, cb_monith,
        {'fun_name': fun_name, 'entity': 'monit'},
    )
    utils._check_callable(option, {'fun_name': fun_name, 'entity': 'option'}, allow_none=True)
    _option = utils._UNUSED_CALLBACK
    _optionh = utils._cast_callback(
        optionh, cb_optionh,
        {'fun_name': fun_name, 'entity': 'option'},
    )
    _nconv = utils._EngineIntScalarType.empty_ctypes()
    _ncv_py = utils._EngineIntScalarType.to_py(_ncv)
    _w_cls = utils._EngineComplex128ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_ncv_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldv_py = utils._EngineFloat64ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    _v_cls = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_n_py, _ncv_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _v = _v_cls.to_ctypes(can_reference_contents=False)
    _nev_py = utils._EngineIntScalarType.to_py(_nev)
    _resid_cls = utils._EngineFloat64ArrayType(
        resid, {'fun_name': fun_name, 'entity': 'resid'},
        exp_shape=(_nev_py+1,),
        check_contents=False,
    )
    _resid = _resid_cls.to_ctypes(can_reference_contents=False)
    _iuser = None
    _ruser = None
    _routine_name = utils._EngineRoutineName(*b'f02ek ')
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _n,
        _nnz,
        _a,
        _icolzp,
        _irowix,
        _nev,
        _ncv,
        _sigma,
        _monit,
        _monith,
        _option,
        _optionh,
        _nconv,
        _w,
        _v,
        _ldv,
        _resid,
        _iuser,
        _ruser,
        _routine_name,
        _errbuf,
        _ierr,
        _routine_name._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 102:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nnz = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nnz > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 202:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nnz = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nnz <= n*n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 103:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, in shifted inverse mode, the jth diagonal element of A',
                    'is not defined, for j = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 104:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, icolzp[0] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: icolzp[0] = 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 204:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, for i = {:s}, icolzp[i-1] = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and icolzp[i] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: icolzp[i-1] < icolzp[i].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 304:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, icolzp[n] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'nnz = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: icolzp[n] = nnz+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 105:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, in specification of column {:s}, and for'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'j = {:s}, irowix[j-1] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'irowix[j] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: irowix[j-1] < irowix[j].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 106:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nev > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 107:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncv = {:s} and nev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ncv > nev+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 207:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncv = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ncv <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 108:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the matrix A-sigma*I is numerically singular and could',
                    'not be inverted. Try perturbing the value of sigma.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 208:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the matrix A-sigma*I is nearly numerically singular and could',
                    'not be inverted. Try perturbing the value of sigma.',
                    'Norm of matrix = {:s},'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Reciprocal condition number = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 109:
            _err_class = utils._UserCallbackExit(
                fun_name=fun_name,
                msg_lines=[
                    'User requested termination in monit, istat = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
                cb_excs=excs,
            )
        elif _ierr_py == 10 and _ifmt == 110:
            _err_class = utils._UserCallbackExit(
                fun_name=fun_name,
                msg_lines=[
                    'User requested termination in option, istat = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
                cb_excs=excs,
            )
        elif _ierr_py == 21 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of iterations <= 0, the option',
                    '\'Iteration Limit\' has been set to {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 22 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected internal error occurred when solving an eigenproblem.',
                    'This error should not occur. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 23 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected internal error occurred when solving an eigenproblem.',
                    'This error should not occur. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 24 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of iterations has been reached.',
                    'The maximum number of iterations = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'The number of converged eigenvalues = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'See the function document for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 25 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No shifts could be applied during a cycle of the',
                    'implicitly restarted Arnoldi iteration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 26 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Could not build an Arnoldi factorization. The size',
                    'of the current Arnoldi factorization = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 27 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Error in internal call to compute eigenvalues and corresponding',
                    'error bounds of the current upper Hessenberg matrix.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 32 and _ifmt == 132:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An unexpected internal error occurred when postprocessing an eigenproblem.',
                    'This error should not occur. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 33 and _ifmt == 133:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The number of eigenvalues found to sufficient accuracy is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 34 and _ifmt == 134:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Internal inconsistency in the number of converged Ritz values.',
                    'Number counted = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'number expected = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 35 and _ifmt == 135:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'During calculation of a real Schur form, there was a failure',
                    'to compute {:s} eigenvalues in a total of'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '{:s} iterations.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 36 and _ifmt == 136:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The computed Schur form could not be reordered by an internal call.',
                    'This routine returned with errno = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 37 and _ifmt == 137:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'In calculating eigenvectors, an internal call returned with an error.',
                    'Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _w_cls.refresh_py(_w)
        _v_cls.refresh_py(_v)
        _resid_cls.refresh_py(_resid)
        _nconv_py = utils._EngineIntScalarType.to_py(_nconv)
        return_data = _nconv_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['nconv'],
            )(
                _nconv_py,
            )
    _err_class.manage()
    return return_data

def real_symm_sparse_eigsys(n, m, k, noits, tol, dot, image, monit, novecs, x, d, data=None):
    u"""
    Selected eigenvalues and eigenvectors of sparse symmetric
    eigenproblem (Black Box).

    For full information please refer to the NAG Library document for
    f02fj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f02/f02fjf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix C.

    m : int
        m, the number of eigenvalues required.

    k : int
        The number of simultaneous iteration vectors to be used. Too
        small a value of `k` may inhibit convergence, while a larger
        value of `k` incurs additional storage and additional work per
        iteration.

    noits : int
        The maximum number of major iterations (eigenvalue sub-problems)
        to be performed. If `noits` <= 0, the value 100 is used in place
        of `noits`.

    tol : float
        A relative tolerance to be used in accepting eigenvalues and
        eigenvectors. If the eigenvalues are required to about t
        significant figures, `tol` should be set to about 10^(-t). d_i
        is accepted as an eigenvalue as soon as two successive
        approximations to d_i differ by less than (|d~_i|*`tol`)/10,
        where d~_i is the latest approximation to d_i. Once an
        eigenvalue has been accepted, an eigenvector is accepted as soon
        as (d_i f_i)/(d_i-d_k) < `tol`, where f_i is the normalized
        residual of the current approximation to the eigenvector (see
        [Further Comments] for further information). The values of the
        f_i and d_i can be printed from `monit`. If `tol` is supplied
        outside the range (epsilon,1.0), where epsilon is the machine
        precision, the value epsilon is used in place of `tol`.

    dot : callable retval = dot(n, z, w, data=None)
        `dot` must return the value w^TBz for given vectors w and z.

        For the standard eigenvalue problem, where B = I, `dot` must
        return the dot product w^Tz.

        Parameters
        ~~~~~~~~~~
        n : int
            The number of elements in the vectors z and w and the order
            of the matrix B.

        z : float, ndarray, shape (`n`)
            The vector z for which w^TBz is required.

        w : float, ndarray, shape (`n`)
            The vector w for which w^TBz is required.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value w^TBz.

    image : callable image(n, z, w, data=None)
        `image` must return the vector w = Cz for a given vector z.

        Parameters
        ~~~~~~~~~~
        n : int
            n, the number of elements in the vectors w and z, and the
            order of the matrix C.

        z : float, ndarray, shape (`n`)
            The vector z for which Cz is required.

        w : float, ndarray, shape (`n`), to be modified in place
            On exit: the vector w = Cz.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    monit : None or callable monit(istate, nextit, nevals, nevecs, k, f,
        d, data=None)
        `monit` is used to monitor the progress of
        ``real_symm_sparse_eigsys``. `monit` may be None if no
        monitoring is actually required. `monit` is called after the
        solution of each eigenvalue sub-problem and also just prior to
        return from ``real_symm_sparse_eigsys``.

        The arguments `istate` and `nextit` allow selective printing by
        `monit`.

        Parameters
        ~~~~~~~~~~
        istate : int
            Specifies the state of ``real_symm_sparse_eigsys``.

            `istate` = 0

              No eigenvalue or eigenvector has just been accepted.

            `istate` = 1

              One or more eigenvalues have been accepted since the last
              call to `monit`.

            `istate` = 2

              One or more eigenvectors have been accepted since the last
              call to `monit`.

            `istate` = 3

              One or more eigenvalues and eigenvectors have been
              accepted since the last call to `monit`.

            `istate` = 4

              Return from ``real_symm_sparse_eigsys`` is about to occur.

        nextit : int
            The number of the next iteration.

        nevals : int
            The number of eigenvalues accepted so far.

        nevecs : int
            The number of eigenvectors accepted so far.

        k : int
            k, the number of simultaneous iteration vectors.

        f : float, ndarray, shape (`k`)
            A vector of error quantities measuring the state of
            convergence of the simultaneous iteration vectors. See `tol`
            and [Further Comments] for further details. Each element of
            `f` is initially set to the value 4.0 and an element remains
            at 4.0 until the corresponding vector is tested.

        d : float, ndarray, shape (`k`)
            `d`[i-1] contains the latest approximation to the absolute
            value of the ith eigenvalue of C.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    novecs : int
        The number of approximate vectors that are being supplied in
        `x`. If `novecs` is outside the range (0,`k`), the value 0 is
        used in place of `novecs`.

    x : float, shape (`n`, `k`), modified in place
        On entry: if 0 < `novecs` <= `k`, the first `novecs` columns of
        `x` must contain approximations to the eigenvectors
        corresponding to the `novecs` eigenvalues of largest absolute
        value of C. Supplying approximate eigenvectors can be useful
        when reasonable approximations are known, or when
        ``real_symm_sparse_eigsys`` is being restarted with a larger
        value of `k`. Otherwise it is not necessary to supply
        approximate vectors, as simultaneous iteration vectors will be
        generated randomly by ``real_symm_sparse_eigsys``.

        On exit: if the function exits successfully or `errno` = 2, 3 or
        4, the first m' columns contain the eigenvectors corresponding
        to the eigenvalues returned in the first m' elements of `d`; and
        the next k-m'-1 columns contain approximations to the
        eigenvectors corresponding to the approximate eigenvalues
        returned in the next k-m'-1 elements of `d`. Here m' is the
        value returned in `m`, the number of eigenvalues actually found.
        The kth column is used as workspace.

    d : float, shape (`k`), modified in place
        On exit: if the function exits successfully or `errno` = 2, 3 or
        4, the first m' elements contain the first m' eigenvalues in
        decreasing order of magnitude; and the next k-m'-1 elements
        contain approximations to the next k-m'-1 eigenvalues. Here m'
        is the value returned in `m`, the number of eigenvalues actually
        found. `d`[k-1] contains the value e where (-e,e) is the latest
        interval over which Chebyshev acceleration is performed.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    m : int
        m', the number of eigenvalues actually found. It is equal to m
        if no exception or warning is raised on exit, and is less than m
        if `errno` = 2, 3 or 4. See [Errors] for further information.

    noits : int
        The number of iterations actually performed.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `k` = *<value>* and `n` = *<value>*.

            Constraint: `k` <= `n`.

        (`errno` 1)
            On entry, `m` = *<value>* and `k` = *<value>*.

            Constraint: `m` < `k`.

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 5)
            Not all requested eigenvalues and vectors have been
            obtained.

        (`errno` 5)
            Convergence of eigenvalue sub-problem occurred.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            Not all requested eigenvalues and vectors have been
            obtained.

        (`errno` 3)
            Not all requested eigenvalues and vectors have been
            obtained.

        (`errno` 4)
            Not all requested eigenvalues and vectors have been
            obtained.

    NagCallbackTerminateWarning
        (`errno` i < 0)
            User set `iflag` negative in `dot` or `image`.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.eigen.real_symm_sparse_eigsys'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_dot_name = fun_name + ':dot'

    doth = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input/Output, iflag.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, z[n].
        utils._EngineFloat64ArrayType.c_type, # Input, w[n].
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Input, lruser.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._BYREF_INT, # Input, liuser.
    )

    def cb_doth(_dot, _dotsub, en_data, iflag, n, z, w, retval, _ruser, lruser, _iuser, liuser):
        """Helper for dot."""
        try:
            _n_doth = utils._EngineIntScalarType.to_py(n.contents)
            _z_doth = utils._EngineFloat64ArrayType.from_pointer(
                z,
                (_n_doth,),
            )
            _w_doth = utils._EngineFloat64ArrayType.from_pointer(
                w,
                (_n_doth,),
            )
            if data is not None:
                _retval_doth = dot(
                    _n_doth,
                    _z_doth,
                    _w_doth,
                    data,
                )
            else:
                _retval_doth = dot(
                    _n_doth,
                    _z_doth,
                    _w_doth,
                )
            utils._unlink_pointer(z)
            utils._unlink_pointer(w)
            utils._EngineFloat64ScalarType(
                _retval_doth, {'fun_name': cb_dot_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            iflag.contents.value = -1
            if excs[-1][0] is not utils.UserCallbackTerminate:
                en_data.contents.hlperr = utils._IERR_HELPER

    cb_image_name = fun_name + ':image'

    imageh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input/Output, iflag.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, z[n].
        utils._EngineFloat64ArrayType.c_type, # Output, w[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Input, lruser.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._BYREF_INT, # Input, liuser.
    )

    def cb_imageh(_image, en_data, iflag, n, z, w, _ruser, lruser, _iuser, liuser):
        """Helper for image."""
        try:
            _n_imageh = utils._EngineIntScalarType.to_py(n.contents)
            _z_imageh = utils._EngineFloat64ArrayType.from_pointer(
                z,
                (_n_imageh,),
            )
            _w_imageh = utils._EngineFloat64ArrayType.from_pointer(
                w,
                (_n_imageh,),
                can_reference_contents=False,
            )
            if data is not None:
                image(
                    _n_imageh,
                    _z_imageh,
                    _w_imageh,
                    data,
                )
            else:
                image(
                    _n_imageh,
                    _z_imageh,
                    _w_imageh,
                )
            utils._unlink_pointer(z)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _w_imageh, {'fun_name': cb_image_name, 'returned': 'w'}, w,
            )
            utils._unlink_pointer(w)
        except:
            excs.append(_sys.exc_info())
            iflag.contents.value = -1
            if excs[-1][0] is not utils.UserCallbackTerminate:
                en_data.contents.hlperr = utils._IERR_HELPER

    monith = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, istate.
        utils._BYREF_INT, # Input, nextit.
        utils._BYREF_INT, # Input, nevals.
        utils._BYREF_INT, # Input, nevecs.
        utils._BYREF_INT, # Input, k.
        utils._EngineFloat64ArrayType.c_type, # Input, f[k].
        utils._EngineFloat64ArrayType.c_type, # Input, d[k].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_monith(_monit, en_data, istate, nextit, nevals, nevecs, k, f, d, _iuser, _ruser):
        """Helper for monit."""
        if monit is None:
            return
        try:
            _istate_monith = utils._EngineIntScalarType.to_py(istate.contents)
            _nextit_monith = utils._EngineIntScalarType.to_py(nextit.contents)
            _nevals_monith = utils._EngineIntScalarType.to_py(nevals.contents)
            _nevecs_monith = utils._EngineIntScalarType.to_py(nevecs.contents)
            _k_monith = utils._EngineIntScalarType.to_py(k.contents)
            _f_monith = utils._EngineFloat64ArrayType.from_pointer(
                f,
                (_k_monith,),
            )
            _d_monith = utils._EngineFloat64ArrayType.from_pointer(
                d,
                (_k_monith,),
            )
            if data is not None:
                monit(
                    _istate_monith,
                    _nextit_monith,
                    _nevals_monith,
                    _nevecs_monith,
                    _k_monith,
                    _f_monith,
                    _d_monith,
                    data,
                )
            else:
                monit(
                    _istate_monith,
                    _nextit_monith,
                    _nevals_monith,
                    _nevecs_monith,
                    _k_monith,
                    _f_monith,
                    _d_monith,
                )
            utils._unlink_pointer(f)
            utils._unlink_pointer(d)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('f02fjft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input/Output, m.
        utils._BYREF_INT, # Input, k.
        utils._BYREF_INT, # Input/Output, noits.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        doth,
        utils._UNUSED_FUNCTYPE,
        imageh,
        utils._UNUSED_FUNCTYPE,
        monith,
        utils._BYREF_INT, # Input, novecs.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[ldx, k].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Output, d[k].
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Input, lruser.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._BYREF_INT, # Input, liuser.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x, 'x'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _noits = utils._EngineIntScalarType(
        noits, {'fun_name': fun_name, 'entity': 'noits'},
    ).to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    utils._check_callable(dot, {'fun_name': fun_name, 'entity': 'dot'})
    _dot = utils._UNUSED_CALLBACK
    _dotsub = utils._UNUSED_CALLBACK
    _doth = utils._cast_callback(
        doth, cb_doth,
        {'fun_name': fun_name, 'entity': 'dot'},
    )
    utils._check_callable(image, {'fun_name': fun_name, 'entity': 'image'})
    _image = utils._UNUSED_CALLBACK
    _imageh = utils._cast_callback(
        imageh, cb_imageh,
        {'fun_name': fun_name, 'entity': 'image'},
    )
    utils._check_callable(monit, {'fun_name': fun_name, 'entity': 'monit'}, allow_none=True)
    _monit = utils._UNUSED_CALLBACK
    _monith = utils._cast_callback(
        monith, cb_monith,
        {'fun_name': fun_name, 'entity': 'monit'},
    )
    _novecs = utils._EngineIntScalarType(
        novecs, {'fun_name': fun_name, 'entity': 'novecs'},
    ).to_ctypes()
    _ldx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _k_py = utils._EngineIntScalarType.to_py(_k)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _k_py),
        mutual_sorder=_m_sorder,
    )
    _x = _x_cls.to_ctypes()
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_k_py,),
        check_contents=False,
    )
    _d = _d_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _lruser = utils._EngineIntScalarType.c_type(0)
    _ruser = None
    _liuser = utils._EngineIntScalarType.c_type(0)
    _iuser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _m,
        _k,
        _noits,
        _tol,
        _dot,
        _dotsub,
        _doth,
        _image,
        _imageh,
        _monit,
        _monith,
        _novecs,
        _x,
        _ldx,
        _d,
        _work,
        _lwork,
        _ruser,
        _lruser,
        _iuser,
        _liuser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py < 0 and _ifmt == 99991:
            _err_class = utils._UserCallbackExit(
                fun_name=fun_name,
                msg_lines=[
                    'User set iflag negative in dot or image.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
                cb_excs=excs,
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: k <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: m < k.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99989:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Not all requested eigenvalues and vectors have been obtained.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99989:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Not all requested eigenvalues and vectors have been obtained.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99989:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Not all requested eigenvalues and vectors have been obtained.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Not all requested eigenvalues and vectors have been obtained.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Convergence of eigenvalue sub-problem occurred.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _m_py = utils._EngineIntScalarType.to_py(_m)
        _noits_py = utils._EngineIntScalarType.to_py(_noits)
        _x_cls.refresh_py(_x)
        _d_cls.refresh_py(_d)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['m', 'noits'],
        )(
            _m_py,
            _noits_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def real_symm_sparse_arnoldi(n, nnz, a, irow, icol, nev, ncv, sigma, monit, option, w, v, resid, data=None, io_manager=None):
    u"""
    Selected eigenvalues and eigenvectors of a real symmetric sparse
    matrix.

    Note: this function uses optional algorithmic parameters, see also:
    ``sparseig.real_symm_init``, ``sparseig.real_symm_iter``,
    ``sparseig.real_symm_proc``, ``sparseig.real_symm_option``,
    ``sparseig.real_symm_monit``.

    For full information please refer to the NAG Library document for
    f02fk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f02/f02fkf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix A.

    nnz : int
        The number of nonzero elements in the lower triangular part of
        the matrix A.

    a : float, shape (`nnz`)
        The array of nonzero elements of the lower triangular part of
        the n by n symmetric matrix A.

    irow : int, shape (`nnz`)
        The row and column indices of the elements supplied in array
        `a`.

        If `irow`[k-1] = i and `icol`[k-1] = j then A_{ij} is stored in
        `a`[k-1]. `irow` does not need to be ordered: an internal sort
        will force the correct ordering.

    icol : int, shape (`nnz`)
        The row and column indices of the elements supplied in array
        `a`.

        If `irow`[k-1] = i and `icol`[k-1] = j then A_{ij} is stored in
        `a`[k-1]. `irow` does not need to be ordered: an internal sort
        will force the correct ordering.

    nev : int
        The number of eigenvalues to be computed.

    ncv : int
        The number of Arnoldi basis vectors to use during the
        computation.

        At present there is no a priori analysis to guide the selection
        of `ncv` relative to `nev`.

        However, it is recommended that `ncv` >= 2*`nev`+1.

        If many problems of the same type are to be solved, you should
        experiment with increasing `ncv` while keeping `nev` fixed for a
        given test problem.

        This will usually decrease the required number of matrix-vector
        operations but it also increases the work and storage required
        to maintain the orthogonal basis vectors.

        The optimal 'cross-over' with respect to computation time is
        problem dependent and must be determined empirically.

    sigma : float
        If the 'Shifted Inverse' mode has been selected then `sigma`
        contains the real shift used; otherwise `sigma` is not
        referenced. This mode can be selected by setting the appropriate
        options in the user-supplied function `option`.

    monit : None or callable monit(ncv, niter, nconv, w, rzest,
        data=None)
        `monit` is used to monitor the progress of
        ``real_symm_sparse_arnoldi``. `monit` may be None if no
        monitoring is actually required. `monit` is called after the
        solution of each eigenvalue sub-problem and also just prior to
        return from ``real_symm_sparse_arnoldi``.

        Parameters
        ~~~~~~~~~~
        ncv : int
            The dimension of the arrays `w` and `rzest`. The number of
            Arnoldi basis vectors used during the computation.

        niter : int
            The number of the current Arnoldi iteration.

        nconv : int
            The number of converged eigenvalues so far.

        w : float, ndarray, shape (`ncv`)
            The first `nconv` elements of this argument contain the
            converged approximate eigenvalues.

        rzest : float, ndarray, shape (`ncv`)
            The first `nconv` elements of this argument contain the Ritz
            estimates (error bounds) on the converged approximate
            eigenvalues.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    option : None or callable option(comm, data=None)
        You can supply non-default options to the Arnoldi eigensolver by
        repeated calls to ``sparseig.real_symm_option`` from within
        `option`. (Please note that it is only necessary to call
        ``sparseig.real_symm_option``; no call to
        ``sparseig.real_symm_init`` is required from within `option`.)
        For example, you can set the mode to 'Shifted Inverse', you can
        increase the 'Iteration Limit' beyond its default and you can
        print varying levels of detail on the iterative process using
        'Print Level'.

        Parameters
        ~~~~~~~~~~
        comm : dict, communication object, modifiable in place
            Communication structure.

            This argument has been initialized to hold the default
            option set.

            If you wish to supply non-default options to the Arnoldi
            eigensolver, this argument may be passed as argument comm in
            a call to ``sparseig.real_symm_option``.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    w : float, shape (`ncv`), modified in place
        On exit: the first `nconv` elements contain the converged
        approximations to the selected eigenvalues.

    v : float, shape (`n`, `ncv`), modified in place
        On exit: contains the eigenvectors associated with the
        eigenvalue lambda_i, for i = 1,2,...,`nconv` (stored in `w`).
        For eigenvalue, lambda_j, the corresponding eigenvector is
        stored in `v`[i-1,j-1], for i = 1,2,...,n.

    resid : float, shape (`nev`), modified in place
        On exit: the residual ||Aw_i-lambda_i w_i||_2 for the estimates
        to the eigenpair lambda_i and w_i is returned in `resid`[i-1],
        for i = 1,2,...,`nconv`.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nconv : int
        The number of converged approximations to the selected
        eigenvalues. On successful exit, this will normally be `nev`.

    Other Parameters
    ----------------
    'Advisory' : int
        Default = advisory message unit number

        If the option 'List' is set then option specifications are
        listed in a 'List' file by setting the option to a file
        identification (unit) number associated with advisory messages
        (see :class:`~naginterfaces.base.utils.FileObjManager` and the
        ``FileObjManager`` method
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`).

    'Defaults' : valueless
        This special keyword may be used to reset all options to their
        default values.

    'Iteration Limit' : int
        Default = 300

        The limit on the number of Lanczos iterations that can be
        performed before ``sparseig.real_symm_iter`` exits.
        If not all requested eigenvalues have converged to within
        'Tolerance' and the number of Lanczos iterations has reached
        this limit then ``sparseig.real_symm_iter`` exits with an error;
        ``sparseig.real_symm_proc`` can still be called subsequently to
        return the number of converged eigenvalues, the converged
        eigenvalues and, if requested, the corresponding eigenvectors.

    'Largest Magnitude' : valueless
        Default

        The Lanczos iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``sparseig.real_symm_iter`` to compute the
        eigenvalues of largest magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Algebraic' part, 'Smallest Magnitude', or 'Smallest Algebraic'
        part; or eigenvalues which are from 'Both Ends' of the algebraic
        spectrum.

    'Both Ends' : valueless
        The Lanczos iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``sparseig.real_symm_iter`` to compute the
        eigenvalues of largest magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Algebraic' part, 'Smallest Magnitude', or 'Smallest Algebraic'
        part; or eigenvalues which are from 'Both Ends' of the algebraic
        spectrum.

    'Largest Algebraic' : valueless
        The Lanczos iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``sparseig.real_symm_iter`` to compute the
        eigenvalues of largest magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Algebraic' part, 'Smallest Magnitude', or 'Smallest Algebraic'
        part; or eigenvalues which are from 'Both Ends' of the algebraic
        spectrum.

    'Smallest Algebraic' : valueless
        The Lanczos iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``sparseig.real_symm_iter`` to compute the
        eigenvalues of largest magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Algebraic' part, 'Smallest Magnitude', or 'Smallest Algebraic'
        part; or eigenvalues which are from 'Both Ends' of the algebraic
        spectrum.

    'Smallest Magnitude' : valueless
        The Lanczos iterative method converges on a number of
        eigenvalues with given properties.
        The default is for ``sparseig.real_symm_iter`` to compute the
        eigenvalues of largest magnitude using 'Largest Magnitude'.
        Alternatively, eigenvalues may be chosen which have 'Largest
        Algebraic' part, 'Smallest Magnitude', or 'Smallest Algebraic'
        part; or eigenvalues which are from 'Both Ends' of the algebraic
        spectrum.

    'Nolist' : valueless
        Default

        Option 'List' enables printing of each option specification as
        it is supplied. 'Nolist' suppresses this printing.

    'List' : valueless
        Option 'List' enables printing of each option specification as
        it is supplied. 'Nolist' suppresses this printing.

    'Monitoring' : int
        Default = -1

        If i > 0, monitoring information is output to unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`)
        i during the solution of each problem; this may be the same as
        the 'Advisory' unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`).
        The type of information produced is dependent on the value of
        'Print Level', see the description of the option 'Print Level'
        for details of the information produced.
        Please see the ``FileObjManager`` method
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`
        to associate a file with a given unit number (see
        :meth:`~naginterfaces.base.utils.FileObjManager.unit_from_fileobj`).

    'Print Level' : int
        Default = 0

        This controls the amount of printing produced by
        ``real_symm_sparse_arnoldi`` as follows.

        [table omitted]

        Note that setting 'Print Level' >= 30 can result in very lengthy
        'Monitoring' output.

    'Regular' : valueless
        Default

        These options define the computational mode which in turn
        defines the form of operation OP(x) to be performed.

        [table omitted]

    'Regular Inverse' : valueless
        These options define the computational mode which in turn
        defines the form of operation OP(x) to be performed.

        [table omitted]

    'Shifted Inverse' : valueless
        These options define the computational mode which in turn
        defines the form of operation OP(x) to be performed.

        [table omitted]

    'Tolerance' : float
        Default = epsilon

        An approximate eigenvalue has deemed to have converged when the
        corresponding Ritz estimate is within 'Tolerance' relative to
        the magnitude of the eigenvalue.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 2)
            On entry, `nnz` = *<value>*.

            Constraint: `nnz` > 0.

        (`errno` 2)
            On entry, `nnz` = *<value>* and `n` = *<value>*.

            Constraint: `nnz` <= `n`*(`n`+1)/2.

        (`errno` 4)
            On entry, for i = *<value>*, `irow`[i-1] = *<value>*.

            Constraint: 1 <= `irow`[i-1] <= `n`.

        (`errno` 5)
            On entry, for i = *<value>*, `icol`[i-1] = *<value>*,
            `irow`[i-1] = *<value>*.

            Constraint: 1 <= `icol`[i-1] <= `irow`[i-1].

        (`errno` 6)
            On entry, `nev` = *<value>*.

            Constraint: `nev` > 0.

        (`errno` 6)
            On entry, `nev` = *<value>* and `n` = *<value>*.

            Constraint: `nev` < (`n`-1).

        (`errno` 7)
            On entry, `ncv` = *<value>* and `nev` = *<value>*.

            Constraint: `ncv` > `nev`.

        (`errno` 7)
            On entry, `ncv` = *<value>* and `n` = *<value>*.

            Constraint: `ncv` <= `n`.

        (`errno` 8)
            On entry, the matrix (A-sigma*I) is numerically singular and
            could not be inverted. Try perturbing the value of sigma.

        (`errno` 20)
            The maximum number of iterations, through the option
            'Iteration Limit', has been set to a non-positive value.

        (`errno` 21)
            The option 'Both Ends' has been set but only 1 eigenvalue is
            requested.

        (`errno` 22)
            The maximum number of iterations has been reached.

            The maximum number of iterations = *<value>*.

            The number of converged eigenvalues = *<value>*.

            See the function document for further details.

        (`errno` 30)
            A serious error, code (*<value>*,*<value>*), has occurred in
            an internal call. Check all function calls and array sizes.
            If the call is correct then please contact NAG for
            assistance.

    Warns
    -----
    NagCallbackTerminateWarning
        (`errno` 9)
            User requested termination in `monit`, `istat` = *<value>*.

        (`errno` 10)
            User requested termination in `option`, `istat` = *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.eigen.real_symm_sparse_arnoldi'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    monith = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ncv.
        utils._BYREF_INT, # Input, niter.
        utils._BYREF_INT, # Input, nconv.
        utils._EngineFloat64ArrayType.c_type, # Input, w[ncv].
        utils._EngineFloat64ArrayType.c_type, # Input, rzest[ncv].
        utils._BYREF_INT, # Input/Output, istat.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_monith(_monit, en_data, ncv, niter, nconv, w, rzest, istat, _iuser, _ruser):
        """Helper for monit."""
        if monit is None:
            return
        try:
            _ncv_monith = utils._EngineIntScalarType.to_py(ncv.contents)
            _niter_monith = utils._EngineIntScalarType.to_py(niter.contents)
            _nconv_monith = utils._EngineIntScalarType.to_py(nconv.contents)
            _w_monith = utils._EngineFloat64ArrayType.from_pointer(
                w,
                (_ncv_monith,),
            )
            _rzest_monith = utils._EngineFloat64ArrayType.from_pointer(
                rzest,
                (_ncv_monith,),
            )
            if data is not None:
                monit(
                    _ncv_monith,
                    _niter_monith,
                    _nconv_monith,
                    _w_monith,
                    _rzest_monith,
                    data,
                )
            else:
                monit(
                    _ncv_monith,
                    _niter_monith,
                    _nconv_monith,
                    _w_monith,
                    _rzest_monith,
                )
            utils._unlink_pointer(w)
            utils._unlink_pointer(rzest)
        except:
            excs.append(_sys.exc_info())
            istat.contents.value = -1
            if excs[-1][0] is not utils.UserCallbackTerminate:
                en_data.contents.hlperr = utils._IERR_HELPER

    optionh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, comm[lcomm].
        utils._BYREF_INT, # Input, lcomm.
        utils._BYREF_INT, # Input/Output, istat.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_optionh(_option, en_data, icomm, licomm, comm, lcomm, istat, _iuser, _ruser):
        """Helper for option."""
        if option is None:
            return
        try:
            _comm_optionh = {'icomm': icomm, 'comm': comm}
            if data is not None:
                option(
                    _comm_optionh,
                    data,
                )
            else:
                option(
                    _comm_optionh,
                )
        except:
            excs.append(_sys.exc_info())
            istat.contents.value = -1
            if excs[-1][0] is not utils.UserCallbackTerminate:
                en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('f02fkft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, nnz.
        utils._EngineFloat64ArrayType.c_type, # Input, a[nnz].
        utils._EngineIntArrayType.c_type, # Input, irow[nnz].
        utils._EngineIntArrayType.c_type, # Input, icol[nnz].
        utils._BYREF_INT, # Input, nev.
        utils._BYREF_INT, # Input, ncv.
        utils._BYREF_FLOAT64, # Input, sigma.
        utils._UNUSED_FUNCTYPE,
        monith,
        utils._UNUSED_FUNCTYPE,
        optionh,
        utils._BYREF_INT, # Output, nconv.
        utils._EngineFloat64ArrayType.c_type, # Output, w[ncv].
        utils._EngineFloat64ArrayType.c_type, # Output, v[ldv, ncv].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineFloat64ArrayType.c_type, # Output, resid[nev].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_STRING, # Input, routine_name(len=6).
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of routine_name.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((v, 'v'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nnz = utils._EngineIntScalarType(
        nnz, {'fun_name': fun_name, 'entity': 'nnz'},
    ).to_ctypes()
    _nnz_py = utils._EngineIntScalarType.to_py(_nnz)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_nnz_py,),
    ).to_ctypes()
    _irow = utils._EngineIntArrayType(
        irow, {'fun_name': fun_name, 'entity': 'irow'},
        exp_shape=(_nnz_py,),
    ).to_ctypes()
    _icol = utils._EngineIntArrayType(
        icol, {'fun_name': fun_name, 'entity': 'icol'},
        exp_shape=(_nnz_py,),
    ).to_ctypes()
    _nev = utils._EngineIntScalarType(
        nev, {'fun_name': fun_name, 'entity': 'nev'},
    ).to_ctypes()
    _ncv = utils._EngineIntScalarType(
        ncv, {'fun_name': fun_name, 'entity': 'ncv'},
    ).to_ctypes()
    _sigma = utils._EngineFloat64ScalarType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
    ).to_ctypes()
    utils._check_callable(monit, {'fun_name': fun_name, 'entity': 'monit'}, allow_none=True)
    _monit = utils._UNUSED_CALLBACK
    _monith = utils._cast_callback(
        monith, cb_monith,
        {'fun_name': fun_name, 'entity': 'monit'},
    )
    utils._check_callable(option, {'fun_name': fun_name, 'entity': 'option'}, allow_none=True)
    _option = utils._UNUSED_CALLBACK
    _optionh = utils._cast_callback(
        optionh, cb_optionh,
        {'fun_name': fun_name, 'entity': 'option'},
    )
    _nconv = utils._EngineIntScalarType.empty_ctypes()
    _ncv_py = utils._EngineIntScalarType.to_py(_ncv)
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_ncv_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldv_py = utils._EngineFloat64ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldv = utils._EngineIntScalarType(
        _ldv_py, {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _v_cls = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_n_py, _ncv_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _v = _v_cls.to_ctypes(can_reference_contents=False)
    _nev_py = utils._EngineIntScalarType.to_py(_nev)
    _resid_cls = utils._EngineFloat64ArrayType(
        resid, {'fun_name': fun_name, 'entity': 'resid'},
        exp_shape=(_nev_py,),
        check_contents=False,
    )
    _resid = _resid_cls.to_ctypes(can_reference_contents=False)
    _iuser = None
    _ruser = None
    _routine_name = utils._EngineRoutineName(*b'f02fk ')
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _n,
        _nnz,
        _a,
        _irow,
        _icol,
        _nev,
        _ncv,
        _sigma,
        _monit,
        _monith,
        _option,
        _optionh,
        _nconv,
        _w,
        _v,
        _ldv,
        _resid,
        _iuser,
        _ruser,
        _routine_name,
        _errbuf,
        _ierr,
        _routine_name._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 102:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nnz = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nnz > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 202:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nnz = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nnz <= n*(n+1)/2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 204:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, for i = {:s}, irow[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: 1 <= irow[i-1] <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 105:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, for i = {:s}, icol[i-1] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'irow[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: 1 <= icol[i-1] <= irow[i-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 106:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nev > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 206:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nev = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nev < (n-1).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 107:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncv = {:s} and nev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ncv > nev.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 207:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncv = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ncv <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 108:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the matrix (A-sigma*I) is numerically singular and could',
                    'not be inverted. Try perturbing the value of sigma.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 109:
            _err_class = utils._UserCallbackExit(
                fun_name=fun_name,
                msg_lines=[
                    'User requested termination in monit, istat = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
                cb_excs=excs,
            )
        elif _ierr_py == 10 and _ifmt == 110:
            _err_class = utils._UserCallbackExit(
                fun_name=fun_name,
                msg_lines=[
                    'User requested termination in option, istat = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
                cb_excs=excs,
            )
        elif _ierr_py == 20 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of iterations, through the option',
                    '\'Iteration Limit\', has been set to a non-positive value.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 21 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The option \'Both Ends\' has been set but only 1 eigenvalue is requested.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 22 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of iterations has been reached.',
                    'The maximum number of iterations = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'The number of converged eigenvalues = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'See the function document for further details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 30 and _ifmt == 3001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A serious error, code ({:s},{:s}),'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'has occurred in an internal call.',
                    'Check all function calls and array sizes.',
                    'If the call is correct then please contact NAG for assistance.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _w_cls.refresh_py(_w)
        _v_cls.refresh_py(_v)
        _resid_cls.refresh_py(_resid)
        _nconv_py = utils._EngineIntScalarType.to_py(_nconv)
        return_data = _nconv_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['nconv'],
            )(
                _nconv_py,
            )
    _err_class.manage()
    return return_data

def complex_gen_eigsys(crit, n, a, wl, wu, mest, w, v):
    u"""
    Selected eigenvalues and eigenvectors of complex nonsymmetric matrix
    (Black Box).

    For full information please refer to the NAG Library document for
    f02gc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f02/f02gcf.html

    Parameters
    ----------
    crit : str, length 1
        Indicates the criterion for selecting eigenvalues.

        `crit` = 'M'

          Eigenvalues are selected according to their moduli: w_l <=
          |lambda_i| <= w_u.

        `crit` = 'R'

          Eigenvalues are selected according to their real parts: w_l <=
          Re(lambda_i) <= w_u.

    n : int
        n, the order of the matrix A.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n general matrix A.

        On exit: contains the Hessenberg form of the balanced input
        matrix A' (see [Further Comments]).

    wl : float
        w_l and w_u, the lower and upper bounds on the criterion for the
        selected eigenvalues (see `crit`).

    wu : float
        w_l and w_u, the lower and upper bounds on the criterion for the
        selected eigenvalues (see `crit`).

    mest : int
        `mest` must be an upper bound on m, the number of eigenvalues
        and eigenvectors selected. No eigenvectors are computed if
        `mest` < m.

    w : complex, shape (`n`), modified in place
        On exit: the first `m` elements of `w` hold the selected
        eigenvalues; elements `m`+1 to `n` contain the other
        eigenvalues.

    v : complex, shape (`n`, `mest`), modified in place
        On exit: contains the selected eigenvectors, with the ith column
        holding the eigenvector associated with the eigenvalue lambda_i
        (stored in `w`[i-1]).

    Returns
    -------
    m : int
        m, the number of eigenvalues actually selected.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `mest` = *<value>*.

            Constraint: `mest` >= 1.

        (`errno` 1)
            On entry, `wu` = *<value>* and `wl` = *<value>*.

            Constraint: `wu` > `wl`.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 1)
            On entry, `crit` = *<value>*.

            Constraint: `crit` = 'M' or 'R'.

        (`errno` 2)
            The QR algorithm failed to converge: only *<value>*
            eigenvalues have been computed; no eigenvectors have been
            computed.

        (`errno` 3)
            There are more than `mest` eigenvalues in the specified
            range. `m` (number of eigenvalues in range) = *<value>* and
            `mest` = *<value>*. No eigenvectors have been computed.
            Rerun with second dimension of `v` = `mest` >= `m`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            Inverse iteration failed to compute all the specified
            eigenvectors. The number of eigenvectors which failed to
            converge is *<value>*. The corresponding columns of `v` are
            set to zero.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.eigen.complex_gen_eigsys'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f02gcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, crit(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_FLOAT64, # Input, wl.
        utils._BYREF_FLOAT64, # Input, wu.
        utils._BYREF_INT, # Input, mest.
        utils._BYREF_INT, # Output, m.
        utils._EngineComplex128ArrayType.c_type, # Output, w[n].
        utils._EngineComplex128ArrayType.c_type, # Output, v[ldv, mest].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineComplex128ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, lwork.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rwork.
        utils._EngineIntArrayType.c_type, # Workspace, iwork.
        utils._EngineLogicalArrayType.c_type, # Workspace, bwork.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of crit.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (v, 'v')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _crit = utils._EngineCharScalarType(
        crit, {'fun_name': fun_name, 'entity': 'crit'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _wl = utils._EngineFloat64ScalarType(
        wl, {'fun_name': fun_name, 'entity': 'wl'},
    ).to_ctypes()
    _wu = utils._EngineFloat64ScalarType(
        wu, {'fun_name': fun_name, 'entity': 'wu'},
    ).to_ctypes()
    _mest = utils._EngineIntScalarType(
        mest, {'fun_name': fun_name, 'entity': 'mest'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType.empty_ctypes()
    _w_cls = utils._EngineComplex128ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _ldv = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    _mest_py = utils._EngineIntScalarType.to_py(_mest)
    _v_cls = utils._EngineComplex128ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_n_py, _mest_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _v = _v_cls.to_ctypes(can_reference_contents=False)
    _work = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _lwork = utils._EngineIntScalarType.c_type(0)
    _rwork = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iwork = utils._EngineIntArrayType.empty_ctypes(0)
    _bwork = utils._EngineLogicalArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _crit,
        _n,
        _a,
        _lda,
        _wl,
        _wu,
        _mest,
        _m,
        _w,
        _v,
        _ldv,
        _work,
        _lwork,
        _rwork,
        _iwork,
        _bwork,
        _errbuf,
        _ierr,
        _crit._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mest = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mest >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, wu = {:s} and wl = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: wu > wl.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, crit = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: crit = \'M\' or \'R\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QR algorithm failed to converge:',
                    'only {:s} eigenvalues have been computed;'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'no eigenvectors have been computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'There are more than mest eigenvalues in the specified range.',
                    'm (number of eigenvalues in range) = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'mest = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'No eigenvectors have been computed.',
                    'Rerun with second dimension of v = mest >= m.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99990:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Inverse iteration failed to compute all the specified eigenvectors.',
                    'The number of eigenvectors which failed to converge is {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'The corresponding columns of v are set to zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _w_cls.refresh_py(_w)
        _v_cls.refresh_py(_v)
        _m_py = utils._EngineIntScalarType.to_py(_m)
        return_data = _m_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['m'],
            )(
                _m_py,
            )
    _err_class.manage()
    return return_data

def real_gen_quad(scal, jobvl, jobvr, sense, tol, n, a, b, c, alphar, alphai, beta, vl, vr, s, bevl, bevr):
    u"""
    Solves the quadratic eigenvalue problem for real matrices.

    For full information please refer to the NAG Library document for
    f02jc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f02/f02jcf.html

    Parameters
    ----------
    scal : int
        Determines the form of scaling to be performed on A, B and C.

        `scal` = 0

          No scaling.

        `scal` = 1 (the recommended value)

          Fan, Lin and Van Dooren scaling if (||B||)/(sqrt(||A||*||C||))
          < 10 and no scaling otherwise where ||Z|| is the Frobenius
          norm of Z.

        `scal` = 2

          Fan, Lin and Van Dooren scaling.

        `scal` = 3

          Tropical scaling with largest root.

        `scal` = 4

          Tropical scaling with smallest root.

    jobvl : str, length 1
        If `jobvl` = 'N', do not compute left eigenvectors.

        If `jobvl` = 'V', compute the left eigenvectors.

        If `sense` = 1, 2, 4, 5, 6 or 7, `jobvl` must be set to 'V'.

    jobvr : str, length 1
        If `jobvr` = 'N', do not compute right eigenvectors.

        If `jobvr` = 'V', compute the right eigenvectors.

        If `sense` = 1, 3, 4, 5, 6 or 7, `jobvr` must be set to 'V'.

    sense : int
        Determines whether, or not, condition numbers and backward
        errors are computed.

        `sense` = 0

          Do not compute condition numbers, or backward errors.

        `sense` = 1

          Just compute condition numbers for the eigenvalues.

        `sense` = 2

          Just compute backward errors for the left eigenpairs.

        `sense` = 3

          Just compute backward errors for the right eigenpairs.

        `sense` = 4

          Compute backward errors for the left and right eigenpairs.

        `sense` = 5

          Compute condition numbers for the eigenvalues and backward
          errors for the left eigenpairs.

        `sense` = 6

          Compute condition numbers for the eigenvalues and backward
          errors for the right eigenpairs.

        `sense` = 7

          Compute condition numbers for the eigenvalues and backward
          errors for the left and right eigenpairs.

    tol : float
        `tol` is used as the tolerance for making decisions on rank in
        the deflation procedure. If `tol` is zero on entry then
        n*machine precision is used in place of `tol`, where machine
        precision is as returned by function ``machine.precision``. A
        diagonal element of a triangular matrix, R, is regarded as zero
        if |r_{jj}| <= `tol`*size(X), or n*machine precision*size(X)
        when `tol` is zero, where size(X) is based on the size of the
        absolute values of the elements of the matrix X containing the
        matrix R. See Hammarling, Munro and Tisseur for the motivation.
        If `tol` is -1.0 on entry then no deflation is attempted. The
        recommended value for `tol` is zero.

    n : int
        The order of the matrices A, B and C.

    a : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: `a` is used as internal workspace, but if `jobvl` = 'V'
        or `jobvr` = 'V', `a` is restored on exit.

    b : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix B.

        On exit: `b` is used as internal workspace, but is restored on
        exit.

    c : float, shape (`n`, `n`), modified in place
        On entry: the n by n matrix C.

        On exit: `c` is used as internal workspace, but if `jobvl` = 'V'
        or `jobvr` = 'V', `c` is restored on exit.

    alphar : float, shape (2*`n`), modified in place
        On exit: `alphar`[j-1], for j = 1,2,...,2n, contains the real
        part of alpha_j for the jth eigenvalue pair (alpha_j,beta_j) of
        the quadratic eigenvalue problem.

    alphai : float, shape (2*`n`), modified in place
        On exit: `alphai`[j-1], for j = 1,2,...,2n, contains the
        imaginary part of alpha_j for the jth eigenvalue pair
        (alpha_j,beta_j) of the quadratic eigenvalue problem. If
        `alphai`[j-1] is zero then the jth eigenvalue is real; if
        `alphai`[j-1] is positive then the jth and (j+1)th eigenvalues
        are a complex conjugate pair, with `alphai`[j] negative.

    beta : float, shape (2*`n`), modified in place
        On exit: `beta`[j-1], for j = 1,2,...,2*n, contains the second
        part of the jth eigenvalue pair (alpha_j,beta_j) of the
        quadratic eigenvalue problem, with beta_j >= 0. Infinite
        eigenvalues have beta_j set to zero.

    vl : None or float, shape (`n`, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvl`='V': 2*`n`; otherwise: 0.

        On exit: if `jobvl` = 'V', the left eigenvectors y_j are stored
        one after another in the columns of `vl`, in the same order as
        the corresponding eigenvalues. If the jth eigenvalue is real,
        then y_j = `vl`[:-1,j-1], the jth column of `vl`. If the jth and
        (j+1)th eigenvalues form a complex conjugate pair, then y_j =
        `vl`[:-1,j-1]+i*`vl`[:-1,j] and y_{j+1} =
        `vl`[:-1,j-1]-i*`vl`[:-1,j]. Each eigenvector will be normalized
        with length unity and with the element of largest modulus real
        and positive.

        If `jobvl` = 'N', `vl` is not referenced.

    vr : None or float, shape (`n`, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvr`='V': 2*`n`; otherwise: 0.

        On exit: if `jobvr` = 'V', the right eigenvectors x_j are stored
        one after another in the columns of `vr`, in the same order as
        the corresponding eigenvalues. If the jth eigenvalue is real,
        then x_j = `vr`[:-1,j-1], the jth column of `vr`. If the jth and
        (j+1)th eigenvalues form a complex conjugate pair, then x_j =
        `vr`[:-1,j-1]+i*`vr`[:-1,j] and x_{j+1} =
        `vr`[:-1,j-1]-i*`vr`[:-1,j]. Each eigenvector will be normalized
        with length unity and with the element of largest modulus real
        and positive.

        If `jobvr` = 'N', `vr` is not referenced.

    s : None or float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `sense` in (1, 5, 6, 7): 2*`n`; otherwise: 0.

        On exit: if `sense` = 1, 5, 6 or 7, `s`[j-1] contains the
        condition number estimate for the jth eigenvalue (large
        condition numbers imply that the problem is near one with
        multiple eigenvalues). Infinite condition numbers are returned
        as the largest model double number (``machine.real_largest``).

        If `sense` = 0, 2, 3 or 4, `s` is not referenced.

    bevl : None or float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `sense` in (2, 4, 5, 7): 2*`n`; otherwise: 0.

        On exit: if `sense` = 2, 4, 5 or 7, `bevl`[j-1] contains the
        backward error estimate for the computed left eigenpair
        (lambda_j,y_j).

        If `sense` = 0, 1, 3 or 6, `bevl` is not referenced.

    bevr : None or float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `sense` in (3, 4, 6, 7): 2*`n`; otherwise: 0.

        On exit: if `sense` = 3, 4, 6 or 7, `bevr`[j-1] contains the
        backward error estimate for the computed right eigenpair
        (lambda_j,x_j).

        If `sense` = 0, 1, 2 or 5, `bevr` is not referenced.

    Returns
    -------
    iwarn : int
        This argument will be positive if there are warnings, otherwise
        this argument will be 0.

        If no exception or warning is raised then:

        - if `iwarn` = 1 then one, or both, of the matrices A and C is
          zero. In this case no scaling is performed, even if `scal` >
          0;

        - if `iwarn` = 2 then the matrices A and C are singular, or
          nearly singular, so the problem is potentially ill-posed;

        - if `iwarn` = 3 then both the conditions for `iwarn` = 1 and
          `iwarn` = 2 above, apply. If `iwarn` = 4, ||`b`|| >=
          10sqrt(||A||·||C||) and backward stability cannot be
          guaranteed.

        If `errno` = 2, this argument returns the value of errno from
        ``lapackeig.dgges``.

        If `errno` = 3, this argument returns the value of errno from
        ``lapackeig.dggev``.

    Raises
    ------
    NagValueError
        (`errno` -19)
            On entry, ldvr = *<value>*, `n` = *<value>* and `jobvr` =
            *<value>*.

            Constraint: when `jobvr` = 'V', ldvr >= `n`.

        (`errno` -17)
            On entry, ldvl = *<value>*, `n` = *<value>* and `jobvl` =
            *<value>*.

            Constraint: when `jobvl` = 'V', ldvl >= `n`.

        (`errno` -6)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, `sense` = *<value>*.

            Constraint: `sense` = 0, 1, 2, 3, 4, 5, 6 or 7.

        (`errno` -3)
            On entry, `sense` = *<value>* and `jobvr` = *<value>*.

            Constraint: when `jobvr` = 'N', `sense` = 0 or 2,

            when `jobvr` = 'V', `sense` = 1, 3, 4, 5, 6 or 7.

        (`errno` -3)
            On entry, `jobvr` = *<value>*.

            Constraint: `jobvr` = 'N' or 'V'.

        (`errno` -2)
            On entry, `sense` = *<value>* and `jobvl` = *<value>*.

            Constraint: when `jobvl` = 'N', `sense` = 0 or 3,

            when `jobvl` = 'V', `sense` = 1, 2, 4, 5, 6 or 7.

        (`errno` -2)
            On entry, `jobvl` = *<value>*.

            Constraint: `jobvl` = 'N' or 'V'.

        (`errno` -1)
            On entry, `scal` = *<value>*.

            Constraint: `scal` = 0, 1, 2, 3 or 4.

        (`errno` 1)
            The quadratic matrix polynomial is nonregular (singular).

        (`errno` 2)
            The QZ iteration failed in ``lapackeig.dgges``.

            `iwarn` returns the value of errno returned by
            ``lapackeig.dgges``. This failure is unlikely to happen, but
            if it does, please contact NAG.

        (`errno` 3)
            The QZ iteration failed in ``lapackeig.dggev``.

            `iwarn` returns the value of errno returned by
            ``lapackeig.dggev``. This failure is unlikely to happen, but
            if it does, please contact NAG.
    """

    excs = []
    fun_name = 'naginterfaces.base.eigen.real_gen_quad'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f02jcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, scal.
        utils._BYREF_STRING, # Input, jobvl(len=1).
        utils._BYREF_STRING, # Input, jobvr(len=1).
        utils._BYREF_INT, # Input, sense.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineFloat64ArrayType.c_type, # Output, alphar[2*n].
        utils._EngineFloat64ArrayType.c_type, # Output, alphai[2*n].
        utils._EngineFloat64ArrayType.c_type, # Output, beta[2*n].
        utils._EngineFloat64ArrayType.c_type, # Output, vl[ldvl, *].
        utils._BYREF_INT, # Input, ldvl.
        utils._EngineFloat64ArrayType.c_type, # Output, vr[ldvr, *].
        utils._BYREF_INT, # Input, ldvr.
        utils._EngineFloat64ArrayType.c_type, # Output, s[*].
        utils._EngineFloat64ArrayType.c_type, # Output, bevl[*].
        utils._EngineFloat64ArrayType.c_type, # Output, bevr[*].
        utils._BYREF_INT, # Output, iwarn.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvl.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (c, 'c'), (vl, 'vl'), (vr, 'vr')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _scal = utils._EngineIntScalarType(
        scal, {'fun_name': fun_name, 'entity': 'scal'},
    ).to_ctypes()
    _jobvl = utils._EngineCharScalarType(
        jobvl, {'fun_name': fun_name, 'entity': 'jobvl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvr = utils._EngineCharScalarType(
        jobvr, {'fun_name': fun_name, 'entity': 'jobvr'}, exp_chrlen=1,
    ).to_ctypes()
    _sense = utils._EngineIntScalarType(
        sense, {'fun_name': fun_name, 'entity': 'sense'},
    ).to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb_py = utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldb = utils._EngineIntScalarType(
        _ldb_py, {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _ldc_py = utils._EngineFloat64ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldc = utils._EngineIntScalarType(
        _ldc_py, {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _alphar_cls = utils._EngineFloat64ArrayType(
        alphar, {'fun_name': fun_name, 'entity': 'alphar'},
        exp_shape=(2*_n_py,),
        check_contents=False,
    )
    _alphar = _alphar_cls.to_ctypes(can_reference_contents=False)
    _alphai_cls = utils._EngineFloat64ArrayType(
        alphai, {'fun_name': fun_name, 'entity': 'alphai'},
        exp_shape=(2*_n_py,),
        check_contents=False,
    )
    _alphai = _alphai_cls.to_ctypes(can_reference_contents=False)
    _beta_cls = utils._EngineFloat64ArrayType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
        exp_shape=(2*_n_py,),
        check_contents=False,
    )
    _beta = _beta_cls.to_ctypes(can_reference_contents=False)
    _ldvl_py = utils._EngineFloat64ArrayType.get_nd(vl, {'fun_name': fun_name, 'entity': 'vl'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldvl = utils._EngineIntScalarType(
        _ldvl_py, {'fun_name': fun_name, 'entity': '_ldvl'},
    ).to_ctypes()
    _jobvl_py = utils._EngineCharScalarType.to_py(_jobvl)
    if _jobvl_py not in ('N', 'n'):
        if vl is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vl'},
                'must not be None when jobvl not in (\'N\', \'n\')',
            )
    if _jobvl_py in ('V', 'v'):
        _vl_dim2 = 2*_n_py
    else:
        _vl_dim2 = 0
    exp_shape_vl = (_n_py, _vl_dim2)
    _vl_cls = utils._EngineFloat64ArrayType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
        exp_shape=exp_shape_vl,
        mutual_sorder=_m_sorder,
        check_contents=False,
        allow_none=True,
    )
    _vl = _vl_cls.to_ctypes(can_reference_contents=False)
    _ldvr_py = utils._EngineFloat64ArrayType.get_nd(vr, {'fun_name': fun_name, 'entity': 'vr'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldvr = utils._EngineIntScalarType(
        _ldvr_py, {'fun_name': fun_name, 'entity': '_ldvr'},
    ).to_ctypes()
    _jobvr_py = utils._EngineCharScalarType.to_py(_jobvr)
    if _jobvr_py not in ('N', 'n'):
        if vr is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vr'},
                'must not be None when jobvr not in (\'N\', \'n\')',
            )
    if _jobvr_py in ('V', 'v'):
        _vr_dim2 = 2*_n_py
    else:
        _vr_dim2 = 0
    exp_shape_vr = (_n_py, _vr_dim2)
    _vr_cls = utils._EngineFloat64ArrayType(
        vr, {'fun_name': fun_name, 'entity': 'vr'},
        exp_shape=exp_shape_vr,
        mutual_sorder=_m_sorder,
        check_contents=False,
        allow_none=True,
    )
    _vr = _vr_cls.to_ctypes(can_reference_contents=False)
    _sense_py = utils._EngineIntScalarType.to_py(_sense)
    if _sense_py not in (0, 2, 3, 4):
        if s is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 's'},
                'must not be None when sense not in (0, 2, 3, 4)',
            )
    if _sense_py in (1, 5, 6, 7):
        _s_dim1 = 2*_n_py
    else:
        _s_dim1 = 0
    exp_shape_s = (_s_dim1,)
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=exp_shape_s,
        check_contents=False,
        allow_none=True,
    )
    _s = _s_cls.to_ctypes(can_reference_contents=False)
    if _sense_py not in (0, 1, 3, 6):
        if bevl is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'bevl'},
                'must not be None when sense not in (0, 1, 3, 6)',
            )
    if _sense_py in (2, 4, 5, 7):
        _bevl_dim1 = 2*_n_py
    else:
        _bevl_dim1 = 0
    exp_shape_bevl = (_bevl_dim1,)
    _bevl_cls = utils._EngineFloat64ArrayType(
        bevl, {'fun_name': fun_name, 'entity': 'bevl'},
        exp_shape=exp_shape_bevl,
        check_contents=False,
        allow_none=True,
    )
    _bevl = _bevl_cls.to_ctypes(can_reference_contents=False)
    if _sense_py not in (0, 1, 2, 5):
        if bevr is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'bevr'},
                'must not be None when sense not in (0, 1, 2, 5)',
            )
    if _sense_py in (3, 4, 6, 7):
        _bevr_dim1 = 2*_n_py
    else:
        _bevr_dim1 = 0
    exp_shape_bevr = (_bevr_dim1,)
    _bevr_cls = utils._EngineFloat64ArrayType(
        bevr, {'fun_name': fun_name, 'entity': 'bevr'},
        exp_shape=exp_shape_bevr,
        check_contents=False,
        allow_none=True,
    )
    _bevr = _bevr_cls.to_ctypes(can_reference_contents=False)
    _iwarn = utils._EngineIntScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _scal,
        _jobvl,
        _jobvr,
        _sense,
        _tol,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _c,
        _ldc,
        _alphar,
        _alphai,
        _beta,
        _vl,
        _ldvl,
        _vr,
        _ldvr,
        _s,
        _bevl,
        _bevr,
        _iwarn,
        _errbuf,
        _ierr,
        _jobvl._length_,
        _jobvr._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -19 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ldvr = {:s}, n = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and jobvr = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[3]),
                    ),
                    'Constraint: when jobvr = \'V\', ldvr >= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -17 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ldvl = {:s}, n = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and jobvl = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[3]),
                    ),
                    'Constraint: when jobvl = \'V\', ldvl >= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -6 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -4 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sense = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: sense = 0, 1, 2, 3, 4, 5, 6 or 7.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -3 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sense = {:s} and jobvr = \'{:s}\'.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_STR.format(_eb_data[2]),
                    ),
                    'Constraint: when jobvr = \'N\', sense = 0 or 2,',
                    'when jobvr = \'V\', sense = 1, 3, 4, 5, 6 or 7.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, jobvr = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: jobvr = \'N\' or \'V\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sense = {:s} and jobvl = \'{:s}\'.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_STR.format(_eb_data[2]),
                    ),
                    'Constraint: when jobvl = \'N\', sense = 0 or 3,',
                    'when jobvl = \'V\', sense = 1, 2, 4, 5, 6 or 7.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, jobvl = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: jobvl = \'N\' or \'V\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, scal = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: scal = 0, 1, 2, 3 or 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The quadratic matrix polynomial is nonregular (singular).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration failed in lapackeig.dgges.',
                    'iwarn returns the value of errno returned',
                    'by lapackeig.dgges. This failure is unlikely to happen, but if it does,',
                    'please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration failed in lapackeig.dggev.',
                    'iwarn returns the value of errno returned',
                    'by lapackeig.dggev. This failure is unlikely to happen, but if it does,',
                    'please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _c_cls.refresh_py(_c)
        _alphar_cls.refresh_py(_alphar)
        _alphai_cls.refresh_py(_alphai)
        _beta_cls.refresh_py(_beta)
        if vl is not None:
            _vl_cls.refresh_py(_vl)
        if vr is not None:
            _vr_cls.refresh_py(_vr)
        if s is not None:
            _s_cls.refresh_py(_s)
        if bevl is not None:
            _bevl_cls.refresh_py(_bevl)
        if bevr is not None:
            _bevr_cls.refresh_py(_bevr)
        _iwarn_py = utils._EngineIntScalarType.to_py(_iwarn)
        return_data = _iwarn_py
    _err_class.manage()
    return return_data

def complex_gen_quad(scal, jobvl, jobvr, sense, tol, n, a, b, c, alpha, beta, vl, vr, s, bevl, bevr):
    u"""
    Solves the quadratic eigenvalue problem for complex matrices.

    For full information please refer to the NAG Library document for
    f02jq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f02/f02jqf.html

    Parameters
    ----------
    scal : int
        Determines the form of scaling to be performed on A, B and C.

        `scal` = 0

          No scaling.

        `scal` = 1 (the recommended value)

          Fan, Lin and Van Dooren scaling if (||B||)/(sqrt(||A||*||C||))
          < 10 and no scaling otherwise where ||Z|| is the Frobenius
          norm of Z.

        `scal` = 2

          Fan, Lin and Van Dooren scaling.

        `scal` = 3

          Tropical scaling with largest root.

        `scal` = 4

          Tropical scaling with smallest root.

    jobvl : str, length 1
        If `jobvl` = 'N', do not compute left eigenvectors.

        If `jobvl` = 'V', compute the left eigenvectors.

        If `sense` = 1, 2, 4, 5, 6 or 7, `jobvl` must be set to 'V'.

    jobvr : str, length 1
        If `jobvr` = 'N', do not compute right eigenvectors.

        If `jobvr` = 'V', compute the right eigenvectors.

        If `sense` = 1, 3, 4, 5, 6 or 7, `jobvr` must be set to 'V'.

    sense : int
        Determines whether, or not, condition numbers and backward
        errors are computed.

        `sense` = 0

          Do not compute condition numbers, or backward errors.

        `sense` = 1

          Just compute condition numbers for the eigenvalues.

        `sense` = 2

          Just compute backward errors for the left eigenpairs.

        `sense` = 3

          Just compute backward errors for the right eigenpairs.

        `sense` = 4

          Compute backward errors for the left and right eigenpairs.

        `sense` = 5

          Compute condition numbers for the eigenvalues and backward
          errors for the left eigenpairs.

        `sense` = 6

          Compute condition numbers for the eigenvalues and backward
          errors for the right eigenpairs.

        `sense` = 7

          Compute condition numbers for the eigenvalues and backward
          errors for the left and right eigenpairs.

    tol : float
        `tol` is used as the tolerance for making decisions on rank in
        the deflation procedure. If `tol` is zero on entry then
        n*machine precision is used in place of `tol`, where machine
        precision is as returned by function ``machine.precision``. A
        diagonal element of a triangular matrix, R, is regarded as zero
        if |r_{jj}| <= `tol`*size(X), or n*machine precision*size(X)
        when `tol` is zero, where size(X) is based on the size of the
        absolute values of the elements of the matrix X containing the
        matrix R. See Hammarling, Munro and Tisseur for the motivation.
        If `tol` is -1.0 on entry then no deflation is attempted. The
        recommended value for `tol` is zero.

    n : int
        The order of the matrices A, B and C.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix A.

        On exit: `a` is used as internal workspace, but if `jobvl` = 'V'
        or `jobvr` = 'V', `a` is restored on exit.

    b : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix B.

        On exit: `b` is used as internal workspace, but is restored on
        exit.

    c : complex, shape (`n`, `n`), modified in place
        On entry: the n by n matrix C.

        On exit: `c` is used as internal workspace, but if `jobvl` = 'V'
        or `jobvr` = 'V', `c` is restored on exit.

    alpha : complex, shape (2*`n`), modified in place
        On exit: `alpha`[j-1], for j = 1,2,...,2n, contains the first
        part of the the jth eigenvalue pair (alpha_j,beta_j) of the
        quadratic eigenvalue problem.

    beta : complex, shape (2*`n`), modified in place
        On exit: `beta`[j-1], for j = 1,2,...,2*n, contains the second
        part of the jth eigenvalue pair (alpha_j,beta_j) of the
        quadratic eigenvalue problem. Although `beta` is declared
        complex, it is actually real and non-negative. Infinite
        eigenvalues have beta_j set to zero.

    vl : None or complex, shape (`n`, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvl`='V': 2*`n`; otherwise: 0.

        On exit: if `jobvl` = 'V', the left eigenvectors y_j are stored
        one after another in the columns of `vl`, in the same order as
        the corresponding eigenvalues. Each eigenvector will be
        normalized with length unity and with the element of largest
        modulus real and positive.

        If `jobvl` = 'N', `vl` is not referenced.

    vr : None or complex, shape (`n`, :), modified in place
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobvr`='V': 2*`n`; otherwise: 0.

        On exit: if `jobvr` = 'V', the right eigenvectors x_j are stored
        one after another in the columns of `vr`, in the same order as
        the corresponding eigenvalues. Each eigenvector will be
        normalized with length unity and with the element of largest
        modulus real and positive.

        If `jobvr` = 'N', `vr` is not referenced.

    s : None or float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `sense` in (1, 5, 6, 7): 2*`n`; otherwise: 0.

        On exit: if `sense` = 1, 5, 6 or 7, `s`[j-1] contains the
        condition number estimate for the jth eigenvalue (large
        condition numbers imply that the problem is near one with
        multiple eigenvalues). Infinite condition numbers are returned
        as the largest model real number (``machine.real_largest``).

        If `sense` = 0, 2, 3 or 4, `s` is not referenced.

    bevl : None or float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `sense` in (2, 4, 5, 7): 2*`n`; otherwise: 0.

        On exit: if `sense` = 2, 4, 5 or 7, `bevl`[j-1] contains the
        backward error estimate for the computed left eigenpair
        (lambda_j,y_j).

        If `sense` = 0, 1, 3 or 6, `bevl` is not referenced.

    bevr : None or float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `sense` in (3, 4, 6, 7): 2*`n`; otherwise: 0.

        On exit: if `sense` = 3, 4, 6 or 7, `bevr`[j-1] contains the
        backward error estimate for the computed right eigenpair
        (lambda_j,x_j).

        If `sense` = 0, 1, 2 or 5, `bevr` is not referenced.

    Returns
    -------
    iwarn : int
        This argument will be positive if there are warnings, otherwise
        this argument will be 0.

        If no exception or warning is raised then:

        - if `iwarn` = 1 then one, or both, of the matrices A and C is
          zero. In this case no scaling is performed, even if `scal` >
          0;

        - if `iwarn` = 2 then the matrices A and C are singular, or
          nearly singular, so the problem is potentially ill-posed;

        - if `iwarn` = 3 then both the conditions for `iwarn` = 1 and
          `iwarn` = 2 above, apply. If `iwarn` = 4, ||`b`|| >=
          10sqrt(||A||·||C||) and backward stability cannot be
          guaranteed.

        If `errno` = 2, this argument returns the value of errno from
        ``lapackeig.zgges``.

        If `errno` = 3, this argument returns the value of errno from
        ``lapackeig.zggev``.

    Raises
    ------
    NagValueError
        (`errno` -18)
            On entry, ldvr = *<value>*, `n` = *<value>* and `jobvr` =
            *<value>*.

            Constraint: when `jobvr` = 'V', ldvr >= `n`.

        (`errno` -16)
            On entry, ldvl = *<value>*, `n` = *<value>* and `jobvl` =
            *<value>*.

            Constraint: when `jobvl` = 'V', ldvl >= `n`.

        (`errno` -6)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, `sense` = *<value>*.

            Constraint: `sense` = 0, 1, 2, 3, 4, 5, 6 or 7.

        (`errno` -3)
            On entry, `sense` = *<value>* and `jobvr` = *<value>*.

            Constraint: when `jobvr` = 'N', `sense` = 0 or 2,

            when `jobvr` = 'V', `sense` = 1, 3, 4, 5, 6 or 7.

        (`errno` -3)
            On entry, `jobvr` = *<value>*.

            Constraint: `jobvr` = 'N' or 'V'.

        (`errno` -2)
            On entry, `sense` = *<value>* and `jobvl` = *<value>*.

            Constraint: when `jobvl` = 'N', `sense` = 0 or 3,

            when `jobvl` = 'V', `sense` = 1, 2, 4, 5, 6 or 7.

        (`errno` -2)
            On entry, `jobvl` = *<value>*.

            Constraint: `jobvl` = 'N' or 'V'.

        (`errno` -1)
            On entry, `scal` = *<value>*.

            Constraint: `scal` = 0, 1, 2, 3 or 4.

        (`errno` 1)
            The quadratic matrix polynomial is nonregular (singular).

        (`errno` 2)
            The QZ iteration failed in ``lapackeig.zgges``.

            `iwarn` returns the value of errno returned by
            ``lapackeig.zgges``. This failure is unlikely to happen, but
            if it does, please contact NAG.

        (`errno` 3)
            The QZ iteration failed in ``lapackeig.zggev``.

            `iwarn` returns the value of errno returned by
            ``lapackeig.zggev``. This failure is unlikely to happen, but
            if it does, please contact NAG.
    """

    excs = []
    fun_name = 'naginterfaces.base.eigen.complex_gen_quad'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f02jqft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, scal.
        utils._BYREF_STRING, # Input, jobvl(len=1).
        utils._BYREF_STRING, # Input, jobvr(len=1).
        utils._BYREF_INT, # Input, sense.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineComplex128ArrayType.c_type, # Output, alpha[2*n].
        utils._EngineComplex128ArrayType.c_type, # Output, beta[2*n].
        utils._EngineComplex128ArrayType.c_type, # Output, vl[ldvl, *].
        utils._BYREF_INT, # Input, ldvl.
        utils._EngineComplex128ArrayType.c_type, # Output, vr[ldvr, *].
        utils._BYREF_INT, # Input, ldvr.
        utils._EngineFloat64ArrayType.c_type, # Output, s[*].
        utils._EngineFloat64ArrayType.c_type, # Output, bevl[*].
        utils._EngineFloat64ArrayType.c_type, # Output, bevr[*].
        utils._BYREF_INT, # Output, iwarn.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvl.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of jobvr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (c, 'c'), (vl, 'vl'), (vr, 'vr')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _scal = utils._EngineIntScalarType(
        scal, {'fun_name': fun_name, 'entity': 'scal'},
    ).to_ctypes()
    _jobvl = utils._EngineCharScalarType(
        jobvl, {'fun_name': fun_name, 'entity': 'jobvl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvr = utils._EngineCharScalarType(
        jobvr, {'fun_name': fun_name, 'entity': 'jobvr'}, exp_chrlen=1,
    ).to_ctypes()
    _sense = utils._EngineIntScalarType(
        sense, {'fun_name': fun_name, 'entity': 'sense'},
    ).to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ldb_py = utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldb = utils._EngineIntScalarType(
        _ldb_py, {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _ldc_py = utils._EngineComplex128ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldc = utils._EngineIntScalarType(
        _ldc_py, {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _c_cls = utils._EngineComplex128ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _c = _c_cls.to_ctypes()
    _alpha_cls = utils._EngineComplex128ArrayType(
        alpha, {'fun_name': fun_name, 'entity': 'alpha'},
        exp_shape=(2*_n_py,),
        check_contents=False,
    )
    _alpha = _alpha_cls.to_ctypes(can_reference_contents=False)
    _beta_cls = utils._EngineComplex128ArrayType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
        exp_shape=(2*_n_py,),
        check_contents=False,
    )
    _beta = _beta_cls.to_ctypes(can_reference_contents=False)
    _ldvl_py = utils._EngineComplex128ArrayType.get_nd(vl, {'fun_name': fun_name, 'entity': 'vl'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldvl = utils._EngineIntScalarType(
        _ldvl_py, {'fun_name': fun_name, 'entity': '_ldvl'},
    ).to_ctypes()
    _jobvl_py = utils._EngineCharScalarType.to_py(_jobvl)
    if _jobvl_py not in ('N', 'n'):
        if vl is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vl'},
                'must not be None when jobvl not in (\'N\', \'n\')',
            )
    if _jobvl_py in ('V', 'v'):
        _vl_dim2 = 2*_n_py
    else:
        _vl_dim2 = 0
    exp_shape_vl = (_n_py, _vl_dim2)
    _vl_cls = utils._EngineComplex128ArrayType(
        vl, {'fun_name': fun_name, 'entity': 'vl'},
        exp_shape=exp_shape_vl,
        mutual_sorder=_m_sorder,
        check_contents=False,
        allow_none=True,
    )
    _vl = _vl_cls.to_ctypes(can_reference_contents=False)
    _ldvr_py = utils._EngineComplex128ArrayType.get_nd(vr, {'fun_name': fun_name, 'entity': 'vr'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldvr = utils._EngineIntScalarType(
        _ldvr_py, {'fun_name': fun_name, 'entity': '_ldvr'},
    ).to_ctypes()
    _jobvr_py = utils._EngineCharScalarType.to_py(_jobvr)
    if _jobvr_py not in ('N', 'n'):
        if vr is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vr'},
                'must not be None when jobvr not in (\'N\', \'n\')',
            )
    if _jobvr_py in ('V', 'v'):
        _vr_dim2 = 2*_n_py
    else:
        _vr_dim2 = 0
    exp_shape_vr = (_n_py, _vr_dim2)
    _vr_cls = utils._EngineComplex128ArrayType(
        vr, {'fun_name': fun_name, 'entity': 'vr'},
        exp_shape=exp_shape_vr,
        mutual_sorder=_m_sorder,
        check_contents=False,
        allow_none=True,
    )
    _vr = _vr_cls.to_ctypes(can_reference_contents=False)
    _sense_py = utils._EngineIntScalarType.to_py(_sense)
    if _sense_py not in (0, 2, 3, 4):
        if s is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 's'},
                'must not be None when sense not in (0, 2, 3, 4)',
            )
    if _sense_py in (1, 5, 6, 7):
        _s_dim1 = 2*_n_py
    else:
        _s_dim1 = 0
    exp_shape_s = (_s_dim1,)
    _s_cls = utils._EngineFloat64ArrayType(
        s, {'fun_name': fun_name, 'entity': 's'},
        exp_shape=exp_shape_s,
        check_contents=False,
        allow_none=True,
    )
    _s = _s_cls.to_ctypes(can_reference_contents=False)
    if _sense_py not in (0, 1, 3, 6):
        if bevl is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'bevl'},
                'must not be None when sense not in (0, 1, 3, 6)',
            )
    if _sense_py in (2, 4, 5, 7):
        _bevl_dim1 = 2*_n_py
    else:
        _bevl_dim1 = 0
    exp_shape_bevl = (_bevl_dim1,)
    _bevl_cls = utils._EngineFloat64ArrayType(
        bevl, {'fun_name': fun_name, 'entity': 'bevl'},
        exp_shape=exp_shape_bevl,
        check_contents=False,
        allow_none=True,
    )
    _bevl = _bevl_cls.to_ctypes(can_reference_contents=False)
    if _sense_py not in (0, 1, 2, 5):
        if bevr is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'bevr'},
                'must not be None when sense not in (0, 1, 2, 5)',
            )
    if _sense_py in (3, 4, 6, 7):
        _bevr_dim1 = 2*_n_py
    else:
        _bevr_dim1 = 0
    exp_shape_bevr = (_bevr_dim1,)
    _bevr_cls = utils._EngineFloat64ArrayType(
        bevr, {'fun_name': fun_name, 'entity': 'bevr'},
        exp_shape=exp_shape_bevr,
        check_contents=False,
        allow_none=True,
    )
    _bevr = _bevr_cls.to_ctypes(can_reference_contents=False)
    _iwarn = utils._EngineIntScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _scal,
        _jobvl,
        _jobvr,
        _sense,
        _tol,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _c,
        _ldc,
        _alpha,
        _beta,
        _vl,
        _ldvl,
        _vr,
        _ldvr,
        _s,
        _bevl,
        _bevr,
        _iwarn,
        _errbuf,
        _ierr,
        _jobvl._length_,
        _jobvr._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -18 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ldvr = {:s}, n = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and jobvr = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[3]),
                    ),
                    'Constraint: when jobvr = \'V\', ldvr >= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -16 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ldvl = {:s}, n = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'and jobvl = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[3]),
                    ),
                    'Constraint: when jobvl = \'V\', ldvl >= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -6 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -4 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sense = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: sense = 0, 1, 2, 3, 4, 5, 6 or 7.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -3 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sense = {:s} and jobvr = \'{:s}\'.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_STR.format(_eb_data[2]),
                    ),
                    'Constraint: when jobvr = \'N\', sense = 0 or 2,',
                    'when jobvr = \'V\', sense = 1, 3, 4, 5, 6 or 7.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, jobvr = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: jobvr = \'N\' or \'V\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sense = {:s} and jobvl = \'{:s}\'.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_STR.format(_eb_data[2]),
                    ),
                    'Constraint: when jobvl = \'N\', sense = 0 or 3,',
                    'when jobvl = \'V\', sense = 1, 2, 4, 5, 6 or 7.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, jobvl = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: jobvl = \'N\' or \'V\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, scal = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: scal = 0, 1, 2, 3 or 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The quadratic matrix polynomial is nonregular (singular).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration failed in lapackeig.zgges.',
                    'iwarn returns the value of errno returned',
                    'by lapackeig.zgges. This failure is unlikely to happen, but if it does,',
                    'please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QZ iteration failed in lapackeig.zggev.',
                    'iwarn returns the value of errno returned',
                    'by lapackeig.zggev. This failure is unlikely to happen,',
                    'but if it does, please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _c_cls.refresh_py(_c)
        _alpha_cls.refresh_py(_alpha)
        _beta_cls.refresh_py(_beta)
        if vl is not None:
            _vl_cls.refresh_py(_vl)
        if vr is not None:
            _vr_cls.refresh_py(_vr)
        if s is not None:
            _s_cls.refresh_py(_s)
        if bevl is not None:
            _bevl_cls.refresh_py(_bevl)
        if bevr is not None:
            _bevr_cls.refresh_py(_bevr)
        _iwarn_py = utils._EngineIntScalarType.to_py(_iwarn)
        return_data = _iwarn_py
    _err_class.manage()
    return return_data

def real_gen_partialsvd(m, n, k, ncv, av, sigma, u, v, resid, data=None, io_manager=None):
    u"""
    Computes leading terms in the singular value decomposition of a real
    general matrix; also computes corresponding left and right singular
    vectors.

    For full information please refer to the NAG Library document for
    f02wg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f02/f02wgf.html

    Parameters
    ----------
    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix A.

    k : int
        k, the number of singular values to be computed.

    ncv : int
        This is the number of Lanczos basis vectors to use during the
        computation of the largest eigenvalues of A^TA (m >= n) or AA^T
        (m < n).

        At present there is no a priori analysis to guide the selection
        of `ncv` relative to `k`.

        However, it is recommended that `ncv` >= 2*`k`+1.

        If many problems of the same type are to be solved, you should
        experiment with varying `ncv` while keeping `k` fixed for a
        given test problem.

        This will usually decrease the required number of matrix-vector
        operations but it also increases the internal storage required
        to maintain the orthogonal basis vectors.

        The optimal 'cross-over' with respect to CPU time is problem
        dependent and must be determined empirically.

    av : callable iflag = av(iflag, m, n, x, ax, data=None)
        `av` must return the vector result of the matrix-vector product
        Ax or A^Tx, as indicated by the input value of `iflag`, for the
        given vector x.

        Parameters
        ~~~~~~~~~~
        iflag : int
            If `iflag` = 1, `ax` must return the m-vector result of the
            matrix-vector product Ax.

            If `iflag` = 2, `ax` must return the n-vector result of the
            matrix-vector product A^Tx.

        m : int
            The number of rows of the matrix A.

        n : int
            The number of columns of the matrix A.

        x : float, ndarray, shape (:)
            The vector to be pre-multiplied by the matrix A or A^T.

        ax : float, ndarray, shape (:), to be modified in place
            On exit: if `iflag` = 1, contains the m-vector result of the
            matrix-vector product Ax.

            If `iflag` = 2, contains the n-vector result of the
            matrix-vector product A^Tx.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        iflag : int
            May be used as a flag to indicate a failure in the
            computation of Ax or A^Tx. If `iflag` is negative on exit
            from `av`, ``real_gen_partialsvd`` will exit immediately
            with errno set to `iflag`.

    sigma : float, shape (`ncv`), modified in place
        On exit: the `nconv` converged singular values are stored in the
        first `nconv` elements of `sigma`.

    u : float, shape (`m`, `ncv`), modified in place
        On exit: the left singular vectors corresponding to the singular
        values stored in `sigma`.

        The ith element of the jth left singular vector u_j is stored in
        `u`[i-1,j-1], for j = 1,2,...,`nconv`, for i = 1,2,...,m.

    v : float, shape (`n`, `ncv`), modified in place
        On exit: the right singular vectors corresponding to the
        singular values stored in `sigma`.

        The ith element of the jth right singular vector v_j is stored
        in `v`[i-1,j-1], for j = 1,2,...,`nconv`, for i = 1,2,...,n.

    resid : float, shape (`ncv`), modified in place
        On exit: the residual ||Av_j-sigma_j u_j||, for m >= n, or
        ||A^Tu_j-sigma_j v_j||, for m < n, for each of the converged
        singular values sigma_j and corresponding left and right
        singular vectors u_j and v_j.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    nconv : int
        The number of converged singular values found.

    Raises
    ------
    NagValueError
        (`errno` i < 0)
            On output from user-defined function `av`, `iflag` was set
            to a negative value, `iflag` = *<value>*.

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 0.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 3)
            On entry, `k` = *<value>*.

            Constraint: `k` > 0.

        (`errno` 4)
            On entry, `k` = *<value>*, `ncv` = *<value>*, `m` =
            *<value>* and `n` = *<value>*.

            Constraint: `k` < `ncv` <= min(`m`, `n`).

        (`errno` 8)
            The maximum number of iterations has been reached. The
            maximum number of iterations = *<value>*. The number of
            converged eigenvalues = *<value>*.

        (`errno` 9)
            No shifts could be applied during a cycle of the implicitly
            restarted Lanczos iteration.

        (`errno` 10)
            Could not build a full Lanczos factorization.

        (`errno` 11)
            The number of eigenvalues found to sufficient accuracy is
            zero.

        (`errno` 20)
            An error occurred during an internal call. Consider
            increasing the size of `ncv` relative to `k`.
    """

    excs = []
    fun_name = 'naginterfaces.base.eigen.real_gen_partialsvd'

    utils.FileObjManager._check_type_compat(
        io_manager, {'fun_name': fun_name, 'entity': 'io_manager'}
    )

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    def cb_print_rech_py(_, en_data, nout, rec, ifail, rec_len):
        # pylint: disable=missing-docstring
        try:
            utils._py_print_rec(
                en_data, utils._EngineIntScalarType.to_py(nout.contents), rec[:rec_len], io_manager, excs, ifail.contents,
                fun_name=fun_name,
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_av_name = fun_name + ':av'

    avh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input/Output, iflag.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[*].
        utils._EngineFloat64ArrayType.c_type, # Output, ax[*].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_avh(_av, en_data, iflag, m, n, x, ax, _iuser, _ruser):
        """Helper for av."""
        try:
            _iflag_avh = utils._EngineIntScalarType.to_py(iflag.contents)
            _m_avh = utils._EngineIntScalarType.to_py(m.contents)
            _n_avh = utils._EngineIntScalarType.to_py(n.contents)
            if _iflag_avh == 1:
                _x_avh_dim1 = _n_avh
            elif _iflag_avh == 2:
                _x_avh_dim1 = _m_avh
            else:
                _x_avh_dim1 = 0
            exp_shape_x = (_x_avh_dim1,)
            _x_avh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                exp_shape_x,
            )
            if _iflag_avh == 1:
                _ax_avh_dim1 = _m_avh
            elif _iflag_avh == 2:
                _ax_avh_dim1 = _n_avh
            else:
                _ax_avh_dim1 = 0
            exp_shape_ax = (_ax_avh_dim1,)
            _ax_avh = utils._EngineFloat64ArrayType.from_pointer(
                ax,
                exp_shape_ax,
                can_reference_contents=False,
            )
            if data is not None:
                _iflag_avh = av(
                    _iflag_avh,
                    _m_avh,
                    _n_avh,
                    _x_avh,
                    _ax_avh,
                    data,
                )
            else:
                _iflag_avh = av(
                    _iflag_avh,
                    _m_avh,
                    _n_avh,
                    _x_avh,
                    _ax_avh,
                )
            utils._EngineIntScalarType(
                _iflag_avh, {'fun_name': cb_av_name, 'returned': 'iflag'},
            ).refresh_ctypes_buffer(iflag)
            utils._unlink_pointer(x)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _ax_avh, {'fun_name': cb_av_name, 'returned': 'ax'}, ax,
            )
            utils._unlink_pointer(ax)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('f02wgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._PRINT_REC_FUNCTYPE,
        utils._PRINT_RECH_FUNCTYPE,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._BYREF_INT, # Input, ncv.
        utils._UNUSED_FUNCTYPE,
        avh,
        utils._BYREF_INT, # Output, nconv.
        utils._EngineFloat64ArrayType.c_type, # Output, sigma[ncv].
        utils._EngineFloat64ArrayType.c_type, # Output, u[ldu, ncv].
        utils._BYREF_INT, # Input, ldu.
        utils._EngineFloat64ArrayType.c_type, # Output, v[ldv, ncv].
        utils._BYREF_INT, # Input, ldv.
        utils._EngineFloat64ArrayType.c_type, # Output, resid[ncv].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((u, 'u'), (v, 'v')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _print_rech = utils._PRINT_RECH_FUNCTYPE(cb_print_rech_py)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _ncv = utils._EngineIntScalarType(
        ncv, {'fun_name': fun_name, 'entity': 'ncv'},
    ).to_ctypes()
    utils._check_callable(av, {'fun_name': fun_name, 'entity': 'av'})
    _av = utils._UNUSED_CALLBACK
    _avh = utils._cast_callback(
        avh, cb_avh,
        {'fun_name': fun_name, 'entity': 'av'},
    )
    _nconv = utils._EngineIntScalarType.empty_ctypes()
    _ncv_py = utils._EngineIntScalarType.to_py(_ncv)
    _sigma_cls = utils._EngineFloat64ArrayType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
        exp_shape=(_ncv_py,),
        check_contents=False,
    )
    _sigma = _sigma_cls.to_ctypes(can_reference_contents=False)
    _ldu = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(u, {'fun_name': fun_name, 'entity': 'u'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldu'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _u_cls = utils._EngineFloat64ArrayType(
        u, {'fun_name': fun_name, 'entity': 'u'},
        exp_shape=(_m_py, _ncv_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _u = _u_cls.to_ctypes(can_reference_contents=False)
    _ldv = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldv'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _v_cls = utils._EngineFloat64ArrayType(
        v, {'fun_name': fun_name, 'entity': 'v'},
        exp_shape=(_n_py, _ncv_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _v = _v_cls.to_ctypes(can_reference_contents=False)
    _resid_cls = utils._EngineFloat64ArrayType(
        resid, {'fun_name': fun_name, 'entity': 'resid'},
        exp_shape=(_ncv_py,),
        check_contents=False,
    )
    _resid = _resid_cls.to_ctypes(can_reference_contents=False)
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        utils._UNUSED_PRINT_REC,
        _print_rech,
        _m,
        _n,
        _k,
        _ncv,
        _av,
        _avh,
        _nconv,
        _sigma,
        _u,
        _ldu,
        _v,
        _ldv,
        _resid,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py < 0 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On output from user-defined function av, iflag was set to a negative value,',
                    'iflag = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}, ncv = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'm = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: k < ncv <= min(m, n).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of iterations has been reached.',
                    'The maximum number of iterations = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'The number of converged eigenvalues = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No shifts could be applied during a cycle of the implicitly restarted',
                    'Lanczos iteration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 10:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Could not build a full Lanczos factorization.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 11:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The number of eigenvalues found to sufficient accuracy is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 20 and _ifmt == 20:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An error occurred during an internal call.',
                    'Consider increasing the size of ncv relative to k.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _sigma_cls.refresh_py(_sigma)
        _u_cls.refresh_py(_u)
        _v_cls.refresh_py(_v)
        _resid_cls.refresh_py(_resid)
        _nconv_py = utils._EngineIntScalarType.to_py(_nconv)
        return_data = _nconv_py
    _err_class.manage()
    return return_data

def real_triang_svd(n, a, ncolb, b, wantq, q, sv, wantp, work):
    u"""
    SVD of real upper triangular matrix (Black Box).

    For full information please refer to the NAG Library document for
    f02wu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f02/f02wuf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix R.

        If `n` = 0, an immediate return is effected.

    a : float, shape (`n`, `n`), modified in place
        On entry: the leading n by n upper triangular part of the array
        `a` must contain the upper triangular matrix R.

        On exit: if `wantp` = True, the n by n part of `a` will contain
        the n by n orthogonal matrix P^T, otherwise the n by n upper
        triangular part of `a` is used as internal workspace, but the
        strictly lower triangular part of `a` is not referenced.

    ncolb : int
        ncolb, the number of columns of the matrix B.

        If `ncolb` = 0, the array `b` is not referenced.

    b : float, shape (:, `ncolb`), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `ncolb`>0: `n`; otherwise: 1.

        On entry: with `ncolb` > 0, the leading n by ncolb part of the
        array `b` must contain the matrix to be transformed.

        On exit: the leading n by ncolb part of the array `b` is
        overwritten by the matrix Q^TB.

    wantq : bool
        Must be True if the matrix Q is required.

        If `wantq` = False, the array `q` is not referenced.

    q : float, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `wantq`= True: `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `wantq`= True: `n`; otherwise: 1.

        On exit: with `wantq` = True, the leading n by n part of the
        array `q` will contain the orthogonal matrix Q. Otherwise the
        array `q` is not referenced.

    sv : float, shape (`n`), modified in place
        On exit:

        - If no exception or warning is raised the array `sv` will
          contain the diagonal elements of the matrix.

        - If `errno` = 1 the array `sv` will contain the diagonal
          elements of the bidiagonal matrix E in the factorization R =
          QEP^T; the superdiagonal elements of E will be contained in
          the first `n`-1 elements of `work`.

    wantp : bool
        Must be True if the matrix P^T is required, in which case P^T is
        overwritten on the array `a`, otherwise `wantp` must be False.

    work : float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `ncolb`=0 and not((`wantq`) and not((`wantp`):
        max(1, 2*(`n`-1)); if (`ncolb`=0 and not((`wantq`) and `wantp`=
        True) or (not((`wantp`) and (`ncolb`>0 or `wantq`= True)):
        max(1, 3*(`n`-1)); otherwise: max(1, 5*(`n`-1)).

        On exit: `work`[0:`n`-1] contains the super-diagonal elements of
        the bidiagonal matrix E computed during the bidiagonalization
        stage; `work`[`n`-1] contains the total number of iterations
        taken by the QR algorithm.

        The rest of the array is used as internal workspace.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, ldq = *<value>* and `n` = *<value>*.

            Constraint: if `wantq` = True, ldq >= `n`.

        (`errno` -1)
            On entry, ldb = *<value>*, `n` = *<value>* and `ncolb` =
            *<value>*.

            Constraint: if `ncolb` > 0, ldb >= `n`.

        (`errno` -1)
            On entry, `ncolb` = *<value>*.

            Constraint: `ncolb` >= 0.

        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The QR algorithm has failed to converge. *<value>* singular
            values have not been found.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.eigen.real_triang_svd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f02wuft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_INT, # Input, ncolb.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ldb, ncolb].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_LOGICAL, # Input, wantq.
        utils._EngineFloat64ArrayType.c_type, # Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineFloat64ArrayType.c_type, # Output, sv[n].
        utils._BYREF_LOGICAL, # Input, wantp.
        utils._EngineFloat64ArrayType.c_type, # Output, work[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (q, 'q')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ncolb = utils._EngineIntScalarType(
        ncolb, {'fun_name': fun_name, 'entity': 'ncolb'},
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _ncolb_py = utils._EngineIntScalarType.to_py(_ncolb)
    if _ncolb_py > 0:
        _b_dim1 = _n_py
    else:
        _b_dim1 = 1
    exp_shape_b = (_b_dim1, _ncolb_py)
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=exp_shape_b,
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _wantq = utils._EngineLogicalScalarType(
        wantq, {'fun_name': fun_name, 'entity': 'wantq'},
    ).to_ctypes()
    _ldq_py = utils._EngineFloat64ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldq = utils._EngineIntScalarType(
        _ldq_py, {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    _wantq_py = utils._EngineLogicalScalarType.to_py(_wantq)
    if _wantq_py:
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _wantq_py:
        _q_dim2 = _n_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineFloat64ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _q = _q_cls.to_ctypes(can_reference_contents=False)
    _sv_cls = utils._EngineFloat64ArrayType(
        sv, {'fun_name': fun_name, 'entity': 'sv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _sv = _sv_cls.to_ctypes(can_reference_contents=False)
    _wantp = utils._EngineLogicalScalarType(
        wantp, {'fun_name': fun_name, 'entity': 'wantp'},
    ).to_ctypes()
    _wantp_py = utils._EngineLogicalScalarType.to_py(_wantp)
    if _ncolb_py == 0 and not _wantq_py and not _wantp_py:
        _work_dim1 = max(1, 2*(_n_py-1))
    elif (_ncolb_py == 0 and not _wantq_py and _wantp_py) or (not _wantp_py and (_ncolb_py > 0 or _wantq_py)):
        _work_dim1 = max(1, 3*(_n_py-1))
    else:
        _work_dim1 = max(1, 5*(_n_py-1))
    exp_shape_work = (_work_dim1,)
    _work_cls = utils._EngineFloat64ArrayType(
        work, {'fun_name': fun_name, 'entity': 'work'},
        exp_shape=exp_shape_work,
        check_contents=False,
    )
    _work = _work_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _ncolb,
        _b,
        _ldb,
        _wantq,
        _q,
        _ldq,
        _sv,
        _wantp,
        _work,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ldq = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if wantq = True, ldq >= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ldb = {:s}, n = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'ncolb = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: if ncolb > 0, ldb >= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncolb = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ncolb >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ifmt == 99994:
            _ierr_fl = 1
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QR algorithm has failed to converge.',
                    '{:s} singular values have not been found.'.format(
                        utils._STR_FMT_INT.format(_ierr_py),
                    ),
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _q_cls.refresh_py(_q)
        _sv_cls.refresh_py(_sv)
        _work_cls.refresh_py(_work)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def complex_triang_svd(n, a, ncolb, b, wantq, q, sv, wantp, rwork):
    u"""
    SVD of complex upper triangular matrix (Black Box).

    For full information please refer to the NAG Library document for
    f02xu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/f02/f02xuf.html

    Parameters
    ----------
    n : int
        n, the order of the matrix R.

        If `n` = 0, an immediate return is effected.

    a : complex, shape (`n`, `n`), modified in place
        On entry: the leading n by n upper triangular part of the array
        `a` must contain the upper triangular matrix R.

        On exit: if `wantp` = True, the n by n part of `a` will contain
        the n by n unitary matrix P^H, otherwise the n by n upper
        triangular part of `a` is used as internal workspace, but the
        strictly lower triangular part of `a` is not referenced.

    ncolb : int
        ncolb, the number of columns of the matrix B.

        If `ncolb` = 0, the array `b` is not referenced.

    b : complex, shape (:, `ncolb`), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `ncolb`>0: `n`; otherwise: 1.

        On entry: if `ncolb` > 0, the leading n by ncolb part of the
        array `b` must contain the matrix to be transformed.

        On exit: is overwritten by the n by ncolb matrix Q^HB.

    wantq : bool
        Must be True if the matrix Q is required.

        If `wantq` = False then the array `q` is not referenced.

    q : complex, shape (:, :), modified in place
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `wantq`= True: `n`; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `wantq`= True: `n`; otherwise: 1.

        On exit: if `wantq` = True, the leading n by n part of the array
        `q` will contain the unitary matrix Q. Otherwise the array `q`
        is not referenced.

    sv : float, shape (`n`), modified in place
        On exit: the n diagonal elements of the matrix S.

    wantp : bool
        Must be True if the matrix P^H is required, in which case P^H is
        returned in the array `a`, otherwise `wantp` must be False.

    rwork : float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `ncolb`=0 and not((`wantq`) and not((`wantp`):
        2*(`n`-1); if (`ncolb`=0 and not((`wantq`) and `wantp`= True) or
        (`ncolb`>0 and not((`wantp`)) or (`wantq`= True and
        not((`wantp`)): 3*(`n`-1); otherwise: 5*(`n`-1).

        On exit: `rwork`[`n`-1] contains the total number of iterations
        taken by the QR algorithm.

        The rest of the array is used as workspace.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, ldq = *<value>* and `n` = *<value>*.

            Constraint: if `wantq` = True, ldq >= `n`.

        (`errno` -1)
            On entry, ldb = *<value>*, `n` = *<value>* and `ncolb` =
            *<value>*.

            Constraint: if `ncolb` > 0, ldb >= `n`.

        (`errno` -1)
            On entry, `ncolb` = *<value>*.

            Constraint: `ncolb` >= 0.

        (`errno` -1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The QR algorithm has failed to converge. *<value>* singular
            values have not been found.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.eigen.complex_triang_svd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('f02xuft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_INT, # Input, ncolb.
        utils._EngineComplex128ArrayType.c_type, # Input/Output, b[ldb, ncolb].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_LOGICAL, # Input, wantq.
        utils._EngineComplex128ArrayType.c_type, # Output, q[ldq, *].
        utils._BYREF_INT, # Input, ldq.
        utils._EngineFloat64ArrayType.c_type, # Output, sv[n].
        utils._BYREF_LOGICAL, # Input, wantp.
        utils._EngineFloat64ArrayType.c_type, # Output, rwork[*].
        utils._EngineComplex128ArrayType.c_type, # Workspace, cwork.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (q, 'q')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a_cls = utils._EngineComplex128ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _a = _a_cls.to_ctypes()
    _ncolb = utils._EngineIntScalarType(
        ncolb, {'fun_name': fun_name, 'entity': 'ncolb'},
    ).to_ctypes()
    _ldb = utils._EngineIntScalarType(
        max(1, utils._EngineComplex128ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')), {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _ncolb_py = utils._EngineIntScalarType.to_py(_ncolb)
    if _ncolb_py > 0:
        _b_dim1 = _n_py
    else:
        _b_dim1 = 1
    exp_shape_b = (_b_dim1, _ncolb_py)
    _b_cls = utils._EngineComplex128ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=exp_shape_b,
        mutual_sorder=_m_sorder,
    )
    _b = _b_cls.to_ctypes()
    _wantq = utils._EngineLogicalScalarType(
        wantq, {'fun_name': fun_name, 'entity': 'wantq'},
    ).to_ctypes()
    _ldq_py = utils._EngineComplex128ArrayType.get_nd(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldq = utils._EngineIntScalarType(
        _ldq_py, {'fun_name': fun_name, 'entity': '_ldq'},
    ).to_ctypes()
    _wantq_py = utils._EngineLogicalScalarType.to_py(_wantq)
    if _wantq_py:
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _wantq_py:
        _q_dim2 = _n_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q_cls = utils._EngineComplex128ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _q = _q_cls.to_ctypes(can_reference_contents=False)
    _sv_cls = utils._EngineFloat64ArrayType(
        sv, {'fun_name': fun_name, 'entity': 'sv'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _sv = _sv_cls.to_ctypes(can_reference_contents=False)
    _wantp = utils._EngineLogicalScalarType(
        wantp, {'fun_name': fun_name, 'entity': 'wantp'},
    ).to_ctypes()
    _wantp_py = utils._EngineLogicalScalarType.to_py(_wantp)
    if _ncolb_py == 0 and not _wantq_py and not _wantp_py:
        _rwork_dim1 = 2*(_n_py-1)
    elif (_ncolb_py == 0 and not _wantq_py and _wantp_py) or (_ncolb_py > 0 and not _wantp_py) or (_wantq_py and not _wantp_py):
        _rwork_dim1 = 3*(_n_py-1)
    else:
        _rwork_dim1 = 5*(_n_py-1)
    exp_shape_rwork = (_rwork_dim1,)
    _rwork_cls = utils._EngineFloat64ArrayType(
        rwork, {'fun_name': fun_name, 'entity': 'rwork'},
        exp_shape=exp_shape_rwork,
        check_contents=False,
    )
    _rwork = _rwork_cls.to_ctypes(can_reference_contents=False)
    _cwork = utils._EngineComplex128ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _lda,
        _ncolb,
        _b,
        _ldb,
        _wantq,
        _q,
        _ldq,
        _sv,
        _wantp,
        _rwork,
        _cwork,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ldq = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if wantq = True, ldq >= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ldb = {:s}, n = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'ncolb = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: if ncolb > 0, ldb >= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ncolb = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ncolb >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == -1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ifmt == 99994:
            _ierr_fl = 1
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The QR algorithm has failed to converge.',
                    '{:s} singular values have not been found.'.format(
                        utils._STR_FMT_INT.format(_ierr_py),
                    ),
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _q_cls.refresh_py(_q)
        _sv_cls.refresh_py(_sv)
        _rwork_cls.refresh_py(_rwork)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data
