# -*- coding: utf-8 -*-
# pylint: disable=bare-except,chained-comparison,invalid-name,len-as-condition,line-too-long,protected-access,redefined-outer-name,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unneeded-not,unused-import
u"""
Interfaces for the NAG Mark 27.0 `quad` Chapter.

``quad`` - Quadrature

Base interfaces.

We do not intend for these lower-level base interfaces to be called
directly from user code. Please refer to the ``naginterfaces.library``
subpackage instead.

This module provides functions for the numerical evaluation of definite
integrals in one or more dimensions and for evaluating weights and
abscissae of integration rules.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01intro.html
"""

# NAG Copyright 2017-2019.

from __future__ import absolute_import

import collections as _collections
import sys as _sys

from math import ceil

from ..base import utils

def dim1_fin_well(a, b, epsr, f, nlimit, data=None):
    u"""
    One-dimensional quadrature, adaptive, finite interval, strategy due
    to Patterson, suitable for well-behaved integrands.

    For full information please refer to the NAG Library document for
    d01ah

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01ahf.html

    Parameters
    ----------
    a : float
        a, the lower limit of integration.

    b : float
        b, the upper limit of integration. It is not necessary that a <
        b.

    epsr : float
        The relative accuracy required.

    f : callable retval = f(x, data=None)
        `f` must return the value of the integrand f at a given point.

        Parameters
        ~~~~~~~~~~
        x : float
            The point at which the integrand f must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of the integrand at `x`.

    nlimit : int
        A limit to the number of function evaluations. If `nlimit` <= 0,
        the function uses a default limit of 10000.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    din : float
        The value of the definite interval.

    npts : int
        The number of function evaluations used in the calculation of
        the integral.

    relerr : float
        A rough estimate of the relative error achieved.

    Raises
    ------
    NagValueError
        (`errno` 3)
            On entry, `epsr` = *<value>*.

            Constraint: `epsr` > 0.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            Integral has failed to converge to requested accuracy.

        (`errno` 2)
            Too many unsuccessful levels of subdivision.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.dim1_fin_well'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, _fsub, en_data, x, retval, _iuser, _ruser):
        """Helper for f."""
        try:
            _x_fh = utils._EngineFloat64ScalarType.to_py(x.contents)
            if data is not None:
                _retval_fh = f(
                    _x_fh,
                    data,
                )
            else:
                _retval_fh = f(
                    _x_fh,
                )
            utils._EngineFloat64ScalarType(
                _retval_fh, {'fun_name': cb_f_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d01ahft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_FLOAT64, # Input, epsr.
        utils._BYREF_INT, # Output, npts.
        utils._BYREF_FLOAT64, # Output, relerr.
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._BYREF_INT, # Input, nlimit.
        utils._BYREF_FLOAT64, # Output, din.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _epsr = utils._EngineFloat64ScalarType(
        epsr, {'fun_name': fun_name, 'entity': 'epsr'},
    ).to_ctypes()
    _npts = utils._EngineIntScalarType.empty_ctypes()
    _relerr = utils._EngineFloat64ScalarType.empty_ctypes()
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fsub = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _nlimit = utils._EngineIntScalarType(
        nlimit, {'fun_name': fun_name, 'entity': 'nlimit'},
    ).to_ctypes()
    _din = utils._EngineFloat64ScalarType.empty_ctypes()
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _a,
        _b,
        _epsr,
        _npts,
        _relerr,
        _f,
        _fsub,
        _fh,
        _nlimit,
        _din,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Integral has failed to converge to requested accuracy.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Too many unsuccessful levels of subdivision.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, epsr = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: epsr > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _din_py = utils._EngineFloat64ScalarType.to_py(_din)
        _npts_py = utils._EngineIntScalarType.to_py(_npts)
        _relerr_py = utils._EngineFloat64ScalarType.to_py(_relerr)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['din', 'npts', 'relerr'],
        )(
            _din_py,
            _npts_py,
            _relerr_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim1_fin_bad(f, a, b, epsabs, epsrel, w, lw, iw, liw, data=None):
    u"""
    One-dimensional quadrature, adaptive, finite interval, strategy due
    to Piessens and de Doncker, allowing for badly behaved integrands.

    For full information please refer to the NAG Library document for
    d01aj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01ajf.html

    Parameters
    ----------
    f : callable retval = f(x, data=None)
        `f` must return the value of the integrand f at a given point.

        Parameters
        ~~~~~~~~~~
        x : float
            The point at which the integrand f must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of the integrand at `x`.

    a : float
        a, the lower limit of integration.

    b : float
        b, the upper limit of integration. It is not necessary that a <
        b.

    epsabs : float
        The absolute accuracy required. If `epsabs` is negative, the
        absolute value is used. See [Accuracy].

    epsrel : float
        The relative accuracy required. If `epsrel` is negative, the
        absolute value is used. See [Accuracy].

    w : float, shape (`lw`), modified in place
        On exit: details of the computation see [Further Comments] for
        more information.

    lw : int
        The value of `lw` (together with that of `liw`) imposes a bound
        on the number of sub-intervals into which the interval of
        integration may be divided by the function. The number of
        sub-intervals cannot exceed `lw`/4. The more difficult the
        integrand, the larger `lw` should be.

    iw : int, shape (`liw`), modified in place
        On exit: `iw`[0] contains the actual number of sub-intervals
        used. The rest of the array is used as workspace.

    liw : int
        The number of sub-intervals into which the interval of
        integration may be divided cannot exceed `liw`.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    result : float
        The approximation to the integral I.

    abserr : float
        An estimate of the modulus of the absolute error, which should
        be an upper bound for |I-`result`|.

    Raises
    ------
    NagValueError
        (`errno` 6)
            On entry, `liw` = *<value>*.

            Constraint: `liw` >= 1.

        (`errno` 6)
            On entry, `lw` = *<value>*.

            Constraint: `lw` >= 4.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The maximum number of subdivisions (LIMIT) has been reached:
            LIMIT = *<value>*, `lw` = *<value>* and `liw` = *<value>*.

        (`errno` 2)
            Round-off error prevents the requested tolerance from being
            achieved: `epsabs` = *<value>* and `epsrel` = *<value>*.

        (`errno` 3)
            Extremely bad integrand behaviour occurs around the
            sub-interval (*<value>*,*<value>*).

        (`errno` 4)
            Round-off error is detected in the extrapolation table.

        (`errno` 5)
            The integral is probably divergent or slowly convergent.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.dim1_fin_bad'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, _fsub, en_data, x, retval, _iuser, _ruser):
        """Helper for f."""
        try:
            _x_fh = utils._EngineFloat64ScalarType.to_py(x.contents)
            if data is not None:
                _retval_fh = f(
                    _x_fh,
                    data,
                )
            else:
                _retval_fh = f(
                    _x_fh,
                )
            utils._EngineFloat64ScalarType(
                _retval_fh, {'fun_name': cb_f_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d01ajft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_FLOAT64, # Input, epsabs.
        utils._BYREF_FLOAT64, # Input, epsrel.
        utils._BYREF_FLOAT64, # Output, result.
        utils._BYREF_FLOAT64, # Output, abserr.
        utils._EngineFloat64ArrayType.c_type, # Output, w[lw].
        utils._BYREF_INT, # Input, lw.
        utils._EngineIntArrayType.c_type, # Output, iw[liw].
        utils._BYREF_INT, # Input, liw.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fsub = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _epsabs = utils._EngineFloat64ScalarType(
        epsabs, {'fun_name': fun_name, 'entity': 'epsabs'},
    ).to_ctypes()
    _epsrel = utils._EngineFloat64ScalarType(
        epsrel, {'fun_name': fun_name, 'entity': 'epsrel'},
    ).to_ctypes()
    _result = utils._EngineFloat64ScalarType.empty_ctypes()
    _abserr = utils._EngineFloat64ScalarType.empty_ctypes()
    _lw = utils._EngineIntScalarType(
        lw, {'fun_name': fun_name, 'entity': 'lw'},
    ).to_ctypes()
    _lw_py = utils._EngineIntScalarType.to_py(_lw)
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_lw_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _liw = utils._EngineIntScalarType(
        liw, {'fun_name': fun_name, 'entity': 'liw'},
    ).to_ctypes()
    _liw_py = utils._EngineIntScalarType.to_py(_liw)
    _iw_cls = utils._EngineIntArrayType(
        iw, {'fun_name': fun_name, 'entity': 'iw'},
        exp_shape=(_liw_py,),
        check_contents=False,
    )
    _iw = _iw_cls.to_ctypes(can_reference_contents=False)
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _f,
        _fsub,
        _fh,
        _a,
        _b,
        _epsabs,
        _epsrel,
        _result,
        _abserr,
        _w,
        _lw,
        _iw,
        _liw,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of subdivisions (LIMIT) has been reached:',
                    'LIMIT = {:s}, lw = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'liw = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Round-off error prevents the requested tolerance from being achieved:',
                    'epsabs = {:s} and epsrel = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99995:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Extremely bad integrand behaviour occurs around the sub-interval',
                    '({:s},{:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99994:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Round-off error is detected in the extrapolation table.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99993:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The integral is probably divergent or slowly convergent.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, liw = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: liw >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lw = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lw >= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _w_cls.refresh_py(_w)
        _iw_cls.refresh_py(_iw)
        _result_py = utils._EngineFloat64ScalarType.to_py(_result)
        _abserr_py = utils._EngineFloat64ScalarType.to_py(_abserr)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['result', 'abserr'],
        )(
            _result_py,
            _abserr_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim1_fin_osc(f, a, b, epsabs, epsrel, w, lw, iw, liw, data=None):
    u"""
    One-dimensional quadrature, adaptive, finite interval, method
    suitable for oscillating functions.

    For full information please refer to the NAG Library document for
    d01ak

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01akf.html

    Parameters
    ----------
    f : callable retval = f(x, data=None)
        `f` must return the value of the integrand f at a given point.

        Parameters
        ~~~~~~~~~~
        x : float
            The point at which the integrand f must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of the integrand at `x`.

    a : float
        a, the lower limit of integration.

    b : float
        b, the upper limit of integration. It is not necessary that a <
        b.

    epsabs : float
        The absolute accuracy required. If `epsabs` is negative, the
        absolute value is used. See [Accuracy].

    epsrel : float
        The relative accuracy required. If `epsrel` is negative, the
        absolute value is used. See [Accuracy].

    w : float, shape (`lw`), modified in place
        On exit: details of the computation see [Further Comments] for
        more information.

    lw : int
        The value of `lw` (together with that of `liw`) imposes a bound
        on the number of sub-intervals into which the interval of
        integration may be divided by the function. The number of
        sub-intervals cannot exceed `lw`/4. The more difficult the
        integrand, the larger `lw` should be.

    iw : int, shape (`liw`), modified in place
        On exit: `iw`[0] contains the actual number of sub-intervals
        used. The rest of the array is used as workspace.

    liw : int
        The number of sub-intervals into which the interval of
        integration may be divided cannot exceed `liw`.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    result : float
        The approximation to the integral I.

    abserr : float
        An estimate of the modulus of the absolute error, which should
        be an upper bound for |I-`result`|.

    Raises
    ------
    NagValueError
        (`errno` 4)
            On entry, `liw` = *<value>*.

            Constraint: `liw` >= 1.

        (`errno` 4)
            On entry, `lw` = *<value>*.

            Constraint: `lw` >= 4.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The maximum number of subdivisions (LIMIT) has been reached:
            LIMIT = *<value>*, `lw` = *<value>* and `liw` = *<value>*.

        (`errno` 2)
            Round-off error prevents the requested tolerance from being
            achieved: `epsabs` = *<value>* and `epsrel` = *<value>*.

        (`errno` 3)
            Extremely bad integrand behaviour occurs around the
            sub-interval (*<value>*,*<value>*).

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.dim1_fin_osc'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, _fsub, en_data, x, retval, _iuser, _ruser):
        """Helper for f."""
        try:
            _x_fh = utils._EngineFloat64ScalarType.to_py(x.contents)
            if data is not None:
                _retval_fh = f(
                    _x_fh,
                    data,
                )
            else:
                _retval_fh = f(
                    _x_fh,
                )
            utils._EngineFloat64ScalarType(
                _retval_fh, {'fun_name': cb_f_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d01akft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_FLOAT64, # Input, epsabs.
        utils._BYREF_FLOAT64, # Input, epsrel.
        utils._BYREF_FLOAT64, # Output, result.
        utils._BYREF_FLOAT64, # Output, abserr.
        utils._EngineFloat64ArrayType.c_type, # Output, w[lw].
        utils._BYREF_INT, # Input, lw.
        utils._EngineIntArrayType.c_type, # Output, iw[liw].
        utils._BYREF_INT, # Input, liw.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fsub = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _epsabs = utils._EngineFloat64ScalarType(
        epsabs, {'fun_name': fun_name, 'entity': 'epsabs'},
    ).to_ctypes()
    _epsrel = utils._EngineFloat64ScalarType(
        epsrel, {'fun_name': fun_name, 'entity': 'epsrel'},
    ).to_ctypes()
    _result = utils._EngineFloat64ScalarType.empty_ctypes()
    _abserr = utils._EngineFloat64ScalarType.empty_ctypes()
    _lw = utils._EngineIntScalarType(
        lw, {'fun_name': fun_name, 'entity': 'lw'},
    ).to_ctypes()
    _lw_py = utils._EngineIntScalarType.to_py(_lw)
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_lw_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _liw = utils._EngineIntScalarType(
        liw, {'fun_name': fun_name, 'entity': 'liw'},
    ).to_ctypes()
    _liw_py = utils._EngineIntScalarType.to_py(_liw)
    _iw_cls = utils._EngineIntArrayType(
        iw, {'fun_name': fun_name, 'entity': 'iw'},
        exp_shape=(_liw_py,),
        check_contents=False,
    )
    _iw = _iw_cls.to_ctypes(can_reference_contents=False)
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _f,
        _fsub,
        _fh,
        _a,
        _b,
        _epsabs,
        _epsrel,
        _result,
        _abserr,
        _w,
        _lw,
        _iw,
        _liw,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of subdivisions (LIMIT) has been reached:',
                    'LIMIT = {:s}, lw = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'liw = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Round-off error prevents the requested tolerance from being achieved:',
                    'epsabs = {:s} and epsrel = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99995:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Extremely bad integrand behaviour occurs around the sub-interval',
                    '({:s},{:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, liw = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: liw >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lw = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lw >= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _w_cls.refresh_py(_w)
        _iw_cls.refresh_py(_iw)
        _result_py = utils._EngineFloat64ScalarType.to_py(_result)
        _abserr_py = utils._EngineFloat64ScalarType.to_py(_abserr)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['result', 'abserr'],
        )(
            _result_py,
            _abserr_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim1_fin_sing(f, a, b, npts, points, epsabs, epsrel, w, lw, iw, liw, data=None):
    u"""
    One-dimensional quadrature, adaptive, finite interval, allowing for
    singularities at user-specified break-points.

    For full information please refer to the NAG Library document for
    d01al

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01alf.html

    Parameters
    ----------
    f : callable retval = f(x, data=None)
        `f` must return the value of the integrand f at a given point.

        Parameters
        ~~~~~~~~~~
        x : float
            The point at which the integrand f must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of the integrand at `x`.

    a : float
        a, the lower limit of integration.

    b : float
        b, the upper limit of integration. It is not necessary that a <
        b.

    npts : int
        The number of user-supplied break-points within the integration
        interval.

    points : float, shape (`npts`)
        The user-specified break-points.

    epsabs : float
        The absolute accuracy required. If `epsabs` is negative, the
        absolute value is used. See [Accuracy].

    epsrel : float
        The relative accuracy required. If `epsrel` is negative, the
        absolute value is used. See [Accuracy].

    w : float, shape (`lw`), modified in place
        On exit: details of the computation see [Further Comments] for
        more information.

    lw : int
        The value of `lw` (together with that of `liw`) imposes a bound
        on the number of sub-intervals into which the interval of
        integration may be divided by the function. The number of
        sub-intervals cannot exceed (`lw`-2*`npts`-4)/4. The more
        difficult the integrand, the larger `lw` should be.

    iw : int, shape (`liw`), modified in place
        On exit: `iw`[0] contains the actual number of sub-intervals
        used. The rest of the array is used as workspace.

    liw : int
        The number of sub-intervals into which the interval of
        integration may be divided cannot exceed (`liw`-`npts`-2)/2.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    result : float
        The approximation to the integral I.

    abserr : float
        An estimate of the modulus of the absolute error, which should
        be an upper bound for |I-`result`|.

    Raises
    ------
    NagValueError
        (`errno` 6)
            On entry, LIMIT = *<value>* and `npts` = *<value>*.

            Constraint: LIMIT > `npts`.

        (`errno` 6)
            On entry, break-points are specified outside (`a`,`b`): `a`
            = *<value>* and `b` = *<value>*.

        (`errno` 6)
            On entry, `npts` = *<value>*.

            Constraint: `npts` >= 0.

        (`errno` 7)
            On entry, `liw` is too small. `liw` = *<value>*. Minimum
            possible dimension: *<value>*.

        (`errno` 7)
            On entry, `lw` is too small. `lw` = *<value>*. Minimum
            possible dimension: *<value>*.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The maximum number of subdivisions (LIMIT) has been reached:
            LIMIT = *<value>* `lw` = *<value>* `liw` = *<value>* `npts`
            = *<value>*.

        (`errno` 2)
            Round-off error prevents the requested tolerance from being
            achieved: `epsabs` = *<value>* and `epsrel` = *<value>*.

        (`errno` 3)
            Extremely bad integrand behaviour occurs around the
            sub-interval (*<value>*,*<value>*).

        (`errno` 4)
            Round-off error is detected in the extrapolation table.

        (`errno` 5)
            The integral is probably divergent or slowly convergent.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.dim1_fin_sing'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, _fsub, en_data, x, retval, _iuser, _ruser):
        """Helper for f."""
        try:
            _x_fh = utils._EngineFloat64ScalarType.to_py(x.contents)
            if data is not None:
                _retval_fh = f(
                    _x_fh,
                    data,
                )
            else:
                _retval_fh = f(
                    _x_fh,
                )
            utils._EngineFloat64ScalarType(
                _retval_fh, {'fun_name': cb_f_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d01alft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_INT, # Input, npts.
        utils._EngineFloat64ArrayType.c_type, # Input, points[npts].
        utils._BYREF_FLOAT64, # Input, epsabs.
        utils._BYREF_FLOAT64, # Input, epsrel.
        utils._BYREF_FLOAT64, # Output, result.
        utils._BYREF_FLOAT64, # Output, abserr.
        utils._EngineFloat64ArrayType.c_type, # Output, w[lw].
        utils._BYREF_INT, # Input, lw.
        utils._EngineIntArrayType.c_type, # Output, iw[liw].
        utils._BYREF_INT, # Input, liw.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fsub = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _npts = utils._EngineIntScalarType(
        npts, {'fun_name': fun_name, 'entity': 'npts'},
    ).to_ctypes()
    _npts_py = utils._EngineIntScalarType.to_py(_npts)
    _points = utils._EngineFloat64ArrayType(
        points, {'fun_name': fun_name, 'entity': 'points'},
        exp_shape=(_npts_py,),
    ).to_ctypes()
    _epsabs = utils._EngineFloat64ScalarType(
        epsabs, {'fun_name': fun_name, 'entity': 'epsabs'},
    ).to_ctypes()
    _epsrel = utils._EngineFloat64ScalarType(
        epsrel, {'fun_name': fun_name, 'entity': 'epsrel'},
    ).to_ctypes()
    _result = utils._EngineFloat64ScalarType.empty_ctypes()
    _abserr = utils._EngineFloat64ScalarType.empty_ctypes()
    _lw = utils._EngineIntScalarType(
        lw, {'fun_name': fun_name, 'entity': 'lw'},
    ).to_ctypes()
    _lw_py = utils._EngineIntScalarType.to_py(_lw)
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_lw_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _liw = utils._EngineIntScalarType(
        liw, {'fun_name': fun_name, 'entity': 'liw'},
    ).to_ctypes()
    _liw_py = utils._EngineIntScalarType.to_py(_liw)
    _iw_cls = utils._EngineIntArrayType(
        iw, {'fun_name': fun_name, 'entity': 'iw'},
        exp_shape=(_liw_py,),
        check_contents=False,
    )
    _iw = _iw_cls.to_ctypes(can_reference_contents=False)
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _f,
        _fsub,
        _fh,
        _a,
        _b,
        _npts,
        _points,
        _epsabs,
        _epsrel,
        _result,
        _abserr,
        _w,
        _lw,
        _iw,
        _liw,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of subdivisions (LIMIT) has been reached:',
                    'LIMIT = {:s} lw = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'liw = {:s} npts = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Round-off error prevents the requested tolerance from being achieved:',
                    'epsabs = {:s} and epsrel = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99994:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Extremely bad integrand behaviour occurs around the sub-interval',
                    '({:s},{:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99993:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Round-off error is detected in the extrapolation table.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99992:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The integral is probably divergent or slowly convergent.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, LIMIT = {:s} and npts = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: LIMIT > npts.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, break-points are specified outside (a,b):',
                    'a = {:s} and b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npts = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npts >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, liw is too small.',
                    'liw = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Minimum possible dimension: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lw is too small.',
                    'lw = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Minimum possible dimension: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _w_cls.refresh_py(_w)
        _iw_cls.refresh_py(_iw)
        _result_py = utils._EngineFloat64ScalarType.to_py(_result)
        _abserr_py = utils._EngineFloat64ScalarType.to_py(_abserr)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['result', 'abserr'],
        )(
            _result_py,
            _abserr_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim1_inf(f, bound, inf, epsabs, epsrel, w, lw, iw, liw, data=None):
    u"""
    One-dimensional quadrature, adaptive, infinite or semi-infinite
    interval.

    For full information please refer to the NAG Library document for
    d01am

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01amf.html

    Parameters
    ----------
    f : callable retval = f(x, data=None)
        `f` must return the value of the integrand f at a given point.

        Parameters
        ~~~~~~~~~~
        x : float
            The point at which the integrand f must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of the integrand at `x`.

    bound : float
        The finite limit of the integration range (if present). `bound`
        is not used if the interval is doubly infinite.

    inf : int
        Indicates the kind of integration range.

        `inf` = 1

          The range is [`bound`, + infinity).

        `inf` = -1

          The range is (- infinity,`bound`].

        `inf` = 2

          The range is (- infinity, + infinity).

    epsabs : float
        The absolute accuracy required. If `epsabs` is negative, the
        absolute value is used. See [Accuracy].

    epsrel : float
        The relative accuracy required. If `epsrel` is negative, the
        absolute value is used. See [Accuracy].

    w : float, shape (`lw`), modified in place
        On exit: details of the computation see [Further Comments] for
        more information.

    lw : int
        The value of `lw` (together with that of `liw`) imposes a bound
        on the number of sub-intervals into which the interval of
        integration may be divided by the function. The number of
        sub-intervals cannot exceed `lw`/4. The more difficult the
        integrand, the larger `lw` should be.

    iw : int, shape (`liw`), modified in place
        On exit: `iw`[0] contains the actual number of sub-intervals
        used. The rest of the array is used as workspace.

    liw : int
        The number of sub-intervals into which the interval of
        integration may be divided cannot exceed `liw`.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    result : float
        The approximation to the integral I.

    abserr : float
        An estimate of the modulus of the absolute error, which should
        be an upper bound for |I-`result`|.

    Raises
    ------
    NagValueError
        (`errno` 6)
            On entry, `inf` = *<value>*.

            Constraint: `inf` = -1, 1 or 2.

        (`errno` 6)
            On entry, `liw` = *<value>*.

            Constraint: `liw` >= 1.

        (`errno` 6)
            On entry, `lw` = *<value>*.

            Constraint: `lw` >= 4.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The maximum number of subdivisions (LIMIT) has been reached:
            LIMIT = *<value>*, `lw` = *<value>* and `liw` = *<value>*.

        (`errno` 2)
            Round-off error prevents the requested tolerance from being
            achieved: `epsabs` = *<value>* and `epsrel` = *<value>*.

        (`errno` 3)
            Extremely bad integrand behaviour occurs around one of the
            sub-intervals (*<value>*,*<value>*) or
            (*<value>*,*<value>*).

        (`errno` 3)
            Extremely bad integrand behaviour occurs around the
            sub-interval (*<value>*,*<value>*).

        (`errno` 4)
            Round-off error is detected in the extrapolation table.

        (`errno` 5)
            The integral is probably divergent or slowly convergent.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.dim1_inf'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, _fsub, en_data, x, retval, _iuser, _ruser):
        """Helper for f."""
        try:
            _x_fh = utils._EngineFloat64ScalarType.to_py(x.contents)
            if data is not None:
                _retval_fh = f(
                    _x_fh,
                    data,
                )
            else:
                _retval_fh = f(
                    _x_fh,
                )
            utils._EngineFloat64ScalarType(
                _retval_fh, {'fun_name': cb_f_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d01amft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._BYREF_FLOAT64, # Input, bound.
        utils._BYREF_INT, # Input, inf.
        utils._BYREF_FLOAT64, # Input, epsabs.
        utils._BYREF_FLOAT64, # Input, epsrel.
        utils._BYREF_FLOAT64, # Output, result.
        utils._BYREF_FLOAT64, # Output, abserr.
        utils._EngineFloat64ArrayType.c_type, # Output, w[lw].
        utils._BYREF_INT, # Input, lw.
        utils._EngineIntArrayType.c_type, # Output, iw[liw].
        utils._BYREF_INT, # Input, liw.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fsub = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _bound = utils._EngineFloat64ScalarType(
        bound, {'fun_name': fun_name, 'entity': 'bound'},
    ).to_ctypes()
    _inf = utils._EngineIntScalarType(
        inf, {'fun_name': fun_name, 'entity': 'inf'},
    ).to_ctypes()
    _epsabs = utils._EngineFloat64ScalarType(
        epsabs, {'fun_name': fun_name, 'entity': 'epsabs'},
    ).to_ctypes()
    _epsrel = utils._EngineFloat64ScalarType(
        epsrel, {'fun_name': fun_name, 'entity': 'epsrel'},
    ).to_ctypes()
    _result = utils._EngineFloat64ScalarType.empty_ctypes()
    _abserr = utils._EngineFloat64ScalarType.empty_ctypes()
    _lw = utils._EngineIntScalarType(
        lw, {'fun_name': fun_name, 'entity': 'lw'},
    ).to_ctypes()
    _lw_py = utils._EngineIntScalarType.to_py(_lw)
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_lw_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _liw = utils._EngineIntScalarType(
        liw, {'fun_name': fun_name, 'entity': 'liw'},
    ).to_ctypes()
    _liw_py = utils._EngineIntScalarType.to_py(_liw)
    _iw_cls = utils._EngineIntArrayType(
        iw, {'fun_name': fun_name, 'entity': 'iw'},
        exp_shape=(_liw_py,),
        check_contents=False,
    )
    _iw = _iw_cls.to_ctypes(can_reference_contents=False)
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _f,
        _fsub,
        _fh,
        _bound,
        _inf,
        _epsabs,
        _epsrel,
        _result,
        _abserr,
        _w,
        _lw,
        _iw,
        _liw,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of subdivisions (LIMIT) has been reached:',
                    'LIMIT = {:s}, lw = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'liw = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Round-off error prevents the requested tolerance from being achieved:',
                    'epsabs = {:s} and epsrel = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99993:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Extremely bad integrand behaviour occurs around one of the sub-intervals',
                    '({:s},{:s}) or'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    '({:s},{:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99994:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Extremely bad integrand behaviour occurs around the sub-interval',
                    '({:s},{:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99992:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Round-off error is detected in the extrapolation table.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99991:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The integral is probably divergent or slowly convergent.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, inf = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: inf = -1, 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, liw = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: liw >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lw = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lw >= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _w_cls.refresh_py(_w)
        _iw_cls.refresh_py(_iw)
        _result_py = utils._EngineFloat64ScalarType.to_py(_result)
        _abserr_py = utils._EngineFloat64ScalarType.to_py(_abserr)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['result', 'abserr'],
        )(
            _result_py,
            _abserr_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim1_fin_wtrig(g, a, b, omega, key, epsabs, epsrel, w, lw, iw, liw, data=None):
    u"""
    One-dimensional quadrature, adaptive, finite interval, weight
    function cos(omega x) or sin(omega x).

    For full information please refer to the NAG Library document for
    d01an

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01anf.html

    Parameters
    ----------
    g : callable retval = g(x, data=None)
        `g` must return the value of the function g at a given point
        `x`.

        Parameters
        ~~~~~~~~~~
        x : float
            The point at which the function g must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of g(x) evaluated at `x`.

    a : float
        a, the lower limit of integration.

    b : float
        b, the upper limit of integration. It is not necessary that a <
        b.

    omega : float
        The argument omega in the weight function of the transform.

    key : int
        Indicates which integral is to be computed.

        `key` = 1

          w(x) = cos(omega x).

        `key` = 2

          w(x) = sin(omega x).

    epsabs : float
        The absolute accuracy required. If `epsabs` is negative, the
        absolute value is used. See [Accuracy].

    epsrel : float
        The relative accuracy required. If `epsrel` is negative, the
        absolute value is used. See [Accuracy].

    w : float, shape (`lw`), modified in place
        On exit: details of the computation see [Further Comments] for
        more information.

    lw : int
        The value of `lw` (together with that of `liw`) imposes a bound
        on the number of sub-intervals into which the interval of
        integration may be divided by the function. The number of
        sub-intervals cannot exceed `lw`/4. The more difficult the
        integrand, the larger `lw` should be.

    iw : int, shape (`liw`), modified in place
        On exit: `iw`[0] contains the actual number of sub-intervals
        used. The rest of the array is used as workspace.

    liw : int
        The number of sub-intervals into which the interval of
        integration may be divided cannot exceed `liw`/2.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    result : float
        The approximation to the integral I.

    abserr : float
        An estimate of the modulus of the absolute error, which should
        be an upper bound for |I-`result`|.

    Raises
    ------
    NagValueError
        (`errno` 6)
            On entry, `key` = *<value>*.

            Constraint: `key` >= 1 and `key` <= 2.

        (`errno` 7)
            On entry, `liw` = *<value>*.

            Constraint: `liw` >= 2.

        (`errno` 7)
            On entry, `lw` = *<value>*.

            Constraint: `lw` >= 4.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The maximum number of subdivisions (LIMIT) has been reached:
            LIMIT = *<value>*, `lw` = *<value>* and `liw` = *<value>*.

        (`errno` 2)
            Round-off error prevents the requested tolerance from being
            achieved: `epsabs` = *<value>* and `epsrel` = *<value>*.

        (`errno` 3)
            Extremely bad integrand behaviour occurs around the
            sub-interval (*<value>*,*<value>*).

        (`errno` 4)
            Round-off error is detected in the extrapolation table.

        (`errno` 5)
            The integral is probably divergent or slowly convergent.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.dim1_fin_wtrig'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_g_name = fun_name + ':g'

    gh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_gh(_g, _gsub, en_data, x, retval, _iuser, _ruser):
        """Helper for g."""
        try:
            _x_gh = utils._EngineFloat64ScalarType.to_py(x.contents)
            if data is not None:
                _retval_gh = g(
                    _x_gh,
                    data,
                )
            else:
                _retval_gh = g(
                    _x_gh,
                )
            utils._EngineFloat64ScalarType(
                _retval_gh, {'fun_name': cb_g_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d01anft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        gh,
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_FLOAT64, # Input, omega.
        utils._BYREF_INT, # Input, key.
        utils._BYREF_FLOAT64, # Input, epsabs.
        utils._BYREF_FLOAT64, # Input, epsrel.
        utils._BYREF_FLOAT64, # Output, result.
        utils._BYREF_FLOAT64, # Output, abserr.
        utils._EngineFloat64ArrayType.c_type, # Output, w[lw].
        utils._BYREF_INT, # Input, lw.
        utils._EngineIntArrayType.c_type, # Output, iw[liw].
        utils._BYREF_INT, # Input, liw.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    utils._check_callable(g, {'fun_name': fun_name, 'entity': 'g'})
    _g = utils._UNUSED_CALLBACK
    _gsub = utils._UNUSED_CALLBACK
    _gh = utils._cast_callback(
        gh, cb_gh,
        {'fun_name': fun_name, 'entity': 'g'},
    )
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _omega = utils._EngineFloat64ScalarType(
        omega, {'fun_name': fun_name, 'entity': 'omega'},
    ).to_ctypes()
    _key = utils._EngineIntScalarType(
        key, {'fun_name': fun_name, 'entity': 'key'},
    ).to_ctypes()
    _epsabs = utils._EngineFloat64ScalarType(
        epsabs, {'fun_name': fun_name, 'entity': 'epsabs'},
    ).to_ctypes()
    _epsrel = utils._EngineFloat64ScalarType(
        epsrel, {'fun_name': fun_name, 'entity': 'epsrel'},
    ).to_ctypes()
    _result = utils._EngineFloat64ScalarType.empty_ctypes()
    _abserr = utils._EngineFloat64ScalarType.empty_ctypes()
    _lw = utils._EngineIntScalarType(
        lw, {'fun_name': fun_name, 'entity': 'lw'},
    ).to_ctypes()
    _lw_py = utils._EngineIntScalarType.to_py(_lw)
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_lw_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _liw = utils._EngineIntScalarType(
        liw, {'fun_name': fun_name, 'entity': 'liw'},
    ).to_ctypes()
    _liw_py = utils._EngineIntScalarType.to_py(_liw)
    _iw_cls = utils._EngineIntArrayType(
        iw, {'fun_name': fun_name, 'entity': 'iw'},
        exp_shape=(_liw_py,),
        check_contents=False,
    )
    _iw = _iw_cls.to_ctypes(can_reference_contents=False)
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _g,
        _gsub,
        _gh,
        _a,
        _b,
        _omega,
        _key,
        _epsabs,
        _epsrel,
        _result,
        _abserr,
        _w,
        _lw,
        _iw,
        _liw,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of subdivisions (LIMIT) has been reached:',
                    'LIMIT = {:s}, lw = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'liw = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Round-off error prevents the requested tolerance from being achieved:',
                    'epsabs = {:s} and epsrel = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99995:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Extremely bad integrand behaviour occurs around the sub-interval',
                    '({:s},{:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99994:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Round-off error is detected in the extrapolation table.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99993:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The integral is probably divergent or slowly convergent.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, key = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: key >= 1 and key <= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, liw = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: liw >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lw = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lw >= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _w_cls.refresh_py(_w)
        _iw_cls.refresh_py(_iw)
        _result_py = utils._EngineFloat64ScalarType.to_py(_result)
        _abserr_py = utils._EngineFloat64ScalarType.to_py(_abserr)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['result', 'abserr'],
        )(
            _result_py,
            _abserr_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim1_fin_wsing(g, a, b, alfa, beta, key, epsabs, epsrel, w, lw, iw, liw, data=None):
    u"""
    One-dimensional quadrature, adaptive, finite interval, weight
    function with end-point singularities of algebraico-logarithmic
    type.

    For full information please refer to the NAG Library document for
    d01ap

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01apf.html

    Parameters
    ----------
    g : callable retval = g(x, data=None)
        `g` must return the value of the function g at a given point
        `x`.

        Parameters
        ~~~~~~~~~~
        x : float
            The point at which the function g must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of g(x) evaluated at `x`.

    a : float
        a, the lower limit of integration.

    b : float
        b, the upper limit of integration.

    alfa : float
        The argument alpha in the weight function.

    beta : float
        The argument beta in the weight function.

    key : int
        Indicates which weight function is to be used.

        `key` = 1

          w(x) = (x-a)^alpha(b-x)^beta.

        `key` = 2

          w(x) = (x-a)^alpha(b-x)^beta ln(x-a).

        `key` = 3

          w(x) = (x-a)^alpha(b-x)^beta ln(b-x).

        `key` = 4

          w(x) = (x-a)^alpha(b-x)^beta ln(x-a)ln(b-x).

    epsabs : float
        The absolute accuracy required. If `epsabs` is negative, the
        absolute value is used. See [Accuracy].

    epsrel : float
        The relative accuracy required. If `epsrel` is negative, the
        absolute value is used. See [Accuracy].

    w : float, shape (`lw`), modified in place
        On exit: details of the computation see [Further Comments] for
        more information.

    lw : int
        The value of `lw` (together with that of `liw`) imposes a bound
        on the number of sub-intervals into which the interval of
        integration may be divided by the function. The number of
        sub-intervals cannot exceed `lw`/4. The more difficult the
        integrand, the larger `lw` should be.

    iw : int, shape (`liw`), modified in place
        On exit: `iw`[0] contains the actual number of sub-intervals
        used. The rest of the array is used as workspace.

    liw : int
        The number of sub-intervals into which the interval of
        integration may be divided cannot exceed `liw`.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    result : float
        The approximation to the integral I.

    abserr : float
        An estimate of the modulus of the absolute error, which should
        be an upper bound for |I-`result`|.

    Raises
    ------
    NagValueError
        (`errno` 4)
            On entry, `key` = *<value>*.

            Constraint: `key` <= 4.

        (`errno` 4)
            On entry, `key` = *<value>*.

            Constraint: `key` >= 1.

        (`errno` 4)
            On entry, `beta` = *<value>*.

            Constraint: `beta` > -1.0.

        (`errno` 4)
            On entry, `alfa` = *<value>*.

            Constraint: `alfa` > -1.0.

        (`errno` 4)
            On entry, `b` = *<value>* and `a` = *<value>*.

            Constraint: `b` > `a`.

        (`errno` 5)
            On entry, `liw` = *<value>*.

            Constraint: `liw` >= 2.

        (`errno` 5)
            On entry, `lw` = *<value>*.

            Constraint: `lw` >= 8.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The maximum number of subdivisions (LIMIT) has been reached:
            LIMIT = *<value>*, `lw` = *<value>* and `liw` = *<value>*.

        (`errno` 2)
            Round-off error prevents the requested tolerance from being
            achieved: `epsabs` = *<value>* and `epsrel` = *<value>*.

        (`errno` 3)
            Extremely bad integrand behaviour occurs around the
            sub-interval (*<value>*,*<value>*).

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.dim1_fin_wsing'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_g_name = fun_name + ':g'

    gh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_gh(_g, _gsub, en_data, x, retval, _iuser, _ruser):
        """Helper for g."""
        try:
            _x_gh = utils._EngineFloat64ScalarType.to_py(x.contents)
            if data is not None:
                _retval_gh = g(
                    _x_gh,
                    data,
                )
            else:
                _retval_gh = g(
                    _x_gh,
                )
            utils._EngineFloat64ScalarType(
                _retval_gh, {'fun_name': cb_g_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d01apft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        gh,
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_FLOAT64, # Input, alfa.
        utils._BYREF_FLOAT64, # Input, beta.
        utils._BYREF_INT, # Input, key.
        utils._BYREF_FLOAT64, # Input, epsabs.
        utils._BYREF_FLOAT64, # Input, epsrel.
        utils._BYREF_FLOAT64, # Output, result.
        utils._BYREF_FLOAT64, # Output, abserr.
        utils._EngineFloat64ArrayType.c_type, # Output, w[lw].
        utils._BYREF_INT, # Input, lw.
        utils._EngineIntArrayType.c_type, # Output, iw[liw].
        utils._BYREF_INT, # Input, liw.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    utils._check_callable(g, {'fun_name': fun_name, 'entity': 'g'})
    _g = utils._UNUSED_CALLBACK
    _gsub = utils._UNUSED_CALLBACK
    _gh = utils._cast_callback(
        gh, cb_gh,
        {'fun_name': fun_name, 'entity': 'g'},
    )
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _alfa = utils._EngineFloat64ScalarType(
        alfa, {'fun_name': fun_name, 'entity': 'alfa'},
    ).to_ctypes()
    _beta = utils._EngineFloat64ScalarType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
    ).to_ctypes()
    _key = utils._EngineIntScalarType(
        key, {'fun_name': fun_name, 'entity': 'key'},
    ).to_ctypes()
    _epsabs = utils._EngineFloat64ScalarType(
        epsabs, {'fun_name': fun_name, 'entity': 'epsabs'},
    ).to_ctypes()
    _epsrel = utils._EngineFloat64ScalarType(
        epsrel, {'fun_name': fun_name, 'entity': 'epsrel'},
    ).to_ctypes()
    _result = utils._EngineFloat64ScalarType.empty_ctypes()
    _abserr = utils._EngineFloat64ScalarType.empty_ctypes()
    _lw = utils._EngineIntScalarType(
        lw, {'fun_name': fun_name, 'entity': 'lw'},
    ).to_ctypes()
    _lw_py = utils._EngineIntScalarType.to_py(_lw)
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_lw_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _liw = utils._EngineIntScalarType(
        liw, {'fun_name': fun_name, 'entity': 'liw'},
    ).to_ctypes()
    _liw_py = utils._EngineIntScalarType.to_py(_liw)
    _iw_cls = utils._EngineIntArrayType(
        iw, {'fun_name': fun_name, 'entity': 'iw'},
        exp_shape=(_liw_py,),
        check_contents=False,
    )
    _iw = _iw_cls.to_ctypes(can_reference_contents=False)
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _g,
        _gsub,
        _gh,
        _a,
        _b,
        _alfa,
        _beta,
        _key,
        _epsabs,
        _epsrel,
        _result,
        _abserr,
        _w,
        _lw,
        _iw,
        _liw,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of subdivisions (LIMIT) has been reached:',
                    'LIMIT = {:s}, lw = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'liw = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99991:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Round-off error prevents the requested tolerance from being achieved:',
                    'epsabs = {:s} and epsrel = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99990:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Extremely bad integrand behaviour occurs around the sub-interval',
                    '({:s},{:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, key = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: key <= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, key = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: key >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, beta = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: beta > -1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, alfa = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: alfa > -1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, b = {:s} and a = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: b > a.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, liw = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: liw >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lw = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lw >= 8.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _w_cls.refresh_py(_w)
        _iw_cls.refresh_py(_iw)
        _result_py = utils._EngineFloat64ScalarType.to_py(_result)
        _abserr_py = utils._EngineFloat64ScalarType.to_py(_abserr)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['result', 'abserr'],
        )(
            _result_py,
            _abserr_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim1_fin_wcauchy(g, a, b, c, epsabs, epsrel, w, lw, iw, liw, data=None):
    u"""
    One-dimensional quadrature, adaptive, finite interval, weight
    function 1/(x-c), Cauchy principal value (Hilbert transform).

    For full information please refer to the NAG Library document for
    d01aq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01aqf.html

    Parameters
    ----------
    g : callable retval = g(x, data=None)
        `g` must return the value of the function g at a given point
        `x`.

        Parameters
        ~~~~~~~~~~
        x : float
            The point at which the function g must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of g(x) evaluated at `x`.

    a : float
        a, the lower limit of integration.

    b : float
        b, the upper limit of integration. It is not necessary that a <
        b.

    c : float
        The argument c in the weight function.

    epsabs : float
        The absolute accuracy required. If `epsabs` is negative, the
        absolute value is used. See [Accuracy].

    epsrel : float
        The relative accuracy required. If `epsrel` is negative, the
        absolute value is used. See [Accuracy].

    w : float, shape (`lw`), modified in place
        On exit: details of the computation see [Further Comments] for
        more information.

    lw : int
        The value of `lw` (together with that of `liw`) imposes a bound
        on the number of sub-intervals into which the interval of
        integration may be divided by the function. The number of
        sub-intervals cannot exceed `lw`/4. The more difficult the
        integrand, the larger `lw` should be.

    iw : int, shape (`liw`), modified in place
        On exit: `iw`[0] contains the actual number of sub-intervals
        used. The rest of the array is used as workspace.

    liw : int
        The number of sub-intervals into which the interval of
        integration may be divided cannot exceed `liw`.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    result : float
        The approximation to the integral I.

    abserr : float
        An estimate of the modulus of the absolute error, which should
        be an upper bound for |I-`result`|.

    Raises
    ------
    NagValueError
        (`errno` 4)
            On entry, `a` = *<value>*, `b` = *<value>* and `c` =
            *<value>*.

            Constraint: `c` != `a` and `c` != `b`.

        (`errno` 5)
            On entry, `liw` = *<value>*.

            Constraint: `liw` >= 1.

        (`errno` 5)
            On entry, `lw` = *<value>*.

            Constraint: `lw` >= 4.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The maximum number of subdivisions (LIMIT) has been reached:
            LIMIT = *<value>*, `lw` = *<value>* and `liw` = *<value>*.

        (`errno` 2)
            Round-off error prevents the requested tolerance from being
            achieved: `epsabs` = *<value>* and `epsrel` = *<value>*.

        (`errno` 3)
            Extremely bad integrand behaviour occurs around the
            sub-interval (*<value>*,*<value>*).

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.dim1_fin_wcauchy'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_g_name = fun_name + ':g'

    gh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_gh(_g, _gsub, en_data, x, retval, _iuser, _ruser):
        """Helper for g."""
        try:
            _x_gh = utils._EngineFloat64ScalarType.to_py(x.contents)
            if data is not None:
                _retval_gh = g(
                    _x_gh,
                    data,
                )
            else:
                _retval_gh = g(
                    _x_gh,
                )
            utils._EngineFloat64ScalarType(
                _retval_gh, {'fun_name': cb_g_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d01aqft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        gh,
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_FLOAT64, # Input, c.
        utils._BYREF_FLOAT64, # Input, epsabs.
        utils._BYREF_FLOAT64, # Input, epsrel.
        utils._BYREF_FLOAT64, # Output, result.
        utils._BYREF_FLOAT64, # Output, abserr.
        utils._EngineFloat64ArrayType.c_type, # Output, w[lw].
        utils._BYREF_INT, # Input, lw.
        utils._EngineIntArrayType.c_type, # Output, iw[liw].
        utils._BYREF_INT, # Input, liw.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    utils._check_callable(g, {'fun_name': fun_name, 'entity': 'g'})
    _g = utils._UNUSED_CALLBACK
    _gsub = utils._UNUSED_CALLBACK
    _gh = utils._cast_callback(
        gh, cb_gh,
        {'fun_name': fun_name, 'entity': 'g'},
    )
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _c = utils._EngineFloat64ScalarType(
        c, {'fun_name': fun_name, 'entity': 'c'},
    ).to_ctypes()
    _epsabs = utils._EngineFloat64ScalarType(
        epsabs, {'fun_name': fun_name, 'entity': 'epsabs'},
    ).to_ctypes()
    _epsrel = utils._EngineFloat64ScalarType(
        epsrel, {'fun_name': fun_name, 'entity': 'epsrel'},
    ).to_ctypes()
    _result = utils._EngineFloat64ScalarType.empty_ctypes()
    _abserr = utils._EngineFloat64ScalarType.empty_ctypes()
    _lw = utils._EngineIntScalarType(
        lw, {'fun_name': fun_name, 'entity': 'lw'},
    ).to_ctypes()
    _lw_py = utils._EngineIntScalarType.to_py(_lw)
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_lw_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _liw = utils._EngineIntScalarType(
        liw, {'fun_name': fun_name, 'entity': 'liw'},
    ).to_ctypes()
    _liw_py = utils._EngineIntScalarType.to_py(_liw)
    _iw_cls = utils._EngineIntArrayType(
        iw, {'fun_name': fun_name, 'entity': 'iw'},
        exp_shape=(_liw_py,),
        check_contents=False,
    )
    _iw = _iw_cls.to_ctypes(can_reference_contents=False)
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _g,
        _gsub,
        _gh,
        _a,
        _b,
        _c,
        _epsabs,
        _epsrel,
        _result,
        _abserr,
        _w,
        _lw,
        _iw,
        _liw,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of subdivisions (LIMIT) has been reached:',
                    'LIMIT = {:s}, lw = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'liw = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Round-off error prevents the requested tolerance from being achieved:',
                    'epsabs = {:s} and epsrel = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99995:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Extremely bad integrand behaviour occurs around the sub-interval',
                    '({:s},{:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a = {:s}, b = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'c = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: c != a and c != b.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, liw = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: liw >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lw = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lw >= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _w_cls.refresh_py(_w)
        _iw_cls.refresh_py(_iw)
        _result_py = utils._EngineFloat64ScalarType.to_py(_result)
        _abserr_py = utils._EngineFloat64ScalarType.to_py(_abserr)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['result', 'abserr'],
        )(
            _result_py,
            _abserr_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim1_indef(a, b, f, relacc, absacc, maxrul, iparm, alpha, data=None):
    u"""
    One-dimensional quadrature, non-adaptive, finite interval with
    provision for indefinite integrals.

    For full information please refer to the NAG Library document for
    d01ar

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01arf.html

    Parameters
    ----------
    a : float
        a, the lower limit of integration.

    b : float
        b, the upper limit of integration. It is not necessary that a <
        b.

    f : callable retval = f(x, data=None)
        `f` must return the value of the integrand at a given point.

        Parameters
        ~~~~~~~~~~
        x : float
            The point in [a,b] at which the integrand f must be
            evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of f(x) evaluated at `x`.

    relacc : float
        The relative accuracy required. If convergence according to
        absolute accuracy is required, `relacc` should be set to zero
        (but see also [Accuracy]). If `relacc` < 0.0, its absolute value
        is used.

        If `iparm` = 2, `relacc` is not used.

    absacc : float
        The absolute accuracy required. If convergence according to
        relative accuracy is required, `absacc` should be set to zero
        (but see also [Accuracy]). If `absacc` < 0.0, its absolute value
        is used.

        If `iparm` = 2, `absacc` is not used.

    maxrul : int
        The maximum number of successive rules that may be used.

    iparm : int
        Indicates the task to be performed by the function.

        `iparm` = 0

          Only the definite integral over [a,b] is evaluated.

        `iparm` = 1

          As well as the definite integral, the expansion of the
          integrand in Legendre polynomials over [a,b] is calculated,
          using the same values of the integrand as used to compute the
          integral. The expansion coefficients, and some other
          quantities, are returned in `alpha` for later use in computing
          indefinite integrals.

        `iparm` = 2

          f(t) is integrated analytically over [a,b] using the
          previously computed expansion, stored in `alpha`. No further
          evaluations of the integrand are required. The function must
          previously have been called with `iparm` = 1 and the interval
          [a,b] must lie within that specified for the previous call. In
          this case only the arguments `a`, `b`, `iparm`, `ans` and
          `alpha` are used.

    alpha : float, shape (390), modified in place
        On entry: if `iparm` = 2, `alpha` must contain the coefficients
        of the Legendre expansions of the integrand, as returned by a
        previous call of ``dim1_indef`` with `iparm` = 1 and a range
        containing the present range.

        If `iparm` = 0 or 1, `alpha` need not be set on entry.

        On exit: if `iparm` = 1, the first m elements of `alpha` hold
        the coefficients of the Legendre expansion of the integrand, and
        the value of m is stored in `alpha`[389]. `alpha` must not be
        changed between a call with `iparm` = 1 and subsequent calls
        with `iparm` = 2.

        If `iparm` = 2, the first m elements of `alpha` are unchanged on
        exit.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    acc : float
        If `iparm` = 0 or 1, `acc` contains the absolute value of the
        difference between the last two successive estimates of the
        integral. This may be used as a measure of the accuracy actually
        achieved.

        If `iparm` = 2, `acc` is not used.

    ans : float
        The estimated value of the integral.

    n : int
        When `iparm` = 0 or 1, `n` contains the number of integrand
        evaluations used in the calculation of the integral.

        If `iparm` = 2, `n` is not used.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `iparm` = *<value>*.

            Constraint: `iparm` = 0, 1 or 2.

        (`errno` 3)
            The function was called with `iparm` = 2, but a previous
            call with `iparm` = 1 has been omitted, or made with zero
            integration interval.

        (`errno` 4)
            The function was called with `iparm` = 2, but the interval
            for indefinite integration is not contained within the
            interval specified in the previous call with `iparm` = 1.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            All `maxrul` rules have been used but the integral has not
            converged to the accuracy requested. `maxrul` = *<value>*.

        (`errno` 1)
            Failure in convergence during the run with `iparm` = 1 in
            which the Legendre expansion was created.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.dim1_indef'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, _fsub, en_data, x, retval, _iuser, _ruser):
        """Helper for f."""
        try:
            _x_fh = utils._EngineFloat64ScalarType.to_py(x.contents)
            if data is not None:
                _retval_fh = f(
                    _x_fh,
                    data,
                )
            else:
                _retval_fh = f(
                    _x_fh,
                )
            utils._EngineFloat64ScalarType(
                _retval_fh, {'fun_name': cb_f_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d01arft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._BYREF_FLOAT64, # Input, relacc.
        utils._BYREF_FLOAT64, # Input, absacc.
        utils._BYREF_INT, # Input, maxrul.
        utils._BYREF_INT, # Input, iparm.
        utils._BYREF_FLOAT64, # Output, acc.
        utils._BYREF_FLOAT64, # Output, ans.
        utils._BYREF_INT, # Output, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, alpha[390].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fsub = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _relacc = utils._EngineFloat64ScalarType(
        relacc, {'fun_name': fun_name, 'entity': 'relacc'},
    ).to_ctypes()
    _absacc = utils._EngineFloat64ScalarType(
        absacc, {'fun_name': fun_name, 'entity': 'absacc'},
    ).to_ctypes()
    _maxrul = utils._EngineIntScalarType(
        maxrul, {'fun_name': fun_name, 'entity': 'maxrul'},
    ).to_ctypes()
    _iparm = utils._EngineIntScalarType(
        iparm, {'fun_name': fun_name, 'entity': 'iparm'},
    ).to_ctypes()
    _acc = utils._EngineFloat64ScalarType.empty_ctypes()
    _ans = utils._EngineFloat64ScalarType.empty_ctypes()
    _n = utils._EngineIntScalarType.empty_ctypes()
    _alpha_cls = utils._EngineFloat64ArrayType(
        alpha, {'fun_name': fun_name, 'entity': 'alpha'},
        exp_shape=(390,),
    )
    _alpha = _alpha_cls.to_ctypes()
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _a,
        _b,
        _f,
        _fsub,
        _fh,
        _relacc,
        _absacc,
        _maxrul,
        _iparm,
        _acc,
        _ans,
        _n,
        _alpha,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'All maxrul rules have been used but the integral has not',
                    'converged to the accuracy requested.',
                    'maxrul = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Failure in convergence during the run with iparm = 1',
                    'in which the Legendre expansion was created.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, iparm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: iparm = 0, 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function was called with iparm = 2, but a previous call with',
                    'iparm = 1 has been omitted, or made with zero integration interval.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function was called with iparm = 2, but the interval for indefinite',
                    'integration is not contained within the interval specified in the',
                    'previous call with iparm = 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _alpha_cls.refresh_py(_alpha)
        _acc_py = utils._EngineFloat64ScalarType.to_py(_acc)
        _ans_py = utils._EngineFloat64ScalarType.to_py(_ans)
        _n_py = utils._EngineIntScalarType.to_py(_n)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['acc', 'ans', 'n'],
        )(
            _acc_py,
            _ans_py,
            _n_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim1_inf_wtrig(g, a, omega, key, epsabs, limlst, erlst, rslst, ierlst, lw, iw, liw, data=None):
    u"""
    One-dimensional quadrature, adaptive, semi-infinite interval, weight
    function cos(omega x) or sin(omega x).

    For full information please refer to the NAG Library document for
    d01as

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01asf.html

    Parameters
    ----------
    g : callable retval = g(x, data=None)
        `g` must return the value of the function g at a given point
        `x`.

        Parameters
        ~~~~~~~~~~
        x : float
            The point at which the function g must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of g(x) evaluated at `x`.

    a : float
        a, the lower limit of integration.

    omega : float
        The argument omega in the weight function of the transform.

    key : int
        Indicates which integral is to be computed.

        `key` = 1

          w(x) = cos(omega x).

        `key` = 2

          w(x) = sin(omega x).

    epsabs : float
        The absolute accuracy required. If `epsabs` is negative, the
        absolute value is used. See [Accuracy].

    limlst : int
        An upper bound on the number of intervals C_k needed for the
        integration.

    erlst : float, shape (`limlst`), modified in place
        On exit: `erlst`[k-1] contains the error estimate corresponding
        to the integral contribution over the interval C_k, for k =
        1,2,...,`lst`.

    rslst : float, shape (`limlst`), modified in place
        On exit: `rslst`[k-1] contains the integral contribution over
        the interval C_k, for k = 1,2,...,`lst`.

    ierlst : int, shape (`limlst`), modified in place
        On exit: `ierlst`[k-1] contains the error flag corresponding to
        `rslst`[k-1], for k = 1,2,...,`lst`.

        In the cases `errno` = 7, 8 or 9, additional information about
        the cause of the error can be obtained from the array `ierlst`,
        as follows:

        `ierlst`[k-1] = 1

          The maximum number of subdivisions = min(`lw`/4,`liw`/2) has
          been achieved on the kth interval.

        `ierlst`[k-1] = 2

          Occurrence of round-off error is detected and prevents the
          tolerance imposed on the kth interval from being achieved.

        `ierlst`[k-1] = 3

          Extremely bad integrand behaviour occurs at some points of the
          kth interval.

        `ierlst`[k-1] = 4

          The integration procedure over the kth interval does not
          converge (to within the required accuracy) due to round-off in
          the extrapolation procedure invoked on this interval. It is
          assumed that the result on this interval is the best which can
          be obtained.

        `ierlst`[k-1] = 5

          The integral over the kth interval is probably divergent or
          slowly convergent. It must be noted that divergence can occur
          with any other value of `ierlst`[k-1].

    lw : int
        The value of `lw` (together with that of `liw`) imposes a bound
        on the number of sub-intervals into which each interval C_k may
        be divided by the function. The number of sub-intervals cannot
        exceed `lw`/4. The more difficult the integrand, the larger `lw`
        should be.

    iw : int, shape (`liw`), modified in place
        On exit: `iw`[0] contains the maximum number of sub-intervals
        actually used for integrating over any of the intervals C_k. The
        rest of the array is used as workspace.

    liw : int
        The number of sub-intervals into which each interval C_k may be
        divided cannot exceed `liw`/2.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    result : float
        The approximation to the integral I.

    abserr : float
        An estimate of the modulus of the absolute error, which should
        be an upper bound for |I-`result`|.

    lst : int
        The number of intervals C_k actually used for the integration.

    Raises
    ------
    NagValueError
        (`errno` 6)
            On entry, `limlst` = *<value>*.

            Constraint: `limlst` >= 3.

        (`errno` 6)
            On entry, `key` = *<value>*.

            Constraint: `key` <= 2.

        (`errno` 6)
            On entry, `key` = *<value>*.

            Constraint: `key` >= 1.

        (`errno` 10)
            On entry, `liw` = *<value>*.

            Constraint: `liw` >= 2.

        (`errno` 10)
            On entry, `lw` = *<value>*.

            Constraint: `lw` >= 4.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The maximum number of subdivisions (LIMIT) has been reached:
            LIMIT = *<value>*, `lw` = *<value>* and `liw` = *<value>*.

        (`errno` 2)
            Round-off error prevents the requested tolerance from being
            achieved: `epsabs` = *<value>*.

        (`errno` 3)
            Extremely bad integrand behaviour occurs around the
            sub-interval (*<value>*,*<value>*).

        (`errno` 4)
            Round-off error is detected in the extrapolation table.

        (`errno` 5)
            The integral is probably divergent or slowly convergent.

        (`errno` 7)
            Bad integration behaviour occurs within interval K: K =
            *<value>*: (*<value>*,*<value>*). Inspect `ierlst` for more
            details.

        (`errno` 8)
            The maximum number of intervals (`limlst`) has been reached:
            `limlst` = *<value>*. Inspect `ierlst` for more details.

        (`errno` 9)
            Extrapolation does not converge to the requested accuracy.
            Inspect `ierlst` for more details.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.dim1_inf_wtrig'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_g_name = fun_name + ':g'

    gh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_gh(_g, _gsub, en_data, x, retval, _iuser, _ruser):
        """Helper for g."""
        try:
            _x_gh = utils._EngineFloat64ScalarType.to_py(x.contents)
            if data is not None:
                _retval_gh = g(
                    _x_gh,
                    data,
                )
            else:
                _retval_gh = g(
                    _x_gh,
                )
            utils._EngineFloat64ScalarType(
                _retval_gh, {'fun_name': cb_g_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d01asft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        gh,
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, omega.
        utils._BYREF_INT, # Input, key.
        utils._BYREF_FLOAT64, # Input, epsabs.
        utils._BYREF_FLOAT64, # Output, result.
        utils._BYREF_FLOAT64, # Output, abserr.
        utils._BYREF_INT, # Input, limlst.
        utils._BYREF_INT, # Output, lst.
        utils._EngineFloat64ArrayType.c_type, # Output, erlst[limlst].
        utils._EngineFloat64ArrayType.c_type, # Output, rslst[limlst].
        utils._EngineIntArrayType.c_type, # Output, ierlst[limlst].
        utils._EngineFloat64ArrayType.c_type, # Workspace, w.
        utils._BYREF_INT, # Input, lw.
        utils._EngineIntArrayType.c_type, # Output, iw[liw].
        utils._BYREF_INT, # Input, liw.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    utils._check_callable(g, {'fun_name': fun_name, 'entity': 'g'})
    _g = utils._UNUSED_CALLBACK
    _gsub = utils._UNUSED_CALLBACK
    _gh = utils._cast_callback(
        gh, cb_gh,
        {'fun_name': fun_name, 'entity': 'g'},
    )
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _omega = utils._EngineFloat64ScalarType(
        omega, {'fun_name': fun_name, 'entity': 'omega'},
    ).to_ctypes()
    _key = utils._EngineIntScalarType(
        key, {'fun_name': fun_name, 'entity': 'key'},
    ).to_ctypes()
    _epsabs = utils._EngineFloat64ScalarType(
        epsabs, {'fun_name': fun_name, 'entity': 'epsabs'},
    ).to_ctypes()
    _result = utils._EngineFloat64ScalarType.empty_ctypes()
    _abserr = utils._EngineFloat64ScalarType.empty_ctypes()
    _limlst = utils._EngineIntScalarType(
        limlst, {'fun_name': fun_name, 'entity': 'limlst'},
    ).to_ctypes()
    _lst = utils._EngineIntScalarType.empty_ctypes()
    _limlst_py = utils._EngineIntScalarType.to_py(_limlst)
    _erlst_cls = utils._EngineFloat64ArrayType(
        erlst, {'fun_name': fun_name, 'entity': 'erlst'},
        exp_shape=(_limlst_py,),
        check_contents=False,
    )
    _erlst = _erlst_cls.to_ctypes(can_reference_contents=False)
    _rslst_cls = utils._EngineFloat64ArrayType(
        rslst, {'fun_name': fun_name, 'entity': 'rslst'},
        exp_shape=(_limlst_py,),
        check_contents=False,
    )
    _rslst = _rslst_cls.to_ctypes(can_reference_contents=False)
    _ierlst_cls = utils._EngineIntArrayType(
        ierlst, {'fun_name': fun_name, 'entity': 'ierlst'},
        exp_shape=(_limlst_py,),
        check_contents=False,
    )
    _ierlst = _ierlst_cls.to_ctypes(can_reference_contents=False)
    _w = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lw = utils._EngineIntScalarType(
        lw, {'fun_name': fun_name, 'entity': 'lw'},
    ).to_ctypes()
    _liw = utils._EngineIntScalarType(
        liw, {'fun_name': fun_name, 'entity': 'liw'},
    ).to_ctypes()
    _liw_py = utils._EngineIntScalarType.to_py(_liw)
    _iw_cls = utils._EngineIntArrayType(
        iw, {'fun_name': fun_name, 'entity': 'iw'},
        exp_shape=(_liw_py,),
        check_contents=False,
    )
    _iw = _iw_cls.to_ctypes(can_reference_contents=False)
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _g,
        _gsub,
        _gh,
        _a,
        _omega,
        _key,
        _epsabs,
        _result,
        _abserr,
        _limlst,
        _lst,
        _erlst,
        _rslst,
        _ierlst,
        _w,
        _lw,
        _iw,
        _liw,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of subdivisions (LIMIT) has been reached:',
                    'LIMIT = {:s}, lw = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'liw = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99993:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Round-off error prevents the requested tolerance from being achieved:',
                    'epsabs = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99987:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Extremely bad integrand behaviour occurs around the sub-interval',
                    '({:s},{:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99992:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Round-off error is detected in the extrapolation table.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99991:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The integral is probably divergent or slowly convergent.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, limlst = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: limlst >= 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, key = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: key <= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, key = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: key >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99990:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Bad integration behaviour occurs within interval K:',
                    'K = {:s}:'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    '({:s},{:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Inspect ierlst for more details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99989:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of intervals (limlst) has been reached:',
                    'limlst = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Inspect ierlst for more details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99988:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Extrapolation does not converge to the requested accuracy.',
                    'Inspect ierlst for more details.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, liw = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: liw >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lw = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lw >= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _erlst_cls.refresh_py(_erlst)
        _rslst_cls.refresh_py(_rslst)
        _ierlst_cls.refresh_py(_ierlst)
        _iw_cls.refresh_py(_iw)
        _result_py = utils._EngineFloat64ScalarType.to_py(_result)
        _abserr_py = utils._EngineFloat64ScalarType.to_py(_abserr)
        _lst_py = utils._EngineIntScalarType.to_py(_lst)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['result', 'abserr', 'lst'],
        )(
            _result_py,
            _abserr_py,
            _lst_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim1_fin_bad_vec(f, a, b, epsabs, epsrel, w, lw, iw, liw, data=None):
    u"""
    One-dimensional quadrature, adaptive, finite interval, variant of
    ``dim1_fin_bad`` efficient on vector machines.

    For full information please refer to the NAG Library document for
    d01at

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01atf.html

    Parameters
    ----------
    f : callable f(x, fv, n, data=None)
        `f` must return the values of the integrand f at a set of
        points.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (`n`)
            The points at which the integrand f must be evaluated.

        fv : float, ndarray, shape (`n`), to be modified in place
            On exit: `fv`[j-1] must contain the value of f at the point
            `x`[j-1], for j = 1,2,...,`n`.

        n : int
            The number of points at which the integrand is to be
            evaluated. The actual value of `n` is always 21.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    a : float
        a, the lower limit of integration.

    b : float
        b, the upper limit of integration. It is not necessary that a <
        b.

    epsabs : float
        The absolute accuracy required. If `epsabs` is negative, the
        absolute value is used. See [Accuracy].

    epsrel : float
        The relative accuracy required. If `epsrel` is negative, the
        absolute value is used. See [Accuracy].

    w : float, shape (`lw`), modified in place
        On exit: details of the computation see [Further Comments] for
        more information.

    lw : int
        The value of `lw` (together with that of `liw`) imposes a bound
        on the number of sub-intervals into which the interval of
        integration may be divided by the function. The number of
        sub-intervals cannot exceed `lw`/4. The more difficult the
        integrand, the larger `lw` should be.

    iw : int, shape (`liw`), modified in place
        On exit: `iw`[0] contains the actual number of sub-intervals
        used. The rest of the array is used as workspace.

    liw : int
        The number of sub-intervals into which the interval of
        integration may be divided cannot exceed `liw`.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    result : float
        The approximation to the integral I.

    abserr : float
        An estimate of the modulus of the absolute error, which should
        be an upper bound for |I-`result`|.

    Raises
    ------
    NagValueError
        (`errno` 6)
            On entry, `liw` = *<value>*.

            Constraint: `liw` >= 1.

        (`errno` 6)
            On entry, `lw` = *<value>*.

            Constraint: `lw` >= 4.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The maximum number of subdivisions (LIMIT) has been reached:
            LIMIT = *<value>*, `lw` = *<value>* and `liw` = *<value>*.

        (`errno` 2)
            Round-off error prevents the requested tolerance from being
            achieved: `epsabs` = *<value>* and `epsrel` = *<value>*.

        (`errno` 3)
            Extremely bad integrand behaviour occurs around the
            sub-interval (*<value>*,*<value>*).

        (`errno` 4)
            Round-off error is detected in the extrapolation table.

        (`errno` 5)
            The integral is probably divergent or slowly convergent.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.dim1_fin_bad_vec'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, fv[n].
        utils._BYREF_INT, # Input, n.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, en_data, x, fv, n, _iuser, _ruser):
        """Helper for f."""
        try:
            _n_fh = utils._EngineIntScalarType.to_py(n.contents)
            _x_fh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_n_fh,),
            )
            _fv_fh = utils._EngineFloat64ArrayType.from_pointer(
                fv,
                (_n_fh,),
                can_reference_contents=False,
            )
            if data is not None:
                f(
                    _x_fh,
                    _fv_fh,
                    _n_fh,
                    data,
                )
            else:
                f(
                    _x_fh,
                    _fv_fh,
                    _n_fh,
                )
            utils._unlink_pointer(x)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _fv_fh, {'fun_name': cb_f_name, 'returned': 'fv'}, fv,
            )
            utils._unlink_pointer(fv)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d01atft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_FLOAT64, # Input, epsabs.
        utils._BYREF_FLOAT64, # Input, epsrel.
        utils._BYREF_FLOAT64, # Output, result.
        utils._BYREF_FLOAT64, # Output, abserr.
        utils._EngineFloat64ArrayType.c_type, # Output, w[lw].
        utils._BYREF_INT, # Input, lw.
        utils._EngineIntArrayType.c_type, # Output, iw[liw].
        utils._BYREF_INT, # Input, liw.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _epsabs = utils._EngineFloat64ScalarType(
        epsabs, {'fun_name': fun_name, 'entity': 'epsabs'},
    ).to_ctypes()
    _epsrel = utils._EngineFloat64ScalarType(
        epsrel, {'fun_name': fun_name, 'entity': 'epsrel'},
    ).to_ctypes()
    _result = utils._EngineFloat64ScalarType.empty_ctypes()
    _abserr = utils._EngineFloat64ScalarType.empty_ctypes()
    _lw = utils._EngineIntScalarType(
        lw, {'fun_name': fun_name, 'entity': 'lw'},
    ).to_ctypes()
    _lw_py = utils._EngineIntScalarType.to_py(_lw)
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_lw_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _liw = utils._EngineIntScalarType(
        liw, {'fun_name': fun_name, 'entity': 'liw'},
    ).to_ctypes()
    _liw_py = utils._EngineIntScalarType.to_py(_liw)
    _iw_cls = utils._EngineIntArrayType(
        iw, {'fun_name': fun_name, 'entity': 'iw'},
        exp_shape=(_liw_py,),
        check_contents=False,
    )
    _iw = _iw_cls.to_ctypes(can_reference_contents=False)
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _f,
        _fh,
        _a,
        _b,
        _epsabs,
        _epsrel,
        _result,
        _abserr,
        _w,
        _lw,
        _iw,
        _liw,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of subdivisions (LIMIT) has been reached:',
                    'LIMIT = {:s}, lw = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'liw = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Round-off error prevents the requested tolerance from being achieved:',
                    'epsabs = {:s} and epsrel = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99995:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Extremely bad integrand behaviour occurs around the sub-interval',
                    '({:s},{:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99994:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Round-off error is detected in the extrapolation table.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99993:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The integral is probably divergent or slowly convergent.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, liw = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: liw >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lw = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lw >= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _w_cls.refresh_py(_w)
        _iw_cls.refresh_py(_iw)
        _result_py = utils._EngineFloat64ScalarType.to_py(_result)
        _abserr_py = utils._EngineFloat64ScalarType.to_py(_abserr)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['result', 'abserr'],
        )(
            _result_py,
            _abserr_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim1_fin_osc_vec(f, a, b, key, epsabs, epsrel, w, lw, iw, liw, data=None):
    u"""
    One-dimensional quadrature, adaptive, finite interval, variant of
    ``dim1_fin_osc`` efficient on vector machines.

    For full information please refer to the NAG Library document for
    d01au

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01auf.html

    Parameters
    ----------
    f : callable f(x, fv, n, data=None)
        `f` must return the values of the integrand f at a set of
        points.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (`n`)
            The points at which the integrand f must be evaluated.

        fv : float, ndarray, shape (`n`), to be modified in place
            On exit: `fv`[j-1] must contain the value of f at the point
            `x`[j-1], for j = 1,2,...,`n`.

        n : int
            The number of points at which the integrand is to be
            evaluated. The actual value of `n` is equal to the number of
            points in the Kronrod rule (see specification of `key`).

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    a : float
        a, the lower limit of integration.

    b : float
        b, the upper limit of integration. It is not necessary that a <
        b.

    key : int
        Indicates which integration rule is to be used.

        `key` = 1

          For the Gauss 7-point and Kronrod 15-point rule.

        `key` = 2

          For the Gauss 10-point and Kronrod 21-point rule.

        `key` = 3

          For the Gauss 15-point and Kronrod 31-point rule.

        `key` = 4

          For the Gauss 20-point and Kronrod 41-point rule.

        `key` = 5

          For the Gauss 25-point and Kronrod 51-point rule.

        `key` = 6

          For the Gauss 30-point and Kronrod 61-point rule.

    epsabs : float
        The absolute accuracy required. If `epsabs` is negative, the
        absolute value is used. See [Accuracy].

    epsrel : float
        The relative accuracy required. If `epsrel` is negative, the
        absolute value is used. See [Accuracy].

    w : float, shape (`lw`), modified in place
        On exit: details of the computation see [Further Comments] for
        more information.

    lw : int
        The value of `lw` (together with that of `liw`) imposes a bound
        on the number of sub-intervals into which the interval of
        integration may be divided by the function. The number of
        sub-intervals cannot exceed `lw`/4. The more difficult the
        integrand, the larger `lw` should be.

    iw : int, shape (`liw`), modified in place
        On exit: `iw`[0] contains the actual number of sub-intervals
        used. The rest of the array is used as workspace.

    liw : int
        The number of sub-intervals into which the interval of
        integration may be divided cannot exceed `liw`.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    result : float
        The approximation to the integral I.

    abserr : float
        An estimate of the modulus of the absolute error, which should
        be an upper bound for |I-`result`|.

    Raises
    ------
    NagValueError
        (`errno` 4)
            On entry, `key` = *<value>*.

            Constraint: `key` >= 1 and `key` <= 6.

        (`errno` 5)
            On entry, `liw` = *<value>*.

            Constraint: `liw` >= 1.

        (`errno` 5)
            On entry, `lw` = *<value>*.

            Constraint: `lw` >= 4.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The maximum number of subdivisions (LIMIT) has been reached:
            LIMIT = *<value>*, `lw` = *<value>* and `liw` = *<value>*.

        (`errno` 2)
            Round-off error prevents the requested tolerance from being
            achieved: `epsabs` = *<value>* and `epsrel` = *<value>*.

        (`errno` 3)
            Extremely bad integrand behaviour occurs around the
            sub-interval (*<value>*,*<value>*).

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.dim1_fin_osc_vec'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, fv[n].
        utils._BYREF_INT, # Input, n.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, en_data, x, fv, n, _iuser, _ruser):
        """Helper for f."""
        try:
            _n_fh = utils._EngineIntScalarType.to_py(n.contents)
            _x_fh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_n_fh,),
            )
            _fv_fh = utils._EngineFloat64ArrayType.from_pointer(
                fv,
                (_n_fh,),
                can_reference_contents=False,
            )
            if data is not None:
                f(
                    _x_fh,
                    _fv_fh,
                    _n_fh,
                    data,
                )
            else:
                f(
                    _x_fh,
                    _fv_fh,
                    _n_fh,
                )
            utils._unlink_pointer(x)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _fv_fh, {'fun_name': cb_f_name, 'returned': 'fv'}, fv,
            )
            utils._unlink_pointer(fv)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d01auft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_INT, # Input, key.
        utils._BYREF_FLOAT64, # Input, epsabs.
        utils._BYREF_FLOAT64, # Input, epsrel.
        utils._BYREF_FLOAT64, # Output, result.
        utils._BYREF_FLOAT64, # Output, abserr.
        utils._EngineFloat64ArrayType.c_type, # Output, w[lw].
        utils._BYREF_INT, # Input, lw.
        utils._EngineIntArrayType.c_type, # Output, iw[liw].
        utils._BYREF_INT, # Input, liw.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _key = utils._EngineIntScalarType(
        key, {'fun_name': fun_name, 'entity': 'key'},
    ).to_ctypes()
    _epsabs = utils._EngineFloat64ScalarType(
        epsabs, {'fun_name': fun_name, 'entity': 'epsabs'},
    ).to_ctypes()
    _epsrel = utils._EngineFloat64ScalarType(
        epsrel, {'fun_name': fun_name, 'entity': 'epsrel'},
    ).to_ctypes()
    _result = utils._EngineFloat64ScalarType.empty_ctypes()
    _abserr = utils._EngineFloat64ScalarType.empty_ctypes()
    _lw = utils._EngineIntScalarType(
        lw, {'fun_name': fun_name, 'entity': 'lw'},
    ).to_ctypes()
    _lw_py = utils._EngineIntScalarType.to_py(_lw)
    _w_cls = utils._EngineFloat64ArrayType(
        w, {'fun_name': fun_name, 'entity': 'w'},
        exp_shape=(_lw_py,),
        check_contents=False,
    )
    _w = _w_cls.to_ctypes(can_reference_contents=False)
    _liw = utils._EngineIntScalarType(
        liw, {'fun_name': fun_name, 'entity': 'liw'},
    ).to_ctypes()
    _liw_py = utils._EngineIntScalarType.to_py(_liw)
    _iw_cls = utils._EngineIntArrayType(
        iw, {'fun_name': fun_name, 'entity': 'iw'},
        exp_shape=(_liw_py,),
        check_contents=False,
    )
    _iw = _iw_cls.to_ctypes(can_reference_contents=False)
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _f,
        _fh,
        _a,
        _b,
        _key,
        _epsabs,
        _epsrel,
        _result,
        _abserr,
        _w,
        _lw,
        _iw,
        _liw,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The maximum number of subdivisions (LIMIT) has been reached:',
                    'LIMIT = {:s}, lw = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'liw = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Round-off error prevents the requested tolerance from being achieved:',
                    'epsabs = {:s} and epsrel = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99995:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Extremely bad integrand behaviour occurs around the sub-interval',
                    '({:s},{:s}).'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, key = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: key >= 1 and key <= 6.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, liw = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: liw >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lw = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lw >= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _w_cls.refresh_py(_w)
        _iw_cls.refresh_py(_iw)
        _result_py = utils._EngineFloat64ScalarType.to_py(_result)
        _abserr_py = utils._EngineFloat64ScalarType.to_py(_abserr)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['result', 'abserr'],
        )(
            _result_py,
            _abserr_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim1_gauss_wgen(itype, a, b, c, d, n, weight, abscis):
    u"""
    Calculation of weights and abscissae for Gaussian quadrature rules,
    general choice of rule.

    For full information please refer to the NAG Library document for
    d01bc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01bcf.html

    Parameters
    ----------
    itype : int
        Indicates the type of quadrature rule.

        `itype` = 0

          Gauss--Legendre, with normal weights.

        `itype` = 1

          Gauss--Jacobi, with normal weights.

        `itype` = -1

          Gauss--Jacobi, with adjusted weights.

        `itype` = 2

          Exponential Gauss, with normal weights.

        `itype` = -2

          Exponential Gauss, with adjusted weights.

        `itype` = 3

          Gauss--Laguerre, with normal weights.

        `itype` = -3

          Gauss--Laguerre, with adjusted weights.

        `itype` = 4

          Gauss--Hermite, with normal weights.

        `itype` = -4

          Gauss--Hermite, with adjusted weights.

        `itype` = 5

          Rational Gauss, with normal weights.

        `itype` = -5

          Rational Gauss, with adjusted weights.

    a : float
        The parameter `a` which occurs in the quadrature formula

    b : float
        The parameter `b` which occurs in the quadrature formula

    c : float
        The parameter `c` which occurs in the quadrature formula

    d : float
        The parameter `d` which occurs in the quadrature formula

    n : int
        n, the number of weights and abscissae to be returned. If
        `itype` = -2 or -4 and `c` != 0.0, an odd value of `n` may raise
        problems (see `errno` = 6).

    weight : float, shape (`n`), modified in place
        On exit: the `n` weights.

    abscis : float, shape (`n`), modified in place
        On exit: the `n` abscissae.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `itype` = *<value>*.

            Constraint: abs(`itype`) < 6.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 3)
            On entry, `a`, `b`, `c`, or `d` is not in the allowed range:
            `a` = *<value>*, `b` = *<value>* `c` = *<value>*, `d` =
            *<value>* and `itype` = *<value>*.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The algorithm for computing eigenvalues of a tridiagonal
            matrix has failed to converge.

        (`errno` 4)
            One or more of the weights are larger than rmax, the largest
            floating point number on this computer (see
            ``machine.real_largest``): rmax = *<value>*.

            Possible solutions are to use a smaller value of n; or, if
            using adjusted weights to change to normal weights.

        (`errno` 5)
            One or more of the weights are too small to be distinguished
            from zero on this machine.

            The underflowing weights are returned as zero, which may be
            a usable approximation.

            Possible solutions are to use a smaller value of n; or, if
            using normal weights, to change to adjusted weights.

        (`errno` 6)
            The contribution of the central abscissa to the summation is
            indeterminate.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.dim1_gauss_wgen'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d01bcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, itype.
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_FLOAT64, # Input, c.
        utils._BYREF_FLOAT64, # Input, d.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Output, weight[n].
        utils._EngineFloat64ArrayType.c_type, # Output, abscis[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _itype = utils._EngineIntScalarType(
        itype, {'fun_name': fun_name, 'entity': 'itype'},
    ).to_ctypes()
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _c = utils._EngineFloat64ScalarType(
        c, {'fun_name': fun_name, 'entity': 'c'},
    ).to_ctypes()
    _d = utils._EngineFloat64ScalarType(
        d, {'fun_name': fun_name, 'entity': 'd'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _weight_cls = utils._EngineFloat64ArrayType(
        weight, {'fun_name': fun_name, 'entity': 'weight'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _weight = _weight_cls.to_ctypes(can_reference_contents=False)
    _abscis_cls = utils._EngineFloat64ArrayType(
        abscis, {'fun_name': fun_name, 'entity': 'abscis'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _abscis = _abscis_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _itype,
        _a,
        _b,
        _c,
        _d,
        _n,
        _weight,
        _abscis,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm for computing eigenvalues of a tridiagonal matrix',
                    'has failed to converge.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itype = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: abs(itype) < 6.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a, b, c, or d is not in the allowed range:',
                    'a = {:s}, b = {:s}'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'c = {:s}, d = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'itype = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[5]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99995:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'One or more of the weights are larger than rmax, the largest floating',
                    'point number on this computer (see machine.real_largest):',
                    'rmax = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Possible solutions are to use a smaller value of n; or,',
                    'if using adjusted weights to change to normal weights.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99994:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'One or more of the weights are too small to be distinguished from zero on',
                    'this machine.',
                    'The underflowing weights are returned as zero, which may be a usable',
                    'approximation.',
                    'Possible solutions are to use a smaller value of n; or,',
                    'if using normal weights, to change to adjusted weights.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99993:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The contribution of the central abscissa to the summation',
                    'is indeterminate.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _weight_cls.refresh_py(_weight)
        _abscis_cls.refresh_py(_abscis)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim1_fin_smooth(f, a, b, epsabs, epsrel, data=None):
    u"""
    One-dimensional quadrature, non-adaptive, finite interval.

    For full information please refer to the NAG Library document for
    d01bd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01bdf.html

    Parameters
    ----------
    f : callable retval = f(x, data=None)
        `f` must return the value of the integrand f at a given point.

        Parameters
        ~~~~~~~~~~
        x : float
            The point at which the integrand f must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of f(x) evaluated at `x`.

    a : float
        a, the lower limit of integration.

    b : float
        b, the upper limit of integration. It is not necessary that a <
        b.

    epsabs : float
        The absolute accuracy required. If `epsabs` is negative, the
        absolute value is used. See [Accuracy].

    epsrel : float
        The relative accuracy required. If `epsrel` is negative, the
        absolute value is used. See [Accuracy].

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    result : float
        The approximation to the integral I.

    abserr : float
        An estimate of the modulus of the absolute error, which should
        be an upper bound for |I-`result`|.

    See Also
    --------
    :meth:`naginterfaces.library.examples.quad.dim1_fin_smooth_ex.main`
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.dim1_fin_smooth'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, _fsub, en_data, x, retval, _iuser, _ruser):
        """Helper for f."""
        try:
            _x_fh = utils._EngineFloat64ScalarType.to_py(x.contents)
            if data is not None:
                _retval_fh = f(
                    _x_fh,
                    data,
                )
            else:
                _retval_fh = f(
                    _x_fh,
                )
            utils._EngineFloat64ScalarType(
                _retval_fh, {'fun_name': cb_f_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d01bdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_FLOAT64, # Input, epsabs.
        utils._BYREF_FLOAT64, # Input, epsrel.
        utils._BYREF_FLOAT64, # Output, result.
        utils._BYREF_FLOAT64, # Output, abserr.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fsub = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _epsabs = utils._EngineFloat64ScalarType(
        epsabs, {'fun_name': fun_name, 'entity': 'epsabs'},
    ).to_ctypes()
    _epsrel = utils._EngineFloat64ScalarType(
        epsrel, {'fun_name': fun_name, 'entity': 'epsrel'},
    ).to_ctypes()
    _result = utils._EngineFloat64ScalarType.empty_ctypes()
    _abserr = utils._EngineFloat64ScalarType.empty_ctypes()
    _iuser = None
    _ruser = None
    engine_ctypes(
        _en_data,
        _f,
        _fsub,
        _fh,
        _a,
        _b,
        _epsabs,
        _epsrel,
        _result,
        _abserr,
        _iuser,
        _ruser,
    )
    utils._handle_cb_exception(excs)
    _result_py = utils._EngineFloat64ScalarType.to_py(_result)
    _abserr_py = utils._EngineFloat64ScalarType.to_py(_abserr)
    return_data = _collections.namedtuple(
        'nag_return_data',
        ['result', 'abserr'],
    )(
        _result_py,
        _abserr_py,
    )
    return return_data

def dim2_fin(ya, yb, phi1, phi2, f, absacc, data=None):
    u"""
    Two-dimensional quadrature, finite region.

    For full information please refer to the NAG Library document for
    d01da

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01daf.html

    Parameters
    ----------
    ya : float
        a, the lower limit of the integral.

    yb : float
        b, the upper limit of the integral. It is not necessary that a <
        b.

    phi1 : callable retval = phi1(y, data=None)
        `phi1` must return the lower limit of the inner integral for a
        given value of y.

        Parameters
        ~~~~~~~~~~
        y : float
            The value of y for which the lower limit must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The lower limit of the inner integral for the given value of
            y.

    phi2 : callable retval = phi2(y, data=None)
        `phi2` must return the upper limit of the inner integral for a
        given value of y.

        Parameters
        ~~~~~~~~~~
        y : float
            The value of y for which the upper limit must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The upper limit of the inner integral for the given value of
            y.

    f : callable retval = f(x, y, data=None)
        `f` must return the value of the integrand f at a given point.

        Parameters
        ~~~~~~~~~~
        x : float
            The coordinates of the point (x,y) at which the integrand f
            must be evaluated.

        y : float
            The coordinates of the point (x,y) at which the integrand f
            must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of f(x) evaluated at `x`.

    absacc : float
        The absolute accuracy requested.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    ans : float
        The estimated value of the integral.

    npts : int
        The total number of function evaluations.

    Raises
    ------
    NagValueError
        (`errno` i mod 10!=0)
            The outer integral has not converged, and n of the inner
            integrals have not converged with 255 points: n = *<value>*.
            `ans` may still contain an approximate estimate of the
            integral, but its reliability will decrease as n increases.

        (`errno` i mod 10 = 0)
            The outer integral has converged, but n of the inner
            integrals have not converged with 255 points: n = *<value>*.
            `ans` may still contain an approximate estimate of the
            integral, but its reliability will decrease as n increases.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The outer integral has not converged with 255 points.
            However, all the inner integrals have converged, and `ans`
            may still contain an approximate estimate of the integral.
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.dim2_fin'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_phi1_name = fun_name + ':phi1'

    phi1h = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, y.
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_phi1h(_phi1, _phi1sub, en_data, y, retval, _iuser, _ruser):
        """Helper for phi1."""
        try:
            _y_phi1h = utils._EngineFloat64ScalarType.to_py(y.contents)
            if data is not None:
                _retval_phi1h = phi1(
                    _y_phi1h,
                    data,
                )
            else:
                _retval_phi1h = phi1(
                    _y_phi1h,
                )
            utils._EngineFloat64ScalarType(
                _retval_phi1h, {'fun_name': cb_phi1_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_phi2_name = fun_name + ':phi2'

    phi2h = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, y.
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_phi2h(_phi2, _phi2sub, en_data, y, retval, _iuser, _ruser):
        """Helper for phi2."""
        try:
            _y_phi2h = utils._EngineFloat64ScalarType.to_py(y.contents)
            if data is not None:
                _retval_phi2h = phi2(
                    _y_phi2h,
                    data,
                )
            else:
                _retval_phi2h = phi2(
                    _y_phi2h,
                )
            utils._EngineFloat64ScalarType(
                _retval_phi2h, {'fun_name': cb_phi2_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Input, y.
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, _fsub, en_data, x, y, retval, _iuser, _ruser):
        """Helper for f."""
        try:
            _x_fh = utils._EngineFloat64ScalarType.to_py(x.contents)
            _y_fh = utils._EngineFloat64ScalarType.to_py(y.contents)
            if data is not None:
                _retval_fh = f(
                    _x_fh,
                    _y_fh,
                    data,
                )
            else:
                _retval_fh = f(
                    _x_fh,
                    _y_fh,
                )
            utils._EngineFloat64ScalarType(
                _retval_fh, {'fun_name': cb_f_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d01daft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, ya.
        utils._BYREF_FLOAT64, # Input, yb.
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        phi1h,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        phi2h,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._BYREF_FLOAT64, # Input, absacc.
        utils._BYREF_FLOAT64, # Output, ans.
        utils._BYREF_INT, # Output, npts.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ya = utils._EngineFloat64ScalarType(
        ya, {'fun_name': fun_name, 'entity': 'ya'},
    ).to_ctypes()
    _yb = utils._EngineFloat64ScalarType(
        yb, {'fun_name': fun_name, 'entity': 'yb'},
    ).to_ctypes()
    utils._check_callable(phi1, {'fun_name': fun_name, 'entity': 'phi1'})
    _phi1 = utils._UNUSED_CALLBACK
    _phi1sub = utils._UNUSED_CALLBACK
    _phi1h = utils._cast_callback(
        phi1h, cb_phi1h,
        {'fun_name': fun_name, 'entity': 'phi1'},
    )
    utils._check_callable(phi2, {'fun_name': fun_name, 'entity': 'phi2'})
    _phi2 = utils._UNUSED_CALLBACK
    _phi2sub = utils._UNUSED_CALLBACK
    _phi2h = utils._cast_callback(
        phi2h, cb_phi2h,
        {'fun_name': fun_name, 'entity': 'phi2'},
    )
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fsub = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _absacc = utils._EngineFloat64ScalarType(
        absacc, {'fun_name': fun_name, 'entity': 'absacc'},
    ).to_ctypes()
    _ans = utils._EngineFloat64ScalarType.empty_ctypes()
    _npts = utils._EngineIntScalarType.empty_ctypes()
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ya,
        _yb,
        _phi1,
        _phi1sub,
        _phi1h,
        _phi2,
        _phi2sub,
        _phi2h,
        _f,
        _fsub,
        _fh,
        _absacc,
        _ans,
        _npts,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py%10 != 0 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The outer integral has not converged, and n of the inner integrals',
                    'have not converged with 255 points: n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'ans may still contain an approximate estimate of the integral,',
                    'but its reliability will decrease as n increases.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py%10 == 0 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The outer integral has converged, but n of the inner integrals',
                    'have not converged with 255 points: n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'ans may still contain an approximate estimate of the integral,',
                    'but its reliability will decrease as n increases.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The outer integral has not converged with 255 points.',
                    'However, all the inner integrals have converged, and ans may still contain',
                    'an approximate estimate of the integral.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ans_py = utils._EngineFloat64ScalarType.to_py(_ans)
        _npts_py = utils._EngineIntScalarType.to_py(_npts)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ans', 'npts'],
        )(
            _ans_py,
            _npts_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def md_adapt_multi(ndim, a, b, mincls, maxcls, nfun, funsub, absreq, relreq, lenwrk, wrkstr, finest, absest, data=None):
    u"""
    Multidimensional adaptive quadrature over hyper-rectangle, multiple
    integrands.

    For full information please refer to the NAG Library document for
    d01ea

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01eaf.html

    Parameters
    ----------
    ndim : int
        n, the number of dimensions of the integrals.

    a : float, shape (`ndim`)
        The lower limits of integration, a_i, for i = 1,2,...,n.

    b : float, shape (`ndim`)
        The upper limits of integration, b_i, for i = 1,2,...,n.

    mincls : int
        Must be set either to the minimum number of `funsub` calls to be
        allowed, in which case `mincls` >= 0 or to a negative value. In
        this case, the function continues the calculation started in a
        previous call with the same integrands and integration limits:
        no arguments other than `mincls`, `maxcls`, `absreq` or `relreq`
        must be changed between the calls.

    maxcls : int
        The maximum number of `funsub` calls to be allowed. In the
        continuation case this is the number of new `funsub` calls to be
        allowed.

    nfun : int
        m, the number of integrands.

    funsub : callable funsub(ndim, z, nfun, f, data=None)
        `funsub` must evaluate the integrands f_i at a given point.

        Parameters
        ~~~~~~~~~~
        ndim : int
            n, the number of dimensions of the integrals.

        z : float, ndarray, shape (`ndim`)
            The coordinates of the point at which the integrands must be
            evaluated.

        nfun : int
            m, the number of integrands.

        f : float, ndarray, shape (`nfun`), to be modified in place
            On exit: the value of the ith integrand at the given point.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    absreq : float
        The absolute accuracy required by you.

    relreq : float
        The relative accuracy required by you.

    lenwrk : int
        The dimension of the array `wrkstr`.

    wrkstr : float, shape (`lenwrk`), modified in place
        On entry: if `mincls` < 0, `wrkstr` must be unchanged from the
        previous call of ``md_adapt_multi``.

        On exit: contains information about the current subdivision
        which could be used in a continuation call.

    finest : float, shape (`nfun`), modified in place
        On exit: `finest`[i-1] specifies the best estimate obtained from
        the ith integral, for i = 1,2,...,m.

    absest : float, shape (`nfun`), modified in place
        On exit: `absest`[i-1] specifies the estimated absolute accuracy
        of `finest`[i-1], for i = 1,2,...,m.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    mincls : int
        Gives the number of `funsub` calls actually used by
        ``md_adapt_multi``. For the continuation case (`mincls` < 0 on
        entry) this is the number of new `funsub` calls on the current
        call to ``md_adapt_multi``.

    Raises
    ------
    NagValueError
        (`errno` 4)
            On entry, `maxcls` = *<value>* and R = *<value>*.

            Constraint: `maxcls` >= R =
            1+`ndim`*(4*`ndim`^2-6*`ndim`+14)/3.

        (`errno` 4)
            On entry, `maxcls` = *<value>* and R = *<value>*.

            Constraint: `maxcls` >= R = 2^`ndim`+2*`ndim`^2+2*`ndim`+1.

        (`errno` 4)
            On entry, `lenwrk` is too small. `lenwrk` = *<value>*.
            Minimum possible dimension: *<value>*.

        (`errno` 4)
            On entry, `maxcls` = *<value>* and `mincls` = *<value>*.

            Constraint: `maxcls` >= `mincls`.

        (`errno` 4)
            On entry, `nfun` = *<value>*.

            Constraint: `nfun` >= 1.

        (`errno` 4)
            On entry, `ndim` = *<value>*.

            Constraint: `ndim` >= 1.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            `maxcls` too small to obtain required accuracy. `maxcls` =
            *<value>*.

        (`errno` 2)
            `lenwrk` too small for the function to continue. `lenwrk` =
            *<value>*.

        (`errno` 3)
            `maxcls` too small to make any progress. `maxcls` =
            *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.md_adapt_multi'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_funsub_name = fun_name + ':funsub'

    funsubh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ndim.
        utils._EngineFloat64ArrayType.c_type, # Input, z[ndim].
        utils._BYREF_INT, # Input, nfun.
        utils._EngineFloat64ArrayType.c_type, # Output, f[nfun].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_funsubh(_funsub, en_data, ndim, z, nfun, f, _iuser, _ruser):
        """Helper for funsub."""
        try:
            _ndim_funsubh = utils._EngineIntScalarType.to_py(ndim.contents)
            _z_funsubh = utils._EngineFloat64ArrayType.from_pointer(
                z,
                (_ndim_funsubh,),
            )
            _nfun_funsubh = utils._EngineIntScalarType.to_py(nfun.contents)
            _f_funsubh = utils._EngineFloat64ArrayType.from_pointer(
                f,
                (_nfun_funsubh,),
                can_reference_contents=False,
            )
            if data is not None:
                funsub(
                    _ndim_funsubh,
                    _z_funsubh,
                    _nfun_funsubh,
                    _f_funsubh,
                    data,
                )
            else:
                funsub(
                    _ndim_funsubh,
                    _z_funsubh,
                    _nfun_funsubh,
                    _f_funsubh,
                )
            utils._unlink_pointer(z)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _f_funsubh, {'fun_name': cb_funsub_name, 'returned': 'f'}, f,
            )
            utils._unlink_pointer(f)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d01eaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ndim.
        utils._EngineFloat64ArrayType.c_type, # Input, a[ndim].
        utils._EngineFloat64ArrayType.c_type, # Input, b[ndim].
        utils._BYREF_INT, # Input/Output, mincls.
        utils._BYREF_INT, # Input, maxcls.
        utils._BYREF_INT, # Input, nfun.
        utils._UNUSED_FUNCTYPE,
        funsubh,
        utils._BYREF_FLOAT64, # Input, absreq.
        utils._BYREF_FLOAT64, # Input, relreq.
        utils._BYREF_INT, # Input, lenwrk.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, wrkstr[lenwrk].
        utils._EngineFloat64ArrayType.c_type, # Output, finest[nfun].
        utils._EngineFloat64ArrayType.c_type, # Output, absest[nfun].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ndim = utils._EngineIntScalarType(
        ndim, {'fun_name': fun_name, 'entity': 'ndim'},
    ).to_ctypes()
    _ndim_py = utils._EngineIntScalarType.to_py(_ndim)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_ndim_py,),
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_ndim_py,),
    ).to_ctypes()
    _mincls = utils._EngineIntScalarType(
        mincls, {'fun_name': fun_name, 'entity': 'mincls'},
    ).to_ctypes()
    _maxcls = utils._EngineIntScalarType(
        maxcls, {'fun_name': fun_name, 'entity': 'maxcls'},
    ).to_ctypes()
    _nfun = utils._EngineIntScalarType(
        nfun, {'fun_name': fun_name, 'entity': 'nfun'},
    ).to_ctypes()
    utils._check_callable(funsub, {'fun_name': fun_name, 'entity': 'funsub'})
    _funsub = utils._UNUSED_CALLBACK
    _funsubh = utils._cast_callback(
        funsubh, cb_funsubh,
        {'fun_name': fun_name, 'entity': 'funsub'},
    )
    _absreq = utils._EngineFloat64ScalarType(
        absreq, {'fun_name': fun_name, 'entity': 'absreq'},
    ).to_ctypes()
    _relreq = utils._EngineFloat64ScalarType(
        relreq, {'fun_name': fun_name, 'entity': 'relreq'},
    ).to_ctypes()
    _lenwrk = utils._EngineIntScalarType(
        lenwrk, {'fun_name': fun_name, 'entity': 'lenwrk'},
    ).to_ctypes()
    _lenwrk_py = utils._EngineIntScalarType.to_py(_lenwrk)
    _wrkstr_cls = utils._EngineFloat64ArrayType(
        wrkstr, {'fun_name': fun_name, 'entity': 'wrkstr'},
        exp_shape=(_lenwrk_py,),
    )
    _wrkstr = _wrkstr_cls.to_ctypes()
    _nfun_py = utils._EngineIntScalarType.to_py(_nfun)
    _finest_cls = utils._EngineFloat64ArrayType(
        finest, {'fun_name': fun_name, 'entity': 'finest'},
        exp_shape=(_nfun_py,),
        check_contents=False,
    )
    _finest = _finest_cls.to_ctypes(can_reference_contents=False)
    _absest_cls = utils._EngineFloat64ArrayType(
        absest, {'fun_name': fun_name, 'entity': 'absest'},
        exp_shape=(_nfun_py,),
        check_contents=False,
    )
    _absest = _absest_cls.to_ctypes(can_reference_contents=False)
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ndim,
        _a,
        _b,
        _mincls,
        _maxcls,
        _nfun,
        _funsub,
        _funsubh,
        _absreq,
        _relreq,
        _lenwrk,
        _wrkstr,
        _finest,
        _absest,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'maxcls too small to obtain required accuracy.',
                    'maxcls = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99992:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'lenwrk too small for the function to continue.',
                    'lenwrk = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99991:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'maxcls too small to make any progress.',
                    'maxcls = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, maxcls = {:s} and R = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: maxcls >= R = 1+ndim*(4*ndim^2-6*ndim+14)/3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, maxcls = {:s} and R = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: maxcls >= R = 2^ndim+2*ndim^2+2*ndim+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lenwrk is too small.',
                    'lenwrk = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Minimum possible dimension: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, maxcls = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'mincls = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: maxcls >= mincls.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nfun = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nfun >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ndim = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ndim >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _mincls_py = utils._EngineIntScalarType.to_py(_mincls)
        _wrkstr_cls.refresh_py(_wrkstr)
        _finest_cls.refresh_py(_finest)
        _absest_cls.refresh_py(_absest)
        return_data = _mincls_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['mincls'],
            )(
                _mincls_py,
            )
    _err_class.manage()
    return return_data

def md_sgq_multi_vec(ni, ndim, f, maxdlv, dinest, errest, ivalid, comm, data=None):
    u"""
    Multi-dimensional quadrature using sparse grids.

    Note: this function uses optional algorithmic parameters, see also:
    ``opt_set``, ``opt_get``.

    For full information please refer to the NAG Library document for
    d01es

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01esf.html

    Parameters
    ----------
    ni : int
        n_i, the number of integrands.

    ndim : int
        d, the number of dimensions.

    f : callable iflag = f(ni, ndim, nx, xtr, nntr, icolzp, irowix, xs,
        qs, fm, iflag, data=None)
        `f` must return the value of the integrands f_j at a set of n_x
        d-dimensional points x_i, implicitly supplied as columns of a
        matrix X(d,n_x).

        If X was supplied explicitly you would find that most of the
        elements attain the same value, x_tr; the larger the number of
        dimensions, the greater the proportion of elements of X would be
        equal to x_tr.

        So, X is effectively a sparse matrix, except that the 'zero'
        elements are replaced by elements that are all equal to the
        value x_tr.

        For this reason X is supplied, as though it were a sparse
        matrix, in compressed column storage (CCS) format (see [the F11
        Introduction]).

        Parameters
        ~~~~~~~~~~
        ni : int
            n_i, the number of integrands.

        ndim : int
            d, the number of dimensions.

        nx : int
            n_x, the number of points x_i, corresponding to the number
            of columns of X, at which the set of integrands must be
            evaluated.

        xtr : float
            x_tr, the value of the trivial elements of X.

        nntr : int
            If `iflag` > 0, the number of non-trivial elements of X.

            If `iflag` = 0, the total number of abscissae from the
            underlying one-dimensional quadrature.

        icolzp : int, ndarray, shape (`nx`+1)
            The set {`icolzp`[i-1],...,`icolzp`[i]-1} contains the
            indices of `irowix` and `xs` corresponding to the
            non-trivial elements of column i of X and hence of the point
            x_i, for i = 1,2,...,n_x.

        irowix : int, ndarray, shape (`nntr`)
            The row indices corresponding to the non-trivial elements of
            X.

        xs : float, ndarray, shape (`nntr`)
            x_{k,i} != x_tr, the non-trivial entries of X.

        qs : int, ndarray, shape (`nntr`)
            q_{k,i} != 1, the indices of the underlying quadrature rules
            corresponding to x_{k,i} != x_tr.

        fm : float, ndarray, shape (`ni`, `nx`), to be modified in place
            On exit: `fm`[p-1,i-1] = f_p(x_i), for p = 1,2,...,n_i, for
            i = 1,2,...,n_x.

        iflag : int
            If `iflag` = 0, this is the first call to `f`. n_x = 1, and
            the entire point x_1 will satisfy x_{k,1} = x_tr, for k =
            1,2,...,d. In addition, `nntr` contains the total number of
            abscissae from the underlying one-dimensional quadrature;
            `xs` contains the complete set of abscissae and `qs`
            contains the corresponding quadrature indices, with `xs`[0]
            = x_tr and `qs`[0] = 1. This will always be called in
            serial.

            In subsequent calls to `f`, `iflag` = 1.

            Subsequent calls may be made from within an OpenMP parallel
            region.

            See [Parallelism and Performance] for details.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        iflag : int
            Set `iflag` < 0 if you wish to force an immediate exit from
            ``md_sgq_multi_vec`` with `errno` = -1.

    maxdlv : int, shape (`ndim`)
        m, the array of maximum levels for each dimension.
        `maxdlv`[j-1], for j = 1,2,...,d, contains m_j, the maximum
        level of quadrature rule dimension j will support.

        The default value, min(m_q, L) will be used if either
        `maxdlv`[j-1] <= 0 or `maxdlv`[j-1] >= min(m_q,L) (for details
        on the default values for m_q and L and on how to change these
        values see the options 'Maximum Level', 'Maximum Quadrature
        Level' and 'Quadrature Rule').

        If `maxdlv`[j-1] = 1 for all j, only one evaluation will be
        performed, and as such no error estimation will be possible.

    dinest : float, shape (`ni`), modified in place
        On exit: `dinest`[p-1] contains the final estimate F^_p of the
        definite integral F_p, for p = 1,2,...,n_i.

    errest : float, shape (`ni`), modified in place
        On exit: `errest`[p-1] contains the final error estimate E_p of
        the definite integral F_p, for p = 1,2,...,n_i.

    ivalid : int, shape (`ni`), modified in place
        On exit: `ivalid`[p-1] indicates the final state of integral p,
        for p = 1,2,...,n_i.

        `ivalid`[p-1] = 0

          The error estimate for integral p was below the requested
          tolerance.

        `ivalid`[p-1] = 1

          The error estimate for integral p was below the requested
          tolerance. The final level used was non-isotropic.

        `ivalid`[p-1] = 2

          The error estimate for integral p was above the requested
          tolerance.

        `ivalid`[p-1] = 3

          The error estimate for integral p was above
          max(0.1|F^_p|,0.01).

        `ivalid`[p-1] < 0

          You aborted the evaluation before an error estimate could be
          made.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``opt_set``.

    data : arbitrary, optional
        User-communication data for callback functions.

    Other Parameters
    ----------------
    'Absolute Tolerance' : float
        Default = sqrt(epsilon)

        r = epsilon_a, the absolute tolerance required.

    'Index Level' : int
        Default = 4

        The maximum level at which function values are stored for later
        use.
        Larger values use increasingly more memory, and require more
        time to access specific elements.
        Lower levels result in more repeated computation.
        The 'Maximum Quadrature Level', m_q is the effective upper limit
        on 'Index Level'.
        If i >= m_q, ``md_sgq_multi_vec`` will use m_q as the value of
        'Index Level'.

        Constraint: i >= 1.

    'Maximum Level' : int
        Default = 5

        i = L, the maximum number of levels to evaluate.

        Constraint: 1 < i <= 20.

        Note: the maximum allowable level in any single dimension, m_q,
        is governed by the 'Quadrature Rule' selected. If a value
        greater than m_q is set, only a subset of subspaces from higher
        levels will be used. Should this subset be empty for a given
        level, computation will consider the preceding level to be the
        maximum level and will terminate.

    'Maximum Nx' : int
        Default = 128

        i = maxn_x, the maximum number of points to evaluate in a single
        call to `f`.

        Constraint: 1 <= i <= 16384.

    'Maximum Quadrature Level' : int
        Queriable only

        i = m_q, the maximum level of the underlying one-dimensional
        quadrature rule (see 'Quadrature Rule').

    'Minimum Level' : int
        Default = 2

        The minimum number of levels which must be evaluated before an
        error estimate is used to determine convergence.

        Constraint: i > 1.

        Note: if the minimum level is greater than the maximum
        computable level, the maximum level will be used.

    'Quadrature Rule' : str
        Default = 'Gauss--Patterson'

        The underlying one-dimensional quadrature rule to be used in the
        construction.
        Open rules do not require evaluations at boundaries.

        'Quadrature Rule' = 'Gauss--Patterson' or 'GP'
          The interlacing Gauss--Patterson rules. Level l uses 2^l -1
          abscissae. All levels are open. These rules provide high order
          accuracy. m_q = 9.

        'Quadrature Rule' = 'Clenshaw--Curtis' or 'CC'
          The interlacing Clenshaw--Curtis rules. Level l uses 2^(l
          -1)+1 abscissae. All levels above level 1 are closed. m_q =
          12.

    'Relative Tolerance' : float
        Default = sqrt(epsilon)

        r = epsilon_a, the relative tolerance required.

    'Summation Precision' : str
        Default = 'HIGHER'

        Determines whether ``md_sgq_multi_vec`` uses working precision
        or higher-than-working precision to accumulate the actions over
        subspaces.

        'Summation Precision' = 'HIGHER' or 'H'
          Higher-than-working precision is used to accumulate the action
          over a subspace, and for the accumulation of all such actions.
          This is more expensive computationally, although this is
          probably negligible in comparison to the cost of evaluating
          the integrands and the overall runtime. This significantly
          reduces variation in the result when changing the number of
          threads.

        'Summation Precision' = 'WORKING' or 'W'
          Working precision is used to accumulate the actions over
          subspaces. This may provide some speedup, particularly if n_i
          or n_t is large. The results of parallel simulations will
          however be more prone to variation.

    'Serial Levels' : int
        Default = 1

        i = s_l, the number of levels to be evaluated in serial before
        initializing parallelization.
        For relatively trivial integrands, this may need to be set
        greater than the default to reduce parallel overhead.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, `ni` = *<value>*.

            Constraint: `ni` >= 1.

        (`errno` 21)
            On entry, `ndim` = *<value>*.

            Constraint: `ndim` >= 1.

        (`errno` 1001)
            Either the option arrays `comm`['iopts'] and `comm`['opts']
            have not been initialized for ``md_sgq_multi_vec``, or they
            have become corrupted.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` -1)
            Exit requested from `f` with `iflag` = *<value>*.

        (`errno` 1)
            The requested accuracy was not achieved for at least one
            integral.

        (`errno` 2)
            No accuracy was achieved for at least one integral.
    """

    from ..base import _nag_d_consts

    excs = []
    fun_name = 'naginterfaces.base.quad.md_sgq_multi_vec'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ni.
        utils._BYREF_INT, # Input, ndim.
        utils._BYREF_INT, # Input, nx.
        utils._BYREF_FLOAT64, # Input, xtr.
        utils._BYREF_INT, # Input, nntr.
        utils._EngineIntArrayType.c_type, # Input, icolzp[nx+1].
        utils._EngineIntArrayType.c_type, # Input, irowix[nntr].
        utils._EngineFloat64ArrayType.c_type, # Input, xs[nntr].
        utils._EngineIntArrayType.c_type, # Input, qs[nntr].
        utils._EngineFloat64ArrayType.c_type, # Output, fm[ni, nx].
        utils._BYREF_INT, # Input/Output, iflag.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, en_data, ni, ndim, nx, xtr, nntr, icolzp, irowix, xs, qs, fm, iflag, _iuser, _ruser):
        """Helper for f."""
        try:
            _ni_fh = utils._EngineIntScalarType.to_py(ni.contents)
            _ndim_fh = utils._EngineIntScalarType.to_py(ndim.contents)
            _nx_fh = utils._EngineIntScalarType.to_py(nx.contents)
            _xtr_fh = utils._EngineFloat64ScalarType.to_py(xtr.contents)
            _nntr_fh = utils._EngineIntScalarType.to_py(nntr.contents)
            _icolzp_fh = utils._EngineIntArrayType.from_pointer(
                icolzp,
                (_nx_fh+1,),
            )
            _irowix_fh = utils._EngineIntArrayType.from_pointer(
                irowix,
                (_nntr_fh,),
            )
            _xs_fh = utils._EngineFloat64ArrayType.from_pointer(
                xs,
                (_nntr_fh,),
            )
            _qs_fh = utils._EngineIntArrayType.from_pointer(
                qs,
                (_nntr_fh,),
            )
            _fm_fh = utils._EngineFloat64ArrayType.from_pointer(
                fm,
                (_ni_fh, _nx_fh),
                can_reference_contents=False,
                sorder=utils._EngineData.default_sorder,
            )
            _iflag_fh = utils._EngineIntScalarType.to_py(iflag.contents)
            if data is not None:
                _iflag_fh = f(
                    _ni_fh,
                    _ndim_fh,
                    _nx_fh,
                    _xtr_fh,
                    _nntr_fh,
                    _icolzp_fh,
                    _irowix_fh,
                    _xs_fh,
                    _qs_fh,
                    _fm_fh,
                    _iflag_fh,
                    data,
                )
            else:
                _iflag_fh = f(
                    _ni_fh,
                    _ndim_fh,
                    _nx_fh,
                    _xtr_fh,
                    _nntr_fh,
                    _icolzp_fh,
                    _irowix_fh,
                    _xs_fh,
                    _qs_fh,
                    _fm_fh,
                    _iflag_fh,
                )
            utils._unlink_pointer(icolzp)
            utils._unlink_pointer(irowix)
            utils._unlink_pointer(xs)
            utils._unlink_pointer(qs)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _fm_fh, {'fun_name': cb_f_name, 'returned': 'fm'}, fm,
            )
            utils._unlink_pointer(fm)
            utils._EngineIntScalarType(
                _iflag_fh, {'fun_name': cb_f_name, 'returned': 'iflag'},
            ).refresh_ctypes_buffer(iflag)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d01esft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ni.
        utils._BYREF_INT, # Input, ndim.
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._EngineIntArrayType.c_type, # Input, maxdlv[ndim].
        utils._EngineFloat64ArrayType.c_type, # Output, dinest[ni].
        utils._EngineFloat64ArrayType.c_type, # Output, errest[ni].
        utils._EngineIntArrayType.c_type, # Output, ivalid[ni].
        utils._EngineIntArrayType.c_type, # Input, iopts[100].
        utils._EngineFloat64ArrayType.c_type, # Input, opts[100].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ni = utils._EngineIntScalarType(
        ni, {'fun_name': fun_name, 'entity': 'ni'},
    ).to_ctypes()
    _ndim = utils._EngineIntScalarType(
        ndim, {'fun_name': fun_name, 'entity': 'ndim'},
    ).to_ctypes()
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _ndim_py = utils._EngineIntScalarType.to_py(_ndim)
    _maxdlv = utils._EngineIntArrayType(
        maxdlv, {'fun_name': fun_name, 'entity': 'maxdlv'},
        exp_shape=(_ndim_py,),
    ).to_ctypes()
    _ni_py = utils._EngineIntScalarType.to_py(_ni)
    _dinest_cls = utils._EngineFloat64ArrayType(
        dinest, {'fun_name': fun_name, 'entity': 'dinest'},
        exp_shape=(_ni_py,),
        check_contents=False,
    )
    _dinest = _dinest_cls.to_ctypes(can_reference_contents=False)
    _errest_cls = utils._EngineFloat64ArrayType(
        errest, {'fun_name': fun_name, 'entity': 'errest'},
        exp_shape=(_ni_py,),
        check_contents=False,
    )
    _errest = _errest_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(_ni_py,),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('iopts', 'opts'),
    )
    _iopts = comm['iopts']
    _opts = comm['opts']
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ni,
        _ndim,
        _f,
        _fh,
        _maxdlv,
        _dinest,
        _errest,
        _ivalid,
        _iopts,
        _opts,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == _nag_d_consts.D01_IERR_USER_EXIT_INT and _ifmt == 1001:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Exit requested from f with iflag = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01ES_IERR_ACC and _ifmt == 1001:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The requested accuracy was not achieved for at least one integral.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01ES_IERR_NOACC and _ifmt == 1001:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'No accuracy was achieved for at least one integral.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01ES_IERR_NI and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ni = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ni >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01ES_IERR_NDIM and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ndim = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ndim >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01_IERR_OPTS_CORRUPT and _ifmt == _nag_d_consts.D01_IFMT_OPTS_CORRUPT:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the option arrays comm[\'iopts\'] and comm[\'opts\'] have not been',
                    'initialized for md_sgq_multi_vec, or they have become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _dinest_cls.refresh_py(_dinest)
        _errest_cls.refresh_py(_errest)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def md_gauss(ndim, nptvec, lwa, weight, abscis, f, data=None):
    u"""
    Multidimensional Gaussian quadrature over hyper-rectangle.

    For full information please refer to the NAG Library document for
    d01fb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01fbf.html

    Parameters
    ----------
    ndim : int
        n, the number of dimensions of the integral.

    nptvec : int, shape (`ndim`)
        `nptvec`[j-1] must specify the number of points in the jth
        dimension of the summation, for j = 1,2,...,n.

    lwa : int
        The dimension of the arrays `weight` and `abscis`.

    weight : float, shape (`lwa`)
        Must contain in succession the weights for the various
        dimensions, i.e., `weight`[k-1] contains the ith weight for the
        jth dimension, with

            k = `nptvec`[0]+`nptvec`[1]+...+`nptvec`[j-2]+i.

    abscis : float, shape (`lwa`)
        Must contain in succession the abscissae for the various
        dimensions, i.e., `abscis`[k-1] contains the ith abscissa for
        the jth dimension, with

            k = `nptvec`[0]+`nptvec`[1]+...+`nptvec`[j-2]+i.

    f : callable retval = f(ndim, x, data=None)
        `f` must return the value of the integrand at a given point.

        Parameters
        ~~~~~~~~~~
        ndim : int
            n, the number of dimensions of the integral.

        x : float, ndarray, shape (`ndim`)
            The coordinates of the point at which the integrand f must
            be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of f(x) evaluated at `x`.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    mdint : float
        The estimate of the integral.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `lwa` is too small. `lwa` = *<value>*. Minimum
            possible dimension: *<value>*.

        (`errno` 1)
            On entry, `ndim` = *<value>*.

            Constraint: `ndim` <= 20.

        (`errno` 1)
            On entry, `ndim` = *<value>*.

            Constraint: `ndim` >= 1.
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.md_gauss'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ndim.
        utils._EngineFloat64ArrayType.c_type, # Input, x[ndim].
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, _fsub, en_data, ndim, x, retval, _iuser, _ruser):
        """Helper for f."""
        try:
            _ndim_fh = utils._EngineIntScalarType.to_py(ndim.contents)
            _x_fh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_ndim_fh,),
            )
            if data is not None:
                _retval_fh = f(
                    _ndim_fh,
                    _x_fh,
                    data,
                )
            else:
                _retval_fh = f(
                    _ndim_fh,
                    _x_fh,
                )
            utils._unlink_pointer(x)
            utils._EngineFloat64ScalarType(
                _retval_fh, {'fun_name': cb_f_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d01fbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ndim.
        utils._EngineIntArrayType.c_type, # Input, nptvec[ndim].
        utils._BYREF_INT, # Input, lwa.
        utils._EngineFloat64ArrayType.c_type, # Input, weight[lwa].
        utils._EngineFloat64ArrayType.c_type, # Input, abscis[lwa].
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._BYREF_FLOAT64, # Output, mdint.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ndim = utils._EngineIntScalarType(
        ndim, {'fun_name': fun_name, 'entity': 'ndim'},
    ).to_ctypes()
    _ndim_py = utils._EngineIntScalarType.to_py(_ndim)
    _nptvec = utils._EngineIntArrayType(
        nptvec, {'fun_name': fun_name, 'entity': 'nptvec'},
        exp_shape=(_ndim_py,),
    ).to_ctypes()
    _lwa = utils._EngineIntScalarType(
        lwa, {'fun_name': fun_name, 'entity': 'lwa'},
    ).to_ctypes()
    _lwa_py = utils._EngineIntScalarType.to_py(_lwa)
    _weight = utils._EngineFloat64ArrayType(
        weight, {'fun_name': fun_name, 'entity': 'weight'},
        exp_shape=(_lwa_py,),
    ).to_ctypes()
    _abscis = utils._EngineFloat64ArrayType(
        abscis, {'fun_name': fun_name, 'entity': 'abscis'},
        exp_shape=(_lwa_py,),
    ).to_ctypes()
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fsub = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _mdint = utils._EngineFloat64ScalarType.empty_ctypes()
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ndim,
        _nptvec,
        _lwa,
        _weight,
        _abscis,
        _f,
        _fsub,
        _fh,
        _mdint,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lwa is too small.',
                    'lwa = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Minimum possible dimension: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ndim = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ndim <= 20.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ndim = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ndim >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _mdint_py = utils._EngineFloat64ScalarType.to_py(_mdint)
        return_data = _mdint_py
    _err_class.manage()
    return return_data

def md_adapt(ndim, a, b, minpts, maxpts, f, eps, data=None):
    u"""
    Multidimensional adaptive quadrature over hyper-rectangle.

    For full information please refer to the NAG Library document for
    d01fc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01fcf.html

    Parameters
    ----------
    ndim : int
        n, the number of dimensions of the integral.

    a : float, shape (`ndim`)
        The lower limits of integration, a_i, for i = 1,2,...,n.

    b : float, shape (`ndim`)
        The upper limits of integration, b_i, for i = 1,2,...,n.

    minpts : int
        Must be set to the minimum number of integrand evaluations to be
        allowed.

    maxpts : int
        The maximum number of integrand evaluations to be allowed.

    f : callable retval = f(ndim, z, data=None)
        `f` must return the value of the integrand at a given point.

        Parameters
        ~~~~~~~~~~
        ndim : int
            n, the number of dimensions of the integral.

        z : float, ndarray, shape (`ndim`)
            The coordinates of the point at which the integrand f must
            be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of the integrand f at the given point.

    eps : float
        The relative error acceptable to you. When the solution is zero
        or very small relative accuracy may not be achievable but you
        may still set `eps` to a reasonable value and check for the
        error exit `errno` = 2.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    minpts : int
        Contains the actual number of integrand evaluations used by
        ``md_adapt``.

    acc : float
        The estimated relative error in `finval`.

    finval : float
        The best estimate obtained for the integral.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `maxpts` is too small. `maxpts` = *<value>*.
            Minimum possible dimension: *<value>*.

        (`errno` 1)
            On entry, `eps` = *<value>*.

            Constraint: `eps` > 0.0.

        (`errno` 1)
            On entry, `minpts` = *<value>* and `maxpts` = *<value>*.

            Constraint: `minpts` <= `maxpts`.

        (`errno` 1)
            On entry, `ndim` = *<value>*.

            Constraint: `ndim` <= 15.

        (`errno` 1)
            On entry, `ndim` = *<value>*.

            Constraint: `ndim` >= 2.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            `maxpts` too small to obtain requested accuracy `eps`:
            `maxpts` = *<value>* and `eps` = *<value>*.

        (`errno` 3)
            lenwrk was too small to complete computation. `finval` and
            `acc` contain estimates of integral and relative error, but
            `acc` is greater than `eps`.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.md_adapt'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ndim.
        utils._EngineFloat64ArrayType.c_type, # Input, z[ndim].
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, _fsub, en_data, ndim, z, retval, _iuser, _ruser):
        """Helper for f."""
        try:
            _ndim_fh = utils._EngineIntScalarType.to_py(ndim.contents)
            _z_fh = utils._EngineFloat64ArrayType.from_pointer(
                z,
                (_ndim_fh,),
            )
            if data is not None:
                _retval_fh = f(
                    _ndim_fh,
                    _z_fh,
                    data,
                )
            else:
                _retval_fh = f(
                    _ndim_fh,
                    _z_fh,
                )
            utils._unlink_pointer(z)
            utils._EngineFloat64ScalarType(
                _retval_fh, {'fun_name': cb_f_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d01fcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ndim.
        utils._EngineFloat64ArrayType.c_type, # Input, a[ndim].
        utils._EngineFloat64ArrayType.c_type, # Input, b[ndim].
        utils._BYREF_INT, # Input/Output, minpts.
        utils._BYREF_INT, # Input, maxpts.
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._BYREF_FLOAT64, # Input, eps.
        utils._BYREF_FLOAT64, # Output, acc.
        utils._BYREF_INT, # Input, lenwrk.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wrkstr.
        utils._BYREF_FLOAT64, # Output, finval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ndim = utils._EngineIntScalarType(
        ndim, {'fun_name': fun_name, 'entity': 'ndim'},
    ).to_ctypes()
    _ndim_py = utils._EngineIntScalarType.to_py(_ndim)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_ndim_py,),
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_ndim_py,),
    ).to_ctypes()
    _minpts = utils._EngineIntScalarType(
        minpts, {'fun_name': fun_name, 'entity': 'minpts'},
    ).to_ctypes()
    _maxpts = utils._EngineIntScalarType(
        maxpts, {'fun_name': fun_name, 'entity': 'maxpts'},
    ).to_ctypes()
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fsub = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _eps = utils._EngineFloat64ScalarType(
        eps, {'fun_name': fun_name, 'entity': 'eps'},
    ).to_ctypes()
    _acc = utils._EngineFloat64ScalarType.empty_ctypes()
    _lenwrk = utils._EngineIntScalarType.c_type(0)
    _wrkstr = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _finval = utils._EngineFloat64ScalarType.empty_ctypes()
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ndim,
        _a,
        _b,
        _minpts,
        _maxpts,
        _f,
        _fsub,
        _fh,
        _eps,
        _acc,
        _lenwrk,
        _wrkstr,
        _finval,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, maxpts is too small.',
                    'maxpts = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Minimum possible dimension: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, eps = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: eps > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, minpts = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'maxpts = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: minpts <= maxpts.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ndim = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ndim <= 15.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ndim = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ndim >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99993:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'maxpts too small to obtain requested accuracy eps:',
                    'maxpts = {:s} and eps = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99992:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'lenwrk was too small to complete computation.',
                    'finval and acc contain estimates of integral and',
                    'relative error, but acc is greater than eps.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _minpts_py = utils._EngineIntScalarType.to_py(_minpts)
        _acc_py = utils._EngineFloat64ScalarType.to_py(_acc)
        _finval_py = utils._EngineFloat64ScalarType.to_py(_finval)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['minpts', 'acc', 'finval'],
        )(
            _minpts_py,
            _acc_py,
            _finval_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def md_sphere(ndim, f, sigma, region, limit, r0, u, data=None):
    u"""
    Multidimensional quadrature, Sag--Szekeres method, general product
    region or n-sphere.

    For full information please refer to the NAG Library document for
    d01fd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01fdf.html

    Parameters
    ----------
    ndim : int
        n, the number of dimensions of the integral.

    f : callable retval = f(ndim, x, data=None)
        `f` must return the value of the integrand f at a given point.

        Parameters
        ~~~~~~~~~~
        ndim : int
            n, the number of dimensions of the integral.

        x : float, ndarray, shape (`ndim`)
            The coordinates of the point at which the integrand f must
            be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of f(x) evaluated at `x`.

    sigma : float
        Indicates the region of integration.

        `sigma` >= 0.0

          The integration is carried out over the n-sphere of radius
          `sigma`, centred at the origin.

        `sigma` < 0.0

          The integration is carried out over the product region
          described by `region`.

    region : None or callable (c, d) = region(ndim, x, j, data=None)
        If `sigma` < 0.0, `region` must evaluate the limits of
        integration in any dimension.

        Parameters
        ~~~~~~~~~~
        ndim : int
            n, the number of dimensions of the integral.

        x : float, ndarray, shape (`ndim`)
            `x`[0],...,`x`[j-2] contain the current values of the first
            (j-1) variables, which may be used if necessary in
            calculating c_j and d_j.

        j : int
            The index j for which the limits of the range of integration
            are required.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        c : float
            The lower limit c_j of the range of x_j.

        d : float
            The upper limit d_j of the range of x_j.

    limit : int
        The approximate maximum number of integrand evaluations to be
        used.

    r0 : float
        The cut-off radius on the unit n-sphere, which may be regarded
        as an adjustable parameter of the method.

    u : float
        Must specify an adjustable parameter of the transformation to
        the unit n-sphere.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    result : float
        The approximation to the integral I.

    ncalls : int
        The actual number of integrand evaluations used. (See also
        [Further Comments].)

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ndim` = *<value>*.

            Constraint: `ndim` <= 30.

        (`errno` 1)
            On entry, `ndim` = *<value>*.

            Constraint: `ndim` >= 1.

        (`errno` 2)
            On entry, `limit` = *<value>*.

            Constraint: `limit` >= 100.

        (`errno` 3)
            On entry, `r0` = *<value>*.

            Constraint: `r0` < 1.0.

        (`errno` 3)
            On entry, `r0` = *<value>*.

            Constraint: `r0` > 0.0.

        (`errno` 4)
            On entry, `u` = *<value>*.

            Constraint: `u` > 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.md_sphere'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ndim.
        utils._EngineFloat64ArrayType.c_type, # Input, x[ndim].
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, _fsub, en_data, ndim, x, retval, _iuser, _ruser):
        """Helper for f."""
        try:
            _ndim_fh = utils._EngineIntScalarType.to_py(ndim.contents)
            _x_fh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_ndim_fh,),
            )
            if data is not None:
                _retval_fh = f(
                    _ndim_fh,
                    _x_fh,
                    data,
                )
            else:
                _retval_fh = f(
                    _ndim_fh,
                    _x_fh,
                )
            utils._unlink_pointer(x)
            utils._EngineFloat64ScalarType(
                _retval_fh, {'fun_name': cb_f_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_region_name = fun_name + ':region'

    regionh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ndim.
        utils._EngineFloat64ArrayType.c_type, # Input, x[ndim].
        utils._BYREF_INT, # Input, j.
        utils._BYREF_FLOAT64, # Output, c.
        utils._BYREF_FLOAT64, # Output, d.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_regionh(_region, en_data, ndim, x, j, c, d, _iuser, _ruser):
        """Helper for region."""
        if region is None:
            return
        try:
            _ndim_regionh = utils._EngineIntScalarType.to_py(ndim.contents)
            _x_regionh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_ndim_regionh,),
            )
            _j_regionh = utils._EngineIntScalarType.to_py(j.contents)
            if data is not None:
                _c_regionh, _d_regionh = region(
                    _ndim_regionh,
                    _x_regionh,
                    _j_regionh,
                    data,
                )
            else:
                _c_regionh, _d_regionh = region(
                    _ndim_regionh,
                    _x_regionh,
                    _j_regionh,
                )
            utils._unlink_pointer(x)
            utils._EngineFloat64ScalarType(
                _c_regionh, {'fun_name': cb_region_name, 'returned': 'c'},
            ).refresh_ctypes_buffer(c)
            utils._EngineFloat64ScalarType(
                _d_regionh, {'fun_name': cb_region_name, 'returned': 'd'},
            ).refresh_ctypes_buffer(d)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d01fdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ndim.
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._BYREF_FLOAT64, # Input, sigma.
        utils._UNUSED_FUNCTYPE,
        regionh,
        utils._BYREF_INT, # Input, limit.
        utils._BYREF_FLOAT64, # Input, r0.
        utils._BYREF_FLOAT64, # Input, u.
        utils._BYREF_FLOAT64, # Output, result.
        utils._BYREF_INT, # Output, ncalls.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ndim = utils._EngineIntScalarType(
        ndim, {'fun_name': fun_name, 'entity': 'ndim'},
    ).to_ctypes()
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fsub = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _sigma = utils._EngineFloat64ScalarType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
    ).to_ctypes()
    _sigma_py = utils._EngineFloat64ScalarType.to_py(_sigma)
    if _sigma_py < 0.0:
        if region is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'region'},
                'must not be None when sigma < 0.0',
            )
    utils._check_callable(region, {'fun_name': fun_name, 'entity': 'region'}, allow_none=True)
    _region = utils._UNUSED_CALLBACK
    _regionh = utils._cast_callback(
        regionh, cb_regionh,
        {'fun_name': fun_name, 'entity': 'region'},
    )
    _limit = utils._EngineIntScalarType(
        limit, {'fun_name': fun_name, 'entity': 'limit'},
    ).to_ctypes()
    _r0 = utils._EngineFloat64ScalarType(
        r0, {'fun_name': fun_name, 'entity': 'r0'},
    ).to_ctypes()
    _u = utils._EngineFloat64ScalarType(
        u, {'fun_name': fun_name, 'entity': 'u'},
    ).to_ctypes()
    _result = utils._EngineFloat64ScalarType.empty_ctypes()
    _ncalls = utils._EngineIntScalarType.empty_ctypes()
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ndim,
        _f,
        _fsub,
        _fh,
        _sigma,
        _region,
        _regionh,
        _limit,
        _r0,
        _u,
        _result,
        _ncalls,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ndim = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ndim <= 30.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ndim = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ndim >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, limit = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: limit >= 100.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, r0 = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: r0 < 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, r0 = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: r0 > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, u = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: u > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _result_py = utils._EngineFloat64ScalarType.to_py(_result)
        _ncalls_py = utils._EngineIntScalarType.to_py(_ncalls)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['result', 'ncalls'],
        )(
            _result_py,
            _ncalls_py,
        )
    _err_class.manage()
    return return_data

def dim1_data(x, y, n):
    u"""
    One-dimensional quadrature, integration of function defined by data
    values, Gill--Miller method.

    For full information please refer to the NAG Library document for
    d01ga

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01gaf.html

    Parameters
    ----------
    x : float, shape (`n`)
        The values of the independent variable, i.e., the
        x_1,x_2,...,x_n.

    y : float, shape (`n`)
        The values of the dependent variable y_i at the points x_i, for
        i = 1,2,...,n.

    n : int
        n, the number of points.

    Returns
    -------
    ans : float
        The estimated value of the integral.

    er : float
        An estimate of the uncertainty in `ans`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 4.

        (`errno` 2)
            On entry, `x`[0] = *<value>* and `x`[1] = *<value>*,
            `x`[I-2] = *<value>*, `x`[I-1] = *<value>* and I =
            *<value>*.

            Constraint: either `x`[0] < `x`[1] < ... < `x`[`n`-1] or
            `x`[0] > `x`[1] > ... > `x`[`n`-1].

        (`errno` 3)
            On entry, I = *<value>*, `x`[I-1] = *<value>* and `x`[I-2] =
            *<value>*.

            Constraint: `x`[I-1] != `x`[I-2].
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.dim1_data'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d01gaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Output, ans.
        utils._BYREF_FLOAT64, # Output, er.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _y = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ans = utils._EngineFloat64ScalarType.empty_ctypes()
    _er = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _y,
        _n,
        _ans,
        _er,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x[0] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'x[1] = {:s}, x[I-2] = {:s},'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'x[I-1] = {:s} and I = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[5]),
                    ),
                    'Constraint: either x[0] < x[1] < ... < x[n-1] or x[0] > x[1] > ... > x[n-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, I = {:s}, x[I-1] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'x[I-2] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: x[I-1] != x[I-2].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ans_py = utils._EngineFloat64ScalarType.to_py(_ans)
        _er_py = utils._EngineFloat64ScalarType.to_py(_er)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ans', 'er'],
        )(
            _ans_py,
            _er_py,
        )
    _err_class.manage()
    return return_data

def md_mcarlo(ndim, a, b, mincls, maxcls, f, eps, lenwrk, wrkstr, finest, data=None):
    u"""
    Multidimensional quadrature over hyper-rectangle, Monte Carlo
    method.

    For full information please refer to the NAG Library document for
    d01gb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01gbf.html

    Parameters
    ----------
    ndim : int
        n, the number of dimensions of the integral.

    a : float, shape (`ndim`)
        The lower limits of integration, a_i, for i = 1,2,...,n.

    b : float, shape (`ndim`)
        The upper limits of integration, b_i, for i = 1,2,...,n.

    mincls : int
        Must be set

        - either to the minimum number of integrand evaluations to be
          allowed, in which case `mincls` >= 0;

        - or to a negative value. In this case, the function assumes
          that a previous call had been made with the same arguments
          `ndim`, `a` and `b` and with either the same integrand (in
          which case ``md_mcarlo`` continues calculation) or a similar
          integrand (in which case ``md_mcarlo`` begins the calculation
          with the subdivision used in the last iteration of the
          previous call). See also `wrkstr`.

    maxcls : int
        The maximum number of integrand evaluations to be allowed. In
        the continuation case this is the number of new integrand
        evaluations to be allowed. These counts do not include zero
        integrand values.

    f : callable retval = f(ndim, x, data=None)
        `f` must return the value of the integrand at a given point.

        Parameters
        ~~~~~~~~~~
        ndim : int
            n, the number of dimensions of the integral.

        x : float, ndarray, shape (`ndim`)
            The coordinates of the point at which the integrand f must
            be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of the integrand f at the given point.

    eps : float
        The relative accuracy required.

    lenwrk : int
        For maximum efficiency, `lenwrk` should be about

            3*`ndim`*(`maxcls`/4)^(1/`ndim`)+7*`ndim`.

        If `lenwrk` is given the value 10*`ndim` then the function uses
        only one iteration of a crude Monte Carlo method with `maxcls`
        sample points.

    wrkstr : float, shape (`lenwrk`), modified in place
        On entry: if `mincls` < 0, `wrkstr` must be unchanged from the
        previous call of ``md_mcarlo`` -- except that for a new
        integrand `wrkstr`[`lenwrk`-1] must be set to 0.0. See also
        `mincls`.

        On exit: contains information about the current sub-interval
        structure which could be used in later calls of ``md_mcarlo``.
        In particular, `wrkstr`[j-1] gives the number of sub-intervals
        used along the jth coordinate axis.

    finest : float
        Must be unchanged from a previous call to ``md_mcarlo``.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    mincls : int
        Contains the number of integrand evaluations actually used by
        ``md_mcarlo``.

    acc : float
        The estimated relative accuracy of `finest`.

    finest : float
        The best estimate obtained for the integral.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `eps` = *<value>*.

            Constraint: `eps` >= 0.0.

        (`errno` 1)
            On entry, `maxcls` = *<value>* and `ndim` = *<value>*.

            Constraint: `maxcls` >= 4*(`ndim`+1).

        (`errno` 1)
            On entry, `maxcls` = *<value>* and `mincls` = *<value>*.

            Constraint: `maxcls` > `mincls`.

        (`errno` 1)
            On entry, `lenwrk` is too small. `lenwrk` = *<value>*.
            Minimum possible dimension: *<value>*.

        (`errno` 1)
            On entry, `ndim` = *<value>*.

            Constraint: `ndim` >= 1.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            `maxcls` too small to obtain required accuracy `eps`.
            `maxcls` = *<value>* and `eps` = *<value>*.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.md_mcarlo'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ndim.
        utils._EngineFloat64ArrayType.c_type, # Input, x[ndim].
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, _fsub, en_data, ndim, x, retval, _iuser, _ruser):
        """Helper for f."""
        try:
            _ndim_fh = utils._EngineIntScalarType.to_py(ndim.contents)
            _x_fh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_ndim_fh,),
            )
            if data is not None:
                _retval_fh = f(
                    _ndim_fh,
                    _x_fh,
                    data,
                )
            else:
                _retval_fh = f(
                    _ndim_fh,
                    _x_fh,
                )
            utils._unlink_pointer(x)
            utils._EngineFloat64ScalarType(
                _retval_fh, {'fun_name': cb_f_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d01gbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ndim.
        utils._EngineFloat64ArrayType.c_type, # Input, a[ndim].
        utils._EngineFloat64ArrayType.c_type, # Input, b[ndim].
        utils._BYREF_INT, # Input/Output, mincls.
        utils._BYREF_INT, # Input, maxcls.
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._BYREF_FLOAT64, # Input, eps.
        utils._BYREF_FLOAT64, # Output, acc.
        utils._BYREF_INT, # Input, lenwrk.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, wrkstr[lenwrk].
        utils._BYREF_FLOAT64, # Input/Output, finest.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ndim = utils._EngineIntScalarType(
        ndim, {'fun_name': fun_name, 'entity': 'ndim'},
    ).to_ctypes()
    _ndim_py = utils._EngineIntScalarType.to_py(_ndim)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_ndim_py,),
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_ndim_py,),
    ).to_ctypes()
    _mincls = utils._EngineIntScalarType(
        mincls, {'fun_name': fun_name, 'entity': 'mincls'},
    ).to_ctypes()
    _maxcls = utils._EngineIntScalarType(
        maxcls, {'fun_name': fun_name, 'entity': 'maxcls'},
    ).to_ctypes()
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fsub = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _eps = utils._EngineFloat64ScalarType(
        eps, {'fun_name': fun_name, 'entity': 'eps'},
    ).to_ctypes()
    _acc = utils._EngineFloat64ScalarType.empty_ctypes()
    _lenwrk = utils._EngineIntScalarType(
        lenwrk, {'fun_name': fun_name, 'entity': 'lenwrk'},
    ).to_ctypes()
    _lenwrk_py = utils._EngineIntScalarType.to_py(_lenwrk)
    _wrkstr_cls = utils._EngineFloat64ArrayType(
        wrkstr, {'fun_name': fun_name, 'entity': 'wrkstr'},
        exp_shape=(_lenwrk_py,),
    )
    _wrkstr = _wrkstr_cls.to_ctypes()
    _finest = utils._EngineFloat64ScalarType(
        finest, {'fun_name': fun_name, 'entity': 'finest'},
    ).to_ctypes()
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ndim,
        _a,
        _b,
        _mincls,
        _maxcls,
        _f,
        _fsub,
        _fh,
        _eps,
        _acc,
        _lenwrk,
        _wrkstr,
        _finest,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, eps = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: eps >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, maxcls = {:s} and ndim = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: maxcls >= 4*(ndim+1).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, maxcls = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'mincls = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: maxcls > mincls.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lenwrk is too small.',
                    'lenwrk = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Minimum possible dimension: {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ndim = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ndim >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99994:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'maxcls too small to obtain required accuracy eps.',
                    'maxcls = {:s} and eps = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _mincls_py = utils._EngineIntScalarType.to_py(_mincls)
        _wrkstr_cls.refresh_py(_wrkstr)
        _finest_py = utils._EngineFloat64ScalarType.to_py(_finest)
        _acc_py = utils._EngineFloat64ScalarType.to_py(_acc)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['mincls', 'acc', 'finest'],
        )(
            _mincls_py,
            _acc_py,
            _finest_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def md_numth(ndim, f, region, npts, vk, nrand, itrans, data=None):
    u"""
    Multidimensional quadrature, general product region,
    number-theoretic method.

    For full information please refer to the NAG Library document for
    d01gc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01gcf.html

    Parameters
    ----------
    ndim : int
        n, the number of dimensions of the integral.

    f : callable retval = f(ndim, x, data=None)
        `f` must return the value of the integrand f at a given point.

        Parameters
        ~~~~~~~~~~
        ndim : int
            n, the number of dimensions of the integral.

        x : float, ndarray, shape (`ndim`)
            The coordinates of the point at which the integrand f must
            be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of f(x) evaluated at `x`.

    region : callable (c, d) = region(ndim, x, j, data=None)
        `region` must evaluate the limits of integration in any
        dimension.

        Parameters
        ~~~~~~~~~~
        ndim : int
            n, the number of dimensions of the integral.

        x : float, ndarray, shape (`ndim`)
            `x`[0],...,`x`[j-2] contain the current values of the first
            (j-1) variables, which may be used if necessary in
            calculating c_j and d_j.

        j : int
            The index j for which the limits of the range of integration
            are required.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        c : float
            The lower limit c_j of the range of x_j.

        d : float
            The upper limit d_j of the range of x_j.

    npts : int
        The Korobov rule to be used. There are two alternatives
        depending on the value of `npts`.

        - 1 <= `npts` <= 6.

          In this case one of six preset rules is chosen using 2129,
          5003, 10007, 20011, 40009 or 80021 points depending on the
          respective value of `npts` being 1, 2, 3, 4, 5 or 6.

        - `npts` > 6.

          `npts` is the number of actual points to be used with
          corresponding optimal coefficients supplied in the array `vk`.

    vk : float, shape (`ndim`), modified in place
        On entry: if `npts` > 6, `vk` must contain the n optimal
        coefficients (which may be calculated using
        ``md_numth_coeff_prime`` or ``md_numth_coeff_2prime``).

        If `npts` <= 6, `vk` need not be set.

        On exit: if `npts` > 6, `vk` is unchanged.

        If `npts` <= 6, `vk` contains the n optimal coefficients used by
        the preset rule.

    nrand : int
        The number of random samples to be generated in the error
        estimation (generally a small value, say 3 to 5, is sufficient).
        The total number of integrand evaluations will be
        `nrand`*`npts`.

    itrans : int
        Indicates whether the periodising transformation is to be used.

        `itrans` = '0'

          The transformation is to be used.

        `itrans` != '0'

          The transformation is to be suppressed (to cover cases where
          the integrand may already be periodic or where you want to
          specify a particular transformation in the definition of `f`).

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    res : float
        The approximation to the integral I.

    err : float
        The standard error as computed from `nrand` sample values. If
        `nrand` = 1, `err` contains zero.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ndim` = *<value>*.

            Constraint: `ndim` <= 20.

        (`errno` 1)
            On entry, `ndim` = *<value>*.

            Constraint: `ndim` >= 1.

        (`errno` 2)
            On entry, `npts` = *<value>*.

            Constraint: `npts` >= 1.

        (`errno` 3)
            On entry, `nrand` = *<value>*.

            Constraint: `nrand` >= 1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.md_numth'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ndim.
        utils._EngineFloat64ArrayType.c_type, # Input, x[ndim].
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, _fsub, en_data, ndim, x, retval, _iuser, _ruser):
        """Helper for f."""
        try:
            _ndim_fh = utils._EngineIntScalarType.to_py(ndim.contents)
            _x_fh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_ndim_fh,),
            )
            if data is not None:
                _retval_fh = f(
                    _ndim_fh,
                    _x_fh,
                    data,
                )
            else:
                _retval_fh = f(
                    _ndim_fh,
                    _x_fh,
                )
            utils._unlink_pointer(x)
            utils._EngineFloat64ScalarType(
                _retval_fh, {'fun_name': cb_f_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_region_name = fun_name + ':region'

    regionh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ndim.
        utils._EngineFloat64ArrayType.c_type, # Input, x[ndim].
        utils._BYREF_INT, # Input, j.
        utils._BYREF_FLOAT64, # Output, c.
        utils._BYREF_FLOAT64, # Output, d.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_regionh(_region, en_data, ndim, x, j, c, d, _iuser, _ruser):
        """Helper for region."""
        try:
            _ndim_regionh = utils._EngineIntScalarType.to_py(ndim.contents)
            _x_regionh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_ndim_regionh,),
            )
            _j_regionh = utils._EngineIntScalarType.to_py(j.contents)
            if data is not None:
                _c_regionh, _d_regionh = region(
                    _ndim_regionh,
                    _x_regionh,
                    _j_regionh,
                    data,
                )
            else:
                _c_regionh, _d_regionh = region(
                    _ndim_regionh,
                    _x_regionh,
                    _j_regionh,
                )
            utils._unlink_pointer(x)
            utils._EngineFloat64ScalarType(
                _c_regionh, {'fun_name': cb_region_name, 'returned': 'c'},
            ).refresh_ctypes_buffer(c)
            utils._EngineFloat64ScalarType(
                _d_regionh, {'fun_name': cb_region_name, 'returned': 'd'},
            ).refresh_ctypes_buffer(d)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d01gcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ndim.
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._UNUSED_FUNCTYPE,
        regionh,
        utils._BYREF_INT, # Input, npts.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, vk[ndim].
        utils._BYREF_INT, # Input, nrand.
        utils._BYREF_INT, # Input, itrans.
        utils._BYREF_FLOAT64, # Output, res.
        utils._BYREF_FLOAT64, # Output, err.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ndim = utils._EngineIntScalarType(
        ndim, {'fun_name': fun_name, 'entity': 'ndim'},
    ).to_ctypes()
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fsub = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    utils._check_callable(region, {'fun_name': fun_name, 'entity': 'region'})
    _region = utils._UNUSED_CALLBACK
    _regionh = utils._cast_callback(
        regionh, cb_regionh,
        {'fun_name': fun_name, 'entity': 'region'},
    )
    _npts = utils._EngineIntScalarType(
        npts, {'fun_name': fun_name, 'entity': 'npts'},
    ).to_ctypes()
    _ndim_py = utils._EngineIntScalarType.to_py(_ndim)
    _vk_cls = utils._EngineFloat64ArrayType(
        vk, {'fun_name': fun_name, 'entity': 'vk'},
        exp_shape=(_ndim_py,),
    )
    _vk = _vk_cls.to_ctypes()
    _nrand = utils._EngineIntScalarType(
        nrand, {'fun_name': fun_name, 'entity': 'nrand'},
    ).to_ctypes()
    _itrans = utils._EngineIntScalarType(
        itrans, {'fun_name': fun_name, 'entity': 'itrans'},
    ).to_ctypes()
    _res = utils._EngineFloat64ScalarType.empty_ctypes()
    _err = utils._EngineFloat64ScalarType.empty_ctypes()
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ndim,
        _f,
        _fsub,
        _fh,
        _region,
        _regionh,
        _npts,
        _vk,
        _nrand,
        _itrans,
        _res,
        _err,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ndim = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ndim <= 20.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ndim = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ndim >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npts = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npts >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nrand = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nrand >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _vk_cls.refresh_py(_vk)
        _res_py = utils._EngineFloat64ScalarType.to_py(_res)
        _err_py = utils._EngineFloat64ScalarType.to_py(_err)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['res', 'err'],
        )(
            _res_py,
            _err_py,
        )
    _err_class.manage()
    return return_data

def md_numth_vec(ndim, vecfun, vecreg, npts, vk, nrand, itrans, data=None):
    u"""
    Multidimensional quadrature, general product region,
    number-theoretic method, variant of ``md_numth`` efficient on vector
    machines.

    For full information please refer to the NAG Library document for
    d01gd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01gdf.html

    Parameters
    ----------
    ndim : int
        n, the number of dimensions of the integral.

    vecfun : callable vecfun(ndim, x, fv, m, data=None)
        `vecfun` must evaluate the integrand at a specified set of
        points.

        Parameters
        ~~~~~~~~~~
        ndim : int
            n, the number of dimensions of the integral.

        x : float, ndarray, shape (`m`, `ndim`)
            The coordinates of the m points at which the integrand must
            be evaluated. `x`[i-1,j-1] contains the jth coordinate of
            the ith point.

        fv : float, ndarray, shape (`m`), to be modified in place
            On exit: `fv`[i-1] must contain the value of the integrand
            of the ith point, i.e., `fv`[i-1] =
            f(`x`[i-1,0],`x`[i-1,1],...,`x`[i-1,`ndim`-1]), for i =
            1,2,...,`m`.

        m : int
            The number of points m at which the integrand is to be
            evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    vecreg : callable vecreg(ndim, x, j, c, d, m, data=None)
        `vecreg` must evaluate the limits of integration in any
        dimension for a set of points.

        Parameters
        ~~~~~~~~~~
        ndim : int
            n, the number of dimensions of the integral.

        x : float, ndarray, shape (`m`, `ndim`)
            For i = 1,2,...,m, `x`[i-1,0], `x`[i-1,1],...,`x`[i-1,j-2]
            contain the current values of the first (j-1) coordinates of
            the ith point, which may be used if necessary in calculating
            the m values of c_j and d_j.

        j : int
            The index j for which the limits of the range of integration
            are required.

        c : float, ndarray, shape (`m`), to be modified in place
            On exit: `c`[i-1] must be set to the lower limit of the
            range for `x`[i-1,j-1], for i = 1,2,...,m.

        d : float, ndarray, shape (`m`), to be modified in place
            On exit: `d`[i-1] must be set to the upper limit of the
            range for `x`[i-1,j-1], for i = 1,2,...,m.

        m : int
            The number of points m at which the limits of integration
            must be specified.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    npts : int
        The Korobov rule to be used. There are two alternatives
        depending on the value of `npts`.

        - 1 <= `npts` <= 6.

          In this case one of six preset rules is chosen using 2129,
          5003, 10007, 20011, 40009 or 80021 points depending on the
          respective value of `npts` being 1, 2, 3, 4, 5 or 6.

        - `npts` > 6.

          `npts` is the number of actual points to be used with
          corresponding optimal coefficients supplied in the array `vk`.

    vk : float, shape (`ndim`), modified in place
        On entry: if `npts` > 6, `vk` must contain the n optimal
        coefficients (which may be calculated using
        ``md_numth_coeff_prime`` or ``md_numth_coeff_2prime``).

        If `npts` <= 6, `vk` need not be set.

        On exit: if `npts` > 6, `vk` is unchanged.

        If `npts` <= 6, `vk` contains the n optimal coefficients used by
        the preset rule.

    nrand : int
        The number of random samples to be generated (generally a small
        value, say 3 to 5, is sufficient). The estimate, `res`, of the
        value of the integral returned by the function is then the
        average of `nrand` calculations with different random origin
        shifts. If `npts` > 6, the total number of integrand evaluations
        will be `nrand`*`npts`. If 1 <= `npts` <= 6, the number of
        integrand evaluations will be `nrand`*p, where p is the number
        of points corresponding to the six preset rules. For reasons of
        efficiency, these values are calculated a number at a time in
        `vecfun`.

    itrans : int
        Indicates whether the periodising transformation is to be used.

        `itrans` = 0

          The transformation is to be used.

        `itrans` = 1

          The transformation is to be suppressed (to cover cases where
          the integrand may already be periodic or where you want to
          specify a particular transformation in the definition of
          `vecfun`).

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    res : float
        The approximation to the integral I.

    err : float
        The standard error as computed from `nrand` sample values. If
        `nrand` = 1, `err` contains zero.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ndim` = *<value>*.

            Constraint: 1 <= `ndim` <= 20.

        (`errno` 2)
            On entry, `npts` = *<value>*.

            Constraint: `npts` >= 1.

        (`errno` 3)
            On entry, `nrand` = *<value>*.

            Constraint: `nrand` >= 1.
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.md_numth_vec'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_vecfun_name = fun_name + ':vecfun'

    vecfunh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ndim.
        utils._EngineFloat64ArrayType.c_type, # Input, x[m, ndim].
        utils._EngineFloat64ArrayType.c_type, # Output, fv[m].
        utils._BYREF_INT, # Input, m.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_vecfunh(_vecfun, en_data, ndim, x, fv, m, _iuser, _ruser):
        """Helper for vecfun."""
        try:
            _ndim_vecfunh = utils._EngineIntScalarType.to_py(ndim.contents)
            _m_vecfunh = utils._EngineIntScalarType.to_py(m.contents)
            _x_vecfunh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_m_vecfunh, _ndim_vecfunh),
                sorder=utils._EngineData.default_sorder,
            )
            _fv_vecfunh = utils._EngineFloat64ArrayType.from_pointer(
                fv,
                (_m_vecfunh,),
                can_reference_contents=False,
            )
            if data is not None:
                vecfun(
                    _ndim_vecfunh,
                    _x_vecfunh,
                    _fv_vecfunh,
                    _m_vecfunh,
                    data,
                )
            else:
                vecfun(
                    _ndim_vecfunh,
                    _x_vecfunh,
                    _fv_vecfunh,
                    _m_vecfunh,
                )
            utils._unlink_pointer(x)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _fv_vecfunh, {'fun_name': cb_vecfun_name, 'returned': 'fv'}, fv,
            )
            utils._unlink_pointer(fv)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    cb_vecreg_name = fun_name + ':vecreg'

    vecregh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ndim.
        utils._EngineFloat64ArrayType.c_type, # Input, x[m, ndim].
        utils._BYREF_INT, # Input, j.
        utils._EngineFloat64ArrayType.c_type, # Output, c[m].
        utils._EngineFloat64ArrayType.c_type, # Output, d[m].
        utils._BYREF_INT, # Input, m.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_vecregh(_vecreg, en_data, ndim, x, j, c, d, m, _iuser, _ruser):
        """Helper for vecreg."""
        try:
            _ndim_vecregh = utils._EngineIntScalarType.to_py(ndim.contents)
            _m_vecregh = utils._EngineIntScalarType.to_py(m.contents)
            _x_vecregh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_m_vecregh, _ndim_vecregh),
                sorder=utils._EngineData.default_sorder,
            )
            _j_vecregh = utils._EngineIntScalarType.to_py(j.contents)
            _c_vecregh = utils._EngineFloat64ArrayType.from_pointer(
                c,
                (_m_vecregh,),
                can_reference_contents=False,
            )
            _d_vecregh = utils._EngineFloat64ArrayType.from_pointer(
                d,
                (_m_vecregh,),
                can_reference_contents=False,
            )
            if data is not None:
                vecreg(
                    _ndim_vecregh,
                    _x_vecregh,
                    _j_vecregh,
                    _c_vecregh,
                    _d_vecregh,
                    _m_vecregh,
                    data,
                )
            else:
                vecreg(
                    _ndim_vecregh,
                    _x_vecregh,
                    _j_vecregh,
                    _c_vecregh,
                    _d_vecregh,
                    _m_vecregh,
                )
            utils._unlink_pointer(x)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _c_vecregh, {'fun_name': cb_vecreg_name, 'returned': 'c'}, c,
            )
            utils._unlink_pointer(c)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _d_vecregh, {'fun_name': cb_vecreg_name, 'returned': 'd'}, d,
            )
            utils._unlink_pointer(d)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d01gdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ndim.
        utils._UNUSED_FUNCTYPE,
        vecfunh,
        utils._UNUSED_FUNCTYPE,
        vecregh,
        utils._BYREF_INT, # Input, npts.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, vk[ndim].
        utils._BYREF_INT, # Input, nrand.
        utils._BYREF_INT, # Input, itrans.
        utils._BYREF_FLOAT64, # Output, res.
        utils._BYREF_FLOAT64, # Output, err.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ndim = utils._EngineIntScalarType(
        ndim, {'fun_name': fun_name, 'entity': 'ndim'},
    ).to_ctypes()
    utils._check_callable(vecfun, {'fun_name': fun_name, 'entity': 'vecfun'})
    _vecfun = utils._UNUSED_CALLBACK
    _vecfunh = utils._cast_callback(
        vecfunh, cb_vecfunh,
        {'fun_name': fun_name, 'entity': 'vecfun'},
    )
    utils._check_callable(vecreg, {'fun_name': fun_name, 'entity': 'vecreg'})
    _vecreg = utils._UNUSED_CALLBACK
    _vecregh = utils._cast_callback(
        vecregh, cb_vecregh,
        {'fun_name': fun_name, 'entity': 'vecreg'},
    )
    _npts = utils._EngineIntScalarType(
        npts, {'fun_name': fun_name, 'entity': 'npts'},
    ).to_ctypes()
    _ndim_py = utils._EngineIntScalarType.to_py(_ndim)
    _vk_cls = utils._EngineFloat64ArrayType(
        vk, {'fun_name': fun_name, 'entity': 'vk'},
        exp_shape=(_ndim_py,),
    )
    _vk = _vk_cls.to_ctypes()
    _nrand = utils._EngineIntScalarType(
        nrand, {'fun_name': fun_name, 'entity': 'nrand'},
    ).to_ctypes()
    _itrans = utils._EngineIntScalarType(
        itrans, {'fun_name': fun_name, 'entity': 'itrans'},
    ).to_ctypes()
    _res = utils._EngineFloat64ScalarType.empty_ctypes()
    _err = utils._EngineFloat64ScalarType.empty_ctypes()
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ndim,
        _vecfun,
        _vecfunh,
        _vecreg,
        _vecregh,
        _npts,
        _vk,
        _nrand,
        _itrans,
        _res,
        _err,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ndim = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 1 <= ndim <= 20.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npts = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npts >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nrand = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nrand >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _vk_cls.refresh_py(_vk)
        _res_py = utils._EngineFloat64ScalarType.to_py(_res)
        _err_py = utils._EngineFloat64ScalarType.to_py(_err)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['res', 'err'],
        )(
            _res_py,
            _err_py,
        )
    _err_class.manage()
    return return_data

def md_numth_coeff_prime(ndim, npts, vk):
    u"""
    Korobov optimal coefficients for use in ``md_numth`` or
    ``md_numth_vec``, when number of points is prime.

    For full information please refer to the NAG Library document for
    d01gy

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01gyf.html

    Parameters
    ----------
    ndim : int
        n, the number of dimensions of the integral.

    npts : int
        p, the number of points to be used.

    vk : float, shape (`ndim`), modified in place
        On exit: the n optimal coefficients.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ndim` = *<value>*.

            Constraint: `ndim` >= 1.

        (`errno` 2)
            On entry, `npts` = *<value>*.

            Constraint: `npts` >= 5.

        (`errno` 3)
            On entry, `npts` = *<value>*.

            Constraint: `npts` must be a prime number.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            The machine precision is insufficient to perform the
            computation exactly. Try reducing `npts`: `npts` =
            *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.md_numth_coeff_prime'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d01gyft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ndim.
        utils._BYREF_INT, # Input, npts.
        utils._EngineFloat64ArrayType.c_type, # Output, vk[ndim].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ndim = utils._EngineIntScalarType(
        ndim, {'fun_name': fun_name, 'entity': 'ndim'},
    ).to_ctypes()
    _npts = utils._EngineIntScalarType(
        npts, {'fun_name': fun_name, 'entity': 'npts'},
    ).to_ctypes()
    _ndim_py = utils._EngineIntScalarType.to_py(_ndim)
    _vk_cls = utils._EngineFloat64ArrayType(
        vk, {'fun_name': fun_name, 'entity': 'vk'},
        exp_shape=(_ndim_py,),
        check_contents=False,
    )
    _vk = _vk_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ndim,
        _npts,
        _vk,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ndim = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ndim >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npts = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npts >= 5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npts = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: npts must be a prime number.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99996:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The machine precision is insufficient to perform the computation',
                    'exactly. Try reducing npts: npts = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _vk_cls.refresh_py(_vk)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def md_numth_coeff_2prime(ndim, np1, np2, vk):
    u"""
    Korobov optimal coefficients for use in ``md_numth`` or
    ``md_numth_vec``, when number of points is product of two primes.

    For full information please refer to the NAG Library document for
    d01gz

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01gzf.html

    Parameters
    ----------
    ndim : int
        n, the number of dimensions of the integral.

    np1 : int
        The larger prime factor p_1 of the number of points in the
        integration rule.

    np2 : int
        The smaller prime factor p_2 of the number of points in the
        integration rule. For maximum efficiency, p_2^2 should be close
        to p_1.

    vk : float, shape (`ndim`), modified in place
        On exit: the n optimal coefficients.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ndim` = *<value>*.

            Constraint: `ndim` >= 1.

        (`errno` 2)
            On entry, `np1` = *<value>* and `np2` = *<value>*.

            Constraint: `np1` > `np2`.

        (`errno` 2)
            On entry, `np2` = *<value>*.

            Constraint: `np2` >= 2.

        (`errno` 2)
            On entry, `np1` = *<value>*.

            Constraint: `np1` >= 5.

        (`errno` 3)
            On entry, `np1`*`np2` exceeds largest machine integer. `np1`
            = *<value>* and `np2` = *<value>*.

        (`errno` 4)
            On entry, `np1` = *<value>*.

            Constraint: `np1` must be a prime number.

        (`errno` 5)
            On entry, `np2` = *<value>*.

            Constraint: `np2` must be a prime number.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 6)
            The machine precision is insufficient to perform the
            computation exactly. Try reducing `np1` or `np2`: `np1` =
            *<value>* and `np2` = *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.md_numth_coeff_2prime'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d01gzft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ndim.
        utils._BYREF_INT, # Input, np1.
        utils._BYREF_INT, # Input, np2.
        utils._EngineFloat64ArrayType.c_type, # Output, vk[ndim].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ndim = utils._EngineIntScalarType(
        ndim, {'fun_name': fun_name, 'entity': 'ndim'},
    ).to_ctypes()
    _np1 = utils._EngineIntScalarType(
        np1, {'fun_name': fun_name, 'entity': 'np1'},
    ).to_ctypes()
    _np2 = utils._EngineIntScalarType(
        np2, {'fun_name': fun_name, 'entity': 'np2'},
    ).to_ctypes()
    _ndim_py = utils._EngineIntScalarType.to_py(_ndim)
    _vk_cls = utils._EngineFloat64ArrayType(
        vk, {'fun_name': fun_name, 'entity': 'vk'},
        exp_shape=(_ndim_py,),
        check_contents=False,
    )
    _vk = _vk_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ndim,
        _np1,
        _np2,
        _vk,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ndim = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ndim >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, np1 = {:s} and np2 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: np1 > np2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, np2 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: np2 >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, np1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: np1 >= 5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, np1*np2 exceeds largest machine integer.',
                    'np1 = {:s} and np2 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, np1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: np1 must be a prime number.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, np2 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: np2 must be a prime number.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99992:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The machine precision is insufficient to perform the computation',
                    'exactly. Try reducing np1 or np2:',
                    'np1 = {:s} and np2 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _vk_cls.refresh_py(_vk)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def md_sphere_bad(f, ndim, radius, epsa, epsr, method, icoord, data=None):
    u"""
    Multidimensional quadrature over an n-sphere, allowing for badly
    behaved integrands.

    For full information please refer to the NAG Library document for
    d01ja

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01jaf.html

    Parameters
    ----------
    f : callable retval = f(ndim, x, data=None)
        `f` must return the value of the integrand f at a given point.

        Parameters
        ~~~~~~~~~~
        ndim : int
            n, the number of dimensions of the integral.

        x : float, ndarray, shape (`ndim`)
            The coordinates of the point at which the integrand f must
            be evaluated. These coordinates are given in Cartesian or
            spherical polar form according to the value of `icoord`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of f(x) evaluated at `x`.

    ndim : int
        n, the dimension of the sphere.

    radius : float
        alpha, the radius of the sphere.

    epsa : float
        The requested absolute tolerance. If `epsa` < 0.0, its absolute
        value is used. See [Accuracy].

    epsr : float
        The requested relative tolerance.

        `epsr` < 0.0

          Its absolute value is used.

        `epsr` < 10*(machine precision)

          The latter value is used as `epsr` by the function. See
          [Accuracy].

    method : int
        Must specify the transformation to be used by the function. The
        choice depends on the behaviour of the integrand and on the
        required accuracy.

        For well-behaved functions and functions with mild singularities
        on the surface of the sphere only:

        `method` = 1

          Low accuracy required.

        `method` = 2

          High accuracy required.

        For functions with severe singularities on the surface of the
        sphere only:

        `method` = 3

          Low accuracy required.

        `method` = 4

          High accuracy required.

          (in this case `icoord` must be set to `icoord` = 2, and the
          function defined in special spherical coordinates).

        For functions with a singularity at the centre of the sphere
        (and possibly with singularities on the surface as well):

        `method` = 5

          Low accuracy required.

        `method` = 6

          High accuracy required.

        `method` = 0 can be used as a default value and is equivalent
        to:

        - `method` = 1 if `epsr` > 10^(-6), and

        - `method` = 2 if `epsr` <= 10^(-6).

        The distinction between low and high required accuracies, as
        mentioned above, depends also on the behaviour of the function.

        Roughly one may assume the critical value of `epsa` and `epsr`
        to be 10^(-6), but the critical value will be smaller for a
        well-behaved integrand and larger for an integrand with severe
        singularities.

    icoord : int
        Must specify which kind of coordinates are used in `f`.

        `icoord` = 0

          Cartesian coordinates x_i, for i = 1,2,...,n.

        `icoord` = 1

          Spherical coordinates (see [Further Comments]): `x`[0] = rho;
          `x`[i-1] = theta_{i-1}, for i = 2,3,...,n.

        `icoord` = 2,

          Special spherical polar coordinates (see [Further Comments]),
          with the additional transformation rho = alpha-lambda: `x`[0]
          = lambda = alpha-rho; `x`[i-1] = theta_{i-1}, for i =
          2,3,...,n.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    result : float
        The approximation to the integral I.

    esterr : float
        An estimate of the modulus of the absolute error.

    nevals : int
        The number of function evaluations used.

    Raises
    ------
    NagValueError
        (`errno` 4)
            On entry, `radius` = *<value>*.

            Constraint: `radius` >= 0.0.

        (`errno` 4)
            On entry, `method` = 4 and `icoord` = *<value>*.

            Constraint: when `method` = 4, `icoord` = 2.

        (`errno` 4)
            On entry, `method` = 3 and `icoord` = *<value>*.

            Constraint: when `method` = 3, `icoord` = 2.

        (`errno` 4)
            On entry, `icoord` = 2 and `method` = *<value>*.

            Constraint: when `icoord` = 2, `method` = 3 or 4.

        (`errno` 4)
            On entry, `ndim` = *<value>*.

            Constraint: `ndim` >= 2.

        (`errno` 4)
            On entry, `ndim` = *<value>*.

            Constraint: `ndim` <= 4.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The required accuracy could not be achieved.

        (`errno` 2)
            The required accuracy could not be achieved.

        (`errno` 3)
            The required accuracy could not be achieved.

            If `method` = 0, 1 or 2, setting `method` = 3 or 4 may help.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.md_sphere_bad'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ndim.
        utils._EngineFloat64ArrayType.c_type, # Input, x[ndim].
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, _fsub, en_data, ndim, x, retval, _iuser, _ruser):
        """Helper for f."""
        try:
            _ndim_fh = utils._EngineIntScalarType.to_py(ndim.contents)
            _x_fh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_ndim_fh,),
            )
            if data is not None:
                _retval_fh = f(
                    _ndim_fh,
                    _x_fh,
                    data,
                )
            else:
                _retval_fh = f(
                    _ndim_fh,
                    _x_fh,
                )
            utils._unlink_pointer(x)
            utils._EngineFloat64ScalarType(
                _retval_fh, {'fun_name': cb_f_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d01jaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._BYREF_INT, # Input, ndim.
        utils._BYREF_FLOAT64, # Input, radius.
        utils._BYREF_FLOAT64, # Input, epsa.
        utils._BYREF_FLOAT64, # Input, epsr.
        utils._BYREF_INT, # Input, method.
        utils._BYREF_INT, # Input, icoord.
        utils._BYREF_FLOAT64, # Output, result.
        utils._BYREF_FLOAT64, # Output, esterr.
        utils._BYREF_INT, # Output, nevals.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fsub = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _ndim = utils._EngineIntScalarType(
        ndim, {'fun_name': fun_name, 'entity': 'ndim'},
    ).to_ctypes()
    _radius = utils._EngineFloat64ScalarType(
        radius, {'fun_name': fun_name, 'entity': 'radius'},
    ).to_ctypes()
    _epsa = utils._EngineFloat64ScalarType(
        epsa, {'fun_name': fun_name, 'entity': 'epsa'},
    ).to_ctypes()
    _epsr = utils._EngineFloat64ScalarType(
        epsr, {'fun_name': fun_name, 'entity': 'epsr'},
    ).to_ctypes()
    _method = utils._EngineIntScalarType(
        method, {'fun_name': fun_name, 'entity': 'method'},
    ).to_ctypes()
    _icoord = utils._EngineIntScalarType(
        icoord, {'fun_name': fun_name, 'entity': 'icoord'},
    ).to_ctypes()
    _result = utils._EngineFloat64ScalarType.empty_ctypes()
    _esterr = utils._EngineFloat64ScalarType.empty_ctypes()
    _nevals = utils._EngineIntScalarType.empty_ctypes()
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _f,
        _fsub,
        _fh,
        _ndim,
        _radius,
        _epsa,
        _epsr,
        _method,
        _icoord,
        _result,
        _esterr,
        _nevals,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The required accuracy could not be achieved.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99988:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The required accuracy could not be achieved.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99989:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The required accuracy could not be achieved.',
                    'If method = 0, 1 or 2, setting method = 3 or 4 may help.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, radius = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: radius >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, method = 4 and icoord = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: when method = 4, icoord = 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, method = 3 and icoord = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: when method = 3, icoord = 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, icoord = 2 and method = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: when icoord = 2, method = 3 or 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ndim = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ndim >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ndim = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ndim <= 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _result_py = utils._EngineFloat64ScalarType.to_py(_result)
        _esterr_py = utils._EngineFloat64ScalarType.to_py(_esterr)
        _nevals_py = utils._EngineIntScalarType.to_py(_nevals)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['result', 'esterr', 'nevals'],
        )(
            _result_py,
            _esterr_py,
            _nevals_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def md_simplex(ndim, vert, f, minord, maxord, finvls, data=None):
    u"""
    Multidimensional quadrature over an n-simplex.

    For full information please refer to the NAG Library document for
    d01pa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01paf.html

    Parameters
    ----------
    ndim : int
        n, the number of dimensions of the integral.

    vert : float, shape (`ndim`+1, 2*(`ndim`+1)), modified in place
        On entry: `vert`[i-1,j-1] must be set to the jth component of
        the ith vertex for the simplex integration region, for j =
        1,2,...,n, for i = 1,2,...,n+1. If `minord` > 0, `vert` must be
        unchanged since the previous call of ``md_simplex``.

        On exit: these values are unchanged. The rest of the array
        `vert` is used for workspace and contains information to be used
        if another call of ``md_simplex`` is made with `minord` > 0. In
        particular `vert`[n,2n+1] contains the volume of the simplex.

    f : callable retval = f(ndim, x, data=None)
        `f` must return the value of the integrand at a given point.

        Parameters
        ~~~~~~~~~~
        ndim : int
            n, the number of dimensions of the integral.

        x : float, ndarray, shape (`ndim`)
            The coordinates of the point at which the integrand f must
            be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of the integrand f at the given point.

    minord : int
        Must specify the highest order of the approximations currently
        available in the array `finvls`.

        `minord` = 0

          Indicates an initial call.

        `minord` > 0

          Indicates that
          `finvls`[0],`finvls`[1],...,`finvls`[`minord`-1] have already
          been computed in a previous call of ``md_simplex``.

    maxord : int
        The highest order of approximation to the integral to be
        computed.

    finvls : float, shape (`maxord`), modified in place
        On entry: if `minord` > 0,
        `finvls`[0],`finvls`[1],...,`finvls`[`minord`-1] must contain
        approximations to the integral previously computed by
        ``md_simplex``.

        On exit: contains these values unchanged, and the newly computed
        values
        `finvls`[`minord`],`finvls`[`minord`+1],...,`finvls`[`maxord`-1].
        `finvls`[j-1] is an approximation to the integral of polynomial
        degree 2j-1.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    minord : int
        `minord` = `maxord`.

    esterr : float
        An absolute error estimate for `finvls`[`maxord`-1].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `maxord` = *<value>* and `minord` = *<value>*.

            Constraint: `maxord` > `minord`.

        (`errno` 1)
            On entry, `minord` = *<value>*.

            Constraint: `minord` >= 0.

        (`errno` 1)
            On entry, `ndim` = *<value>*.

            Constraint: `ndim` >= 2.

        (`errno` 2)
            The volume of the simplex integration region is too large or
            too small to be represented on the machine.
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.md_simplex'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ndim.
        utils._EngineFloat64ArrayType.c_type, # Input, x[ndim].
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, _fsub, en_data, ndim, x, retval, _iuser, _ruser):
        """Helper for f."""
        try:
            _ndim_fh = utils._EngineIntScalarType.to_py(ndim.contents)
            _x_fh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_ndim_fh,),
            )
            if data is not None:
                _retval_fh = f(
                    _ndim_fh,
                    _x_fh,
                    data,
                )
            else:
                _retval_fh = f(
                    _ndim_fh,
                    _x_fh,
                )
            utils._unlink_pointer(x)
            utils._EngineFloat64ScalarType(
                _retval_fh, {'fun_name': cb_f_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d01paft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ndim.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, vert[ldvert, sdvert].
        utils._BYREF_INT, # Input, ldvert.
        utils._BYREF_INT, # Input, sdvert.
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._BYREF_INT, # Input/Output, minord.
        utils._BYREF_INT, # Input, maxord.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, finvls[maxord].
        utils._BYREF_FLOAT64, # Output, esterr.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((vert, 'vert'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _ndim = utils._EngineIntScalarType(
        ndim, {'fun_name': fun_name, 'entity': 'ndim'},
    ).to_ctypes()
    _ldvert_py = utils._EngineFloat64ArrayType.get_nd(vert, {'fun_name': fun_name, 'entity': 'vert'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldvert = utils._EngineIntScalarType(
        _ldvert_py, {'fun_name': fun_name, 'entity': '_ldvert'},
    ).to_ctypes()
    _sdvert_py = utils._EngineFloat64ArrayType.get_nd(vert, {'fun_name': fun_name, 'entity': 'vert'}, exp_rank=2, sorder=_m_sorder, dim_type='s')
    _sdvert = utils._EngineIntScalarType(
        _sdvert_py, {'fun_name': fun_name, 'entity': '_sdvert'},
    ).to_ctypes()
    _ndim_py = utils._EngineIntScalarType.to_py(_ndim)
    _vert_cls = utils._EngineFloat64ArrayType(
        vert, {'fun_name': fun_name, 'entity': 'vert'},
        exp_shape=(_ndim_py+1, 2*(_ndim_py+1)),
        mutual_sorder=_m_sorder,
    )
    _vert = _vert_cls.to_ctypes()
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fsub = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _minord = utils._EngineIntScalarType(
        minord, {'fun_name': fun_name, 'entity': 'minord'},
    ).to_ctypes()
    _maxord = utils._EngineIntScalarType(
        maxord, {'fun_name': fun_name, 'entity': 'maxord'},
    ).to_ctypes()
    _maxord_py = utils._EngineIntScalarType.to_py(_maxord)
    _finvls_cls = utils._EngineFloat64ArrayType(
        finvls, {'fun_name': fun_name, 'entity': 'finvls'},
        exp_shape=(_maxord_py,),
    )
    _finvls = _finvls_cls.to_ctypes()
    _esterr = utils._EngineFloat64ScalarType.empty_ctypes()
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ndim,
        _vert,
        _ldvert,
        _sdvert,
        _f,
        _fsub,
        _fh,
        _minord,
        _maxord,
        _finvls,
        _esterr,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, maxord = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'minord = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: maxord > minord.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, minord = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: minord >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ndim = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ndim >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The volume of the simplex integration region is too',
                    'large or too small to be represented on the machine.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _vert_cls.refresh_py(_vert)
        _minord_py = utils._EngineIntScalarType.to_py(_minord)
        _finvls_cls.refresh_py(_finvls)
        _esterr_py = utils._EngineFloat64ScalarType.to_py(_esterr)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['minord', 'esterr'],
        )(
            _minord_py,
            _esterr_py,
        )
    _err_class.manage()
    return return_data

def dim1_gen_vec_multi_rcomm(irevcm, ni, a, b, needi, x, lenx, nx, fm, dinest, errest, comm, nagcal):
    u"""
    One-dimensional quadrature, adaptive, finite interval, multiple
    integrands, vectorized abscissae, reverse communication.

    For full information please refer to the NAG Library document for
    d01ra

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01raf.html

    Parameters
    ----------
    irevcm : int
        On initial entry: `irevcm` = 1.

        `irevcm` = 1

          Sets up data structures in `comm`['icom'] and `comm`['com']
          and starts a new integration.

        On intermediate entry: `irevcm` should normally be left
        unchanged. However, if `irevcm` is set to a negative value,
        ``dim1_gen_vec_multi_rcomm`` will terminate, (see `irevcm` = 11
        and `irevcm` = 12 above).

    ni : int
        n_i, the number of integrands.

    a : float
        a, the lower bound of the domain.

    b : float
        b, the upper bound of the domain.

        If |b-a| < 10 epsilon, where epsilon is the machine precision
        (see ``machine.precision``), ``dim1_gen_vec_multi_rcomm`` will
        return `dinest`[j-1] = `errest`[j-1] = 0.0, for j = 1,2,...,n_i.

    needi : int, shape (`ni`), modified in place
        On initial entry: need not be set.

        On intermediate exit: `needi`[j-1] indicates what action must be
        taken for integral j = 1,2,...n_i (see `irevcm`).

        `needi`[j-1] = 0

          Do not provide f_j(x_i). Any provided values will be ignored.

        `needi`[j-1] = 1

          The values f_j(x_i) must be provided in `fm`[j-1,i-1], for i =
          1,2,...,n_x.

        `needi`[j-1] = 2

          The values f_j(x_i) are not required, however the error
          estimate for integral j is still above the requested
          tolerance. If you wish to provide values for the evaluation of
          integral j, set `needi`[j-1] = 1, and supply f_j(x_i) in
          `fm`[j-1,i-1], for i = 1,2,...,n_x.

        `needi`[j-1] = 3

          The error estimate for integral j cannot be improved to below
          the requested tolerance directly, either because no more new
          splits may be performed due to exhaustion, or due to the
          detection of extremely bad integrand behaviour. However,
          providing the values f_j(x_i) may still lead to some
          improvement, and may lead to an acceptable error estimate
          indirectly using Wynn's epsilon algorithm. If you wish to
          provide values for the evaluation of integral j, set
          `needi`[j-1] = 1, and supply f_j(x_i) in `fm`[j-1,i-1], for i
          = 1,2,...,n_x.

        `needi`[j-1] = 4

          The error estimate of integral j is below the requested
          tolerance. If you believe this to be false, if for example the
          result in `dinest`[j-1] is greatly different to what you may
          expect, you may force the algorithm to re-evaluate this
          conclusion by including the evaluations of integrand j at x_i,
          for i = 1,2,...,n_x, and setting `needi`[j-1] = 1. Integral
          and error estimation will be performed again during the next
          iteration.

        On intermediate entry: `needi`[j-1] may be used to indicate what
        action you have taken for integral j.

        `needi`[j-1] = 1

          You have provided the values f_j(x_i) in `fm`[j-1,i-1], for i
          = 1,2,...,n_x.

        `needi`[j-1] < 0

          You are abandoning the evaluation of integral j. The current
          values of `dinest`[j-1] and `errest`[j-1] will be returned on
          final completion.

        Otherwise you have not provided the value f_j(x_i).

        On final exit: `needi`[j-1] indicates the final state of
        integral j.

        `needi`[j-1] = 0

          The error estimate for F_j is below the requested tolerance.

        `needi`[j-1] = 1

          The error estimate for F_j is below the requested tolerance
          after extrapolation.

        `needi`[j-1] = 2

          The error estimate for F_j is above the requested tolerance.

        `needi`[j-1] = 3

          The error estimate for F_j is above the requested tolerance,
          and extremely bad behaviour of integral j has been detected.

        `needi`[j-1] < 0

          You prohibited further evaluation of integral j.

    x : float, shape (`lenx`), modified in place
        On initial entry: if 'Primary Division Mode' = 'AUTOMATIC', `x`
        need not be set. This is the default behaviour.

        If 'Primary Division Mode' = 'MANUAL', `x` is used to supply a
        set of initial 'break-points' inside the domain of integration.

        Specifically, `x`[i-1] must contain a break-point x_i^0, for i =
        1,2,...,(s_pri-1), where s_pri is the number of 'Primary
        Divisions'.

        On intermediate exit: `x`[i-1] is the abscissa x_i, for i =
        1,2,...,n_x, at which the appropriate integrals must be
        evaluated.

    lenx : int
        Currently `lenx` = max(122, s_pri-1) will be sufficient for all
        cases.

    nx : int
        On initial entry: need not be set.

        On intermediate entry: must not be changed.

    fm : float, shape (ldfmrq, sdfmrq)
        On initial entry: need not be set.

        On intermediate entry: if indicated by `needi`[j-1] you must
        supply the values f_j(x_i) in `fm`[j-1,i-1], for j =
        1,2,...,n_i, for i = 1,2,...,n_x.

    dinest : float, shape (`ni`), modified in place
        On initial entry: need not be set.

        On intermediate entry: must not be altered.

        On exit: contains the current estimates of the `ni` integrals.
        If `irevcm` = 0, this will be the final solution.

    errest : float, shape (`ni`), modified in place
        On initial entry: need not be set.

        On intermediate entry: must not be altered.

        On exit: contains the current error estimates for the `ni`
        integrals. If `irevcm` = 0, `errest` contains the final error
        estimates of the `ni` integrals.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``opt_set``.

    nagcal : int
        Indicates what level of printing is required.

        itrace = 0

          No printing will be displayed.

        itrace = 1

          Upon completion, any intervals where bad behaviour was
          detected will be printed, as well as the estimate of the
          definite integral and error for any integrals which behaved
          badly over this interval.

    Returns
    -------
    irevcm : int
        On intermediate exit: `irevcm` = 11 or 12.

        `irevcm` requests the integrands f_j(x_i) be evaluated for all
        required j in 1,...,n_i as indicated by `needi`, and at all the
        points x_i, for i = 1,2,...,n_x.

        Abscissae x_i are provided in `x`[i-1] and f_j(x_i) must be
        returned in `fm`[j-1,i-1].

        During the initial solve phase:

        `irevcm` = 11

          Function values are required to construct the initial
          estimates of the definite integrals.

          If `needi`[j-1] = 1, f_j(x_i) must be supplied in
          `fm`[j-1,i-1].

          This will be the case unless you have abandoned the evaluation
          of specific integrals on a previous call.

          If `needi`[j-1] = 0, you have previously abandoned the
          evaluation of integral j, and hence should not supply the
          value of f_j(x_i).

          `dinest` and `errest` contain incomplete information during
          this phase.

          As such you should not abandon the evaluation of any integrals
          during this phase unless you do not require their estimate.

          If `irevcm` is set to a negative value during this phase,
          `needi`[j-1], for j = 1,2,...,n_i, will be set to this
          negative value and `errno` = -1 will be returned.

        During the adaptive solve phase:

        `irevcm` = 12

          Function values are required to improve the estimates of the
          definite integrals.

          If `needi`[j-1] = 0, any evaluation of f_j(x_i) will be
          discarded, so there is no need to provide them.

          If `needi`[j-1] = 1, f_j(x_i) must be provided in
          `fm`[j-1,i-1].

          If `needi`[j-1] = 2, 3 or 4, the current error estimate of
          integral j does not require integrand j to be evaluated and
          provided in `fm`[j-1,i-1].

          Should you choose to, integrand j can be evaluated in which
          case `needi`[j-1] must be set to 1.

          `dinest` and `errest` contain complete information during this
          phase.

          If `irevcm` is set to a negative value during this phase
          `errno` = 1, 2 or 3 will be returned and the elements of
          `needi` will reflect the current state of the adaptive
          process.

        On final exit: `irevcm` = 0.

        `irevcm` = 0

          Indicates the algorithm has completed.

    sid : int
        For advanced users.

        On intermediate exit: this argument identifies a specific set of
        abscissae, x, returned during the integration process. When a
        new set of abscissae are generated the value of this argument is
        incremented by 1. Advanced users may store calculations required
        for an identified set x, and reuse them should
        ``dim1_gen_vec_multi_rcomm`` return the same value of this
        argument, i.e., the same set of abscissae was used.

    nx : int
        On intermediate exit: n_x, the number of abscissae at which
        integrands are required.

    Other Parameters
    ----------------
    'Absolute Interval Minimum' : float
        Default = 128.0 epsilon

        r = delta_a, the absolute lower limit for a segment to be
        considered for subdivision.
        See also 'Relative Interval Minimum' and [Further Comments].

        Constraint: r >= 128 epsilon.

    'Absolute Tolerance' : float
        Default = 1024 epsilon

        r = epsilon_a, the absolute tolerance required.
        See also 'Relative Tolerance' and [Description].

        Constraint: r >= 0.0.

    'Extrapolation' : str
        Default = ON

        Activate or deactivate the use of the epsilon algorithm (Wynn).
        'Extrapolation' often reduces the number of iterations required
        to achieve the desired solution, but it can occasionally lead to
        premature convergence towards an incorrect answer.

        ON
          Use extrapolation.

        OFF
          Disable extrapolation.

    'Extrapolation Safeguard' : float
        Default = 1.0e-12

        r = epsilon_safe.
        If epsilon_q is the estimated error from the quadrature
        evaluation alone, and epsilon_ex is the error estimate
        determined using extrapolation, then the extrapolated solution
        will only be accepted if epsilon_safeepsilon_q <= epsilon_ex.

    'Maximum Subdivisions' : int
        Default = 50

        i = max_sdiv, the maximum number of subdivisions the algorithm
        may use in the adaptive phase, forming at most an additional
        (2*max_sdiv) segments.

    'Primary Divisions' : int
        Default = 1

        i = s_pri, the number of initial segments of the domain [a,b].
        By default the initial segment is the entire domain.

        Constraint: 0 < i < 1000000.

    'Primary Division Mode' : str
        Default = 'AUTOMATIC'

        Determines how the initial set of s_pri segments will be
        generated.

        AUTOMATIC
          ``dim1_gen_vec_multi_rcomm`` will automatically generate s_pri
          segments of equal size covering the interval [a,b].

        MANUAL
          ``dim1_gen_vec_multi_rcomm`` will use the break-points x_i^0,
          for i = 1,2,...,s_pri-1, supplied in `x` on initial entry to
          generate the initial segments covering [a,b]. These may be
          supplied in any order, however it will be more efficient to
          supply them in ascending (or descending if a > b) order.
          Repeated break-points are allowed, although this will generate
          fewer initial segments.

        Note: an absolute bound on the size of an initial segment of
        10.0 epsilon is automatically applied in all cases, and will
        result in fewer initial subdivisions being generated if
        automatically generated or supplied break-points result in
        segments smaller than this.

    'Prioritize Error' : str
        Default = 'LEVEL'

        Indicates how new subdivisions of segments sustaining
        unacceptable local errors for integrals should be prioritized.

        LEVEL
          Segments with lower level with unsatisfactory error estimates
          will be chosen over segments with greater error on higher
          levels. This will probably lead to more integrals being
          improved in earlier iterations of the algorithm, and hence
          will probably lead to fewer repeated returns (see argument
          `sid`), and to more integrals being satisfactorily estimated
          if computational exhaustion occurs.

        MAXERR
          The segment with the worst overall error will be split,
          regardless of level. This will more rapidly improve the worst
          integral estimates, although it will probably result in the
          fewest integrals being improved in earlier iterations, and may
          hence lead to more repeated returns (see argument `sid`), and
          potentially fewer integrals satisfying the requested
          tolerances if computational exhaustion occurs.

    'Quadrature Rule' : str
        Default = GK15

        The basic quadrature rule to be used during the integration.
        Currently 6 Gauss--Kronrod rules are available, all identifiable
        by the letters GK followed by the number of points required by
        the Kronrod rule.
        Higher order rules generally provide higher accuracy with fewer
        subdivisons.
        However, for integrands with sharp singularities, lower order
        rules may be more efficient, particularly if the integrand away
        from the singularity is well behaved.
        With higher order rules, you may need to increase the 'Absolute
        Interval Minimum' and the 'Relative Interval Minimum' to
        maintain numerical difference between the abscissae and the
        segment bounds.

        GK15
          The Gauss--Kronrod rule based on 7 Gauss points and 15 Kronrod
          points.

        GK21
          The Gauss--Kronrod rule based on 10 Gauss points and 21
          Kronrod points. This is the rule used by ``dim1_fin_bad_vec``

        GK31
          The Gauss--Kronrod rule based on 15 Gauss points and 31
          Kronrod points.

        GK41
          The Gauss--Kronrod rule based on 20 Gauss points and 41
          Kronrod points.

        GK51
          The Gauss--Kronrod rule based on 25 Gauss points and 51
          Kronrod points.

        GK61
          The Gauss--Kronrod rule based on 30 Gauss points and 61
          Kronrod points. This is the highest order rule, most suitable
          for highly oscilliatory integrals.

    'Relative Interval Minimum' : float
        Default = 1.0e-6

        r = delta_r, the relative factor in the lower limit,
        delta_r|b-a|, for a segment to be considered for subdivision.
        See also 'Absolute Interval Minimum' and [Further Comments].

        Constraint: r >= 0.0.

    'Relative Tolerance' : float
        Default = sqrt(epsilon)

        r = epsilon_r, the required relative tolerance.
        See also 'Absolute Tolerance' and [Description].

        Constraint: r >= 0.0.

        Note: setting both epsilon_r = epsilon_a = 0.0 is possible,
        although it will most likely result in an excessive amount of
        computational effort.

    'Index LDI' : int
        query only

        I_ldi, the index of `icom` required for obtaining ldi.
        See [Details of the Computation].

    'Index LDR' : int
        query only

        I_ldr, the index of `icom` required for obtaining ldr.
        See [Details of the Computation].

    'Index NSDIV' : int
        query only

        I_nsdiv, the index of `icom` required for obtaining nsdiv.
        See [Details of the Computation].

    'Index NSEG' : int
        query only

        I_nseg, the index of `icom` required for obtaining nseg.
        See [Details of the Computation].

    'Index FCP' : int
        query only

        I_fcp, the index of `icom` required for obtaining fcp.
        See [Details of the Computation].

    'Index EVALSP' : int
        query only

        I_evalsp, the index of `icom` required for obtaining evalsp.
        See [Details of the Computation].

    'Index DSP' : int
        query only

        I_dsp, the index of `icom` required for obtaining dsp.
        See [Details of the Computation].

    'Index ESP' : int
        query only

        I_esp, the index of `icom` required for obtaining esp.
        See [Details of the Computation].

    'Index SINFOIP' : int
        query only

        I_sinfoip, the index of `icom` required for obtaining sinfoip.
        See [Details of the Computation].

    'Index SINFORP' : int
        query only

        I_sinforp, the index of `icom` required for obtaining sinforp.
        See [Details of the Computation].

    Raises
    ------
    NagValueError
        (`errno` 11)
            `irevcm` had an illegal value.

            On entry, `irevcm` = *<value>*.

        (`errno` 21)
            On entry, `ni` = *<value>*.

            Constraint: `ni` >= 1.

        (`errno` 71)
            On entry, 'Primary Division Mode' = 'MANUAL' and at least
            one supplied break-point in `x` is outside of the domain of
            integration.

        (`errno` 81)
            `lenx` is insufficient for the chosen options.

            On entry, `lenx` = *<value>*.

            Constraint: `lenx` >= *<value>*.

        (`errno` 1001)
            Either the option arrays `comm`['iopts'] and `comm`['opts']
            have not been initialized for ``dim1_gen_vec_multi_rcomm``,
            or they have become corrupted.

        (`errno` 1101)
            On entry, one of `comm`['icom'] and `comm`['com'] has become
            corrupted.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` -1)
            Evaluation of all integrals has been stopped during the
            initial phase.

        (`errno` 1)
            At least one error estimate exceeded the requested
            tolerances.

        (`errno` 2)
            Extremely bad behaviour was detected for at least one
            integral.

        (`errno` 3)
            Extremely bad behaviour was detected for at least one
            integral. At least one other integral error estimate was
            above the requested tolerance.
    """

    from ..base import _nag_d_consts
    from ..base import _nag_d_consts_engine

    excs = []
    fun_name = 'naginterfaces.base.quad.dim1_gen_vec_multi_rcomm'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d01raft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input/Output, irevcm.
        utils._BYREF_INT, # Input, ni.
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_INT, # Output, sid.
        utils._EngineIntArrayType.c_type, # Input/Output, needi[ni].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[lenx].
        utils._BYREF_INT, # Input, lenx.
        utils._BYREF_INT, # Input/Output, nx.
        utils._EngineFloat64ArrayType.c_type, # Input, fm[ldfm, *].
        utils._BYREF_INT, # Input, ldfm.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, dinest[ni].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, errest[ni].
        utils._EngineIntArrayType.c_type, # Input, iopts[100].
        utils._EngineFloat64ArrayType.c_type, # Input, opts[100].
        utils._BYREF_INT, # Input, nagcal.
        utils._EngineIntArrayType.c_type, # Input/Output, icom[licom].
        utils._BYREF_INT, # Input, licom.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, com[lcom].
        utils._BYREF_INT, # Input, lcom.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((fm, 'fm'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _irevcm = utils._EngineIntScalarType(
        irevcm, {'fun_name': fun_name, 'entity': 'irevcm'},
    ).to_ctypes()
    _ni = utils._EngineIntScalarType(
        ni, {'fun_name': fun_name, 'entity': 'ni'},
    ).to_ctypes()
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _sid = utils._EngineIntScalarType.empty_ctypes()
    _ni_py = utils._EngineIntScalarType.to_py(_ni)
    _needi_cls = utils._EngineIntArrayType(
        needi, {'fun_name': fun_name, 'entity': 'needi'},
        exp_shape=(_ni_py,),
    )
    _needi = _needi_cls.to_ctypes()
    _lenx = utils._EngineIntScalarType(
        lenx, {'fun_name': fun_name, 'entity': 'lenx'},
    ).to_ctypes()
    _lenx_py = utils._EngineIntScalarType.to_py(_lenx)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_lenx_py,),
    )
    _x = _x_cls.to_ctypes()
    _nx = utils._EngineIntScalarType(
        nx, {'fun_name': fun_name, 'entity': 'nx'},
    ).to_ctypes()
    _ldfm_py = utils._EngineFloat64ArrayType.get_nd(fm, {'fun_name': fun_name, 'entity': 'fm'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldfm = utils._EngineIntScalarType(
        _ldfm_py, {'fun_name': fun_name, 'entity': '_ldfm'},
    ).to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('icom', 'com'),
        req_keys=('iopts', 'opts'),
    )
    if utils._EngineIntArrayType.get_element(comm['iopts'], {'fun_name': fun_name, 'entity': 'comm[\'iopts\']'}, (19,), validate=False) == _nag_d_consts_engine.D01RA_FLEAD_I:
        _vl_sdfmrq_py = utils._EngineIntArrayType.get_element(comm['iopts'], {'fun_name': fun_name, 'entity': 'comm[\'iopts\']'}, (38,), validate=False)
    else:
        _vl_sdfmrq_py = _ni_py
    if utils._EngineIntArrayType.get_element(comm['iopts'], {'fun_name': fun_name, 'entity': 'comm[\'iopts\']'}, (19,), validate=False) == _nag_d_consts_engine.D01RA_FLEAD_I:
        _vl_ldfmrq_py = _ni_py
    else:
        _vl_ldfmrq_py = utils._EngineIntArrayType.get_element(comm['iopts'], {'fun_name': fun_name, 'entity': 'comm[\'iopts\']'}, (38,), validate=False)
    _fm = utils._EngineFloat64ArrayType(
        fm, {'fun_name': fun_name, 'entity': 'fm'},
        exp_shape=(_vl_ldfmrq_py, _vl_sdfmrq_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _dinest_cls = utils._EngineFloat64ArrayType(
        dinest, {'fun_name': fun_name, 'entity': 'dinest'},
        exp_shape=(_ni_py,),
    )
    _dinest = _dinest_cls.to_ctypes()
    _errest_cls = utils._EngineFloat64ArrayType(
        errest, {'fun_name': fun_name, 'entity': 'errest'},
        exp_shape=(_ni_py,),
    )
    _errest = _errest_cls.to_ctypes()
    _iopts = comm['iopts']
    _opts = comm['opts']
    _nagcal = utils._EngineIntScalarType(
        nagcal, {'fun_name': fun_name, 'entity': 'nagcal'},
    ).to_ctypes()
    _vl_s_pri_py = utils._EngineIntArrayType.get_element(comm['iopts'], {'fun_name': fun_name, 'entity': 'comm[\'iopts\']'}, (23,), validate=False)
    if _comm_needs_init:
        _icom = utils._EngineIntArrayType.empty_ctypes(50+5*_ni_py+(_vl_s_pri_py+100)*(5+_ni_py))
        comm['icom'] = _icom
    else:
        _icom = comm['icom']
    _licom_py = utils._EngineIntArrayType.get_shape(comm['icom'], {'fun_name': fun_name, 'entity': 'comm[\'icom\']'}, exp_rank=1, validate=False)[0]
    _licom = utils._EngineIntScalarType(
        _licom_py, {'fun_name': fun_name, 'entity': '_licom'},
    ).to_ctypes()
    if _comm_needs_init:
        _com = utils._EngineFloat64ArrayType.empty_ctypes(94+9*_ni_py+int(ceil((_ni_py//2)))+(_vl_s_pri_py+100)*(2+int(ceil((_ni_py//2)))+2*_ni_py))
        comm['com'] = _com
    else:
        _com = comm['com']
    _lcom_py = utils._EngineFloat64ArrayType.get_shape(comm['com'], {'fun_name': fun_name, 'entity': 'comm[\'com\']'}, exp_rank=1, validate=False)[0]
    _lcom = utils._EngineIntScalarType(
        _lcom_py, {'fun_name': fun_name, 'entity': '_lcom'},
    ).to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _irevcm,
        _ni,
        _a,
        _b,
        _sid,
        _needi,
        _x,
        _lenx,
        _nx,
        _fm,
        _ldfm,
        _dinest,
        _errest,
        _iopts,
        _opts,
        _nagcal,
        _icom,
        _licom,
        _com,
        _lcom,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _irevcm_py = utils._EngineIntScalarType.to_py(_irevcm)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _irevcm_py != 0 or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == _nag_d_consts.D01_IERR_USER_EXIT_INT and _ifmt == _nag_d_consts.D01_IFMT_USER_EXIT:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Evaluation of all integrals has been stopped during the initial phase.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01RA_IERR_NOTSAT and _ifmt == _nag_d_consts.D01RA_IFMT_NOTSAT:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'At least one error estimate exceeded the requested tolerances.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01RA_IERR_VBAD and _ifmt == _nag_d_consts.D01RA_IFMT_VBAD:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Extremely bad behaviour was detected for at least one integral.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01RA_IERR_VBAD_NOTSAT and _ifmt == _nag_d_consts.D01RA_IFMT_VBAD_NOTSAT:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Extremely bad behaviour was detected for at least one integral.',
                    'At least one other integral error estimate was above the',
                    'requested tolerance.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01RA_IERR_IREVCM and _ifmt == _nag_d_consts.D01RA_IFMT_IREVCM:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'irevcm had an illegal value.',
                    'On entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01RA_IERR_NI and _ifmt == _nag_d_consts.D01RA_IFMT_NI:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ni = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ni >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01RA_IERR_BNDS and _ifmt == _nag_d_consts.D01RA_IFMT_BNDS:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, \'Primary Division Mode\' = \'MANUAL\'',
                    'and at least one supplied break-point in x is outside',
                    'of the domain of integration.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01RA_IERR_LENX and _ifmt == _nag_d_consts.D01RA_IFMT_LENX:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'lenx is insufficient for the chosen options.',
                    'On entry, lenx = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lenx >= {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01_IERR_OPTS_CORRUPT and _ifmt == _nag_d_consts.D01_IFMT_OPTS_CORRUPT:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the option arrays comm[\'iopts\'] and comm[\'opts\'] have not been',
                    'initialized for dim1_gen_vec_multi_rcomm, or they have become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01_IERR_COM_CORRUPT and _ifmt == _nag_d_consts.D01_IFMT_COM_CORRUPT:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, one of comm[\'icom\'] and comm[\'com\'] has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _needi_cls.refresh_py(_needi)
        _x_cls.refresh_py(_x)
        _nx_py = utils._EngineIntScalarType.to_py(_nx)
        _dinest_cls.refresh_py(_dinest)
        _errest_cls.refresh_py(_errest)
        _sid_py = utils._EngineIntScalarType.to_py(_sid)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['irevcm', 'sid', 'nx'],
        )(
            _irevcm_py,
            _sid_py,
            _nx_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

@utils._withdrawal(
    'naginterfaces.base.quad.withdraw_1d_gen_vec_multi_diagnostic',
    mkwdrn='28',
)
def withdraw_1d_gen_vec_multi_diagnostic(monit, ni, dinest, errest, comm, data=None):
    u"""
    Diagnostic function for ``dim1_gen_vec_multi_rcomm``.

    Note: ``withdraw_1d_gen_vec_multi_diagnostic`` will be removed in
    naginterfaces 28.0.0.
    There is no replacement for this routine.

    For full information please refer to the NAG Library document for
    d01rb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01rbf.html

    Parameters
    ----------
    monit : None or callable monit(ni, ns, dinest, errest, fcount,
        sinfoi, evals, ldi, sinfor, fs, es, ldr, data=None)
        `monit` allows you to examine details of the adaptive process
        after a call of ``dim1_gen_vec_multi_rcomm`` returning irevcm =
        11 or 12.

        Additionally, after a call of ``dim1_gen_vec_multi_rcomm``
        returning irevcm = 0, `monit` may enhance the computed solution.

        Parameters
        ~~~~~~~~~~
        ni : int
            n_i, the number of integrands specified in
            ``dim1_gen_vec_multi_rcomm``.

        ns : int
            n_s, the number of segments formed during the adaptive
            procedure that can currently be examined. n_s =
            2n_sdiv+s_pri, where n_sdiv is the number of segments that
            have been subdivided (the number of subdivisions).

        dinest : float, ndarray, shape (`ni`), to be modified in place
            On entry: `dinest`[j-1] contains the current estimate of the
            definite integral of integrand j, for j = 1,2,...,n_i.

            On exit: you may refine the estimates in this argument. This
            should only be done after ``dim1_gen_vec_multi_rcomm`` has
            returned irevcm = 0.

        errest : float, ndarray, shape (`ni`), to be modified in place
            On entry: `errest`[j-1] contains the current error estimate
            associated with integral j, for j = 1,2,...,n_i.

            On exit: you may refine the estimates in this argument. This
            should only be done after ``dim1_gen_vec_multi_rcomm`` has
            returned irevcm = 0.

        fcount : int, ndarray, shape (`ni`)
            `fcount`[j-1] contains the number of different
            approximations of integral j calculated so far, for j =
            1,2,...,n_i.

        sinfoi : int, ndarray, shape ((`ns`-1)*`ldi`+5)
            `sinfoi` contains information about the hierarchy of
            splitting each segment k, for k = 1,2,...,n_s.

            `sinfoi`[`ldi`*(k-1)]

              Contains a unique identifier, the sid, related to segment
              k.

            `sinfoi`[`ldi`*(k-1)+1]

              Contains the parent segment number of segment k, the
              reference to the segment that was split to create segment
              k.

            `sinfoi`[`ldi`*(k-1)+2] and `sinfoi`[`ldi`*(k-1)+3]

              Contain the segment numbers of the two child segments
              formed from segment k, if segment k has been split. If
              segment k has not been split, these will be negative.

            `sinfoi`[`ldi`*(k-1)+4]

              Contains the level at which the segment exists,
              corresponding to n_a+1, where n_a is the number of
              ancestor segments of segment k.

              If `sinfoi`[`ldi`*(k-1)+4] < 0, this segment is considered
              too small for further splitting, and its level is
              |`sinfoi`[`ldi`*(k-1)+4]|.

        evals : int, ndarray, shape ((`ns`-1)*`ldi`+`ni`)
            Contains information to indicate whether an estimate of the
            integral j has been obtained over segment k, and if so
            whether this evaluation still contributes to the
            approximation in `dinest`[j-1], for k = 1,2,...,n_s, for j =
            1,2,...,n_i.

            `evals`[`ldi`*(k-1)+j-1] = 0

              Indicates that integral j has not been evaluated over
              segment k.

            `evals`[`ldi`*(k-1)+j-1] = 1

              Indicates that integral j has been evaluated over segment
              k, and that this evaluation contributes to the total
              approximation `dinest`[j-1].

            `evals`[`ldi`*(k-1)+j-1] = 2

              Indicates that integral j has been evaluated over segment
              k, that this evaluation contributes to the total
              approximation `dinest`[j-1], and that you have requested
              no further evaluation of this integral at this segment by
              setting needi[j-1] < 0 in ``dim1_gen_vec_multi_rcomm``.

            `evals`[`ldi`*(k-1)+j-1] = 3

              Indicates that integral j has been evaluated over segment
              k, and this evaluation no longer contributes to
              `dinest`[j-1].

            `evals`[`ldi`*(k-1)+j-1] = 4

              Indicates that integral j has been evaluated over segment
              k, that this evaluation contributes to the total
              approximation `dinest`[j-1], and that this segment is too
              small for any further splitting to be performed. Integral
              j also has a local error estimate over this segment above
              the requested tolerance. Such segments cause
              ``dim1_gen_vec_multi_rcomm`` to return `errno` = 2 or 3,
              indicating extremely bad behaviour.

            `evals`[`ldi`*(k-1)+j-1] = 5

              Indicates that integral j has been evaluated over segment
              k, that this evaluation contributes to the total
              approximation of `dinest`[j-1], and that this segment is
              too small for any further splitting to be performed. The
              local error estimate is however below the requested
              tolerance.

        ldi : int
            The leading dimension of arrays `sinfoi` and `evals`.

        sinfor : float, ndarray, shape ((`ns`-1)*`ldr`+2)
            `sinfor` contains the bounds of each segment k, for k =
            1,2,...,n_s.

            `sinfor`[`ldr`*(k-1)]

              Contains the lower bound of segment k.

            `sinfor`[`ldr`*(k-1)+1]

              Contains the upper bound of segment k.

        fs : float, ndarray, shape ((`ns`-1)*`ldr`+`ni`)
            `fs`[`ldr`*(k-1)+j-1] contains the definite integral
            estimate of the jth integral over the kth segment, for k =
            1,2,...,n_s, for j = 1,2,...,n_i.

        es : float, ndarray, shape ((`ns`-1)*`ldr`+`ni`)
            `es`[`ldr`*(k-1)+j-1] contains the definite integral error
            estimate of the jth integral over the kth segment, for k =
            1,2,...,n_s, for j = 1,2,...,n_i.

        ldr : int
            The leading dimension of arrays `sinfor`, `fs` and `es`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    ni : int
        n_i, the number of integrals.

    dinest : float, shape (`ni`), modified in place
        On entry: `dinest`[j-1] contains the current estimate of the
        definite integral of integrand j, for j = 1,2,...,n_i.

        On exit: `dinest` is not altered by
        ``withdraw_1d_gen_vec_multi_diagnostic`` directly. It may be
        changed by `monit`.

    errest : float, shape (`ni`), modified in place
        On entry: `errest`[j-1] contains the current error estimate
        associated with integral j, for j = 1,2,...,n_i.

        On exit: `errest` is not altered by
        ``withdraw_1d_gen_vec_multi_diagnostic`` directly. It may be
        changed by `monit`.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim1_gen_vec_multi_rcomm``.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    licusd : int
        The number of elements of the array `comm`['icom'], passed to
        ``dim1_gen_vec_multi_rcomm``, actually used.

    lcusd : int
        The number of elements of `comm`['com'] used by
        ``dim1_gen_vec_multi_rcomm``.

    Raises
    ------
    NagValueError
        (`errno` 21)
            On entry, `ni` is not consistent with that used to construct
            the communication arrays.

            On entry, `ni` = *<value>*.

            Constraint: `ni` = *<value>*.

        (`errno` 1101)
            Either the communication arrays have not been created by
            ``dim1_gen_vec_multi_rcomm``, or they have become corrupted.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base import _nag_d_consts

    excs = []
    fun_name = 'naginterfaces.base.quad.withdraw_1d_gen_vec_multi_diagnostic'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_monit_name = fun_name + ':monit'

    monith = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ni.
        utils._BYREF_INT, # Input, ns.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, dinest[ni].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, errest[ni].
        utils._EngineIntArrayType.c_type, # Input, fcount[ni].
        utils._EngineIntArrayType.c_type, # Input, sinfoi[(ns-1)*ldi+5].
        utils._EngineIntArrayType.c_type, # Input, evals[(ns-1)*ldi+ni].
        utils._BYREF_INT, # Input, ldi.
        utils._EngineFloat64ArrayType.c_type, # Input, sinfor[(ns-1)*ldr+2].
        utils._EngineFloat64ArrayType.c_type, # Input, fs[(ns-1)*ldr+ni].
        utils._EngineFloat64ArrayType.c_type, # Input, es[(ns-1)*ldr+ni].
        utils._BYREF_INT, # Input, ldr.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_monith(_monit, en_data, ni, ns, dinest, errest, fcount, sinfoi, evals, ldi, sinfor, fs, es, ldr, _iuser, _ruser):
        """Helper for monit."""
        if monit is None:
            return
        try:
            _ni_monith = utils._EngineIntScalarType.to_py(ni.contents)
            _ns_monith = utils._EngineIntScalarType.to_py(ns.contents)
            _dinest_monith = utils._EngineFloat64ArrayType.from_pointer(
                dinest,
                (_ni_monith,),
            )
            _errest_monith = utils._EngineFloat64ArrayType.from_pointer(
                errest,
                (_ni_monith,),
            )
            _fcount_monith = utils._EngineIntArrayType.from_pointer(
                fcount,
                (_ni_monith,),
            )
            _ldi_monith = utils._EngineIntScalarType.to_py(ldi.contents)
            _sinfoi_monith = utils._EngineIntArrayType.from_pointer(
                sinfoi,
                ((_ns_monith-1)*_ldi_monith+5,),
            )
            _evals_monith = utils._EngineIntArrayType.from_pointer(
                evals,
                ((_ns_monith-1)*_ldi_monith+_ni_monith,),
            )
            _ldr_monith = utils._EngineIntScalarType.to_py(ldr.contents)
            _sinfor_monith = utils._EngineFloat64ArrayType.from_pointer(
                sinfor,
                ((_ns_monith-1)*_ldr_monith+2,),
            )
            _fs_monith = utils._EngineFloat64ArrayType.from_pointer(
                fs,
                ((_ns_monith-1)*_ldr_monith+_ni_monith,),
            )
            _es_monith = utils._EngineFloat64ArrayType.from_pointer(
                es,
                ((_ns_monith-1)*_ldr_monith+_ni_monith,),
            )
            if data is not None:
                monit(
                    _ni_monith,
                    _ns_monith,
                    _dinest_monith,
                    _errest_monith,
                    _fcount_monith,
                    _sinfoi_monith,
                    _evals_monith,
                    _ldi_monith,
                    _sinfor_monith,
                    _fs_monith,
                    _es_monith,
                    _ldr_monith,
                    data,
                )
            else:
                monit(
                    _ni_monith,
                    _ns_monith,
                    _dinest_monith,
                    _errest_monith,
                    _fcount_monith,
                    _sinfoi_monith,
                    _evals_monith,
                    _ldi_monith,
                    _sinfor_monith,
                    _fs_monith,
                    _es_monith,
                    _ldr_monith,
                )
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _dinest_monith, {'fun_name': cb_monit_name, 'returned': 'dinest'}, dinest,
            )
            utils._unlink_pointer(dinest)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _errest_monith, {'fun_name': cb_monit_name, 'returned': 'errest'}, errest,
            )
            utils._unlink_pointer(errest)
            utils._unlink_pointer(fcount)
            utils._unlink_pointer(sinfoi)
            utils._unlink_pointer(evals)
            utils._unlink_pointer(sinfor)
            utils._unlink_pointer(fs)
            utils._unlink_pointer(es)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d01rbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._UNUSED_FUNCTYPE,
        monith,
        utils._BYREF_INT, # Input, ni.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, dinest[ni].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, errest[ni].
        utils._EngineIntArrayType.c_type, # Input, icom[licom].
        utils._BYREF_INT, # Input, licom.
        utils._BYREF_INT, # Output, licusd.
        utils._EngineFloat64ArrayType.c_type, # Input, com[lcom].
        utils._BYREF_INT, # Input, lcom.
        utils._BYREF_INT, # Output, lcusd.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    utils._check_callable(monit, {'fun_name': fun_name, 'entity': 'monit'}, allow_none=True)
    _monit = utils._UNUSED_CALLBACK
    _monith = utils._cast_callback(
        monith, cb_monith,
        {'fun_name': fun_name, 'entity': 'monit'},
    )
    _ni = utils._EngineIntScalarType(
        ni, {'fun_name': fun_name, 'entity': 'ni'},
    ).to_ctypes()
    _ni_py = utils._EngineIntScalarType.to_py(_ni)
    _dinest_cls = utils._EngineFloat64ArrayType(
        dinest, {'fun_name': fun_name, 'entity': 'dinest'},
        exp_shape=(_ni_py,),
    )
    _dinest = _dinest_cls.to_ctypes()
    _errest_cls = utils._EngineFloat64ArrayType(
        errest, {'fun_name': fun_name, 'entity': 'errest'},
        exp_shape=(_ni_py,),
    )
    _errest = _errest_cls.to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icom', 'com'),
    )
    _licom_py = utils._EngineIntArrayType.get_shape(comm['icom'], {'fun_name': fun_name, 'entity': 'comm[\'icom\']'}, exp_rank=1)[0]
    _licom = utils._EngineIntScalarType(
        _licom_py, {'fun_name': fun_name, 'entity': '_licom'},
    ).to_ctypes()
    _icom = comm['icom']
    _licusd = utils._EngineIntScalarType.empty_ctypes()
    _lcom_py = utils._EngineFloat64ArrayType.get_shape(comm['com'], {'fun_name': fun_name, 'entity': 'comm[\'com\']'}, exp_rank=1)[0]
    _lcom = utils._EngineIntScalarType(
        _lcom_py, {'fun_name': fun_name, 'entity': '_lcom'},
    ).to_ctypes()
    _com = comm['com']
    _lcusd = utils._EngineIntScalarType.empty_ctypes()
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _monit,
        _monith,
        _ni,
        _dinest,
        _errest,
        _icom,
        _licom,
        _licusd,
        _com,
        _lcom,
        _lcusd,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == _nag_d_consts.D01RB_IERR_NI and _ifmt == _nag_d_consts.D01RB_IFMT_NI:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ni is not consistent with that used to construct',
                    'the communication arrays.',
                    'On entry, ni = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ni = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01_IERR_COM_CORRUPT and _ifmt == _nag_d_consts.D01_IFMT_COM_CORRUPT:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the communication arrays have not been created by dim1_gen_vec_multi_rcomm,',
                    'or they have become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _dinest_cls.refresh_py(_dinest)
        _errest_cls.refresh_py(_errest)
        _licusd_py = utils._EngineIntScalarType.to_py(_licusd)
        _lcusd_py = utils._EngineIntScalarType.to_py(_lcusd)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['licusd', 'lcusd'],
        )(
            _licusd_py,
            _lcusd_py,
        )
    _err_class.manage()
    return return_data

def dim1_gen_vec_multi_dimreq(ni, iopts, opts):
    u"""
    Determine required array dimensions for
    ``dim1_gen_vec_multi_rcomm``.

    For full information please refer to the NAG Library document for
    d01rc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01rcf.html

    Parameters
    ----------
    ni : int
        n_i, the number of integrals which will be approximated in the
        subsequent call to ``dim1_gen_vec_multi_rcomm``.

    iopts : int, shape (100)
        The integer option array as returned by ``opt_set``.

    opts : float, shape (100)
        The real option array as returned by ``opt_set``.

    Returns
    -------
    lenxrq : int
        lenxrq, the minimum dimension of the array x that can be used in
        a subsequent call to ``dim1_gen_vec_multi_rcomm``.

    ldfmrq : int
        ldfmrq, the minimum leading dimension of the array fm that can
        be used in a subsequent call to ``dim1_gen_vec_multi_rcomm``.

    sdfmrq : int
        sdfmrq, the minimum second dimension of the array fm that can be
        used in a subsequent call to ``dim1_gen_vec_multi_rcomm``.

        Note: the minimum dimension of the array fm is ldfmrq*sdfmrq.

    licmin : int
        licmin, the minimum dimension of the array icom that must be
        passed to ``dim1_gen_vec_multi_rcomm`` to enable it to calculate
        a single approximation to all the n_i integrals over the
        interval [a,b] with s_pri initial segments.

    licmax : int
        licmax the dimension of the array icom that must be passed to
        ``dim1_gen_vec_multi_rcomm`` to enable it to exhaust the
        adaptive process controlled by the currently set options for the
        n_i integrals over the interval [a,b] with s_pri initial
        segments.

    lcmin : int
        lcmin, the minimum dimension of the array com that must be
        passed to ``dim1_gen_vec_multi_rcomm`` to enable it to calculate
        a single approximation to all the n_i integrals over the
        interval [a,b] with s_pri initial segments.

    lcmax : int
        lcmax, the dimension of the array com that must be passed to
        ``dim1_gen_vec_multi_rcomm`` to enable it to exhaust the
        adaptive process controlled by the currently set options for the
        n_i integrals over the interval [a,b] with s_pri initial
        segments.

    Raises
    ------
    NagValueError
        (`errno` 21)
            On entry, `ni` = *<value>*.

            Constraint: `ni` > 0.

        (`errno` 1001)
            One of the option arrays `iopts` or `opts` has become
            corrupted. Re-initialize the arrays using ``opt_set``.
    """

    from ..base import _nag_d_consts

    excs = []
    fun_name = 'naginterfaces.base.quad.dim1_gen_vec_multi_dimreq'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d01rcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ni.
        utils._BYREF_INT, # Output, lenxrq.
        utils._BYREF_INT, # Output, ldfmrq.
        utils._BYREF_INT, # Output, sdfmrq.
        utils._BYREF_INT, # Output, licmin.
        utils._BYREF_INT, # Output, licmax.
        utils._BYREF_INT, # Output, lcmin.
        utils._BYREF_INT, # Output, lcmax.
        utils._EngineIntArrayType.c_type, # Input, iopts[100].
        utils._EngineFloat64ArrayType.c_type, # Input, opts[100].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ni = utils._EngineIntScalarType(
        ni, {'fun_name': fun_name, 'entity': 'ni'},
    ).to_ctypes()
    _lenxrq = utils._EngineIntScalarType.empty_ctypes()
    _ldfmrq = utils._EngineIntScalarType.empty_ctypes()
    _sdfmrq = utils._EngineIntScalarType.empty_ctypes()
    _licmin = utils._EngineIntScalarType.empty_ctypes()
    _licmax = utils._EngineIntScalarType.empty_ctypes()
    _lcmin = utils._EngineIntScalarType.empty_ctypes()
    _lcmax = utils._EngineIntScalarType.empty_ctypes()
    _iopts = utils._EngineIntArrayType(
        iopts, {'fun_name': fun_name, 'entity': 'iopts'},
        exp_shape=(100,),
    ).to_ctypes()
    _opts = utils._EngineFloat64ArrayType(
        opts, {'fun_name': fun_name, 'entity': 'opts'},
        exp_shape=(100,),
    ).to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ni,
        _lenxrq,
        _ldfmrq,
        _sdfmrq,
        _licmin,
        _licmax,
        _lcmin,
        _lcmax,
        _iopts,
        _opts,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == _nag_d_consts.D01RA_IERR_NI and _ifmt == _nag_d_consts.D01RA_IFMT_NI:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ni = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ni > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01_IERR_OPTS_CORRUPT and _ifmt == _nag_d_consts.D01_IFMT_OPTS_CORRUPT:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'One of the option arrays iopts or opts has become corrupted.',
                    'Re-initialize the arrays using opt_set.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _lenxrq_py = utils._EngineIntScalarType.to_py(_lenxrq)
        _ldfmrq_py = utils._EngineIntScalarType.to_py(_ldfmrq)
        _sdfmrq_py = utils._EngineIntScalarType.to_py(_sdfmrq)
        _licmin_py = utils._EngineIntScalarType.to_py(_licmin)
        _licmax_py = utils._EngineIntScalarType.to_py(_licmax)
        _lcmin_py = utils._EngineIntScalarType.to_py(_lcmin)
        _lcmax_py = utils._EngineIntScalarType.to_py(_lcmax)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['lenxrq', 'ldfmrq', 'sdfmrq', 'licmin', 'licmax', 'lcmin', 'lcmax'],
        )(
            _lenxrq_py,
            _ldfmrq_py,
            _sdfmrq_py,
            _licmin_py,
            _licmax_py,
            _lcmin_py,
            _lcmax_py,
        )
    _err_class.manage()
    return return_data

def dim1_fin_gonnet_vec(a, b, f, epsabs, epsrel, data=None):
    u"""
    One-dimensional quadrature, adaptive, finite interval, strategy due
    to Gonnet, allowing for badly behaved integrands.

    For full information please refer to the NAG Library document for
    d01rg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01rgf.html

    Parameters
    ----------
    a : float
        a, the lower limit of integration.

    b : float
        b, the upper limit of integration. It is not necessary that a <
        b.

        Note: if `a` = `b`, the function will immediately return
        `dinest` = 0.0, `errest` = 0.0 and `nevals` = 0.

    f : callable iflag = f(x, nx, fv, iflag, data=None)
        `f` must return the value of the integrand f at a set of points.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (`nx`)
            The abscissae, x_i, for i = 1,2,...,`nx`, at which function
            values are required.

        nx : int
            The number of abscissae at which a function value is
            required.

        fv : float, ndarray, shape (`nx`), to be modified in place
            On exit: `fv` must contain the values of the integrand f.
            `fv`[i-1] = f(x_i) for all i = 1,2,...,`nx`.

        iflag : int
            `iflag` = 0.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        iflag : int
            Set `iflag` < 0 to force an immediate exit with `errno` =
            -1.

    epsabs : float
        The absolute accuracy required.

        If `epsabs` is negative, |`epsabs`| is used.

        See [Accuracy].

        If `epsabs` = 0.0, only the relative error will be used.

    epsrel : float
        The relative accuracy required.

        If `epsrel` is negative, |`epsrel`| is used.

        See [Accuracy].

        If `epsrel` = 0.0, only the absolute error will be used
        otherwise the actual value of `epsrel` used by
        ``dim1_fin_gonnet_vec`` is max(machine precision, |`epsrel`|).

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    dinest : float
        The estimate of the definite integral `f`.

    errest : float
        The error estimate of the definite integral `f`.

    nevals : int
        The total number of points at which the integrand, f, has been
        evaluated.

    Raises
    ------
    NagValueError
        (`errno` 14)
            Both `epsabs` = 0.0 and `epsrel` = 0.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` -1)
            Exit requested from `f` with `iflag` = *<value>*.

        (`errno` 1)
            The requested accuracy was not achieved. Consider using
            larger values of `epsabs` and `epsrel`.

        (`errno` 2)
            The integral is probably divergent or slowly convergent.
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.dim1_fin_gonnet_vec'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, x[nx].
        utils._BYREF_INT, # Input, nx.
        utils._EngineFloat64ArrayType.c_type, # Output, fv[nx].
        utils._BYREF_INT, # Input/Output, iflag.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, en_data, x, nx, fv, iflag, _iuser, _ruser):
        """Helper for f."""
        try:
            _nx_fh = utils._EngineIntScalarType.to_py(nx.contents)
            _x_fh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_nx_fh,),
            )
            _fv_fh = utils._EngineFloat64ArrayType.from_pointer(
                fv,
                (_nx_fh,),
                can_reference_contents=False,
            )
            _iflag_fh = utils._EngineIntScalarType.to_py(iflag.contents)
            if data is not None:
                _iflag_fh = f(
                    _x_fh,
                    _nx_fh,
                    _fv_fh,
                    _iflag_fh,
                    data,
                )
            else:
                _iflag_fh = f(
                    _x_fh,
                    _nx_fh,
                    _fv_fh,
                    _iflag_fh,
                )
            utils._unlink_pointer(x)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _fv_fh, {'fun_name': cb_f_name, 'returned': 'fv'}, fv,
            )
            utils._unlink_pointer(fv)
            utils._EngineIntScalarType(
                _iflag_fh, {'fun_name': cb_f_name, 'returned': 'iflag'},
            ).refresh_ctypes_buffer(iflag)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d01rgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._BYREF_FLOAT64, # Input, epsabs.
        utils._BYREF_FLOAT64, # Input, epsrel.
        utils._BYREF_FLOAT64, # Output, dinest.
        utils._BYREF_FLOAT64, # Output, errest.
        utils._BYREF_INT, # Output, nevals.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _epsabs = utils._EngineFloat64ScalarType(
        epsabs, {'fun_name': fun_name, 'entity': 'epsabs'},
    ).to_ctypes()
    _epsrel = utils._EngineFloat64ScalarType(
        epsrel, {'fun_name': fun_name, 'entity': 'epsrel'},
    ).to_ctypes()
    _dinest = utils._EngineFloat64ScalarType.empty_ctypes()
    _errest = utils._EngineFloat64ScalarType.empty_ctypes()
    _nevals = utils._EngineIntScalarType.empty_ctypes()
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _a,
        _b,
        _f,
        _fh,
        _epsabs,
        _epsrel,
        _dinest,
        _errest,
        _nevals,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == -1 and _ifmt == 99995:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Exit requested from f with iflag = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The requested accuracy was not achieved.',
                    'Consider using larger values of epsabs and epsrel.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The integral is probably divergent or slowly convergent.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Both epsabs = 0.0 and epsrel = 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _dinest_py = utils._EngineFloat64ScalarType.to_py(_dinest)
        _errest_py = utils._EngineFloat64ScalarType.to_py(_errest)
        _nevals_py = utils._EngineIntScalarType.to_py(_nevals)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['dinest', 'errest', 'nevals'],
        )(
            _dinest_py,
            _errest_py,
            _nevals_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim1_gauss_wres(key, a, b, n, weight, abscis):
    u"""
    Pre-computed weights and abscissae for Gaussian quadrature rules,
    restricted choice of rule.

    For full information please refer to the NAG Library document for
    d01tb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01tbf.html

    Parameters
    ----------
    key : int
        Indicates the quadrature formula.

        `key` = 0

          Gauss--Legendre quadrature on a finite interval, using normal
          weights.

        `key` = 3

          Gauss--Laguerre quadrature on a semi-infinite interval, using
          normal weights.

        `key` = -3

          Gauss--Laguerre quadrature on a semi-infinite interval, using
          adjusted weights.

        `key` = 4

          Gauss--Hermite quadrature on an infinite interval, using
          normal weights.

        `key` = -4

          Gauss--Hermite quadrature on an infinite interval, using
          adjusted weights.

        `key` = -5

          Rational Gauss quadrature on a semi-infinite interval, using
          adjusted weights.

    a : float
        The quantities a and b as described in the appropriate
        sub-section of [Description].

    b : float
        The quantities a and b as described in the appropriate
        sub-section of [Description].

    n : int
        n, the number of weights and abscissae to be returned.

    weight : float, shape (`n`), modified in place
        On exit: the `n` weights.

    abscis : float, shape (`n`), modified in place
        On exit: the `n` abscissae.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, `key` = *<value>*.

            Constraint: `key` = 0, 3, -3, 4, -4 or -5.

        (`errno` 12)
            The value of `a` and/or `b` is invalid for Gauss-Laguerre
            quadrature.

            On entry, `key` = *<value>*.

            On entry, `a` = *<value>* and `b` = *<value>*.

            Constraint: abs(`b`) > 0.0.

        (`errno` 12)
            The value of `a` and/or `b` is invalid for Gauss-Hermite
            quadrature.

            On entry, `key` = *<value>*.

            On entry, `a` = *<value>* and `b` = *<value>*.

            Constraint: `b` > 0.0.

        (`errno` 12)
            The value of `a` and/or `b` is invalid for rational Gauss
            quadrature.

            On entry, `key` = *<value>*.

            On entry, `a` = *<value>* and `b` = *<value>*.

            Constraint: abs(`a`+`b`) > 0.0.

        (`errno` 14)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The `n`-point rule is not among those stored.

            On entry: `n` = *<value>*.

            `n`-rule used: `n` = *<value>*.

        (`errno` 2)
            Underflow occurred in calculation of normal weights.

            Reduce `n` or use adjusted weights: `n` = *<value>*.

        (`errno` 3)
            No nonzero weights were generated for the provided
            parameters.
    """

    from ..base import _nag_d_consts

    excs = []
    fun_name = 'naginterfaces.base.quad.dim1_gauss_wres'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d01tbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, key.
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Output, weight[n].
        utils._EngineFloat64ArrayType.c_type, # Output, abscis[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _key = utils._EngineIntScalarType(
        key, {'fun_name': fun_name, 'entity': 'key'},
    ).to_ctypes()
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _weight_cls = utils._EngineFloat64ArrayType(
        weight, {'fun_name': fun_name, 'entity': 'weight'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _weight = _weight_cls.to_ctypes(can_reference_contents=False)
    _abscis_cls = utils._EngineFloat64ArrayType(
        abscis, {'fun_name': fun_name, 'entity': 'abscis'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _abscis = _abscis_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _key,
        _a,
        _b,
        _n,
        _weight,
        _abscis,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == _nag_d_consts.D01TB_IERR_WARN_N and _ifmt == _nag_d_consts.D01TB_IFMT_WARN_N:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The n-point rule is not among those stored.',
                    'On entry: n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'n-rule used: n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01TB_IERR_WEIGHT_UFLOW and _ifmt == _nag_d_consts.D01TB_IFMT_WEIGHT_UFLOW:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Underflow occurred in calculation of normal weights.',
                    'Reduce n or use adjusted weights: n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01TB_IERR_W_ALL_ZERO and _ifmt == _nag_d_consts.D01TB_IFMT_W_ALL_ZERO:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'No nonzero weights were generated for the provided parameters.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01TB_IERR_KEY and _ifmt == _nag_d_consts.D01TB_IFMT_KEY:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, key = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: key = 0, 3, -3, 4, -4 or -5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01TB_IERR_A_B_KEY and _ifmt == _nag_d_consts.D01TB_IFMT_AB_KEY_GAUSS_LAG:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The value of a and/or b is invalid for Gauss-Laguerre quadrature.',
                    'On entry, key = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'On entry, a = {:s} and b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: abs(b) > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01TB_IERR_A_B_KEY and _ifmt == _nag_d_consts.D01TB_IFMT_AB_KEY_GAUSS_HERM:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The value of a and/or b is invalid for Gauss-Hermite quadrature.',
                    'On entry, key = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'On entry, a = {:s} and b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: b > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01TB_IERR_A_B_KEY and _ifmt == _nag_d_consts.D01TB_IFMT_AB_KEY_GAUSS_RAT:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The value of a and/or b is invalid for rational Gauss quadrature.',
                    'On entry, key = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'On entry, a = {:s} and b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: abs(a+b) > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01TB_IERR_N and _ifmt == _nag_d_consts.D01TB_IFMT_N_LT_1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _weight_cls.refresh_py(_weight)
        _abscis_cls.refresh_py(_abscis)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def dim1_gauss_wrec(n, a, b, c, muzero, weight, abscis):
    u"""
    Calculation of weights and abscissae for Gaussian quadrature rules,
    method of Golub and Welsch.

    For full information please refer to the NAG Library document for
    d01td

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01tdf.html

    Parameters
    ----------
    n : int
        n, the number of Gauss points required. The resulting quadrature
        rule will be exact for all polynomials of degree 2n-1.

    a : float, shape (`n`)
        `a` contains the coefficients a(j).

    b : float, shape (`n`), modified in place
        On entry: `b` contains the coefficients b(j).

        On exit: elements of `b` are altered to make the underlying
        eigenvalue problem symmetric.

    c : float, shape (`n`), modified in place
        On entry: `c` contains the coefficients c(j).

        On exit: elements of `c` are altered to make the underlying
        eigenvalue problem symmetric.

    muzero : float
        `muzero` contains the definite integral of the weight function
        for the interval of interest.

    weight : float, shape (`n`), modified in place
        On exit: `weight`[j-1] contains the weight corresponding to the
        jth abscissa.

    abscis : float, shape (`n`), modified in place
        On exit: `abscis`[j-1] the jth abscissa.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The number of weights and abscissae requested (`n`) is less
            than 1: `n` = *<value>*.

        (`errno` 4)
            Unexpected failure in eigenvalue computation. Please contact
            NAG.

        (`errno` 5)
            The algorithm failed to converge. The ith diagonal was not
            zero: i = *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.dim1_gauss_wrec'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d01tdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, a[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, b[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c[n].
        utils._BYREF_FLOAT64, # Input, muzero.
        utils._EngineFloat64ArrayType.c_type, # Output, weight[n].
        utils._EngineFloat64ArrayType.c_type, # Output, abscis[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py,),
    )
    _b = _b_cls.to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_n_py,),
    )
    _c = _c_cls.to_ctypes()
    _muzero = utils._EngineFloat64ScalarType(
        muzero, {'fun_name': fun_name, 'entity': 'muzero'},
    ).to_ctypes()
    _weight_cls = utils._EngineFloat64ArrayType(
        weight, {'fun_name': fun_name, 'entity': 'weight'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _weight = _weight_cls.to_ctypes(can_reference_contents=False)
    _abscis_cls = utils._EngineFloat64ArrayType(
        abscis, {'fun_name': fun_name, 'entity': 'abscis'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _abscis = _abscis_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _a,
        _b,
        _c,
        _muzero,
        _weight,
        _abscis,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The number of weights and abscissae requested (n) is less than 1:',
                    'n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Unexpected failure in eigenvalue computation. Please contact NAG.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm failed to converge. The ith diagonal was not zero:',
                    'i = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        _c_cls.refresh_py(_c)
        _weight_cls.refresh_py(_weight)
        _abscis_cls.refresh_py(_abscis)
        return_data = None
    _err_class.manage()
    return return_data

def dim1_gauss_recm(n, mu, a, b, c):
    u"""
    Generates recursion coefficients needed by ``dim1_gauss_wrec`` to
    calculate a Gaussian quadrature rule.

    For full information please refer to the NAG Library document for
    d01te

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01tef.html

    Parameters
    ----------
    n : int
        n, the number of weights and abscissae required.

    mu : float, shape (2*`n`+1)
        `mu`[i] must contain the value of the moment with respect to x^i
        i.e., intw(x)x^idx, for i = 0,1,...,2n.

    a : float, shape (`n`), modified in place
        On exit: values helping define the three term recurrence used by
        ``dim1_gauss_wrec``.

    b : float, shape (`n`), modified in place
        On exit: values helping define the three term recurrence used by
        ``dim1_gauss_wrec``.

    c : float, shape (`n`), modified in place
        On exit: values helping define the three term recurrence used by
        ``dim1_gauss_wrec``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The number of weights and abscissae requested (`n`) is less
            than 1: `n` = *<value>*.

        (`errno` 2)
            The problem is too ill conditioned, it breaks down at row
            *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.dim1_gauss_recm'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d01teft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, mu[2*n+1].
        utils._EngineFloat64ArrayType.c_type, # Output, a[n].
        utils._EngineFloat64ArrayType.c_type, # Output, b[n].
        utils._EngineFloat64ArrayType.c_type, # Output, c[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _mu = utils._EngineFloat64ArrayType(
        mu, {'fun_name': fun_name, 'entity': 'mu'},
        exp_shape=(2*_n_py+1,),
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _a = _a_cls.to_ctypes(can_reference_contents=False)
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _b = _b_cls.to_ctypes(can_reference_contents=False)
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _mu,
        _a,
        _b,
        _c,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The number of weights and abscissae requested (n) is less than 1:',
                    'n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The problem is too ill conditioned, it breaks down at row',
                    '{:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _b_cls.refresh_py(_b)
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def dim1_gauss_vec(key, a, b, n, f, data=None):
    u"""
    One-dimensional Gaussian quadrature, choice of weight functions
    (vectorized).

    For full information please refer to the NAG Library document for
    d01ua

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01uaf.html

    Parameters
    ----------
    key : int
        Indicates the quadrature formula.

        `key` = 0

          Gauss--Legendre quadrature on a finite interval, using normal
          weights.

        `key` = 3

          Gauss--Laguerre quadrature on a semi-infinite interval, using
          normal weights.

        `key` = -3

          Gauss--Laguerre quadrature on a semi-infinite interval, using
          adjusted weights.

        `key` = 4

          Gauss--Hermite quadrature on an infinite interval, using
          normal weights.

        `key` = -4

          Gauss--Hermite quadrature on an infinite interval, using
          adjusted weights.

        `key` = -5

          Rational Gauss quadrature on a semi-infinite interval, using
          adjusted weights.

    a : float
        The quantities a and b as described in the appropriate
        subsection of [Description].

    b : float
        The quantities a and b as described in the appropriate
        subsection of [Description].

    n : int
        n, the number of abscissae to be used.

    f : callable iflag = f(x, nx, fv, iflag, data=None)
        `f` must return the value of the integrand f, or the normalized
        integrand g, at a specified point.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (`nx`)
            The abscissae, x_i, for i = 1,2,...,n_x at which function
            values are required.

        nx : int
            n_x, the number of abscissae.

        fv : float, ndarray, shape (`nx`), to be modified in place
            On exit: if adjusted weights are used, the values of the
            integrand f. `fv`[i-1] = f(x_i), for i = 1,2,...,n_x.

            Otherwise the values of the normalized integrand g.
            `fv`[i-1] = g(x_i), for i = 1,2,...,n_x.

        iflag : int
            `iflag` = 0.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        iflag : int
            Set `iflag` < 0 if you wish to force an immediate exit from
            ``dim1_gauss_vec`` with `errno` = -1.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    dinest : float
        The estimate of the definite integral.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, `key` = *<value>*.

            Constraint: `key` = 0, 3, -3, 4, -4 or -5.

        (`errno` 12)
            The value of `a` and/or `b` is invalid.

            On entry, `key` = *<value>*.

            On entry, `a` = *<value>* and `b` = *<value>*.

            Constraint: abs(`b`) > 0.0.

        (`errno` 12)
            The value of `a` and/or `b` is invalid.

            On entry, `key` = *<value>*.

            On entry, `a` = *<value>* and `b` = *<value>*.

            Constraint: `b` > 0.0.

        (`errno` 12)
            The value of `a` and/or `b` is invalid.

            On entry, `key` = *<value>*.

            On entry, `a` = *<value>* and `b` = *<value>*.

            Constraint: abs(`a`+`b`) > 0.0.

        (`errno` 14)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` -1)
            Exit requested from `f` with `iflag` = *<value>*.

        (`errno` 1)
            The n-point rule is not among those stored.

            On entry: `n` = *<value>*.

            n-point rule used: `n` = *<value>*.

        (`errno` 2)
            Underflow occurred in calculation of normal weights.

            Reduce `n` or use adjusted weights: `n` = *<value>*.

        (`errno` 3)
            No nonzero weights were generated for the provided
            parameters.
    """

    from ..base import _nag_d_consts

    excs = []
    fun_name = 'naginterfaces.base.quad.dim1_gauss_vec'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, x[nx].
        utils._BYREF_INT, # Input, nx.
        utils._EngineFloat64ArrayType.c_type, # Output, fv[nx].
        utils._BYREF_INT, # Input/Output, iflag.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, en_data, x, nx, fv, iflag, _iuser, _ruser):
        """Helper for f."""
        try:
            _nx_fh = utils._EngineIntScalarType.to_py(nx.contents)
            _x_fh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_nx_fh,),
            )
            _fv_fh = utils._EngineFloat64ArrayType.from_pointer(
                fv,
                (_nx_fh,),
                can_reference_contents=False,
            )
            _iflag_fh = utils._EngineIntScalarType.to_py(iflag.contents)
            if data is not None:
                _iflag_fh = f(
                    _x_fh,
                    _nx_fh,
                    _fv_fh,
                    _iflag_fh,
                    data,
                )
            else:
                _iflag_fh = f(
                    _x_fh,
                    _nx_fh,
                    _fv_fh,
                    _iflag_fh,
                )
            utils._unlink_pointer(x)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _fv_fh, {'fun_name': cb_f_name, 'returned': 'fv'}, fv,
            )
            utils._unlink_pointer(fv)
            utils._EngineIntScalarType(
                _iflag_fh, {'fun_name': cb_f_name, 'returned': 'iflag'},
            ).refresh_ctypes_buffer(iflag)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d01uaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, key.
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_INT, # Input, n.
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._BYREF_FLOAT64, # Output, dinest.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _key = utils._EngineIntScalarType(
        key, {'fun_name': fun_name, 'entity': 'key'},
    ).to_ctypes()
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _dinest = utils._EngineFloat64ScalarType.empty_ctypes()
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _key,
        _a,
        _b,
        _n,
        _f,
        _fh,
        _dinest,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == _nag_d_consts.D01_IERR_USER_EXIT_INT and _ifmt == _nag_d_consts.D01_IFMT_USER_EXIT:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Exit requested from f with iflag = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01TB_IERR_WARN_N and _ifmt == _nag_d_consts.D01TB_IFMT_WARN_N:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The n-point rule is not among those stored.',
                    'On entry: n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'n-point rule used: n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01TB_IERR_WEIGHT_UFLOW and _ifmt == _nag_d_consts.D01TB_IFMT_WEIGHT_UFLOW:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Underflow occurred in calculation of normal weights.',
                    'Reduce n or use adjusted weights: n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01TB_IERR_W_ALL_ZERO and _ifmt == _nag_d_consts.D01TB_IFMT_W_ALL_ZERO:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'No nonzero weights were generated for the provided parameters.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01TB_IERR_KEY and _ifmt == _nag_d_consts.D01TB_IFMT_KEY:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, key = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: key = 0, 3, -3, 4, -4 or -5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01TB_IERR_A_B_KEY and _ifmt == _nag_d_consts.D01TB_IFMT_AB_KEY_GAUSS_LAG:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The value of a and/or b is invalid.',
                    'On entry, key = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'On entry, a = {:s} and b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: abs(b) > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01TB_IERR_A_B_KEY and _ifmt == _nag_d_consts.D01TB_IFMT_AB_KEY_GAUSS_HERM:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The value of a and/or b is invalid.',
                    'On entry, key = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'On entry, a = {:s} and b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: b > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01TB_IERR_A_B_KEY and _ifmt == _nag_d_consts.D01TB_IFMT_AB_KEY_GAUSS_RAT:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The value of a and/or b is invalid.',
                    'On entry, key = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'On entry, a = {:s} and b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: abs(a+b) > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_d_consts.D01TB_IERR_N and _ifmt == _nag_d_consts.D01TB_IFMT_N_LT_1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _dinest_py = utils._EngineFloat64ScalarType.to_py(_dinest)
        return_data = _dinest_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['dinest'],
            )(
                _dinest_py,
            )
    _err_class.manage()
    return return_data

def dim1_inf_exp_wt(f, n, data=None):
    u"""
    Non-automatic function to evaluate int_0^infinity exp(-x^2)f(x)dx.

    For full information please refer to the NAG Library document for
    d01ub

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01ubf.html

    Parameters
    ----------
    f : callable istop = f(x, fv, n, istop, data=None)
        `f` must return the integrand function values f(x_i) for the
        given x_i, for i = 1,2,...,`n`.

        Parameters
        ~~~~~~~~~~
        x : float, ndarray, shape (`n`)
            The points at which the integrand function f must be
            evaluated.

        fv : float, ndarray, shape (`n`), to be modified in place
            On exit: `fv`[i-1] must contain the value of the integrand
            f(x_i) evaluated at the point `x`[i-1], for i = 1,2,...,`n`.

        n : int
            This argument specifies the number of weights and abscissae
            to be used.

        istop : int
            `istop` = 0.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        istop : int
            You may set `istop` to a negative number if at any time it
            is impossible to evaluate the function f(x). In this case
            ``dim1_inf_exp_wt`` halts with errno set to the value of
            `istop` and the value returned in `ans` will be that of a
            non-signalling NaN.

    n : int
        `n` specifies the number of weights and abscissae to be used.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    ans : float
        If no exception or warning is raised, this argument contains an
        approximation to the integral. Otherwise, this argument will be
        a non-signalling NaN.

    Raises
    ------
    NagValueError
        (`errno` i < 0)
            The user has halted the calculation.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: 1 <= `n` <= 25.

        (`errno` 2)
            On entry, `n` = *<value>*.

            `n` is not one of the allowed values.
    """

    excs = []
    fun_name = 'naginterfaces.base.quad.dim1_inf_exp_wt'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, fv[n].
        utils._BYREF_INT, # Input, n.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Input/Output, istop.
    )

    def cb_fh(_f, en_data, x, fv, n, _iuser, _ruser, istop):
        """Helper for f."""
        try:
            _n_fh = utils._EngineIntScalarType.to_py(n.contents)
            _x_fh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_n_fh,),
            )
            _fv_fh = utils._EngineFloat64ArrayType.from_pointer(
                fv,
                (_n_fh,),
                can_reference_contents=False,
            )
            _istop_fh = utils._EngineIntScalarType.to_py(istop.contents)
            if data is not None:
                _istop_fh = f(
                    _x_fh,
                    _fv_fh,
                    _n_fh,
                    _istop_fh,
                    data,
                )
            else:
                _istop_fh = f(
                    _x_fh,
                    _fv_fh,
                    _n_fh,
                    _istop_fh,
                )
            utils._unlink_pointer(x)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _fv_fh, {'fun_name': cb_f_name, 'returned': 'fv'}, fv,
            )
            utils._unlink_pointer(fv)
            utils._EngineIntScalarType(
                _istop_fh, {'fun_name': cb_f_name, 'returned': 'istop'},
            ).refresh_ctypes_buffer(istop)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d01ubft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Output, ans.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _ans = utils._EngineFloat64ScalarType.empty_ctypes()
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _f,
        _fh,
        _n,
        _ans,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py < 0 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The user has halted the calculation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 1 <= n <= 25.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'n is not one of the allowed values.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ans_py = utils._EngineFloat64ScalarType.to_py(_ans)
        return_data = _ans_py
    _err_class.manage()
    return return_data

def opt_set(optstr, comm):
    u"""
    Option setting function.

    For full information please refer to the NAG Library document for
    d01zk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01zkf.html

    Parameters
    ----------
    optstr : str
        A string identifying the option to be set.

        Initialize = function name

          Initialize the option arrays `comm`['iopts'] and
          `comm`['opts'] for use with function function name, where
          function name is the short name associated with the function
          of interest.

        Defaults

          Resets all options to their default values.

        option = optval

          See [Optional Parameters] for details of valid values for
          option and optval. The equals sign (=) delimiter must be used
          to separate the option from its optval value.

        `optstr` is case insensitive.

        Each token in the option and optval component must be separated
        by at least one space.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    Raises
    ------
    NagValueError
        (`errno` 14)
            On entry, attempting to initialize the option arrays but
            specified function name was not valid: name = *<value>*.

        (`errno` 21)
            On entry, either the option arrays have not been initialized
            or they have been corrupted.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 11)
            On entry, the option in `optstr` was not recognized:
            `optstr` = *<value>*.

        (`errno` 12)
            On entry, the expected delimiter '=' was not found in
            `optstr`: `optstr` = *<value>*.

        (`errno` 13)
            On entry, could not convert the specified optval to an
            integer: `optstr` = *<value>*.

        (`errno` 13)
            On entry, could not convert the specified optval to a real:
            `optstr` = *<value>*.

        (`errno` 15)
            On entry, the optval supplied for the integer option is not
            valid.

            `optstr` = *<value>*.

        (`errno` 16)
            On entry, the optval supplied for the real option is not
            valid.

            `optstr` = *<value>*.

        (`errno` 17)
            On entry, the optval supplied for the character option is
            not valid.

            `optstr` = *<value>*.
    """

    from ..base import _nag_e_consts

    excs = []
    fun_name = 'naginterfaces.base.quad.opt_set'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d01zkft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, optstr(len=*).
        utils._EngineIntArrayType.c_type, # Input/Output, iopts[liopts].
        utils._BYREF_INT, # Input, liopts.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, opts[lopts].
        utils._BYREF_INT, # Input, lopts.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of optstr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _optstr = utils._EngineCharScalarType(
        optstr, {'fun_name': fun_name, 'entity': 'optstr'}, exp_chrlen=None,
    ).to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('iopts', 'opts'),
    )
    if _comm_needs_init:
        _iopts = utils._EngineIntArrayType.empty_ctypes(100)
        comm['iopts'] = _iopts
    else:
        _iopts = comm['iopts']
    _liopts_py = utils._EngineIntArrayType.get_shape(comm['iopts'], {'fun_name': fun_name, 'entity': 'comm[\'iopts\']'}, exp_rank=1, validate=False)[0]
    _liopts = utils._EngineIntScalarType(
        _liopts_py, {'fun_name': fun_name, 'entity': '_liopts'},
    ).to_ctypes()
    if _comm_needs_init:
        _opts = utils._EngineFloat64ArrayType.empty_ctypes(100)
        comm['opts'] = _opts
    else:
        _opts = comm['opts']
    _lopts_py = utils._EngineFloat64ArrayType.get_shape(comm['opts'], {'fun_name': fun_name, 'entity': 'comm[\'opts\']'}, exp_rank=1, validate=False)[0]
    _lopts = utils._EngineIntScalarType(
        _lopts_py, {'fun_name': fun_name, 'entity': '_lopts'},
    ).to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _optstr,
        _iopts,
        _liopts,
        _opts,
        _lopts,
        _errbuf,
        _ierr,
        _optstr._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        _optstr_py = utils._EngineCharScalarType.to_py(_optstr)
        if _ierr_py == _nag_e_consts.IERR_OPT_UNKNOWN and _ifmt == _nag_e_consts.IFMT_OPT_UNKNOWN:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the option in optstr was not recognized:',
                    'optstr = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_optstr_py),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_OPT_NO_EQUAL and _ifmt == _nag_e_consts.IFMT_OPT_NO_EQUAL:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the expected delimiter \'=\' was not found in optstr:',
                    'optstr = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_optstr_py),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_OPT_NON_NUMERIC and _ifmt == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, could not convert the specified optval to an integer:',
                    'optstr = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_optstr_py),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_OPT_NON_NUMERIC and _ifmt == 3:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, could not convert the specified optval to a real:',
                    'optstr = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_optstr_py),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_OPT_RNAME and _ifmt == _nag_e_consts.IFMT_OPT_RNAME:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, attempting to initialize the option arrays but',
                    'specified function name was not valid:',
                    'name = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_OPT_IVALUE and _ifmt == _nag_e_consts.IERR_OPT_IVALUE:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the optval supplied for the integer option is not',
                    'valid.',
                    'optstr = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_optstr_py),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_OPT_RVALUE and _ifmt == _nag_e_consts.IERR_OPT_RVALUE:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the optval supplied for the real option is not valid.',
                    'optstr = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_optstr_py),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_OPT_CVALUE and _ifmt == _nag_e_consts.IFMT_OPT_CVALUE_NOREC:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the optval supplied for the character option is not',
                    'valid.',
                    'optstr = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_optstr_py),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_OPTS_NOT_INIT and _ifmt == _nag_e_consts.IFMT_OPTS_NOT_INIT:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, either the option arrays have not been initialized or they have',
                    'been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_OPTS_CORRUPT and _ifmt == _nag_e_consts.IFMT_OPTS_NOT_LINKED:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, either the option arrays have not been initialized or they have',
                    'been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_OPTS_CORRUPT and _ifmt == _nag_e_consts.IFMT_OPTS_CORRUPT:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, either the option arrays have not been initialized or they have',
                    'been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def opt_get(optstr, comm):
    u"""
    Option getting function.

    For full information please refer to the NAG Library document for
    d01zl

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d01/d01zlf.html

    Parameters
    ----------
    optstr : str
        A string identifying the option whose current value is required.
        See [Optional Parameters] for information on valid options. In
        addition, the following is a valid option:

        Identify

          In which case ``opt_get`` returns in `cvalue` the 6 character
          function name supplied to ``opt_set`` when the option arrays
          `comm`['iopts'] and `comm`['opts'] were initialized.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``opt_set``.

    Returns
    -------
    ivalue : int
        If the option supplied in `optstr` is an integer valued
        parameter, `ivalue` will hold that value.

    rvalue : float
        If the option supplied in `optstr` is a real valued parameter,
        `rvalue` will hold that value.

    cvalue : str
        If the option supplied in `optstr` is a character valued
        parameter, this argument will hold that value. this argument
        will also contain additional information for some integer and
        real valued parameters, as indicated by `optype`.

    optype : int
        Indicates whether the option supplied in `optstr` is an integer,
        real or character valued parameter and hence which of `ivalue`,
        `rvalue` or `cvalue` holds the current value.

        `optype` = 1

          `optstr` is an integer valued option; its current value has
          been returned in `ivalue`.

        `optype` = 2

          `optstr` is a real valued option; its current value has been
          returned in `rvalue`.

        `optype` = 3

          `optstr` is a character valued option; its current value has
          been returned in `cvalue`.

        `optype` = 4

          `optstr` is an integer valued option; its current value has
          been returned in `ivalue`. Additional information has been
          returned in `cvalue`.

        `optype` = 5

          `optstr` is a real valued option; its current value has been
          returned in `rvalue`. Additional information has been returned
          in `cvalue`.

    Raises
    ------
    NagValueError
        (`errno` 61)
            The arrays `comm`['iopts'] and `comm`['opts'] have either
            not been initialized, have become corrupted, or are not
            compatible with this option setting function.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 11)
            On entry, the option in `optstr` was not recognized:
            `optstr` = *<value>*.
    """

    from ..base import _nag_d_consts
    from ..base import _nag_e_consts

    excs = []
    fun_name = 'naginterfaces.base.quad.opt_get'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d01zlft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, optstr(len=*).
        utils._BYREF_INT, # Output, ivalue.
        utils._BYREF_FLOAT64, # Output, rvalue.
        utils._BYREF_STRING, # Output, cvalue(len=*).
        utils._BYREF_INT, # Output, optype.
        utils._EngineIntArrayType.c_type, # Input, iopts[liopts].
        utils._BYREF_INT, # Input, liopts.
        utils._EngineFloat64ArrayType.c_type, # Input, opts[lopts].
        utils._BYREF_INT, # Input, lopts.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of optstr.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cvalue.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _optstr = utils._EngineCharScalarType(
        optstr, {'fun_name': fun_name, 'entity': 'optstr'}, exp_chrlen=None,
    ).to_ctypes()
    _ivalue = utils._EngineIntScalarType.empty_ctypes()
    _rvalue = utils._EngineFloat64ScalarType.empty_ctypes()
    _cvalue = utils._EngineCharScalarType.empty_ctypes(chrlen=39)
    _optype = utils._EngineIntScalarType.empty_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('iopts', 'opts'),
    )
    _liopts_py = utils._EngineIntArrayType.get_shape(comm['iopts'], {'fun_name': fun_name, 'entity': 'comm[\'iopts\']'}, exp_rank=1)[0]
    _liopts = utils._EngineIntScalarType(
        _liopts_py, {'fun_name': fun_name, 'entity': '_liopts'},
    ).to_ctypes()
    _iopts = comm['iopts']
    _lopts_py = utils._EngineFloat64ArrayType.get_shape(comm['opts'], {'fun_name': fun_name, 'entity': 'comm[\'opts\']'}, exp_rank=1)[0]
    _lopts = utils._EngineIntScalarType(
        _lopts_py, {'fun_name': fun_name, 'entity': '_lopts'},
    ).to_ctypes()
    _opts = comm['opts']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _optstr,
        _ivalue,
        _rvalue,
        _cvalue,
        _optype,
        _iopts,
        _liopts,
        _opts,
        _lopts,
        _errbuf,
        _ierr,
        _optstr._length_,
        _cvalue._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        _optstr_py = utils._EngineCharScalarType.to_py(_optstr)
        if _ierr_py == _nag_e_consts.IERR_OPT_UNKNOWN and _ifmt == _nag_e_consts.IFMT_OPT_UNKNOWN:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the option in optstr was not recognized:',
                    'optstr = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_optstr_py),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_OPTS_CORRUPT_GETTER and _ifmt == _nag_e_consts.IFMT_OPTS_NOT_INIT:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The arrays comm[\'iopts\'] and comm[\'opts\'] have either not been initialized,',
                    'have become corrupted, or are not compatible with this option',
                    'setting function.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_OPTS_CORRUPT_GETTER and _ifmt == _nag_e_consts.IFMT_OPTS_TOO_SMALL:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The arrays comm[\'iopts\'] and comm[\'opts\'] have either not been initialized,',
                    'have become corrupted, or are not compatible with this option',
                    'setting function.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_OPTS_CORRUPT_GETTER and _ifmt == _nag_e_consts.IFMT_OPT_NOMATCH:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The arrays comm[\'iopts\'] and comm[\'opts\'] have either not been initialized,',
                    'have become corrupted, or are not compatible with this option',
                    'setting function.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_OPTS_CORRUPT_GETTER and _ifmt == _nag_e_consts.IFMT_OPTS_NOT_LINKED:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The arrays comm[\'iopts\'] and comm[\'opts\'] have either not been initialized,',
                    'have become corrupted, or are not compatible with this option',
                    'setting function.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == _nag_e_consts.IERR_OPTS_CORRUPT_GETTER and _ifmt == _nag_e_consts.IFMT_OPTS_CORRUPT:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The arrays comm[\'iopts\'] and comm[\'opts\'] have either not been initialized,',
                    'have become corrupted, or are not compatible with this option',
                    'setting function.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ivalue_py = utils._EngineIntScalarType.to_py(_ivalue)
        _rvalue_py = utils._EngineFloat64ScalarType.to_py(_rvalue)
        _optype_py = utils._EngineIntScalarType.to_py(_optype)
        if _optype_py == _nag_e_consts.OPT_TYPE_IDENTIFY:
            _optype_py = _nag_e_consts.OPT_TYPE_CHAR
            if _ivalue_py == _nag_d_consts.D01ES_SOLVER_ID:
                _cvalue_py = 'd01es'
            elif _ivalue_py == _nag_d_consts.D01RA_SOLVER_ID:
                _cvalue_py = 'd01ra'
            else:
                utils._EngineErrorExit(
                    fun_name=fun_name,
                    msg_lines=[
                        'The communication structure comm has either not been initialized,',
                        'has become corrupted, or is not compatible with this option',
                        'setting function.',
                    ],
                    errcodes=(_nag_e_consts.IERR_OPTS_CORRUPT_GETTER,),
                ).manage()
            _ivalue_py = 0
        else:
            _cvalue_py = utils._EngineCharScalarType.to_py(_cvalue)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ivalue', 'rvalue', 'cvalue', 'optype'],
        )(
            _ivalue_py,
            _rvalue_py,
            _cvalue_py,
            _optype_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data
