# -*- coding: utf-8 -*-
# pylint: disable=bare-except,chained-comparison,invalid-name,len-as-condition,line-too-long,protected-access,redefined-outer-name,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unneeded-not,unused-import
u"""
Interfaces for the NAG Mark 27.0 `wav` Chapter.

``wav`` - Wavelet Transforms

Base interfaces.

We do not intend for these lower-level base interfaces to be called
directly from user code. Please refer to the ``naginterfaces.library``
subpackage instead.

This module is concerned with the analysis of datasets (or functions or
operators) in terms of frequency and scale components using wavelet
transforms.
Wavelet transforms have been applied in many fields from time series
analysis to image processing and the localization in either frequency or
scale that they provide is useful for data compression or denoising.
In general the standard wavelet transform uses dilation and scaling of a
chosen function, psi(t), (called the mother wavelet) such that

    psi_{a,b}(t) = 1/(sqrt(a))psi((t-b)/a)

where a gives the scaling and b determines the translation.
Wavelet methods can be divided into continuous transforms and discrete
transforms.
In the continuous case, the pair a and b are real numbers with a>0.
For the discrete transform, a and b can be chosen as a = 2^(-j), b =
k2^(-j) for integers j, k

    psi_{j,k}(t) = 2^(j/2)psi(2^jt-k).

The continuous real valued, one-dimensional wavelet transform (CWT) is
included in this module.
The discrete wavelet transform (DWT) at a single level together with its
inverse and the multi-level DWT with inverse are also provided for one,
two and three dimensions.
The Maximal Overlap DWT (MODWT) together with its inverse and the
multi-level MODWT with inverse are provided for one dimension.
The choice of wavelet for CWT includes the Morlet wavelet and
derivatives of a Gaussian while the DWT and MODWT offer the orthogonal
wavelets of Daubechies and a selection of biorthogonal wavelets.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09intro.html
"""

# NAG Copyright 2017-2019.

import collections as _collections
import sys as _sys

from ..base import utils

def dim1_init(wavnam, wtrans, mode, n):
    u"""
    One-dimensional wavelet filter initialization.

    For full information please refer to the NAG Library document for
    c09aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09aaf.html

    Parameters
    ----------
    wavnam : str
        The name of the mother wavelet. See [the C09 Introduction] for
        details.

        `wavnam` = 'HAAR'

          Haar wavelet, also known as 'DB1' as a special case of the
          Daubechies wavelet.

        `wavnam` = 'DBn', where n = 2,3,...,38

          Daubechies wavelet with n vanishing moments (2n coefficients).
          For example, `wavnam` = 'DB4' is the name for the Daubechies
          wavelet with 4 vanishing moments (8 coefficients).

        `wavnam` = 'COIFn', where n = 1,2,...,17

          Coiflet wavelet of order n.

        `wavnam` = 'BEYL'

          Beylkin wavelet.

        `wavnam` = 'VAID'

          Vaidyanathan wavelet.

        `wavnam` = 'SYMn', where n = 2,3,...,20

          Symlet wavelet of order n.

        `wavnam` = 'BIORx.y', where x.y can be one of 1.1, 1.3, 1.5,
        2.2, 2.4, 2.6, 2.8, 3.1, 3.3, 3.5, 3.7, 3.9, 4.4, 5.5 or 6.8

          Biorthogonal wavelet of order x.y. For example `wavnam` =
          'BIOR3.1' is the name for the biorthogonal wavelet of order
          3.1.

        `wavnam` = 'RBIOx.y', where x.y can be one of 1.1, 1.3, 1.5,
        2.2, 2.4, 2.6, 2.8, 3.1, 3.3, 3.5, 3.7, 3.9, 4.4, 5.5 or 6.8

          Reverse biorthogonal wavelet of order x.y. For example
          `wavnam` = 'RBIO3.1' is the name for the reverse biorthogonal
          wavelet of order 3.1.

    wtrans : str, length 1
        The type of discrete wavelet transform that is to be applied.

        `wtrans` = 'S'

          Single-level decomposition or reconstruction by discrete
          wavelet transform.

        `wtrans` = 'M'

          Multiresolution, by a multi-level DWT or its inverse.

        `wtrans` = 'T'

          Single-level decomposition or reconstruction by maximal
          overlap discrete wavelet transform.

        `wtrans` = 'U'

          Multi-level resolution by a maximal overlap discrete wavelet
          transform or its inverse.

    mode : str, length 1
        The end extension method. Note that only periodic end extension
        is currently available for the MODWT.

        `mode` = 'P'

          Periodic end extension.

        `mode` = 'H'

          Half-point symmetric end extension.

        `mode` = 'W'

          Whole-point symmetric end extension.

        `mode` = 'Z'

          Zero end extension.

    n : int
        The number of elements, n, in the input data array, x.

    Returns
    -------
    nwlmax : int
        The maximum number of levels of resolution, l_max, that can be
        computed when a multi-level discrete wavelet transform is
        applied. It is such that 2^(l_max) <= n < 2^(l_max+1), for l_max
        an integer.

    nf : int
        The filter length, n_f, for the supplied mother wavelet. This is
        used to determine the number of coefficients to be generated by
        the chosen transform.

    nwc : int
        For a single-level transform (`wtrans` = 'S' or 'T'), the number
        of approximation coefficients that would be generated for the
        given problem size, mother wavelet, extension method and type of
        transform; this is also the corresponding number of detail
        coefficients. For a multi-level transform (`wtrans` = 'M' or
        'U') the total number of coefficients that would be generated
        over l_max levels and with ikeep = 1 for MODWT.

    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `wavnam` not recognised: `wavnam` = *<value>*.

        (`errno` 2)
            On entry, `wtrans` = *<value>*.

            Constraint: `wtrans` = 'S', 'M', 'T' or 'U'.

        (`errno` 3)
            On entry, `mode` = *<value>*.

            Constraint: `mode` = 'P', 'H', 'W' or 'Z'.

        (`errno` 3)
            On entry, `wtrans` = 'T' or 'U' and `mode` != 'P'.

            Constraint: `mode` = 'P' when `wtrans` = 'T' or 'U'.

        (`errno` 4)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 2.
    """

    excs = []
    fun_name = 'naginterfaces.base.wav.dim1_init'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c09aaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, wavid.
        utils._BYREF_INT, # Input, itrans.
        utils._BYREF_INT, # Input, lmode.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Output, nwlmax.
        utils._BYREF_INT, # Output, nf.
        utils._BYREF_INT, # Output, nwc.
        utils._EngineIntArrayType.c_type, # Output, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _wavnam = utils._EngineCharScalarType(
        wavnam, {'fun_name': fun_name, 'entity': 'wavnam'}, exp_chrlen=None,
    ).to_ctypes()
    _wavnam_py = utils._EngineCharScalarType.to_py(_wavnam)
    if _wavnam_py.upper() == 'HAAR':
        _wavid = utils._EngineIntScalarType.c_type(101)
    elif _wavnam_py.upper() == 'DB1':
        _wavid = utils._EngineIntScalarType.c_type(101)
    elif _wavnam_py.upper() == 'DB2':
        _wavid = utils._EngineIntScalarType.c_type(102)
    elif _wavnam_py.upper() == 'DB3':
        _wavid = utils._EngineIntScalarType.c_type(103)
    elif _wavnam_py.upper() == 'DB4':
        _wavid = utils._EngineIntScalarType.c_type(104)
    elif _wavnam_py.upper() == 'DB5':
        _wavid = utils._EngineIntScalarType.c_type(105)
    elif _wavnam_py.upper() == 'DB6':
        _wavid = utils._EngineIntScalarType.c_type(106)
    elif _wavnam_py.upper() == 'DB7':
        _wavid = utils._EngineIntScalarType.c_type(107)
    elif _wavnam_py.upper() == 'DB8':
        _wavid = utils._EngineIntScalarType.c_type(108)
    elif _wavnam_py.upper() == 'DB9':
        _wavid = utils._EngineIntScalarType.c_type(109)
    elif _wavnam_py.upper() == 'DB10':
        _wavid = utils._EngineIntScalarType.c_type(110)
    elif _wavnam_py.upper() == 'DB11':
        _wavid = utils._EngineIntScalarType.c_type(111)
    elif _wavnam_py.upper() == 'DB12':
        _wavid = utils._EngineIntScalarType.c_type(112)
    elif _wavnam_py.upper() == 'DB13':
        _wavid = utils._EngineIntScalarType.c_type(113)
    elif _wavnam_py.upper() == 'DB14':
        _wavid = utils._EngineIntScalarType.c_type(114)
    elif _wavnam_py.upper() == 'DB15':
        _wavid = utils._EngineIntScalarType.c_type(115)
    elif _wavnam_py.upper() == 'DB16':
        _wavid = utils._EngineIntScalarType.c_type(116)
    elif _wavnam_py.upper() == 'DB17':
        _wavid = utils._EngineIntScalarType.c_type(117)
    elif _wavnam_py.upper() == 'DB18':
        _wavid = utils._EngineIntScalarType.c_type(118)
    elif _wavnam_py.upper() == 'DB19':
        _wavid = utils._EngineIntScalarType.c_type(119)
    elif _wavnam_py.upper() == 'DB20':
        _wavid = utils._EngineIntScalarType.c_type(120)
    elif _wavnam_py.upper() == 'DB21':
        _wavid = utils._EngineIntScalarType.c_type(121)
    elif _wavnam_py.upper() == 'DB22':
        _wavid = utils._EngineIntScalarType.c_type(122)
    elif _wavnam_py.upper() == 'DB23':
        _wavid = utils._EngineIntScalarType.c_type(123)
    elif _wavnam_py.upper() == 'DB24':
        _wavid = utils._EngineIntScalarType.c_type(124)
    elif _wavnam_py.upper() == 'DB25':
        _wavid = utils._EngineIntScalarType.c_type(125)
    elif _wavnam_py.upper() == 'DB26':
        _wavid = utils._EngineIntScalarType.c_type(126)
    elif _wavnam_py.upper() == 'DB27':
        _wavid = utils._EngineIntScalarType.c_type(127)
    elif _wavnam_py.upper() == 'DB28':
        _wavid = utils._EngineIntScalarType.c_type(128)
    elif _wavnam_py.upper() == 'DB29':
        _wavid = utils._EngineIntScalarType.c_type(129)
    elif _wavnam_py.upper() == 'DB30':
        _wavid = utils._EngineIntScalarType.c_type(130)
    elif _wavnam_py.upper() == 'DB31':
        _wavid = utils._EngineIntScalarType.c_type(131)
    elif _wavnam_py.upper() == 'DB32':
        _wavid = utils._EngineIntScalarType.c_type(132)
    elif _wavnam_py.upper() == 'DB33':
        _wavid = utils._EngineIntScalarType.c_type(133)
    elif _wavnam_py.upper() == 'DB34':
        _wavid = utils._EngineIntScalarType.c_type(134)
    elif _wavnam_py.upper() == 'DB35':
        _wavid = utils._EngineIntScalarType.c_type(135)
    elif _wavnam_py.upper() == 'DB36':
        _wavid = utils._EngineIntScalarType.c_type(136)
    elif _wavnam_py.upper() == 'DB37':
        _wavid = utils._EngineIntScalarType.c_type(137)
    elif _wavnam_py.upper() == 'DB38':
        _wavid = utils._EngineIntScalarType.c_type(138)
    elif _wavnam_py.upper() == 'COIF1':
        _wavid = utils._EngineIntScalarType.c_type(161)
    elif _wavnam_py.upper() == 'COIF2':
        _wavid = utils._EngineIntScalarType.c_type(162)
    elif _wavnam_py.upper() == 'COIF3':
        _wavid = utils._EngineIntScalarType.c_type(163)
    elif _wavnam_py.upper() == 'COIF4':
        _wavid = utils._EngineIntScalarType.c_type(164)
    elif _wavnam_py.upper() == 'COIF5':
        _wavid = utils._EngineIntScalarType.c_type(165)
    elif _wavnam_py.upper() == 'COIF6':
        _wavid = utils._EngineIntScalarType.c_type(166)
    elif _wavnam_py.upper() == 'COIF7':
        _wavid = utils._EngineIntScalarType.c_type(167)
    elif _wavnam_py.upper() == 'COIF8':
        _wavid = utils._EngineIntScalarType.c_type(168)
    elif _wavnam_py.upper() == 'COIF9':
        _wavid = utils._EngineIntScalarType.c_type(169)
    elif _wavnam_py.upper() == 'COIF10':
        _wavid = utils._EngineIntScalarType.c_type(170)
    elif _wavnam_py.upper() == 'COIF11':
        _wavid = utils._EngineIntScalarType.c_type(171)
    elif _wavnam_py.upper() == 'COIF12':
        _wavid = utils._EngineIntScalarType.c_type(172)
    elif _wavnam_py.upper() == 'COIF13':
        _wavid = utils._EngineIntScalarType.c_type(173)
    elif _wavnam_py.upper() == 'COIF14':
        _wavid = utils._EngineIntScalarType.c_type(174)
    elif _wavnam_py.upper() == 'COIF15':
        _wavid = utils._EngineIntScalarType.c_type(175)
    elif _wavnam_py.upper() == 'COIF16':
        _wavid = utils._EngineIntScalarType.c_type(176)
    elif _wavnam_py.upper() == 'COIF17':
        _wavid = utils._EngineIntScalarType.c_type(177)
    elif _wavnam_py.upper() == 'BEYL':
        _wavid = utils._EngineIntScalarType.c_type(201)
    elif _wavnam_py.upper() == 'VAID':
        _wavid = utils._EngineIntScalarType.c_type(206)
    elif _wavnam_py.upper() == 'SYM2':
        _wavid = utils._EngineIntScalarType.c_type(222)
    elif _wavnam_py.upper() == 'SYM3':
        _wavid = utils._EngineIntScalarType.c_type(223)
    elif _wavnam_py.upper() == 'SYM4':
        _wavid = utils._EngineIntScalarType.c_type(224)
    elif _wavnam_py.upper() == 'SYM5':
        _wavid = utils._EngineIntScalarType.c_type(225)
    elif _wavnam_py.upper() == 'SYM6':
        _wavid = utils._EngineIntScalarType.c_type(226)
    elif _wavnam_py.upper() == 'SYM7':
        _wavid = utils._EngineIntScalarType.c_type(227)
    elif _wavnam_py.upper() == 'SYM8':
        _wavid = utils._EngineIntScalarType.c_type(228)
    elif _wavnam_py.upper() == 'SYM9':
        _wavid = utils._EngineIntScalarType.c_type(229)
    elif _wavnam_py.upper() == 'SYM10':
        _wavid = utils._EngineIntScalarType.c_type(230)
    elif _wavnam_py.upper() == 'SYM11':
        _wavid = utils._EngineIntScalarType.c_type(231)
    elif _wavnam_py.upper() == 'SYM12':
        _wavid = utils._EngineIntScalarType.c_type(232)
    elif _wavnam_py.upper() == 'SYM13':
        _wavid = utils._EngineIntScalarType.c_type(233)
    elif _wavnam_py.upper() == 'SYM14':
        _wavid = utils._EngineIntScalarType.c_type(234)
    elif _wavnam_py.upper() == 'SYM15':
        _wavid = utils._EngineIntScalarType.c_type(235)
    elif _wavnam_py.upper() == 'SYM16':
        _wavid = utils._EngineIntScalarType.c_type(236)
    elif _wavnam_py.upper() == 'SYM17':
        _wavid = utils._EngineIntScalarType.c_type(237)
    elif _wavnam_py.upper() == 'SYM18':
        _wavid = utils._EngineIntScalarType.c_type(238)
    elif _wavnam_py.upper() == 'SYM19':
        _wavid = utils._EngineIntScalarType.c_type(239)
    elif _wavnam_py.upper() == 'SYM20':
        _wavid = utils._EngineIntScalarType.c_type(240)
    elif _wavnam_py.upper() == 'BIOR1.1':
        _wavid = utils._EngineIntScalarType.c_type(311)
    elif _wavnam_py.upper() == 'BIOR1.3':
        _wavid = utils._EngineIntScalarType.c_type(313)
    elif _wavnam_py.upper() == 'BIOR1.5':
        _wavid = utils._EngineIntScalarType.c_type(315)
    elif _wavnam_py.upper() == 'BIOR2.2':
        _wavid = utils._EngineIntScalarType.c_type(322)
    elif _wavnam_py.upper() == 'BIOR2.4':
        _wavid = utils._EngineIntScalarType.c_type(324)
    elif _wavnam_py.upper() == 'BIOR2.6':
        _wavid = utils._EngineIntScalarType.c_type(326)
    elif _wavnam_py.upper() == 'BIOR2.8':
        _wavid = utils._EngineIntScalarType.c_type(328)
    elif _wavnam_py.upper() == 'BIOR3.1':
        _wavid = utils._EngineIntScalarType.c_type(331)
    elif _wavnam_py.upper() == 'BIOR3.3':
        _wavid = utils._EngineIntScalarType.c_type(333)
    elif _wavnam_py.upper() == 'BIOR3.5':
        _wavid = utils._EngineIntScalarType.c_type(335)
    elif _wavnam_py.upper() == 'BIOR3.7':
        _wavid = utils._EngineIntScalarType.c_type(337)
    elif _wavnam_py.upper() == 'BIOR3.9':
        _wavid = utils._EngineIntScalarType.c_type(339)
    elif _wavnam_py.upper() == 'BIOR4.4':
        _wavid = utils._EngineIntScalarType.c_type(344)
    elif _wavnam_py.upper() == 'BIOR5.5':
        _wavid = utils._EngineIntScalarType.c_type(355)
    elif _wavnam_py.upper() == 'BIOR6.8':
        _wavid = utils._EngineIntScalarType.c_type(368)
    elif _wavnam_py.upper() == 'RBIO1.1':
        _wavid = utils._EngineIntScalarType.c_type(411)
    elif _wavnam_py.upper() == 'RBIO1.3':
        _wavid = utils._EngineIntScalarType.c_type(413)
    elif _wavnam_py.upper() == 'RBIO1.5':
        _wavid = utils._EngineIntScalarType.c_type(415)
    elif _wavnam_py.upper() == 'RBIO2.2':
        _wavid = utils._EngineIntScalarType.c_type(422)
    elif _wavnam_py.upper() == 'RBIO2.4':
        _wavid = utils._EngineIntScalarType.c_type(424)
    elif _wavnam_py.upper() == 'RBIO2.6':
        _wavid = utils._EngineIntScalarType.c_type(426)
    elif _wavnam_py.upper() == 'RBIO2.8':
        _wavid = utils._EngineIntScalarType.c_type(428)
    elif _wavnam_py.upper() == 'RBIO3.1':
        _wavid = utils._EngineIntScalarType.c_type(431)
    elif _wavnam_py.upper() == 'RBIO3.3':
        _wavid = utils._EngineIntScalarType.c_type(433)
    elif _wavnam_py.upper() == 'RBIO3.5':
        _wavid = utils._EngineIntScalarType.c_type(435)
    elif _wavnam_py.upper() == 'RBIO3.7':
        _wavid = utils._EngineIntScalarType.c_type(437)
    elif _wavnam_py.upper() == 'RBIO3.9':
        _wavid = utils._EngineIntScalarType.c_type(439)
    elif _wavnam_py.upper() == 'RBIO4.4':
        _wavid = utils._EngineIntScalarType.c_type(444)
    elif _wavnam_py.upper() == 'RBIO5.5':
        _wavid = utils._EngineIntScalarType.c_type(455)
    elif _wavnam_py.upper() == 'RBIO6.8':
        _wavid = utils._EngineIntScalarType.c_type(468)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, wavnam not recognised: wavnam = {:s}.'.format(
                    utils._STR_FMT_STR.format(_wavnam_py),
                ),
            ],
            errcodes=(1,),
        )
        _err_class.manage()
    _wtrans = utils._EngineCharScalarType(
        wtrans, {'fun_name': fun_name, 'entity': 'wtrans'}, exp_chrlen=1,
    ).to_ctypes()
    _wtrans_py = utils._EngineCharScalarType.to_py(_wtrans)
    if _wtrans_py.upper() == 'S':
        _itrans = utils._EngineIntScalarType.c_type(1)
    elif _wtrans_py.upper() == 'M':
        _itrans = utils._EngineIntScalarType.c_type(2)
    elif _wtrans_py.upper() == 'T':
        _itrans = utils._EngineIntScalarType.c_type(3)
    elif _wtrans_py.upper() == 'U':
        _itrans = utils._EngineIntScalarType.c_type(4)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, wtrans = {:s}.'.format(
                    utils._STR_FMT_STR.format(_wtrans_py),
                ),
                'Constraint: wtrans = \'S\', \'M\', \'T\' or \'U\'.',
            ],
            errcodes=(2,),
        )
        _err_class.manage()
    _mode = utils._EngineCharScalarType(
        mode, {'fun_name': fun_name, 'entity': 'mode'}, exp_chrlen=1,
    ).to_ctypes()
    _mode_py = utils._EngineCharScalarType.to_py(_mode)
    if _mode_py.upper() == 'H':
        _lmode = utils._EngineIntScalarType.c_type(2)
    elif _mode_py.upper() == 'W':
        _lmode = utils._EngineIntScalarType.c_type(3)
    elif _mode_py.upper() == 'P':
        _lmode = utils._EngineIntScalarType.c_type(1)
    elif _mode_py.upper() == 'Z':
        _lmode = utils._EngineIntScalarType.c_type(4)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, mode = {:s}.'.format(
                    utils._STR_FMT_STR.format(_mode_py),
                ),
                'Constraint: mode = \'P\', \'H\', \'W\' or \'Z\'.',
            ],
            errcodes=(3,),
        )
        _err_class.manage()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nwlmax = utils._EngineIntScalarType.empty_ctypes()
    _nf = utils._EngineIntScalarType.empty_ctypes()
    _nwc = utils._EngineIntScalarType.empty_ctypes()
    _licomm = utils._EngineIntScalarType.c_type(100)
    _icomm = utils._EngineIntArrayType.empty_ctypes(100)
    _comm_py = {}
    _comm_py['icomm'] = _icomm
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _wavid,
        _itrans,
        _lmode,
        _n,
        _nwlmax,
        _nf,
        _nwc,
        _icomm,
        _licomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 3 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, wtrans = \'T\' or \'U\' and mode != \'P\'.',
                    'Constraint: mode = \'P\' when wtrans = \'T\' or \'U\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _nwlmax_py = utils._EngineIntScalarType.to_py(_nwlmax)
        _nf_py = utils._EngineIntScalarType.to_py(_nf)
        _nwc_py = utils._EngineIntScalarType.to_py(_nwc)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['nwlmax', 'nf', 'nwc', 'comm'],
        )(
            _nwlmax_py,
            _nf_py,
            _nwc_py,
            _comm_py,
        )
    _err_class.manage()
    return return_data

def dim2_init(wavnam, wtrans, mode, m, n):
    u"""
    Two-dimensional wavelet filter initialization.

    For full information please refer to the NAG Library document for
    c09ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09abf.html

    Parameters
    ----------
    wavnam : str
        The name of the mother wavelet. See [the C09 Introduction] for
        details.

        `wavnam` = 'HAAR'

          Haar wavelet, also known as 'DB1' as a special case of the
          Daubechies wavelet.

        `wavnam` = 'DBn', where n = 2,3,...,38

          Daubechies wavelet with n vanishing moments (2n coefficients).
          For example, `wavnam` = 'DB4' is the name for the Daubechies
          wavelet with 4 vanishing moments (8 coefficients).

        `wavnam` = 'COIFn', where n = 1,2,...,17

          Coiflet wavelet of order n.

        `wavnam` = 'BEYL'

          Beylkin wavelet.

        `wavnam` = 'VAID'

          Vaidyanathan wavelet.

        `wavnam` = 'SYMn', where n = 2,3,...,20

          Symlet wavelet of order n.

        `wavnam` = 'BIORx.y', where x.y can be one of 1.1, 1.3, 1.5,
        2.2, 2.4, 2.6, 2.8, 3.1, 3.3, 3.5, 3.7, 3.9, 4.4, 5.5 or 6.8

          Biorthogonal wavelet of order x.y. For example `wavnam` =
          'BIOR3.1' is the name for the biorthogonal wavelet of order
          3.1.

        `wavnam` = 'RBIOx.y', where x.y can be one of 1.1, 1.3, 1.5,
        2.2, 2.4, 2.6, 2.8, 3.1, 3.3, 3.5, 3.7, 3.9, 4.4, 5.5 or 6.8

          Reverse biorthogonal wavelet of order x.y. For example
          `wavnam` = 'RBIO3.1' is the name for the reverse biorthogonal
          wavelet of order 3.1.

    wtrans : str, length 1
        The type of discrete wavelet transform that is to be applied.

        `wtrans` = 'S'

          Single-level decomposition or reconstruction by discrete
          wavelet transform.

        `wtrans` = 'M'

          Multiresolution, by a multi-level DWT or its inverse.

    mode : str, length 1
        The end extension method.

        `mode` = 'P'

          Periodic end extension.

        `mode` = 'H'

          Half-point symmetric end extension.

        `mode` = 'W'

          Whole-point symmetric end extension.

        `mode` = 'Z'

          Zero end extension.

    m : int
        The number of elements, m, in the first dimension (number of
        rows of data matrix A) of the input data.

    n : int
        The number of elements, n, in the second dimension (number of
        columns of data matrix A) of the input data.

    Returns
    -------
    nwlmax : int
        The maximum number of levels of resolution, l_max, that can be
        computed if a multi-level discrete wavelet transform is applied
        (`wtrans` = 'M'). It is such that 2^(l_max) <= min(m, n) <
        2^(l_max+1), for l_max an integer.

        If `wtrans` = 'S', `nwlmax` is not set.

    nf : int
        The filter length, n_f, for the supplied mother wavelet. This is
        used to determine the number of coefficients to be generated by
        the chosen transform.

    nwct : int
        The total number of wavelet coefficients, n_ct, that will be
        generated. When `wtrans` = 'S' the number of rows required in
        each of the output coefficient matrices can be calculated as
        n_cm = n_ct/(4n_cn). When `wtrans` = 'M' the length of the array
        used to store all of the coefficient matrices must be at least
        n_ct.

    nwcn : int
        For a single-level transform (`wtrans` = 'S'), the number of
        coefficients that would be generated in the second dimension,
        n_cn, for each coefficient type. For a multi-level transform
        (`wtrans` = 'M') this is set to 1.

    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `wavnam` not recognised: `wavnam` = *<value>*.

        (`errno` 2)
            On entry, `wtrans` = *<value>*.

            Constraint: `wtrans` = 'S' or 'M'.

        (`errno` 3)
            On entry, `mode` = *<value>*.

            Constraint: `mode` = 'P', 'H', 'W' or 'Z'.

        (`errno` 4)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 2.

        (`errno` 4)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 2.
    """

    excs = []
    fun_name = 'naginterfaces.base.wav.dim2_init'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c09abft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, wavid.
        utils._BYREF_INT, # Input, itrans.
        utils._BYREF_INT, # Input, lmode.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Output, nwlmax.
        utils._BYREF_INT, # Output, nf.
        utils._BYREF_INT, # Output, nwct.
        utils._BYREF_INT, # Output, nwcn.
        utils._EngineIntArrayType.c_type, # Output, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _wavnam = utils._EngineCharScalarType(
        wavnam, {'fun_name': fun_name, 'entity': 'wavnam'}, exp_chrlen=None,
    ).to_ctypes()
    _wavnam_py = utils._EngineCharScalarType.to_py(_wavnam)
    if _wavnam_py.upper() == 'HAAR':
        _wavid = utils._EngineIntScalarType.c_type(101)
    elif _wavnam_py.upper() == 'DB1':
        _wavid = utils._EngineIntScalarType.c_type(101)
    elif _wavnam_py.upper() == 'DB2':
        _wavid = utils._EngineIntScalarType.c_type(102)
    elif _wavnam_py.upper() == 'DB3':
        _wavid = utils._EngineIntScalarType.c_type(103)
    elif _wavnam_py.upper() == 'DB4':
        _wavid = utils._EngineIntScalarType.c_type(104)
    elif _wavnam_py.upper() == 'DB5':
        _wavid = utils._EngineIntScalarType.c_type(105)
    elif _wavnam_py.upper() == 'DB6':
        _wavid = utils._EngineIntScalarType.c_type(106)
    elif _wavnam_py.upper() == 'DB7':
        _wavid = utils._EngineIntScalarType.c_type(107)
    elif _wavnam_py.upper() == 'DB8':
        _wavid = utils._EngineIntScalarType.c_type(108)
    elif _wavnam_py.upper() == 'DB9':
        _wavid = utils._EngineIntScalarType.c_type(109)
    elif _wavnam_py.upper() == 'DB10':
        _wavid = utils._EngineIntScalarType.c_type(110)
    elif _wavnam_py.upper() == 'DB11':
        _wavid = utils._EngineIntScalarType.c_type(111)
    elif _wavnam_py.upper() == 'DB12':
        _wavid = utils._EngineIntScalarType.c_type(112)
    elif _wavnam_py.upper() == 'DB13':
        _wavid = utils._EngineIntScalarType.c_type(113)
    elif _wavnam_py.upper() == 'DB14':
        _wavid = utils._EngineIntScalarType.c_type(114)
    elif _wavnam_py.upper() == 'DB15':
        _wavid = utils._EngineIntScalarType.c_type(115)
    elif _wavnam_py.upper() == 'DB16':
        _wavid = utils._EngineIntScalarType.c_type(116)
    elif _wavnam_py.upper() == 'DB17':
        _wavid = utils._EngineIntScalarType.c_type(117)
    elif _wavnam_py.upper() == 'DB18':
        _wavid = utils._EngineIntScalarType.c_type(118)
    elif _wavnam_py.upper() == 'DB19':
        _wavid = utils._EngineIntScalarType.c_type(119)
    elif _wavnam_py.upper() == 'DB20':
        _wavid = utils._EngineIntScalarType.c_type(120)
    elif _wavnam_py.upper() == 'DB21':
        _wavid = utils._EngineIntScalarType.c_type(121)
    elif _wavnam_py.upper() == 'DB22':
        _wavid = utils._EngineIntScalarType.c_type(122)
    elif _wavnam_py.upper() == 'DB23':
        _wavid = utils._EngineIntScalarType.c_type(123)
    elif _wavnam_py.upper() == 'DB24':
        _wavid = utils._EngineIntScalarType.c_type(124)
    elif _wavnam_py.upper() == 'DB25':
        _wavid = utils._EngineIntScalarType.c_type(125)
    elif _wavnam_py.upper() == 'DB26':
        _wavid = utils._EngineIntScalarType.c_type(126)
    elif _wavnam_py.upper() == 'DB27':
        _wavid = utils._EngineIntScalarType.c_type(127)
    elif _wavnam_py.upper() == 'DB28':
        _wavid = utils._EngineIntScalarType.c_type(128)
    elif _wavnam_py.upper() == 'DB29':
        _wavid = utils._EngineIntScalarType.c_type(129)
    elif _wavnam_py.upper() == 'DB30':
        _wavid = utils._EngineIntScalarType.c_type(130)
    elif _wavnam_py.upper() == 'DB31':
        _wavid = utils._EngineIntScalarType.c_type(131)
    elif _wavnam_py.upper() == 'DB32':
        _wavid = utils._EngineIntScalarType.c_type(132)
    elif _wavnam_py.upper() == 'DB33':
        _wavid = utils._EngineIntScalarType.c_type(133)
    elif _wavnam_py.upper() == 'DB34':
        _wavid = utils._EngineIntScalarType.c_type(134)
    elif _wavnam_py.upper() == 'DB35':
        _wavid = utils._EngineIntScalarType.c_type(135)
    elif _wavnam_py.upper() == 'DB36':
        _wavid = utils._EngineIntScalarType.c_type(136)
    elif _wavnam_py.upper() == 'DB37':
        _wavid = utils._EngineIntScalarType.c_type(137)
    elif _wavnam_py.upper() == 'DB38':
        _wavid = utils._EngineIntScalarType.c_type(138)
    elif _wavnam_py.upper() == 'COIF1':
        _wavid = utils._EngineIntScalarType.c_type(161)
    elif _wavnam_py.upper() == 'COIF2':
        _wavid = utils._EngineIntScalarType.c_type(162)
    elif _wavnam_py.upper() == 'COIF3':
        _wavid = utils._EngineIntScalarType.c_type(163)
    elif _wavnam_py.upper() == 'COIF4':
        _wavid = utils._EngineIntScalarType.c_type(164)
    elif _wavnam_py.upper() == 'COIF5':
        _wavid = utils._EngineIntScalarType.c_type(165)
    elif _wavnam_py.upper() == 'COIF6':
        _wavid = utils._EngineIntScalarType.c_type(166)
    elif _wavnam_py.upper() == 'COIF7':
        _wavid = utils._EngineIntScalarType.c_type(167)
    elif _wavnam_py.upper() == 'COIF8':
        _wavid = utils._EngineIntScalarType.c_type(168)
    elif _wavnam_py.upper() == 'COIF9':
        _wavid = utils._EngineIntScalarType.c_type(169)
    elif _wavnam_py.upper() == 'COIF10':
        _wavid = utils._EngineIntScalarType.c_type(170)
    elif _wavnam_py.upper() == 'COIF11':
        _wavid = utils._EngineIntScalarType.c_type(171)
    elif _wavnam_py.upper() == 'COIF12':
        _wavid = utils._EngineIntScalarType.c_type(172)
    elif _wavnam_py.upper() == 'COIF13':
        _wavid = utils._EngineIntScalarType.c_type(173)
    elif _wavnam_py.upper() == 'COIF14':
        _wavid = utils._EngineIntScalarType.c_type(174)
    elif _wavnam_py.upper() == 'COIF15':
        _wavid = utils._EngineIntScalarType.c_type(175)
    elif _wavnam_py.upper() == 'COIF16':
        _wavid = utils._EngineIntScalarType.c_type(176)
    elif _wavnam_py.upper() == 'COIF17':
        _wavid = utils._EngineIntScalarType.c_type(177)
    elif _wavnam_py.upper() == 'BEYL':
        _wavid = utils._EngineIntScalarType.c_type(201)
    elif _wavnam_py.upper() == 'VAID':
        _wavid = utils._EngineIntScalarType.c_type(206)
    elif _wavnam_py.upper() == 'SYM2':
        _wavid = utils._EngineIntScalarType.c_type(222)
    elif _wavnam_py.upper() == 'SYM3':
        _wavid = utils._EngineIntScalarType.c_type(223)
    elif _wavnam_py.upper() == 'SYM4':
        _wavid = utils._EngineIntScalarType.c_type(224)
    elif _wavnam_py.upper() == 'SYM5':
        _wavid = utils._EngineIntScalarType.c_type(225)
    elif _wavnam_py.upper() == 'SYM6':
        _wavid = utils._EngineIntScalarType.c_type(226)
    elif _wavnam_py.upper() == 'SYM7':
        _wavid = utils._EngineIntScalarType.c_type(227)
    elif _wavnam_py.upper() == 'SYM8':
        _wavid = utils._EngineIntScalarType.c_type(228)
    elif _wavnam_py.upper() == 'SYM9':
        _wavid = utils._EngineIntScalarType.c_type(229)
    elif _wavnam_py.upper() == 'SYM10':
        _wavid = utils._EngineIntScalarType.c_type(230)
    elif _wavnam_py.upper() == 'SYM11':
        _wavid = utils._EngineIntScalarType.c_type(231)
    elif _wavnam_py.upper() == 'SYM12':
        _wavid = utils._EngineIntScalarType.c_type(232)
    elif _wavnam_py.upper() == 'SYM13':
        _wavid = utils._EngineIntScalarType.c_type(233)
    elif _wavnam_py.upper() == 'SYM14':
        _wavid = utils._EngineIntScalarType.c_type(234)
    elif _wavnam_py.upper() == 'SYM15':
        _wavid = utils._EngineIntScalarType.c_type(235)
    elif _wavnam_py.upper() == 'SYM16':
        _wavid = utils._EngineIntScalarType.c_type(236)
    elif _wavnam_py.upper() == 'SYM17':
        _wavid = utils._EngineIntScalarType.c_type(237)
    elif _wavnam_py.upper() == 'SYM18':
        _wavid = utils._EngineIntScalarType.c_type(238)
    elif _wavnam_py.upper() == 'SYM19':
        _wavid = utils._EngineIntScalarType.c_type(239)
    elif _wavnam_py.upper() == 'SYM20':
        _wavid = utils._EngineIntScalarType.c_type(240)
    elif _wavnam_py.upper() == 'BIOR1.1':
        _wavid = utils._EngineIntScalarType.c_type(311)
    elif _wavnam_py.upper() == 'BIOR1.3':
        _wavid = utils._EngineIntScalarType.c_type(313)
    elif _wavnam_py.upper() == 'BIOR1.5':
        _wavid = utils._EngineIntScalarType.c_type(315)
    elif _wavnam_py.upper() == 'BIOR2.2':
        _wavid = utils._EngineIntScalarType.c_type(322)
    elif _wavnam_py.upper() == 'BIOR2.4':
        _wavid = utils._EngineIntScalarType.c_type(324)
    elif _wavnam_py.upper() == 'BIOR2.6':
        _wavid = utils._EngineIntScalarType.c_type(326)
    elif _wavnam_py.upper() == 'BIOR2.8':
        _wavid = utils._EngineIntScalarType.c_type(328)
    elif _wavnam_py.upper() == 'BIOR3.1':
        _wavid = utils._EngineIntScalarType.c_type(331)
    elif _wavnam_py.upper() == 'BIOR3.3':
        _wavid = utils._EngineIntScalarType.c_type(333)
    elif _wavnam_py.upper() == 'BIOR3.5':
        _wavid = utils._EngineIntScalarType.c_type(335)
    elif _wavnam_py.upper() == 'BIOR3.7':
        _wavid = utils._EngineIntScalarType.c_type(337)
    elif _wavnam_py.upper() == 'BIOR3.9':
        _wavid = utils._EngineIntScalarType.c_type(339)
    elif _wavnam_py.upper() == 'BIOR4.4':
        _wavid = utils._EngineIntScalarType.c_type(344)
    elif _wavnam_py.upper() == 'BIOR5.5':
        _wavid = utils._EngineIntScalarType.c_type(355)
    elif _wavnam_py.upper() == 'BIOR6.8':
        _wavid = utils._EngineIntScalarType.c_type(368)
    elif _wavnam_py.upper() == 'RBIO1.1':
        _wavid = utils._EngineIntScalarType.c_type(411)
    elif _wavnam_py.upper() == 'RBIO1.3':
        _wavid = utils._EngineIntScalarType.c_type(413)
    elif _wavnam_py.upper() == 'RBIO1.5':
        _wavid = utils._EngineIntScalarType.c_type(415)
    elif _wavnam_py.upper() == 'RBIO2.2':
        _wavid = utils._EngineIntScalarType.c_type(422)
    elif _wavnam_py.upper() == 'RBIO2.4':
        _wavid = utils._EngineIntScalarType.c_type(424)
    elif _wavnam_py.upper() == 'RBIO2.6':
        _wavid = utils._EngineIntScalarType.c_type(426)
    elif _wavnam_py.upper() == 'RBIO2.8':
        _wavid = utils._EngineIntScalarType.c_type(428)
    elif _wavnam_py.upper() == 'RBIO3.1':
        _wavid = utils._EngineIntScalarType.c_type(431)
    elif _wavnam_py.upper() == 'RBIO3.3':
        _wavid = utils._EngineIntScalarType.c_type(433)
    elif _wavnam_py.upper() == 'RBIO3.5':
        _wavid = utils._EngineIntScalarType.c_type(435)
    elif _wavnam_py.upper() == 'RBIO3.7':
        _wavid = utils._EngineIntScalarType.c_type(437)
    elif _wavnam_py.upper() == 'RBIO3.9':
        _wavid = utils._EngineIntScalarType.c_type(439)
    elif _wavnam_py.upper() == 'RBIO4.4':
        _wavid = utils._EngineIntScalarType.c_type(444)
    elif _wavnam_py.upper() == 'RBIO5.5':
        _wavid = utils._EngineIntScalarType.c_type(455)
    elif _wavnam_py.upper() == 'RBIO6.8':
        _wavid = utils._EngineIntScalarType.c_type(468)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, wavnam not recognised: wavnam = {:s}.'.format(
                    utils._STR_FMT_STR.format(_wavnam_py),
                ),
            ],
            errcodes=(1,),
        )
        _err_class.manage()
    _wtrans = utils._EngineCharScalarType(
        wtrans, {'fun_name': fun_name, 'entity': 'wtrans'}, exp_chrlen=1,
    ).to_ctypes()
    _wtrans_py = utils._EngineCharScalarType.to_py(_wtrans)
    if _wtrans_py.upper() == 'S':
        _itrans = utils._EngineIntScalarType.c_type(1)
    elif _wtrans_py.upper() == 'M':
        _itrans = utils._EngineIntScalarType.c_type(2)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, wtrans = {:s}.'.format(
                    utils._STR_FMT_STR.format(_wtrans_py),
                ),
                'Constraint: wtrans = \'S\' or \'M\'.',
            ],
            errcodes=(2,),
        )
        _err_class.manage()
    _mode = utils._EngineCharScalarType(
        mode, {'fun_name': fun_name, 'entity': 'mode'}, exp_chrlen=1,
    ).to_ctypes()
    _mode_py = utils._EngineCharScalarType.to_py(_mode)
    if _mode_py.upper() == 'H':
        _lmode = utils._EngineIntScalarType.c_type(2)
    elif _mode_py.upper() == 'W':
        _lmode = utils._EngineIntScalarType.c_type(3)
    elif _mode_py.upper() == 'P':
        _lmode = utils._EngineIntScalarType.c_type(1)
    elif _mode_py.upper() == 'Z':
        _lmode = utils._EngineIntScalarType.c_type(4)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, mode = {:s}.'.format(
                    utils._STR_FMT_STR.format(_mode_py),
                ),
                'Constraint: mode = \'P\', \'H\', \'W\' or \'Z\'.',
            ],
            errcodes=(3,),
        )
        _err_class.manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _nwlmax = utils._EngineIntScalarType.empty_ctypes()
    _nf = utils._EngineIntScalarType.empty_ctypes()
    _nwct = utils._EngineIntScalarType.empty_ctypes()
    _nwcn = utils._EngineIntScalarType.empty_ctypes()
    _licomm = utils._EngineIntScalarType.c_type(180)
    _icomm = utils._EngineIntArrayType.empty_ctypes(180)
    _comm_py = {}
    _comm_py['icomm'] = _icomm
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _wavid,
        _itrans,
        _lmode,
        _m,
        _n,
        _nwlmax,
        _nf,
        _nwct,
        _nwcn,
        _icomm,
        _licomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 104:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _nwlmax_py = utils._EngineIntScalarType.to_py(_nwlmax)
        _nf_py = utils._EngineIntScalarType.to_py(_nf)
        _nwct_py = utils._EngineIntScalarType.to_py(_nwct)
        _nwcn_py = utils._EngineIntScalarType.to_py(_nwcn)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['nwlmax', 'nf', 'nwct', 'nwcn', 'comm'],
        )(
            _nwlmax_py,
            _nf_py,
            _nwct_py,
            _nwcn_py,
            _comm_py,
        )
    _err_class.manage()
    return return_data

def dim3_init(wavnam, wtrans, mode, m, n, fr):
    u"""
    Three-dimensional wavelet filter initialization.

    For full information please refer to the NAG Library document for
    c09ac

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09acf.html

    Parameters
    ----------
    wavnam : str
        The name of the mother wavelet. See [the C09 Introduction] for
        details.

        `wavnam` = 'HAAR'

          Haar wavelet, also known as 'DB1' as a special case of the
          Daubechies wavelet.

        `wavnam` = 'DBn', where n = 2,3,...,38

          Daubechies wavelet with n vanishing moments (2n coefficients).
          For example, `wavnam` = 'DB4' is the name for the Daubechies
          wavelet with 4 vanishing moments (8 coefficients).

        `wavnam` = 'COIFn', where n = 1,2,...,17

          Coiflet wavelet of order n.

        `wavnam` = 'BEYL'

          Beylkin wavelet.

        `wavnam` = 'VAID'

          Vaidyanathan wavelet.

        `wavnam` = 'SYMn', where n = 2,3,...,20

          Symlet wavelet of order n.

        `wavnam` = 'BIORx.y', where x.y can be one of 1.1, 1.3, 1.5,
        2.2, 2.4, 2.6, 2.8, 3.1, 3.3, 3.5, 3.7, 3.9, 4.4, 5.5 or 6.8

          Biorthogonal wavelet of order x.y. For example `wavnam` =
          'BIOR3.1' is the name for the biorthogonal wavelet of order
          3.1.

        `wavnam` = 'RBIOx.y', where x.y can be one of 1.1, 1.3, 1.5,
        2.2, 2.4, 2.6, 2.8, 3.1, 3.3, 3.5, 3.7, 3.9, 4.4, 5.5 or 6.8

          Reverse biorthogonal wavelet of order x.y. For example
          `wavnam` = 'RBIO3.1' is the name for the reverse biorthogonal
          wavelet of order 3.1.

    wtrans : str, length 1
        The type of discrete wavelet transform that is to be applied.

        `wtrans` = 'S'

          Single-level decomposition or reconstruction by discrete
          wavelet transform.

        `wtrans` = 'M'

          Multiresolution, by a multi-level DWT or its inverse.

    mode : str, length 1
        The end extension method.

        `mode` = 'P'

          Periodic end extension.

        `mode` = 'H'

          Half-point symmetric end extension.

        `mode` = 'W'

          Whole-point symmetric end extension.

        `mode` = 'Z'

          Zero end extension.

    m : int
        The number of elements, m, in the first dimension (number of
        rows of each two-dimensional frame) of the input data, A.

    n : int
        The number of elements, n, in the second dimension (number of
        columns of each two-dimensional frame) of the input data, A.

    fr : int
        The number of elements, fr, in the third dimension (number of
        frames) of the input data, A.

    Returns
    -------
    nwlmax : int
        The maximum number of levels of resolution, l_max, that can be
        computed if a multi-level discrete wavelet transform is applied
        (`wtrans` = 'M'). It is such that 2^(l_max) <= min(m,n,fr) <
        2^(l_max+1), for l_max an integer.

        If `wtrans` = 'S', `nwlmax` is not set.

    nf : int
        The filter length, n_f, for the supplied mother wavelet. This is
        used to determine the number of coefficients to be generated by
        the chosen transform.

    nwct : int
        The total number of wavelet coefficients, n_ct, that will be
        generated. When `wtrans` = 'S' the number of rows required in
        each of the output coefficient arrays can be calculated as n_cm
        = n_ct/(8*n_cn*n_cfr). When `wtrans` = 'M' the length of the
        array used to store all of the coefficient matrices must be at
        least n_ct.

    nwcn : int
        For a single-level transform (`wtrans` = 'S'), the number of
        coefficients that would be generated in the second dimension,
        n_cn, for each coefficient type. For a multi-level transform
        (`wtrans` = 'M') this is set to 1.

    nwcfr : int
        For a single-level transform (`wtrans` = 'S'), the number of
        coefficients that would be generated in the third dimension,
        n_cfr, for each coefficient type. For a multi-level transform
        (`wtrans` = 'M') this is set to 1.

    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `wavnam` not recognised: `wavnam` = *<value>*.

        (`errno` 2)
            On entry, `wtrans` = *<value>*.

            Constraint: `wtrans` = 'S' or 'M'.

        (`errno` 3)
            On entry, `mode` = *<value>*.

            Constraint: `mode` = 'P', 'H', 'W' or 'Z'.

        (`errno` 4)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 2.

        (`errno` 4)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 2.

        (`errno` 4)
            On entry, `fr` = *<value>*.

            Constraint: `fr` >= 2.
    """

    excs = []
    fun_name = 'naginterfaces.base.wav.dim3_init'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c09acft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, wavid.
        utils._BYREF_INT, # Input, itrans.
        utils._BYREF_INT, # Input, lmode.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, fr.
        utils._BYREF_INT, # Output, nwlmax.
        utils._BYREF_INT, # Output, nf.
        utils._BYREF_INT, # Output, nwct.
        utils._BYREF_INT, # Output, nwcn.
        utils._BYREF_INT, # Output, nwcfr.
        utils._EngineIntArrayType.c_type, # Output, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _wavnam = utils._EngineCharScalarType(
        wavnam, {'fun_name': fun_name, 'entity': 'wavnam'}, exp_chrlen=None,
    ).to_ctypes()
    _wavnam_py = utils._EngineCharScalarType.to_py(_wavnam)
    if _wavnam_py.upper() == 'HAAR':
        _wavid = utils._EngineIntScalarType.c_type(101)
    elif _wavnam_py.upper() == 'DB1':
        _wavid = utils._EngineIntScalarType.c_type(101)
    elif _wavnam_py.upper() == 'DB2':
        _wavid = utils._EngineIntScalarType.c_type(102)
    elif _wavnam_py.upper() == 'DB3':
        _wavid = utils._EngineIntScalarType.c_type(103)
    elif _wavnam_py.upper() == 'DB4':
        _wavid = utils._EngineIntScalarType.c_type(104)
    elif _wavnam_py.upper() == 'DB5':
        _wavid = utils._EngineIntScalarType.c_type(105)
    elif _wavnam_py.upper() == 'DB6':
        _wavid = utils._EngineIntScalarType.c_type(106)
    elif _wavnam_py.upper() == 'DB7':
        _wavid = utils._EngineIntScalarType.c_type(107)
    elif _wavnam_py.upper() == 'DB8':
        _wavid = utils._EngineIntScalarType.c_type(108)
    elif _wavnam_py.upper() == 'DB9':
        _wavid = utils._EngineIntScalarType.c_type(109)
    elif _wavnam_py.upper() == 'DB10':
        _wavid = utils._EngineIntScalarType.c_type(110)
    elif _wavnam_py.upper() == 'DB11':
        _wavid = utils._EngineIntScalarType.c_type(111)
    elif _wavnam_py.upper() == 'DB12':
        _wavid = utils._EngineIntScalarType.c_type(112)
    elif _wavnam_py.upper() == 'DB13':
        _wavid = utils._EngineIntScalarType.c_type(113)
    elif _wavnam_py.upper() == 'DB14':
        _wavid = utils._EngineIntScalarType.c_type(114)
    elif _wavnam_py.upper() == 'DB15':
        _wavid = utils._EngineIntScalarType.c_type(115)
    elif _wavnam_py.upper() == 'DB16':
        _wavid = utils._EngineIntScalarType.c_type(116)
    elif _wavnam_py.upper() == 'DB17':
        _wavid = utils._EngineIntScalarType.c_type(117)
    elif _wavnam_py.upper() == 'DB18':
        _wavid = utils._EngineIntScalarType.c_type(118)
    elif _wavnam_py.upper() == 'DB19':
        _wavid = utils._EngineIntScalarType.c_type(119)
    elif _wavnam_py.upper() == 'DB20':
        _wavid = utils._EngineIntScalarType.c_type(120)
    elif _wavnam_py.upper() == 'DB21':
        _wavid = utils._EngineIntScalarType.c_type(121)
    elif _wavnam_py.upper() == 'DB22':
        _wavid = utils._EngineIntScalarType.c_type(122)
    elif _wavnam_py.upper() == 'DB23':
        _wavid = utils._EngineIntScalarType.c_type(123)
    elif _wavnam_py.upper() == 'DB24':
        _wavid = utils._EngineIntScalarType.c_type(124)
    elif _wavnam_py.upper() == 'DB25':
        _wavid = utils._EngineIntScalarType.c_type(125)
    elif _wavnam_py.upper() == 'DB26':
        _wavid = utils._EngineIntScalarType.c_type(126)
    elif _wavnam_py.upper() == 'DB27':
        _wavid = utils._EngineIntScalarType.c_type(127)
    elif _wavnam_py.upper() == 'DB28':
        _wavid = utils._EngineIntScalarType.c_type(128)
    elif _wavnam_py.upper() == 'DB29':
        _wavid = utils._EngineIntScalarType.c_type(129)
    elif _wavnam_py.upper() == 'DB30':
        _wavid = utils._EngineIntScalarType.c_type(130)
    elif _wavnam_py.upper() == 'DB31':
        _wavid = utils._EngineIntScalarType.c_type(131)
    elif _wavnam_py.upper() == 'DB32':
        _wavid = utils._EngineIntScalarType.c_type(132)
    elif _wavnam_py.upper() == 'DB33':
        _wavid = utils._EngineIntScalarType.c_type(133)
    elif _wavnam_py.upper() == 'DB34':
        _wavid = utils._EngineIntScalarType.c_type(134)
    elif _wavnam_py.upper() == 'DB35':
        _wavid = utils._EngineIntScalarType.c_type(135)
    elif _wavnam_py.upper() == 'DB36':
        _wavid = utils._EngineIntScalarType.c_type(136)
    elif _wavnam_py.upper() == 'DB37':
        _wavid = utils._EngineIntScalarType.c_type(137)
    elif _wavnam_py.upper() == 'DB38':
        _wavid = utils._EngineIntScalarType.c_type(138)
    elif _wavnam_py.upper() == 'COIF1':
        _wavid = utils._EngineIntScalarType.c_type(161)
    elif _wavnam_py.upper() == 'COIF2':
        _wavid = utils._EngineIntScalarType.c_type(162)
    elif _wavnam_py.upper() == 'COIF3':
        _wavid = utils._EngineIntScalarType.c_type(163)
    elif _wavnam_py.upper() == 'COIF4':
        _wavid = utils._EngineIntScalarType.c_type(164)
    elif _wavnam_py.upper() == 'COIF5':
        _wavid = utils._EngineIntScalarType.c_type(165)
    elif _wavnam_py.upper() == 'COIF6':
        _wavid = utils._EngineIntScalarType.c_type(166)
    elif _wavnam_py.upper() == 'COIF7':
        _wavid = utils._EngineIntScalarType.c_type(167)
    elif _wavnam_py.upper() == 'COIF8':
        _wavid = utils._EngineIntScalarType.c_type(168)
    elif _wavnam_py.upper() == 'COIF9':
        _wavid = utils._EngineIntScalarType.c_type(169)
    elif _wavnam_py.upper() == 'COIF10':
        _wavid = utils._EngineIntScalarType.c_type(170)
    elif _wavnam_py.upper() == 'COIF11':
        _wavid = utils._EngineIntScalarType.c_type(171)
    elif _wavnam_py.upper() == 'COIF12':
        _wavid = utils._EngineIntScalarType.c_type(172)
    elif _wavnam_py.upper() == 'COIF13':
        _wavid = utils._EngineIntScalarType.c_type(173)
    elif _wavnam_py.upper() == 'COIF14':
        _wavid = utils._EngineIntScalarType.c_type(174)
    elif _wavnam_py.upper() == 'COIF15':
        _wavid = utils._EngineIntScalarType.c_type(175)
    elif _wavnam_py.upper() == 'COIF16':
        _wavid = utils._EngineIntScalarType.c_type(176)
    elif _wavnam_py.upper() == 'COIF17':
        _wavid = utils._EngineIntScalarType.c_type(177)
    elif _wavnam_py.upper() == 'BEYL':
        _wavid = utils._EngineIntScalarType.c_type(201)
    elif _wavnam_py.upper() == 'VAID':
        _wavid = utils._EngineIntScalarType.c_type(206)
    elif _wavnam_py.upper() == 'SYM2':
        _wavid = utils._EngineIntScalarType.c_type(222)
    elif _wavnam_py.upper() == 'SYM3':
        _wavid = utils._EngineIntScalarType.c_type(223)
    elif _wavnam_py.upper() == 'SYM4':
        _wavid = utils._EngineIntScalarType.c_type(224)
    elif _wavnam_py.upper() == 'SYM5':
        _wavid = utils._EngineIntScalarType.c_type(225)
    elif _wavnam_py.upper() == 'SYM6':
        _wavid = utils._EngineIntScalarType.c_type(226)
    elif _wavnam_py.upper() == 'SYM7':
        _wavid = utils._EngineIntScalarType.c_type(227)
    elif _wavnam_py.upper() == 'SYM8':
        _wavid = utils._EngineIntScalarType.c_type(228)
    elif _wavnam_py.upper() == 'SYM9':
        _wavid = utils._EngineIntScalarType.c_type(229)
    elif _wavnam_py.upper() == 'SYM10':
        _wavid = utils._EngineIntScalarType.c_type(230)
    elif _wavnam_py.upper() == 'SYM11':
        _wavid = utils._EngineIntScalarType.c_type(231)
    elif _wavnam_py.upper() == 'SYM12':
        _wavid = utils._EngineIntScalarType.c_type(232)
    elif _wavnam_py.upper() == 'SYM13':
        _wavid = utils._EngineIntScalarType.c_type(233)
    elif _wavnam_py.upper() == 'SYM14':
        _wavid = utils._EngineIntScalarType.c_type(234)
    elif _wavnam_py.upper() == 'SYM15':
        _wavid = utils._EngineIntScalarType.c_type(235)
    elif _wavnam_py.upper() == 'SYM16':
        _wavid = utils._EngineIntScalarType.c_type(236)
    elif _wavnam_py.upper() == 'SYM17':
        _wavid = utils._EngineIntScalarType.c_type(237)
    elif _wavnam_py.upper() == 'SYM18':
        _wavid = utils._EngineIntScalarType.c_type(238)
    elif _wavnam_py.upper() == 'SYM19':
        _wavid = utils._EngineIntScalarType.c_type(239)
    elif _wavnam_py.upper() == 'SYM20':
        _wavid = utils._EngineIntScalarType.c_type(240)
    elif _wavnam_py.upper() == 'BIOR1.1':
        _wavid = utils._EngineIntScalarType.c_type(311)
    elif _wavnam_py.upper() == 'BIOR1.3':
        _wavid = utils._EngineIntScalarType.c_type(313)
    elif _wavnam_py.upper() == 'BIOR1.5':
        _wavid = utils._EngineIntScalarType.c_type(315)
    elif _wavnam_py.upper() == 'BIOR2.2':
        _wavid = utils._EngineIntScalarType.c_type(322)
    elif _wavnam_py.upper() == 'BIOR2.4':
        _wavid = utils._EngineIntScalarType.c_type(324)
    elif _wavnam_py.upper() == 'BIOR2.6':
        _wavid = utils._EngineIntScalarType.c_type(326)
    elif _wavnam_py.upper() == 'BIOR2.8':
        _wavid = utils._EngineIntScalarType.c_type(328)
    elif _wavnam_py.upper() == 'BIOR3.1':
        _wavid = utils._EngineIntScalarType.c_type(331)
    elif _wavnam_py.upper() == 'BIOR3.3':
        _wavid = utils._EngineIntScalarType.c_type(333)
    elif _wavnam_py.upper() == 'BIOR3.5':
        _wavid = utils._EngineIntScalarType.c_type(335)
    elif _wavnam_py.upper() == 'BIOR3.7':
        _wavid = utils._EngineIntScalarType.c_type(337)
    elif _wavnam_py.upper() == 'BIOR3.9':
        _wavid = utils._EngineIntScalarType.c_type(339)
    elif _wavnam_py.upper() == 'BIOR4.4':
        _wavid = utils._EngineIntScalarType.c_type(344)
    elif _wavnam_py.upper() == 'BIOR5.5':
        _wavid = utils._EngineIntScalarType.c_type(355)
    elif _wavnam_py.upper() == 'BIOR6.8':
        _wavid = utils._EngineIntScalarType.c_type(368)
    elif _wavnam_py.upper() == 'RBIO1.1':
        _wavid = utils._EngineIntScalarType.c_type(411)
    elif _wavnam_py.upper() == 'RBIO1.3':
        _wavid = utils._EngineIntScalarType.c_type(413)
    elif _wavnam_py.upper() == 'RBIO1.5':
        _wavid = utils._EngineIntScalarType.c_type(415)
    elif _wavnam_py.upper() == 'RBIO2.2':
        _wavid = utils._EngineIntScalarType.c_type(422)
    elif _wavnam_py.upper() == 'RBIO2.4':
        _wavid = utils._EngineIntScalarType.c_type(424)
    elif _wavnam_py.upper() == 'RBIO2.6':
        _wavid = utils._EngineIntScalarType.c_type(426)
    elif _wavnam_py.upper() == 'RBIO2.8':
        _wavid = utils._EngineIntScalarType.c_type(428)
    elif _wavnam_py.upper() == 'RBIO3.1':
        _wavid = utils._EngineIntScalarType.c_type(431)
    elif _wavnam_py.upper() == 'RBIO3.3':
        _wavid = utils._EngineIntScalarType.c_type(433)
    elif _wavnam_py.upper() == 'RBIO3.5':
        _wavid = utils._EngineIntScalarType.c_type(435)
    elif _wavnam_py.upper() == 'RBIO3.7':
        _wavid = utils._EngineIntScalarType.c_type(437)
    elif _wavnam_py.upper() == 'RBIO3.9':
        _wavid = utils._EngineIntScalarType.c_type(439)
    elif _wavnam_py.upper() == 'RBIO4.4':
        _wavid = utils._EngineIntScalarType.c_type(444)
    elif _wavnam_py.upper() == 'RBIO5.5':
        _wavid = utils._EngineIntScalarType.c_type(455)
    elif _wavnam_py.upper() == 'RBIO6.8':
        _wavid = utils._EngineIntScalarType.c_type(468)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, wavnam not recognised: wavnam = {:s}.'.format(
                    utils._STR_FMT_STR.format(_wavnam_py),
                ),
            ],
            errcodes=(1,),
        )
        _err_class.manage()
    _wtrans = utils._EngineCharScalarType(
        wtrans, {'fun_name': fun_name, 'entity': 'wtrans'}, exp_chrlen=1,
    ).to_ctypes()
    _wtrans_py = utils._EngineCharScalarType.to_py(_wtrans)
    if _wtrans_py.upper() == 'S':
        _itrans = utils._EngineIntScalarType.c_type(1)
    elif _wtrans_py.upper() == 'M':
        _itrans = utils._EngineIntScalarType.c_type(2)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, wtrans = {:s}.'.format(
                    utils._STR_FMT_STR.format(_wtrans_py),
                ),
                'Constraint: wtrans = \'S\' or \'M\'.',
            ],
            errcodes=(2,),
        )
        _err_class.manage()
    _mode = utils._EngineCharScalarType(
        mode, {'fun_name': fun_name, 'entity': 'mode'}, exp_chrlen=1,
    ).to_ctypes()
    _mode_py = utils._EngineCharScalarType.to_py(_mode)
    if _mode_py.upper() == 'H':
        _lmode = utils._EngineIntScalarType.c_type(2)
    elif _mode_py.upper() == 'W':
        _lmode = utils._EngineIntScalarType.c_type(3)
    elif _mode_py.upper() == 'P':
        _lmode = utils._EngineIntScalarType.c_type(1)
    elif _mode_py.upper() == 'Z':
        _lmode = utils._EngineIntScalarType.c_type(4)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, mode = {:s}.'.format(
                    utils._STR_FMT_STR.format(_mode_py),
                ),
                'Constraint: mode = \'P\', \'H\', \'W\' or \'Z\'.',
            ],
            errcodes=(3,),
        )
        _err_class.manage()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _fr = utils._EngineIntScalarType(
        fr, {'fun_name': fun_name, 'entity': 'fr'},
    ).to_ctypes()
    _nwlmax = utils._EngineIntScalarType.empty_ctypes()
    _nf = utils._EngineIntScalarType.empty_ctypes()
    _nwct = utils._EngineIntScalarType.empty_ctypes()
    _nwcn = utils._EngineIntScalarType.empty_ctypes()
    _nwcfr = utils._EngineIntScalarType.empty_ctypes()
    _licomm = utils._EngineIntScalarType.c_type(260)
    _icomm = utils._EngineIntArrayType.empty_ctypes(260)
    _comm_py = {}
    _comm_py['icomm'] = _icomm
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _wavid,
        _itrans,
        _lmode,
        _m,
        _n,
        _fr,
        _nwlmax,
        _nf,
        _nwct,
        _nwcn,
        _nwcfr,
        _icomm,
        _licomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 104:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 204:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, fr = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: fr >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _nwlmax_py = utils._EngineIntScalarType.to_py(_nwlmax)
        _nf_py = utils._EngineIntScalarType.to_py(_nf)
        _nwct_py = utils._EngineIntScalarType.to_py(_nwct)
        _nwcn_py = utils._EngineIntScalarType.to_py(_nwcn)
        _nwcfr_py = utils._EngineIntScalarType.to_py(_nwcfr)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['nwlmax', 'nf', 'nwct', 'nwcn', 'nwcfr', 'comm'],
        )(
            _nwlmax_py,
            _nf_py,
            _nwct_py,
            _nwcn_py,
            _nwcfr_py,
            _comm_py,
        )
    _err_class.manage()
    return return_data

def dim1_cont(wavnam, wparam, n, x, nscal, scales, c):
    u"""
    One-dimensional real continuous wavelet transform.

    For full information please refer to the NAG Library document for
    c09ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09baf.html

    Parameters
    ----------
    wavnam : str
        The name of the mother wavelet. See [the C09 Introduction] for
        details.

        `wavnam` = 'MORLET'

          Morlet wavelet.

        `wavnam` = 'DGAUSS'

          Derivative of a Gaussian wavelet.

        `wavnam` = 'MEXHAT'

          Mexican hat wavelet.

    wparam : int
        The nondimensional wave number for the Morlet wavelet or the
        order of the derivative for the Gaussian wavelet. It is not
        referenced when `wavnam` = 'MEXHAT'.

    n : int
        The size, n, of the input dataset x.

    x : float, shape (`n`)
        `x` contains the input dataset `x`[j-1] = x_j, for j =
        1,2,...,n.

    nscal : int
        The number of scales to be computed.

    scales : int, shape (`nscal`)
        The scales at which the transform is to be computed.

    c : float, shape (`nscal`, `n`), modified in place
        On exit: the transform coefficients at the requested scales,
        where `c`[i-1,j-1] is the transform coefficient C_{i,j} at scale
        i and position j.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `wavnam` not recognised: `wavnam` = *<value>*.

        (`errno` 2)
            On entry, `wavnam` = 'MORLET' and `wparam` = *<value>*.

            Constraint: if `wavnam` = 'MORLET', 5 <= `wparam` <= 20.

        (`errno` 2)
            On entry, `wavnam` = 'DGAUSS' and `wparam` = *<value>*.

            Constraint: if `wavnam` = 'DGAUSS', 1 <= `wparam` <= 8.

        (`errno` 3)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 2.

        (`errno` 5)
            On entry, `nscal` = *<value>*.

            Constraint: `nscal` >= 1.
    """

    excs = []
    fun_name = 'naginterfaces.base.wav.dim1_cont'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c09baft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, wavid.
        utils._BYREF_INT, # Input, wparam.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._BYREF_INT, # Input, nscal.
        utils._EngineIntArrayType.c_type, # Input, scales[nscal].
        utils._EngineFloat64ArrayType.c_type, # Output, c[nscal, n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((c, 'c'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _wavnam = utils._EngineCharScalarType(
        wavnam, {'fun_name': fun_name, 'entity': 'wavnam'}, exp_chrlen=None,
    ).to_ctypes()
    _wavnam_py = utils._EngineCharScalarType.to_py(_wavnam)
    if _wavnam_py.upper() == 'MORLET':
        _wavid = utils._EngineIntScalarType.c_type(11)
    elif _wavnam_py.upper() == 'DGAUSS':
        _wavid = utils._EngineIntScalarType.c_type(12)
    elif _wavnam_py.upper() == 'MEXHAT':
        _wavid = utils._EngineIntScalarType.c_type(13)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, wavnam not recognised: wavnam = {:s}.'.format(
                    utils._STR_FMT_STR.format(_wavnam_py),
                ),
            ],
            errcodes=(1,),
        )
        _err_class.manage()
    _wparam = utils._EngineIntScalarType(
        wparam, {'fun_name': fun_name, 'entity': 'wparam'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _nscal = utils._EngineIntScalarType(
        nscal, {'fun_name': fun_name, 'entity': 'nscal'},
    ).to_ctypes()
    _nscal_py = utils._EngineIntScalarType.to_py(_nscal)
    _scales = utils._EngineIntArrayType(
        scales, {'fun_name': fun_name, 'entity': 'scales'},
        exp_shape=(_nscal_py,),
    ).to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_nscal_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _wavid,
        _wparam,
        _n,
        _x,
        _nscal,
        _scales,
        _c,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 102:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, wavnam = \'MORLET\' and wparam = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if wavnam = \'MORLET\', 5 <= wparam <= 20.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 202:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, wavnam = \'DGAUSS\' and wparam = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if wavnam = \'DGAUSS\', 1 <= wparam <= 8.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nscal = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nscal >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def dim1_sngl_fwd(n, x, lenc, ca, cd, comm):
    u"""
    One-dimensional discrete wavelet transform.

    For full information please refer to the NAG Library document for
    c09ca

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09caf.html

    Parameters
    ----------
    n : int
        The number of elements, n, in the data array x.

    x : float, shape (`n`)
        `x` contains the input dataset x_i, for i = 1,2,...,n.

    lenc : int
        This must be at least the number, n_c, of approximation
        coefficients, C_a, and detail coefficients, C_d, of the discrete
        wavelet transform as returned in nwc by the call to the
        initialization function ``dim1_init``.

    ca : float, shape (`lenc`), modified in place
        On exit: `ca`[i-1] contains the ith approximation coefficient,
        C_a(i), for i = 1,2,...,n_c.

    cd : float, shape (`lenc`), modified in place
        On exit: `cd`[i-1] contains the ith detail coefficient, C_d(i),
        for i = 1,2,...,n_c.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim1_init``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` is inconsistent with the value passed to the
            initialization function: `n` = *<value>*, `n` should be
            *<value>*.

        (`errno` 3)
            On entry, array dimension `lenc` not large enough: `lenc` =
            *<value>* but must be at least *<value>*.

        (`errno` 6)
            Either the initialization function has not been called first
            or array `comm`['icomm'] has been corrupted.

        (`errno` 6)
            Either the initialization function was called with `wtrans`
            = 'M' or array `comm`['icomm'] has been corrupted.
    """

    excs = []
    fun_name = 'naginterfaces.base.wav.dim1_sngl_fwd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c09caft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._BYREF_INT, # Input, lenc.
        utils._EngineFloat64ArrayType.c_type, # Output, ca[lenc].
        utils._EngineFloat64ArrayType.c_type, # Output, cd[lenc].
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _lenc = utils._EngineIntScalarType(
        lenc, {'fun_name': fun_name, 'entity': 'lenc'},
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm',),
    )
    _ca_cls = utils._EngineFloat64ArrayType(
        ca, {'fun_name': fun_name, 'entity': 'ca'},
        exp_shape=(utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (7,), validate=False),),
        check_contents=False,
    )
    _ca = _ca_cls.to_ctypes(can_reference_contents=False)
    _cd_cls = utils._EngineFloat64ArrayType(
        cd, {'fun_name': fun_name, 'entity': 'cd'},
        exp_shape=(utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (7,), validate=False),),
        check_contents=False,
    )
    _cd = _cd_cls.to_ctypes(can_reference_contents=False)
    _licomm_py = utils._EngineIntArrayType.get_shape(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, exp_rank=1, validate=False)[0]
    _licomm = utils._EngineIntScalarType(
        _licomm_py, {'fun_name': fun_name, 'entity': '_licomm'},
    ).to_ctypes()
    _icomm = comm['icomm']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _lenc,
        _ca,
        _cd,
        _icomm,
        _licomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n is inconsistent with the value passed to the',
                    'initialization function: n = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'n should be {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array dimension lenc not large enough:',
                    'lenc = {:s} but must be at least {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 106:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function has not been called first',
                    'or array comm[\'icomm\'] has been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 206:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function was called with wtrans = \'M\'',
                    'or array comm[\'icomm\'] has been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ca_cls.refresh_py(_ca)
        _cd_cls.refresh_py(_cd)
        return_data = None
    _err_class.manage()
    return return_data

def dim1_sngl_inv(lenc, ca, cd, n, y, comm):
    u"""
    One-dimensional inverse discrete wavelet transform.

    For full information please refer to the NAG Library document for
    c09cb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09cbf.html

    Parameters
    ----------
    lenc : int
        The dimension of the arrays `ca` and `cd`.

    ca : float, shape (`lenc`)
        The n_c approximation coefficients, C_a. These will normally be
        the result of some transformation on the coefficients computed
        by ``dim1_sngl_fwd``.

    cd : float, shape (`lenc`)
        The n_c detail coefficients, C_d. These will normally be the
        result of some transformation on the coefficients computed by
        ``dim1_sngl_fwd``.

    n : int
        n, the length of the original data array from which the wavelet
        coefficients were computed by ``dim1_sngl_fwd`` and the length
        of the data array `y` that is to be reconstructed by this
        function.

    y : float, shape (`n`), modified in place
        On exit: the reconstructed data based on approximation and
        detail coefficients C_a and C_d and the transform options
        supplied to the initialization function ``dim1_init``.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim1_init``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, array dimension `lenc` not large enough: `lenc` =
            *<value>* but must be at least *<value>*.

        (`errno` 4)
            On entry, `n` is inconsistent with the value passed to the
            initialization function: `n` = *<value>*, `n` should be
            *<value>*.

        (`errno` 6)
            Either the initialization function has not been called first
            or array `comm`['icomm'] has been corrupted.

        (`errno` 6)
            Either the initialization function was called with `wtrans`
            = 'M' or array `comm`['icomm'] has been corrupted.
    """

    excs = []
    fun_name = 'naginterfaces.base.wav.dim1_sngl_inv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c09cbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, lenc.
        utils._EngineFloat64ArrayType.c_type, # Input, ca[lenc].
        utils._EngineFloat64ArrayType.c_type, # Input, cd[lenc].
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Output, y[n].
        utils._EngineIntArrayType.c_type, # Input, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _lenc = utils._EngineIntScalarType(
        lenc, {'fun_name': fun_name, 'entity': 'lenc'},
    ).to_ctypes()
    _lenc_py = utils._EngineIntScalarType.to_py(_lenc)
    _ca = utils._EngineFloat64ArrayType(
        ca, {'fun_name': fun_name, 'entity': 'ca'},
        exp_shape=(_lenc_py,),
    ).to_ctypes()
    _cd = utils._EngineFloat64ArrayType(
        cd, {'fun_name': fun_name, 'entity': 'cd'},
        exp_shape=(_lenc_py,),
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _y = _y_cls.to_ctypes(can_reference_contents=False)
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm',),
    )
    _licomm_py = utils._EngineIntArrayType.get_shape(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, exp_rank=1)[0]
    _licomm = utils._EngineIntScalarType(
        _licomm_py, {'fun_name': fun_name, 'entity': '_licomm'},
    ).to_ctypes()
    _icomm = comm['icomm']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _lenc,
        _ca,
        _cd,
        _n,
        _y,
        _icomm,
        _licomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array dimension lenc not large enough:',
                    'lenc = {:s} but must be at least {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n is inconsistent with the value passed to the',
                    'initialization function: n = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'n should be {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 106:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function has not been called first',
                    'or array comm[\'icomm\'] has been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 206:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function was called with wtrans = \'M\'',
                    'or array comm[\'icomm\'] has been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _y_cls.refresh_py(_y)
        return_data = None
    _err_class.manage()
    return return_data

def dim1_multi_fwd(n, x, lenc, c, nwl, dwtlev, comm):
    u"""
    One-dimensional multi-level discrete wavelet transform.

    For full information please refer to the NAG Library document for
    c09cc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09ccf.html

    Parameters
    ----------
    n : int
        The number of elements, n, in the data array x.

    x : float, shape (`n`)
        `x` contains the one-dimensional input dataset x_i, for i =
        1,2,...,n.

    lenc : int
        `c` must be large enough to contain the number, n_c, of wavelet
        coefficients. The maximum value of n_c is returned in nwc by the
        call to the initialization function ``dim1_init`` and
        corresponds to the DWT being continued for the maximum number of
        levels possible for the given data set. When the number of
        levels, n_fwd, is chosen to be less than the maximum, then n_c
        is correspondingly smaller and `lenc` can be reduced by noting
        that the number of coefficients at each level is given by ⌈n_/2⌉
        for `mode` = 'P' in ``dim1_init`` and floor((n_+n_f-1)/2) for
        `mode` = 'H', 'W' or 'Z', where n_ is the number of input data
        at that level and n_f is the filter length provided by the call
        to ``dim1_init``. At the final level the storage is doubled to
        contain the set of approximation coefficients.

    c : float, shape (`lenc`), modified in place
        On exit: let q(i) denote the number of coefficients (of each
        type) produced by the wavelet transform at level i, for i =
        n_fwd,...,1. These values are returned in `dwtlev`. Setting k_1
        = q(n_fwd) and k_{j+1} = k_j+q(n_fwd-j+1), for j =
        1,2,...,n_fwd, the coefficients are stored as follows:

        `c`[i-1], for i = 1,2,...,k_1

          Contains the level n_fwd approximation coefficients, a_n_fwd.

        `c`[i-1], for i = k_1+1,...,k_2

          Contains the level n_fwd detail coefficients d_n_fwd.

        `c`[i-1], for i = k_j+1,...,k_{j+1}

          Contains the level n_fwd-j+1 detail coefficients, for j =
          2,3,...,n_fwd.

    nwl : int
        The number of levels, n_fwd, in the multi-level resolution to be
        performed.

    dwtlev : int, shape (`nwl`+1), modified in place
        On exit: the number of transform coefficients at each level.
        `dwtlev`[0] and `dwtlev`[1] contain the number, q(n_fwd), of
        approximation and detail coefficients respectively, for the
        final level of resolution (these are equal); `dwtlev`[i-1]
        contains the number of detail coefficients, q(n_fwd-i+2), for
        the (n_fwd-i+2)th level, for i = 3,4,...,n_fwd+1.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim1_init``.

    Returns
    -------
    csize : int
        The minimum length needed for `c`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` is inconsistent with the value passed to the
            initialization function: `n` = *<value>*, `n` should be
            *<value>*.

        (`errno` 5)
            On entry, `nwl` is larger than the maximum number of levels
            returned by the initialization function: `nwl` = *<value>*,
            maximum = *<value>*.

        (`errno` 5)
            On entry, `nwl` = *<value>*.

            Constraint: `nwl` >= 1.

        (`errno` 7)
            Either the initialization function has not been called first
            or array `comm`['icomm'] has been corrupted.

        (`errno` 7)
            Either the initialization function was called with `wtrans`
            = 'S' or array `comm`['icomm'] has been corrupted.
    """

    excs = []
    fun_name = 'naginterfaces.base.wav.dim1_multi_fwd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c09ccft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._BYREF_INT, # Input, lenc.
        utils._EngineFloat64ArrayType.c_type, # Output, c[lenc].
        utils._BYREF_INT, # Input, nwl.
        utils._EngineIntArrayType.c_type, # Output, dwtlev[nwl+1].
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._BYREF_INT, # Output, csize.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _lenc = utils._EngineIntScalarType(
        lenc, {'fun_name': fun_name, 'entity': 'lenc'},
    ).to_ctypes()
    _lenc_py = utils._EngineIntScalarType.to_py(_lenc)
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_lenc_py,),
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    _nwl = utils._EngineIntScalarType(
        nwl, {'fun_name': fun_name, 'entity': 'nwl'},
    ).to_ctypes()
    _nwl_py = utils._EngineIntScalarType.to_py(_nwl)
    _dwtlev_cls = utils._EngineIntArrayType(
        dwtlev, {'fun_name': fun_name, 'entity': 'dwtlev'},
        exp_shape=(_nwl_py+1,),
        check_contents=False,
    )
    _dwtlev = _dwtlev_cls.to_ctypes(can_reference_contents=False)
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm',),
    )
    _licomm_py = utils._EngineIntArrayType.get_shape(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, exp_rank=1, validate=False)[0]
    _licomm = utils._EngineIntScalarType(
        _licomm_py, {'fun_name': fun_name, 'entity': '_licomm'},
    ).to_ctypes()
    _icomm = comm['icomm']
    _csize = utils._EngineIntScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _lenc,
        _c,
        _nwl,
        _dwtlev,
        _icomm,
        _licomm,
        _csize,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n is inconsistent with the value passed to the',
                    'initialization function: n = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'n should be {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineNoErrorExit()
        elif _ierr_py == 5 and _ifmt == 105:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nwl is larger than the maximum number of levels returned',
                    'by the initialization function: nwl = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'maximum = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 205:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nwl = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nwl >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 107:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function has not been called first',
                    'or array comm[\'icomm\'] has been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 207:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function was called with wtrans = \'S\'',
                    'or array comm[\'icomm\'] has been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _c_cls.refresh_py(_c)
        _dwtlev_cls.refresh_py(_dwtlev)
        _csize_py = utils._EngineIntScalarType.to_py(_csize)
        return_data = _csize_py
    _err_class.manage()
    return return_data

def dim1_multi_inv(nwlinv, lenc, c, n, y, comm):
    u"""
    One-dimensional inverse multi-level discrete wavelet transform.

    For full information please refer to the NAG Library document for
    c09cd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09cdf.html

    Parameters
    ----------
    nwlinv : int
        The number of levels to be used in the inverse multi-level
        transform. The number of levels must be less than or equal to
        n_fwd, which has the value of argument nwl as used in the
        computation of the wavelet coefficients using
        ``dim1_multi_fwd``. The data will be reconstructed to level
        (nwl-`nwlinv`), where level 0 is the original input dataset
        provided to ``dim1_multi_fwd``.

    lenc : int
        The dimension of the array `c`.

    c : float, shape (`lenc`)
        The coefficients of a multi-level wavelet transform of the
        dataset.

        Let q(i) be the number of coefficients (of each type) at level
        i, for i = n_fwd,...,1.

        Then, setting k_1 = q(n_fwd) and k_{j+1} = k_j+q(n_fwd-j+1), for
        j = 1,2,...,n_fwd, the coefficients are stored in `c` as
        follows:

        `c`[i-1], for i = 1,2,...,k_1

          Contains the level n_fwd approximation coefficients, a_n_fwd.

        `c`[i-1], for i = k_1+1,...,k_2

          Contains the level n_fwd detail coefficients d_n_fwd.

        `c`[i-1], for i = k_j+1,...,k_{j+1}

          Contains the level n_fwd-j+1 detail coefficients, for j =
          2,3,...,n_fwd.

        The values q(i), for i = n_fwd,...,1, are contained in dwtlev
        which is produced as output by a preceding call to
        ``dim1_multi_fwd``.

        See ``dim1_multi_fwd`` for details.

    n : int
        n, the length of the data array, y, to be reconstructed. For a
        full reconstruction of nwl levels, where nwl is as supplied to
        ``dim1_multi_fwd``, this must be the same as argument n used in
        the call to ``dim1_multi_fwd``. For a partial reconstruction of
        `nwlinv` < nwl, this must be equal to dwtlev[`nwlinv`+1], as
        returned from ``dim1_multi_fwd``.

    y : float, shape (`n`), modified in place
        On exit: the dataset reconstructed from the multi-level wavelet
        transform coefficients and the transformation options supplied
        to the initialization function ``dim1_init``.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim1_init``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nwlinv` is larger than the number of levels
            computed by the preceding call to ``dim1_multi_fwd``:
            `nwlinv` = *<value>*, expected = *<value>*.

        (`errno` 1)
            On entry, `nwlinv` = *<value>*.

            Constraint: `nwlinv` >= 1.

        (`errno` 2)
            On entry, `lenc` is set too small: `lenc` = *<value>*.

            Constraint: `lenc` >= *<value>*.

        (`errno` 4)
            On entry, `n` is inconsistent with the value passed to the
            initialization function: `n` = *<value>*, `n` should be
            *<value>*.

        (`errno` 6)
            Either the initialization function has not been called first
            or array `comm`['icomm'] has been corrupted.

        (`errno` 6)
            Either the initialization function was called with `wtrans`
            = 'S' or array `comm`['icomm'] has been corrupted.
    """

    excs = []
    fun_name = 'naginterfaces.base.wav.dim1_multi_inv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c09cdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, nwlinv.
        utils._BYREF_INT, # Input, lenc.
        utils._EngineFloat64ArrayType.c_type, # Input, c[lenc].
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Output, y[n].
        utils._EngineIntArrayType.c_type, # Input, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _nwlinv = utils._EngineIntScalarType(
        nwlinv, {'fun_name': fun_name, 'entity': 'nwlinv'},
    ).to_ctypes()
    _lenc = utils._EngineIntScalarType(
        lenc, {'fun_name': fun_name, 'entity': 'lenc'},
    ).to_ctypes()
    _lenc_py = utils._EngineIntScalarType.to_py(_lenc)
    _c = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_lenc_py,),
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _y = _y_cls.to_ctypes(can_reference_contents=False)
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm',),
    )
    _licomm_py = utils._EngineIntArrayType.get_shape(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, exp_rank=1)[0]
    _licomm = utils._EngineIntScalarType(
        _licomm_py, {'fun_name': fun_name, 'entity': '_licomm'},
    ).to_ctypes()
    _icomm = comm['icomm']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _nwlinv,
        _lenc,
        _c,
        _n,
        _y,
        _icomm,
        _licomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nwlinv is larger than the number of levels computed',
                    'by the preceding call to dim1_multi_fwd: nwlinv = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'expected = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nwlinv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nwlinv >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lenc is set too small: lenc = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lenc >= {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n is inconsistent with the value passed to the',
                    'initialization function: n = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'n should be {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 106:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function has not been called first',
                    'or array comm[\'icomm\'] has been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 206:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function was called with wtrans = \'S\'',
                    'or array comm[\'icomm\'] has been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _y_cls.refresh_py(_y)
        return_data = None
    _err_class.manage()
    return return_data

def dim1_mxolap_fwd(n, x, lenc, ca, cd, comm):
    u"""
    One-dimensional maximal overlap discrete wavelet transform (MODWT).

    For full information please refer to the NAG Library document for
    c09da

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09daf.html

    Parameters
    ----------
    n : int
        The number of elements, n, in the data array x.

    x : float, shape (`n`)
        `x` contains the input dataset x_i, for i = 1,2,...,n.

    lenc : int
        This must be at least the number, n_c, of approximation
        coefficients, C_a, and detail coefficients, C_d, of the discrete
        wavelet transform as returned in nwc by the call to the
        initialization function ``dim1_init``. Note that n_c = n for
        periodic end extension, but this is not the case for other end
        extension methods which will be available in future releases.

    ca : float, shape (`lenc`), modified in place
        On exit: `ca`[i-1] contains the ith approximation coefficient,
        C_a(i), for i = 1,2,...,n_c.

    cd : float, shape (`lenc`), modified in place
        On exit: `cd`[i-1] contains the ith detail coefficient, C_d(i),
        for i = 1,2,...,n_c.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim1_init``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` is inconsistent with the value passed to the
            initialization function: `n` = *<value>*, `n` should be
            *<value>*.

        (`errno` 3)
            On entry, array dimension `lenc` not large enough: `lenc` =
            *<value>* but must be at least *<value>*.

        (`errno` 6)
            On entry, the initialization function ``dim1_init`` has not
            been called first or it has not been called with `wtrans` =
            'T', or the communication array `comm`['icomm'] has become
            corrupted.
    """

    excs = []
    fun_name = 'naginterfaces.base.wav.dim1_mxolap_fwd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c09daft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._BYREF_INT, # Input, lenc.
        utils._EngineFloat64ArrayType.c_type, # Output, ca[lenc].
        utils._EngineFloat64ArrayType.c_type, # Output, cd[lenc].
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _lenc = utils._EngineIntScalarType(
        lenc, {'fun_name': fun_name, 'entity': 'lenc'},
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm',),
    )
    _ca_cls = utils._EngineFloat64ArrayType(
        ca, {'fun_name': fun_name, 'entity': 'ca'},
        exp_shape=(utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (7,), validate=False),),
        check_contents=False,
    )
    _ca = _ca_cls.to_ctypes(can_reference_contents=False)
    _cd_cls = utils._EngineFloat64ArrayType(
        cd, {'fun_name': fun_name, 'entity': 'cd'},
        exp_shape=(utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (7,), validate=False),),
        check_contents=False,
    )
    _cd = _cd_cls.to_ctypes(can_reference_contents=False)
    _licomm_py = utils._EngineIntArrayType.get_shape(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, exp_rank=1, validate=False)[0]
    _licomm = utils._EngineIntScalarType(
        _licomm_py, {'fun_name': fun_name, 'entity': '_licomm'},
    ).to_ctypes()
    _icomm = comm['icomm']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _lenc,
        _ca,
        _cd,
        _icomm,
        _licomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n is inconsistent with the value passed to the',
                    'initialization function: n = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'n should be {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array dimension lenc not large enough:',
                    'lenc = {:s} but must be at least {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the initialization function dim1_init has not been called first',
                    'or it has not been called with wtrans = \'T\',',
                    'or the communication array comm[\'icomm\'] has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ca_cls.refresh_py(_ca)
        _cd_cls.refresh_py(_cd)
        return_data = None
    _err_class.manage()
    return return_data

def dim1_mxolap_inv(lenc, ca, cd, n, y, comm):
    u"""
    One-dimensional inverse maximal overlap discrete wavelet transform
    (IMODWT).

    For full information please refer to the NAG Library document for
    c09db

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09dbf.html

    Parameters
    ----------
    lenc : int
        The dimension of the arrays `ca` and `cd`.

    ca : float, shape (`lenc`)
        The n_c approximation coefficients, C_a. These will normally be
        the result of some transformation on the coefficients computed
        by ``dim1_mxolap_fwd``.

    cd : float, shape (`lenc`)
        The n_c detail coefficients, C_d. These will normally be the
        result of some transformation on the coefficients computed by
        ``dim1_mxolap_fwd``.

    n : int
        n, the length of the original data array from which the wavelet
        coefficients were computed by ``dim1_mxolap_fwd`` and the length
        of the data array `y` that is to be reconstructed by this
        function.

    y : float, shape (`n`), modified in place
        On exit: the reconstructed data based on approximation and
        detail coefficients C_a and C_d and the transform options
        supplied to the initialization function ``dim1_init``.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim1_init``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, array dimension `lenc` not large enough: `lenc` =
            *<value>* but must be at least *<value>*.

        (`errno` 4)
            On entry, `n` is inconsistent with the value passed to the
            initialization function: `n` = *<value>*, `n` should be
            *<value>*.

        (`errno` 6)
            On entry, the initialization function ``dim1_init`` has not
            been called first or it has not been called with `wtrans` =
            'T', or the communication array `comm`['icomm'] has become
            corrupted.
    """

    excs = []
    fun_name = 'naginterfaces.base.wav.dim1_mxolap_inv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c09dbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, lenc.
        utils._EngineFloat64ArrayType.c_type, # Input, ca[lenc].
        utils._EngineFloat64ArrayType.c_type, # Input, cd[lenc].
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Output, y[n].
        utils._EngineIntArrayType.c_type, # Input, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _lenc = utils._EngineIntScalarType(
        lenc, {'fun_name': fun_name, 'entity': 'lenc'},
    ).to_ctypes()
    _lenc_py = utils._EngineIntScalarType.to_py(_lenc)
    _ca = utils._EngineFloat64ArrayType(
        ca, {'fun_name': fun_name, 'entity': 'ca'},
        exp_shape=(_lenc_py,),
    ).to_ctypes()
    _cd = utils._EngineFloat64ArrayType(
        cd, {'fun_name': fun_name, 'entity': 'cd'},
        exp_shape=(_lenc_py,),
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _y = _y_cls.to_ctypes(can_reference_contents=False)
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm',),
    )
    _licomm_py = utils._EngineIntArrayType.get_shape(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, exp_rank=1)[0]
    _licomm = utils._EngineIntScalarType(
        _licomm_py, {'fun_name': fun_name, 'entity': '_licomm'},
    ).to_ctypes()
    _icomm = comm['icomm']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _lenc,
        _ca,
        _cd,
        _n,
        _y,
        _icomm,
        _licomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array dimension lenc not large enough:',
                    'lenc = {:s} but must be at least {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n is inconsistent with the value passed to the',
                    'initialization function: n = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'n should be {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the initialization function dim1_init has not been called first',
                    'or it has not been called with wtrans = \'T\',',
                    'or the communication array comm[\'icomm\'] has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _y_cls.refresh_py(_y)
        return_data = None
    _err_class.manage()
    return return_data

def dim1_mxolap_multi_fwd(n, x, keepa, lenc, c, nwl, comm):
    u"""
    One-dimensional multi-level maximal overlap discrete wavelet
    transform (MODWT).

    For full information please refer to the NAG Library document for
    c09dc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09dcf.html

    Parameters
    ----------
    n : int
        The number of elements, n, in the data array x.

    x : float, shape (`n`)
        `x` contains the input dataset x_i, for i = 1,2,...,n.

    keepa : str, length 1
        Determines whether the approximation coefficients are stored in
        array `c` for every level of the computed transform or else only
        for the final level. In both cases, the detail coefficients are
        stored in `c` for every level computed.

        `keepa` = 'A'

          Retain approximation coefficients for all levels computed.

        `keepa` = 'F'

          Retain approximation coefficients for only the final level
          computed.

    lenc : int
        `c` must be large enough to contain the number of wavelet
        coefficients.

        If `keepa` = 'F', the total number of coefficients, n_c, is
        returned in nwc by the call to the initialization function
        ``dim1_init`` and corresponds to the MODWT being continued for
        the maximum number of levels possible for the given data set.

        When the number of levels, n_l, is chosen to be less than the
        maximum, then the number of stored coefficients is
        correspondingly smaller and `lenc` can be reduced by noting that
        n_d detail coefficients are stored at each level, with the
        storage increased at the final level to contain the n_a
        approximation coefficients.

        If `keepa` = 'A', n_d detail coefficients and n_a approximation
        coefficients are stored for each level computed, requiring
        `lenc` >= n_l*(n_a+n_d) = 2*n_l*n_a, since the numbers of stored
        approximation and detail coefficients are equal.

        The number of approximation (or detail) coefficients at each
        level, n_a, is returned in `na`.

    c : float, shape (`lenc`), modified in place
        On exit: the coefficients of a multi-level wavelet transform of
        the dataset.

        The coefficients are stored in `c` as follows:

        If `keepa` = 'F',

        `c`[0:n_a]

          Contains the level n_l approximation coefficients;

        `c`[ n_a+(i-1)*n_d+1 -1: n_a+i*n_d ]

          Contains the level (n_l-i+1) detail coefficients, for i =
          1,2,...,n_l;

        If `keepa` = 'A',

        `c`[ (i-1)*n_a+1 -1: i*n_a ]

          Contains the level (n_l-i+1) approximation coefficients, for i
          = 1,2,...,n_l;

        `c`[ n_l*n_a+(i-1)*n_d+1 -1: n_l*n_a+i*n_d ]

          Contains the level i detail coefficients, for i = 1,2,...,n_l;

        The values n_a and n_d denote the numbers of approximation and
        detail coefficients respectively, which are equal and returned
        in `na`.

    nwl : int
        The number of levels, n_l, in the multi-level resolution to be
        performed.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim1_init``.

    Returns
    -------
    na : int
        `na` contains the number of approximation coefficients, n_a, at
        each level which is equal to the number of detail coefficients,
        n_d. With periodic end extension (`mode` = 'P' in ``dim1_init``)
        this is the same as the length, `n`, of the data array, `x`.

    csize : int
        The minimum length needed for `c`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` is inconsistent with the value passed to the
            initialization function: `n` = *<value>*, `n` should be
            *<value>*.

        (`errno` 6)
            On entry, `nwl` is larger than the maximum number of levels
            returned by the initialization function: `nwl` = *<value>*,
            maximum = *<value>*.

        (`errno` 6)
            On entry, `nwl` = *<value>*.

            Constraint: `nwl` >= 1.

        (`errno` 8)
            On entry, the initialization function ``dim1_init`` has not
            been called first or it has not been called with `wtrans` =
            'U', or the communication array `comm`['icomm'] has become
            corrupted.

        (`errno` 31)
            On entry, `keepa` = *<value>*.

            Constraint: `keepa` = 'F' or 'A'.
    """

    excs = []
    fun_name = 'naginterfaces.base.wav.dim1_mxolap_multi_fwd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c09dcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._BYREF_INT, # Input, ikeep.
        utils._BYREF_INT, # Input, lenc.
        utils._EngineFloat64ArrayType.c_type, # Output, c[lenc].
        utils._BYREF_INT, # Input, nwl.
        utils._BYREF_INT, # Output, na.
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._BYREF_INT, # Output, csize.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _keepa = utils._EngineCharScalarType(
        keepa, {'fun_name': fun_name, 'entity': 'keepa'}, exp_chrlen=1,
    ).to_ctypes()
    _keepa_py = utils._EngineCharScalarType.to_py(_keepa)
    if _keepa_py.upper() == 'F':
        _ikeep = utils._EngineIntScalarType.c_type(0)
    elif _keepa_py.upper() == 'A':
        _ikeep = utils._EngineIntScalarType.c_type(1)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, keepa = {:s}.'.format(
                    utils._STR_FMT_STR.format(_keepa_py),
                ),
                'Constraint: keepa = \'F\' or \'A\'.',
            ],
            errcodes=(2,),
        )
        _err_class.manage()
    _lenc = utils._EngineIntScalarType(
        lenc, {'fun_name': fun_name, 'entity': 'lenc'},
    ).to_ctypes()
    _lenc_py = utils._EngineIntScalarType.to_py(_lenc)
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_lenc_py,),
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    _nwl = utils._EngineIntScalarType(
        nwl, {'fun_name': fun_name, 'entity': 'nwl'},
    ).to_ctypes()
    _na = utils._EngineIntScalarType.empty_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm',),
    )
    _licomm_py = utils._EngineIntArrayType.get_shape(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, exp_rank=1, validate=False)[0]
    _licomm = utils._EngineIntScalarType(
        _licomm_py, {'fun_name': fun_name, 'entity': '_licomm'},
    ).to_ctypes()
    _icomm = comm['icomm']
    _csize = utils._EngineIntScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _ikeep,
        _lenc,
        _c,
        _nwl,
        _na,
        _icomm,
        _licomm,
        _csize,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n is inconsistent with the value passed to the',
                    'initialization function: n = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'n should be {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineNoErrorExit()
        elif _ierr_py == 6 and _ifmt == 106:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nwl is larger than the maximum number of levels returned',
                    'by the initialization function: nwl = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'maximum = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 206:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nwl = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nwl >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the initialization function dim1_init has not been called first',
                    'or it has not been called with wtrans = \'U\',',
                    'or the communication array comm[\'icomm\'] has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _c_cls.refresh_py(_c)
        _na_py = utils._EngineIntScalarType.to_py(_na)
        _csize_py = utils._EngineIntScalarType.to_py(_csize)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['na', 'csize'],
        )(
            _na_py,
            _csize_py,
        )
    _err_class.manage()
    return return_data

def dim1_mxolap_multi_inv(nwlinv, keepa, lenc, c, n, y, comm):
    u"""
    One-dimensional inverse multi-level maximal overlap discrete wavelet
    transform (IMODWT).

    For full information please refer to the NAG Library document for
    c09dd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09ddf.html

    Parameters
    ----------
    nwlinv : int
        The number of levels to be used in the inverse multi-level
        transform. The number of levels must be less than or equal to
        n_fwd, which has the value of argument nwl as used in the
        computation of the wavelet coefficients using
        ``dim1_mxolap_multi_fwd``. The data will be reconstructed to
        level (nwl-`nwlinv`), where level 0 is the original input
        dataset provided to ``dim1_mxolap_multi_fwd``.

    keepa : str, length 1
        Determines whether the approximation coefficients are stored in
        array `c` for every level of the computed transform or else only
        for the final level. In both cases, the detail coefficients are
        stored in `c` for every level computed.

        `keepa` = 'A'

          Retain approximation coefficients for all levels computed.

        `keepa` = 'F'

          Retain approximation coefficients for only the final level
          computed.

    lenc : int
        The dimension of the array `c`.

    c : float, shape (`lenc`)
        The coefficients of a multi-level wavelet transform of the
        dataset.

        The coefficients are stored in `c` as follows:

        If `keepa` = 'F',

        `c`[0:n_a]

          Contains the level n_l approximation coefficients;

        `c`[ n_a + (i-1)*n_d+1 -1: n_a+i*n_d ]

          Contains the level (n_l-i+1) detail coefficients, for i =
          1,2,...,n_l;

        If `keepa` = 'A',

        `c`[(i-1) *n_a:i*n_a]

          Contains the level (n_l-i+1) approximation coefficients, for i
          = 1,2,...,n_l;

        `c`[n_l *n_a+(i-1) *n_d:n_l *n_a+i*n_d]

          Contains the level i detail coefficients, for i = 1,2,...,n_l.

        The values n_a and n_d denote the numbers of approximation and
        detail coefficients respectively, which are equal.

        This number is returned as output in na from a preceding call to
        ``dim1_mxolap_multi_fwd``.

        See ``dim1_mxolap_multi_fwd`` for details.

    n : int
        n, the length of the data array, y, to be reconstructed.

    y : float, shape (`n`), modified in place
        On exit: the dataset reconstructed from the multi-level wavelet
        transform coefficients and the transformation options supplied
        to the initialization function ``dim1_init``.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim1_init``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nwlinv` is larger than the number of levels
            computed by the preceding call to ``dim1_mxolap_multi_fwd``:
            `nwlinv` = *<value>*, expected *<value>*.

        (`errno` 1)
            On entry, `nwlinv` = *<value>*.

            Constraint: `nwlinv` >= 1.

        (`errno` 3)
            On entry, `lenc` is set too small: `lenc` = *<value>*.

            Constraint: `lenc` >= *<value>*.

        (`errno` 5)
            On entry, `n` is inconsistent with the value passed to the
            initialization function: `n` = *<value>*, `n` should be
            *<value>*.

        (`errno` 7)
            On entry, the initialization function ``dim1_init`` has not
            been called first or it has not been called with `wtrans` =
            'U', or the communication array `comm`['icomm'] has become
            corrupted.

        (`errno` 21)
            On entry, `keepa` = *<value>*.

            Constraint: `keepa` = 'F' or 'A'.
    """

    excs = []
    fun_name = 'naginterfaces.base.wav.dim1_mxolap_multi_inv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c09ddft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, nwlinv.
        utils._BYREF_INT, # Input, ikeep.
        utils._BYREF_INT, # Input, lenc.
        utils._EngineFloat64ArrayType.c_type, # Input, c[lenc].
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Output, y[n].
        utils._EngineIntArrayType.c_type, # Input, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _nwlinv = utils._EngineIntScalarType(
        nwlinv, {'fun_name': fun_name, 'entity': 'nwlinv'},
    ).to_ctypes()
    _keepa = utils._EngineCharScalarType(
        keepa, {'fun_name': fun_name, 'entity': 'keepa'}, exp_chrlen=1,
    ).to_ctypes()
    _keepa_py = utils._EngineCharScalarType.to_py(_keepa)
    if _keepa_py.upper() == 'F':
        _ikeep = utils._EngineIntScalarType.c_type(0)
    elif _keepa_py.upper() == 'A':
        _ikeep = utils._EngineIntScalarType.c_type(1)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, keepa = {:s}.'.format(
                    utils._STR_FMT_STR.format(_keepa_py),
                ),
                'Constraint: keepa = \'F\' or \'A\'.',
            ],
            errcodes=(2,),
        )
        _err_class.manage()
    _lenc = utils._EngineIntScalarType(
        lenc, {'fun_name': fun_name, 'entity': 'lenc'},
    ).to_ctypes()
    _lenc_py = utils._EngineIntScalarType.to_py(_lenc)
    _c = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_lenc_py,),
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _y = _y_cls.to_ctypes(can_reference_contents=False)
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm',),
    )
    _licomm_py = utils._EngineIntArrayType.get_shape(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, exp_rank=1)[0]
    _licomm = utils._EngineIntScalarType(
        _licomm_py, {'fun_name': fun_name, 'entity': '_licomm'},
    ).to_ctypes()
    _icomm = comm['icomm']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _nwlinv,
        _ikeep,
        _lenc,
        _c,
        _n,
        _y,
        _icomm,
        _licomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nwlinv is larger than the number of levels computed',
                    'by the preceding call to dim1_mxolap_multi_fwd: nwlinv = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'expected {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nwlinv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nwlinv >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lenc is set too small: lenc = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lenc >= {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n is inconsistent with the value passed to the',
                    'initialization function: n = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'n should be {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the initialization function dim1_init has not been called first',
                    'or it has not been called with wtrans = \'U\',',
                    'or the communication array comm[\'icomm\'] has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _y_cls.refresh_py(_y)
        return_data = None
    _err_class.manage()
    return return_data

def dim2_sngl_fwd(m, n, a, ca, ch, cv, cd, comm):
    u"""
    Two-dimensional discrete wavelet transform.

    For full information please refer to the NAG Library document for
    c09ea

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09eaf.html

    Parameters
    ----------
    m : int
        Number of rows, m, of data matrix A.

    n : int
        Number of columns, n, of data matrix A.

    a : float, shape (`m`, `n`)
        The m by n data matrix A.

    ca : float, shape (n_cm, n_cn), modified in place
        On exit: contains the n_cm by n_cn matrix of approximation
        coefficients, C_a.

    ch : float, shape (n_cm, n_cn), modified in place
        On exit: contains the n_cm by n_cn matrix of horizontal
        coefficients, C_h.

    cv : float, shape (n_cm, n_cn), modified in place
        On exit: contains the n_cm by n_cn matrix of vertical
        coefficients, C_v.

    cd : float, shape (n_cm, n_cn), modified in place
        On exit: contains the n_cm by n_cn matrix of diagonal
        coefficients, C_d.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim2_init``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` = *<value>*, the value of n on
            initialization (see ``dim2_init``).

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` = *<value>*, the value of m on
            initialization (see ``dim2_init``).

        (`errno` 6)
            Either the initialization function has not been called first
            or `comm`['icomm'] has been corrupted.

        (`errno` 6)
            Either the initialization function was called with `wtrans`
            = 'M' or `comm`['icomm'] has been corrupted.
    """

    excs = []
    fun_name = 'naginterfaces.base.wav.dim2_sngl_fwd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c09eaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Output, ca[ldca, *].
        utils._BYREF_INT, # Input, ldca.
        utils._EngineFloat64ArrayType.c_type, # Output, ch[ldch, *].
        utils._BYREF_INT, # Input, ldch.
        utils._EngineFloat64ArrayType.c_type, # Output, cv[ldcv, *].
        utils._BYREF_INT, # Input, ldcv.
        utils._EngineFloat64ArrayType.c_type, # Output, cd[ldcd, *].
        utils._BYREF_INT, # Input, ldcd.
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (ca, 'ca'), (ch, 'ch'), (cv, 'cv'), (cd, 'cd')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldca_py = utils._EngineFloat64ArrayType.get_nd(ca, {'fun_name': fun_name, 'entity': 'ca'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldca = utils._EngineIntScalarType(
        _ldca_py, {'fun_name': fun_name, 'entity': '_ldca'},
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm',),
    )
    _vl_n_cn_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (7,), validate=False)
    _vl_n_cm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (9,), validate=False)
    _ca_cls = utils._EngineFloat64ArrayType(
        ca, {'fun_name': fun_name, 'entity': 'ca'},
        exp_shape=(_vl_n_cm_py, _vl_n_cn_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _ca = _ca_cls.to_ctypes(can_reference_contents=False)
    _ldch_py = utils._EngineFloat64ArrayType.get_nd(ch, {'fun_name': fun_name, 'entity': 'ch'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldch = utils._EngineIntScalarType(
        _ldch_py, {'fun_name': fun_name, 'entity': '_ldch'},
    ).to_ctypes()
    _ch_cls = utils._EngineFloat64ArrayType(
        ch, {'fun_name': fun_name, 'entity': 'ch'},
        exp_shape=(_vl_n_cm_py, _vl_n_cn_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _ch = _ch_cls.to_ctypes(can_reference_contents=False)
    _ldcv_py = utils._EngineFloat64ArrayType.get_nd(cv, {'fun_name': fun_name, 'entity': 'cv'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldcv = utils._EngineIntScalarType(
        _ldcv_py, {'fun_name': fun_name, 'entity': '_ldcv'},
    ).to_ctypes()
    _cv_cls = utils._EngineFloat64ArrayType(
        cv, {'fun_name': fun_name, 'entity': 'cv'},
        exp_shape=(_vl_n_cm_py, _vl_n_cn_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _cv = _cv_cls.to_ctypes(can_reference_contents=False)
    _ldcd_py = utils._EngineFloat64ArrayType.get_nd(cd, {'fun_name': fun_name, 'entity': 'cd'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldcd = utils._EngineIntScalarType(
        _ldcd_py, {'fun_name': fun_name, 'entity': '_ldcd'},
    ).to_ctypes()
    _cd_cls = utils._EngineFloat64ArrayType(
        cd, {'fun_name': fun_name, 'entity': 'cd'},
        exp_shape=(_vl_n_cm_py, _vl_n_cn_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _cd = _cd_cls.to_ctypes(can_reference_contents=False)
    _licomm_py = utils._EngineIntArrayType.get_shape(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, exp_rank=1, validate=False)[0]
    _licomm = utils._EngineIntScalarType(
        _licomm_py, {'fun_name': fun_name, 'entity': '_licomm'},
    ).to_ctypes()
    _icomm = comm['icomm']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _m,
        _n,
        _a,
        _lda,
        _ca,
        _ldca,
        _ch,
        _ldch,
        _cv,
        _ldcv,
        _cd,
        _ldcd,
        _icomm,
        _licomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n = {:s}, the value of n on initialization'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    '(see dim2_init).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 201:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m = {:s}, the value of m on'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'initialization (see dim2_init).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 106:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function has not been called first',
                    'or comm[\'icomm\'] has been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 206:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function was called with wtrans = \'M\'',
                    'or comm[\'icomm\'] has been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ca_cls.refresh_py(_ca)
        _ch_cls.refresh_py(_ch)
        _cv_cls.refresh_py(_cv)
        _cd_cls.refresh_py(_cd)
        return_data = None
    _err_class.manage()
    return return_data

def dim2_sngl_inv(m, n, ca, ch, cv, cd, b, comm):
    u"""
    Two-dimensional inverse discrete wavelet transform.

    For full information please refer to the NAG Library document for
    c09eb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09ebf.html

    Parameters
    ----------
    m : int
        Number of rows, m, of data matrix B.

    n : int
        Number of columns, n, of data matrix B.

    ca : float, shape (n_cm, n_cn)
        Contains the n_cm by n_cn matrix of approximation coefficients,
        C_a. This array will normally be the result of some
        transformation on the coefficients computed by function
        ``dim2_sngl_fwd``.

    ch : float, shape (n_cm, n_cn)
        Contains the n_cm by n_cn matrix of horizontal coefficients,
        C_h. This array will normally be the result of some
        transformation on the coefficients computed by function
        ``dim2_sngl_fwd``.

    cv : float, shape (n_cm, n_cn)
        Contains the n_cm by n_cn matrix of vertical coefficients, C_v.
        This array will normally be the result of some transformation on
        the coefficients computed by function ``dim2_sngl_fwd``.

    cd : float, shape (n_cm, n_cn)
        Contains the n_cm by n_cn matrix of diagonal coefficients, C_d.
        This array will normally be the result of some transformation on
        the coefficients computed by function ``dim2_sngl_fwd``.

    b : float, shape (`m`, `n`), modified in place
        On exit: the m by n reconstructed matrix, B, based on the input
        approximation, horizontal, vertical and diagonal coefficients
        and the transform options supplied to the initialization
        function ``dim2_init``.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim2_init``.

    Raises
    ------
    NagValueError
        (`errno` 4)
            On entry, `n` = *<value>*.

            Constraint: `n` = *<value>*, the value of n on
            initialization (see ``dim2_init``).

        (`errno` 4)
            On entry, `m` = *<value>*.

            Constraint: `m` = *<value>*, the value of m on
            initialization (see ``dim2_init``).

        (`errno` 6)
            Either the initialization function has not been called first
            or `comm`['icomm'] has been corrupted.

        (`errno` 6)
            Either the initialization function was called with `wtrans`
            = 'M' or `comm`['icomm'] has been corrupted.
    """

    excs = []
    fun_name = 'naginterfaces.base.wav.dim2_sngl_inv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c09ebft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, ca[ldca, *].
        utils._BYREF_INT, # Input, ldca.
        utils._EngineFloat64ArrayType.c_type, # Input, ch[ldch, *].
        utils._BYREF_INT, # Input, ldch.
        utils._EngineFloat64ArrayType.c_type, # Input, cv[ldcv, *].
        utils._BYREF_INT, # Input, ldcv.
        utils._EngineFloat64ArrayType.c_type, # Input, cd[ldcd, *].
        utils._BYREF_INT, # Input, ldcd.
        utils._EngineFloat64ArrayType.c_type, # Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineIntArrayType.c_type, # Input, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ca, 'ca'), (ch, 'ch'), (cv, 'cv'), (cd, 'cd'), (b, 'b')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _ldca_py = utils._EngineFloat64ArrayType.get_nd(ca, {'fun_name': fun_name, 'entity': 'ca'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldca = utils._EngineIntScalarType(
        _ldca_py, {'fun_name': fun_name, 'entity': '_ldca'},
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm',),
    )
    _vl_n_cn_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (7,), validate=False)
    _vl_n_cm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (9,), validate=False)
    _ca = utils._EngineFloat64ArrayType(
        ca, {'fun_name': fun_name, 'entity': 'ca'},
        exp_shape=(_vl_n_cm_py, _vl_n_cn_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldch_py = utils._EngineFloat64ArrayType.get_nd(ch, {'fun_name': fun_name, 'entity': 'ch'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldch = utils._EngineIntScalarType(
        _ldch_py, {'fun_name': fun_name, 'entity': '_ldch'},
    ).to_ctypes()
    _ch = utils._EngineFloat64ArrayType(
        ch, {'fun_name': fun_name, 'entity': 'ch'},
        exp_shape=(_vl_n_cm_py, _vl_n_cn_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldcv_py = utils._EngineFloat64ArrayType.get_nd(cv, {'fun_name': fun_name, 'entity': 'cv'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldcv = utils._EngineIntScalarType(
        _ldcv_py, {'fun_name': fun_name, 'entity': '_ldcv'},
    ).to_ctypes()
    _cv = utils._EngineFloat64ArrayType(
        cv, {'fun_name': fun_name, 'entity': 'cv'},
        exp_shape=(_vl_n_cm_py, _vl_n_cn_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldcd_py = utils._EngineFloat64ArrayType.get_nd(cd, {'fun_name': fun_name, 'entity': 'cd'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldcd = utils._EngineIntScalarType(
        _ldcd_py, {'fun_name': fun_name, 'entity': '_ldcd'},
    ).to_ctypes()
    _cd = utils._EngineFloat64ArrayType(
        cd, {'fun_name': fun_name, 'entity': 'cd'},
        exp_shape=(_vl_n_cm_py, _vl_n_cn_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldb_py = utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldb = utils._EngineIntScalarType(
        _ldb_py, {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _b = _b_cls.to_ctypes(can_reference_contents=False)
    _licomm_py = utils._EngineIntArrayType.get_shape(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, exp_rank=1)[0]
    _licomm = utils._EngineIntScalarType(
        _licomm_py, {'fun_name': fun_name, 'entity': '_licomm'},
    ).to_ctypes()
    _icomm = comm['icomm']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _m,
        _n,
        _ca,
        _ldca,
        _ch,
        _ldch,
        _cv,
        _ldcv,
        _cd,
        _ldcd,
        _b,
        _ldb,
        _icomm,
        _licomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 4 and _ifmt == 104:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n = {:s}, the value of n on initialization'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    '(see dim2_init).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 204:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m = {:s}, the value of m on'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'initialization (see dim2_init).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 106:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function has not been called first',
                    'or comm[\'icomm\'] has been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 206:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function was called with wtrans = \'M\'',
                    'or comm[\'icomm\'] has been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def dim2_multi_fwd(m, n, a, lenc, c, nwl, dwtlvm, dwtlvn, comm):
    u"""
    Two-dimensional multi-level discrete wavelet transform.

    For full information please refer to the NAG Library document for
    c09ec

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09ecf.html

    Parameters
    ----------
    m : int
        Number of rows, m, of data matrix A.

    n : int
        Number of columns, n, of data matrix A.

    a : float, shape (`m`, `n`)
        The m by n data matrix A.

    lenc : int
        `c` must be large enough to contain, n_ct, wavelet coefficients.
        The maximum value of n_ct is returned in nwct by the call to the
        initialization function ``dim2_init`` and corresponds to the DWT
        being continued for the maximum number of levels possible for
        the given data set. When the number of levels, n_fwd, is chosen
        to be less than the maximum, l_max, then n_ct is correspondingly
        smaller and `lenc` can be reduced by noting that the vertical,
        horizontal and diagonal coefficients are stored at every level
        and that in addition the approximation coefficients are stored
        for the final level only. The number of coefficients stored at
        each level is given by 3*⌈m_/2⌉*⌈n_/2⌉ for `mode` = 'P' in
        ``dim2_init`` and 3*floor((m_+n_f-1) / 2)*floor((n_+n_f-1) / 2)
        for `mode` = 'H', 'W' or 'Z', where the input data is of
        dimension m_*n_ at that level and n_f is the filter length nf
        provided by the call to ``dim2_init``. At the final level the
        storage is 4/3 times this value to contain the set of
        approximation coefficients.

    c : float, shape (`lenc`), modified in place
        On exit: the coefficients of the discrete wavelet transform. If
        you need to access or modify the approximation coefficients or
        any specific set of detail coefficients then the use of
        ``dim2_coeff_ext`` or ``dim2_coeff_ins`` is recommended. For
        completeness the following description provides details of
        precisely how the coefficient are stored in `c` but this
        information should only be required in rare cases.

        Let q(i) denote the number of coefficients (of each type) at
        level i, for i = 1,2,...,n_fwd, such that q(i) = `dwtlvm`[ n_fwd
        - i + 1 -1]*`dwtlvn`[ n_fwd - i + 1 -1].

        Then, letting k_1 = q(n_fwd) and k_{j+1} = k_j+q(n_fwd-⌈j/3⌉+1),
        for j = 1,2,...,3n_fwd, the coefficients are stored in `c` as
        follows:

        `c`[i-1], for i = 1,2,...,k_1

          Contains the level n_fwd approximation coefficients, a_n_fwd.

        `c`[i-1], for i = k_j+1,...,k_{j+1}

          Contains the level n_fwd-⌈j/3⌉+1 vertical, horizontal and
          diagonal coefficients. These are:

          - vertical coefficients if mod(j, 3) = 1;

          - horizontal coefficients if mod(j, 3) = 2;

          - diagonal coefficients if mod(j, 3) = 0,

        for j = 1,...,3*n_fwd.

    nwl : int
        The number of levels, n_fwd, in the multi-level resolution to be
        performed.

    dwtlvm : int, shape (`nwl`), modified in place
        On exit: the number of coefficients in the first dimension for
        each coefficient type at each level. `dwtlvm`[i-1] contains the
        number of coefficients in the first dimension (for each
        coefficient type computed) at the (n_fwd-i+1)th level of
        resolution, for i = 1,2,...,n_fwd. Thus for the first n_fwd-1
        levels of resolution, `dwtlvm`[n_fwd-i] is the size of the first
        dimension of the matrices of vertical, horizontal and diagonal
        coefficients computed at this level; for the final level of
        resolution, `dwtlvm`[0] is the size of the first dimension of
        the matrices of approximation, vertical, horizontal and diagonal
        coefficients computed.

    dwtlvn : int, shape (`nwl`), modified in place
        On exit: the number of coefficients in the second dimension for
        each coefficient type at each level. `dwtlvn`[i-1] contains the
        number of coefficients in the second dimension (for each
        coefficient type computed) at the (n_fwd-i+1)th level of
        resolution, for i = 1,2,...,n_fwd. Thus for the first n_fwd-1
        levels of resolution, `dwtlvn`[n_fwd-i] is the size of the
        second dimension of the matrices of vertical, horizontal and
        diagonal coefficients computed at this level; for the final
        level of resolution, `dwtlvn`[0] is the size of the second
        dimension of the matrices of approximation, vertical, horizontal
        and diagonal coefficients computed.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim2_init``.

    Returns
    -------
    csize : int
        The minimum length needed for `c`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` = *<value>*, the value of n on
            initialization (see ``dim2_init``).

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` = *<value>*, the value of m on
            initialization (see ``dim2_init``).

        (`errno` 5)
            On entry, `nwl` = *<value>* and nwlmax = *<value>* in
            ``dim2_init``.

            Constraint: `nwl` <= nwlmax in ``dim2_init``.

        (`errno` 5)
            On entry, `nwl` = *<value>*.

            Constraint: `nwl` >= 1.

        (`errno` 7)
            Either the initialization function has not been called first
            or `comm`['icomm'] has been corrupted.

        (`errno` 7)
            Either the initialization function was called with `wtrans`
            = 'S' or `comm`['icomm'] has been corrupted.
    """

    excs = []
    fun_name = 'naginterfaces.base.wav.dim2_multi_fwd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c09ecft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_INT, # Input, lenc.
        utils._EngineFloat64ArrayType.c_type, # Output, c[lenc].
        utils._BYREF_INT, # Input, nwl.
        utils._EngineIntArrayType.c_type, # Output, dwtlvm[nwl].
        utils._EngineIntArrayType.c_type, # Output, dwtlvn[nwl].
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._BYREF_INT, # Output, csize.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _lenc = utils._EngineIntScalarType(
        lenc, {'fun_name': fun_name, 'entity': 'lenc'},
    ).to_ctypes()
    _lenc_py = utils._EngineIntScalarType.to_py(_lenc)
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_lenc_py,),
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    _nwl = utils._EngineIntScalarType(
        nwl, {'fun_name': fun_name, 'entity': 'nwl'},
    ).to_ctypes()
    _nwl_py = utils._EngineIntScalarType.to_py(_nwl)
    _dwtlvm_cls = utils._EngineIntArrayType(
        dwtlvm, {'fun_name': fun_name, 'entity': 'dwtlvm'},
        exp_shape=(_nwl_py,),
        check_contents=False,
    )
    _dwtlvm = _dwtlvm_cls.to_ctypes(can_reference_contents=False)
    _dwtlvn_cls = utils._EngineIntArrayType(
        dwtlvn, {'fun_name': fun_name, 'entity': 'dwtlvn'},
        exp_shape=(_nwl_py,),
        check_contents=False,
    )
    _dwtlvn = _dwtlvn_cls.to_ctypes(can_reference_contents=False)
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm',),
    )
    _licomm_py = utils._EngineIntArrayType.get_shape(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, exp_rank=1, validate=False)[0]
    _licomm = utils._EngineIntScalarType(
        _licomm_py, {'fun_name': fun_name, 'entity': '_licomm'},
    ).to_ctypes()
    _icomm = comm['icomm']
    _csize = utils._EngineIntScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _m,
        _n,
        _a,
        _lda,
        _lenc,
        _c,
        _nwl,
        _dwtlvm,
        _dwtlvn,
        _icomm,
        _licomm,
        _csize,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n = {:s}, the value of n on initialization'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    '(see dim2_init).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 201:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m = {:s}, the value of m on'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'initialization (see dim2_init).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineNoErrorExit()
        elif _ierr_py == 5 and _ifmt == 105:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nwl = {:s} and nwlmax = {:s} in'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'dim2_init.',
                    'Constraint: nwl <= nwlmax in',
                    'dim2_init.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 205:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nwl = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nwl >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 107:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function has not been called first',
                    'or comm[\'icomm\'] has been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 207:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function was called with wtrans = \'S\'',
                    'or comm[\'icomm\'] has been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _c_cls.refresh_py(_c)
        _dwtlvm_cls.refresh_py(_dwtlvm)
        _dwtlvn_cls.refresh_py(_dwtlvn)
        _csize_py = utils._EngineIntScalarType.to_py(_csize)
        return_data = _csize_py
    _err_class.manage()
    return return_data

def dim2_multi_inv(nwlinv, lenc, c, m, n, b, comm):
    u"""
    Two-dimensional inverse multi-level discrete wavelet transform.

    For full information please refer to the NAG Library document for
    c09ed

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09edf.html

    Parameters
    ----------
    nwlinv : int
        The number of levels to be used in the inverse multi-level
        transform. The number of levels must be less than or equal to
        n_fwd, which has the value of argument nwl as used in the
        computation of the wavelet coefficients using
        ``dim2_multi_fwd``. The data will be reconstructed to level
        (nwl-`nwlinv`), where level 0 is the original input dataset
        provided to ``dim2_multi_fwd``.

    lenc : int
        The dimension of the array `c`.

    c : float, shape (`lenc`)
        The coefficients of a multi-level wavelet transform of the
        original matrix, A, which may have been filtered or otherwise
        manipulated.

        Let q(i) be the number of coefficients (of each type) at level
        i, for i = n_fwd,...,1.

        Then, setting k_1 = q(n_fwd) and k_{j+1} = k_j+q(n_fwd-⌈j/3⌉+1),
        for j = 1,2,...,3n_fwd, the coefficients are stored in `c` as
        follows:

        `c`[i-1], for i = 1,2,...,k_1

          Contains the level n_fwd approximation coefficients, a_n_fwd.

        `c`[i-1], for i = k_j+1,...,k_{j+1}

          Contains the level n_fwd-⌈j/3⌉+1 vertical, horizontal and
          diagonal coefficients. These are:

          - vertical coefficients if mod(j, 3) = 1;

          - horizontal coefficients if mod(j, 3) = 2;

          - diagonal coefficients if mod(j, 3) = 0,

        for j = 1,...,3*n_fwd.

        Note that the coefficients in `c` may be extracted according to
        level and type into two-dimensional arrays using
        ``dim2_coeff_ext``, and inserted using ``dim2_coeff_ins``.

    m : int
        The number of elements, m, in the first dimension of the
        reconstructed matrix B. For a full reconstruction of nwl levels,
        where nwl is as supplied to ``dim2_multi_fwd``, this must be the
        same as argument m used in the call to ``dim2_multi_fwd``. For a
        partial reconstruction of `nwlinv` < nwl levels, this must be
        equal to dwtlvm[`nwlinv`], as returned from ``dim2_multi_fwd``.

    n : int
        The number of elements, n, in the second dimension of the
        reconstructed matrix B. For a full reconstruction of nwl levels,
        where nwl is as supplied to ``dim3_multi_fwd``, this must be the
        same as argument n used in the call to ``dim2_multi_fwd``. For a
        partial reconstruction of `nwlinv` < nwl, this must be equal to
        dwtlvn[`nwlinv`], as returned from ``dim2_multi_fwd``.

    b : float, shape (`m`, `n`), modified in place
        On exit: the m by n reconstructed matrix, B, based on the input
        multi-level wavelet transform coefficients and the transform
        options supplied to the initialization function ``dim2_init``.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim2_init``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nwlinv` = *<value>* and n_fwd = *<value>*.

            Constraint: `nwlinv` <= n_fwd.

        (`errno` 1)
            On entry, `nwlinv` = *<value>*.

            Constraint: `nwlinv` >= 1.

        (`errno` 3)
            On entry, `lenc` = *<value>*.

            Constraint: `lenc` >= *<value>*, the total number of
            coefficients generated by the preceding call to
            ``dim2_multi_fwd``.

        (`errno` 4)
            On entry, `n` = *<value>*.

            Constraint: `n` >= *<value>*, the number of coefficients in
            the second dimension at the required level of
            reconstruction.

        (`errno` 4)
            On entry, `m` = *<value>*.

            Constraint: `m` >= *<value>*, the number of coefficients in
            the first dimension at the required level of reconstruction.

        (`errno` 6)
            Either the initialization function has not been called first
            or `comm`['icomm'] has been corrupted.

        (`errno` 6)
            Either the initialization function was called with `wtrans`
            = 'S' or `comm`['icomm'] has been corrupted.
    """

    excs = []
    fun_name = 'naginterfaces.base.wav.dim2_multi_inv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c09edft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, nwlinv.
        utils._BYREF_INT, # Input, lenc.
        utils._EngineFloat64ArrayType.c_type, # Input, c[lenc].
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Output, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineIntArrayType.c_type, # Input, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _nwlinv = utils._EngineIntScalarType(
        nwlinv, {'fun_name': fun_name, 'entity': 'nwlinv'},
    ).to_ctypes()
    _lenc = utils._EngineIntScalarType(
        lenc, {'fun_name': fun_name, 'entity': 'lenc'},
    ).to_ctypes()
    _lenc_py = utils._EngineIntScalarType.to_py(_lenc)
    _c = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_lenc_py,),
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _ldb_py = utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldb = utils._EngineIntScalarType(
        _ldb_py, {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _b = _b_cls.to_ctypes(can_reference_contents=False)
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm',),
    )
    _licomm_py = utils._EngineIntArrayType.get_shape(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, exp_rank=1)[0]
    _licomm = utils._EngineIntScalarType(
        _licomm_py, {'fun_name': fun_name, 'entity': '_licomm'},
    ).to_ctypes()
    _icomm = comm['icomm']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _nwlinv,
        _lenc,
        _c,
        _m,
        _n,
        _b,
        _ldb,
        _icomm,
        _licomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nwlinv = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'n_fwd = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nwlinv <= n_fwd.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nwlinv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nwlinv >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lenc = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lenc >= {:s}, the total number of coefficients'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'generated by the preceding call to dim2_multi_fwd.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 104:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= {:s}, the number of coefficients in the'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'second dimension at the required level of reconstruction.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 204:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= {:s}, the number of coefficients in the'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'first dimension at the required level of reconstruction.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 106:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function has not been called first',
                    'or comm[\'icomm\'] has been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 206:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function was called with wtrans = \'S\'',
                    'or comm[\'icomm\'] has been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def dim2_coeff_ext(ilev, cindex, lenc, c, d, comm):
    u"""
    Two-dimensional discrete wavelet transform coefficient extraction.

    For full information please refer to the NAG Library document for
    c09ey

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09eyf.html

    Parameters
    ----------
    ilev : int
        The level at which coefficients are to be extracted.

    cindex : int
        Identifies which coefficients to extract. The coefficients are
        identified as follows:

        `cindex` = 0

          The approximation coefficients, produced by application of the
          low pass filter over columns and rows of the original matrix
          (LL). The approximation coefficients are available only for
          `ilev` = nwl, where nwl is the value used in a preceding call
          to ``dim2_multi_fwd``.

        `cindex` = 1

          The vertical detail coefficients produced by applying the low
          pass filter over columns of the original matrix and the high
          pass filter over rows (LH).

        `cindex` = 2

          The horizontal detail coefficients produced by applying the
          high pass filter over columns of the original matrix and the
          low pass filter over rows (HL).

        `cindex` = 3

          The diagonal detail coefficients produced by applying the high
          pass filter over columns and rows of the original matrix (HH).

    lenc : int
        The dimension of the array `c`.

    c : float, shape (`lenc`)
        DWT coefficients, as computed by a preceding call to
        ``dim2_multi_fwd``.

    d : float, shape (n_cm, n_cn), modified in place
        On exit: the requested coefficients.

        If `ilev` = nwl (as used in ``dim2_multi_fwd``) and `cindex` =
        0, the n_cm by n_cn approximation coefficients a_{ij} are stored
        in `d`[i-1,j-1], for j = 1,2,...,n_cn, for i = 1,2,...,n_cm.

        Otherwise the n_cm by n_cn level `ilev` detail coefficients (of
        type specified by `cindex`) d_{ij} are stored in `d`[i-1,j-1],
        for j = 1,2,...,n_cn, for i = 1,2,...,n_cm.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim2_init``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ilev` = *<value>*.

            Constraint: `ilev` >= 1.

        (`errno` 1)
            On entry, `ilev` = *<value>* and nwl = *<value>*.

            Constraint: `ilev` <= nwl, where nwl is the number of levels
            used in the call to ``dim2_multi_fwd``.

        (`errno` 2)
            On entry, `cindex` = *<value>*.

            Constraint: `cindex` >= 0.

        (`errno` 2)
            On entry, `cindex` = *<value>*.

            Constraint: `cindex` <= 3.

        (`errno` 3)
            On entry, `lenc` = *<value>* and n_ct = *<value>*.

            Constraint: `lenc` >= n_ct, where n_ct is the number of DWT
            coefficients computed in a previous call to
            ``dim2_multi_fwd``.

        (`errno` 5)
            On entry, `ilev` = *<value>* and nwl = *<value>*, but
            `cindex` = 0.

            Constraint: `cindex` > 0 when `ilev` < nwl in the preceding
            call to ``dim2_multi_fwd``.

        (`errno` 6)
            Either the initialization function has not been called first
            or `comm`['icomm'] has been corrupted.

        (`errno` 6)
            Either the initialization function was called with `wtrans`
            = 'S' or `comm`['icomm'] has been corrupted.
    """

    excs = []
    fun_name = 'naginterfaces.base.wav.dim2_coeff_ext'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c09eyft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ilev.
        utils._BYREF_INT, # Input, cindex.
        utils._BYREF_INT, # Input, lenc.
        utils._EngineFloat64ArrayType.c_type, # Input, c[lenc].
        utils._EngineFloat64ArrayType.c_type, # Output, d[ldd, *].
        utils._BYREF_INT, # Input, ldd.
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((d, 'd'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _ilev = utils._EngineIntScalarType(
        ilev, {'fun_name': fun_name, 'entity': 'ilev'},
    ).to_ctypes()
    _cindex = utils._EngineIntScalarType(
        cindex, {'fun_name': fun_name, 'entity': 'cindex'},
    ).to_ctypes()
    _lenc = utils._EngineIntScalarType(
        lenc, {'fun_name': fun_name, 'entity': 'lenc'},
    ).to_ctypes()
    _lenc_py = utils._EngineIntScalarType.to_py(_lenc)
    _c = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_lenc_py,),
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm',),
    )
    _ilev_py = utils._EngineIntScalarType.to_py(_ilev)
    _vl_n_cm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((100+utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (11,), validate=False)+1-_ilev_py)-1,), validate=False)
    _ldd_py = utils._EngineFloat64ArrayType.get_nd(d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldd = utils._EngineIntScalarType(
        _ldd_py, {'fun_name': fun_name, 'entity': '_ldd'},
    ).to_ctypes()
    _vl_n_cn_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((20+utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (11,), validate=False)+1-_ilev_py)-1,), validate=False)
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_vl_n_cm_py, _vl_n_cn_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _d = _d_cls.to_ctypes(can_reference_contents=False)
    _licomm_py = utils._EngineIntArrayType.get_shape(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, exp_rank=1, validate=False)[0]
    _licomm = utils._EngineIntScalarType(
        _licomm_py, {'fun_name': fun_name, 'entity': '_licomm'},
    ).to_ctypes()
    _icomm = comm['icomm']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ilev,
        _cindex,
        _lenc,
        _c,
        _d,
        _ldd,
        _icomm,
        _licomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 100:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ilev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ilev >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 200:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ilev = {:s} and nwl = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ilev <= nwl, where nwl is the number of',
                    'levels used in the call to dim2_multi_fwd.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 100:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, cindex = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: cindex >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 200:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, cindex = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: cindex <= 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 100:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lenc = {:s} and n_ct = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: lenc >= n_ct, where n_ct is the number of DWT coefficients',
                    'computed in a previous call to dim2_multi_fwd.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 100:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ilev = {:s} and nwl = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'but cindex = 0.',
                    'Constraint: cindex > 0 when ilev < nwl in the preceding call to dim2_multi_fwd.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 100:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function has not been called first',
                    'or comm[\'icomm\'] has been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 200:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function was called with wtrans = \'S\'',
                    'or comm[\'icomm\'] has been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _d_cls.refresh_py(_d)
        return_data = None
    _err_class.manage()
    return return_data

def dim2_coeff_ins(ilev, cindex, lenc, c, d, comm):
    u"""
    Two-dimensional discrete wavelet transform coefficient insertion.

    For full information please refer to the NAG Library document for
    c09ez

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09ezf.html

    Parameters
    ----------
    ilev : int
        The level at which coefficients are to be inserted.

    cindex : int
        Identifies which coefficients to insert. The coefficients are
        identified as follows:

        `cindex` = 0

          The approximation coefficients, produced by application of the
          low pass filter over columns and rows of the original matrix
          (LL). The approximation coefficients are present only for
          `ilev` = nwl, where nwl is the value used in a preceding call
          to ``dim2_multi_fwd``.

        `cindex` = 1

          The vertical detail coefficients produced by applying the low
          pass filter over columns of the original matrix and the high
          pass filter over rows (LH).

        `cindex` = 2

          The horizontal detail coefficients produced by applying the
          high pass filter over columns of the original matrix and the
          low pass filter over rows (HL).

        `cindex` = 3

          The diagonal detail coefficients produced by applying the high
          pass filter over columns and rows of the original matrix (HH).

    lenc : int
        The dimension of the array `c`.

    c : float, shape (`lenc`), modified in place
        On entry: contains the DWT coefficients inserted by previous
        calls to ``dim2_coeff_ins``, or computed by a previous call to
        ``dim2_multi_fwd``.

        On exit: contains the same DWT coefficients provided on entry
        except for those identified by `ilev` and `cindex`, which are
        updated with the values supplied in `d`, inserted into the
        correct locations as expected by the reconstruction function
        ``dim2_multi_inv``.

    d : float, shape (n_cm, n_cn)
        The coefficients to be inserted.

        If `ilev` = nwl (as used in ``dim2_multi_fwd``) and `cindex` =
        0, the n_cm by n_cn manipulated approximation coefficients
        a_{ij} must be stored in `d`[i-1,j-1], for i = 1,2,...,n_cn, for
        i = 1,2,...,n_cm.

        Otherwise the n_cm by n_cn manipulated level `ilev` detail
        coefficients (of type specified by `cindex`) d_{ij} must be
        stored in `d`[i-1,j-1], for j = 1,2,...,n_cn, for i =
        1,2,...,n_cm.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim2_init``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ilev` = *<value>*.

            Constraint: `ilev` >= 1.

        (`errno` 1)
            On entry, `ilev` = *<value>* and nwl = *<value>*.

            Constraint: `ilev` <= nwl, where nwl is the number of levels
            used in the call to ``dim2_multi_fwd``.

        (`errno` 2)
            On entry, `cindex` = *<value>*.

            Constraint: `cindex` >= 0.

        (`errno` 2)
            On entry, `cindex` = *<value>*.

            Constraint: `cindex` <= 3.

        (`errno` 3)
            On entry, `lenc` = *<value>* and n_ct = *<value>*.

            Constraint: `lenc` >= n_ct, where n_ct is the number of DWT
            coefficients computed in a previous call to
            ``dim2_multi_fwd``.

        (`errno` 5)
            On entry, `ilev` = *<value>* and nwl = *<value>*, but
            `cindex` = 0.

            Constraint: `cindex` > 0 when `ilev` < nwl in the preceding
            call to ``dim2_multi_fwd``.

        (`errno` 6)
            Either the initialization function has not been called first
            or `comm`['icomm'] has been corrupted.

        (`errno` 6)
            Either the initialization function was called with `wtrans`
            = 'S' or `comm`['icomm'] has been corrupted.
    """

    excs = []
    fun_name = 'naginterfaces.base.wav.dim2_coeff_ins'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c09ezft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ilev.
        utils._BYREF_INT, # Input, cindex.
        utils._BYREF_INT, # Input, lenc.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c[lenc].
        utils._EngineFloat64ArrayType.c_type, # Input, d[ldd, *].
        utils._BYREF_INT, # Input, ldd.
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((d, 'd'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _ilev = utils._EngineIntScalarType(
        ilev, {'fun_name': fun_name, 'entity': 'ilev'},
    ).to_ctypes()
    _cindex = utils._EngineIntScalarType(
        cindex, {'fun_name': fun_name, 'entity': 'cindex'},
    ).to_ctypes()
    _lenc = utils._EngineIntScalarType(
        lenc, {'fun_name': fun_name, 'entity': 'lenc'},
    ).to_ctypes()
    _lenc_py = utils._EngineIntScalarType.to_py(_lenc)
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_lenc_py,),
    )
    _c = _c_cls.to_ctypes()
    _ldd_py = utils._EngineFloat64ArrayType.get_nd(d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldd = utils._EngineIntScalarType(
        _ldd_py, {'fun_name': fun_name, 'entity': '_ldd'},
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm',),
    )
    _ilev_py = utils._EngineIntScalarType.to_py(_ilev)
    _vl_n_cn_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((20+utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (11,), validate=False)+1-_ilev_py)-1,), validate=False)
    _vl_n_cm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((100+utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (11,), validate=False)+1-_ilev_py)-1,), validate=False)
    _d = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_vl_n_cm_py, _vl_n_cn_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _licomm_py = utils._EngineIntArrayType.get_shape(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, exp_rank=1, validate=False)[0]
    _licomm = utils._EngineIntScalarType(
        _licomm_py, {'fun_name': fun_name, 'entity': '_licomm'},
    ).to_ctypes()
    _icomm = comm['icomm']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ilev,
        _cindex,
        _lenc,
        _c,
        _d,
        _ldd,
        _icomm,
        _licomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 100:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ilev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ilev >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 200:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ilev = {:s} and nwl = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ilev <= nwl, where nwl is the number of',
                    'levels used in the call to dim2_multi_fwd.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 100:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, cindex = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: cindex >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 200:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, cindex = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: cindex <= 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 100:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lenc = {:s} and n_ct = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: lenc >= n_ct, where n_ct is the number of DWT coefficients',
                    'computed in a previous call to dim2_multi_fwd.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 100:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ilev = {:s} and nwl = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'but cindex = 0.',
                    'Constraint: cindex > 0 when ilev < nwl in the preceding call to dim2_multi_fwd.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 100:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function has not been called first',
                    'or comm[\'icomm\'] has been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 200:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function was called with wtrans = \'S\'',
                    'or comm[\'icomm\'] has been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def dim3_sngl_fwd(m, n, fr, a, lenc, c, comm):
    u"""
    Three-dimensional discrete wavelet transform.

    For full information please refer to the NAG Library document for
    c09fa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09faf.html

    Parameters
    ----------
    m : int
        The number of rows of each two-dimensional frame.

    n : int
        The number of columns of each two-dimensional frame.

    fr : int
        The number of two-dimensional frames.

    a : float, shape (`m`, `n`, `fr`)
        The m by n by fr three-dimensional input data A, where A_{ijk}
        is stored in `a`[i-1,j-1,k-1].

    lenc : int
        The dimension of the array `c`.

    c : float, shape (`lenc`), modified in place
        On exit: the coefficients of the discrete wavelet transform. If
        you need to access or modify the approximation coefficients or
        any specific set of detail coefficients then the use of
        ``dim3_coeff_ext`` or ``dim3_coeff_ins`` is recommended. For
        completeness the following description provides details of
        precisely how the coefficients are stored in `c` but this
        information should only be required in rare cases.

        The 8 sets of coefficients are stored in the following order:
        approximation coefficients (LLL) first, followed by 7 sets of
        detail coefficients: LLH, LHL, LHH, HLL, HLH, HHL, HHH, where L
        indicates the low pass filter, and H the high pass filter being
        applied to, respectively, the columns of length `m`, the rows of
        length `n` and then the frames of length `fr`.

        Note that for computational efficiency reasons each set of
        coefficients is stored in the order n_cfr*n_cm*n_cn (see output
        arguments nwcfr, nwct and nwcn in ``dim3_init``).

        See [Example] for details of how to access each set of
        coefficients in order to perform extraction from `c` following a
        call to this function, or insertion into `c` before a call to
        the three-dimensional inverse function ``dim3_sngl_inv``.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim3_init``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` = *<value>*, the value of `n` on
            initialization (see ``dim3_init``).

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` = *<value>*, the value of `m` on
            initialization (see ``dim3_init``).

        (`errno` 1)
            On entry, `fr` = *<value>*.

            Constraint: `fr` = *<value>*, the value of `fr` on
            initialization (see ``dim3_init``).

        (`errno` 3)
            On entry, `lenc` = *<value>* and n_ct = *<value>*.

            Constraint: `lenc` >= n_ct, where n_ct is the number of DWT
            coefficients returned by ``dim3_init`` in argument nwct.

        (`errno` 6)
            Either the communication array `comm`['icomm'] has been
            corrupted or there has not been a prior call to the
            initialization function ``dim3_init``.

        (`errno` 6)
            Either the communication array `comm`['icomm'] has been
            corrupted or `wtrans` = 'M' was used in the prior call to
            the initialization function ``dim3_init``.
    """

    excs = []
    fun_name = 'naginterfaces.base.wav.dim3_sngl_fwd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c09faft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, fr.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, sda, fr].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_INT, # Input, sda.
        utils._BYREF_INT, # Input, lenc.
        utils._EngineFloat64ArrayType.c_type, # Output, c[lenc].
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _fr = utils._EngineIntScalarType(
        fr, {'fun_name': fun_name, 'entity': 'fr'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=3, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _sda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=3, sorder=_m_sorder, dim_type='s')
    _sda = utils._EngineIntScalarType(
        _sda_py, {'fun_name': fun_name, 'entity': '_sda'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _fr_py = utils._EngineIntScalarType.to_py(_fr)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py, _fr_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _lenc = utils._EngineIntScalarType(
        lenc, {'fun_name': fun_name, 'entity': 'lenc'},
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm',),
    )
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (10,), validate=False),),
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    _licomm_py = utils._EngineIntArrayType.get_shape(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, exp_rank=1, validate=False)[0]
    _licomm = utils._EngineIntScalarType(
        _licomm_py, {'fun_name': fun_name, 'entity': '_licomm'},
    ).to_ctypes()
    _icomm = comm['icomm']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _m,
        _n,
        _fr,
        _a,
        _lda,
        _sda,
        _lenc,
        _c,
        _icomm,
        _licomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n = {:s}, the value of n on initialization'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    '(see dim3_init).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 201:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m = {:s}, the value of m on'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'initialization (see dim3_init).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 301:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, fr = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: fr = {:s}, the value of fr on'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'initialization (see dim3_init).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 103:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lenc = {:s} and n_ct = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: lenc >= n_ct, where n_ct is the number of DWT coefficients',
                    'returned by dim3_init in argument nwct.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 106:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the communication array comm[\'icomm\'] has been corrupted or there has not',
                    'been a prior call to the initialization function dim3_init.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 206:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the communication array comm[\'icomm\'] has been corrupted or',
                    'wtrans = \'M\' was used in the prior call to the initialization',
                    'function dim3_init.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data

def dim3_sngl_inv(m, n, fr, lenc, c, b, comm):
    u"""
    Three-dimensional inverse discrete wavelet transform.

    For full information please refer to the NAG Library document for
    c09fb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09fbf.html

    Parameters
    ----------
    m : int
        The number of rows of each two-dimensional frame.

    n : int
        The number of columns of each two-dimensional frame.

    fr : int
        The number two-dimensional frames.

    lenc : int
        The dimension of the array `c`.

    c : float, shape (`lenc`)
        The coefficients of the discrete wavelet transform. This will
        normally be the result of some transformation on the
        coefficients computed by function ``dim3_sngl_fwd``.

        Note that the coefficients in `c` may be extracted according to
        type into three-dimensional arrays using ``dim3_coeff_ext``, and
        inserted using ``dim3_coeff_ins``.

    b : float, shape (`m`, `n`, `fr`), modified in place
        On exit: the m by n by fr reconstructed array, B, with B_{ijk}
        stored in `b`[i-1,j-1,k-1]. The reconstruction is based on the
        input wavelet coefficients and the transform options supplied to
        the initialization function ``dim3_init``.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim3_init``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` = *<value>*, the value of `n` on
            initialization (see ``dim3_init``).

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` = *<value>*, the value of `m` on
            initialization (see ``dim3_init``).

        (`errno` 1)
            On entry, `fr` = *<value>*.

            Constraint: `fr` = *<value>*, the value of `fr` on
            initialization (see ``dim3_init``).

        (`errno` 3)
            On entry, `lenc` = *<value>* and n_ct = *<value>*.

            Constraint: `lenc` >= n_ct, where n_ct is the number of DWT
            coefficients returned by ``dim3_init`` in argument nwct.

        (`errno` 6)
            Either the communication array `comm`['icomm'] has been
            corrupted or there has not been a prior call to the
            initialization function ``dim3_init``.

        (`errno` 6)
            Either the communication array `comm`['icomm'] has been
            corrupted or `wtrans` = 'M' was used in the prior call to
            the initialization function ``dim3_init``.
    """

    excs = []
    fun_name = 'naginterfaces.base.wav.dim3_sngl_inv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c09fbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, fr.
        utils._BYREF_INT, # Input, lenc.
        utils._EngineFloat64ArrayType.c_type, # Input, c[lenc].
        utils._EngineFloat64ArrayType.c_type, # Output, b[ldb, sdb, fr].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Input, sdb.
        utils._EngineIntArrayType.c_type, # Input, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _fr = utils._EngineIntScalarType(
        fr, {'fun_name': fun_name, 'entity': 'fr'},
    ).to_ctypes()
    _lenc = utils._EngineIntScalarType(
        lenc, {'fun_name': fun_name, 'entity': 'lenc'},
    ).to_ctypes()
    _lenc_py = utils._EngineIntScalarType.to_py(_lenc)
    _c = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_lenc_py,),
    ).to_ctypes()
    _ldb_py = utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=3, sorder=_m_sorder, dim_type='l')
    _ldb = utils._EngineIntScalarType(
        _ldb_py, {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _sdb_py = utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=3, sorder=_m_sorder, dim_type='s')
    _sdb = utils._EngineIntScalarType(
        _sdb_py, {'fun_name': fun_name, 'entity': '_sdb'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _fr_py = utils._EngineIntScalarType.to_py(_fr)
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_m_py, _n_py, _fr_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _b = _b_cls.to_ctypes(can_reference_contents=False)
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm',),
    )
    _licomm_py = utils._EngineIntArrayType.get_shape(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, exp_rank=1)[0]
    _licomm = utils._EngineIntScalarType(
        _licomm_py, {'fun_name': fun_name, 'entity': '_licomm'},
    ).to_ctypes()
    _icomm = comm['icomm']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _m,
        _n,
        _fr,
        _lenc,
        _c,
        _b,
        _ldb,
        _sdb,
        _icomm,
        _licomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n = {:s}, the value of n on initialization'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    '(see dim3_init).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 201:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m = {:s}, the value of m on'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'initialization (see dim3_init).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 301:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, fr = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: fr = {:s}, the value of fr on'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'initialization (see dim3_init).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 103:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lenc = {:s} and n_ct = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: lenc >= n_ct, where n_ct is the number of DWT coefficients',
                    'returned by dim3_init in argument nwct.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 106:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the communication array comm[\'icomm\'] has been corrupted or there has not',
                    'been a prior call to the initialization function dim3_init.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 206:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the communication array comm[\'icomm\'] has been corrupted or',
                    'wtrans = \'M\' was used in the prior call to the initialization',
                    'function dim3_init.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def dim3_multi_fwd(m, n, fr, a, lenc, c, nwl, dwtlvm, dwtlvn, dwtlvfr, comm):
    u"""
    Three-dimensional multi-level discrete wavelet transform.

    For full information please refer to the NAG Library document for
    c09fc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09fcf.html

    Parameters
    ----------
    m : int
        The number of rows of each two-dimensional frame.

    n : int
        The number of columns of each two-dimensional frame.

    fr : int
        The number of two-dimensional frames.

    a : float, shape (`m`, `n`, `fr`)
        The m by n by fr three-dimensional input data A, where with
        A_{ijk} stored in `a`[i-1,j-1,k-1].

    lenc : int
        The dimension of the array `c`.

    c : float, shape (`lenc`), modified in place
        On exit: the coefficients of the discrete wavelet transform. If
        you need to access or modify the approximation coefficients or
        any specific set of detail coefficients then the use of
        ``dim3_coeff_ext`` or ``dim3_coeff_ins`` is recommended. For
        completeness the following description provides details of
        precisely how the coefficients are stored in `c` but this
        information should only be required in rare cases.

        Let q(i) denote the number of coefficients of each type at level
        i, for i = 1,2,...,n_fwd, such that q(i) =
        `dwtlvm`[n_fwd-i]*`dwtlvn`[n_fwd-i]*`dwtlvfr`[n_fwd-i].

        Then, letting k_1 = q(n_fwd) and k_{j+1} = k_j+q(n_fwd-⌈j/7⌉+1),
        for j = 1,2,...,7n_fwd, the coefficients are stored in `c` as
        follows:

        `c`[i-1], for i = 1,2,...,k_1

          Contains the level n_fwd approximation coefficients, a_n_fwd.
          Note that for computational efficiency reasons these
          coefficients are stored as
          `dwtlvm`[0]*`dwtlvn`[0]*`dwtlvfr`[0] in `c`.

        `c`[i-1], for i = k_j+1,...,k_{j+1}

          Contains the level n_fwd-⌈j/7⌉+1 detail coefficients. These
          are:

          - LLH coefficients if mod(j, 7) = 1;

          - LHL coefficients if mod(j, 7) = 2;

          - LHH coefficients if mod(j, 7) = 3;

          - HLL coefficients if mod(j, 7) = 4;

          - HLH coefficients if mod(j, 7) = 5;

          - HHL coefficients if mod(j, 7) = 6;

          - HHH coefficients if mod(j, 7) = 0,

        for j = 1,...,7*n_fwd. See [the C09 Introduction] for a
        description of how these coefficients are produced.

          Note that for computational efficiency reasons these
          coefficients are stored as `dwtlvfr`[ ⌈j/7⌉ -1]*`dwtlvm`[
          ⌈j/7⌉ -1]*`dwtlvn`[ ⌈j/7⌉ -1] in `c`.

    nwl : int
        The number of levels, n_fwd, in the multi-level resolution to be
        performed.

    dwtlvm : int, shape (`nwl`), modified in place
        On exit: the number of coefficients in the first dimension for
        each coefficient type at each level. `dwtlvm`[i-1] contains the
        number of coefficients in the first dimension (for each
        coefficient type computed) at the (n_fwd-i+1)th level of
        resolution, for i = 1,2,...,n_fwd.

    dwtlvn : int, shape (`nwl`), modified in place
        On exit: the number of coefficients in the second dimension for
        each coefficient type at each level. `dwtlvn`[i-1] contains the
        number of coefficients in the second dimension (for each
        coefficient type computed) at the (n_fwd-i+1)th level of
        resolution, for i = 1,2,...,n_fwd.

    dwtlvfr : int, shape (`nwl`), modified in place
        On exit: the number of coefficients in the third dimension for
        each coefficient type at each level. `dwtlvfr`[i-1] contains the
        number of coefficients in the third dimension (for each
        coefficient type computed) at the (n_fwd-i+1)th level of
        resolution, for i = 1,2,...,n_fwd.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim3_init``.

    Returns
    -------
    csize : int
        The minimum length needed for `c`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` = *<value>*, the value of n on
            initialization (see ``dim3_init``).

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` = *<value>*, the value of m on
            initialization (see ``dim3_init``).

        (`errno` 1)
            On entry, `fr` = *<value>*.

            Constraint: `fr` = *<value>*, the value of fr on
            initialization (see ``dim3_init``).

        (`errno` 5)
            On entry, `nwl` = *<value>* and nwlmax = *<value>* in
            ``dim3_init``.

            Constraint: `nwl` <= nwlmax in ``dim3_init``.

        (`errno` 5)
            On entry, `nwl` = *<value>*.

            Constraint: `nwl` >= 1.

        (`errno` 6)
            Either the communication array `comm`['icomm'] has been
            corrupted or there has not been a prior call to the
            initialization function ``dim3_init``.

        (`errno` 6)
            Either the initialization function was called with `wtrans`
            = 'S' or the communication array `comm`['icomm'] has been
            corrupted.
    """

    excs = []
    fun_name = 'naginterfaces.base.wav.dim3_multi_fwd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c09fcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, fr.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, sda, fr].
        utils._BYREF_INT, # Input, lda.
        utils._BYREF_INT, # Input, sda.
        utils._BYREF_INT, # Input, lenc.
        utils._EngineFloat64ArrayType.c_type, # Output, c[lenc].
        utils._BYREF_INT, # Input, nwl.
        utils._EngineIntArrayType.c_type, # Output, dwtlvm[nwl].
        utils._EngineIntArrayType.c_type, # Output, dwtlvn[nwl].
        utils._EngineIntArrayType.c_type, # Output, dwtlvfr[nwl].
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._BYREF_INT, # Output, csize.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _fr = utils._EngineIntScalarType(
        fr, {'fun_name': fun_name, 'entity': 'fr'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=3, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _sda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=3, sorder=_m_sorder, dim_type='s')
    _sda = utils._EngineIntScalarType(
        _sda_py, {'fun_name': fun_name, 'entity': '_sda'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _fr_py = utils._EngineIntScalarType.to_py(_fr)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_m_py, _n_py, _fr_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _lenc = utils._EngineIntScalarType(
        lenc, {'fun_name': fun_name, 'entity': 'lenc'},
    ).to_ctypes()
    _lenc_py = utils._EngineIntScalarType.to_py(_lenc)
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_lenc_py,),
        check_contents=False,
    )
    _c = _c_cls.to_ctypes(can_reference_contents=False)
    _nwl = utils._EngineIntScalarType(
        nwl, {'fun_name': fun_name, 'entity': 'nwl'},
    ).to_ctypes()
    _nwl_py = utils._EngineIntScalarType.to_py(_nwl)
    _dwtlvm_cls = utils._EngineIntArrayType(
        dwtlvm, {'fun_name': fun_name, 'entity': 'dwtlvm'},
        exp_shape=(_nwl_py,),
        check_contents=False,
    )
    _dwtlvm = _dwtlvm_cls.to_ctypes(can_reference_contents=False)
    _dwtlvn_cls = utils._EngineIntArrayType(
        dwtlvn, {'fun_name': fun_name, 'entity': 'dwtlvn'},
        exp_shape=(_nwl_py,),
        check_contents=False,
    )
    _dwtlvn = _dwtlvn_cls.to_ctypes(can_reference_contents=False)
    _dwtlvfr_cls = utils._EngineIntArrayType(
        dwtlvfr, {'fun_name': fun_name, 'entity': 'dwtlvfr'},
        exp_shape=(_nwl_py,),
        check_contents=False,
    )
    _dwtlvfr = _dwtlvfr_cls.to_ctypes(can_reference_contents=False)
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm',),
    )
    _licomm_py = utils._EngineIntArrayType.get_shape(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, exp_rank=1, validate=False)[0]
    _licomm = utils._EngineIntScalarType(
        _licomm_py, {'fun_name': fun_name, 'entity': '_licomm'},
    ).to_ctypes()
    _icomm = comm['icomm']
    _csize = utils._EngineIntScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _m,
        _n,
        _fr,
        _a,
        _lda,
        _sda,
        _lenc,
        _c,
        _nwl,
        _dwtlvm,
        _dwtlvn,
        _dwtlvfr,
        _icomm,
        _licomm,
        _csize,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n = {:s}, the value of n on'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'initialization (see dim3_init).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 201:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m = {:s}, the value of m on'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'initialization (see dim3_init).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 301:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, fr = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: fr = {:s}, the value of fr on'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'initialization (see dim3_init).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineNoErrorExit()
        elif _ierr_py == 5 and _ifmt == 105:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nwl = {:s} and nwlmax = {:s} in'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'dim3_init.',
                    'Constraint: nwl <= nwlmax in dim3_init.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 205:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nwl = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nwl >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 106:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the communication array comm[\'icomm\'] has been corrupted or there has not',
                    'been a prior call to the initialization function dim3_init.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 206:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function was called with wtrans = \'S\'',
                    'or the communication array comm[\'icomm\'] has been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _c_cls.refresh_py(_c)
        _dwtlvm_cls.refresh_py(_dwtlvm)
        _dwtlvn_cls.refresh_py(_dwtlvn)
        _dwtlvfr_cls.refresh_py(_dwtlvfr)
        _csize_py = utils._EngineIntScalarType.to_py(_csize)
        return_data = _csize_py
    _err_class.manage()
    return return_data

def dim3_mxolap_multi_inv(nwlinv, lenc, c, m, n, fr, b, comm):
    u"""
    Three-dimensional inverse multi-level discrete wavelet transform.

    For full information please refer to the NAG Library document for
    c09fd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09fdf.html

    Parameters
    ----------
    nwlinv : int
        The number of levels to be used in the inverse multi-level
        transform. The number of levels must be less than or equal to
        n_fwd, which has the value of argument nwl as used in the
        computation of the wavelet coefficients using
        ``dim3_multi_fwd``. The data will be reconstructed to level
        (nwl-`nwlinv`), where level 0 is the original input dataset
        provided to ``dim3_multi_fwd``.

    lenc : int
        The dimension of the array `c`.

    c : float, shape (`lenc`)
        The coefficients of the multi-level discrete wavelet transform.
        This will normally be the result of some transformation on the
        coefficients computed by function ``dim3_multi_fwd``.

        Note that the coefficients in `c` may be extracted according to
        level and type into three-dimensional arrays using
        ``dim3_coeff_ext``, and inserted using ``dim3_coeff_ins``.

    m : int
        The number of elements, m, in the first dimension of the
        reconstructed array B. For a full reconstruction of nwl levels,
        where nwl is as supplied to ``dim3_multi_fwd``, this must be the
        same as argument m used in a preceding call to
        ``dim3_multi_fwd``. For a partial reconstruction of `nwlinv` <
        nwl levels, this must be equal to dwtlvm[`nwlinv`], as returned
        from ``dim3_multi_fwd``

    n : int
        The number of elements, n, in the second dimension of the
        reconstructed array B. For a full reconstruction of nwl, levels,
        where nwl is as supplied to ``dim3_multi_fwd``, this must be the
        same as argument n used in a preceding call to
        ``dim3_multi_fwd``. For a partial reconstruction of `nwlinv` <
        nwl levels, this must be equal to dwtlvn[`nwlinv`], as returned
        from ``dim3_multi_fwd``.

    fr : int
        The number of elements, fr, in the third dimension of the
        reconstructed array B. For a full reconstruction of nwl levels,
        where nwl is as supplied to ``dim3_multi_fwd``, this must be the
        same as argument `fr` used in a preceding call to
        ``dim3_multi_fwd``. For a partial reconstruction of `nwlinv` <
        nwl levels, this must be equal to dwtlvfr[`nwlinv`], as returned
        from ``dim3_multi_fwd``.

    b : float, shape (`m`, `n`, `fr`), modified in place
        On exit: the m by n by fr reconstructed array, B, with B_{ijk}
        stored in `b`[i-1,j-1,k-1]. The reconstruction is based on the
        input multi-level wavelet transform coefficients and the
        transform options supplied to the initialization function
        ``dim3_init``.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim3_init``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nwlinv` = *<value>* and nwl = *<value>* where nwl
            is as used in the computation of the wavelet coefficients by
            a call to ``dim3_multi_fwd``.

            Constraint: `nwlinv` <= nwl as used in the call to
            ``dim3_multi_fwd``.

        (`errno` 1)
            On entry, `nwlinv` = *<value>*.

            Constraint: `nwlinv` >= 1.

        (`errno` 3)
            On entry, `lenc` = *<value>*.

            Constraint: `lenc` >= *<value>*, the number of wavelet
            coefficients required for a transform operating on `nwlinv`
            levels. If `nwlinv` = nwlmax, the maximum number of levels
            as returned by the initial call to ``dim3_init``, `lenc`
            must be at least n_ct, the value returned in nwct by the
            same call to ``dim3_init``.

        (`errno` 4)
            On entry, `n` = *<value>*.

            Constraint: `n` >= *<value>*, the number of coefficients in
            the second dimension at the required level of
            reconstruction.

        (`errno` 4)
            On entry, `m` = *<value>*.

            Constraint: `m` >= *<value>*, the number of coefficients in
            the first dimension at the required level of reconstruction.

        (`errno` 4)
            On entry, `fr` = *<value>*.

            Constraint: `fr` >= *<value>*, the number of coefficients in
            the third dimension at the required level of reconstruction.

        (`errno` 6)
            Either the communication array `comm`['icomm'] has been
            corrupted or there has not been a prior call to the
            initialization function ``dim3_init``.

        (`errno` 6)
            Either the initialization function was called with `wtrans`
            = 'S' or the communication array `comm`['icomm'] has been
            corrupted.
    """

    excs = []
    fun_name = 'naginterfaces.base.wav.dim3_mxolap_multi_inv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c09fdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, nwlinv.
        utils._BYREF_INT, # Input, lenc.
        utils._EngineFloat64ArrayType.c_type, # Input, c[lenc].
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, fr.
        utils._EngineFloat64ArrayType.c_type, # Output, b[ldb, sdb, fr].
        utils._BYREF_INT, # Input, ldb.
        utils._BYREF_INT, # Input, sdb.
        utils._EngineIntArrayType.c_type, # Input, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((b, 'b'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _nwlinv = utils._EngineIntScalarType(
        nwlinv, {'fun_name': fun_name, 'entity': 'nwlinv'},
    ).to_ctypes()
    _lenc = utils._EngineIntScalarType(
        lenc, {'fun_name': fun_name, 'entity': 'lenc'},
    ).to_ctypes()
    _lenc_py = utils._EngineIntScalarType.to_py(_lenc)
    _c = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_lenc_py,),
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _fr = utils._EngineIntScalarType(
        fr, {'fun_name': fun_name, 'entity': 'fr'},
    ).to_ctypes()
    _ldb_py = utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=3, sorder=_m_sorder, dim_type='l')
    _ldb = utils._EngineIntScalarType(
        _ldb_py, {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _sdb_py = utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=3, sorder=_m_sorder, dim_type='s')
    _sdb = utils._EngineIntScalarType(
        _sdb_py, {'fun_name': fun_name, 'entity': '_sdb'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _fr_py = utils._EngineIntScalarType.to_py(_fr)
    _b_cls = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_m_py, _n_py, _fr_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _b = _b_cls.to_ctypes(can_reference_contents=False)
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm',),
    )
    _licomm_py = utils._EngineIntArrayType.get_shape(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, exp_rank=1)[0]
    _licomm = utils._EngineIntScalarType(
        _licomm_py, {'fun_name': fun_name, 'entity': '_licomm'},
    ).to_ctypes()
    _icomm = comm['icomm']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _nwlinv,
        _lenc,
        _c,
        _m,
        _n,
        _fr,
        _b,
        _ldb,
        _sdb,
        _icomm,
        _licomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nwlinv = {:s} and nwl = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'where nwl is as used in the computation of the wavelet',
                    'coefficients by a call to dim3_multi_fwd.',
                    'Constraint: nwlinv <= nwl as used in the call to dim3_multi_fwd.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nwlinv = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nwlinv >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lenc = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lenc >= {:s}, the number of wavelet'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'coefficients required for a transform operating on nwlinv levels. If',
                    'nwlinv = nwlmax, the maximum number of levels as returned',
                    'by the initial call to dim3_init, lenc must be at least',
                    'n_ct, the value returned in nwct',
                    'by the same call to dim3_init.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 104:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= {:s}, the number of coefficients in the'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'second dimension at the required level of reconstruction.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 204:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= {:s}, the number of coefficients in the'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'first dimension at the required level of reconstruction.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 304:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, fr = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: fr >= {:s}, the number of coefficients in the'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'third dimension at the required level of reconstruction.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 106:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the communication array comm[\'icomm\'] has been corrupted or there has not',
                    'been a prior call to the initialization function dim3_init.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 206:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function was called with wtrans = \'S\'',
                    'or the communication array comm[\'icomm\'] has been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _b_cls.refresh_py(_b)
        return_data = None
    _err_class.manage()
    return return_data

def dim3_coeff_ext(ilev, cindex, lenc, c, d, comm):
    u"""
    Three-dimensional discrete wavelet transform coefficient extraction.

    For full information please refer to the NAG Library document for
    c09fy

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09fyf.html

    Parameters
    ----------
    ilev : int
        The level at which coefficients are to be extracted.

        If `ilev` = 0, it is assumed that the coefficient array `c` was
        produced by a preceding call to the single level function
        ``dim3_sngl_fwd``.

        If `ilev` > 0, it is assumed that the coefficient array `c` was
        produced by a preceding call to the multi-level function
        ``dim3_multi_fwd``.

    cindex : int
        Identifies which coefficients to extract. The coefficients are
        identified as follows:

        `cindex` = 0

          The approximation coefficients, produced by application of the
          low pass filter over columns, rows and frames of A (LLL).
          After a call to the multi-level transform function
          ``dim3_multi_fwd`` (which implies that `ilev` > 0) the
          approximation coefficients are available only for `ilev` =
          nwl, where nwl is the value used in a preceding call to
          ``dim3_multi_fwd``.

        `cindex` = 1

          The detail coefficients produced by applying the low pass
          filter over columns and rows of A and the high pass filter
          over frames (LLH).

        `cindex` = 2

          The detail coefficients produced by applying the low pass
          filter over columns, high pass filter over rows and low pass
          filter over frames of A (LHL).

        `cindex` = 3

          The detail coefficients produced by applying the low pass
          filter over columns of A and high pass filter over rows and
          frames (LHH).

        `cindex` = 4

          The detail coefficients produced by applying the high pass
          filter over columns of A and low pass filter over rows and
          frames (HLL).

        `cindex` = 5

          The detail coefficients produced by applying the high pass
          filter over columns, low pass filter over rows and high pass
          filter over frames of A (HLH).

        `cindex` = 6

          The detail coefficients produced by applying the high pass
          filter over columns and rows of A and the low pass filter over
          frames (HHL).

        `cindex` = 7

          The detail coefficients produced by applying the high pass
          filter over columns, rows and frames of A (HHH).

    lenc : int
        The dimension of the array `c`.

    c : float, shape (`lenc`)
        DWT coefficients, as computed by ``dim3_sngl_fwd`` or
        ``dim3_multi_fwd``.

    d : float, shape (n_cm, n_cn, n_cfr), modified in place
        On exit: the requested coefficients.

        If the DWT coefficients were computed by ``dim3_sngl_fwd`` then

        - if `cindex` = 0, the approximation coefficients are stored in
          `d`[i-1,j-1,k-1], for k = 1,2,...,n_cfr, for j = 1,2,...,n_cn,
          for i = 1,2,...,n_cm;

        - if 1 <= `cindex` <= 7, the detail coefficients, as indicated
          by `cindex`, are stored in `d`[i-1,j-1,k-1], for k =
          1,2,...,n_cfr, for j = 1,2,...,n_cn, for i = 1,2,...,n_cm.

        If the DWT coefficients were computed by ``dim3_multi_fwd`` then

        - if `cindex` = 0 and `ilev` = nwl, the approximation
          coefficients are stored in `d`[i-1,j-1,k-1], for k =
          1,2,...,n_cfr, for j = 1,2,...,n_cn, for i = 1,2,...,n_cm;

        - if 1 <= `cindex` <= 7, the detail coefficients, as indicated
          by `cindex`, for level `ilev` are stored in `d`[i-1,j-1,k-1],
          for k = 1,2,...,n_cfr, for j = 1,2,...,n_cn, for i =
          1,2,...,n_cm.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim3_init``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ilev` = *<value>*.

            Constraint: `ilev` = 0 following a call to the single level
            function ``dim3_sngl_fwd``.

        (`errno` 1)
            On entry, `ilev` = *<value>*.

            Constraint: `ilev` > 0 following a call to the multi-level
            function ``dim3_multi_fwd``.

        (`errno` 1)
            On entry, `ilev` = *<value>* and nwl = *<value>*.

            Constraint: `ilev` <= nwl, where nwl is the number of levels
            used in the call to ``dim3_multi_fwd``.

        (`errno` 2)
            On entry, `cindex` = *<value>*.

            Constraint: `cindex` >= 0.

        (`errno` 2)
            On entry, `cindex` = *<value>*.

            Constraint: `cindex` <= 7.

        (`errno` 3)
            On entry, `lenc` = *<value>* and n_ct = *<value>*.

            Constraint: `lenc` >= n_ct, where n_ct is the number of DWT
            coefficients computed in the preceding call to
            ``dim3_sngl_fwd``.

        (`errno` 3)
            On entry, `lenc` = *<value>* and n_ct = *<value>*.

            Constraint: `lenc` >= n_ct, where n_ct is the number of DWT
            coefficients computed in the preceding call to
            ``dim3_multi_fwd``.

        (`errno` 5)
            On entry, `ilev` = *<value>* and nwl = *<value>*, but
            `cindex` = 0.

            Constraint: `cindex` > 0 when `ilev` < nwl in the preceding
            call to ``dim3_multi_fwd``.

        (`errno` 6)
            Either the initialization function has not been called first
            or `comm`['icomm'] has been corrupted.
    """

    excs = []
    fun_name = 'naginterfaces.base.wav.dim3_coeff_ext'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c09fyft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ilev.
        utils._BYREF_INT, # Input, cindex.
        utils._BYREF_INT, # Input, lenc.
        utils._EngineFloat64ArrayType.c_type, # Input, c[lenc].
        utils._EngineFloat64ArrayType.c_type, # Output, d[ldd, sdd, *].
        utils._BYREF_INT, # Input, ldd.
        utils._BYREF_INT, # Input, sdd.
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((d, 'd'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _ilev = utils._EngineIntScalarType(
        ilev, {'fun_name': fun_name, 'entity': 'ilev'},
    ).to_ctypes()
    _cindex = utils._EngineIntScalarType(
        cindex, {'fun_name': fun_name, 'entity': 'cindex'},
    ).to_ctypes()
    _lenc = utils._EngineIntScalarType(
        lenc, {'fun_name': fun_name, 'entity': 'lenc'},
    ).to_ctypes()
    _lenc_py = utils._EngineIntScalarType.to_py(_lenc)
    _c = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_lenc_py,),
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm',),
    )
    _ilev_py = utils._EngineIntScalarType.to_py(_ilev)
    if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (2,), validate=False) == 1:
        _vl_n_cm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (9,), validate=False)
    else:
        _vl_n_cm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((100+utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (11,), validate=False)+1-_ilev_py)-1,), validate=False)
    if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (2,), validate=False) == 1:
        _vl_n_cn_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (7,), validate=False)
    else:
        _vl_n_cn_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((20+utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (11,), validate=False)+1-_ilev_py)-1,), validate=False)
    _ldd_py = utils._EngineFloat64ArrayType.get_nd(d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=3, sorder=_m_sorder, dim_type='l')
    _ldd = utils._EngineIntScalarType(
        _ldd_py, {'fun_name': fun_name, 'entity': '_ldd'},
    ).to_ctypes()
    _sdd_py = utils._EngineFloat64ArrayType.get_nd(d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=3, sorder=_m_sorder, dim_type='s')
    _sdd = utils._EngineIntScalarType(
        _sdd_py, {'fun_name': fun_name, 'entity': '_sdd'},
    ).to_ctypes()
    if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (2,), validate=False) == 1:
        _vl_n_cfr_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (12,), validate=False)
    else:
        _vl_n_cfr_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((180+utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (11,), validate=False)+1-_ilev_py)-1,), validate=False)
    _d_cls = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_vl_n_cm_py, _vl_n_cn_py, _vl_n_cfr_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _d = _d_cls.to_ctypes(can_reference_contents=False)
    _licomm_py = utils._EngineIntArrayType.get_shape(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, exp_rank=1, validate=False)[0]
    _licomm = utils._EngineIntScalarType(
        _licomm_py, {'fun_name': fun_name, 'entity': '_licomm'},
    ).to_ctypes()
    _icomm = comm['icomm']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ilev,
        _cindex,
        _lenc,
        _c,
        _d,
        _ldd,
        _sdd,
        _icomm,
        _licomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 200:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ilev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ilev = 0 following a call to the single level function dim3_sngl_fwd.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 300:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ilev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ilev > 0 following a call to the multi-level function dim3_multi_fwd.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 400:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ilev = {:s} and nwl = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ilev <= nwl, where nwl is the number of',
                    'levels used in the call to dim3_multi_fwd.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 100:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, cindex = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: cindex >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 200:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, cindex = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: cindex <= 7.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 100:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lenc = {:s} and n_ct = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: lenc >= n_ct, where n_ct is the number of DWT coefficients',
                    'computed in the preceding call to dim3_sngl_fwd.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 200:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lenc = {:s} and n_ct = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: lenc >= n_ct, where n_ct is the number of DWT coefficients',
                    'computed in the preceding call to dim3_multi_fwd.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 100:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ilev = {:s} and nwl = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'but cindex = 0.',
                    'Constraint: cindex > 0 when ilev < nwl in the preceding call to dim3_multi_fwd.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 100:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function has not been called first',
                    'or comm[\'icomm\'] has been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _d_cls.refresh_py(_d)
        return_data = None
    _err_class.manage()
    return return_data

def dim3_coeff_ins(ilev, cindex, lenc, c, d, comm):
    u"""
    Three-dimensional discrete wavelet transform coefficient insertion.

    For full information please refer to the NAG Library document for
    c09fz

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c09/c09fzf.html

    Parameters
    ----------
    ilev : int
        The level at which coefficients are to be inserted.

        If `ilev` = 0, it is assumed that the coefficient array `c` was
        produced by a preceding call to the single level function
        ``dim3_sngl_fwd``.

        If `ilev` > 0, it is assumed that the coefficient array `c` was
        produced by a preceding call to the multi-level function
        ``dim3_multi_fwd``.

    cindex : int
        Identifies which coefficients to insert. The coefficients are
        identified as follows:

        `cindex` = 0

          The approximation coefficients, produced by application of the
          low pass filter over columns, rows and frames of A (LLL).
          After a call to the multi-level transform function
          ``dim3_multi_fwd`` (which implies that `ilev` > 0) the
          approximation coefficients are present only for `ilev` = nwl,
          where nwl is the value used in a preceding call to
          ``dim3_multi_fwd``.

        `cindex` = 1

          The detail coefficients produced by applying the low pass
          filter over columns and rows of A and the high pass filter
          over frames (LLH).

        `cindex` = 2

          The detail coefficients produced by applying the low pass
          filter over columns, high pass filter over rows and low pass
          filter over frames of A (LHL).

        `cindex` = 3

          The detail coefficients produced by applying the low pass
          filter over columns of A and high pass filter over rows and
          frames (LHH).

        `cindex` = 4

          The detail coefficients produced by applying the high pass
          filter over columns of A and low pass filter over rows and
          frames (HLL).

        `cindex` = 5

          The detail coefficients produced by applying the high pass
          filter over columns, low pass filter over rows and high pass
          filter over frames of A (HLH).

        `cindex` = 6

          The detail coefficients produced by applying the high pass
          filter over columns and rows of A and the low pass filter over
          frames (HHL).

        `cindex` = 7

          The detail coefficients produced by applying the high pass
          filter over columns, rows and frames of A (HHH).

    lenc : int
        The dimension of the array `c`.

    c : float, shape (`lenc`), modified in place
        On entry: contains the DWT coefficients inserted by previous
        calls to ``dim3_coeff_ins``, or computed by a previous call to
        either ``dim3_sngl_fwd`` or ``dim3_multi_fwd``.

        On exit: contains the same DWT coefficients provided on entry
        except for those identified by `ilev` and `cindex`, which are
        updated with the values supplied in `d`, inserted into the
        correct locations as expected by one of the reconstruction
        functions ``dim3_sngl_inv`` (if ``dim3_sngl_fwd`` was called
        previously) or ``dim3_mxolap_multi_inv`` (if ``dim3_multi_fwd``
        was called previously).

    d : float, shape (n_cm, n_cn, n_cfr)
        The coefficients to be inserted.

        If the DWT coefficients were computed by ``dim3_sngl_fwd`` then

        - if `cindex` = 0, the approximation coefficients must be stored
          in `d`[i-1,j-1,k-1], for k = 1,2,...,n_cfr, for j =
          1,2,...,n_cn, for i = 1,2,...,n_cm;

        - if 1 <= `cindex` <= 7, the detail coefficients, as indicated
          by `cindex`, must be stored in `d`[i-1,j-1,k-1], for k =
          1,2,...,n_cfr, for j = 1,2,...,n_cn, for i = 1,2,...,n_cm.

        If the DWT coefficients were computed by ``dim3_multi_fwd`` then

        - if `cindex` = 0 and `ilev` = nwl, the approximation
          coefficients must be stored in `d`[i-1,j-1,k-1], for k =
          1,2,...,n_cfr, for j = 1,2,...,n_cn, for i = 1,2,...,n_cm;

        - if 1 <= `cindex` <= 7, the detail coefficients, as indicated
          by `cindex`, for level `ilev` must be stored in
          `d`[i-1,j-1,k-1], for k = 1,2,...,n_cfr, for j = 1,2,...,n_cn,
          for i = 1,2,...,n_cm.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``dim3_init``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ilev` = *<value>*.

            Constraint: `ilev` = 0 following a call to the single level
            function ``dim3_sngl_fwd``.

        (`errno` 1)
            On entry, `ilev` = *<value>*.

            Constraint: `ilev` > 0 following a call to the multi-level
            function ``dim3_multi_fwd``.

        (`errno` 1)
            On entry, `ilev` = *<value>* and nwl = *<value>*.

            Constraint: `ilev` <= nwl, where nwl is the number of levels
            used in the call to ``dim3_multi_fwd``.

        (`errno` 2)
            On entry, `cindex` = *<value>*.

            Constraint: `cindex` >= 0.

        (`errno` 2)
            On entry, `cindex` = *<value>*.

            Constraint: `cindex` <= 7.

        (`errno` 3)
            On entry, `lenc` = *<value>* and n_ct = *<value>*.

            Constraint: `lenc` >= n_ct, where n_ct is the number of DWT
            coefficients computed in a previous call to
            ``dim3_sngl_fwd``.

        (`errno` 3)
            On entry, `lenc` = *<value>* and n_ct = *<value>*.

            Constraint: `lenc` >= n_ct, where n_ct is the number of DWT
            coefficients computed in a previous call to
            ``dim3_multi_fwd``.

        (`errno` 5)
            On entry, `ilev` = *<value>* and nwl = *<value>*, but
            `cindex` = 0.

            Constraint: `cindex` > 0 when `ilev` < nwl in the preceding
            call to ``dim3_multi_fwd``.

        (`errno` 6)
            Either the initialization function has not been called first
            or `comm`['icomm'] has been corrupted.
    """

    excs = []
    fun_name = 'naginterfaces.base.wav.dim3_coeff_ins'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c09fzft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ilev.
        utils._BYREF_INT, # Input, cindex.
        utils._BYREF_INT, # Input, lenc.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c[lenc].
        utils._EngineFloat64ArrayType.c_type, # Input, d[ldd, sdd, *].
        utils._BYREF_INT, # Input, ldd.
        utils._BYREF_INT, # Input, sdd.
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((d, 'd'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _ilev = utils._EngineIntScalarType(
        ilev, {'fun_name': fun_name, 'entity': 'ilev'},
    ).to_ctypes()
    _cindex = utils._EngineIntScalarType(
        cindex, {'fun_name': fun_name, 'entity': 'cindex'},
    ).to_ctypes()
    _lenc = utils._EngineIntScalarType(
        lenc, {'fun_name': fun_name, 'entity': 'lenc'},
    ).to_ctypes()
    _lenc_py = utils._EngineIntScalarType.to_py(_lenc)
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(_lenc_py,),
    )
    _c = _c_cls.to_ctypes()
    _ldd_py = utils._EngineFloat64ArrayType.get_nd(d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=3, sorder=_m_sorder, dim_type='l')
    _ldd = utils._EngineIntScalarType(
        _ldd_py, {'fun_name': fun_name, 'entity': '_ldd'},
    ).to_ctypes()
    _sdd_py = utils._EngineFloat64ArrayType.get_nd(d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=3, sorder=_m_sorder, dim_type='s')
    _sdd = utils._EngineIntScalarType(
        _sdd_py, {'fun_name': fun_name, 'entity': '_sdd'},
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('icomm',),
    )
    _ilev_py = utils._EngineIntScalarType.to_py(_ilev)
    if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (2,), validate=False) == 1:
        _vl_n_cfr_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (12,), validate=False)
    else:
        _vl_n_cfr_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((180+utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (11,), validate=False)+1-_ilev_py)-1,), validate=False)
    if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (2,), validate=False) == 1:
        _vl_n_cm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (9,), validate=False)
    else:
        _vl_n_cm_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((100+utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (11,), validate=False)+1-_ilev_py)-1,), validate=False)
    if utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (2,), validate=False) == 1:
        _vl_n_cn_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (7,), validate=False)
    else:
        _vl_n_cn_py = utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, ((20+utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (11,), validate=False)+1-_ilev_py)-1,), validate=False)
    _d = utils._EngineFloat64ArrayType(
        d, {'fun_name': fun_name, 'entity': 'd'},
        exp_shape=(_vl_n_cm_py, _vl_n_cn_py, _vl_n_cfr_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _licomm_py = utils._EngineIntArrayType.get_shape(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, exp_rank=1, validate=False)[0]
    _licomm = utils._EngineIntScalarType(
        _licomm_py, {'fun_name': fun_name, 'entity': '_licomm'},
    ).to_ctypes()
    _icomm = comm['icomm']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ilev,
        _cindex,
        _lenc,
        _c,
        _d,
        _ldd,
        _sdd,
        _icomm,
        _licomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 200:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ilev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ilev = 0 following a call to the single level function dim3_sngl_fwd.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 300:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ilev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ilev > 0 following a call to the multi-level function dim3_multi_fwd.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 400:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ilev = {:s} and nwl = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ilev <= nwl, where nwl is the number of',
                    'levels used in the call to dim3_multi_fwd.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 100:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, cindex = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: cindex >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 200:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, cindex = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: cindex <= 7.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 100:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lenc = {:s} and n_ct = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: lenc >= n_ct, where n_ct is the number of DWT coefficients',
                    'computed in a previous call to dim3_sngl_fwd.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 200:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lenc = {:s} and n_ct = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: lenc >= n_ct, where n_ct is the number of DWT coefficients',
                    'computed in a previous call to dim3_multi_fwd.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 100:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ilev = {:s} and nwl = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'but cindex = 0.',
                    'Constraint: cindex > 0 when ilev < nwl in the preceding call to dim3_multi_fwd.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 100:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function has not been called first',
                    'or comm[\'icomm\'] has been corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _c_cls.refresh_py(_c)
        return_data = None
    _err_class.manage()
    return return_data
