# -*- coding: utf-8 -*-
# pylint: disable=chained-comparison,invalid-name,line-too-long,protected-access,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unused-import
u"""
Interfaces for the NAG Mark 27.0 `correg` Chapter.

``correg`` - Correlation and Regression Analysis

This module is concerned with two techniques

- correlation analysis and

- regression modelling,

both of which are concerned with determining the inter-relationships
among two or more variables.

Other modules of the NAG Library which cover similar problems are
submodule ``fit`` and submodule ``opt``. Submodule ``fit`` functions may
be used to fit linear models by criteria other than least squares, and
also for polynomial regression; submodule ``opt`` functions may be used
to fit nonlinear models and linearly constrained linear models.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02intro.html

See Also
--------
``naginterfaces.library.examples.correg`` :
    This subpackage contains examples for the ``correg`` module.
    See also the :ref:`library_correg_ex` subsection.
"""

# NAG Copyright 2017-2019.

import collections as _collections

from ..base import utils as _b_utils
from ..library import _utils

def corrmat_nearest(g, errtol=0.0, maxits=0, maxit=0):
    u"""
    Computes the nearest correlation matrix to a real square matrix,
    using the method of Qi and Sun.

    For full information please refer to the NAG Library document for
    g02aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02aaf.html

    Parameters
    ----------
    g : float, array-like, shape (n, n)
        G, the initial matrix.

    errtol : float, optional
        The termination tolerance for the Newton iteration. If `errtol`
        <= 0.0, n*sqrt(machine precision) is used.

    maxits : int, optional
        `maxits` specifies the maximum number of iterations used for the
        iterative scheme used to solve the linear algebraic equations at
        each Newton step.

        If `maxits` <= 0, 100 is used.

    maxit : int, optional
        Specifies the maximum number of Newton iterations.

        If `maxit` <= 0, 200 is used.

    Returns
    -------
    x : float, ndarray, shape (n, n)
        Contains the nearest correlation matrix.

    itera : int
        The number of Newton steps taken.

    feval : int
        The number of function evaluations of the dual problem.

    nrmgrd : float
        The norm of the gradient of the last Newton step.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n > 0.

        (`errno` 2)
            Newton iteration fails to converge in *<value>* iterations.

        (`errno` 4)
            An intermediate eigenproblem could not be solved. This
            should not occur. Please contact NAG with details of your
            call.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            Machine precision is limiting convergence.

            The array returned in `x` may still be of interest.

    See Also
    --------
    :meth:`naginterfaces.library.examples.correg.corrmat_nearest_ex.main`
    """

    from ..base.correg import corrmat_nearest as b_corrmat_nearest

    fun_name = 'naginterfaces.library.correg.corrmat_nearest'

    _g = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'g'},
        g,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_g, 'g'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_g, {'fun_name': fun_name, 'entity': 'g'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _x = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _n_py),
        sorder=_m_sorder,
    )
    try:
        _return_scalars = b_corrmat_nearest(
            _g,
            _n,
            errtol,
            maxits,
            maxit,
            _x,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _x_py = _x
    _itera_py = _return_scalars.itera
    _feval_py = _return_scalars.feval
    _nrmgrd_py = _return_scalars.nrmgrd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'itera', 'feval', 'nrmgrd'],
    )(
        _x_py,
        _itera_py,
        _feval_py,
        _nrmgrd_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def corrmat_nearest_bounded(g, opt, alpha=None, w=None, errtol=0.0, maxits=0, maxit=0):
    u"""
    Computes the nearest correlation matrix to a real square matrix,
    augmenting ``corrmat_nearest`` to incorporate weights and bounds.

    For full information please refer to the NAG Library document for
    g02ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02abf.html

    Parameters
    ----------
    g : float, array-like, shape (n, n)
        G, the initial matrix.

    opt : str, length 1
        Indicates the problem to be solved.

        `opt` = 'A'

          The lower bound problem is solved.

        `opt` = 'W'

          The weighted norm problem is solved.

        `opt` = 'B'

          Both problems are solved.

    alpha : None or float, optional
        The value of alpha.

        If `opt` = 'W', `alpha` need not be set.

    w : None or float, array-like, shape (n), optional
        The square roots of the diagonal elements of W, that is the
        diagonal of W^(1/2).

        If `opt` = 'A', `w` is not referenced and may be None.

    errtol : float, optional
        The termination tolerance for the Newton iteration. If `errtol`
        <= 0.0, n*sqrt(machine precision) is used.

    maxits : int, optional
        Specifies the maximum number of iterations to be used by the
        iterative scheme to solve the linear algebraic equations at each
        Newton step.

        If `maxits` <= 0, 2*n is used.

    maxit : int, optional
        Specifies the maximum number of Newton iterations.

        If `maxit` <= 0, 200 is used.

    Returns
    -------
    x : float, ndarray, shape (n, n)
        Contains the nearest correlation matrix.

    itera : int
        The number of Newton steps taken.

    feval : int
        The number of function evaluations of the dual problem.

    nrmgrd : float
        The norm of the gradient of the last Newton step.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n > 0.

        (`errno` 1)
            On entry, the value of `opt` is invalid.

            Constraint: `opt` = 'A', 'W' or 'B'.

        (`errno` 1)
            On entry, `alpha` = *<value>*.

            Constraint: 0.0 < `alpha` < 1.0.

        (`errno` 1)
            On entry, all elements of `w` were not positive.

            Constraint: `w`[i-1] > 0.0, for all i.

        (`errno` 2)
            Newton iteration fails to converge in *<value>* iterations.
            Increase `maxit` or check the call to the function.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            The machine precision is limiting convergence. In this
            instance the returned value of `x` may be useful.

        (`errno` 4)
            An intermediate eigenproblem could not be solved. This
            should not occur. Please contact NAG with details of your
            call.
    """

    from ..base.correg import corrmat_nearest_bounded as b_corrmat_nearest_bounded

    fun_name = 'naginterfaces.library.correg.corrmat_nearest_bounded'

    _g = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'g'},
        g,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_g, 'g'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_g, {'fun_name': fun_name, 'entity': 'g'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _opt = _b_utils._EngineCharScalarType(
        opt, {'fun_name': fun_name, 'entity': 'opt'}, exp_chrlen=1,
    ).to_ctypes()
    _opt_py = _b_utils._EngineCharScalarType.to_py(_opt)
    if _opt_py not in ('A', 'a'):
        if w is None:
            raise _b_utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'w'},
                'must not be None when opt not in (\'A\', \'a\')',
            )
    _w = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'w'},
        w,
        copy=True,
        allow_none=True,
    )
    _x = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _n_py),
        sorder=_m_sorder,
    )
    try:
        _return_scalars = b_corrmat_nearest_bounded(
            _g,
            _n,
            opt,
            alpha,
            _w,
            errtol,
            maxits,
            maxit,
            _x,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _x_py = _x
    _itera_py = _return_scalars.itera
    _feval_py = _return_scalars.feval
    _nrmgrd_py = _return_scalars.nrmgrd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'itera', 'feval', 'nrmgrd'],
    )(
        _x_py,
        _itera_py,
        _feval_py,
        _nrmgrd_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def corrmat_nearest_kfactor(g, k, errtol=0.0, maxit=0):
    u"""
    Computes the nearest correlation matrix with k-factor structure to a
    real square matrix.

    For full information please refer to the NAG Library document for
    g02ae

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02aef.html

    Parameters
    ----------
    g : float, array-like, shape (n, n)
        G, the initial matrix.

    k : int
        k, the number of factors and columns of X.

    errtol : float, optional
        The termination tolerance for the projected gradient norm. See
        references for further details. If `errtol` <= 0.0, 0.01 is
        used. This is often a suitable default value.

    maxit : int, optional
        Specifies the maximum number of iterations in the spectral
        projected gradient method.

        If `maxit` <= 0, 40000 is used.

    Returns
    -------
    x : float, ndarray, shape (n, `k`)
        Contains the matrix X.

    itera : int
        The number of steps taken in the spectral projected gradient
        method.

    feval : int
        The number of evaluations of ||G-XX^T+diag(XX^T-I)||_F.

    nrmpgd : float
        The norm of the projected gradient at the final iteration.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n > 0.

        (`errno` 1)
            On entry, `k` = *<value>* and n = *<value>*.

            Constraint: 0 < `k` <= n.

        (`errno` 2)
            Spectral gradient method fails to converge in *<value>*
            iterations.
    """

    from ..base.correg import corrmat_nearest_kfactor as b_corrmat_nearest_kfactor

    fun_name = 'naginterfaces.library.correg.corrmat_nearest_kfactor'

    _g = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'g'},
        g,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_g, 'g'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_g, {'fun_name': fun_name, 'entity': 'g'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _k = _b_utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = _b_utils._EngineIntScalarType.to_py(_k)
    _x = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _k_py),
        sorder=_m_sorder,
    )
    _return_scalars = b_corrmat_nearest_kfactor(
        _g,
        _n,
        _k,
        errtol,
        maxit,
        _x,
    )
    _x_py = _x
    _itera_py = _return_scalars.itera
    _feval_py = _return_scalars.feval
    _nrmpgd_py = _return_scalars.nrmpgd
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'itera', 'feval', 'nrmpgd'],
    )(
        _x_py,
        _itera_py,
        _feval_py,
        _nrmpgd_py,
    )
    return _return_data

def corrmat_h_weight(g, alpha, h, errtol=0.0, maxit=0):
    u"""
    Computes the nearest correlation matrix to a real square matrix,
    using element-wise weighting.

    For full information please refer to the NAG Library document for
    g02aj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02ajf.html

    Parameters
    ----------
    g : float, array-like, shape (n, n)
        G, the initial matrix.

    alpha : float
        The value of alpha.

        If `alpha` < 0.0, 0.0 is used.

    h : float, array-like, shape (n, n)
        The matrix of weights H.

    errtol : float, optional
        The termination tolerance for the iteration. If `errtol` <= 0.0,
        n*sqrt(machine precision) is used. See [Accuracy] for further
        details.

    maxit : int, optional
        Specifies the maximum number of iterations to be used.

        If `maxit` <= 0, 200 is used.

    Returns
    -------
    x : float, ndarray, shape (n, n)
        Contains the nearest correlation matrix.

    itera : int
        The number of iterations taken.

    norm : float
        The value of ||H.(G-X)||_F after the final iteration.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n > 0.

        (`errno` 5)
            On entry, `alpha` = *<value>*.

            Constraint: `alpha` < 1.0.

        (`errno` 6)
            On entry, one or more of the off-diagonal elements of H were
            negative.

        (`errno` 7)
            Function failed to converge in *<value>* iterations.

            Increase `maxit` or check the call to the function.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 8)
            Failure to solve intermediate eigenproblem. This should not
            occur. Please contact NAG with details of your call.
    """

    from ..base.correg import corrmat_h_weight as b_corrmat_h_weight

    fun_name = 'naginterfaces.library.correg.corrmat_h_weight'

    _g = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'g'},
        g,
        copy=True,
    )
    _h = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'h'},
        h,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_g, 'g'), (_h, 'h')),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_g, {'fun_name': fun_name, 'entity': 'g'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _x = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _n_py),
        sorder=_m_sorder,
    )
    try:
        _return_scalars = b_corrmat_h_weight(
            _g,
            _n,
            alpha,
            _h,
            errtol,
            maxit,
            _x,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _x_py = _x
    _itera_py = _return_scalars.itera
    _norm_py = _return_scalars.norm
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'itera', 'norm'],
    )(
        _x_py,
        _itera_py,
        _norm_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def corrmat_nearest_rank(g, rank, errtol=0.0, ranktol=0.0, maxits=0, maxit=0):
    u"""
    Computes the rank-constrained nearest correlation matrix to a real
    square matrix, using the method of Qi and Sun.

    For full information please refer to the NAG Library document for
    g02ak

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02akf.html

    Parameters
    ----------
    g : float, array-like, shape (n, n)
        G~, the initial matrix.

    rank : int
        r, the upper bound for the rank of X.

    errtol : float, optional
        The termination tolerance for the convergence measure of the
        objective function value.

        If `errtol` <= 0.0, then a value of 1.0e-5 is used.

        See [Algorithmic Details] for further details.

    ranktol : float, optional
        The feasibility tolerance for the rank constraint.

        If `ranktol` <= 0.0, sqrt(machine precision) is used.

        See [Algorithmic Details] for further details.

    maxits : int, optional
        Specifies the maximum number of iterations used for the
        majorization approach to solve penalized problems at each level
        of penalty parameter.

        If `maxits` <= 0, then a value of 100 is used.

    maxit : int, optional
        Specifies the maximum number of iterations for the penalty
        method, i.e., the maximum level of penalty parameter.

        If `maxit` <= 0, then a value of 200 is used.

    Returns
    -------
    x : float, ndarray, shape (n, n)
        X, the nearest correlation matrix of rank r.

    f : float
        The difference between X and G given by 1/2||X-G||_F^2.

    rankerr : float
        The rank error of X, defined as sum_i = r+1^n(lambda_i), given
        that lambda_i denote eigenvalues of X sorted in non-increasing
        order.

    nsub : int
        The total number of majorized problems that have been solved,
        i.e., the total number of calls for ``corrmat_nearest``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n > 0.

        (`errno` 4)
            On entry, `rank` = *<value>* and n = *<value>*.

            Constraint: 0 < `rank` <= n.

        (`errno` 5)
            Majorized penalty approach fails to converge in `maxit`
            level of penalty iterations.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 6)
            Convergence is limited by machine precision. The objective
            function value or rank is decreasing very slowly.

            The array returned in `x` may still be of interest.
    """

    from ..base.correg import corrmat_nearest_rank as b_corrmat_nearest_rank

    fun_name = 'naginterfaces.library.correg.corrmat_nearest_rank'

    _g = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'g'},
        g,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_g, 'g'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_g, {'fun_name': fun_name, 'entity': 'g'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _x = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _n_py),
        sorder=_m_sorder,
    )
    try:
        _return_scalars = b_corrmat_nearest_rank(
            _g,
            _n,
            rank,
            errtol,
            ranktol,
            maxits,
            maxit,
            _x,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _x_py = _x
    _f_py = _return_scalars.f
    _rankerr_py = _return_scalars.rankerr
    _nsub_py = _return_scalars.nsub
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'f', 'rankerr', 'nsub'],
    )(
        _x_py,
        _f_py,
        _rankerr_py,
        _nsub_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def corrmat_shrinking(g, k, errtol=0.0, eigtol=0.0):
    u"""
    Computes a correlation matrix from an approximate matrix with fixed
    submatrix.

    For full information please refer to the NAG Library document for
    g02an

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02anf.html

    Parameters
    ----------
    g : float, array-like, shape (n, n)
        G, the initial matrix.

    k : int
        k, the order of the leading principal submatrix A.

    errtol : float, optional
        The termination tolerance for the iteration.

        If `errtol` <= 0.0, sqrt(machine precision) is used.

        See [Accuracy] for further details.

    eigtol : float, optional
        The tolerance used in determining the definiteness of A.

        If lambda_min(A) > n*λ_max(A)*`eigtol`, where lambda_min(A) and
        lambda_max(A) denote the minimum and maximum eigenvalues of A
        respectively, A is positive definite.

        If `eigtol` <= 0, machine precision is used.

    Returns
    -------
    x : float, ndarray, shape (n, n)
        Contains the matrix X.

    alpha : float
        alpha.

    itera : int
        The number of iterations taken.

    eigmin : float
        The smallest eigenvalue of the leading principal submatrix A.

    norm : float
        The value of ||G-X||_F after the final iteration.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n > 0.

        (`errno` 3)
            On entry, `k` = *<value>* and n = *<value>*.

            Constraint: n >= `k` > 0.

        (`errno` 5)
            The k by k principal leading submatrix of the initial matrix
            G is not positive definite.

        (`errno` 6)
            Failure to solve intermediate eigenproblem. This should not
            occur. Please contact NAG.
    """

    from ..base.correg import corrmat_shrinking as b_corrmat_shrinking

    fun_name = 'naginterfaces.library.correg.corrmat_shrinking'

    _g = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'g'},
        g,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_g, 'g'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_g, {'fun_name': fun_name, 'entity': 'g'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _x = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _n_py),
        sorder=_m_sorder,
    )
    _return_scalars = b_corrmat_shrinking(
        _g,
        _n,
        k,
        errtol,
        eigtol,
        _x,
    )
    _x_py = _x
    _alpha_py = _return_scalars.alpha
    _itera_py = _return_scalars.itera
    _eigmin_py = _return_scalars.eigmin
    _norm_py = _return_scalars.norm
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'alpha', 'itera', 'eigmin', 'norm'],
    )(
        _x_py,
        _alpha_py,
        _itera_py,
        _eigmin_py,
        _norm_py,
    )
    return _return_data

def corrmat_target(g, theta, h, errtol=0.0, eigtol=0.0):
    u"""
    Computes a correlation matrix from an approximate one using a
    specified target matrix.

    For full information please refer to the NAG Library document for
    g02ap

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02apf.html

    Parameters
    ----------
    g : float, array-like, shape (n, n)
        G, the initial matrix.

    theta : float
        The value of theta. If `theta` < 0.0, 0.0 is used.

    h : float, array-like, shape (n, n)
        The matrix of weights H.

    errtol : float, optional
        The termination tolerance for the iteration.

        If `errtol` <= 0.0, sqrt(machine precision) is used.

        See [Accuracy] for further details.

    eigtol : float, optional
        The tolerance used in determining the definiteness of the target
        matrix T = H.G.

        If lambda_min(T) > n*λ_max(T)*`eigtol`, where lambda_min(T) and
        lambda_max(T) denote the minimum and maximum eigenvalues of T
        respectively, T is positive definite.

        If `eigtol` <= 0, machine precision is used.

    Returns
    -------
    h : float, ndarray, shape (n, n)
        A symmetric matrix 1/2(H+H^T) with its diagonal elements set to
        1.0.

    x : float, ndarray, shape (n, n)
        Contains the matrix X.

    alpha : float
        The constant alpha used in the formation of X.

    itera : int
        The number of iterations taken.

    eigmin : float
        The smallest eigenvalue of the target matrix T.

    norm : float
        The value of ||G-X||_F after the final iteration.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n > 0.

        (`errno` 3)
            On entry, `theta` = *<value>*.

            Constraint: `theta` < 1.0.

        (`errno` 6)
            The target matrix is not positive definite.

        (`errno` 7)
            Failure to solve intermediate eigenproblem. This should not
            occur. Please contact NAG.
    """

    from ..base.correg import corrmat_target as b_corrmat_target

    fun_name = 'naginterfaces.library.correg.corrmat_target'

    _g = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'g'},
        g,
        copy=True,
    )
    _h = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'h'},
        h,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_g, 'g'), (_h, 'h')),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_g, {'fun_name': fun_name, 'entity': 'g'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _x = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _n_py),
        sorder=_m_sorder,
    )
    _return_scalars = b_corrmat_target(
        _g,
        _n,
        theta,
        _h,
        errtol,
        eigtol,
        _x,
    )
    _h_py = _h
    _x_py = _x
    _alpha_py = _return_scalars.alpha
    _itera_py = _return_scalars.itera
    _eigmin_py = _return_scalars.eigmin
    _norm_py = _return_scalars.norm
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['h', 'x', 'alpha', 'itera', 'eigmin', 'norm'],
    )(
        _h_py,
        _x_py,
        _alpha_py,
        _itera_py,
        _eigmin_py,
        _norm_py,
    )
    return _return_data

def corrmat_fixed(g, alpha, h, m, errtol=0.0, maxit=0):
    u"""
    Computes the nearest correlation matrix to a real square matrix,
    with fixed elements.

    For full information please refer to the NAG Library document for
    g02as

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02asf.html

    Parameters
    ----------
    g : float, array-like, shape (n, n)
        G~, the initial matrix.

    alpha : float
        The value of alpha.

        If `alpha` < 0.0, a value of 0.0 is used.

    h : int, array-like, shape (n, n)
        The symmetric matrix H. If an element of H is 1 then the
        corresponding element in G is fixed in the output X. Only the
        strictly lower triangular part of H need be set.

    m : int
        The number of previous iterates to use in the Anderson
        acceleration. If `m` = 0, Anderson acceleration is not used. See
        [Accuracy] for further details.

        If `m` < 0, a value of 4 is used.

    errtol : float, optional
        The termination tolerance for the iteration.

        If `errtol` <= 0.0, sqrt(machine precision) is used.

        See [Accuracy] for further details.

    maxit : int, optional
        Specifies the maximum number of iterations.

        If `maxit` <= 0, a value of 200 is used.

    Returns
    -------
    x : float, ndarray, shape (n, n)
        Contains the matrix X.

    its : int
        The number of iterations taken.

    fnorm : float
        The value of ||G-X||_F after the final iteration.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n > 0.

        (`errno` 4)
            On entry, `alpha` = *<value>*.

            Constraint: `alpha` < 1.0.

        (`errno` 5)
            On entry, `m` = *<value>* and n = *<value>*.

            Constraint: `m` <= n*n.

        (`errno` 7)
            Function failed to converge in *<value>* iterations.

            A solution may not exist, however, try increasing `maxit`.

        (`errno` 8)
            Failure during Anderson acceleration.

            Consider setting `m` = 0 and recomputing.

        (`errno` 9)
            The fixed element G_{{ij}}, lies outside the interval
            [-1,1], for i = *<value>* and j = *<value>*.
    """

    from ..base.correg import corrmat_fixed as b_corrmat_fixed

    fun_name = 'naginterfaces.library.correg.corrmat_fixed'

    _g = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'g'},
        g,
        copy=True,
    )
    _h = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'h'},
        h,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_g, 'g'), (_h, 'h')),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_g, {'fun_name': fun_name, 'entity': 'g'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _x = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _n_py),
        sorder=_m_sorder,
    )
    _return_scalars = b_corrmat_fixed(
        _g,
        _n,
        alpha,
        _h,
        errtol,
        maxit,
        m,
        _x,
    )
    _x_py = _x
    _its_py = _return_scalars.its
    _fnorm_py = _return_scalars.fnorm
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'its', 'fnorm'],
    )(
        _x_py,
        _its_py,
        _fnorm_py,
    )
    return _return_data

def coeffs_pearson(x):
    u"""
    Pearson product-moment correlation coefficients, all variables, no
    missing values.

    For full information please refer to the NAG Library document for
    g02ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02baf.html

    Parameters
    ----------
    x : float, array-like, shape (n, m)
        `x`[i-1,j-1] must be set to x_{ij}, the ith observation on the
        jth variable, for j = 1,2,...,m, for i = 1,2,...,n.

    Returns
    -------
    xbar : float, ndarray, shape (m)
        The mean value, x__j, of the jth variable, for j = 1,2,...,m.

    std : float, ndarray, shape (m)
        The standard deviation, s_j, of the jth variable, for j =
        1,2,...,m.

    ssp : float, ndarray, shape (m, m)
        `ssp`[j-1,k-1] is the cross-product of deviations S_{jk}, for k
        = 1,2,...,m, for j = 1,2,...,m.

    r : float, ndarray, shape (m, m)
        `r`[j-1,k-1] is the product-moment correlation coefficient
        R_{jk} between the jth and kth variables, for k = 1,2,...,m, for
        j = 1,2,...,m.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m >= 2.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.correg import coeffs_pearson as b_coeffs_pearson

    fun_name = 'naginterfaces.library.correg.coeffs_pearson'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _xbar = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    _std = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    _ssp = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _m_py),
        sorder=_m_sorder,
    )
    _r = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _m_py),
        sorder=_m_sorder,
    )
    b_coeffs_pearson(
        _n,
        _m,
        _x,
        _xbar,
        _std,
        _ssp,
        _r,
    )
    _xbar_py = _xbar
    _std_py = _std
    _ssp_py = _ssp
    _r_py = _r
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['xbar', 'std', 'ssp', 'r'],
    )(
        _xbar_py,
        _std_py,
        _ssp_py,
        _r_py,
    )
    return _return_data

def coeffs_pearson_miss_case(x, miss, xmiss):
    u"""
    Pearson product-moment correlation coefficients, all variables,
    casewise treatment of missing values.

    For full information please refer to the NAG Library document for
    g02bb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02bbf.html

    Parameters
    ----------
    x : float, array-like, shape (n, m)
        `x`[i-1,j-1] must be set to x_{ij}, the value of the ith
        observation on the jth variable, for j = 1,2,...,m, for i =
        1,2,...,n.

    miss : int, array-like, shape (m)
        `miss`[j-1] must be set equal to 1 if a missing value, xm_j, is
        to be specified for the jth variable in the array `x`, or set
        equal to 0 otherwise. Values of `miss` must be given for all m
        variables in the array `x`.

    xmiss : float, array-like, shape (m)
        `xmiss`[j-1] must be set to the missing value, xm_j, to be
        associated with the jth variable in the array `x`, for those
        variables for which missing values are specified by means of the
        array `miss` (see [Accuracy]).

    Returns
    -------
    xbar : float, ndarray, shape (m)
        The mean value, x__j, of the jth variable, for j = 1,2,...,m.

    std : float, ndarray, shape (m)
        The standard deviation, s_j, of the jth variable, for j =
        1,2,...,m.

    ssp : float, ndarray, shape (m, m)
        `ssp`[j-1,k-1] is the cross-product of deviations S_{jk}, for k
        = 1,2,...,m, for j = 1,2,...,m.

    r : float, ndarray, shape (m, m)
        `r`[j-1,k-1] is the product-moment correlation coefficient
        R_{jk} between the jth and kth variables, for k = 1,2,...,m, for
        j = 1,2,...,m.

    ncases : int
        The number of cases actually used in the calculations (when
        cases involving missing values have been eliminated).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m >= 2.

        (`errno` 4)
            After observations with missing values were omitted, no
            cases remained.

        (`errno` 5)
            After observations with missing values were omitted, only
            one case remained.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.correg import coeffs_pearson_miss_case as b_coeffs_pearson_miss_case

    fun_name = 'naginterfaces.library.correg.coeffs_pearson_miss_case'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _miss = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'miss'},
        miss,
        copy=True,
    )
    _xmiss = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'xmiss'},
        xmiss,
        copy=True,
    )
    _xbar = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    _std = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    _ssp = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _m_py),
        sorder=_m_sorder,
    )
    _r = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _m_py),
        sorder=_m_sorder,
    )
    _return_scalar = b_coeffs_pearson_miss_case(
        _n,
        _m,
        _x,
        _miss,
        _xmiss,
        _xbar,
        _std,
        _ssp,
        _r,
    )
    _xbar_py = _xbar
    _std_py = _std
    _ssp_py = _ssp
    _r_py = _r
    _ncases_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['xbar', 'std', 'ssp', 'r', 'ncases'],
    )(
        _xbar_py,
        _std_py,
        _ssp_py,
        _r_py,
        _ncases_py,
    )
    return _return_data

def coeffs_pearson_miss_pair(n, x, miss, xmiss):
    u"""
    Pearson product-moment correlation coefficients, all variables,
    pairwise treatment of missing values.

    For full information please refer to the NAG Library document for
    g02bc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02bcf.html

    Parameters
    ----------
    n : int
        n, the number of observations or cases.

    x : float, array-like, shape (`n`, m)
        `x`[i-1,j-1] must be set to x_{ij}, the value of the ith
        observation on the jth variable, for j = 1,2,...,m, for i =
        1,2,...,n.

    miss : int, array-like, shape (m)
        `miss`[j-1] must be set equal to 1 if a missing value, xm_j, is
        to be specified for the jth variable in the array `x`, or set
        equal to 0 otherwise. Values of `miss` must be given for all m
        variables in the array `x`.

    xmiss : float, array-like, shape (m)
        `xmiss`[j-1] must be set to the missing value, xm_j, to be
        associated with the jth variable in the array `x`, for those
        variables for which missing values are specified by means of the
        array `miss` (see [Accuracy]).

    Returns
    -------
    xbar : float, ndarray, shape (m)
        The mean value, x__j, of the jth variable, for j = 1,2,...,m.

    std : float, ndarray, shape (m)
        The standard deviation, s_j, of the jth variable, for j =
        1,2,...,m.

    ssp : float, ndarray, shape (m, m)
        `ssp`[j-1,k-1] is the cross-product of deviations S_{jk}, for k
        = 1,2,...,m, for j = 1,2,...,m.

    r : float, ndarray, shape (m, m)
        `r`[j-1,k-1] is the product-moment correlation coefficient
        R_{jk} between the jth and kth variables, for k = 1,2,...,m, for
        j = 1,2,...,m.

    ncases : int
        The minimum number of cases used in the calculation of any of
        the sums of squares and cross-products and correlation
        coefficients (when cases involving missing values have been
        eliminated).

    cnt : float, ndarray, shape (m, m)
        `cnt`[j-1,k-1] is the number of cases, c_{jk}, actually used in
        the calculation of S_{jk}, and R_{jk}, the sum of cross-products
        and correlation coefficient for the jth and kth variables, for k
        = 1,2,...,m, for j = 1,2,...,m.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 2.

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m >= 2.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            After observations with missing values were omitted, fewer
            than two cases remained for at least one pair of variables.
            (The pairs of variables involved can be determined by
            examination of the contents of the array `cnt`). All means,
            standard deviations, sums of squares and cross-products, and
            correlation coefficients based on two or more cases are
            returned by the function even if `errno` = 4.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.correg import coeffs_pearson_miss_pair as b_coeffs_pearson_miss_pair

    fun_name = 'naginterfaces.library.correg.coeffs_pearson_miss_pair'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'),),
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _miss = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'miss'},
        miss,
        copy=False,
    )
    _xmiss = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'xmiss'},
        xmiss,
        copy=False,
    )
    _xbar = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    _std = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    _ssp = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _m_py),
        sorder=_m_sorder,
    )
    _r = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _m_py),
        sorder=_m_sorder,
    )
    _cnt = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _m_py),
        sorder=_m_sorder,
    )
    try:
        _return_scalar = b_coeffs_pearson_miss_pair(
            n,
            _m,
            _x,
            _miss,
            _xmiss,
            _xbar,
            _std,
            _ssp,
            _r,
            _cnt,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _xbar_py = _xbar
    _std_py = _std
    _ssp_py = _ssp
    _r_py = _r
    _ncases_py = _return_scalar
    _cnt_py = _cnt
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['xbar', 'std', 'ssp', 'r', 'ncases', 'cnt'],
    )(
        _xbar_py,
        _std_py,
        _ssp_py,
        _r_py,
        _ncases_py,
        _cnt_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def coeffs_zero(x):
    u"""
    Correlation-like coefficients (about zero), all variables, no
    missing values.

    For full information please refer to the NAG Library document for
    g02bd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02bdf.html

    Parameters
    ----------
    x : float, array-like, shape (n, m)
        `x`[i-1,j-1] must be set to the value of x_{ij}, the ith
        observation on the jth variable, for j = 1,2,...,m, for i =
        1,2,...,n.

    Returns
    -------
    xbar : float, ndarray, shape (m)
        `xbar`[j-1] contains the mean value, x__j, of the jth variable,
        for j = 1,2,...,m.

    std : float, ndarray, shape (m)
        The standard deviation, s_j, of the jth variable, for j =
        1,2,...,m.

    sspz : float, ndarray, shape (m, m)
        `sspz`[j-1,k-1] is the cross-product about zero, S~_{jk}, for k
        = 1,2,...,m, for j = 1,2,...,m.

    rz : float, ndarray, shape (m, m)
        `rz`[j-1,k-1] is the correlation-like coefficient, R~_{jk},
        between the jth and kth variables, for k = 1,2,...,m, for j =
        1,2,...,m.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m >= 2.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.correg import coeffs_zero as b_coeffs_zero

    fun_name = 'naginterfaces.library.correg.coeffs_zero'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _xbar = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    _std = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    _sspz = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _m_py),
        sorder=_m_sorder,
    )
    _rz = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _m_py),
        sorder=_m_sorder,
    )
    b_coeffs_zero(
        _n,
        _m,
        _x,
        _xbar,
        _std,
        _sspz,
        _rz,
    )
    _xbar_py = _xbar
    _std_py = _std
    _sspz_py = _sspz
    _rz_py = _rz
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['xbar', 'std', 'sspz', 'rz'],
    )(
        _xbar_py,
        _std_py,
        _sspz_py,
        _rz_py,
    )
    return _return_data

def coeffs_zero_miss_case(x, miss, xmiss):
    u"""
    Correlation-like coefficients (about zero), all variables, casewise
    treatment of missing values.

    For full information please refer to the NAG Library document for
    g02be

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02bef.html

    Parameters
    ----------
    x : float, array-like, shape (n, m)
        `x`[i-1,j-1] must be set to x_{ij}, the value of the ith
        observation on the jth variable, for j = 1,2,...,m, for i =
        1,2,...,n.

    miss : int, array-like, shape (m)
        `miss`[j-1] must be set equal to 1 if a missing value, xm_j, is
        to be specified for the jth variable in the array `x`, or set
        equal to 0 otherwise. Values of `miss` must be given for all m
        variables in the array `x`.

    xmiss : float, array-like, shape (m)
        `xmiss`[j-1] must be set to the missing value, xm_j, to be
        associated with the jth variable in the array `x`, for those
        variables for which missing values are specified by means of the
        array `miss` (see [Accuracy]).

    Returns
    -------
    xbar : float, ndarray, shape (m)
        The mean value, x__j, of the jth variable, for j = 1,2,...,m.

    std : float, ndarray, shape (m)
        The standard deviation, s_j, of the jth variable, for j =
        1,2,...,m.

    sspz : float, ndarray, shape (m, m)
        `sspz`[j-1,k-1] is the cross-product about zero, S~_{jk}, for k
        = 1,2,...,m, for j = 1,2,...,m.

    rz : float, ndarray, shape (m, m)
        `rz`[j-1,k-1] is the correlation-like coefficient, R~_{jk},
        between the jth and kth variables, for k = 1,2,...,m, for j =
        1,2,...,m.

    ncases : int
        The number of cases actually used in the calculations (when
        cases involving missing values have been eliminated).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m >= 2.

        (`errno` 4)
            After observations with missing values were omitted, no
            cases remained.

        (`errno` 5)
            After observations with missing values were omitted, only
            one case remained.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.correg import coeffs_zero_miss_case as b_coeffs_zero_miss_case

    fun_name = 'naginterfaces.library.correg.coeffs_zero_miss_case'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _miss = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'miss'},
        miss,
        copy=True,
    )
    _xmiss = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'xmiss'},
        xmiss,
        copy=True,
    )
    _xbar = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    _std = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    _sspz = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _m_py),
        sorder=_m_sorder,
    )
    _rz = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _m_py),
        sorder=_m_sorder,
    )
    _return_scalar = b_coeffs_zero_miss_case(
        _n,
        _m,
        _x,
        _miss,
        _xmiss,
        _xbar,
        _std,
        _sspz,
        _rz,
    )
    _xbar_py = _xbar
    _std_py = _std
    _sspz_py = _sspz
    _rz_py = _rz
    _ncases_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['xbar', 'std', 'sspz', 'rz', 'ncases'],
    )(
        _xbar_py,
        _std_py,
        _sspz_py,
        _rz_py,
        _ncases_py,
    )
    return _return_data

def coeffs_zero_miss_pair(x, miss, xmiss):
    u"""
    Correlation-like coefficients (about zero), all variables, pairwise
    treatment of missing values.

    For full information please refer to the NAG Library document for
    g02bf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02bff.html

    Parameters
    ----------
    x : float, array-like, shape (n, m)
        `x`[i-1,j-1] must be set to x_{ij}, the value of the ith
        observation on the jth variable, for j = 1,2,...,m, for i =
        1,2,...,n.

    miss : int, array-like, shape (m)
        `miss`[j-1] must be set equal to 1 if a missing value, xm_j, is
        to be specified for the jth variable in the array `x`, or set
        equal to 0 otherwise. Values of `miss` must be given for all m
        variables in the array `x`.

    xmiss : float, array-like, shape (m)
        `xmiss`[j-1] must be set to the missing value, xm_j, to be
        associated with the jth variable in the array `x`, for those
        variables for which missing values are specified by means of the
        array `miss` (see [Accuracy]).

    Returns
    -------
    xbar : float, ndarray, shape (m)
        The mean value, x__j, of the jth variable, for j = 1,2,...,m.

    std : float, ndarray, shape (m)
        The standard deviation, s_j, of the jth variable, for j =
        1,2,...,m.

    sspz : float, ndarray, shape (m, m)
        `sspz`[j-1,k-1] is the cross-product about zero, S~_{jk}, for k
        = 1,2,...,m, for j = 1,2,...,m.

    rz : float, ndarray, shape (m, m)
        `rz`[j-1,k-1] is the correlation-like coefficient, R~_{jk},
        between the jth and kth variables, for k = 1,2,...,m, for j =
        1,2,...,m.

    ncases : int
        The minimum number of cases used in the calculation of any of
        the sums of squares and cross-products and correlation-like
        coefficients (when cases involving missing values have been
        eliminated).

    cnt : float, ndarray, shape (m, m)
        `cnt`[j-1,k-1] is the number of cases, c_{jk}, actually used in
        the calculation of S~_{jk}, and R~_{jk}, the sum of
        cross-products and correlation-like coefficient for the jth and
        kth variables, for k = 1,2,...,m, for j = 1,2,...,m.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m >= 2.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            After observations with missing values were omitted, fewer
            than two cases remained for at least one pair of variables.
            (The pairs of variables involved can be determined by
            examination of the contents of the array `cnt`). All means,
            standard deviations, sums of squares and cross-products, and
            correlation-like coefficients based on two or more cases are
            returned by the function even if `errno` = 4.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.correg import coeffs_zero_miss_pair as b_coeffs_zero_miss_pair

    fun_name = 'naginterfaces.library.correg.coeffs_zero_miss_pair'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _miss = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'miss'},
        miss,
        copy=False,
    )
    _xmiss = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'xmiss'},
        xmiss,
        copy=False,
    )
    _xbar = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    _std = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    _sspz = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _m_py),
        sorder=_m_sorder,
    )
    _rz = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _m_py),
        sorder=_m_sorder,
    )
    _cnt = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _m_py),
        sorder=_m_sorder,
    )
    try:
        _return_scalar = b_coeffs_zero_miss_pair(
            _n,
            _m,
            _x,
            _miss,
            _xmiss,
            _xbar,
            _std,
            _sspz,
            _rz,
            _cnt,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _xbar_py = _xbar
    _std_py = _std
    _sspz_py = _sspz
    _rz_py = _rz
    _ncases_py = _return_scalar
    _cnt_py = _cnt
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['xbar', 'std', 'sspz', 'rz', 'ncases', 'cnt'],
    )(
        _xbar_py,
        _std_py,
        _sspz_py,
        _rz_py,
        _ncases_py,
        _cnt_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def coeffs_pearson_subset(x, kvar):
    u"""
    Pearson product-moment correlation coefficients, subset of
    variables, no missing values.

    For full information please refer to the NAG Library document for
    g02bg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02bgf.html

    Parameters
    ----------
    x : float, array-like, shape (n, m)
        `x`[i-1,j-1] must be set to x_{ij}, the value of the ith
        observation on the jth variable, for j = 1,2,...,m, for i =
        1,2,...,n.

    kvar : int, array-like, shape (nvars)
        `kvar`[j] must be set to the column number in `x` of the jth
        variable for which information is required, for j = 0,...,p-1.

    Returns
    -------
    xbar : float, ndarray, shape (nvars)
        The mean value, x__j, of the variable specified in `kvar`[j-1],
        for j = 1,2,...,p.

    std : float, ndarray, shape (nvars)
        The standard deviation, s_j, of the variable specified in
        `kvar`[j-1], for j = 1,2,...,p.

    ssp : float, ndarray, shape (nvars, nvars)
        `ssp`[j-1,k-1] is the cross-product of deviations, S_{jk}, for
        the variables specified in `kvar`[j-1] and `kvar`[k-1], for k =
        1,2,...,p, for j = 1,2,...,p.

    r : float, ndarray, shape (nvars, nvars)
        `r`[j-1,k-1] is the product-moment correlation coefficient,
        R_{jk}, between the variables specified in `kvar`[j-1] and
        `kvar`[k-1], for k = 1,2,...,p, for j = 1,2,...,p.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, nvars = *<value>* and m = *<value>*.

            Constraint: nvars >= 2 and nvars <= m.

        (`errno` 4)
            On entry, i = *<value>*, `kvar`[i-1] = *<value>* and m =
            *<value>*.

            Constraint: 1 <= `kvar`[i-1] <= m.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.correg import coeffs_pearson_subset as b_coeffs_pearson_subset

    fun_name = 'naginterfaces.library.correg.coeffs_pearson_subset'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _kvar = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'kvar'},
        kvar,
        copy=False,
    )
    _nvars_py = _b_utils._EngineIntArrayType.get_shape(_kvar, {'fun_name': fun_name, 'entity': 'kvar'}, exp_rank=1)[0]
    _nvars = _b_utils._EngineIntScalarType(
        _nvars_py, {'fun_name': fun_name, 'entity': '_nvars'},
    ).to_ctypes()
    _xbar = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nvars_py,),
    )
    _std = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nvars_py,),
    )
    _ssp = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nvars_py, _nvars_py),
        sorder=_m_sorder,
    )
    _r = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nvars_py, _nvars_py),
        sorder=_m_sorder,
    )
    b_coeffs_pearson_subset(
        _n,
        _m,
        _x,
        _nvars,
        _kvar,
        _xbar,
        _std,
        _ssp,
        _r,
    )
    _xbar_py = _xbar
    _std_py = _std
    _ssp_py = _ssp
    _r_py = _r
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['xbar', 'std', 'ssp', 'r'],
    )(
        _xbar_py,
        _std_py,
        _ssp_py,
        _r_py,
    )
    return _return_data

def coeffs_pearson_subset_miss_case(x, miss, xmiss, mistyp, kvar):
    u"""
    Pearson product-moment correlation coefficients, subset of
    variables, casewise treatment of missing values.

    For full information please refer to the NAG Library document for
    g02bh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02bhf.html

    Parameters
    ----------
    x : float, array-like, shape (n, m)
        `x`[i-1,j-1] must be set to x_{ij}, the value of the ith
        observation on the jth variable, for j = 1,2,...,m, for i =
        1,2,...,n.

    miss : int, array-like, shape (m)
        `miss`[j-1] must be set equal to 1 if a missing value, xm_j, is
        to be specified for the jth variable in the array `x`, or set
        equal to 0 otherwise. Values of `miss` must be given for all m
        variables in the array `x`.

    xmiss : float, array-like, shape (m)
        `xmiss`[j-1] must be set to the missing value, xm_j, to be
        associated with the jth variable in the array `x`, for those
        variables for which missing values are specified by means of the
        array `miss` (see [Accuracy]).

    mistyp : int
        Indicates the manner in which missing observations are to be
        treated.

        `mistyp` = 1

          A case is excluded if it contains a missing value for any of
          the variables 1,2,...,m.

        `mistyp` = 0

          A case is excluded if it contains a missing value for any of
          the p(<= m) variables specified in the array `kvar`.

    kvar : int, array-like, shape (nvars)
        `kvar`[j] must be set to the column number in `x` of the jth
        variable for which information is required, for j = 0,...,p-1.

    Returns
    -------
    xbar : float, ndarray, shape (nvars)
        The mean value, of x__j, of the variable specified in
        `kvar`[j-1], for j = 1,2,...,p.

    std : float, ndarray, shape (nvars)
        The standard deviation, s_j, of the variable specified in
        `kvar`[j-1], for j = 1,2,...,p.

    ssp : float, ndarray, shape (nvars, nvars)
        `ssp`[j-1,k-1] is the cross-product of deviations, S_{jk}, for
        the variables specified in `kvar`[j-1] and `kvar`[k-1], for k =
        1,2,...,p, for j = 1,2,...,p.

    r : float, ndarray, shape (nvars, nvars)
        `r`[j-1,k-1] is the product-moment correlation coefficient,
        R_{jk}, between the variables specified in `kvar`[j-1] and
        `kvar`[k-1], for k = 1,2,...,p, for j = 1,2,...,p.

    ncases : int
        The number of cases actually used in the calculations (when
        cases involving missing values have been eliminated).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, nvars = *<value>* and m = *<value>*.

            Constraint: nvars >= 2 and nvars <= m.

        (`errno` 4)
            On entry, i = *<value>*, `kvar`[i-1] = *<value>* and m =
            *<value>*.

            Constraint: 1 <= `kvar`[i-1] <= m.

        (`errno` 5)
            On entry, `mistyp` = *<value>*.

            Constraint: `mistyp` = 0 or 1.

        (`errno` 6)
            After observations with missing values were omitted, no
            cases remained.

        (`errno` 7)
            After observations with missing values were omitted, only
            one case remained.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.correg import coeffs_pearson_subset_miss_case as b_coeffs_pearson_subset_miss_case

    fun_name = 'naginterfaces.library.correg.coeffs_pearson_subset_miss_case'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _miss = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'miss'},
        miss,
        copy=True,
    )
    _xmiss = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'xmiss'},
        xmiss,
        copy=True,
    )
    _kvar = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'kvar'},
        kvar,
        copy=False,
    )
    _nvars_py = _b_utils._EngineIntArrayType.get_shape(_kvar, {'fun_name': fun_name, 'entity': 'kvar'}, exp_rank=1)[0]
    _nvars = _b_utils._EngineIntScalarType(
        _nvars_py, {'fun_name': fun_name, 'entity': '_nvars'},
    ).to_ctypes()
    _xbar = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nvars_py,),
    )
    _std = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nvars_py,),
    )
    _ssp = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nvars_py, _nvars_py),
        sorder=_m_sorder,
    )
    _r = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nvars_py, _nvars_py),
        sorder=_m_sorder,
    )
    _return_scalar = b_coeffs_pearson_subset_miss_case(
        _n,
        _m,
        _x,
        _miss,
        _xmiss,
        mistyp,
        _nvars,
        _kvar,
        _xbar,
        _std,
        _ssp,
        _r,
    )
    _xbar_py = _xbar
    _std_py = _std
    _ssp_py = _ssp
    _r_py = _r
    _ncases_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['xbar', 'std', 'ssp', 'r', 'ncases'],
    )(
        _xbar_py,
        _std_py,
        _ssp_py,
        _r_py,
        _ncases_py,
    )
    return _return_data

def coeffs_pearson_subset_miss_pair(x, miss, xmiss, kvar):
    u"""
    Pearson product-moment correlation coefficients, subset of
    variables, pairwise treatment of missing values.

    For full information please refer to the NAG Library document for
    g02bj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02bjf.html

    Parameters
    ----------
    x : float, array-like, shape (n, m)
        `x`[i-1,j-1] must be set to x_{ij}, the value of the ith
        observation on the jth variable, for j = 1,2,...,m, for i =
        1,2,...,n.

    miss : int, array-like, shape (m)
        `miss`[j-1] must be set equal to 1 if a missing value, xm_j, is
        to be specified for the jth variable in the array `x`, or set
        equal to 0 otherwise. Values of `miss` must be given for all m
        variables in the array `x`.

    xmiss : float, array-like, shape (m)
        `xmiss`[j-1] must be set to the missing value, xm_j, to be
        associated with the jth variable in the array `x`, for those
        variables for which missing values are specified by means of the
        array `miss` (see [Accuracy]).

    kvar : int, array-like, shape (nvars)
        `kvar`[j] must be set to the column number in `x` of the jth
        variable for which information is required, for j = 0,...,p-1.

    Returns
    -------
    xbar : float, ndarray, shape (nvars)
        The mean value, x__j, of the variable specified in `kvar`[j-1],
        for j = 1,2,...,p.

    std : float, ndarray, shape (nvars)
        The standard deviation, s_j, of the variable specified in
        `kvar`[j-1], for j = 1,2,...,p.

    ssp : float, ndarray, shape (nvars, nvars)
        `ssp`[j-1,k-1] is the cross-product of deviations, S_{jk}, for
        the variables specified in `kvar`[j-1] and `kvar`[k-1], for k =
        1,2,...,p, for j = 1,2,...,p.

    r : float, ndarray, shape (nvars, nvars)
        `r`[j-1,k-1] is the product-moment correlation coefficient,
        R_{jk}, between the variables specified in `kvar`[j-1] and
        `kvar`[k-1], for k = 1,2,...,p, for j = 1,2,...,p.

    ncases : int
        The minimum number of cases used in the calculation of any of
        the sums of squares and cross-products and correlation
        coefficients (when cases involving missing values have been
        eliminated).

    cnt : float, ndarray, shape (nvars, nvars)
        `cnt`[j-1,k-1] is the number of cases, c_{jk}, actually used in
        the calculation of S_{jk}, and R_{jk}, the sum of cross-products
        and correlation coefficient for the variables specified in
        `kvar`[j-1] and `kvar`[k-1], for k = 1,2,...,p, for j =
        1,2,...,p.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, nvars = *<value>* and m = *<value>*.

            Constraint: nvars >= 2 and nvars <= m.

        (`errno` 4)
            On entry, i = *<value>*, `kvar`[i-1] = *<value>* and m =
            *<value>*.

            Constraint: 1 <= `kvar`[i-1] <= m.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 5)
            After observations with missing values were omitted, fewer
            than two cases remained.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.correg import coeffs_pearson_subset_miss_pair as b_coeffs_pearson_subset_miss_pair

    fun_name = 'naginterfaces.library.correg.coeffs_pearson_subset_miss_pair'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _miss = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'miss'},
        miss,
        copy=False,
    )
    _xmiss = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'xmiss'},
        xmiss,
        copy=False,
    )
    _kvar = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'kvar'},
        kvar,
        copy=False,
    )
    _nvars_py = _b_utils._EngineIntArrayType.get_shape(_kvar, {'fun_name': fun_name, 'entity': 'kvar'}, exp_rank=1)[0]
    _nvars = _b_utils._EngineIntScalarType(
        _nvars_py, {'fun_name': fun_name, 'entity': '_nvars'},
    ).to_ctypes()
    _xbar = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nvars_py,),
    )
    _std = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nvars_py,),
    )
    _ssp = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nvars_py, _nvars_py),
        sorder=_m_sorder,
    )
    _r = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nvars_py, _nvars_py),
        sorder=_m_sorder,
    )
    _cnt = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nvars_py, _nvars_py),
        sorder=_m_sorder,
    )
    try:
        _return_scalar = b_coeffs_pearson_subset_miss_pair(
            _n,
            _m,
            _x,
            _miss,
            _xmiss,
            _nvars,
            _kvar,
            _xbar,
            _std,
            _ssp,
            _r,
            _cnt,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _xbar_py = _xbar
    _std_py = _std
    _ssp_py = _ssp
    _r_py = _r
    _ncases_py = _return_scalar
    _cnt_py = _cnt
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['xbar', 'std', 'ssp', 'r', 'ncases', 'cnt'],
    )(
        _xbar_py,
        _std_py,
        _ssp_py,
        _r_py,
        _ncases_py,
        _cnt_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def coeffs_zero_subset(x, kvar):
    u"""
    Correlation-like coefficients (about zero), subset of variables, no
    missing values.

    For full information please refer to the NAG Library document for
    g02bk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02bkf.html

    Parameters
    ----------
    x : float, array-like, shape (n, m)
        `x`[i-1,j-1] must be set to x_{ij}, the value of the ith
        observation on the jth variable, for j = 1,2,...,m, for i =
        1,2,...,n.

    kvar : int, array-like, shape (nvars)
        `kvar`[j] must be set to the column number in `x` of the jth
        variable for which information is required, for j = 0,...,p-1.

    Returns
    -------
    xbar : float, ndarray, shape (nvars)
        The mean value, x__j, of the variable specified in `kvar`[j-1],
        for j = 1,2,...,p.

    std : float, ndarray, shape (nvars)
        The standard deviation, s_j, of the variable specified in
        `kvar`[j-1], for j = 1,2,...,p.

    sspz : float, ndarray, shape (nvars, nvars)
        `sspz`[j-1,k-1] is the cross-product about zero, S~_{jk}, for
        the variables specified in `kvar`[j-1] and `kvar`[k-1], for k =
        1,2,...,p, for j = 1,2,...,p.

    rz : float, ndarray, shape (nvars, nvars)
        `rz`[j-1,k-1] is the correlation-like coefficient, R~_{jk},
        between the variables specified in `kvar`[j-1] and `kvar`[k-1],
        for k = 1,2,...,p, for j = 1,2,...,p.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, nvars = *<value>* and m = *<value>*.

            Constraint: nvars >= 2 and nvars <= m.

        (`errno` 4)
            On entry, i = *<value>*, `kvar`[i-1] = *<value>* and m =
            *<value>*.

            Constraint: 1 <= `kvar`[i-1] <= m.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.correg import coeffs_zero_subset as b_coeffs_zero_subset

    fun_name = 'naginterfaces.library.correg.coeffs_zero_subset'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _kvar = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'kvar'},
        kvar,
        copy=False,
    )
    _nvars_py = _b_utils._EngineIntArrayType.get_shape(_kvar, {'fun_name': fun_name, 'entity': 'kvar'}, exp_rank=1)[0]
    _nvars = _b_utils._EngineIntScalarType(
        _nvars_py, {'fun_name': fun_name, 'entity': '_nvars'},
    ).to_ctypes()
    _xbar = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nvars_py,),
    )
    _std = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nvars_py,),
    )
    _sspz = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nvars_py, _nvars_py),
        sorder=_m_sorder,
    )
    _rz = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nvars_py, _nvars_py),
        sorder=_m_sorder,
    )
    b_coeffs_zero_subset(
        _n,
        _m,
        _x,
        _nvars,
        _kvar,
        _xbar,
        _std,
        _sspz,
        _rz,
    )
    _xbar_py = _xbar
    _std_py = _std
    _sspz_py = _sspz
    _rz_py = _rz
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['xbar', 'std', 'sspz', 'rz'],
    )(
        _xbar_py,
        _std_py,
        _sspz_py,
        _rz_py,
    )
    return _return_data

def coeffs_zero_subset_miss_case(x, miss, xmiss, mistyp, kvar):
    u"""
    Correlation-like coefficients (about zero), subset of variables,
    casewise treatment of missing values.

    For full information please refer to the NAG Library document for
    g02bl

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02blf.html

    Parameters
    ----------
    x : float, array-like, shape (n, m)
        `x`[i-1,j-1] must be set to x_{ij}, the value of the ith
        observation on the jth variable, for j = 1,2,...,m, for i =
        1,2,...,n.

    miss : int, array-like, shape (m)
        `miss`[j-1] must be set equal to 1 if a missing value, xm_j, is
        to be specified for the jth variable in the array `x`, or set
        equal to 0 otherwise. Values of `miss` must be given for all m
        variables in the array `x`.

    xmiss : float, array-like, shape (m)
        `xmiss`[j-1] must be set to the missing value, xm_j, to be
        associated with the jth variable in the array `x`, for those
        variables for which missing values are specified by means of the
        array `miss` (see [Accuracy]).

    mistyp : int
        Indicates the manner in which missing observations are to be
        treated.

        `mistyp` = 1

          A case is excluded if it contains a missing value for any of
          the variables 1,2,...,m.

        `mistyp` = 0

          A case is excluded if it contains a missing value for any of
          the p(<= m) variables specified in the array `kvar`.

    kvar : int, array-like, shape (nvars)
        `kvar`[j] must be set to the column number in `x` of the jth
        variable for which information is required, for j = 0,...,p-1.

    Returns
    -------
    xbar : float, ndarray, shape (nvars)
        The mean value, x__j, of the variable specified in `kvar`[j-1],
        for j = 1,2,...,p.

    std : float, ndarray, shape (nvars)
        The standard deviation, s_j, of the variable specified in
        `kvar`[j-1], for j = 1,2,...,p.

    sspz : float, ndarray, shape (nvars, nvars)
        `sspz`[j-1,k-1] is the cross-product about zero, S~_{jk}, for
        the variables specified in `kvar`[j-1] and `kvar`[k-1], for k =
        1,2,...,p, for j = 1,2,...,p.

    rz : float, ndarray, shape (nvars, nvars)
        `rz`[j-1,k-1] is the correlation-like coefficient, R~_{jk},
        between the variables specified in `kvar`[j-1] and `kvar`[k-1],
        for k = 1,2,...,p, for j = 1,2,...,p.

    ncases : int
        The number of cases actually used in the calculations (when
        cases involving missing values have been eliminated).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, nvars = *<value>* and m = *<value>*.

            Constraint: nvars >= 2 and nvars <= m.

        (`errno` 4)
            On entry, i = *<value>*, `kvar`[i-1] = *<value>* and m =
            *<value>*.

            Constraint: 1 <= `kvar`[i-1] <= m.

        (`errno` 5)
            On entry, `mistyp` = *<value>*.

            Constraint: `mistyp` = 0 or 1.

        (`errno` 6)
            After observations with missing values were omitted, no
            cases remained.

        (`errno` 7)
            After observations with missing values were omitted, only
            one case remained.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.correg import coeffs_zero_subset_miss_case as b_coeffs_zero_subset_miss_case

    fun_name = 'naginterfaces.library.correg.coeffs_zero_subset_miss_case'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _miss = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'miss'},
        miss,
        copy=True,
    )
    _xmiss = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'xmiss'},
        xmiss,
        copy=True,
    )
    _kvar = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'kvar'},
        kvar,
        copy=False,
    )
    _nvars_py = _b_utils._EngineIntArrayType.get_shape(_kvar, {'fun_name': fun_name, 'entity': 'kvar'}, exp_rank=1)[0]
    _nvars = _b_utils._EngineIntScalarType(
        _nvars_py, {'fun_name': fun_name, 'entity': '_nvars'},
    ).to_ctypes()
    _xbar = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nvars_py,),
    )
    _std = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nvars_py,),
    )
    _sspz = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nvars_py, _nvars_py),
        sorder=_m_sorder,
    )
    _rz = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nvars_py, _nvars_py),
        sorder=_m_sorder,
    )
    _return_scalar = b_coeffs_zero_subset_miss_case(
        _n,
        _m,
        _x,
        _miss,
        _xmiss,
        mistyp,
        _nvars,
        _kvar,
        _xbar,
        _std,
        _sspz,
        _rz,
    )
    _xbar_py = _xbar
    _std_py = _std
    _sspz_py = _sspz
    _rz_py = _rz
    _ncases_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['xbar', 'std', 'sspz', 'rz', 'ncases'],
    )(
        _xbar_py,
        _std_py,
        _sspz_py,
        _rz_py,
        _ncases_py,
    )
    return _return_data

def coeffs_zero_subset_miss_pair(x, miss, xmiss, kvar):
    u"""
    Correlation-like coefficients (about zero), subset of variables,
    pairwise treatment of missing values.

    For full information please refer to the NAG Library document for
    g02bm

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02bmf.html

    Parameters
    ----------
    x : float, array-like, shape (n, m)
        `x`[i-1,j-1] must be set to x_{ij}, the value of the ith
        observation on the jth variable, for j = 1,2,...,m, for i =
        1,2,...,n.

    miss : int, array-like, shape (m)
        `miss`[j-1] must be set equal to 1 if a missing value, xm_j, is
        to be specified for the jth variable in the array `x`, or set
        equal to 0 otherwise. Values of `miss` must be given for all m
        variables in the array `x`.

    xmiss : float, array-like, shape (m)
        `xmiss`[j-1] must be set to the missing value, xm_j, to be
        associated with the jth variable in the array `x`, for those
        variables for which missing values are specified by means of the
        array `miss` (see [Accuracy]).

    kvar : int, array-like, shape (nvars)
        `kvar`[j] must be set to the column number in `x` of the jth
        variable for which information is required, for j = 0,...,p-1.

    Returns
    -------
    xbar : float, ndarray, shape (nvars)
        The mean value, x__j, of the variable specified in `kvar`[j-1],
        for j = 1,2,...,p.

    std : float, ndarray, shape (nvars)
        The standard deviation, s_j, of the variable specified in
        `kvar`[j-1], for j = 1,2,...,p.

    sspz : float, ndarray, shape (nvars, nvars)
        `sspz`[j-1,k-1] is the cross-product about zero, S~_{jk}, for
        the variables specified in `kvar`[j-1] and `kvar`[k-1], for k =
        1,2,...,p, for j = 1,2,...,p.

    rz : float, ndarray, shape (nvars, nvars)
        `rz`[j-1,k-1] is the correlation-like coefficient, R~_{jk},
        between the variables specified in `kvar`[j-1] and `kvar`[k-1],
        for k = 1,2,...,p, for j = 1,2,...,p.

    ncases : int
        The minimum number of cases used in the calculation of any of
        the sums of squares and cross-products and correlation-like
        coefficients (when cases involving missing values have been
        eliminated).

    cnt : float, ndarray, shape (nvars, nvars)
        `cnt`[j-1,k-1] is the number of cases, c_{jk}, actually used in
        the calculation of the sum of cross-product and correlation-like
        coefficient for the variables specified in `kvar`[j-1] and
        `kvar`[k-1], for k = 1,2,...,p, for j = 1,2,...,p.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, nvars = *<value>* and m = *<value>*.

            Constraint: nvars >= 2 and nvars <= m.

        (`errno` 4)
            On entry, i = *<value>*, `kvar`[i-1] = *<value>* and m =
            *<value>*.

            Constraint: 1 <= `kvar`[i-1] <= m.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 5)
            After observations with missing values were omitted, fewer
            than two cases remained for at least one pair of variables.
            (The pairs of variables involved can be determined by
            examination of the contents of the array `cnt`). All means,
            standard deviations, sums of squares and cross-products, and
            correlation-like coefficients based on two or more cases are
            returned by the function even if `errno` = 5.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.correg import coeffs_zero_subset_miss_pair as b_coeffs_zero_subset_miss_pair

    fun_name = 'naginterfaces.library.correg.coeffs_zero_subset_miss_pair'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _miss = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'miss'},
        miss,
        copy=False,
    )
    _xmiss = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'xmiss'},
        xmiss,
        copy=False,
    )
    _kvar = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'kvar'},
        kvar,
        copy=False,
    )
    _nvars_py = _b_utils._EngineIntArrayType.get_shape(_kvar, {'fun_name': fun_name, 'entity': 'kvar'}, exp_rank=1)[0]
    _nvars = _b_utils._EngineIntScalarType(
        _nvars_py, {'fun_name': fun_name, 'entity': '_nvars'},
    ).to_ctypes()
    _xbar = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nvars_py,),
    )
    _std = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nvars_py,),
    )
    _sspz = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nvars_py, _nvars_py),
        sorder=_m_sorder,
    )
    _rz = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nvars_py, _nvars_py),
        sorder=_m_sorder,
    )
    _cnt = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nvars_py, _nvars_py),
        sorder=_m_sorder,
    )
    try:
        _return_scalar = b_coeffs_zero_subset_miss_pair(
            _n,
            _m,
            _x,
            _miss,
            _xmiss,
            _nvars,
            _kvar,
            _xbar,
            _std,
            _sspz,
            _rz,
            _cnt,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _xbar_py = _xbar
    _std_py = _std
    _sspz_py = _sspz
    _rz_py = _rz
    _ncases_py = _return_scalar
    _cnt_py = _cnt
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['xbar', 'std', 'sspz', 'rz', 'ncases', 'cnt'],
    )(
        _xbar_py,
        _std_py,
        _sspz_py,
        _rz_py,
        _ncases_py,
        _cnt_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def coeffs_kspearman_overwrite(x, itype):
    u"""
    Kendall/Spearman non-parametric rank correlation coefficients, no
    missing values, overwriting input data.

    For full information please refer to the NAG Library document for
    g02bn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02bnf.html

    Parameters
    ----------
    x : float, array-like, shape (n, m)
        `x`[i-1,j-1] must be set to x_{ij}, the value of the ith
        observation on the jth variable, for j = 1,2,...,m, for i =
        1,2,...,n.

    itype : int
        The type of correlation coefficients which are to be calculated.

        `itype` = -1

          Only Kendall's tau coefficients are calculated.

        `itype` = 0

          Both Kendall's tau and Spearman's coefficients are calculated.

        `itype` = 1

          Only Spearman's coefficients are calculated.

    Returns
    -------
    x : float, ndarray, shape (n, m)
        `x`[i-1,j-1] contains the rank y_{ij} of the observation x_{ij},
        for j = 1,2,...,m, for i = 1,2,...,n.

    rr : float, ndarray, shape (m, m)
        The requested correlation coefficients.

        If only Kendall's tau coefficients are requested (`itype` = -1),
        `rr`[j-1,k-1] contains Kendall's tau for the jth and kth
        variables.

        If only Spearman's coefficients are requested (`itype` = 1),
        `rr`[j-1,k-1] contains Spearman's rank correlation coefficient
        for the jth and kth variables.

        If both Kendall's tau and Spearman's coefficients are requested
        (`itype` = 0), the upper triangle of `rr` contains the Spearman
        coefficients and the lower triangle the Kendall coefficients.

        That is, for the jth and kth variables, where j is less than k,
        `rr`[j-1,k-1] contains the Spearman rank correlation
        coefficient, and `rr`[k-1,j-1] contains Kendall's tau, for k =
        1,2,...,m, for j = 1,2,...,m.

        (Diagonal terms, `rr`[j-1,j-1], are unity for all three values
        of `itype`.)

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m >= 2.

        (`errno` 4)
            On entry, `itype` = *<value>*.

            Constraint: `itype` = -1 or 1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.correg import coeffs_kspearman_overwrite as b_coeffs_kspearman_overwrite

    fun_name = 'naginterfaces.library.correg.coeffs_kspearman_overwrite'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _rr = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _m_py),
        sorder=_m_sorder,
    )
    b_coeffs_kspearman_overwrite(
        _n,
        _m,
        _x,
        itype,
        _rr,
    )
    _x_py = _x
    _rr_py = _rr
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'rr'],
    )(
        _x_py,
        _rr_py,
    )
    return _return_data

def coeffs_kspearman_miss_case_overwrite(x, miss, xmiss, itype):
    u"""
    Kendall/Spearman non-parametric rank correlation coefficients,
    casewise treatment of missing values, overwriting input data.

    For full information please refer to the NAG Library document for
    g02bp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02bpf.html

    Parameters
    ----------
    x : float, array-like, shape (n, m)
        `x`[i-1,j-1] must be set to x_{ij}, the value of the ith
        observation on the jth variable, for j = 1,2,...,m, for i =
        1,2,...,n.

    miss : int, array-like, shape (m)
        `miss`[j-1] must be set to 1 if a missing value, xm_j, is to be
        specified for the jth variable in the array `x`, or set equal to
        0 otherwise. Values of `miss` must be given for all m variables
        in the array `x`.

    xmiss : float, array-like, shape (m)
        `xmiss`[j-1] must be set to the missing value, xm_j, to be
        associated with the jth variable in the array `x`, for those
        variables for which missing values are specified by means of the
        array `miss` (see [Accuracy]).

    itype : int
        The type of correlation coefficients which are to be calculated.

        `itype` = -1

          Only Kendall's tau coefficients are calculated.

        `itype` = 0

          Both Kendall's tau and Spearman's coefficients are calculated.

        `itype` = 1

          Only Spearman's coefficients are calculated.

    Returns
    -------
    x : float, ndarray, shape (n, m)
        `x`[i-1,j-1] contains the rank y_{ij} of the observation x_{ij},
        for j = 1,2,...,m, for i = 1,2,...,n. (For those observations
        containing missing values, and therefore excluded from the
        calculation, y_{ij} = 0, for j = 1,2,...,m.)

    rr : float, ndarray, shape (m, m)
        The requested correlation coefficients.

        If only Kendall's tau coefficients are requested (`itype` = -1),
        `rr`[j-1,k-1] contains Kendall's tau for the jth and kth
        variables.

        If only Spearman's coefficients are requested (`itype` = 1),
        `rr`[j-1,k-1] contains Spearman's rank correlation coefficient
        for the jth and kth variables.

        If both Kendall's tau and Spearman's coefficients are requested
        (`itype` = 0), the upper triangle of `rr` contains the Spearman
        coefficients and the lower triangle the Kendall coefficients.

        That is, for the jth and kth variables, where j is less than k,
        `rr`[j-1,k-1] contains the Spearman rank correlation
        coefficient, and `rr`[k-1,j-1] contains Kendall's tau, for k =
        1,2,...,m, for j = 1,2,...,m.

        (Diagonal terms, `rr`[j-1,j-1], are unity for all three values
        of `itype`.)

    ncases : int
        The number of cases, n_c, actually used in the calculations
        (when cases involving missing values have been eliminated).

    incase : int, ndarray, shape (n)
        `incase`[i-1] holds the value 1 if the ith case was included in
        the calculations, and the value 0 if the ith case contained a
        missing value for at least one variable. That is, `incase`[i-1]
        = w_i (see [Description]), for i = 1,2,...,n.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m >= 2.

        (`errno` 4)
            On entry, `itype` = *<value>*.

            Constraint: `itype` = -1 or 1.

        (`errno` 5)
            After observations with missing values were omitted, fewer
            than two cases remained.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.correg import coeffs_kspearman_miss_case_overwrite as b_coeffs_kspearman_miss_case_overwrite

    fun_name = 'naginterfaces.library.correg.coeffs_kspearman_miss_case_overwrite'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _miss = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'miss'},
        miss,
        copy=True,
    )
    _xmiss = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'xmiss'},
        xmiss,
        copy=True,
    )
    _rr = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _m_py),
        sorder=_m_sorder,
    )
    _incase = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    _return_scalar = b_coeffs_kspearman_miss_case_overwrite(
        _n,
        _m,
        _x,
        _miss,
        _xmiss,
        itype,
        _rr,
        _incase,
    )
    _x_py = _x
    _rr_py = _rr
    _ncases_py = _return_scalar
    _incase_py = _incase
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'rr', 'ncases', 'incase'],
    )(
        _x_py,
        _rr_py,
        _ncases_py,
        _incase_py,
    )
    return _return_data

def coeffs_kspearman(x, itype):
    u"""
    Kendall/Spearman non-parametric rank correlation coefficients, no
    missing values, preserving input data.

    For full information please refer to the NAG Library document for
    g02bq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02bqf.html

    Parameters
    ----------
    x : float, array-like, shape (n, m)
        `x`[i-1,j-1] must be set to data value x_{ij}, the value of the
        ith observation on the jth variable, for j = 1,2,...,m, for i =
        1,2,...,n.

    itype : int
        The type of correlation coefficients which are to be calculated.

        `itype` = -1

          Only Kendall's tau coefficients are calculated.

        `itype` = 0

          Both Kendall's tau and Spearman's coefficients are calculated.

        `itype` = 1

          Only Spearman's coefficients are calculated.

    Returns
    -------
    rr : float, ndarray, shape (m, m)
        The requested correlation coefficients.

        If only Kendall's tau coefficients are requested (`itype` = -1),
        `rr`[j-1,k-1] contains Kendall's tau for the jth and kth
        variables.

        If only Spearman's coefficients are requested (`itype` = 1),
        `rr`[j-1,k-1] contains Spearman's rank correlation coefficient
        for the jth and kth variables.

        If both Kendall's tau and Spearman's coefficients are requested
        (`itype` = 0), the upper triangle of `rr` contains the Spearman
        coefficients and the lower triangle the Kendall coefficients.

        That is, for the jth and kth variables, where j is less than k,
        `rr`[j-1,k-1] contains the Spearman rank correlation
        coefficient, and `rr`[k-1,j-1] contains Kendall's tau, for k =
        1,2,...,m, for j = 1,2,...,m.

        (Diagonal terms, `rr`[j-1,j-1], are unity for all three values
        of `itype`.)

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m >= 2.

        (`errno` 4)
            On entry, `itype` = *<value>*.

            Constraint: `itype` = -1 or 1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.correg import coeffs_kspearman as b_coeffs_kspearman

    fun_name = 'naginterfaces.library.correg.coeffs_kspearman'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _rr = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _m_py),
        sorder=_m_sorder,
    )
    b_coeffs_kspearman(
        _n,
        _m,
        _x,
        itype,
        _rr,
    )
    _rr_py = _rr
    return _rr_py

def coeffs_kspearman_miss_case(x, miss, xmiss, itype):
    u"""
    Kendall/Spearman non-parametric rank correlation coefficients,
    casewise treatment of missing values, preserving input data.

    For full information please refer to the NAG Library document for
    g02br

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02brf.html

    Parameters
    ----------
    x : float, array-like, shape (n, m)
        `x`[i-1,j-1] must be set to x_{ij}, the value of the ith
        observation on the jth variable, where i = 1,2,...,n and j =
        1,2,...,m.

    miss : int, array-like, shape (m)
        `miss`[j-1] must be set equal to 1 if a missing value, xm_j, is
        to be specified for the jth variable in the array `x`, or set
        equal to 0 otherwise. Values of `miss` must be given for all m
        variables in the array `x`.

    xmiss : float, array-like, shape (m)
        `xmiss`[j-1] must be set to the missing value, xm_j, to be
        associated with the jth variable in the array `x`, for those
        variables for which missing values are specified by means of the
        array `miss` (see [Accuracy]).

    itype : int
        The type of correlation coefficients which are to be calculated.

        `itype` = -1

          Only Kendall's tau coefficients are calculated.

        `itype` = 0

          Both Kendall's tau and Spearman's coefficients are calculated.

        `itype` = 1

          Only Spearman's coefficients are calculated.

    Returns
    -------
    rr : float, ndarray, shape (m, m)
        The requested correlation coefficients.

        If only Kendall's tau coefficients are requested (`itype` = -1),
        `rr`[j-1,k-1] contains Kendall's tau for the jth and kth
        variables.

        If only Spearman's coefficients are requested (`itype` = 1),
        `rr`[j-1,k-1] contains Spearman's rank correlation coefficient
        for the jth and kth variables.

        If both Kendall's tau and Spearman's coefficients are requested
        (`itype` = 0), the upper triangle of `rr` contains the Spearman
        coefficients and the lower triangle the Kendall coefficients.

        That is, for the jth and kth variables, where j is less than k,
        `rr`[j-1,k-1] contains the Spearman rank correlation
        coefficient, and `rr`[k-1,j-1] contains Kendall's tau, for k =
        1,2,...,m, for j = 1,2,...,m.

        (Diagonal terms, `rr`[j-1,j-1], are unity for all three values
        of `itype`.)

    ncases : int
        The number of cases, n_c, actually used in the calculations
        (when cases involving missing values have been eliminated).

    incase : int, ndarray, shape (n)
        `incase`[i-1] holds the value 1 if the ith case was included in
        the calculations, and the value 0 if the ith case contained a
        missing value for at least one variable. That is, `incase`[i-1]
        = w_i (see [Description]), for i = 1,2,...,n.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m >= 2.

        (`errno` 4)
            On entry, `itype` = *<value>*.

            Constraint: `itype` = -1 or 1.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 5)
            After observations with missing values were omitted, fewer
            than two cases remained.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.

    See Also
    --------
    :meth:`naginterfaces.library.examples.correg.coeffs_kspearman_miss_case_ex.main`
    """

    from ..base.correg import coeffs_kspearman_miss_case as b_coeffs_kspearman_miss_case

    fun_name = 'naginterfaces.library.correg.coeffs_kspearman_miss_case'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _miss = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'miss'},
        miss,
        copy=True,
    )
    _xmiss = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'xmiss'},
        xmiss,
        copy=True,
    )
    _rr = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _m_py),
        sorder=_m_sorder,
    )
    _incase = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalar = b_coeffs_kspearman_miss_case(
            _n,
            _m,
            _x,
            _miss,
            _xmiss,
            itype,
            _rr,
            _incase,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _rr_py = _rr
    _ncases_py = _return_scalar
    _incase_py = _incase
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['rr', 'ncases', 'incase'],
    )(
        _rr_py,
        _ncases_py,
        _incase_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def coeffs_kspearman_miss_pair(x, miss, xmiss, itype):
    u"""
    Kendall/Spearman non-parametric rank correlation coefficients,
    pairwise treatment of missing values.

    For full information please refer to the NAG Library document for
    g02bs

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02bsf.html

    Parameters
    ----------
    x : float, array-like, shape (n, m)
        `x`[i-1,j-1] must be set to x_{ij}, the value of the ith
        observation on the jth variable, for j = 1,2,...,m, for i =
        1,2,...,n.

    miss : int, array-like, shape (m)
        `miss`[j-1] must be set equal to 1 if a missing value, xm_j, is
        to be specified for the jth variable in the array `x`, or set
        equal to 0 otherwise. Values of `miss` must be given for all m
        variables in the array `x`.

    xmiss : float, array-like, shape (m)
        `xmiss`[j-1] must be set to the missing value, xm_j, to be
        associated with the jth variable in the array `x`, for those
        variables for which missing values are specified by means of the
        array `miss` (see [Accuracy]).

    itype : int
        The type of correlation coefficients which are to be calculated.

        `itype` = -1

          Only Kendall's tau coefficients are calculated.

        `itype` = 0

          Both Kendall's tau and Spearman's coefficients are calculated.

        `itype` = 1

          Only Spearman's coefficients are calculated.

    Returns
    -------
    rr : float, ndarray, shape (m, m)
        The requested correlation coefficients.

        If only Kendall's tau coefficients are requested (`itype` = -1),
        `rr`[j-1,k-1] contains Kendall's tau for the jth and kth
        variables.

        If only Spearman's coefficients are requested (`itype` = 1),
        `rr`[j-1,k-1] contains Spearman's rank correlation coefficient
        for the jth and kth variables.

        If both Kendall's tau and Spearman's coefficients are requested
        (`itype` = 0), the upper triangle of `rr` contains the Spearman
        coefficients and the lower triangle the Kendall coefficients.

        That is, for the jth and kth variables, where j is less than k,
        `rr`[j-1,k-1] contains the Spearman rank correlation
        coefficient, and `rr`[k-1,j-1] contains Kendall's tau, for k =
        1,2,...,m, for j = 1,2,...,m.

        (Diagonal terms, `rr`[j-1,j-1], are unity for all three values
        of `itype`.)

    ncases : int
        The minimum number of cases used in the calculation of any of
        the correlation coefficients (when cases involving missing
        values have been eliminated).

    cnt : float, ndarray, shape (m, m)
        The number of cases, n_{jk}, actually used in the calculation of
        the rank correlation coefficient for the jth and kth variables,
        for k = 1,2,...,m, for j = 1,2,...,m.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, m = *<value>*.

            Constraint: m >= 2.

        (`errno` 4)
            On entry, `itype` = *<value>*.

            Constraint: `itype` = -1 or 1.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 5)
            After observations with missing values were omitted, fewer
            than two cases remained for at least one pair of variables.
            (The pairs of variables involved can be determined by
            examination of the contents of the array `cnt`). All means,
            standard deviations, sums of squares and cross-products, and
            correlation-like coefficients based on two or more cases are
            returned by the function even if `errno` = 5.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.correg import coeffs_kspearman_miss_pair as b_coeffs_kspearman_miss_pair

    fun_name = 'naginterfaces.library.correg.coeffs_kspearman_miss_pair'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _miss = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'miss'},
        miss,
        copy=False,
    )
    _xmiss = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'xmiss'},
        xmiss,
        copy=False,
    )
    _rr = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _m_py),
        sorder=_m_sorder,
    )
    _cnt = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _m_py),
        sorder=_m_sorder,
    )
    try:
        _return_scalar = b_coeffs_kspearman_miss_pair(
            _n,
            _m,
            _x,
            _miss,
            _xmiss,
            itype,
            _rr,
            _cnt,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _rr_py = _rr
    _ncases_py = _return_scalar
    _cnt_py = _cnt
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['rr', 'ncases', 'cnt'],
    )(
        _rr_py,
        _ncases_py,
        _cnt_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def ssqmat_update(wt, x, sw, xbar, c, mean='M'):
    u"""
    Update a weighted sum of squares matrix with a new observation.

    For full information please refer to the NAG Library document for
    g02bt

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02btf.html

    Parameters
    ----------
    wt : float
        The weight to use for the current observation, w_i.

        For unweighted means and cross-products set `wt` = 1.0.

        The use of a suitable negative value of `wt`, e.g., -w_i will
        have the effect of deleting the observation.

    x : float, array-like, shape (m*incx)
        `x`[(j-1)*incx] must contain the value of the jth variable for
        the current observation, j = 1,2,...,m.

    sw : float
        The sum of weights for the previous observations, W_{i-1}.

        `sw` = 0.0

          The update procedure is initialized.

        `sw`+`wt` = 0.0

          All elements of `xbar` and `c` are set to zero.

    xbar : float, array-like, shape (m)
        If `sw` = 0.0, `xbar` is initialized, otherwise `xbar`[j-1] must
        contain the weighted mean of the jth variable for the previous
        (i-1) observations, x__j(i-1), for j = 1,2,...,m.

    c : float, array-like, shape ((m*m+m)/2)
        If `sw` != 0.0, `c` must contain the upper triangular part of
        the matrix of weighted sums of squares and cross-products or
        weighted sums of squares and cross-products of deviations about
        the mean. It is stored packed form by column, i.e., the
        cross-product between the jth and kth variable, k >= j, is
        stored in `c`[k*(k-1)/2+j-1].

    mean : str, length 1, optional
        Indicates whether ``ssqmat_update`` is to calculate sums of
        squares and cross-products, or sums of squares and
        cross-products of deviations about the mean.

        `mean` = 'M'

          The sums of squares and cross-products of deviations about the
          mean are calculated.

        `mean` = 'Z'

          The sums of squares and cross-products are calculated.

    Returns
    -------
    sw : float
        Contains the updated sum of weights, W_i.

    xbar : float, ndarray, shape (m)
        `xbar`[j-1] contains the weighted mean of the jth variable,
        x__j(i), for j = 1,2,...,m.

    c : float, ndarray, shape ((m*m+m)/2)
        The update sums of squares and cross-products stored as on
        input.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, incx = *<value>*.

            Constraint: incx >= 1.

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 2)
            On entry, `sw` = *<value>*.

            Constraint: `sw` >= 0.0.

        (`errno` 3)
            On entry, (`sw`+`wt`) = *<value>*.

            Constraint: (`sw`+`wt`) >= 0.0.

        (`errno` 4)
            On entry, `mean` = *<value>*.

            Constraint: `mean` = 'M' or 'Z'.
    """

    from ..base.correg import ssqmat_update as b_ssqmat_update

    fun_name = 'naginterfaces.library.correg.ssqmat_update'

    _xbar = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'xbar'},
        xbar,
        copy=True,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_xbar, {'fun_name': fun_name, 'entity': 'xbar'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
    )
    _return_scalar = b_ssqmat_update(
        mean,
        _m,
        wt,
        _x,
        sw,
        _xbar,
        _c,
    )
    _sw_py = _return_scalar
    _xbar_py = _xbar
    _c_py = _c
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['sw', 'xbar', 'c'],
    )(
        _sw_py,
        _xbar_py,
        _c_py,
    )
    return _return_data

def ssqmat(x, mean='M', wt=None):
    u"""
    Computes a weighted sum of squares matrix.

    For full information please refer to the NAG Library document for
    g02bu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02buf.html

    Parameters
    ----------
    x : float, array-like, shape (n, m)
        `x`[i-1,j-1] must contain the ith observation on the jth
        variable, for j = 1,2,...,m, for i = 1,2,...,n.

    mean : str, length 1, optional
        Indicates whether ``ssqmat`` is to calculate sums of squares and
        cross-products, or sums of squares and cross-products of
        deviations about the mean.

        `mean` = 'M'

          The sums of squares and cross-products of deviations about the
          mean are calculated.

        `mean` = 'Z'

          The sums of squares and cross-products are calculated.

    wt : None or float, array-like, shape (n), optional
        The optional weights of each observation. If weights are not
        provided then `wt` must be set to None, otherwise `wt`[i-1] must
        contain the weight for the ith observation.

    Returns
    -------
    sw : float
        The sum of weights.

        If `wt` is None, `sw` contains the number of observations, n.

    wmean : float, ndarray, shape (m)
        The sample means. `wmean`[j-1] contains the mean for the jth
        variable.

    c : float, ndarray, shape ((m*m+m)/2)
        The cross-products.

        If `mean` = 'M', `c` contains the upper triangular part of the
        matrix of (weighted) sums of squares and cross-products of
        deviations about the mean.

        If `mean` = 'Z', `c` contains the upper triangular part of the
        matrix of (weighted) sums of squares and cross-products.

        These are stored packed by columns, i.e., the cross-product
        between the jth and kth variable, k >= j, is stored in
        `c`[k*(k-1)/2+j-1].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 2)
            On entry, `mean` = *<value>*.

            Constraint: `mean` = 'M' or 'Z'.

        (`errno` 3)
            On entry, weight = *<value>*.

            Constraint: weight = 'W' or 'U'.

        (`errno` 4)
            On entry, `wt`[*<value>*] < 0.0.

            Constraint: `wt`[i-1] >= 0.0, for i = 1,2,...,n.

    See Also
    --------
    :meth:`naginterfaces.library.examples.correg.lars_param_ex.main`
    :meth:`naginterfaces.library.examples.correg.linregm_fit_stepwise_ex.main`
    """

    from ..base.correg import ssqmat as b_ssqmat

    fun_name = 'naginterfaces.library.correg.ssqmat'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _wt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'wt'},
        wt,
        copy=False,
        allow_none=True,
    )
    _wmean = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    _c = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        ((_m_py*_m_py+_m_py)//2,),
    )
    _return_scalar = b_ssqmat(
        mean,
        _n,
        _m,
        _x,
        _wt,
        _wmean,
        _c,
    )
    _sw_py = _return_scalar
    _wmean_py = _wmean
    _c_py = _c
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['sw', 'wmean', 'c'],
    )(
        _sw_py,
        _wmean_py,
        _c_py,
    )
    return _return_data

def ssqmat_to_corrmat(m, r):
    u"""
    Computes a correlation matrix from a sum of squares matrix.

    For full information please refer to the NAG Library document for
    g02bw

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02bwf.html

    Parameters
    ----------
    m : int
        m, the number of variables.

    r : float, array-like, shape ((`m`*`m`+`m`)/2)
        Contains the upper triangular part of the sums of squares and
        cross-products matrix of deviations from the mean. These are
        stored packed by column, i.e., the cross-product between
        variable j and k, k >= j, is stored in `r`[(k*(k-1)/2+j)-1].

    Returns
    -------
    r : float, ndarray, shape ((`m`*`m`+`m`)/2)
        The Pearson product-moment correlation coefficients.

        These are stored packed by column corresponding to the input
        cross-products.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            A variable has a zero variance. All correlations involving
            the variable with zero variance will be returned as zero.
    """

    from ..base.correg import ssqmat_to_corrmat as b_ssqmat_to_corrmat

    fun_name = 'naginterfaces.library.correg.ssqmat_to_corrmat'

    _r = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'r'},
        r,
        copy=True,
    )
    try:
        b_ssqmat_to_corrmat(
            m,
            _r,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _r_py = _r
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['r'],
    )(
        _r_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _r_py

def corrmat(x, nonzwt='W', wt=None):
    u"""
    Computes (optionally weighted) correlation and covariance matrices.

    For full information please refer to the NAG Library document for
    g02bx

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02bxf.html

    Parameters
    ----------
    x : float, array-like, shape (n, m)
        `x`[i-1,j-1] must contain the ith observation for the jth
        variable, for j = 1,2,...,m, for i = 1,2,...,n.

    nonzwt : str, length 1, optional
        The variance calculation uses a divisor which is either the
        number of weights or the number of nonzero weights.

    wt : None or float, array-like, shape (n), optional
        w, the optional frequency weighting for each observation, with
        `wt`[i-1] = w_i. Usually w_i will be an integral value
        corresponding to the number of observations associated with the
        ith data value, or zero if the ith data value is to be ignored.
        If weight = 'U', w_i is set to 1 for all i and `wt` is not
        referenced.

    Returns
    -------
    xbar : float, ndarray, shape (m)
        The sample means. `xbar`[j-1] contains the mean of the jth
        variable.

    std : float, ndarray, shape (m)
        The standard deviations. `std`[j-1] contains the standard
        deviation for the jth variable.

    v : float, ndarray, shape (m, m)
        The variance-covariance matrix. `v`[j-1,k-1] contains the
        covariance between variables j and k, for k = 1,2,...,m, for j =
        1,2,...,m.

    r : float, ndarray, shape (m, m)
        The matrix of Pearson product-moment correlation coefficients.
        `r`[j-1,k-1] contains the correlation coefficient between
        variables j and k.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n > 1.

        (`errno` 2)
            On entry, weight = *<value>*.

            Constraint: weight = 'U', 'V' or 'W'

        (`errno` 3)
            On entry, at least one value of `wt` is negative.

            Constraint: `wt`[i-1] >= 0, for i = 1,2,...,n.

        (`errno` 4)
            On entry, *<value>* observations have nonzero weight.

            Constraint: at least two observations must have a nonzero
            weight.

        (`errno` 4)
            On entry, Sum of the weights is *<value>*.

            Constraint: Sum of the weights must be greater than 1.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 5)
            A variable has a zero variance. In this case `v` and `std`
            are returned as calculated but `r` will contain zero for any
            correlation involving a variable with zero variance.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.correg import corrmat as b_corrmat

    fun_name = 'naginterfaces.library.correg.corrmat'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _wt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'wt'},
        wt,
        copy=False,
        allow_none=True,
    )
    _xbar = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    _std = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    _v = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _m_py),
        sorder=_m_sorder,
    )
    _r = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _m_py),
        sorder=_m_sorder,
    )
    try:
        b_corrmat(
            nonzwt,
            _n,
            _m,
            _x,
            _wt,
            _xbar,
            _std,
            _v,
            _r,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _xbar_py = _xbar
    _std_py = _std
    _v_py = _v
    _r_py = _r
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['xbar', 'std', 'v', 'r'],
    )(
        _xbar_py,
        _std_py,
        _v_py,
        _r_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def corrmat_partial(ny, nx, isz, r):
    u"""
    Computes partial correlation/variance-covariance matrix from
    correlation/variance-covariance matrix computed by ``corrmat``.

    For full information please refer to the NAG Library document for
    g02by

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02byf.html

    Parameters
    ----------
    ny : int
        The number of Y variables, n_y, for which partial correlation
        coefficients are to be computed.

    nx : int
        The number of X variables, n_x, which are to be considered as
        fixed.

    isz : int, array-like, shape (m)
        Indicates which variables belong to set X and Y.

        `isz`[i-1] < 0

          The ith variable is a Y variable, for i = 1,2,...,m.

        `isz`[i-1] > 0

          The ith variable is a X variable.

        `isz`[i-1] = 0

          The ith variable is not included in the computations.

    r : float, array-like, shape (m, m)
        The variance-covariance or correlation matrix for the m
        variables as given by ``corrmat``. Only the upper triangle need
        be given.

    Returns
    -------
    p : float, ndarray, shape (`ny`, `ny`)
        The strict upper triangle of `p` contains the strict upper
        triangular part of the n_y by n_y partial correlation matrix.
        The lower triangle contains the lower triangle of the n_y by n_y
        partial variance-covariance matrix if the matrix given in `r` is
        a variance-covariance matrix. If the matrix given in `r` is a
        partial correlation matrix then the variance-covariance matrix
        is for standardized variables.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ny` = *<value>*, `nx` = *<value>* and m =
            *<value>*.

            Constraint: `ny`+`nx` <= m.

        (`errno` 1)
            On entry, `nx` = *<value>*.

            Constraint: `nx` >= 1.

        (`errno` 1)
            On entry, `ny` = *<value>*.

            Constraint: `ny` > 1.

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m > 2.

        (`errno` 2)
            On entry, *<value>* values of `isz` are greater than zero
            and `nx` = *<value>*.

            Constraint: exactly `nx` values of `isz` must be greater
            than zero.

        (`errno` 2)
            On entry, *<value>* values of `isz` are less than zero and
            `ny` = *<value>*.

            Constraint: exactly `ny` values of `isz` must be less than
            zero.

        (`errno` 3)
            On entry, the square root of the correlation matrix of the
            independent variables is singular.

            Try removing some of the X variables by setting the
            appropriate element of `isz` = 0.

        (`errno` 3)
            On entry, the correlation matrix of the independent
            variables is singular.

            Try removing some of the X variables by setting the
            appropriate element of `isz` = 0.

        (`errno` 4)
            On entry, an element of the partial correlation matrix is
            greater than 1.

            Constraint: `r` must be positive definite.

        (`errno` 4)
            On entry, a diagonal element of the partial covariance
            matrix is zero.

            Constraint: `r` must be positive definite.

        (`errno` 4)
            On entry, a diagonal element of the partial covariance
            matrix is zero and an element of the partial correlation
            matrix is greater than 1.

            Constraint: `r` must be positive definite.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.correg import corrmat_partial as b_corrmat_partial

    fun_name = 'naginterfaces.library.correg.corrmat_partial'

    _r = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'r'},
        r,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_r, 'r'),),
    )
    _isz = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'isz'},
        isz,
        copy=False,
    )
    _m_py = _b_utils._EngineIntArrayType.get_shape(_isz, {'fun_name': fun_name, 'entity': 'isz'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _ny = _b_utils._EngineIntScalarType(
        ny, {'fun_name': fun_name, 'entity': 'ny'},
    ).to_ctypes()
    _ny_py = _b_utils._EngineIntScalarType.to_py(_ny)
    _p = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_ny_py, _ny_py),
        sorder=_m_sorder,
    )
    b_corrmat_partial(
        _m,
        _ny,
        nx,
        _isz,
        _r,
        _p,
    )
    _p_py = _p
    return _p_py

def ssqmat_combine(xsw, xmean, xc, ysw, ymean, yc, mean='M'):
    u"""
    Combines two sums of squares matrices, for use after ``ssqmat``.

    For full information please refer to the NAG Library document for
    g02bz

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02bzf.html

    Parameters
    ----------
    xsw : float
        W_x, the sum of weights, from the first set of data, X. If the
        data is unweighted then this will be the number of observations
        in the first dataset.

    xmean : float, array-like, shape (m)
        mu_x, the sample means for the first set of data, X.

    xc : float, array-like, shape ((m*m+m)/2)
        C_x, the sums of squares and cross-products matrix for the first
        set of data, X, as returned by ``ssqmat``.

        ``ssqmat``, returns this matrix packed by columns, i.e., the
        cross-product between the jth and kth variable, k >= j, is
        stored in `xc`[k*(k-1)/2+j-1].

        No check is made that C_x is a valid cross-products matrix.

    ysw : float
        W_y, the sum of weights, from the second set of data, Y. If the
        data is unweighted then this will be the number of observations
        in the second dataset.

    ymean : float, array-like, shape (m)
        mu_y, the sample means for the second set of data, Y.

    yc : float, array-like, shape ((m*m+m)/2)
        C_y, the sums of squares and cross-products matrix for the
        second set of data, Y, as returned by ``ssqmat``.

        ``ssqmat``, returns this matrix packed by columns, i.e., the
        cross-product between the jth and kth variable, k >= j, is
        stored in `yc`[k*(k-1)/2+j-1].

        No check is made that C_y is a valid cross-products matrix.

    mean : str, length 1, optional
        Indicates whether the matrices supplied in `xc` and `yc` are
        sums of squares and cross-products, or sums of squares and
        cross-products of deviations about the mean.

        `mean` = 'M'

          Sums of squares and cross-products of deviations about the
          mean have been supplied.

        `mean` = 'Z'

          Sums of squares and cross-products have been supplied.

    Returns
    -------
    xsw : float
        W_z, the sum of weights, from the combined dataset, Z. If both
        datasets are unweighted then this will be the number of
        observations in the combined dataset.

    xmean : float, ndarray, shape (m)
        mu_z, the sample means for the combined data, Z.

    xc : float, ndarray, shape ((m*m+m)/2)
        C_z, the sums of squares and cross-products matrix for the
        combined dataset, Z.

        This matrix is again stored packed by columns.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, `mean` = *<value>*.

            Constraint: `mean` = 'M' or 'Z'.

        (`errno` 21)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 31)
            On entry, `xsw` = *<value>*.

            Constraint: `xsw` >= 0.0.

        (`errno` 61)
            On entry, `ysw` = *<value>*.

            Constraint: `ysw` >= 0.0.
    """

    from ..base.correg import ssqmat_combine as b_ssqmat_combine

    fun_name = 'naginterfaces.library.correg.ssqmat_combine'

    _xmean = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'xmean'},
        xmean,
        copy=True,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_xmean, {'fun_name': fun_name, 'entity': 'xmean'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _xc = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'xc'},
        xc,
        copy=True,
    )
    _ymean = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ymean'},
        ymean,
        copy=False,
    )
    _yc = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'yc'},
        yc,
        copy=False,
    )
    _return_scalar = b_ssqmat_combine(
        mean,
        _m,
        xsw,
        _xmean,
        _xc,
        ysw,
        _ymean,
        _yc,
    )
    _xsw_py = _return_scalar
    _xmean_py = _xmean
    _xc_py = _xc
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['xsw', 'xmean', 'xc'],
    )(
        _xsw_py,
        _xmean_py,
        _xc_py,
    )
    return _return_data

def linregs_const(x, y):
    u"""
    Simple linear regression with constant term, no missing values.

    For full information please refer to the NAG Library document for
    g02ca

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02caf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        `x`[i-1] must contain x_i, for i = 1,2,...,n.

    y : float, array-like, shape (n)
        `y`[i-1] must contain y_i, for i = 1,2,...,n.

    Returns
    -------
    result : float, ndarray, shape (20)
        The following information:

        [table omitted]

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n > 2.

        (`errno` 2)
            On entry, all n values of at least one of `x` and `y` are
            identical.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.correg import linregs_const as b_linregs_const

    fun_name = 'naginterfaces.library.correg.linregs_const'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _result = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (20,),
    )
    b_linregs_const(
        _n,
        _x,
        _y,
        _result,
    )
    _result_py = _result
    return _result_py

def linregs_noconst(x, y):
    u"""
    Simple linear regression without constant term, no missing values.

    For full information please refer to the NAG Library document for
    g02cb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02cbf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        `x`[i-1] must contain x_i, for i = 1,2,...,n.

    y : float, array-like, shape (n)
        `y`[i-1] must contain y_i, for i = 1,2,...,n.

    Returns
    -------
    result : float, ndarray, shape (20)
        The following information:

        [table omitted]

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n > 2.

        (`errno` 2)
            On entry, all n values of at least one of `x` and `y` are
            identical.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.correg import linregs_noconst as b_linregs_noconst

    fun_name = 'naginterfaces.library.correg.linregs_noconst'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _result = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (20,),
    )
    b_linregs_noconst(
        _n,
        _x,
        _y,
        _result,
    )
    _result_py = _result
    return _result_py

def linregs_const_miss(x, y, xmiss, ymiss):
    u"""
    Simple linear regression with constant term, missing values.

    For full information please refer to the NAG Library document for
    g02cc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02ccf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        `x`[i-1] must contain x_i, for i = 1,2,...,n.

    y : float, array-like, shape (n)
        `y`[i-1] must contain y_i, for i = 1,2,...,n.

    xmiss : float
        The value xm which is to be taken as the missing value for the
        variable x. See [Accuracy].

    ymiss : float
        The value ym which is to be taken as the missing value for the
        variable y. See [Accuracy].

    Returns
    -------
    result : float, ndarray, shape (21)
        The following information:

        [table omitted]

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n > 2.

        (`errno` 2)
            After observations with missing values were omitted, two or
            fewer cases remained.

        (`errno` 3)
            After observations with missing values were omitted, all
            remaining values of at least one of `x` and `y` were
            identical.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.correg import linregs_const_miss as b_linregs_const_miss

    fun_name = 'naginterfaces.library.correg.linregs_const_miss'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _result = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (21,),
    )
    b_linregs_const_miss(
        _n,
        _x,
        _y,
        xmiss,
        ymiss,
        _result,
    )
    _result_py = _result
    return _result_py

def linregs_noconst_miss(x, y, xmiss, ymiss):
    u"""
    Simple linear regression without constant term, missing values.

    For full information please refer to the NAG Library document for
    g02cd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02cdf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        `x`[i-1] must contain x_i, for i = 1,2,...,n.

    y : float, array-like, shape (n)
        `y`[i-1] must contain y_i, for i = 1,2,...,n.

    xmiss : float
        The value xm, which is to be taken as the missing value for the
        variable x (see [Accuracy]).

    ymiss : float
        The value ym, which is to be taken as the missing value for the
        variable y (see [Accuracy]).

    Returns
    -------
    result : float, ndarray, shape (21)
        The following information:

        [table omitted]

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            After observations with missing values were omitted, fewer
            than two cases remained.

        (`errno` 3)
            After observations with missing values were omitted, all
            remaining values of at least one of `x` and `y` were
            identical.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.correg import linregs_noconst_miss as b_linregs_noconst_miss

    fun_name = 'naginterfaces.library.correg.linregs_noconst_miss'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _result = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (21,),
    )
    b_linregs_noconst_miss(
        _n,
        _x,
        _y,
        xmiss,
        ymiss,
        _result,
    )
    _result_py = _result
    return _result_py

def linregm_service_select(xbar, std, ssp, r, korder):
    u"""
    Service function for multiple linear regression, select elements
    from vectors and matrices.

    For full information please refer to the NAG Library document for
    g02ce

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02cef.html

    Parameters
    ----------
    xbar : float, array-like, shape (n)
        `xbar`[i-1] must be set to x__i, the mean of variable i, for i =
        1,2,...,n.

    std : float, array-like, shape (n)
        `std`[i-1] must be set to s_i, the standard deviation of
        variable i, for i = 1,2,...,n.

    ssp : float, array-like, shape (n, n)
        `ssp`[i-1,j-1] must be set to the sum of cross-products of
        deviations from means S_{ij} (or about zero, S~_{ij}) for
        variables i and j, for j = 1,2,...,n, for i = 1,2,...,n.

    r : float, array-like, shape (n, n)
        `r`[i-1,j-1] must be set to the Pearson product-moment
        correlation coefficient R_{ij} (or the correlation-like
        coefficient, R~_{ij}) for variables i and j, for j = 1,2,...,n,
        for i = 1,2,...,n.

    korder : int, array-like, shape (m)
        `korder`[i-1] must be set to the number of the original variable
        which is to be the ith variable in the output vectors and
        matrices, for i = 1,2,...,m.

    Returns
    -------
    xbar2 : float, ndarray, shape (m)
        The mean of variable i, `xbar`[i-1], where i = `korder`[k-1],
        for k = 1,2,...,m. (The array `xbar2` must differ from `xbar`
        and `std`.)

    std2 : float, ndarray, shape (m)
        The standard deviation of variable i, `std`[i-1], where i =
        `korder`[k-1], for k = 1,2,...,m. (The array `std2` must differ
        from both `xbar` and `std`.)

    ssp2 : float, ndarray, shape (m, m)
        `ssp2`[k-1,l-1] contains the value of `ssp`[i-1,j-1], where i =
        `korder`[k-1] and j = `korder`[l-1], for l = 1,2,...,m, for k =
        1,2,...,m. (The array `ssp2` must differ from both `ssp` and
        `r`.)

        That is to say: on exit, `ssp2`[k-1,l-1] contains the sum of
        cross-products of deviations from means S_{ij} (or about zero,
        S~_{ij}).

    r2 : float, ndarray, shape (m, m)
        `r2`[k-1,l-1] contains the value of `r`[i-1,j-1], where i =
        `korder`[k-1] and j = `korder`[l-1], for l = 1,2,...,m, for k =
        1,2,...,m. (The array `r2` must differ from both `ssp` and `r`.)

        That is to say: on exit, `r2`[k-1,l-1] contains the Pearson
        product-moment coefficient R_{ij} (or the correlation-like
        coefficient, R~_{ij}).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, n = *<value>* and m = *<value>*.

            Constratint: n >= m.

        (`errno` 4)
            On entry, `korder`[*<value>*] = *<value>* and n = *<value>*.

            Constraint: 1 <= `korder`[i-1] <= n, for i = 1,2,...,m.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.correg import linregm_service_select as b_linregm_service_select

    fun_name = 'naginterfaces.library.correg.linregm_service_select'

    _ssp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ssp'},
        ssp,
        copy=False,
    )
    _r = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'r'},
        r,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_ssp, 'ssp'), (_r, 'r')),
    )
    _xbar = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'xbar'},
        xbar,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_xbar, {'fun_name': fun_name, 'entity': 'xbar'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _std = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'std'},
        std,
        copy=False,
    )
    _korder = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'korder'},
        korder,
        copy=False,
    )
    _m_py = _b_utils._EngineIntArrayType.get_shape(_korder, {'fun_name': fun_name, 'entity': 'korder'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _xbar2 = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    _std2 = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    _ssp2 = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _m_py),
        sorder=_m_sorder,
    )
    _r2 = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _m_py),
        sorder=_m_sorder,
    )
    b_linregm_service_select(
        _n,
        _xbar,
        _std,
        _ssp,
        _r,
        _m,
        _korder,
        _xbar2,
        _std2,
        _ssp2,
        _r2,
    )
    _xbar2_py = _xbar2
    _std2_py = _std2
    _ssp2_py = _ssp2
    _r2_py = _r2
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['xbar2', 'std2', 'ssp2', 'r2'],
    )(
        _xbar2_py,
        _std2_py,
        _ssp2_py,
        _r2_py,
    )
    return _return_data

def linregm_service_reorder(korder, xbar, std, ssp, r):
    u"""
    Service function for multiple linear regression, reorder elements of
    vectors and matrices.

    For full information please refer to the NAG Library document for
    g02cf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02cff.html

    Parameters
    ----------
    korder : int, array-like, shape (n)
        `korder`[i-1] must be set to the number of the original variable
        which is to be the ith variable in the re-arranged data, for i =
        1,2,...,n.

    xbar : float, array-like, shape (n)
        `xbar`[i-1] must be set to the mean of variable i, for i =
        1,2,...,n.

    std : float, array-like, shape (n)
        `std`[i-1] must be set to the standard deviation of variable i,
        for i = 1,2,...,n.

    ssp : float, array-like, shape (n, n)
        `ssp`[i-1,j-1] must be set to the sum of cross-products of
        deviations from means S_{ij} (or about zero S~_{ij}) for
        variables i and j, for j = 1,2,...,n, for i = 1,2,...,n.

    r : float, array-like, shape (n, n)
        `r`[i-1,j-1] must be set to the Pearson product-moment
        correlation coefficient R_{ij} (or the correlation-like
        coefficient R~_{ij}) for variables i and j, for j = 1,2,...,n,
        for i = 1,2,...,n.

    Returns
    -------
    xbar : float, ndarray, shape (n)
        `xbar`[i-1] contains the mean of variable k where k =
        `korder`[i-1], for i = 1,2,...,n.

    std : float, ndarray, shape (n)
        `std`[i-1] contains the standard deviation of variable k where k
        = `korder`[i-1], for i = 1,2,...,n.

    ssp : float, ndarray, shape (n, n)
        `ssp`[i-1,j-1] contains the sum of cross-products of deviations
        from means S_{kl} (or about zero S~_{kl}) for variables k and l,
        where k = `korder`[i-1], and l = `korder`[j-1], i,j = 1,2,...,n.

    r : float, ndarray, shape (n, n)
        `r`[i-1,j-1] contains the Pearson product-moment correlation
        coefficient R_{kl} (or the correlation-like coefficient R~_{kl})
        for variables k and l, where k = `korder`[i-1] and l =
        `korder`[j-1], for j = 1,2,...,n, for i = 1,2,...,n.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 3)
            On entry, `korder`[*<value>*] = *<value>* and n = *<value>*.

            Constraint: 1 <= `korder`[i-1] <= n, for i = 1,2,...,m.

        (`errno` 4)
            On entry, there is not a one-to-one correspondence between
            the old variables and the new variables; at least one of the
            original variables is not included in the new set, and
            consequently at least one other variable has been included
            more than once.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.correg import linregm_service_reorder as b_linregm_service_reorder

    fun_name = 'naginterfaces.library.correg.linregm_service_reorder'

    _korder = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'korder'},
        korder,
        copy=False,
    )
    _n_py = _b_utils._EngineIntArrayType.get_shape(_korder, {'fun_name': fun_name, 'entity': 'korder'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _xbar = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'xbar'},
        xbar,
        copy=True,
    )
    _std = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'std'},
        std,
        copy=True,
    )
    _ssp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ssp'},
        ssp,
        copy=True,
    )
    _r = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'r'},
        r,
        copy=True,
    )
    b_linregm_service_reorder(
        _n,
        _korder,
        _xbar,
        _std,
        _ssp,
        _r,
    )
    _xbar_py = _xbar
    _std_py = _std
    _ssp_py = _ssp
    _r_py = _r
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['xbar', 'std', 'ssp', 'r'],
    )(
        _xbar_py,
        _std_py,
        _ssp_py,
        _r_py,
    )
    return _return_data

def linregm_coeffs_const(n, xbar, ssp, r):
    u"""
    Multiple linear regression, from correlation coefficients, with
    constant term.

    For full information please refer to the NAG Library document for
    g02cg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02cgf.html

    Parameters
    ----------
    n : int
        The number of cases n, used in calculating the sums of squares
        and cross-products and correlation coefficients.

    xbar : float, array-like, shape (k+1)
        `xbar`[i-1] must be set to x__i, the mean value of the ith
        variable, for i = 1,2,...,k+1; the mean of the dependent
        variable must be contained in `xbar`[k].

    ssp : float, array-like, shape (k+1, k+1)
        `ssp`[i-1,j-1] must be set to S_{ij}, the sum of cross-products
        of deviations from means for the ith and jth variables, for j =
        1,2,...,k+1, for i = 1,2,...,k+1; terms involving the dependent
        variable appear in row k+1 and column k+1.

    r : float, array-like, shape (k+1, k+1)
        `r`[i-1,j-1] must be set to R_{ij}, the Pearson product-moment
        correlation coefficient for the ith and jth variables, for j =
        1,2,...,k+1, for i = 1,2,...,k+1; terms involving the dependent
        variable appear in row k+1 and column k+1.

    Returns
    -------
    result : float, ndarray, shape (13)
        The following information:

        [table omitted]

    coef : float, ndarray, shape (k, 3)
        For i = 1,2,...,k, the following information:

        `coef`[i-1,0]

          b_i, the regression coefficient for the ith variable.

        `coef`[i-1,1]

          se(b_i), the standard error of the regression coefficient for
          the ith variable.

        `coef`[i-1,2]

          t(b_i), the t value of the regression coefficient for the ith
          variable.

    con : float, ndarray, shape (3)
        The following information:

        [table omitted]

    rinv : float, ndarray, shape (k, k)
        The inverse of the matrix of correlation coefficients for the
        independent variables; that is, the inverse of the matrix
        consisting of the first k rows and columns of `r`.

    c : float, ndarray, shape (k, k)
        The modified inverse matrix, where

        - `c`[i-1,j-1] = `r`[i-1,j-1]*`rinv`[i-1,j-1]/`ssp`[i-1,j-1],
          for j = 1,2,...,k, for i = 1,2,...,k.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, k = *<value>*.

            Constraint: k >= 1.

        (`errno` 3)
            On entry, `n` = *<value>* and k = *<value>*.

            Constraint: `n` > k+1.

        (`errno` 5)
            The k by k partition of `r` which requires inversion is not
            positive definite.

        (`errno` 6)
            The refinement following inversion has failed.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.correg import linregm_coeffs_const as b_linregm_coeffs_const

    fun_name = 'naginterfaces.library.correg.linregm_coeffs_const'

    _ssp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ssp'},
        ssp,
        copy=False,
    )
    _r = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'r'},
        r,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_ssp, 'ssp'), (_r, 'r')),
    )
    _k = _b_utils._EngineIntScalarType(
        _b_utils._EngineFloat64ArrayType.get_shape(_ssp, {'fun_name': fun_name, 'entity': 'ssp'}, exp_rank=2)[1]-1, {'fun_name': fun_name, 'entity': '_k'},
    ).to_ctypes()
    _xbar = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'xbar'},
        xbar,
        copy=False,
    )
    _result = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (13,),
    )
    _k_py = _b_utils._EngineIntScalarType.to_py(_k)
    _coef = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_k_py, 3),
        sorder=_m_sorder,
    )
    _con = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (3,),
    )
    _rinv = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_k_py, _k_py),
        sorder=_m_sorder,
    )
    _c = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_k_py, _k_py),
        sorder=_m_sorder,
    )
    b_linregm_coeffs_const(
        n,
        _k,
        _xbar,
        _ssp,
        _r,
        _result,
        _coef,
        _con,
        _rinv,
        _c,
    )
    _result_py = _result
    _coef_py = _coef
    _con_py = _con
    _rinv_py = _rinv
    _c_py = _c
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['result', 'coef', 'con', 'rinv', 'c'],
    )(
        _result_py,
        _coef_py,
        _con_py,
        _rinv_py,
        _c_py,
    )
    return _return_data

def linregm_coeffs_noconst(n, sspz, rz):
    u"""
    Multiple linear regression, from correlation-like coefficients,
    without constant term.

    For full information please refer to the NAG Library document for
    g02ch

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02chf.html

    Parameters
    ----------
    n : int
        n, the number of cases used in calculating the sums of squares
        and cross-products and correlation-like coefficients.

    sspz : float, array-like, shape (k+1, k+1)
        `sspz`[i-1,j-1] must be set to S~_{ij}, the sum of
        cross-products about zero for the ith and jth variables, for j =
        1,2,...,k+1, for i = 1,2,...,k+1; terms involving the dependent
        variable appear in row k+1 and column k+1.

    rz : float, array-like, shape (k+1, k+1)
        `rz`[i-1,j-1] must be set to R~_{ij}, the correlation-like
        coefficient for the ith and jth variables, for j = 1,2,...,k+1,
        for i = 1,2,...,k+1; coefficients involving the dependent
        variable appear in row k+1 and column k+1.

    Returns
    -------
    result : float, ndarray, shape (13)
        The following information:

        [table omitted]

    coef : float, ndarray, shape (k, 3)
        For i = 1,2,...,k, the following information:

        `coef`[i-1,0]

          b_i, the regression coefficient for the ith variable.

        `coef`[i-1,1]

          se(b_i), the standard error of the regression coefficient for
          the ith variable.

        `coef`[i-1,2]

          t(b_i), the t value of the regression coefficient for the ith
          variable.

    rznv : float, ndarray, shape (k, k)
        The inverse of the matrix of correlation-like coefficients for
        the independent variables; that is, the inverse of the matrix
        consisting of the first k rows and columns of `rz`.

    cz : float, ndarray, shape (k, k)
        The modified inverse matrix, C, where

            `cz`[i-1,j-1] =
            `rz`[i-1,j-1]*`rznv`[i-1,j-1]/`sspz`[i-1,j-1], i,j =
            1,2,...,k.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, k = *<value>*.

            Constraint: k >= 1.

        (`errno` 3)
            On entry, `n` = *<value>* and k = *<value>*.

            Constraint: `n` >= k+1.

        (`errno` 5)
            The K by K partition of `rz` which requires inversion is not
            positive definite.

        (`errno` 6)
            The refinement following the actual inversion has failed.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.correg import linregm_coeffs_noconst as b_linregm_coeffs_noconst

    fun_name = 'naginterfaces.library.correg.linregm_coeffs_noconst'

    _sspz = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'sspz'},
        sspz,
        copy=False,
    )
    _rz = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'rz'},
        rz,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_sspz, 'sspz'), (_rz, 'rz')),
    )
    _k = _b_utils._EngineIntScalarType(
        _b_utils._EngineFloat64ArrayType.get_shape(_sspz, {'fun_name': fun_name, 'entity': 'sspz'}, exp_rank=2)[1]-1, {'fun_name': fun_name, 'entity': '_k'},
    ).to_ctypes()
    _result = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (13,),
    )
    _k_py = _b_utils._EngineIntScalarType.to_py(_k)
    _coef = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_k_py, 3),
        sorder=_m_sorder,
    )
    _rznv = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_k_py, _k_py),
        sorder=_m_sorder,
    )
    _cz = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_k_py, _k_py),
        sorder=_m_sorder,
    )
    b_linregm_coeffs_noconst(
        n,
        _k,
        _sspz,
        _rz,
        _result,
        _coef,
        _rznv,
        _cz,
    )
    _result_py = _result
    _coef_py = _coef
    _rznv_py = _rznv
    _cz_py = _cz
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['result', 'coef', 'rznv', 'cz'],
    )(
        _result_py,
        _coef_py,
        _rznv_py,
        _cz_py,
    )
    return _return_data

def linregm_fit(x, isx, y, mean='M', wt=None, tol=0.000001):
    u"""
    Fits a general (multiple) linear regression model.

    For full information please refer to the NAG Library document for
    g02da

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02daf.html

    Parameters
    ----------
    x : float, array-like, shape (n, m)
        `x`[i-1,j-1] must contain the ith observation for the jth
        independent variable, for j = 1,2,...,m, for i = 1,2,...,n.

    isx : int, array-like, shape (m)
        Indicates which independent variables are to be included in the
        model.

        `isx`[j-1] > 0

          The variable contained in the jth column of `x` is included in
          the regression model.

    y : float, array-like, shape (n)
        y, the observations on the dependent variable.

    mean : str, length 1, optional
        Indicates if a mean term is to be included.

        `mean` = 'M'

          A mean term, intercept, will be included in the model.

        `mean` = 'Z'

          The model will pass through the origin, zero-point.

    wt : None or float, array-like, shape (n), optional
        If provided `wt` must contain the weights to be used with the
        model.

        If `wt`[i-1] = 0.0, the ith observation is not included in the
        model, in which case the effective number of observations is the
        number of observations with nonzero weights.

        The values of `res` and `h` will be set to zero for observations
        with zero weights.

        If `wt` is not provided the effective number of observations is
        n.

    tol : float, optional
        The value of `tol` is used to decide if the independent
        variables are of full rank and if not what is the rank of the
        independent variables. The smaller the value of `tol` the
        stricter the criterion for selecting the singular value
        decomposition. If `tol` = 0.0, the singular value decomposition
        will never be used; this may cause run time errors or inaccurate
        results if the independent variables are not of full rank.

    Returns
    -------
    rss : float
        The residual sum of squares for the regression.

    idf : int
        The degrees of freedom associated with the residual sum of
        squares.

    b : float, ndarray, shape (ip)
        `b`[i-1], i = 1,2,...,ip contains the least squares estimates of
        the parameters of the regression model, beta^.

        If `mean` = 'M', `b`[0] will contain the estimate of the mean
        parameter and `b`[i] will contain the coefficient of the
        variable contained in column j of `x`, where `isx`[j-1] is the
        ith positive value in the array `isx`.

        If `mean` = 'Z', `b`[i-1] will contain the coefficient of the
        variable contained in column j of `x`, where `isx`[j-1] is the
        ith positive value in the array `isx`.

    se : float, ndarray, shape (ip)
        `se`[i-1], i = 1,2,...,ip contains the standard errors of the ip
        parameter estimates given in `b`.

    cov : float, ndarray, shape (ip*(ip+1)/2)
        The first ip*(ip+1)/2 elements of `cov` contain the upper
        triangular part of the variance-covariance matrix of the ip
        parameter estimates given in `b`. They are stored packed by
        column, i.e., the covariance between the parameter estimate
        given in `b`[i-1] and the parameter estimate given in `b`[j-1],
        j >= i, is stored in `cov`[j*(j-1)/2+i-1].

    res : float, ndarray, shape (n)
        The (weighted) residuals, r_i, for i = 1,2,...,n.

    h : float, ndarray, shape (n)
        The diagonal elements of H, h_i, for i = 1,2,...,n.

    q : float, ndarray, shape (n, ip+1)
        The results of the QR decomposition:

        - the first column of `q` contains c;

        - the upper triangular part of columns 2 to ip+1 contain the R
          matrix;

        - the strictly lower triangular part of columns 2 to ip+1
          contain details of the Q matrix.

    svd : bool
        If a singular value decomposition has been performed then `svd`
        will be True, otherwise `svd` will be False.

    irank : int
        The rank of the independent variables.

        If `svd` = False, `irank` = ip.

        If `svd` = True, `irank` is an estimate of the rank of the
        independent variables.

        `irank` is calculated as the number of singular values greater
        that `tol`* (largest singular value).

        It is possible for the SVD to be carried out but `irank` to be
        returned as ip.

    p : float, ndarray, shape (2*ip+ip*ip)
        Details of the QR decomposition and SVD if used.

        If `svd` = False, only the first ip elements of `p` are used
        these will contain the zeta values for the QR decomposition (see
        ``lapackeig.dgeqrf`` for details).

        If `svd` = True, the first ip elements of `p` will contain the
        zeta values for the QR decomposition (see ``lapackeig.dgeqrf``
        for details) and the next ip elements of `p` contain singular
        values.

        The following ip by ip elements contain the matrix P* stored by
        columns.

    wk : float, ndarray, shape (max(2, 5*(ip-1)+ip*ip))
        If on exit `svd` = True, `wk` contains information which is
        needed by ``linregm_fit_newvar``; otherwise `wk` is used as
        workspace.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, ip = *<value>*.

            Constraint: ip >= 1.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` >= 0.0.

        (`errno` 1)
            On entry, ip = *<value>* and n = *<value>*.

            Constraint: ip <= n.

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, weight = *<value>*.

            Constraint: weight = 'U' or 'W'.

        (`errno` 2)
            On entry, `mean` = *<value>*.

            Constraint: `mean` = 'M' or 'Z'.

        (`errno` 3)
            On entry, `wt`[*<value>*] < 0.0.

            Constraint: `wt`[i-1] >= 0.0, for i = 1,2,...,n.

        (`errno` 4)
            On entry, `isx`[*<value>*] < 0.

            Constraint: `isx`[i-1] >= 0.0, for i = 1,2,...,m.

        (`errno` 4)
            On entry, ip = *<value>*.

            Constraint: ip must be compatible with the number of nonzero
            elements in `isx`.

        (`errno` 6)
            SVD solution failed to converge.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 5)
            The degrees of freedom for the residuals are zero, i.e., the
            designated number of arguments is equal to the effective
            number of observations. In this case the parameter estimates
            will be returned along with the diagonal elements of H, but
            neither standard errors nor the variance-covariance matrix
            will be calculated.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.

    See Also
    --------
    :meth:`naginterfaces.library.examples.correg.linregm_fit_ex.main`
    """

    from ..base.correg import linregm_fit as b_linregm_fit

    fun_name = 'naginterfaces.library.correg.linregm_fit'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _isx = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'isx'},
        isx,
        copy=False,
    )
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _wt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'wt'},
        wt,
        copy=False,
        allow_none=True,
    )
    _vl_p_py = 0
    for _vl_i_py in range(1, _m_py+1):
        if _b_utils._EngineIntArrayType.get_element(_isx, {'fun_name': fun_name, 'entity': 'isx'}, (_vl_i_py-1,)) > 0:
            _vl_p_py = _vl_p_py + 1
    _mean = _b_utils._EngineCharScalarType(
        mean, {'fun_name': fun_name, 'entity': 'mean'}, exp_chrlen=1,
    ).to_ctypes()
    _mean_py = _b_utils._EngineCharScalarType.to_py(_mean)
    if _mean_py in ('M', 'm'):
        _vl_ip_py = _vl_p_py+1
    else:
        _vl_ip_py = _vl_p_py
    _b = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_ip_py,),
    )
    _se = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_ip_py,),
    )
    _cov = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_ip_py*(_vl_ip_py+1)//2,),
    )
    _res = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _h = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _q = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _vl_ip_py+1),
        sorder=_m_sorder,
    )
    _p = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (2*_vl_ip_py+_vl_ip_py*_vl_ip_py,),
    )
    _wk = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (max(2, 5*(_vl_ip_py-1)+_vl_ip_py*_vl_ip_py),),
    )
    try:
        _return_scalars = b_linregm_fit(
            mean,
            _n,
            _x,
            _m,
            _isx,
            _y,
            _wt,
            _b,
            _se,
            _cov,
            _res,
            _h,
            _q,
            _p,
            tol,
            _wk,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _rss_py = _return_scalars.rss
    _idf_py = _return_scalars.idf
    _b_py = _b
    _se_py = _se
    _cov_py = _cov
    _res_py = _res
    _h_py = _h
    _q_py = _q
    _svd_py = _return_scalars.svd
    _irank_py = _return_scalars.irank
    _p_py = _p
    _wk_py = _wk
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['rss', 'idf', 'b', 'se', 'cov', 'res', 'h', 'q', 'svd', 'irank', 'p', 'wk'],
    )(
        _rss_py,
        _idf_py,
        _b_py,
        _se_py,
        _cov_py,
        _res_py,
        _h_py,
        _q_py,
        _svd_py,
        _irank_py,
        _p_py,
        _wk_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def linregm_obs_edit(update, isx, q, x, ix, y, rss, mean='M', wt=None):
    u"""
    Add/delete an observation to/from a general linear regression model.

    For full information please refer to the NAG Library document for
    g02dc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02dcf.html

    Parameters
    ----------
    update : str, length 1
        Indicates if an observation is to be added or deleted.

        `update` = 'A'

          The observation is added.

        `update` = 'D'

          The observation is deleted.

    isx : int, array-like, shape (m)
        If `isx`[j-1] is greater than 0, the value contained in
        `x`[(j-1)*`ix`] is to be included as a value of x^T, for j =
        1,2,...,m.

    q : float, array-like, shape (ip, ip+1)
        Must be array `q` as output by ``linregm_fit``,
        ``linregm_var_add``, ``linregm_var_del`` or
        ``linregm_fit_onestep``, or a previous call to
        ``linregm_obs_edit``.

    x : float, array-like, shape ((m-1)*`ix`+1)
        The ip values for the dependent variables of the new
        observation, x^T. The positions will depend on the value of
        `ix`.

    ix : int
        The increment for elements of `x`. Two situations are common:

        `ix` = 1

          The values of x are to be chosen from consecutive locations in
          `x`, i.e., `x`[0],`x`[1],...,`x`[m-1].

        `ix` = ldx

          The values of x are to be chosen from a row of a
          two-dimensional array with first dimension ldx, i.e.,
          `x`[0],`x`[ldx],...,`x`[(m-1)ldx].

    y : float
        The value of the dependent variable for the new observation,
        y_new.

    rss : float
        The value of the residual sums of squares for the original set
        of observations.

    mean : str, length 1, optional
        Indicates if a mean has been used in the model.

        `mean` = 'M'

          A mean term or intercept will have been included in the model
          by ``linregm_fit``.

        `mean` = 'Z'

          A model with no mean term or intercept will have been fitted
          by ``linregm_fit``.

    wt : None or float, optional
        If provided, `wt` must contain the weight to be used with the
        new observation.

        If `wt` is None, the observation is not included in the model.

    Returns
    -------
    q : float, ndarray, shape (ip, ip+1)
        The first ip elements of the first column of `q` will contain
        c_1^* the upper triangular part of columns 2 to ip+1 will
        contain R* the remainder is unchanged.

    rss : float
        The updated values of the residual sums of squares.

        Note: this will only be valid if the model is of full rank.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, *<value>* elements of `isx` > 0 instead of ip-1
            (for mean) = *<value>*.

        (`errno` 1)
            On entry, `ix` = *<value>*.

            Constraint: `ix` >= 1.

        (`errno` 1)
            On entry, *<value>* elements of `isx` > 0 instead of ip =
            *<value>*.

        (`errno` 1)
            On entry, `mean` = *<value>*.

            Constraint: `mean` = 'M' or 'Z'.

        (`errno` 1)
            On entry, `rss` = *<value>*.

            Constraint: `rss` >= 0.0.

        (`errno` 1)
            On entry, `update` = *<value>*.

            Constraint: `update` = 'A' or 'D'.

        (`errno` 1)
            On entry, weight = *<value>*.

            Constraint: weight = 'U' or 'W'.

        (`errno` 1)
            On entry, ip = *<value>*.

            Constraint: ip >= 1.

        (`errno` 2)
            On entry, `wt` = *<value>*.

            Constraint: `wt` >= 0.0.

        (`errno` 3)
            The R matrix could not be updated.

        (`errno` 4)
            The residual sums of squares cannot be updated.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.correg import linregm_obs_edit as b_linregm_obs_edit

    fun_name = 'naginterfaces.library.correg.linregm_obs_edit'

    _isx = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'isx'},
        isx,
        copy=False,
    )
    _m_py = _b_utils._EngineIntArrayType.get_shape(_isx, {'fun_name': fun_name, 'entity': 'isx'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _vl_p_py = 0
    for _vl_i_py in range(1, _m_py+1):
        if _b_utils._EngineIntArrayType.get_element(_isx, {'fun_name': fun_name, 'entity': 'isx'}, (_vl_i_py-1,)) > 0:
            _vl_p_py = _vl_p_py + 1
    _mean = _b_utils._EngineCharScalarType(
        mean, {'fun_name': fun_name, 'entity': 'mean'}, exp_chrlen=1,
    ).to_ctypes()
    _mean_py = _b_utils._EngineCharScalarType.to_py(_mean)
    if _mean_py in ('M', 'm'):
        _vl_ip_py = _vl_p_py+1
    else:
        _vl_ip_py = _vl_p_py
    _q = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'q'},
        q,
        copy=True,
    )
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _return_scalar = b_linregm_obs_edit(
        update,
        mean,
        _m,
        _isx,
        _q,
        _x,
        ix,
        y,
        wt,
        rss,
    )
    _q_py = _q
    _rss_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['q', 'rss'],
    )(
        _q_py,
        _rss_py,
    )
    return _return_data

def linregm_update(q, rss, p, tol=0.000001):
    u"""
    Estimates of linear parameters and general linear regression model
    from updated model.

    For full information please refer to the NAG Library document for
    g02dd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02ddf.html

    Parameters
    ----------
    q : float, array-like, shape (:, ip+1)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `rss`<=0.0: n; otherwise: ip.

        Must be the array `q` as output by ``linregm_obs_edit``,
        ``linregm_var_add``, ``linregm_var_del`` or
        ``linregm_fit_onestep``. If on entry `rss` <= 0.0 then all n
        elements of c are needed. This is provided by functions
        ``linregm_var_add``, ``linregm_var_del`` or
        ``linregm_fit_onestep``.

    rss : float
        Either the residual sum of squares or a value less than or equal
        to 0.0 to indicate that the residual sum of squares is to be
        calculated by the function.

    p : float, array-like, shape (ip*ip+2*ip)
        Must be array `p` as output by ``linregm_fit``,
        ``linregm_var_add`` or ``linregm_fit_onestep``, or a previous
        call to ``linregm_update``.

    tol : float, optional
        The value of `tol` is used to decide if the independent
        variables are of full rank and, if not, what is the rank of the
        independent variables. The smaller the value of `tol` the
        stricter the criterion for selecting the singular value
        decomposition. If `tol` = 0.0, the singular value decomposition
        will never be used, this may cause run time errors or
        inaccuracies if the independent variables are not of full rank.

    Returns
    -------
    rss : float
        If `rss` <= 0.0 on entry, then on exit `rss` will contain the
        residual sum of squares as calculated by ``linregm_update``.

        If `rss` was positive on entry, it will be unchanged.

    idf : int
        The degrees of freedom associated with the residual sum of
        squares.

    b : float, ndarray, shape (ip)
        The estimates of the p parameters, beta^.

    se : float, ndarray, shape (ip)
        The standard errors of the p parameters given in `b`.

    cov : float, ndarray, shape (ip*(ip+1)/2)
        The upper triangular part of the variance-covariance matrix of
        the p parameter estimates given in `b`. They are stored packed
        by column, i.e., the covariance between the parameter estimate
        given in `b`[i-1] and the parameter estimate given in `b`[j-1],
        j >= i, is stored in `cov`[j*(j-1)/2+i-1].

    svd : bool
        If a singular value decomposition has been performed, `svd` =
        True, otherwise `svd` = False.

    irank : int
        The rank of the independent variables.

        If `svd` = False, `irank` = ip.

        If `svd` = True, `irank` is an estimate of the rank of the
        independent variables.

        `irank` is calculated as the number of singular values greater
        than `tol`* (largest singular value).

        It is possible for the SVD to be carried out but `irank` to be
        returned as ip.

    p : float, ndarray, shape (ip*ip+2*ip)
        Contains details of the singular value decomposition if used.

        If `svd` = False, `p` is not referenced.

        If `svd` = True, the first ip elements of `p` are unchanged, the
        next ip values contain the singular values.

        The following ip*ip values contain the matrix P* stored by
        columns.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` >= 0.0.

        (`errno` 1)
            On entry, ldq = *<value>* and ip = *<value>*.

            Constraint: if `rss` <= 0.0, ldq >= n.

        (`errno` 1)
            On entry, ip = *<value>*.

            Constraint: ip >= 1.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 2)
            The degrees of freedom for error are less than or equal to
            0. In this case the estimates of beta are returned but not
            the standard errors or covariances.

        (`errno` 3)
            SVD solution failed to converge.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.correg import linregm_update as b_linregm_update

    fun_name = 'naginterfaces.library.correg.linregm_update'

    _q = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'q'},
        q,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _ip = _b_utils._EngineIntScalarType(
        _b_utils._EngineFloat64ArrayType.get_shape(_q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2)[1]-1, {'fun_name': fun_name, 'entity': '_ip'},
    ).to_ctypes()
    _ip_py = _b_utils._EngineIntScalarType.to_py(_ip)
    _b = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_ip_py,),
    )
    _se = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_ip_py,),
    )
    _cov = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_ip_py*(_ip_py+1)//2,),
    )
    _p = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'p'},
        p,
        copy=True,
    )
    _return_scalars = b_linregm_update(
        _n,
        _ip,
        _q,
        rss,
        _b,
        _se,
        _cov,
        _p,
        tol,
    )
    _rss_py = _return_scalars.rss
    _p_py = _p
    _idf_py = _return_scalars.idf
    _b_py = _b
    _se_py = _se
    _cov_py = _cov
    _svd_py = _return_scalars.svd
    _irank_py = _return_scalars.irank
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['rss', 'idf', 'b', 'se', 'cov', 'svd', 'irank', 'p'],
    )(
        _rss_py,
        _idf_py,
        _b_py,
        _se_py,
        _cov_py,
        _svd_py,
        _irank_py,
        _p_py,
    )
    return _return_data

def linregm_var_add(q, p, x, wt=None, tol=0.000001):
    u"""
    Add a new independent variable to a general linear regression model.

    For full information please refer to the NAG Library document for
    g02de

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02def.html

    Parameters
    ----------
    q : float, array-like, shape (n, ip+2)
        If ip != 0, `q` must contain the results of the QR decomposition
        for the model with p parameters as returned by ``linregm_fit``
        or a previous call to ``linregm_var_add``.

        If ip = 0, the first column of `q` should contain the n values
        of the dependent variable, y.

    p : float, array-like, shape (ip+1)
        Contains further details of the QR decomposition used. The first
        ip elements of `p` must contain the zeta values for the QR
        decomposition (see ``lapackeig.dgeqrf`` for details).

        The first ip elements of array `p` are provided by
        ``linregm_fit`` or by previous calls to ``linregm_var_add``.

    x : float, array-like, shape (n)
        x, the new independent variable.

    wt : None or float, array-like, shape (n), optional
        If provided `wt` must contain the weights to be used with the
        model.

        If `wt`[i-1] = 0.0, the ith observation is not included in the
        model, in which case the effective number of observations is the
        number of observations with nonzero weights.

        If `wt` is not provided the effective number of observations is
        n.

    tol : float, optional
        The value of `tol` is used to decide if the new independent
        variable is linearly related to independent variables already
        included in the model. If the new variable is linearly related
        then c is not updated. The smaller the value of `tol` the
        stricter the criterion for deciding if there is a linear
        relationship.

    Returns
    -------
    q : float, ndarray, shape (n, ip+2)
        The results of the QR decomposition for the model with p+1
        parameters:

        - the first column of `q` contains the updated value of c;

        - the columns 2 to ip+1 are unchanged;

        - the first ip+1 elements of column ip+2 contain the new column
          of R, while the remaining n-ip-1 elements contain details of
          the matrix Q_{p+1}.

    p : float, ndarray, shape (ip+1)
        The first ip elements of `p` are unchanged and the (ip+1)th
        element contains the zeta value for Q_{p+1}.

    rss : float
        The residual sum of squares for the new fitted model.

        Note: this will only be valid if the model is of full rank, see
        [Further Comments].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` > 0.0.

        (`errno` 1)
            On entry, ip = *<value>* and n = *<value>*.

            Constraint: ip < n.

        (`errno` 1)
            On entry, weight = *<value>*.

            Constraint: weight = 'W' or 'U'.

        (`errno` 1)
            On entry, ip = *<value>*.

            Constraint: ip >= 0.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 2)
            On entry, `wt`[*<value>*] < 0.0.

            Constraint: `wt`[i-1] >= 0.0, for i = 1,2,...,n.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            `x` variable is a linear combination of existing model
            terms.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.correg import linregm_var_add as b_linregm_var_add

    fun_name = 'naginterfaces.library.correg.linregm_var_add'

    _q = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'q'},
        q,
        copy=True,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _p = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'p'},
        p,
        copy=True,
    )
    _ip = _b_utils._EngineIntScalarType(
        _b_utils._EngineFloat64ArrayType.get_shape(_p, {'fun_name': fun_name, 'entity': 'p'}, exp_rank=1)[1]-1, {'fun_name': fun_name, 'entity': '_ip'},
    ).to_ctypes()
    _wt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'wt'},
        wt,
        copy=False,
        allow_none=True,
    )
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    try:
        _return_scalar = b_linregm_var_add(
            _n,
            _ip,
            _q,
            _p,
            _wt,
            _x,
            tol,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _q_py = _q
    _p_py = _p
    _rss_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['q', 'p', 'rss'],
    )(
        _q_py,
        _p_py,
        _rss_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def linregm_var_del(q, indx, rss):
    u"""
    Delete an independent variable from a general linear regression
    model.

    For full information please refer to the NAG Library document for
    g02df

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02dff.html

    Parameters
    ----------
    q : float, array-like, shape (ip, ip+1)
        The results of the QR decomposition as returned by functions
        ``linregm_fit``, ``linregm_obs_edit``, ``linregm_var_add`` or
        ``linregm_fit_onestep``, or previous calls to
        ``linregm_var_del``.

    indx : int
        Indicates which independent variable is to be deleted from the
        model.

    rss : float
        The residual sum of squares for the full regression.

    Returns
    -------
    q : float, ndarray, shape (ip, ip+1)
        The updated QR decomposition.

    rss : float
        The residual sum of squares with the (`indx`)th variable
        removed. Note that the residual sum of squares will only be
        valid if the regression is of full rank, otherwise the residual
        sum of squares should be obtained using ``linregm_update``.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `rss` = *<value>*.

            Constraint: `rss` >= 0.0.

        (`errno` 1)
            On entry, `indx` = *<value>*.

            Constraint: `indx` >= 1 and `indx` <= ip.

        (`errno` 1)
            On entry, ip = *<value>*.

            Constraint: ip >= 1.

        (`errno` 2)
            On entry, `q`[*<value>*,*<value>*] = 0.0.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.correg import linregm_var_del as b_linregm_var_del

    fun_name = 'naginterfaces.library.correg.linregm_var_del'

    _q = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'q'},
        q,
        copy=True,
    )
    _ip_py = _b_utils._EngineFloat64ArrayType.get_shape(_q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2)[0]
    _ip = _b_utils._EngineIntScalarType(
        _ip_py, {'fun_name': fun_name, 'entity': '_ip'},
    ).to_ctypes()
    _return_scalar = b_linregm_var_del(
        _ip,
        _q,
        indx,
        rss,
    )
    _q_py = _q
    _rss_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['q', 'rss'],
    )(
        _q_py,
        _rss_py,
    )
    return _return_data

def linregm_fit_newvar(rss, irank, cov, q, svd, p, y, wk, wt=None):
    u"""
    Fits a general linear regression model to new dependent variable.

    For full information please refer to the NAG Library document for
    g02dg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02dgf.html

    Parameters
    ----------
    rss : float
        The residual sum of squares for the original dependent variable.

    irank : int
        The rank of the independent variables, as given by
        ``linregm_fit``.

    cov : float, array-like, shape (ip*(ip+1)/2)
        The covariance matrix of the parameter estimates as given by
        ``linregm_fit``.

    q : float, array-like, shape (n, ip+1)
        The results of the QR decomposition as returned by
        ``linregm_fit``.

    svd : bool
        Indicates if a singular value decomposition was used by
        ``linregm_fit``.

        `svd` = True

          A singular value decomposition was used by ``linregm_fit``.

        `svd` = False

          A singular value decomposition was not used by
          ``linregm_fit``.

    p : float, array-like, shape (:)
        Note: the required length for this argument is determined as
        follows: if not((`svd`): ip; otherwise: ip*ip+2*ip.

        Details of the QR decomposition and SVD, if used, as returned in
        array `p` by ``linregm_fit``.

        If `svd` = False, only the first ip elements of `p` are used;
        these contain the zeta values for the QR decomposition (see
        ``lapackeig.dgeqrf`` for details).

        If `svd` = True, the first ip elements of `p` contain the zeta
        values for the QR decomposition (see ``lapackeig.dgeqrf`` for
        details) and the next ip*ip+ip elements of `p` contain details
        of the singular value decomposition.

    y : float, array-like, shape (n)
        The new dependent variable, y_new.

    wk : float, array-like, shape (5*(ip-1)+ip*ip)
        If `svd` = True, `wk` must be unaltered from the previous call
        to ``linregm_fit`` or ``linregm_fit_newvar``.

        If `svd` = False, `wk` is used as workspace.

    wt : None or float, array-like, shape (n), optional
        If provided `wt` must contain the weights to be used with the
        model.

        If `wt`[i-1] = 0.0, the ith observation is not included in the
        model, in which case the effective number of observations is the
        number of observations with nonzero weights.

        If `wt` is not provided the effective number of observations is
        n.

    Returns
    -------
    rss : float
        The residual sum of squares for the new dependent variable.

    cov : float, ndarray, shape (ip*(ip+1)/2)
        The upper triangular part of the variance-covariance matrix of
        the ip parameter estimates given in `b`. They are stored packed
        by column, i.e., the covariance between the parameter estimate
        given in `b`[i-1] and the parameter estimate given in `b`[j-1],
        j >= i, is stored in `cov`[(j*(j-1)/2+i)-1].

    q : float, ndarray, shape (n, ip+1)
        The first column of `q` contains the new values of c, the
        remainder of `q` will be unchanged.

    b : float, ndarray, shape (ip)
        The least squares estimates of the parameters of the regression
        model, beta^.

    se : float, ndarray, shape (ip)
        The standard error of the estimates of the parameters.

    res : float, ndarray, shape (n)
        The residuals for the new regression model.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `irank` = *<value>*.

            Constraint: `irank` > 0.

        (`errno` 1)
            On entry, `irank` = *<value>* and ip = *<value>*.

            Constraint: if `svd` = False, `irank` = ip.

        (`errno` 1)
            On entry, `irank` = *<value>* and ip = *<value>*.

            Constraint: if `svd` = True, `irank` <= ip.

        (`errno` 1)
            On entry, weight = *<value>*.

            Constraint: weight = 'W' or 'U'.

        (`errno` 1)
            On entry, `rss` = *<value>*.

            Constraint: `rss` > 0.0.

        (`errno` 1)
            On entry, n = *<value>* and ip = *<value>*.

            Constraint: n >= ip.

        (`errno` 1)
            On entry, ip = *<value>*.

            Constraint: ip >= 1.

        (`errno` 2)
            On entry, `wt`[*<value>*] < 0.0.

            Constraint: `wt`[i-1] >= 0.0, for i = 1,2,...,n.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.correg import linregm_fit_newvar as b_linregm_fit_newvar

    fun_name = 'naginterfaces.library.correg.linregm_fit_newvar'

    _q = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'q'},
        q,
        copy=True,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _wt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'wt'},
        wt,
        copy=False,
        allow_none=True,
    )
    _ip = _b_utils._EngineIntScalarType(
        _b_utils._EngineFloat64ArrayType.get_shape(_q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=2)[1]-1, {'fun_name': fun_name, 'entity': '_ip'},
    ).to_ctypes()
    _cov = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'cov'},
        cov,
        copy=True,
    )
    _p = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'p'},
        p,
        copy=False,
    )
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _ip_py = _b_utils._EngineIntScalarType.to_py(_ip)
    _b = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_ip_py,),
    )
    _se = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_ip_py,),
    )
    _res = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _wk = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'wk'},
        wk,
        copy=False,
    )
    _return_scalar = b_linregm_fit_newvar(
        _n,
        _wt,
        rss,
        _ip,
        irank,
        _cov,
        _q,
        svd,
        _p,
        _y,
        _b,
        _se,
        _res,
        _wk,
    )
    _rss_py = _return_scalar
    _cov_py = _cov
    _q_py = _q
    _b_py = _b
    _se_py = _se
    _res_py = _res
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['rss', 'cov', 'q', 'b', 'se', 'res'],
    )(
        _rss_py,
        _cov_py,
        _q_py,
        _b_py,
        _se_py,
        _res_py,
    )
    return _return_data

def linregm_constrain(p, c, b, rss, idf):
    u"""
    Estimates and standard errors of parameters of a general linear
    regression model for given constraints.

    For full information please refer to the NAG Library document for
    g02dk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02dkf.html

    Parameters
    ----------
    p : float, array-like, shape (ip*ip+2*ip)
        As returned by ``linregm_fit`` and ``linregm_update``.

    c : float, array-like, shape (ip, iconst)
        The iconst constraints stored by column, i.e., the ith
        constraint is stored in the ith column of `c`.

    b : float, array-like, shape (ip)
        The parameter estimates computed by using the singular value
        decomposition, beta^_svd.

    rss : float
        The residual sum of squares as returned by ``linregm_fit`` or
        ``linregm_update``.

    idf : int
        The degrees of freedom associated with the residual sum of
        squares as returned by ``linregm_fit`` or ``linregm_update``.

    Returns
    -------
    b : float, ndarray, shape (ip)
        The parameter estimates of the parameters with the constraints
        imposed, beta^_c.

    se : float, ndarray, shape (ip)
        The standard error of the parameter estimates in `b`.

    cov : float, ndarray, shape (ip*(ip+1)/2)
        The upper triangular part of the variance-covariance matrix of
        the ip parameter estimates given in `b`. They are stored packed
        by column, i.e., the covariance between the parameter estimate
        given in `b`[i-1] and the parameter estimate given in `b`[j-1],
        j >= i, is stored in `cov`[(j*(j-1)/2+i)-1].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `idf` = *<value>*.

            Constraint: `idf` > 0.

        (`errno` 1)
            On entry, iconst = *<value>*.

            Constraint: iconst > 0.

        (`errno` 1)
            On entry, `rss` = *<value>*.

            Constraint: `rss` > 0.0.

        (`errno` 1)
            On entry, iconst = *<value>* and ip = *<value>*.

            Constraint: iconst < ip.

        (`errno` 1)
            On entry, ip = *<value>*.

            Constraint: ip >= 1.

        (`errno` 2)
            `c` does not give a model of full rank.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.correg import linregm_constrain as b_linregm_constrain

    fun_name = 'naginterfaces.library.correg.linregm_constrain'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=False,
    )
    _ip_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[0]
    _ip = _b_utils._EngineIntScalarType(
        _ip_py, {'fun_name': fun_name, 'entity': '_ip'},
    ).to_ctypes()
    _iconst_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[1]
    _iconst = _b_utils._EngineIntScalarType(
        _iconst_py, {'fun_name': fun_name, 'entity': '_iconst'},
    ).to_ctypes()
    _p = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'p'},
        p,
        copy=False,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _se = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_ip_py,),
    )
    _cov = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_ip_py*(_ip_py+1)//2,),
    )
    b_linregm_constrain(
        _ip,
        _iconst,
        _p,
        _c,
        _b,
        rss,
        idf,
        _se,
        _cov,
    )
    _b_py = _b
    _se_py = _se
    _cov_py = _cov
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['b', 'se', 'cov'],
    )(
        _b_py,
        _se_py,
        _cov_py,
    )
    return _return_data

def linregm_estfunc(irank, b, cov, p, f, tol=0.0):
    u"""
    Computes estimable function of a general linear regression model and
    its standard error.

    For full information please refer to the NAG Library document for
    g02dn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02dnf.html

    Parameters
    ----------
    irank : int
        k, the rank of the independent variables.

    b : float, array-like, shape (ip)
        The ip values of the estimates of the parameters of the model,
        beta^.

    cov : float, array-like, shape (ip*(ip+1)/2)
        The upper triangular part of the variance-covariance matrix of
        the ip parameter estimates given in `b`. They are stored packed
        by column, i.e., the covariance between the parameter estimate
        given in `b`[i-1] and the parameter estimate given in `b`[j-1],
        j >= i, is stored in `cov`[(j*(j-1)/2+i)-1].

    p : float, array-like, shape (ip*ip+2*ip)
        As returned by ``linregm_fit`` and ``linregm_update``.

    f : float, array-like, shape (ip)
        f, the linear function to be estimated.

    tol : float, optional
        eta, the tolerance value used in the check for estimability.

        If `tol` <= 0.0 then sqrt(epsilon), where epsilon is the machine
        precision, is used instead.

    Returns
    -------
    est : bool
        Indicates if the function was estimable.

        `est` = True

          The function is estimable.

        `est` = False

          The function is not estimable and `stat`, `sestat` and `t` are
          not set.

    stat : float
        If `est` = True, `stat` contains the estimate of the function,
        f^T beta^.

    sestat : float
        If `est` = True, `sestat` contains the standard error of the
        estimate of the function, se(F).

    t : float
        If `est` = True, `t` contains the t-statistic for the test of
        the function being equal to zero.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `irank` = *<value>* and ip = *<value>*.

            Constraint: `irank` <= ip.

        (`errno` 1)
            On entry, `irank` = *<value>*.

            Constraint: `irank` >= 1.

        (`errno` 1)
            On entry, ip = *<value>*.

            Constraint: ip >= 1.

        (`errno` 3)
            Standard error of statistic = 0.0; this may be due to
            rounding errors if the standard error is very small or due
            to mis-specified inputs `cov` and `f`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            On entry, `irank` = ip, i.e., model of full rank. In this
            case `est` is returned as true and all statistics are
            calculated.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.correg import linregm_estfunc as b_linregm_estfunc

    fun_name = 'naginterfaces.library.correg.linregm_estfunc'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
    )
    _ip_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=1)[0]
    _ip = _b_utils._EngineIntScalarType(
        _ip_py, {'fun_name': fun_name, 'entity': '_ip'},
    ).to_ctypes()
    _cov = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'cov'},
        cov,
        copy=False,
    )
    _p = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'p'},
        p,
        copy=False,
    )
    _f = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'f'},
        f,
        copy=False,
    )
    try:
        _return_scalars = b_linregm_estfunc(
            _ip,
            irank,
            _b,
            _cov,
            _p,
            _f,
            tol,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _est_py = _return_scalars.est
    _stat_py = _return_scalars.stat
    _sestat_py = _return_scalars.sestat
    _t_py = _return_scalars.t
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['est', 'stat', 'sestat', 't'],
    )(
        _est_py,
        _stat_py,
        _sestat_py,
        _t_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def linregm_rssq(x, vname, isx, y, mean='M', wt=None):
    u"""
    Computes residual sums of squares for all possible linear
    regressions for a set of independent variables.

    For full information please refer to the NAG Library document for
    g02ea

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02eaf.html

    Parameters
    ----------
    x : float, array-like, shape (n, m)
        `x`[i-1,j-1] must contain the ith observation for the jth
        independent variable, for j = 1,2,...,m, for i = 1,2,...,n.

    vname : str, array-like, shape (m)
        `vname`[j-1] must contain the name of the variable in column j
        of `x`, for j = 1,2,...,m.

    isx : int, array-like, shape (m)
        Indicates which independent variables are to be considered in
        the model.

        `isx`[j-1] >= 2

          The variable contained in the jth column of `x` is included in
          all regression models, i.e., is a forced variable.

        `isx`[j-1] = 1

          The variable contained in the jth column of `x` is included in
          the set from which the regression models are chosen, i.e., is
          a free variable.

        `isx`[j-1] = 0

          The variable contained in the jth column of `x` is not
          included in the models.

    y : float, array-like, shape (n)
        `y`[i-1] must contain the ith observation on the dependent
        variable, y_i, for i = 1,2,...,n.

    mean : str, length 1, optional
        Indicates if a mean term is to be included.

        `mean` = 'M'

          A mean term, intercept, will be included in the model.

        `mean` = 'Z'

          The model will pass through the origin, zero-point.

    wt : None or float, array-like, shape (n), optional
        If provided `wt` must contain the weights to be used with the
        model.

        If `wt`[i-1] = 0.0, the ith observation is not included in the
        model, in which case the effective number of observations is the
        number of observations with nonzero weights.

        If `wt` is not provided the effective number of observations is
        n.

    Returns
    -------
    nmod : int
        The total number of models for which residual sums of squares
        have been calculated.

    modl : str, ndarray, shape (max(2^k, m), m)
        The first `nterms`[i-1] elements of the ith row of `modl`
        contain the names of the independent variables, as given in
        `vname`, that are included in the ith model.

    rss : float, ndarray, shape (max(2^k, m))
        `rss`[i-1] contains the residual sum of squares for the ith
        model, for i = 1,2,...,`nmod`.

    nterms : int, ndarray, shape (max(2^k, m))
        `nterms`[i-1] contains the number of independent variables in
        the ith model, not including the mean if one is fitted, for i =
        1,2,...,`nmod`.

    mrank : int, ndarray, shape (max(2^k, m))
        `mrank`[i-1] contains the rank of the residual sum of squares
        for the ith model.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, ldmodl = *<value>* and m = *<value>*.

            Constraint: ldmodl >= m.

        (`errno` 1)
            On entry, weight = *<value>*.

            Constraint: weight = 'W' or 'U'.

        (`errno` 1)
            On entry, `mean` = *<value>*.

            Constraint: `mean` = 'M' or 'Z'.

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 2.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, `wt`[*<value>*] < 0.0.

            Constraint: `wt`[i-1] >= 0.0, for i = 1,2,...,n.

        (`errno` 3)
            On entry, `isx`[*<value>*] < 0.

            Constraint: `isx`[i-1] >= 0, for i = 1,2,...,m.

        (`errno` 3)
            There are no free variables, i.e., no element of `isx` = 1.

        (`errno` 4)
            On entry, ldmodl = *<value>* and number of possible models
            is *<value>*.

            Constraint: ldmodl >= the number of possible models.

        (`errno` 5)
            On entry, the number of independent variables to be
            considered (forced plus free plus mean if included) is
            greater or equal to the effective number of observations.

        (`errno` 6)
            The full model is not of full rank, i.e., some of the
            independent variables may be linear combinations of other
            independent variables. Variables must be excluded from the
            model in order to give full rank.
    """

    from ..base.correg import linregm_rssq as b_linregm_rssq

    fun_name = 'naginterfaces.library.correg.linregm_rssq'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _vname = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vname'},
        vname,
        copy=False,
        exp_class=_b_utils._EngineCharArrayType,
    )
    _isx = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'isx'},
        isx,
        copy=False,
    )
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _wt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'wt'},
        wt,
        copy=False,
        allow_none=True,
    )
    _vl_k_py = 0
    for _vl_i_py in range(1, _m_py+1):
        if _b_utils._EngineIntArrayType.get_element(_isx, {'fun_name': fun_name, 'entity': 'isx'}, (_vl_i_py-1,)) == 1:
            _vl_k_py = _vl_k_py + 1
    _enmod = _b_utils._EngineIntScalarType(
        max(2**_vl_k_py, _m_py), {'fun_name': fun_name, 'entity': '_enmod'},
    ).to_ctypes()
    _modl = _b_utils._EngineCharArrayType.empty_ndarray(
        (max(2**_vl_k_py, _m_py), _m_py),
        sorder=_m_sorder,
        chrlen=max(len(v) for v in _vname),
    )
    _rss = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (max(2**_vl_k_py, _m_py),),
    )
    _nterms = _b_utils._EngineIntArrayType.empty_ndarray(
        (max(2**_vl_k_py, _m_py),),
    )
    _mrank = _b_utils._EngineIntArrayType.empty_ndarray(
        (max(2**_vl_k_py, _m_py),),
    )
    _return_scalar = b_linregm_rssq(
        mean,
        _n,
        _m,
        _x,
        _vname,
        _isx,
        _y,
        _wt,
        _enmod,
        _modl,
        _rss,
        _nterms,
        _mrank,
    )
    _nmod_py = _return_scalar
    _modl_py = _modl
    _rss_py = _rss
    _nterms_py = _nterms
    _mrank_py = _mrank
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['nmod', 'modl', 'rss', 'nterms', 'mrank'],
    )(
        _nmod_py,
        _modl_py,
        _rss_py,
        _nterms_py,
        _mrank_py,
    )
    return _return_data

def linregm_rssq_stat(n, sigsq, tss, nterms, rss, mean='M'):
    u"""
    Calculates R^2 and C_P values from residual sums of squares.

    For full information please refer to the NAG Library document for
    g02ec

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02ecf.html

    Parameters
    ----------
    n : int
        n, the number of observations used in the regression model.

    sigsq : float
        The best estimate of true variance of the errors, sigma^^2.

    tss : float
        The total sum of squares for the regression model.

    nterms : int, array-like, shape (nmod)
        `nterms`[i-1] must contain the number of independent variables
        (not counting the mean) fitted to the ith model, for i =
        1,2,...,nmod.

    rss : float, array-like, shape (nmod)
        `rss`[i-1] must contain the residual sum of squares for the ith
        model.

    mean : str, length 1, optional
        Indicates if a mean term is to be included.

        `mean` = 'M'

          A mean term, intercept, will be included in the model.

        `mean` = 'Z'

          The model will pass through the origin, zero-point.

    Returns
    -------
    rsq : float, ndarray, shape (nmod)
        `rsq`[i-1] contains the R^2-value for the ith model, for i =
        1,2,...,nmod.

    cp : float, ndarray, shape (nmod)
        `cp`[i-1] contains the C_p-value for the ith model, for i =
        1,2,...,nmod.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `mean` = *<value>*.

            Constraint: `mean` = 'M' or 'Z'.

        (`errno` 1)
            On entry, `tss` = *<value>*.

            Constraint: `tss` > 0.0.

        (`errno` 1)
            On entry, `sigsq` = *<value>*.

            Constraint: `sigsq` > 0.0.

        (`errno` 1)
            On entry, nmod = *<value>*.

            Constraint: nmod > 0.

        (`errno` 2)
            On entry: the number of parameters, p, is *<value>* and `n`
            = *<value>*.

            Constraint: `n` >= 2*p.

        (`errno` 3)
            On entry, `rss`[*<value>*] = *<value>* and `tss` =
            *<value>*.

            Constraint: `rss`[i] <= `tss`, for all i.

        (`errno` 4)
            A value of C_p is less than 0.0. This may occur if `sigsq`
            is too large or if `rss`, `n` or IP are incorrect.
    """

    from ..base.correg import linregm_rssq_stat as b_linregm_rssq_stat

    fun_name = 'naginterfaces.library.correg.linregm_rssq_stat'

    _nterms = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'nterms'},
        nterms,
        copy=False,
    )
    _nmod_py = _b_utils._EngineIntArrayType.get_shape(_nterms, {'fun_name': fun_name, 'entity': 'nterms'}, exp_rank=1)[0]
    _nmod = _b_utils._EngineIntScalarType(
        _nmod_py, {'fun_name': fun_name, 'entity': '_nmod'},
    ).to_ctypes()
    _rss = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'rss'},
        rss,
        copy=False,
    )
    _rsq = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nmod_py,),
    )
    _cp = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nmod_py,),
    )
    b_linregm_rssq_stat(
        mean,
        n,
        sigsq,
        tss,
        _nmod,
        _nterms,
        _rss,
        _rsq,
        _cp,
    )
    _rsq_py = _rsq
    _cp_py = _cp
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['rsq', 'cp'],
    )(
        _rsq_py,
        _cp_py,
    )
    return _return_data

def linregm_fit_onestep(istep, x, vname, isx, y, model, nterm, rss, idf, ifr, free, q, p, mean='M', wt=None, fin=2.0):
    u"""
    Fits a linear regression model by forward selection.

    For full information please refer to the NAG Library document for
    g02ee

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02eef.html

    Parameters
    ----------
    istep : int
        Indicates which step in the forward selection process is to be
        carried out.

        `istep` = 0

          The process is initialized.

    x : float, array-like, shape (n, m)
        `x`[i-1,j-1] must contain the ith observation for the jth
        independent variable, for j = 1,2,...,m, for i = 1,2,...,n.

    vname : str, array-like, shape (m)
        `vname`[j-1] must contain the name of the independent variable
        in column j of `x`, for j = 1,2,...,m.

    isx : int, array-like, shape (m)
        Indicates which independent variables could be considered for
        inclusion in the regression.

        `isx`[j-1] >= 2

          The variable contained in the jth column of `x` is
          automatically included in the regression model, for j =
          1,2,...,m.

        `isx`[j-1] = 1

          The variable contained in the jth column of `x` is considered
          for inclusion in the regression model, for j = 1,2,...,m.

        `isx`[j-1] = 0

          The variable in the jth column is not considered for inclusion
          in the model, for j = 1,2,...,m.

    y : float, array-like, shape (n)
        The dependent variable.

    model : str, array-like, shape (maxip)
        If `istep` = 0, `model` need not be set.

        If `istep` != 0, `model` must contain the values returned by the
        previous call to ``linregm_fit_onestep``.

    nterm : int
        If `istep` = 0, `nterm` need not be set.

        If `istep` != 0, `nterm` must contain the value returned by the
        previous call to ``linregm_fit_onestep``.

    rss : float
        If `istep` = 0, `rss` need not be set.

        If `istep` != 0, `rss` must contain the value returned by the
        previous call to ``linregm_fit_onestep``.

    idf : int
        If `istep` = 0, `idf` need not be set.

        If `istep` != 0, `idf` must contain the value returned by the
        previous call to ``linregm_fit_onestep``.

    ifr : int
        If `istep` = 0, `ifr` need not be set.

        If `istep` != 0, `ifr` must contain the value returned by the
        previous call to ``linregm_fit_onestep``.

    free : str, array-like, shape (maxip)
        If `istep` = 0, `free` need not be set.

        If `istep` != 0, `free` must contain the values returned by the
        previous call to ``linregm_fit_onestep``.

    q : float, array-like, shape (n, maxip+2)
        If `istep` = 0, `q` need not be set.

        If `istep` != 0, `q` must contain the values returned by the
        previous call to ``linregm_fit_onestep``.

    p : float, array-like, shape (maxip+1)
        If `istep` = 0, `p` need not be set.

        If `istep` != 0, `p` must contain the values returned by the
        previous call to ``linregm_fit_onestep``.

    mean : str, length 1, optional
        Indicates if a mean term is to be included.

        `mean` = 'M'

          A mean term, intercept, will be included in the model.

        `mean` = 'Z'

          The model will pass through the origin, zero-point.

    wt : None or float, array-like, shape (n), optional
        If provided `wt` must contain the weights to be used with the
        model.

        If `wt`[i-1] = 0.0, the ith observation is not included in the
        model, in which case the effective number of observations is the
        number of observations with nonzero weights.

        If `wt` is not provided the effective number of observations is
        n.

    fin : float, optional
        The critical value of the F statistic for the term to be
        included in the model, F_c.

    Returns
    -------
    istep : int
        Is incremented by 1.

    addvar : bool
        Indicates if a variable has been added to the model.

        `addvar` = True

          A variable has been added to the model.

        `addvar` = False

          No variable had an F value greater than F_c and none were
          added to the model.

    newvar : str
        If `addvar` = True, `newvar` contains the name of the variable
        added to the model.

    chrss : float
        If `addvar` = True, `chrss` contains the change in the residual
        sum of squares due to adding variable `newvar`.

    f : float
        If `addvar` = True, `f` contains the F statistic for the
        inclusion of the variable in `newvar`.

    model : str, ndarray, shape (maxip)
        The names of the variables in the current model.

    nterm : int
        The number of independent variables in the current model, not
        including the mean, if any.

    rss : float
        The residual sums of squares for the current model.

    idf : int
        The degrees of freedom for the residual sum of squares for the
        current model.

    ifr : int
        The number of free independent variables, i.e., the number of
        variables not in the model that are still being considered for
        selection.

    free : str, ndarray, shape (maxip)
        The first `ifr` values of `free` contain the names of the free
        variables.

    exss : float, ndarray, shape (maxip)
        The first `ifr` values of `exss` contain what would be the
        change in regression sum of squares if the free variables had
        been added to the model, i.e., the extra sum of squares for the
        free variables. `exss`[i-1] contains what would be the change in
        regression sum of squares if the variable `free`[i-1] had been
        added to the model.

    q : float, ndarray, shape (n, maxip+2)
        The results of the QR decomposition for the current model:

        - the first column of `q` contains c = Q^Ty (or Q^TW^(1/2)y
          where W is the vector of weights if used);

        - the upper triangular part of columns 2 to p+1 contain the R
          matrix;

        - the strictly lower triangular part of columns 2 to p+1 contain
          details of the Q matrix;

        - the remaining p+1 to p+`ifr` columns of contain Q^TX_free (or
          Q^TW^(1/2)X_free),

        where p = `nterm`, or p = `nterm`+1 if `mean` = 'M'.

    p : float, ndarray, shape (maxip+1)
        The first p elements of `p` contain details of the QR
        decomposition, where p = `nterm`, or p = `nterm`+1 if `mean` =
        'M'.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `fin` = *<value>*.

            Constraint: `fin` >= 0.0.

        (`errno` 1)
            On entry, `rss` = *<value>*.

            Constraint: `rss` > 0.0.

        (`errno` 1)
            On entry, `istep` = *<value>* and `nterm` = *<value>*.

            Constraint: if `istep` != 0, `nterm` > 0.

        (`errno` 1)
            On entry, `istep` = *<value>*.

            Constraint: `istep` >= 0.

        (`errno` 1)
            On entry, weight = *<value>*.

            Constraint: weight = 'W' or 'U'.

        (`errno` 1)
            On entry, `mean` = *<value>*.

            Constraint: `mean` = 'M' or 'Z'.

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, `wt`[*<value>*] < 0.0.

            Constraint: `wt`[i-1] >= 0.0, for i = 1,2,...,n.

        (`errno` 3)
            On entry, number of forced variables >= n.

        (`errno` 3)
            Degrees of freedom for error will equal 0 if new variable is
            added, i.e., the number of variables in the model plus 1 is
            equal to the effective number of observations.

        (`errno` 4)
            On entry, maxip = *<value>*.

            Constraint: maxip must be large enough to accommodate the
            number of terms given by `isx`.

        (`errno` 4)
            On entry, `isx`[*<value>*] < 0.

            Constraint: `isx`[i-1] >= 0, for i = 1,2,...,m.

        (`errno` 4)
            On entry, `isx`[i-1] = 0, for all i = 1,2,...,m.

            Constraint: at least one value of `isx` must be nonzero.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 5)
            On entry, the variables forced into the model are not of
            full rank, i.e., some of these variables are linear
            combinations of others.

        (`errno` 6)
            There are no free variables, i.e., no element of `isx` = 0.

        (`errno` 7)
            The value of the change in the sum of squares is greater
            than the input value of `rss`. This may occur due to
            rounding errors if the true residual sum of squares for the
            new model is small relative to the residual sum of squares
            for the previous model.
    """

    from ..base.correg import linregm_fit_onestep as b_linregm_fit_onestep

    fun_name = 'naginterfaces.library.correg.linregm_fit_onestep'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _vname = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vname'},
        vname,
        copy=False,
    )
    _isx = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'isx'},
        isx,
        copy=False,
    )
    _model = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'model'},
        model,
        copy=True,
    )
    _maxip_py = _b_utils._EngineCharArrayType.get_shape(_model, {'fun_name': fun_name, 'entity': 'model'}, exp_rank=1)[0]
    _maxip = _b_utils._EngineIntScalarType(
        _maxip_py, {'fun_name': fun_name, 'entity': '_maxip'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _wt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'wt'},
        wt,
        copy=False,
        allow_none=True,
    )
    _free = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'free'},
        free,
        copy=True,
    )
    _exss = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_maxip_py,),
    )
    _q = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'q'},
        q,
        copy=True,
    )
    _p = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'p'},
        p,
        copy=True,
    )
    try:
        _return_scalars = b_linregm_fit_onestep(
            istep,
            mean,
            _n,
            _m,
            _x,
            _vname,
            _isx,
            _maxip,
            _y,
            _wt,
            fin,
            _model,
            nterm,
            rss,
            idf,
            ifr,
            _free,
            _exss,
            _q,
            _p,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _istep_py = _return_scalars.istep
    _model_py = _model
    _nterm_py = _return_scalars.nterm
    _rss_py = _return_scalars.rss
    _idf_py = _return_scalars.idf
    _ifr_py = _return_scalars.ifr
    _free_py = _free
    _q_py = _q
    _p_py = _p
    _addvar_py = _return_scalars.addvar
    _newvar_py = _return_scalars.newvar
    _chrss_py = _return_scalars.chrss
    _f_py = _return_scalars.f
    _exss_py = _exss
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['istep', 'addvar', 'newvar', 'chrss', 'f', 'model', 'nterm', 'rss', 'idf', 'ifr', 'free', 'exss', 'q', 'p'],
    )(
        _istep_py,
        _addvar_py,
        _newvar_py,
        _chrss_py,
        _f_py,
        _model_py,
        _nterm_py,
        _rss_py,
        _idf_py,
        _ifr_py,
        _free_py,
        _exss_py,
        _q_py,
        _p_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def linregm_fit_stepwise(n, wmean, c, sw, isx, fin=4.0, fout=None, tau=0.000001, monlev=0, monfun=None, data=None, io_manager=None):
    u"""
    Stepwise linear regression.

    For full information please refer to the NAG Library document for
    g02ef

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02eff.html

    Parameters
    ----------
    n : int
        The number of observations used in the calculations.

    wmean : float, array-like, shape (m+1)
        The mean of the design matrix, Z.

    c : float, array-like, shape ((m+1)*(m+2)/2)
        The upper-triangular variance-covariance matrix packed by column
        for the design matrix, Z. Because the function computes the
        correlation matrix R from `c`, the variance-covariance matrix
        need only be supplied up to a scaling factor.

    sw : float
        If weights were used to calculate `c` then `sw` is the sum of
        positive weight values; otherwise `sw` is the number of
        observations used to calculate `c`.

    isx : int, array-like, shape (m)
        The value of `isx`[j-1] determines the set of variables used to
        perform full stepwise model selection, for j = 1,2,...,m.

        `isx`[j-1] = -1

          To exclude the variable corresponding to the jth column of X
          from the final model.

        `isx`[j-1] = 1

          To consider the variable corresponding to the jth column of X
          for selection in the final model.

        `isx`[j-1] = 2

          To force the inclusion of the variable corresponding to the
          jth column of X in the final model.

    fin : float, optional
        The value of the variance ratio which an explanatory variable
        must exceed to be included in a model.

    fout : float, optional
        Note: if this argument is None then a default value will be
        used, determined as follows: `fin`.

        The explanatory variable in a model with the lowest variance
        ratio value is removed from the model if its value is less than
        `fout`. `fout` is usually set equal to the value of `fin`; a
        value less than `fin` is occasionally preferred.

    tau : float, optional
        The tolerance, tau, for detecting collinearities between
        variables when adding or removing an explanatory variable from a
        model. Explanatory variables deemed to be collinear are excluded
        from the final model.

    monlev : int, optional
        A value of 1 for `monlev` enables monitoring of the model
        selection process; a value of 0 disables it.

    monfun : None or callable monfun(flag, var, val, data=None),
        optional
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        The function for monitoring the model selection process.

        Parameters
        ~~~~~~~~~~
        flag : str, length 1
            The value of `flag` indicates the stage of the stepwise
            selection of explanatory variables.

            `flag` = 'A'

              Variable `var` was added to the current model.

            `flag` = 'B'

              Beginning the backward elimination step.

            `flag` = 'C'

              Variable `var` failed the collinearity test and is
              excluded from the model.

            `flag` = 'D'

              Variable `var` was dropped from the current model.

            `flag` = 'F'

              Beginning the forward selection step

            `flag` = 'K'

              Backward elimination did not remove any variables from the
              current model.

            `flag` = 'S'

              Starting stepwise selection procedure.

            `flag` = 'V'

              The variance ratio for variable `var` takes the value
              `val`.

            `flag` = 'X'

              Finished stepwise selection procedure.

        var : int
            The index of the explanatory variable in the design matrix Z
            to which `flag` pertains.

        val : float
            If `flag` = 'V', `val` is the variance ratio value for the
            coefficient associated with explanatory variable index
            `var`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    isx : int, ndarray, shape (m)
        The value of `isx`[j-1] indicates the status of the jth
        explanatory variable in the model.

        `isx`[j-1] = -1

          Forced exclusion.

        `isx`[j-1] = 0

          Excluded.

        `isx`[j-1] = 1

          Selected.

        `isx`[j-1] = 2

          Forced selection.

    b : float, ndarray, shape (m+1)
        `b`[0] contains the estimate for the intercept term in the
        fitted model. If `isx`[j-1] != 0, then `b`[j] contains the
        estimate for the jth explanatory variable in the fitted model;
        otherwise `b`[j] = 0.

    se : float, ndarray, shape (m+1)
        `se`[j-1] contains the standard error for the estimate of
        `b`[j-1], for j = 1,2,...,m+1.

    rsq : float
        The R^2-statistic for the fitted regression model.

    rms : float
        The mean square of residuals for the fitted regression model.

    df : int
        The number of degrees of freedom for the sum of squares of
        residuals.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m > 1.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 1.

        (`errno` 1)
            On entry, `sw` = *<value>*.

            Constraint: `sw` > 1.0.

        (`errno` 1)
            On entry, `fin` = *<value>*.

            Constraint: `fin` > 0.0.

        (`errno` 1)
            On entry, `fout` = *<value>*; `fin` = *<value>*.

            Constraint: 0.0 <= `fout` <= `fin`.

        (`errno` 1)
            On entry, `tau` = *<value>*.

            Constraint: `tau` > 0.0.

        (`errno` 1)
            On entry, `monlev` = *<value>*.

            Constraint: `monlev` = 0 or 1.

        (`errno` 2)
            On entry at least one diagonal element of `c` <= 0.0.

            Constraint: `c` must be positive definite.

        (`errno` 2)
            On entry, `isx`[*<value>*] = *<value>*.

            Constraint: `isx`[j-1] = -1, 1 or 2, for j = 1,2,...,m.

        (`errno` 2)
            On entry, `isx`[i-1] != 1, for all i = 1,2,...,m.

            Constraint: there must be at least one free variable.

        (`errno` 4)
            All variables are collinear, no model to select.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            Matrix not positive definite, results may be inaccurate.

    See Also
    --------
    :meth:`naginterfaces.library.examples.correg.linregm_fit_stepwise_ex.main`
    """

    from ..base.correg import linregm_fit_stepwise as b_linregm_fit_stepwise

    def cb_monfun(flag, var, val, data=None):
        """Wrapping user's monfun. Called as base monfun."""
        if data is not None:
            monfun(
                flag,
                var,
                val,
                data,
            )
        else:
            monfun(
                flag,
                var,
                val,
            )

    fun_name = 'naginterfaces.library.correg.linregm_fit_stepwise'

    _wmean = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'wmean'},
        wmean,
        copy=False,
    )
    _m = _b_utils._EngineIntScalarType(
        _b_utils._EngineFloat64ArrayType.get_shape(_wmean, {'fun_name': fun_name, 'entity': 'wmean'}, exp_rank=1)[0]-1, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=False,
    )
    _isx = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'isx'},
        isx,
        copy=True,
    )
    _fin = _b_utils._EngineFloat64ScalarType(
        fin, {'fun_name': fun_name, 'entity': 'fin'},
    ).to_ctypes()
    _fin_py = _b_utils._EngineFloat64ScalarType.to_py(_fin)
    _fout = _fin_py if fout is None else fout
    _m_py = _b_utils._EngineIntScalarType.to_py(_m)
    _b = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py+1,),
    )
    _se = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py+1,),
    )
    _b_utils._check_callable(monfun, {'fun_name': fun_name, 'entity': 'monfun'}, allow_none=True)
    _monfun = None if monfun is None else cb_monfun
    try:
        _return_scalars = b_linregm_fit_stepwise(
            _m,
            n,
            _wmean,
            _c,
            sw,
            _isx,
            fin,
            _fout,
            tau,
            _b,
            _se,
            monlev,
            _monfun,
            data,
            io_manager,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _isx_py = _isx
    _b_py = _b
    _se_py = _se
    _rsq_py = _return_scalars.rsq
    _rms_py = _return_scalars.rms
    _df_py = _return_scalars.df
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['isx', 'b', 'se', 'rsq', 'rms', 'df'],
    )(
        _isx_py,
        _b_py,
        _se_py,
        _rsq_py,
        _rms_py,
        _df_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def linregm_stat_resinf(n, ip, res, h, rms):
    u"""
    Calculates standardized residuals and influence statistics.

    For full information please refer to the NAG Library document for
    g02fa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02faf.html

    Parameters
    ----------
    n : int
        n, the number of observations included in the regression.

    ip : int
        p, the number of linear parameters estimated in the regression
        model.

    res : float, array-like, shape (nres)
        The residuals, r_i.

    h : float, array-like, shape (nres)
        The diagonal elements of H, h_i, corresponding to the residuals
        in `res`.

    rms : float
        The estimate of sigma^2 based on all n observations, s^2, i.e.,
        the residual mean square.

    Returns
    -------
    sres : float, ndarray, shape (nres, 4)
        The standardized residuals and influence statistics.

        For the observation with residual, r_i, given in `res`[i-1].

        `sres`[i-1,0]

          Is the internally standardized residual, RI_i.

        `sres`[i-1,1]

          Is the externally standardized residual, RE_i.

        `sres`[i-1,2]

          Is Cook's D statistic, D_i.

        `sres`[i-1,3]

          Is Atkinson's T statistic, T_i.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, nres = *<value>* and `n` = *<value>*.

            Constraint: nres <= `n`.

        (`errno` 1)
            On entry, `rms` = *<value>*.

            Constraint: `rms` > 0.0.

        (`errno` 1)
            On entry, `ip` = *<value>* and `n` = *<value>*.

            Constraint: `n`-1 > `ip`.

        (`errno` 1)
            On entry, `ip` = *<value>*.

            Constraint: `ip` >= 1.

        (`errno` 1)
            On entry, nres = *<value>*.

            Constraint: nres >= 1.

        (`errno` 2)
            On entry, `h`[*<value>*] = *<value>*.

            Constraint: 0.0 < `h`[i] < 1.0, for all i.

        (`errno` 3)
            On entry, a value in `res` is too large for given `rms`.
            `res`[*<value>*] = *<value>* and `rms` = *<value>*.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.correg import linregm_stat_resinf as b_linregm_stat_resinf

    fun_name = 'naginterfaces.library.correg.linregm_stat_resinf'

    _res = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'res'},
        res,
        copy=False,
    )
    _nres_py = _b_utils._EngineFloat64ArrayType.get_shape(_res, {'fun_name': fun_name, 'entity': 'res'}, exp_rank=1)[0]
    _nres = _b_utils._EngineIntScalarType(
        _nres_py, {'fun_name': fun_name, 'entity': '_nres'},
    ).to_ctypes()
    _h = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'h'},
        h,
        copy=False,
    )
    _sres = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nres_py, 4),
        sorder=_b_utils._EngineData.default_sorder,
    )
    b_linregm_stat_resinf(
        n,
        ip,
        _nres,
        _res,
        _h,
        rms,
        _sres,
    )
    _sres_py = _sres
    return _sres_py

def linregm_stat_durbwat(ip, res):
    u"""
    Computes Durbin--Watson test statistic.

    For full information please refer to the NAG Library document for
    g02fc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02fcf.html

    Parameters
    ----------
    ip : int
        p, the number of independent variables in the regression model,
        including the mean.

    res : float, array-like, shape (n)
        The residuals, r_1,r_2,...,r_n.

    Returns
    -------
    d : float
        The Durbin--Watson statistic, d.

    pdl : float
        Lower bound for the significance of the Durbin--Watson
        statistic, p_l.

    pdu : float
        Upper bound for the significance of the Durbin--Watson
        statistic, p_u.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ip` = *<value>*.

            Constraint: `ip` >= 1.

        (`errno` 1)
            On entry, n = *<value>* and `ip` = *<value>*.

            Constraint: n > `ip`.

        (`errno` 2)
            On entry, mean of `res` = *<value>*.

            Constraint: the mean of the residuals <= sqrt(epsilon),
            where epsilon = machine precision.

        (`errno` 3)
            On entry, all residuals are identical.
    """

    from ..base.correg import linregm_stat_durbwat as b_linregm_stat_durbwat

    fun_name = 'naginterfaces.library.correg.linregm_stat_durbwat'

    _res = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'res'},
        res,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_res, {'fun_name': fun_name, 'entity': 'res'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _return_scalars = b_linregm_stat_durbwat(
        _n,
        ip,
        _res,
    )
    _d_py = _return_scalars.d
    _pdl_py = _return_scalars.pdl
    _pdu_py = _return_scalars.pdu
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['d', 'pdl', 'pdu'],
    )(
        _d_py,
        _pdl_py,
        _pdu_py,
    )
    return _return_data

def glm_normal(x, isx, y, link='I', mean='M', wt=None, s=0.0, a=0.0, v=None, tol=0.0, maxit=10, iprint=0, eps=0.0, io_manager=None):
    u"""
    Fits a generalized linear model with Normal errors.

    For full information please refer to the NAG Library document for
    g02ga

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02gaf.html

    Parameters
    ----------
    x : float, array-like, shape (n, m)
        `x`[i-1,j-1] must contain the ith observation for the jth
        independent variable, for j = 1,2,...,m, for i = 1,2,...,n.

    isx : int, array-like, shape (m)
        Indicates which independent variables are to be included in the
        model.

        If `isx`[j-1] > 0, the variable contained in the jth column of
        `x` is included in the regression model.

    y : float, array-like, shape (n)
        The observations on the dependent variable, y_i, for i =
        1,2,...,n.

    link : str, length 1, optional
        Indicates which link function is to be used.

        `link` = 'E'

          An exponent link is used.

        `link` = 'I'

          An identity link is used.

        `link` = 'L'

          A log link is used.

        `link` = 'S'

          A square root link is used.

        `link` = 'R'

          A reciprocal link is used.

    mean : str, length 1, optional
        Indicates if a mean term is to be included.

        `mean` = 'M'

          A mean term, intercept, will be included in the model.

        `mean` = 'Z'

          The model will pass through the origin, zero-point.

    wt : None or float, array-like, shape (n), optional
        If provided `wt` must contain the weights to be used with the
        model.

        If `wt`[i-1] = 0.0, the ith observation is not included in the
        model, in which case the effective number of observations is the
        number of observations with nonzero weights.

        If `wt` is not provided the effective number of observations is
        n.

    s : float, optional
        The scale parameter for the model, sigma^2.

        If `s` = 0.0, the scale parameter is estimated with the function
        using the residual mean square.

    a : float, optional
        If `link` = 'E', `a` must contain the power of the exponential.

        If `link` != 'E', `a` is not referenced.

    v : None or float, array-like, shape (n, ip+7), optional
        If `v` is not None, `v`[i-1,6], for i = 1,2,...,n, must contain
        the offset values o_i.

        All other values need not be set.

    tol : float, optional
        Indicates the accuracy required for the fit of the model.

        The iterative weighted least squares procedure is deemed to have
        converged if the absolute change in deviance between
        interactions is less than `tol`*(1.0+current residual sum of
        squares).

        This is approximately an absolute precision if the residual sum
        of squares is small and a relative precision if the residual sum
        of squares is large.

        If 0.0 <= `tol` < machine precision, ``glm_normal`` will use
        10*machine precision.

    maxit : int, optional
        The maximum number of iterations for the iterative weighted
        least squares.

        If `maxit` = 0, a default value of 10 is used.

    iprint : int, optional
        Indicates if the printing of information on the iterations is
        required.

        `iprint` <= 0

          There is no printing.

        `iprint` > 0

          Every `iprint` iteration, the following is printed:

          - the deviance,

          - the current estimates,

          - and if the weighted least squares equations are singular,
            then this is indicated.

        When printing occurs the output is directed to the file object
        associated with the advisory I/O unit (see
        :class:`~naginterfaces.base.utils.FileObjManager`).

    eps : float, optional
        The value of `eps` is used to decide if the independent
        variables are of full rank and, if not, what is the rank of the
        independent variables. The smaller the value of `eps` the
        stricter the criterion for selecting the singular value
        decomposition.

        If 0.0 <= `eps` < machine precision, the function will use
        machine precision instead.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    s : float
        If on input `s` = 0.0, `s` contains the estimated value of the
        scale parameter, sigma^^2.

        If on input `s` != 0.0, `s` is unchanged on exit.

    rss : float
        The residual sum of squares for the fitted model.

    idf : int
        The degrees of freedom associated with the residual sum of
        squares for the fitted model.

    b : float, ndarray, shape (ip)
        The estimates of the parameters of the generalized linear model,
        beta^.

        If `mean` = 'M', `b`[0] will contain the estimate of the mean
        parameter and `b`[i] will contain the coefficient of the
        variable contained in column j of `x`, where `isx`[j-1] is the
        ith positive value in the array `isx`.

        If `mean` = 'Z', `b`[i-1] will contain the coefficient of the
        variable contained in column j of `x`, where `isx`[j-1] is the
        ith positive value in the array `isx`.

    irank : int
        The rank of the independent variables.

        If the model is of full rank, `irank` = ip.

        If the model is not of full rank, `irank` is an estimate of the
        rank of the independent variables. `irank` is calculated as the
        number of singular values greater than `eps`* (largest singular
        value).

        It is possible for the SVD to be carried out but for `irank` to
        be returned as ip.

    se : float, ndarray, shape (ip)
        The standard errors of the linear parameters.

        `se`[i-1] contains the standard error of the parameter estimate
        in `b`[i-1], for i = 1,2,...,ip.

    cov : float, ndarray, shape (ip*(ip+1)/2)
        The upper triangular part of the variance-covariance matrix of
        the ip parameter estimates given in `b`. They are stored packed
        by column, i.e., the covariance between the parameter estimate
        given in `b`[i-1] and the parameter estimate given in `b`[j-1],
        j >= i, is stored in `cov`[(j*(j-1)/2+i)-1].

    v : float, ndarray, shape (n, ip+7)
        Auxiliary information on the fitted model.

        [table omitted]

        If the model is not of full rank, i.e., `irank` < ip, the first
        ip rows of columns 8 to ip+7 contain the P* matrix.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, ip = *<value>*.

            Constraint: ip >= 1.

        (`errno` 1)
            On entry, `eps` = *<value>*.

            Constraint: `eps` >= 0.0.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` >= 0.0.

        (`errno` 1)
            On entry, `maxit` = *<value>*.

            Constraint: `maxit` >= 0.

        (`errno` 1)
            On entry, offset = *<value>*.

            Constraint: offset = 'Y' or 'N'.

        (`errno` 1)
            On entry, `link` = *<value>*.

            Constraint: `link` = 'E', 'I', 'L', 'S' or 'R'.

        (`errno` 1)
            On entry, `a` = 0.0 and `link` = 'E'.

            Constraint: if `link` = 'E', `a` != 0.0.

        (`errno` 1)
            On entry, `s` = *<value>*.

            Constraint: `s` >= 0.0.

        (`errno` 1)
            On entry, weight = *<value>*.

            Constraint: weight = 'W' or 'U'.

        (`errno` 1)
            On entry, `mean` = *<value>*.

            Constraint: `mean` = 'M' or 'Z'.

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, `wt`[*<value>*] < 0.0.

            Constraint: `wt`[i-1] >= 0.0, for i = 1,2,...,n.

        (`errno` 3)
            On entry, `isx`[*<value>*] < 0.

            Constraint: `isx`[j-1] >= 0.0, for j = 1,2,...,m.

        (`errno` 3)
            On entry, ip incompatible with number of nonzero values of
            `isx`: ip = *<value>*.

        (`errno` 3)
            Number of requested x-variables greater than n.

        (`errno` 4)
            A fitted value is at a boundary. This will only occur with
            `link` = 'L', 'R' or 'E'. This may occur if there are small
            values of y and the model is not suitable for the data. The
            model should be reformulated with, perhaps, some
            observations dropped.

        (`errno` 5)
            SVD solution failed to converge.

        (`errno` 6)
            The iterative weighted least squares has failed to converge
            in `maxit` (or default 10) iterations. The value of `maxit`
            could be increased but it may be advantageous to examine the
            convergence using the `iprint` option. This may indicate
            that the convergence is slow because the solution is at a
            boundary in which case it may be better to reformulate the
            model.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 7)
            The rank of the model has changed during the weighted least
            squares iterations. The estimate for beta returned may be
            reasonable, but you should check how the deviance has
            changed during iterations.

        (`errno` 8)
            The degrees of freedom for error are 0. A saturated model
            has been fitted.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.

    See Also
    --------
    :meth:`naginterfaces.library.examples.correg.glm_normal_ex.main`
    """

    from ..base.correg import glm_normal as b_glm_normal

    fun_name = 'naginterfaces.library.correg.glm_normal'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _v = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
        copy=True,
        allow_none=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'), (_v, 'v')),
    )
    if v is None:
        _offset = 'N'
        _offset_py = _offset
    else:
        _offset = 'Y'
        _offset_py = _offset
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _isx = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'isx'},
        isx,
        copy=False,
    )
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _wt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'wt'},
        wt,
        copy=False,
        allow_none=True,
    )
    _vl_p_py = 0
    for _vl_i_py in range(1, _m_py+1):
        if _b_utils._EngineIntArrayType.get_element(_isx, {'fun_name': fun_name, 'entity': 'isx'}, (_vl_i_py-1,)) > 0:
            _vl_p_py = _vl_p_py + 1
    _mean = _b_utils._EngineCharScalarType(
        mean, {'fun_name': fun_name, 'entity': 'mean'}, exp_chrlen=1,
    ).to_ctypes()
    _mean_py = _b_utils._EngineCharScalarType.to_py(_mean)
    if _mean_py in ('M', 'm'):
        _vl_ip_py = _vl_p_py+1
    else:
        _vl_ip_py = _vl_p_py
    _b = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_ip_py,),
    )
    _se = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_ip_py,),
    )
    _cov = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_ip_py*(_vl_ip_py+1)//2,),
    )
    if v is None:
        _v = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            (_n_py, _vl_ip_py+7),
            sorder=_m_sorder,
        )
    try:
        _return_scalars = b_glm_normal(
            link,
            mean,
            _offset,
            _n,
            _x,
            _m,
            _isx,
            _y,
            _wt,
            s,
            a,
            _b,
            _se,
            _cov,
            _v,
            tol,
            maxit,
            iprint,
            eps,
            io_manager,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _s_py = _return_scalars.s
    _v_py = _v
    _rss_py = _return_scalars.rss
    _idf_py = _return_scalars.idf
    _b_py = _b
    _irank_py = _return_scalars.irank
    _se_py = _se
    _cov_py = _cov
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['s', 'rss', 'idf', 'b', 'irank', 'se', 'cov', 'v'],
    )(
        _s_py,
        _rss_py,
        _idf_py,
        _b_py,
        _irank_py,
        _se_py,
        _cov_py,
        _v_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def glm_binomial(x, isx, y, t, link='G', mean='M', wt=None, v=None, tol=0.0, maxit=10, iprint=0, eps=0.0, io_manager=None):
    u"""
    Fits a generalized linear model with binomial errors.

    For full information please refer to the NAG Library document for
    g02gb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02gbf.html

    Parameters
    ----------
    x : float, array-like, shape (n, m)
        `x`[i-1,j-1] must contain the ith observation for the jth
        independent variable, for j = 1,2,...,m, for i = 1,2,...,n.

    isx : int, array-like, shape (m)
        Indicates which independent variables are to be included in the
        model.

        If `isx`[j-1] > 0, the variable contained in the jth column of
        `x` is included in the regression model.

    y : float, array-like, shape (n)
        The observations on the dependent variable, y_i, for i =
        1,2,...,n.

    t : float, array-like, shape (n)
        t, the binomial denominator.

    link : str, length 1, optional
        Indicates which link function is to be used.

        `link` = 'G'

          A logistic link is used.

        `link` = 'P'

          A probit link is used.

        `link` = 'C'

          A complementary log-log link is used.

    mean : str, length 1, optional
        Indicates if a mean term is to be included.

        `mean` = 'M'

          A mean term, intercept, will be included in the model.

        `mean` = 'Z'

          The model will pass through the origin, zero-point.

    wt : None or float, array-like, shape (n), optional
        If provided `wt` must contain the weights to be used with the
        model.

        If `wt`[i-1] = 0.0, the ith observation is not included in the
        model, in which case the effective number of observations is the
        number of observations with nonzero weights.

        If `wt` is not provided the effective number of observations is
        n.

    v : None or float, array-like, shape (n, ip+7), optional
        If `v` is not None, `v`[i-1,6], for i = 1,2,...,n must contain
        the offset values o_i.

        All other values need not be set.

    tol : float, optional
        Indicates the accuracy required for the fit of the model.

        The iterative weighted least squares procedure is deemed to have
        converged if the absolute change in deviance between iterations
        is less than `tol`*(1.0+Current Deviance).

        This is approximately an absolute precision if the deviance is
        small and a relative precision if the deviance is large.

        If 0.0 <= `tol` < machine precision, the function will use
        10*machine precision instead.

    maxit : int, optional
        The maximum number of iterations for the iterative weighted
        least squares.

        If `maxit` = 0, a default value of 10 is used.

    iprint : int, optional
        Indicates if the printing of information on the iterations is
        required.

        `iprint` <= 0

          There is no printing.

        `iprint` > 0

          The following is printed every `iprint` iterations:

          - the deviance,

          - the current estimates,

          - and if the weighted least squares equations are singular,
            then this is indicated.

        When printing occurs the output is directed to the file object
        associated with the advisory I/O unit (see
        :class:`~naginterfaces.base.utils.FileObjManager`).

    eps : float, optional
        The value of `eps` is used to decide if the independent
        variables are of full rank and, if not, what is the rank of the
        independent variables. The smaller the value of `eps` the
        stricter the criterion for selecting the singular value
        decomposition.

        If 0.0 <= `eps` < machine precision, the function will use
        machine precision instead.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    dev : float
        The deviance for the fitted model.

    idf : int
        The degrees of freedom associated with the deviance for the
        fitted model.

    b : float, ndarray, shape (ip)
        The estimates of the parameters of the generalized linear model,
        beta^.

        If `mean` = 'M', the first element of `b` will contain the
        estimate of the mean parameter and `b`[i] will contain the
        coefficient of the variable contained in column j of `x`, where
        `isx`[j-1] is the ith positive value in the array `isx`.

        If `mean` = 'Z', `b`[i-1] will contain the coefficient of the
        variable contained in column j of `x`, where `isx`[j-1] is the
        ith positive value in the array `isx`.

    irank : int
        The rank of the independent variables.

        If the model is of full rank, `irank` = ip.

        If the model is not of full rank, `irank` is an estimate of the
        rank of the independent variables. `irank` is calculated as the
        number of singular values greater that `eps`*(largest singular
        value).

        It is possible for the SVD to be carried out but for `irank` to
        be returned as ip.

    se : float, ndarray, shape (ip)
        The standard errors of the linear parameters.

        `se`[i-1] contains the standard error of the parameter estimate
        in `b`[i-1], for i = 1,2,...,ip.

    cov : float, ndarray, shape (ip*(ip+1)/2)
        The upper triangular part of the variance-covariance matrix of
        the ip parameter estimates given in `b`. They are stored in
        packed form by column, i.e., the covariance between the
        parameter estimate given in `b`[i-1] and the parameter estimate
        given in `b`[j-1], j >= i, is stored in `cov`[(j*(j-1)/2+i)-1].

    v : float, ndarray, shape (n, ip+7)
        Auxiliary information on the fitted model.

        [table omitted]

        If the model is not of full rank, i.e., `irank` < ip, the first
        ip rows of columns 8 to ip+7 contain the P* matrix.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, ip = *<value>*.

            Constraint: ip >= 1.

        (`errno` 1)
            On entry, `eps` = *<value>*.

            Constraint: `eps` >= 0.0.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` >= 0.0.

        (`errno` 1)
            On entry, `maxit` = *<value>*.

            Constraint: `maxit` >= 0.

        (`errno` 1)
            On entry, offset = *<value>*.

            Constraint: offset = 'Y' or 'N'.

        (`errno` 1)
            On entry, `link` = *<value>*.

            Constraint: `link` = 'G', 'P' or 'C'.

        (`errno` 1)
            On entry, weight = *<value>*.

            Constraint: weight = 'W' or 'U'.

        (`errno` 1)
            On entry, `mean` = *<value>*.

            Constraint: `mean` = 'M' or 'Z'.

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, `wt`[*<value>*] < 0.0.

            Constraint: `wt`[i-1] >= 0.0, for i = 1,2,...,n.

        (`errno` 3)
            On entry, `isx`[*<value>*] < 0.

            Constraint: `isx`[j-1] >= 0.0, for j = 1,2,...,m.

        (`errno` 3)
            On entry, ip incompatible with number of nonzero values of
            `isx`: ip = *<value>*.

        (`errno` 3)
            Number of requested x-variables greater than n.

        (`errno` 4)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i-1] >= 0, for i = 1,2,...,n.

        (`errno` 5)
            On entry, `y`[*<value>*] = *<value>* and `t`[*<value>*] =
            *<value>*.

            Constraint: 0.0 <= `y`[i-1] <= `t`[i-1], for i = 1,2,...,n.

        (`errno` 6)
            A fitted value is at the boundary, i.e., 0.0 or 1.0. This
            may occur if there are y values of 0.0 or t and the model is
            too complex for the data. The model should be reformulated
            with, perhaps, some observations dropped.

        (`errno` 7)
            SVD solution failed to converge.

        (`errno` 8)
            The iterative weighted least squares has failed to converge
            in `maxit` (or default 10) iterations. The value of `maxit`
            could be increased but it may be advantageous to examine the
            convergence using the `iprint` option. This may indicate
            that the convergence is slow because the solution is at a
            boundary in which case it may be better to reformulate the
            model.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 9)
            The rank of the model has changed during the weighted least
            squares iterations. The estimate for beta returned may be
            reasonable, but you should check how the deviance has
            changed during iterations.

        (`errno` 10)
            Degrees of freedom for error are 0.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.

    See Also
    --------
    :meth:`naginterfaces.library.examples.correg.glm_binomial_ex.main`
    """

    from ..base.correg import glm_binomial as b_glm_binomial

    fun_name = 'naginterfaces.library.correg.glm_binomial'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _v = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
        copy=True,
        allow_none=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'), (_v, 'v')),
    )
    if v is None:
        _offset = 'N'
        _offset_py = _offset
    else:
        _offset = 'Y'
        _offset_py = _offset
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _isx = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'isx'},
        isx,
        copy=False,
    )
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
    )
    _wt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'wt'},
        wt,
        copy=False,
        allow_none=True,
    )
    _vl_p_py = 0
    for _vl_i_py in range(1, _m_py+1):
        if _b_utils._EngineIntArrayType.get_element(_isx, {'fun_name': fun_name, 'entity': 'isx'}, (_vl_i_py-1,)) > 0:
            _vl_p_py = _vl_p_py + 1
    _mean = _b_utils._EngineCharScalarType(
        mean, {'fun_name': fun_name, 'entity': 'mean'}, exp_chrlen=1,
    ).to_ctypes()
    _mean_py = _b_utils._EngineCharScalarType.to_py(_mean)
    if _mean_py in ('M', 'm'):
        _vl_ip_py = _vl_p_py+1
    else:
        _vl_ip_py = _vl_p_py
    _b = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_ip_py,),
    )
    _se = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_ip_py,),
    )
    _cov = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_ip_py*(_vl_ip_py+1)//2,),
    )
    if v is None:
        _v = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            (_n_py, _vl_ip_py+7),
            sorder=_m_sorder,
        )
    try:
        _return_scalars = b_glm_binomial(
            link,
            mean,
            _offset,
            _n,
            _x,
            _m,
            _isx,
            _y,
            _t,
            _wt,
            _b,
            _se,
            _cov,
            _v,
            tol,
            maxit,
            iprint,
            eps,
            io_manager,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _v_py = _v
    _dev_py = _return_scalars.dev
    _idf_py = _return_scalars.idf
    _b_py = _b
    _irank_py = _return_scalars.irank
    _se_py = _se
    _cov_py = _cov
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['dev', 'idf', 'b', 'irank', 'se', 'cov', 'v'],
    )(
        _dev_py,
        _idf_py,
        _b_py,
        _irank_py,
        _se_py,
        _cov_py,
        _v_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def glm_poisson(x, isx, y, link='L', mean='M', wt=None, a=0.0, v=None, tol=0.0, maxit=10, iprint=0, eps=0.0, io_manager=None):
    u"""
    Fits a generalized linear model with Poisson errors.

    For full information please refer to the NAG Library document for
    g02gc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02gcf.html

    Parameters
    ----------
    x : float, array-like, shape (n, m)
        The matrix of all possible independent variables. `x`[i-1,j-1]
        must contain the ijth element of `x`, for j = 1,2,...,m, for i =
        1,2,...,n.

    isx : int, array-like, shape (m)
        Indicates which independent variables are to be included in the
        model.

        If `isx`[j-1] > 0, the variable contained in the jth column of
        `x` is included in the regression model.

    y : float, array-like, shape (n)
        y, observations on the dependent variable.

    link : str, length 1, optional
        Indicates which link function is to be used.

        `link` = 'E'

          An exponent link is used.

        `link` = 'I'

          An identity link is used.

        `link` = 'L'

          A log link is used.

        `link` = 'S'

          A square root link is used.

        `link` = 'R'

          A reciprocal link is used.

    mean : str, length 1, optional
        Indicates if a mean term is to be included.

        `mean` = 'M'

          A mean term, intercept, will be included in the model.

        `mean` = 'Z'

          The model will pass through the origin, zero-point.

    wt : None or float, array-like, shape (n), optional
        If provided `wt` must contain the weights to be used with the
        model.

        If `wt`[i-1] = 0.0, the ith observation is not included in the
        model, in which case the effective number of observations is the
        number of observations with nonzero weights.

        If `wt` is not provided the effective number of observations is
        n.

    a : float, optional
        If `link` = 'E', `a` must contain the power of the exponential.

        If `link` != 'E', `a` is not referenced.

    v : None or float, array-like, shape (n, ip+7), optional
        If `v` is not None, `v`[i-1,6], for i = 1,2,...,n must contain
        the offset values o_i.

        All other values need not be set.

    tol : float, optional
        Indicates the accuracy required for the fit of the model.

        The iterative weighted least squares procedure is deemed to have
        converged if the absolute change in deviance between iterations
        is less than `tol`*(1.0+Current Deviance).

        This is approximately an absolute precision if the deviance is
        small and a relative precision if the deviance is large.

        If 0.0 <= `tol` < machine precision, the function will use
        10*machine precision instead.

    maxit : int, optional
        The maximum number of iterations for the iterative weighted
        least squares.

        If `maxit` = 0, a default value of 10 is used.

    iprint : int, optional
        Indicates if the printing of information on the iterations is
        required.

        `iprint` <= 0

          There is no printing.

        `iprint` > 0

          Every `iprint` iteration, the following are printed:

          - the deviance;

          - the current estimates;

          - and if the weighted least squares equations are singular,
            then this is indicated.

        When printing occurs the output is directed to the file object
        associated with the advisory I/O unit (see
        :class:`~naginterfaces.base.utils.FileObjManager`).

    eps : float, optional
        The value of `eps` is used to decide if the independent
        variables are of full rank and, if not, what is the rank of the
        independent variables. The smaller the value of `eps` the
        stricter the criterion for selecting the singular value
        decomposition.

        If 0.0 <= `eps` < machine precision, the function will use
        machine precision instead.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    dev : float
        The deviance for the fitted model.

    idf : int
        The degrees of freedom asociated with the deviance for the
        fitted model.

    b : float, ndarray, shape (ip)
        The estimates of the parameters of the generalized linear model,
        beta^.

        If `mean` = 'M', the first element of `b` will contain the
        estimate of the mean parameter and `b`[i] will contain the
        coefficient of the variable contained in column j of `x`, where
        `isx`[j-1] is the ith positive value in the array `isx`.

        If `mean` = 'Z', `b`[i-1] will contain the coefficient of the
        variable contained in column j of `x`, where `isx`[j-1] is the
        ith positive value in the array `isx`.

    irank : int
        The rank of the independent variables.

        If the model is of full rank, `irank` = ip.

        If the model is not of full rank, `irank` is an estimate of the
        rank of the independent variables. `irank` is calculated as the
        number of singular values greater that `eps`*(largest singular
        value).

        It is possible for the SVD to be carried out but for `irank` to
        be returned as ip.

    se : float, ndarray, shape (ip)
        The standard errors of the linear parameters.

        `se`[i-1] contains the standard error of the parameter estimate
        in `b`[i-1], for i = 1,2,...,ip.

    cov : float, ndarray, shape (ip*(ip+1)/2)
        The upper triangular part of the variance-covariance matrix of
        the ip parameter estimates given in `b`. They are stored packed
        by column, i.e., the covariance between the parameter estimate
        given in `b`[i-1] and the parameter estimate given in `b`[j-1],
        j >= i, is stored in `cov`[(j*(j-1)/2+i)-1].

    v : float, ndarray, shape (n, ip+7)
        Auxiliary information on the fitted model.

        [table omitted]

        If the model is not of full rank, i.e., `irank` < ip, the first
        ip rows of columns 8 to ip+7 contain the P* matrix.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, ip = *<value>*.

            Constraint: ip >= 1.

        (`errno` 1)
            On entry, `eps` = *<value>*.

            Constraint: `eps` >= 0.0.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` >= 0.0.

        (`errno` 1)
            On entry, `maxit` = *<value>*.

            Constraint: `maxit` >= 0.

        (`errno` 1)
            On entry, offset = *<value>*.

            Constraint: offset = 'Y' or 'N'.

        (`errno` 1)
            On entry, `link` = *<value>*.

            Constraint: `link` = 'E', 'I', 'L', 'S' or 'R'.

        (`errno` 1)
            On entry, `a` = 0.0 and `link` = 'E'.

            Constraint: if `link` = 'E', `a` != 0.0.

        (`errno` 1)
            On entry, weight = *<value>*.

            Constraint: weight = 'W' or 'U'.

        (`errno` 1)
            On entry, `mean` = *<value>*.

            Constraint: `mean` = 'M' or 'Z'.

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, `wt`[*<value>*] < 0.0.

            Constraint: `wt`[i-1] >= 0.0, for i = 1,2,...,n.

        (`errno` 3)
            On entry, `isx`[*<value>*] < 0.

            Constraint: `isx`[j-1] >= 0.0, for j = 1,2,...,m.

        (`errno` 3)
            On entry, ip incompatible with number of nonzero values of
            `isx`: ip = *<value>*.

        (`errno` 3)
            Number of requested x-variables greater than n.

        (`errno` 4)
            On entry, `y`[*<value>*] = *<value>*.

            Constraint: `y`[i-1] <= 0.0, for i = 1,2,...,n.

        (`errno` 5)
            A fitted value is at the boundary, i.e., mu^= 0.0. This may
            occur if there are y values of 0.0 and the model is too
            complex for the data. The model should be reformulated with,
            perhaps, some observations dropped.

        (`errno` 6)
            SVD solution failed to converge.

        (`errno` 7)
            The iterative weighted least squares has failed to converge
            in `maxit` (or default 10) iterations. The value of `maxit`
            could be increased but it may be advantageous to examine the
            convergence using the `iprint` option. This may indicate
            that the convergence is slow because the solution is at a
            boundary in which case it may be better to reformulate the
            model.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 8)
            The rank of the model has changed during the weighted least
            squares iterations. The estimate for beta returned may be
            reasonable, but you should check how the deviance has
            changed during iterations.

        (`errno` 9)
            The degrees of freedom for error are 0. A saturated model
            has been fitted.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.

    See Also
    --------
    :meth:`naginterfaces.library.examples.surviv.coxmodel_ex.main`
    """

    from ..base.correg import glm_poisson as b_glm_poisson

    fun_name = 'naginterfaces.library.correg.glm_poisson'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _v = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
        copy=True,
        allow_none=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'), (_v, 'v')),
    )
    if v is None:
        _offset = 'N'
        _offset_py = _offset
    else:
        _offset = 'Y'
        _offset_py = _offset
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _isx = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'isx'},
        isx,
        copy=False,
    )
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _wt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'wt'},
        wt,
        copy=False,
        allow_none=True,
    )
    _vl_p_py = 0
    for _vl_i_py in range(1, _m_py+1):
        if _b_utils._EngineIntArrayType.get_element(_isx, {'fun_name': fun_name, 'entity': 'isx'}, (_vl_i_py-1,)) > 0:
            _vl_p_py = _vl_p_py + 1
    _mean = _b_utils._EngineCharScalarType(
        mean, {'fun_name': fun_name, 'entity': 'mean'}, exp_chrlen=1,
    ).to_ctypes()
    _mean_py = _b_utils._EngineCharScalarType.to_py(_mean)
    if _mean_py in ('M', 'm'):
        _vl_ip_py = _vl_p_py+1
    else:
        _vl_ip_py = _vl_p_py
    _b = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_ip_py,),
    )
    _se = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_ip_py,),
    )
    _cov = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_ip_py*(_vl_ip_py+1)//2,),
    )
    if v is None:
        _v = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            (_n_py, _vl_ip_py+7),
            sorder=_m_sorder,
        )
    try:
        _return_scalars = b_glm_poisson(
            link,
            mean,
            _offset,
            _n,
            _x,
            _m,
            _isx,
            _y,
            _wt,
            a,
            _b,
            _se,
            _cov,
            _v,
            tol,
            maxit,
            iprint,
            eps,
            io_manager,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _v_py = _v
    _dev_py = _return_scalars.dev
    _idf_py = _return_scalars.idf
    _b_py = _b
    _irank_py = _return_scalars.irank
    _se_py = _se
    _cov_py = _cov
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['dev', 'idf', 'b', 'irank', 'se', 'cov', 'v'],
    )(
        _dev_py,
        _idf_py,
        _b_py,
        _irank_py,
        _se_py,
        _cov_py,
        _v_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def glm_gamma(x, isx, y, link='R', mean='M', wt=None, s=0, a=0.0, v=None, tol=0.0, maxit=10, iprint=0, eps=0.0, io_manager=None):
    u"""
    Fits a generalized linear model with gamma errors.

    For full information please refer to the NAG Library document for
    g02gd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02gdf.html

    Parameters
    ----------
    x : float, array-like, shape (n, m)
        `x`[i-1,j-1] must contain the ith observation for the jth
        independent variable, for j = 1,2,...,m, for i = 1,2,...,n.

    isx : int, array-like, shape (m)
        Indicates which independent variables are to be included in the
        model.

        If `isx`[j-1] > 0, the variable contained in the jth column of
        `x` is included in the regression model.

    y : float, array-like, shape (n)
        y, the dependent variable.

    link : str, length 1, optional
        Indicates which link function is to be used.

        `link` = 'E'

          An exponential link is used.

        `link` = 'I'

          An identity link is used.

        `link` = 'L'

          A log link is used.

        `link` = 'S'

          A square root link is used.

        `link` = 'R'

          A reciprocal link is used.

    mean : str, length 1, optional
        Indicates if a mean term is to be included.

        `mean` = 'M'

          A mean term, intercept, will be included in the model.

        `mean` = 'Z'

          The model will pass through the origin, zero-point.

    wt : None or float, array-like, shape (n), optional
        If provided `wt` must contain the weights to be used with the
        model.

        If `wt`[i-1] = 0.0, the ith observation is not included in the
        model, in which case the effective number of observations is the
        number of observations with nonzero weights.

        If `wt` is not provided the effective number of observations is
        n.

    s : float, optional
        The scale parameter for the gamma model, nu^-1.

        `s` = 0.0

          The scale parameter is estimated with the function using the
          formula described in [Description].

    a : float, optional
        If `link` = 'E', `a` must contain the power of the exponential.

        If `link` != 'E', `a` is not referenced.

    v : None or float, array-like, shape (n, ip+7), optional
        The offset values o_i, otherwise `v` need not be set and no
        offset is used.

    tol : float, optional
        Indicates the accuracy required for the fit of the model.

        The iterative weighted least squares procedure is deemed to have
        converged if the absolute change in deviance between iterations
        is less than `tol`*(1.0+Current Deviance).

        This is approximately an absolute precision if the deviance is
        small and a relative precision if the deviance is large.

        If 0.0 <= `tol` < machine precision then the function will use
        10*machine precision instead.

    maxit : int, optional
        The maximum number of iterations for the iterative weighted
        least squares.

        `maxit` = 0

          A default value of 10 is used.

    iprint : int, optional
        Indicates if the printing of information on the iterations is
        required.

        `iprint` <= 0

          There is no printing.

        `iprint` > 0

          Every `iprint` iteration, the following are printed:

          - the deviance;

          - the current estimates;

          - and if the weighted least squares equations are singular,
            then this is indicated.

        When printing occurs the output is directed to the file object
        associated with the advisory I/O unit (see
        :class:`~naginterfaces.base.utils.FileObjManager`).

    eps : float, optional
        The value of `eps` is used to decide if the independent
        variables are of full rank and, if not, what is the rank of the
        independent variables. The smaller the value of `eps` the
        stricter the criterion for selecting the singular value
        decomposition.

        If 0.0 <= `eps` < machine precision, the function will use
        machine precision instead.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    s : float
        If on input `s` = 0.0, `s` contains the estimated value of the
        scale parameter, nu^^(-1).

        If on input `s` != 0.0, `s` is unchanged on exit.

    dev : float
        The adjusted deviance for the fitted model.

    idf : int
        The degrees of freedom asociated with the deviance for the
        fitted model.

    b : float, ndarray, shape (ip)
        The estimates of the parameters of the generalized linear model,
        beta^.

        If `mean` = 'M', the first element of `b` will contain the
        estimate of the mean parameter and `b`[i] will contain the
        coefficient of the variable contained in column j of `x`, where
        `isx`[j-1] is the ith positive value in the array `isx`.

        If `mean` = 'Z', `b`[i-1] will contain the coefficient of the
        variable contained in column j of `x`, where `isx`[j-1] is the
        ith positive value in the array `isx`.

    irank : int
        The rank of the independent variables.

        If the model is of full rank, `irank` = ip.

        If the model is not of full rank, `irank` is an estimate of the
        rank of the independent variables. `irank` is calculated as the
        number of singular values greater that `eps`*(largest singular
        value).

        It is possible for the SVD to be carried out but for `irank` to
        be returned as ip.

    se : float, ndarray, shape (ip)
        The standard errors of the linear parameters.

        `se`[i-1] contains the standard error of the parameter estimate
        in `b`[i-1], for i = 1,2,...,ip.

    cov : float, ndarray, shape (ip*(ip+1)/2)
        The upper triangular part of the variance-covariance matrix of
        the ip parameter estimates given in `b`. They are stored in
        packed form by column, i.e., the covariance between the
        parameter estimate given in `b`[i-1] and the parameter estimate
        given in `b`[j-1], j >= i, is stored in `cov`[(j*(j-1)/2+i)-1].

    v : float, ndarray, shape (n, ip+7)
        Auxiliary information on the fitted model.

        [table omitted]

        If the model is not of full rank, i.e., `irank` < ip, the first
        ip rows of columns 8 to ip+7 contain the P* matrix.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, ip = *<value>*.

            Constraint: ip >= 1.

        (`errno` 1)
            On entry, `eps` = *<value>*.

            Constraint: `eps` >= 0.0.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` >= 0.0.

        (`errno` 1)
            On entry, `maxit` = *<value>*.

            Constraint: `maxit` >= 0.

        (`errno` 1)
            On entry, offset = *<value>*.

            Constraint: offset = 'Y' or 'N'.

        (`errno` 1)
            On entry, `link` = *<value>*.

            Constraint: `link` = 'E', 'I', 'L', 'S' or 'R'.

        (`errno` 1)
            On entry, `a` = 0.0 and `link` = 'E'.

            Constraint: if `link` = 'E', `a` != 0.0.

        (`errno` 1)
            On entry, `s` = *<value>*.

            Constraint: `s` >= 0.0.

        (`errno` 1)
            On entry, weight = *<value>*.

            Constraint: weight = 'W' or 'U'.

        (`errno` 1)
            On entry, `mean` = *<value>*.

            Constraint: `mean` = 'M' or 'Z'.

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, `wt`[*<value>*] < 0.0.

            Constraint: `wt`[i-1] >= 0.0, for i = 1,2,...,n.

        (`errno` 3)
            On entry, `isx`[*<value>*] < 0.

            Constraint: `isx`[j-1] >= 0.0, for j = 1,2,...,m.

        (`errno` 3)
            On entry, ip incompatible with number of nonzero values of
            `isx`: ip = *<value>*.

        (`errno` 3)
            Number of requested x-variables greater than n.

        (`errno` 4)
            On entry, `y`[*<value>*] = *<value>*.

            Constraint: `y`[i-1] <= 0.0, for i = 1,2,...,n.

        (`errno` 5)
            A fitted value is at the boundary, i.e., mu^= 0.0. This may
            occur if there are y values of 0.0 and the model is too
            complex for the data. The model should be reformulated with,
            perhaps, some observations dropped.

        (`errno` 6)
            SVD solution failed to converge.

        (`errno` 7)
            The iterative weighted least squares has failed to converge
            in `maxit` (or default 10) iterations. The value of `maxit`
            could be increased but it may be advantageous to examine the
            convergence using the `iprint` option. This may indicate
            that the convergence is slow because the solution is at a
            boundary in which case it may be better to reformulate the
            model.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 8)
            The rank of the model has changed during the weighted least
            squares iterations. The estimate for beta returned may be
            reasonable, but you should check how the deviance has
            changed during iterations.

        (`errno` 9)
            The degrees of freedom for error are 0. A saturated model
            has been fitted.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.correg import glm_gamma as b_glm_gamma

    fun_name = 'naginterfaces.library.correg.glm_gamma'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _v = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
        copy=True,
        allow_none=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'), (_v, 'v')),
    )
    if v is None:
        _offset = 'N'
        _offset_py = _offset
    else:
        _offset = 'Y'
        _offset_py = _offset
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _isx = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'isx'},
        isx,
        copy=False,
    )
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _wt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'wt'},
        wt,
        copy=False,
        allow_none=True,
    )
    _vl_p_py = 0
    for _vl_i_py in range(1, _m_py+1):
        if _b_utils._EngineIntArrayType.get_element(_isx, {'fun_name': fun_name, 'entity': 'isx'}, (_vl_i_py-1,)) > 0:
            _vl_p_py = _vl_p_py + 1
    _mean = _b_utils._EngineCharScalarType(
        mean, {'fun_name': fun_name, 'entity': 'mean'}, exp_chrlen=1,
    ).to_ctypes()
    _mean_py = _b_utils._EngineCharScalarType.to_py(_mean)
    if _mean_py in ('M', 'm'):
        _vl_ip_py = _vl_p_py+1
    else:
        _vl_ip_py = _vl_p_py
    _b = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_ip_py,),
    )
    _se = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_ip_py,),
    )
    _cov = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_ip_py*(_vl_ip_py+1)//2,),
    )
    if v is None:
        _v = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            (_n_py, _vl_ip_py+7),
            sorder=_m_sorder,
        )
    try:
        _return_scalars = b_glm_gamma(
            link,
            mean,
            _offset,
            _n,
            _x,
            _m,
            _isx,
            _y,
            _wt,
            s,
            a,
            _b,
            _se,
            _cov,
            _v,
            tol,
            maxit,
            iprint,
            eps,
            io_manager,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _s_py = _return_scalars.s
    _v_py = _v
    _dev_py = _return_scalars.dev
    _idf_py = _return_scalars.idf
    _b_py = _b
    _irank_py = _return_scalars.irank
    _se_py = _se
    _cov_py = _cov
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['s', 'dev', 'idf', 'b', 'irank', 'se', 'cov', 'v'],
    )(
        _s_py,
        _dev_py,
        _idf_py,
        _b_py,
        _irank_py,
        _se_py,
        _cov_py,
        _v_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def glm_constrain(v, c, b, s):
    u"""
    Estimates and standard errors of parameters of a general linear
    model for given constraints.

    For full information please refer to the NAG Library document for
    g02gk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02gkf.html

    Parameters
    ----------
    v : float, array-like, shape (ip, ip+7)
        The array `v` as returned by ``glm_normal``, ``glm_binomial``,
        ``glm_poisson`` or ``glm_gamma``.

    c : float, array-like, shape (ip, iconst)
        Contains the iconst constraints stored by column, i.e., the ith
        constraint is stored in the ith column of `c`.

    b : float, array-like, shape (ip)
        The parameter estimates computed by using the singular value
        decomposition, beta^_svd.

    s : float
        The estimate of the scale parameter.

        For results from ``glm_normal`` and ``glm_gamma`` then `s` is
        the scale parameter for the model.

        For results from ``glm_binomial`` and ``glm_poisson`` then `s`
        should be set to 1.0.

    Returns
    -------
    b : float, ndarray, shape (ip)
        The parameter estimates of the parameters with the constraints
        imposed, beta^_c.

    se : float, ndarray, shape (ip)
        The standard error of the parameter estimates in `b`.

    cov : float, ndarray, shape (ip*(ip+1)/2)
        The upper triangular part of the variance-covariance matrix of
        the ip parameter estimates given in `b`. They are stored packed
        by column, i.e., the covariance between the parameter estimate
        given in `b`[i-1] and the parameter estimate given in `b`[j-1],
        j >= i, is stored in `cov`[(j*(j-1)/2+i)-1].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, iconst = *<value>*.

            Constraint: iconst > 0.

        (`errno` 1)
            On entry, `s` = *<value>*.

            Constraint: `s` > 0.0.

        (`errno` 1)
            On entry, iconst = *<value>* and ip = *<value>*.

            Constraint: iconst < ip.

        (`errno` 1)
            On entry, ip = *<value>*.

            Constraint: ip >= 1.

        (`errno` 2)
            `c` does not give a model of full rank.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.correg import glm_constrain as b_glm_constrain

    fun_name = 'naginterfaces.library.correg.glm_constrain'

    _v = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
        copy=False,
    )
    _ip_py = _b_utils._EngineFloat64ArrayType.get_shape(_v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2)[0]
    _ip = _b_utils._EngineIntScalarType(
        _ip_py, {'fun_name': fun_name, 'entity': '_ip'},
    ).to_ctypes()
    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=False,
    )
    _iconst_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[1]
    _iconst = _b_utils._EngineIntScalarType(
        _iconst_py, {'fun_name': fun_name, 'entity': '_iconst'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _se = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_ip_py,),
    )
    _cov = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_ip_py*(_ip_py+1)//2,),
    )
    b_glm_constrain(
        _ip,
        _iconst,
        _v,
        _c,
        _b,
        s,
        _se,
        _cov,
    )
    _b_py = _b
    _se_py = _se
    _cov_py = _cov
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['b', 'se', 'cov'],
    )(
        _b_py,
        _se_py,
        _cov_py,
    )
    return _return_data

def glm_estfunc(irank, b, cov, v, f, tol=0.0):
    u"""
    Computes estimable function of a generalized linear model and its
    standard error.

    For full information please refer to the NAG Library document for
    g02gn

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02gnf.html

    Parameters
    ----------
    irank : int
        k, the rank of the dependent variables.

    b : float, array-like, shape (ip)
        The ip values of the estimates of the parameters of the model,
        beta^.

    cov : float, array-like, shape (ip*(ip+1)/2)
        The upper triangular part of the variance-covariance matrix of
        the ip parameter estimates given in `b`. They are stored packed
        by column, i.e., the covariance between the parameter estimate
        given in `b`[i-1] and the parameter estimate given in `b`[j-1],
        j >= i, is stored in `cov`[(j*(j-1)/2+i)-1].

    v : float, array-like, shape (ip, ip+7)
        As returned by ``glm_normal``, ``glm_binomial``, ``glm_poisson``
        and ``glm_gamma``.

    f : float, array-like, shape (ip)
        f, the linear function to be estimated.

    tol : float, optional
        The tolerance value used in the check for estimability, eta.

        If `tol` <= 0.0 then sqrt(epsilon), where epsilon is the machine
        precision, is used instead.

    Returns
    -------
    est : bool
        Indicates if the function was estimable.

        `est` = True

          The function is estimable.

        `est` = False

          The function is not estimable and `stat`, `sestat` and `z` are
          not set.

    stat : float
        If `est` = True, `stat` contains the estimate of the function,
        f^T beta^

    sestat : float
        If `est` = True, `sestat` contains the standard error of the
        estimate of the function, se(F).

    z : float
        If `est` = True, `z` contains the z statistic for the test of
        the function being equal to zero.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `irank` = *<value>* and ip = *<value>*.

            Constraint: `irank` <= ip.

        (`errno` 1)
            On entry, `irank` = *<value>*.

            Constraint: `irank` >= 1.

        (`errno` 1)
            On entry, ip = *<value>*.

            Constraint: ip >= 1.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            `irank` = ip. In this case `est` is returned as true and all
            statistics are calculated.

        (`errno` 3)
            Standard error of statistic = 0.0; this may be due to
            rounding errors if the standard error is very small or due
            to mis-specified inputs `cov` and `f`.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.correg import glm_estfunc as b_glm_estfunc

    fun_name = 'naginterfaces.library.correg.glm_estfunc'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
    )
    _ip_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=1)[0]
    _ip = _b_utils._EngineIntScalarType(
        _ip_py, {'fun_name': fun_name, 'entity': '_ip'},
    ).to_ctypes()
    _cov = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'cov'},
        cov,
        copy=False,
    )
    _v = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
        copy=False,
    )
    _f = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'f'},
        f,
        copy=False,
    )
    try:
        _return_scalars = b_glm_estfunc(
            _ip,
            irank,
            _b,
            _cov,
            _v,
            _f,
            tol,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _est_py = _return_scalars.est
    _stat_py = _return_scalars.stat
    _sestat_py = _return_scalars.sestat
    _z_py = _return_scalars.z
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['est', 'stat', 'sestat', 'z'],
    )(
        _est_py,
        _stat_py,
        _sestat_py,
        _z_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def glm_predict(errfn, x, isx, b, cov, vfobs, link=None, mean='M', t=None, off=None, wt=None, s=0.0, a=0.0):
    u"""
    Computes a predicted value and its associated standard error based
    on a previously fitted generalized linear model.

    For full information please refer to the NAG Library document for
    g02gp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02gpf.html

    Parameters
    ----------
    errfn : str, length 1
        Indicates the distribution used to model the dependent variable,
        y.

        `errfn` = 'B'

          The binomial distribution is used.

        `errfn` = 'G'

          The gamma distribution is used.

        `errfn` = 'N'

          The Normal (Gaussian) distribution is used.

        `errfn` = 'P'

          The Poisson distribution is used.

    x : float, array-like, shape (n, m)
        `x`[i-1,j-1] must contain the ith observation for the jth
        independent variable, for j = 1,2,...,m, for i = 1,2,...,n.

    isx : int, array-like, shape (m)
        Indicates which independent variables are to be included in the
        model.

        If `isx`[j-1] > 0, the variable contained in the jth column of
        `x` is included in the regression model.

    b : float, array-like, shape (ip)
        The model parameters, beta.

        If `mean` = 'M', `b`[0] must contain the mean parameter and
        `b`[i] the coefficient of the variable contained in the jth
        independent `x`, where `isx`[j-1] is the ith positive value in
        the array `isx`.

        If `mean` = 'Z', `b`[i-1] must contain the coefficient of the
        variable contained in the jth independent `x`, where `isx`[j-1]
        is the ith positive value in the array `isx`.

    cov : float, array-like, shape (ip*(ip+1)/2)
        The upper triangular part of the variance-covariance matrix, C,
        of the model parameters. This matrix should be supplied packed
        by column, i.e., the covariance between parameters beta_i and
        beta_j, that is the values stored in `b`[i-1] and `b`[j-1],
        should be supplied in `cov`[j*(j-1)/2+i-1], for j = i,...,ip,
        for i = 1,2,...,ip.

    vfobs : bool
        If `vfobs` = True, the variance of future observations is
        included in the standard error of the predicted variable (i.e.,
        I_fobs = 1), otherwise I_fobs = 0.

    link : str, length 1, optional
        Note: if this argument is None then a default value will be
        used, determined as follows: if `errfn` = 'B': 'G'; if `errfn` =
        'G': 'R'; if `errfn` = 'P': 'L'; otherwise: 'I'.

        Indicates which link function is to be used.

        `link` = 'C'

          A complementary log-log link is used.

        `link` = 'E'

          An exponent link is used.

        `link` = 'G'

          A logistic link is used.

        `link` = 'I'

          An identity link is used.

        `link` = 'L'

          A log link is used.

        `link` = 'P'

          A probit link is used.

        `link` = 'R'

          A reciprocal link is used.

        `link` = 'S'

          A square root link is used.

        Details on the functional form of the different links can be
        found in [the G02 Introduction].

    mean : str, length 1, optional
        Indicates if a mean term is to be included.

        `mean` = 'M'

          A mean term, intercept, will be included in the model.

        `mean` = 'Z'

          The model will pass through the origin, zero-point.

    t : None or float, array-like, shape (:), optional
        Note: the required length for this argument is determined as
        follows: if `errfn`='B': n; otherwise: 0.

        If `errfn` = 'B', `t`[i-1] must contain the binomial
        denominator, t_i, for the ith observation.

        Otherwise `t` is not referenced and may be None.

    off : None or float, array-like, shape (:), optional
        Note: the required length for this argument is determined as
        follows: if `off` is not None: n; otherwise: 0.

        If an offset is required, then `off`[i-1] must contain the value
        of the offset o_i, for the ith observation. Otherwise `off` must
        be None.

    wt : None or float, array-like, shape (:), optional
        Note: the required length for this argument is determined as
        follows: if `wt` is not None and `vfobs`= True: n; otherwise: 0.

        If weighted estimates are required then `wt`[i-1] must contain
        the weight, omega_i for the ith observation. Otherwise `wt` must
        be supplied as None.

        If `wt`[i-1] = 0.0, the ith observation is not included in the
        model, in which case the effective number of observations is the
        number of observations with positive weights.

        If `wt` = None, the effective number of observations is n.

        If the variance of future observations is not included in the
        standard error of the predicted variable, `wt` is not
        referenced.

    s : float, optional
        If `errfn` = 'N' or 'G' and `vfobs` = True, the scale parameter,
        phi.

        Otherwise `s` is not referenced and phi = 1.

    a : float, optional
        If `link` = 'E', `a` must contain the power of the exponential.

        If `link` != 'E', `a` is not referenced.

    Returns
    -------
    eta : float, ndarray, shape (n)
        The linear predictor, eta.

    seeta : float, ndarray, shape (n)
        The standard error of the linear predictor, se(eta).

    pred : float, ndarray, shape (n)
        The predicted value, y^.

    sepred : float, ndarray, shape (n)
        The standard error of the predicted value, se(y^). If
        `pred`[i-1] could not be calculated, ``glm_predict`` returns
        `errno` = 22, and `sepred`[i-1] is set to -99.0.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `errfn` = *<value>*.

            Constraint: `errfn` = 'B', 'G', 'N' or 'P'.

        (`errno` 2)
            On entry, `link` = *<value>*.

            Constraint: if `errfn` = 'B', `link` = 'C', 'G' or 'P',

            otherwise, `link` = 'E', 'I', 'L', 'R' or 'S'.

        (`errno` 2)
            On entry, `errfn` = *<value>* and `link` = *<value>*.

            Constraint: if `errfn` = 'B', `link` = 'C', 'G' or 'P',

            otherwise, `link` = 'E', 'I', 'L', 'R' or 'S'.

        (`errno` 3)
            On entry, `mean` = *<value>*.

            Constraint: `mean` = 'M' or 'Z'.

        (`errno` 5)
            On entry, weight = *<value>*.

            Constraint: if `vfobs` = True, weight = 'U' or 'W'.

        (`errno` 6)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 9)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 10)
            On entry, `isx`[*<value>*] < 0.

            Constraint: `isx`[j-1] >= 0.0, for j = 1,2,...,m.

        (`errno` 11)
            On entry, ip = *<value>*.

            Constraint: ip > 0.

        (`errno` 12)
            On entry, `t`[*<value>*] = *<value>*.

            Constraint: `t`[i-1] >= 0.0, for all i.

        (`errno` 14)
            On entry, `wt`[*<value>*] = *<value>*.

            Constraint: `wt`[i-1] >= 0.0, for all i.

        (`errno` 15)
            On entry, `s` = *<value>*.

            Constraint: `s` > 0.0.

        (`errno` 16)
            On entry, `a` = 0.0.

            Constraint: if `link` = 'E', `a` != 0.0.

        (`errno` 18)
            On entry, `cov`[*<value>*] = *<value>*.

            Constraint: `cov`[i] >= 0.0 for at least one diagonal
            element.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 22)
            At least one predicted value could not be calculated as
            required. `sepred` is set to -99.0 for affected predicted
            values.

    See Also
    --------
    :meth:`naginterfaces.library.examples.correg.glm_binomial_ex.main`
    """

    from ..base.correg import glm_predict as b_glm_predict

    fun_name = 'naginterfaces.library.correg.glm_predict'

    _errfn = _b_utils._EngineCharScalarType(
        errfn, {'fun_name': fun_name, 'entity': 'errfn'}, exp_chrlen=1,
    ).to_ctypes()
    _errfn_py = _b_utils._EngineCharScalarType.to_py(_errfn)
    if _errfn_py in ('B', 'b'):
        _link = 'G'
    elif _errfn_py in ('G', 'g'):
        _link = 'R'
    elif _errfn_py in ('P', 'p'):
        _link = 'L'
    else:
        _link = 'I'
    _link = _link if link is None else link
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _isx = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'isx'},
        isx,
        copy=False,
    )
    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
        allow_none=True,
    )
    _off = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'off'},
        off,
        copy=False,
        allow_none=True,
    )
    _wt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'wt'},
        wt,
        copy=False,
        allow_none=True,
    )
    _vl_p_py = 0
    for _vl_i_py in range(1, _m_py+1):
        if _b_utils._EngineIntArrayType.get_element(_isx, {'fun_name': fun_name, 'entity': 'isx'}, (_vl_i_py-1,)) > 0:
            _vl_p_py = _vl_p_py + 1
    _mean = _b_utils._EngineCharScalarType(
        mean, {'fun_name': fun_name, 'entity': 'mean'}, exp_chrlen=1,
    ).to_ctypes()
    _mean_py = _b_utils._EngineCharScalarType.to_py(_mean)
    if _mean_py in ('M', 'm'):
        _vl_ip_py = _vl_p_py+1
    else:
        _vl_ip_py = _vl_p_py
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
    )
    _cov = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'cov'},
        cov,
        copy=False,
    )
    _eta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _seeta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _pred = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _sepred = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        b_glm_predict(
            errfn,
            _link,
            mean,
            _n,
            _x,
            _m,
            _isx,
            _t,
            _off,
            _wt,
            s,
            a,
            _b,
            _cov,
            vfobs,
            _eta,
            _seeta,
            _pred,
            _sepred,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _eta_py = _eta
    _seeta_py = _seeta
    _pred_py = _pred
    _sepred_py = _sepred
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['eta', 'seeta', 'pred', 'sepred'],
    )(
        _eta_py,
        _seeta_py,
        _pred_py,
        _sepred_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def robustm(indw, ipsi, isigma, indc, x, y, cpsi, h1, h2, h3, cucv, dchi, theta, sigma, tol=5e-5, maxit=50, nitmon=0, io_manager=None):
    u"""
    Robust regression, standard M-estimates.

    For full information please refer to the NAG Library document for
    g02ha

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02haf.html

    Parameters
    ----------
    indw : int
        Specifies the type of regression to be performed.

        `indw` < 0

          Mallows type regression with Maronna's proposed weights.

        `indw` = 0

          Huber type regression.

        `indw` > 0

          Schweppe type regression with Krasker--Welsch weights.

    ipsi : int
        Specifies which psi function is to be used.

        `ipsi` = 0

          psi(t) = t, i.e., least squares.

        `ipsi` = 1

          Huber's function.

        `ipsi` = 2

          Hampel's piecewise linear function.

        `ipsi` = 3

          Andrew's sine wave.

        `ipsi` = 4

          Tukey's bi-weight.

    isigma : int
        Specifies how sigma is to be estimated.

        `isigma` < 0

          sigma is estimated by median absolute deviation of residuals.

        `isigma` = 0

          sigma is held constant at its initial value.

        `isigma` > 0

          sigma is estimated using the chi function.

    indc : int
        If `indw` != 0, `indc` specifies the approximations used in
        estimating the covariance matrix of theta^.

        `indc` = 1

          Averaging over residuals.

        `indc` != 1

          Replacing expected by observed.

        `indw` = 0

          `indc` is not referenced.

    x : float, array-like, shape (n, m)
        The values of the X matrix, i.e., the independent variables.
        `x`[i-1,j-1] must contain the ijth element of X, for j =
        1,2,...,m, for i = 1,2,...,n.

        If `indw` < 0, then during calculations the elements of `x` will
        be transformed as described in [Description].

        Before exit the inverse transformation will be applied.

        As a result there may be slight differences between the input
        `x` and the output `x`.

    y : float, array-like, shape (n)
        The data values of the dependent variable.

        `y`[i-1] must contain the value of y for the ith observation,
        for i = 1,2,...,n.

        If `indw` < 0, then during calculations the elements of `y` will
        be transformed as described in [Description].

        Before exit the inverse transformation will be applied.

        As a result there may be slight differences between the input
        `y` and the output `y`.

    cpsi : float
        If `ipsi` = 1, `cpsi` must specify the parameter, c, of Huber's
        psi function.

        If `ipsi` != 1 on entry, `cpsi` is not referenced.

    h1 : float
        If `ipsi` = 2, `h1`, `h2`, and `h3` must specify the parameters
        h_1, h_2, and h_3, of Hampel's piecewise linear psi function.
        `h1`, `h2`, and `h3` are not referenced if `ipsi` != 2.

    h2 : float
        If `ipsi` = 2, `h1`, `h2`, and `h3` must specify the parameters
        h_1, h_2, and h_3, of Hampel's piecewise linear psi function.
        `h1`, `h2`, and `h3` are not referenced if `ipsi` != 2.

    h3 : float
        If `ipsi` = 2, `h1`, `h2`, and `h3` must specify the parameters
        h_1, h_2, and h_3, of Hampel's piecewise linear psi function.
        `h1`, `h2`, and `h3` are not referenced if `ipsi` != 2.

    cucv : float
        If `indw` < 0, must specify the value of the constant, c, of the
        function u for Maronna's proposed weights.

        If `indw` > 0, must specify the value of the function u for the
        Krasker--Welsch weights.

        If `indw` = 0, is not referenced.

    dchi : float
        d, the constant of the chi function. `dchi` is not referenced if
        `ipsi` = 0, or if `isigma` <= 0.

    theta : float, array-like, shape (m)
        Starting values of the parameter vector theta. These may be
        obtained from least squares regression. Alternatively if
        `isigma` < 0 and `sigma` = 1 or if `isigma` > 0 and `sigma`
        approximately equals the standard deviation of the dependent
        variable, y, then `theta`[i-1] = 0.0, for i = 1,2,...,m may
        provide reasonable starting values.

    sigma : float
        A starting value for the estimation of sigma. `sigma` should be
        approximately the standard deviation of the residuals from the
        model evaluated at the value of theta given by `theta` on entry.

    tol : float, optional
        The relative precision for the calculation of A (if `indw` !=
        0), the estimates of theta and the estimate of sigma (if
        `isigma` != 0). Convergence is assumed when the relative change
        in all elements being considered is less than `tol`.

        If `indw` < 0 and `isigma` < 0, `tol` is also used to determine
        the precision of beta_1.

        It is advisable for `tol` to be greater than 100*machine
        precision.

    maxit : int, optional
        The maximum number of iterations that should be used in the
        calculation of A (if `indw` != 0), and of the estimates of theta
        and sigma, and of beta_1 (if `indw` < 0 and `isigma` < 0).

        A value of `maxit` = 50 should be adequate for most uses.

    nitmon : int, optional
        The amount of information that is printed on each iteration.

        `nitmon` = 0

          No information is printed.

        `nitmon` != 0

          The current estimate of theta, the change in theta during the
          current iteration and the current value of sigma are printed
          on the first and every abs(`nitmon`) iterations.

        Also, if `indw` != 0 and `nitmon` > 0, then information on the
        iterations to calculate A is printed.

        This is the current estimate of A and the maximum value of
        S_{ij} (see [Description]).

        When printing occurs the output is directed to the file object
        associated with the advisory I/O unit (see
        :class:`~naginterfaces.base.utils.FileObjManager`).

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    x : float, ndarray, shape (n, m)
        Unchanged, except as described above.

    y : float, ndarray, shape (n)
        Unchanged, except as described above.

    theta : float, ndarray, shape (m)
        `theta`[i-1] contains the M-estimate of theta_i, for i =
        1,2,...,m.

    sigma : float
        Contains the final estimate of sigma if `isigma` != 0 or the
        value assigned on entry if `isigma` = 0.

    c : float, ndarray, shape (m, m)
        The diagonal elements of `c` contain the estimated asymptotic
        standard errors of the estimates of theta, i.e., `c`[i-1,i-1]
        contains the estimated asymptotic standard error of the estimate
        contained in `theta`[i-1].

        The elements above the diagonal contain the estimated asymptotic
        correlation between the estimates of theta, i.e., `c`[i-1,j-1],
        1 <= i < j <= m contains the asymptotic correlation between the
        estimates contained in `theta`[i-1] and `theta`[j-1].

        The elements below the diagonal contain the estimated asymptotic
        covariance between the estimates of theta, i.e., `c`[i-1,j-1], 1
        <= j < i <= m contains the estimated asymptotic covariance
        between the estimates contained in `theta`[i-1] and
        `theta`[j-1].

    rs : float, ndarray, shape (n)
        The residuals from the model evaluated at final value of
        `theta`, i.e., `rs` contains the vector (y-X theta^).

    wgt : float, ndarray, shape (n)
        The vector of weights. `wgt`[i-1] contains the weight for the
        ith observation, for i = 1,2,...,n.

    stat : float, ndarray, shape (4)
        The following values are assigned to `stat`:

        - `stat`[0] = beta_1 if `isigma` < 0, or `stat`[0] = beta_2 if
          `isigma` > 0.

        - `stat`[1] = number of iterations used to calculate A.

        - `stat`[2] = number of iterations used to calculate final
          estimates of theta and sigma.

        - `stat`[3] = k, the rank of the weighted least squares
          equations.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>* and ldx = *<value>*.

            Constraint: ldx >= n.

        (`errno` 1)
            On entry, n = *<value>* and m = *<value>*.

            Constraint: n > m.

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry: `ipsi` = *<value>*.

            Constraint: `ipsi` = 0, 1, 2, 3 or 4.

        (`errno` 3)
            On entry, `cucv` = *<value>*.

            Constraint: `cucv` >= m.

        (`errno` 3)
            On entry, `cucv` = *<value>*.

            Constraint: `cucv` >= sqrt(m).

        (`errno` 3)
            On entry: `h1`, `h2`, `h3` incorrectly set.

            Constraint: 0.0 <= `h1` <= `h2` <= `h3` and `h3` > 0.0.

        (`errno` 3)
            On entry: `cpsi` = *<value>*.

            Constraint: `cpsi` > 0.0.

        (`errno` 3)
            On entry, `dchi` = *<value>*.

            Constraint: `dchi` > 0.0.

        (`errno` 3)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 4)
            On entry, `maxit` = *<value>*.

            Constraint: `maxit` > 0.

        (`errno` 4)
            On entry, `tol` = *<value>*.

            Constraint: `tol` > 0.0.

        (`errno` 5)
            The number of iterations required to calculate the weights
            exceeds `maxit`. (Only if `indw` != 0.)

        (`errno` 6)
            The number of iterations required to calculate beta_1
            exceeds `maxit`. (Only if `indw` < 0 and `isigma` < 0.)

        (`errno` 7)
            Iterations to calculate estimates of `theta` failed to
            converge in `maxit` iterations: `maxit` = *<value>*.

        (`errno` 7)
            The number of iterations required to calculate theta and
            sigma exceeds `maxit`.

        (`errno` 12)
            Error degrees of freedom n-k <= 0, where n = *<value>* and
            the rank of `x`, k = *<value>*.

        (`errno` 12)
            Estimated value of `sigma` is zero.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 8)
            Weighted least squares equations not of full rank: rank =
            *<value>*.

        (`errno` 9)
            Failure to invert matrix while calculating covariance.

        (`errno` 10)
            Factor for covariance matrix = 0, uncorrected (X^TX)^-1
            given.

        (`errno` 11)
            Variance of an element of `theta` <= 0.0, correlations set
            to 0.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.correg import robustm as b_robustm

    fun_name = 'naginterfaces.library.correg.robustm'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=True,
    )
    _theta = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'theta'},
        theta,
        copy=True,
    )
    _c = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _m_py),
        sorder=_m_sorder,
    )
    _rs = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _wgt = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _stat = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (4,),
    )
    try:
        _return_scalar = b_robustm(
            indw,
            ipsi,
            isigma,
            indc,
            _n,
            _m,
            _x,
            _y,
            cpsi,
            h1,
            h2,
            h3,
            cucv,
            dchi,
            _theta,
            sigma,
            _c,
            _rs,
            _wgt,
            tol,
            maxit,
            nitmon,
            _stat,
            io_manager,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _x_py = _x
    _y_py = _y
    _theta_py = _theta
    _sigma_py = _return_scalar
    _c_py = _c
    _rs_py = _rs
    _wgt_py = _wgt
    _stat_py = _stat
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'y', 'theta', 'sigma', 'c', 'rs', 'wgt', 'stat'],
    )(
        _x_py,
        _y_py,
        _theta_py,
        _sigma_py,
        _c_py,
        _rs_py,
        _wgt_py,
        _stat_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def robustm_wts(ucv, x, a, bl=0.9, bd=0.9, tol=5e-5, maxit=50, nitmon=0, data=None, io_manager=None):
    u"""
    Robust regression, compute weights for use with ``robustm_user``.

    For full information please refer to the NAG Library document for
    g02hb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02hbf.html

    Parameters
    ----------
    ucv : callable retval = ucv(t, data=None)
        `ucv` must return the value of the function u for a given value
        of its argument.

        The value of u must be non-negative.

        Parameters
        ~~~~~~~~~~
        t : float
            The argument for which `ucv` must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of u(t) evaluated at `t`.

    x : float, array-like, shape (n, m)
        The real matrix X, i.e., the independent variables. `x`[i-1,j-1]
        must contain the ijth element of `x`, for j = 1,2,...,m, for i =
        1,2,...,n.

    a : float, array-like, shape (m*(m+1)/2)
        An initial estimate of the lower triangular real matrix A. Only
        the lower triangular elements must be given and these should be
        stored row-wise in the array.

        The diagonal elements must be != 0, although in practice will
        usually be > 0.

        If the magnitudes of the columns of X are of the same order the
        identity matrix will often provide a suitable initial value for
        A.

        If the columns of X are of different magnitudes, the diagonal
        elements of the initial value of A should be approximately
        inversely proportional to the magnitude of the columns of X.

    bl : float, optional
        The magnitude of the bound for the off-diagonal elements of S_k.

    bd : float, optional
        The magnitude of the bound for the diagonal elements of S_k.

    tol : float, optional
        The relative precision for the final value of A. Iteration will
        stop when the maximum value of |s_{jl}| is less than `tol`.

    maxit : int, optional
        The maximum number of iterations that will be used during the
        calculation of A.

        A value of `maxit` = 50 will often be adequate.

    nitmon : int, optional
        Determines the amount of information that is printed on each
        iteration.

        `nitmon` > 0

          The value of A and the maximum value of |s_{jl}| will be
          printed at the first and every `nitmon` iterations.

        `nitmon` <= 0

          No iteration monitoring is printed.

        When printing occurs the output is directed to the file object
        associated with the advisory I/O unit (see
        :class:`~naginterfaces.base.utils.FileObjManager`).

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    a : float, ndarray, shape (m*(m+1)/2)
        The lower triangular elements of the matrix A, stored row-wise.

    z : float, ndarray, shape (n)
        The value ||z_i||_2, for i = 1,2,...,n.

    nit : int
        The number of iterations performed.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>* and ldx = *<value>*.

            Constraint: ldx >= n.

        (`errno` 1)
            On entry, n = *<value>* and m = *<value>*.

            Constraint: n >= m.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 2)
            On entry, `bd` = *<value>*.

            Constraint: `bd` > 0.0.

        (`errno` 2)
            On entry, `bl` = *<value>*.

            Constraint: `bl` > 0.0.

        (`errno` 2)
            On entry, `maxit` = *<value>*.

            Constraint: `maxit` > 0.

        (`errno` 2)
            On entry, `tol` = *<value>*.

            Constraint: `tol` > 0.0.

        (`errno` 2)
            On entry, i = *<value>* and the ith diagonal element of A is
            0.

            Constraint: all diagonal elements of A must be non-zero.

        (`errno` 3)
            Value returned by `ucv` function < 0: u(*<value>*) =
            *<value>*.

            The value of u must be non-negative.

        (`errno` 4)
            Iterations to calculate weights failed to converge in
            `maxit` iterations: `maxit` = *<value>*.
    """

    from ..base.correg import robustm_wts as b_robustm_wts

    def cb_ucv(t, data=None):
        """Wrapping user's ucv. Called as base ucv."""
        if data is not None:
            _retval_ucv = ucv(
                t,
                data,
            )
        else:
            _retval_ucv = ucv(
                t,
            )
        return _retval_ucv

    fun_name = 'naginterfaces.library.correg.robustm_wts'

    _b_utils._check_callable(ucv, {'fun_name': fun_name, 'entity': 'ucv'})
    _ucv = cb_ucv
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _z = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _return_scalar = b_robustm_wts(
        _ucv,
        _n,
        _m,
        _x,
        _a,
        _z,
        bl,
        bd,
        tol,
        maxit,
        nitmon,
        data,
        io_manager,
    )
    _a_py = _a
    _z_py = _z
    _nit_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['a', 'z', 'nit'],
    )(
        _a_py,
        _z_py,
        _nit_py,
    )
    return _return_data

def robustm_user(psi, psip0, beta, indw, isigma, x, y, wgt, theta, sigma, chi=None, tol=5e-5, eps=5e-6, maxit=50, nitmon=0, data=None, io_manager=None):
    u"""
    Robust regression, compute regression with user-supplied functions
    and weights.

    For full information please refer to the NAG Library document for
    g02hd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02hdf.html

    Parameters
    ----------
    psi : callable retval = psi(t, data=None)
        `psi` must return the value of the weight function psi for a
        given value of its argument.

        Parameters
        ~~~~~~~~~~
        t : float
            The argument for which `psi` must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of the weight function psi evaluated at `t`.

    psip0 : float
        The value of psi'(0).

    beta : float
        If `isigma` < 0, `beta` must specify the value of beta_1.

        For Huber and Schweppe type regressions, beta_1 is the 75th
        percentile of the standard Normal distribution (see
        ``stat.inv_cdf_normal``).

        For Mallows type regression beta_1 is the solution to

            1/nsum_i = 1^n Phi(beta_1/sqrt(w_i)) = 0.75,

        where Phi is the standard Normal cumulative distribution
        function (see ``specfun.cdf_normal``).

        If `isigma` > 0, `beta` must specify the value of beta_2.

            [table omitted]

        where phi is the standard normal density, i.e., 1/(sqrt(2
        pi))exp(-1/2x^2).

        If `isigma` = 0, `beta` is not referenced.

    indw : int
        Determines the type of regression to be performed.

        `indw` = 0

          Huber type regression.

        `indw` < 0

          Mallows type regression.

        `indw` > 0

          Schweppe type regression.

    isigma : int
        Determines how sigma is to be estimated.

        `isigma` = 0

          sigma is held constant at its initial value.

        `isigma` < 0

          sigma is estimated by median absolute deviation of residuals.

        `isigma` > 0

          sigma is estimated using the chi function.

    x : float, array-like, shape (n, m)
        The values of the X matrix, i.e., the independent variables.
        `x`[i-1,j-1] must contain the ijth element of `x`, for j =
        1,2,...,m, for i = 1,2,...,n.

        If `indw` < 0, during calculations the elements of `x` will be
        transformed as described in [Description].

        Before exit the inverse transformation will be applied.

        As a result there may be slight differences between the input
        `x` and the output `x`.

    y : float, array-like, shape (n)
        The data values of the dependent variable.

        `y`[i-1] must contain the value of y for the ith observation,
        for i = 1,2,...,n.

        If `indw` < 0, during calculations the elements of `y` will be
        transformed as described in [Description].

        Before exit the inverse transformation will be applied.

        As a result there may be slight differences between the input
        `y` and the output `y`.

    wgt : float, array-like, shape (n)
        The weight for the ith observation, for i = 1,2,...,n.

        If `indw` < 0, during calculations elements of `wgt` will be
        transformed as described in [Description].

        Before exit the inverse transformation will be applied.

        As a result there may be slight differences between the input
        `wgt` and the output `wgt`.

        If `wgt`[i-1] <= 0, the ith observation is not included in the
        analysis.

        If `indw` = 0, `wgt` is not referenced.

    theta : float, array-like, shape (m)
        Starting values of the parameter vector theta. These may be
        obtained from least squares regression. Alternatively if
        `isigma` < 0 and `sigma` = 1 or if `isigma` > 0 and `sigma`
        approximately equals the standard deviation of the dependent
        variable, y, then `theta`[i-1] = 0.0, for i = 1,2,...,m may
        provide reasonable starting values.

    sigma : float
        A starting value for the estimation of sigma. `sigma` should be
        approximately the standard deviation of the residuals from the
        model evaluated at the value of theta given by `theta` on entry.

    chi : None or callable retval = chi(t, data=None), optional
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        If `isigma` > 0, `chi` must return the value of the weight
        function chi for a given value of its argument.

        The value of chi must be non-negative.

        Parameters
        ~~~~~~~~~~
        t : float
            The argument for which `chi` must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of the weight function chi evaluated at `t`.

    tol : float, optional
        The relative precision for the final estimates. Convergence is
        assumed when both the relative change in the value of `sigma`
        and the relative change in the value of each element of `theta`
        are less than `tol`.

        It is advisable for `tol` to be greater than 100*machine
        precision.

    eps : float, optional
        A relative tolerance to be used to determine the rank of X. See
        ``linsys.real_gen_solve`` for further details.

        If `eps` < machine precision or `eps` > 1.0, machine precision
        will be used in place of `tol`.

        A reasonable value for `eps` is 5.0*10^(-6) where this value is
        possible.

    maxit : int, optional
        The maximum number of iterations that should be used during the
        estimation.

        A value of `maxit` = 50 should be adequate for most uses.

    nitmon : int, optional
        Determines the amount of information that is printed on each
        iteration.

        `nitmon` <= 0

          No information is printed.

        `nitmon` > 0

          On the first and every `nitmon` iterations the values of
          `sigma`, `theta` and the change in `theta` during the
          iteration are printed.

        When printing occurs the output is directed to the file object
        associated with the advisory I/O unit (see
        :class:`~naginterfaces.base.utils.FileObjManager`).

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    x : float, ndarray, shape (n, m)
        Unchanged, except as described above.

    y : float, ndarray, shape (n)
        Unchanged, except as described above.

    wgt : float, ndarray, shape (n)
        Unchanged, except as described above.

    theta : float, ndarray, shape (m)
        The M-estimate of theta_i, for i = 1,2,...,m.

    k : int
        The column rank of the matrix X.

    sigma : float
        The final estimate of sigma if `isigma` != 0 or the value
        assigned on entry if `isigma` = 0.

    rs : float, ndarray, shape (n)
        The residuals from the model evaluated at final value of
        `theta`, i.e., `rs` contains the vector (y-X theta^).

    nit : int
        The number of iterations that were used during the estimation.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>* and ldx = *<value>*.

            Constraint: ldx >= n.

        (`errno` 1)
            On entry, n = *<value>* and m = *<value>*.

            Constraint: n > m.

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, `beta` = *<value>*.

            Constraint: `beta` > 0.0.

        (`errno` 2)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` > 0.0.

        (`errno` 3)
            On entry, `maxit` = *<value>*.

            Constraint: `maxit` > 0.

        (`errno` 3)
            On entry, `tol` = *<value>*.

            Constraint: `tol` > 0.0.

        (`errno` 4)
            Value given by `chi` function < 0: `chi`(*<value>*) =
            *<value>*.

            The value of `chi` must be non-negative.

        (`errno` 5)
            Estimated value of `sigma` is zero.

        (`errno` 6)
            Iterations to solve the weighted least squares equations
            failed to converge.

        (`errno` 8)
            The function has failed to converge in `maxit` iterations.

        (`errno` 9)
            Having removed cases with zero weight, the value of n-`k` <=
            0, i.e., no degree of freedom for error. This error will
            only occur if `isigma` > 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 7)
            The weighted least squares equations are not of full rank.
            This may be due to the X matrix not being of full rank, in
            which case the results will be valid. It may also occur if
            some of the G_{{ii}} values become very small or zero, see
            [Further Comments]. The rank of the equations is given by
            `k`. If the matrix just fails the test for nonsingularity
            then the result `errno` = 7 and `k` = m is possible (see
            ``linsys.real_gen_solve``).
    """

    from ..base.correg import robustm_user as b_robustm_user

    def cb_chi(t, data=None):
        """Wrapping user's chi. Called as base chi."""
        if data is not None:
            _retval_chi = chi(
                t,
                data,
            )
        else:
            _retval_chi = chi(
                t,
            )
        return _retval_chi

    def cb_psi(t, data=None):
        """Wrapping user's psi. Called as base psi."""
        if data is not None:
            _retval_psi = psi(
                t,
                data,
            )
        else:
            _retval_psi = psi(
                t,
            )
        return _retval_psi

    fun_name = 'naginterfaces.library.correg.robustm_user'

    _b_utils._check_callable(chi, {'fun_name': fun_name, 'entity': 'chi'}, allow_none=True)
    _chi = None if chi is None else cb_chi
    _b_utils._check_callable(psi, {'fun_name': fun_name, 'entity': 'psi'})
    _psi = cb_psi
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=True,
    )
    _wgt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'wgt'},
        wgt,
        copy=True,
    )
    _theta = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'theta'},
        theta,
        copy=True,
    )
    _rs = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalars = b_robustm_user(
            _chi,
            _psi,
            psip0,
            beta,
            indw,
            isigma,
            _n,
            _m,
            _x,
            _y,
            _wgt,
            _theta,
            sigma,
            _rs,
            tol,
            eps,
            maxit,
            nitmon,
            data,
            io_manager,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _x_py = _x
    _y_py = _y
    _wgt_py = _wgt
    _theta_py = _theta
    _sigma_py = _return_scalars.sigma
    _k_py = _return_scalars.k
    _rs_py = _rs
    _nit_py = _return_scalars.nit
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'y', 'wgt', 'theta', 'k', 'sigma', 'rs', 'nit'],
    )(
        _x_py,
        _y_py,
        _wgt_py,
        _theta_py,
        _k_py,
        _sigma_py,
        _rs_py,
        _nit_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def robustm_user_varmat(psi, psp, indw, indc, sigma, x, rs, wgt, data=None):
    u"""
    Robust regression, variance-covariance matrix following
    ``robustm_user``.

    For full information please refer to the NAG Library document for
    g02hf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02hff.html

    Parameters
    ----------
    psi : callable retval = psi(t, data=None)
        `psi` must return the value of the psi function for a given
        value of its argument.

        Parameters
        ~~~~~~~~~~
        t : float
            The argument for which `psi` must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of the function psi evaluated at `t`.

    psp : callable retval = psp(t, data=None)
        `psp` must return the value of psi'(t) = d/dt psi(t) for a given
        value of its argument.

        Parameters
        ~~~~~~~~~~
        t : float
            The argument for which `psp` must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of psi'(t) evaluated at `t`.

    indw : int
        The type of regression for which the asymptotic
        variance-covariance matrix is to be calculated.

        `indw` = -1

          Mallows type regression.

        `indw` = 0

          Huber type regression.

        `indw` = 1

          Schweppe type regression.

    indc : int
        If `indw` != 0, `indc` must specify the approximation to be
        used.

        If `indc` = 1, averaging over residuals.

        If `indc` = 0, replacing expected by observed.

        If `indw` = 0, `indc` is not referenced.

    sigma : float
        The value of sigma^, as given by ``robustm_user``.

    x : float, array-like, shape (n, m)
        The values of the X matrix, i.e., the independent variables.
        `x`[i-1,j-1] must contain the ijth element of X, for j =
        1,2,...,m, for i = 1,2,...,n.

    rs : float, array-like, shape (n)
        The residuals from the bounded influence regression. These are
        given by ``robustm_user``.

    wgt : float, array-like, shape (n)
        If `indw` != 0, `wgt` must contain the vector of weights used by
        the bounded influence regression. These should be used with
        ``robustm_user``.

        If `indw` = 0, `wgt` is not referenced.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    c : float, ndarray, shape (m, m)
        The estimate of the variance-covariance matrix.

    wk : float, ndarray, shape (m*(n+m+1)+2*n)
        If `indw` != 0, `wk`[i-1], for i = 1,2,...,n, will contain the
        diagonal elements of the matrix D and `wk`[i-1], for i =
        n+1,...,2n, will contain the diagonal elements of matrix P.

        The rest of the array is used as workspace.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, m = *<value>* and ldc = *<value>*.

            Constraint: ldc >= m.

        (`errno` 1)
            On entry, n = *<value>* and ldx = *<value>*.

            Constraint: ldx >= n.

        (`errno` 1)
            On entry, n = *<value>* and m = *<value>*.

            Constraint: n > m.

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 2)
            On entry, `sigma` = *<value>*.

            Constraint: `sigma` >= 0.0.

        (`errno` 3)
            S_1 matrix is singular or almost singular.

        (`errno` 3)
            X^TX matrix not positive definite.

        (`errno` 4)
            Correction factor = 0 (Huber type regression).
    """

    from ..base.correg import robustm_user_varmat as b_robustm_user_varmat

    def cb_psi(t, data=None):
        """Wrapping user's psi. Called as base psi."""
        if data is not None:
            _retval_psi = psi(
                t,
                data,
            )
        else:
            _retval_psi = psi(
                t,
            )
        return _retval_psi

    def cb_psp(t, data=None):
        """Wrapping user's psp. Called as base psp."""
        if data is not None:
            _retval_psp = psp(
                t,
                data,
            )
        else:
            _retval_psp = psp(
                t,
            )
        return _retval_psp

    fun_name = 'naginterfaces.library.correg.robustm_user_varmat'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'),),
    )
    _b_utils._check_callable(psi, {'fun_name': fun_name, 'entity': 'psi'})
    _psi = cb_psi
    _b_utils._check_callable(psp, {'fun_name': fun_name, 'entity': 'psp'})
    _psp = cb_psp
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _rs = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'rs'},
        rs,
        copy=False,
    )
    _wgt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'wgt'},
        wgt,
        copy=False,
    )
    _c = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _m_py),
        sorder=_m_sorder,
    )
    _wk = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py*(_n_py+_m_py+1)+2*_n_py,),
    )
    b_robustm_user_varmat(
        _psi,
        _psp,
        indw,
        indc,
        sigma,
        _n,
        _m,
        _x,
        _rs,
        _wgt,
        _c,
        _wk,
        data,
    )
    _c_py = _c
    _wk_py = _wk
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['c', 'wk'],
    )(
        _c_py,
        _wk_py,
    )
    return _return_data

def robustm_corr_huber(x, eps, maxit=150, nitmon=0, tol=5e-5, io_manager=None):
    u"""
    Calculates a robust estimation of a covariance matrix, Huber's
    weight function.

    For full information please refer to the NAG Library document for
    g02hk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02hkf.html

    Parameters
    ----------
    x : float, array-like, shape (n, m)
        `x`[i-1,j-1] must contain the ith observation for the jth
        variable, for j = 1,2,...,m, for i = 1,2,...,n.

    eps : float
        epsilon, the expected fraction of gross errors expected in the
        sample.

    maxit : int, optional
        The maximum number of iterations that will be used during the
        calculation of the covariance matrix.

    nitmon : int, optional
        Indicates the amount of information on the iteration that is
        printed.

        `nitmon` > 0

          The value of A, theta and delta (see [Accuracy]) will be
          printed at the first and every `nitmon` iterations.

        `nitmon` <= 0

          No iteration monitoring is printed.

        When printing occurs the output is directed to the file object
        associated with the advisory I/O unit (see
        :class:`~naginterfaces.base.utils.FileObjManager`).

    tol : float, optional
        The relative precision for the final estimates of the covariance
        matrix.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    cov : float, ndarray, shape (m*(m+1)/2)
        A robust estimate of the covariance matrix, C. The upper
        triangular part of the matrix C is stored packed by columns.
        C_{ij} is returned in `cov`[(j*(j-1)/2+i)-1], i <= j.

    theta : float, ndarray, shape (m)
        The robust estimate of the location parameters theta_j, for j =
        1,2,...,m.

    nit : int
        The number of iterations performed.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `maxit` = *<value>*.

            Constraint: `maxit` > 0.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` > 0.0.

        (`errno` 1)
            On entry, `eps` = *<value>*.

            Constraint: 0.0 <= `eps` <= 1.0.

        (`errno` 1)
            On entry, n = *<value>* and m = *<value>*.

            Constraint: n >= m.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 2)
            On entry, a variable has a constant value, i.e., all
            elements in column *<value>* of `x` are identical.

        (`errno` 3)
            The iterative procedure to find C has failed to converge in
            `maxit` iterations.

        (`errno` 4)
            The iterative procedure to find C has become unstable. This
            may happen if the value of `eps` is too large for the
            sample.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.correg import robustm_corr_huber as b_robustm_corr_huber

    fun_name = 'naginterfaces.library.correg.robustm_corr_huber'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _cov = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py*(_m_py+1)//2,),
    )
    _theta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    _return_scalar = b_robustm_corr_huber(
        _n,
        _m,
        _x,
        eps,
        _cov,
        _theta,
        maxit,
        nitmon,
        tol,
        io_manager,
    )
    _cov_py = _cov
    _theta_py = _theta
    _nit_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['cov', 'theta', 'nit'],
    )(
        _cov_py,
        _theta_py,
        _nit_py,
    )
    return _return_data

def robustm_corr_user_deriv(ucv, indm, x, a, theta, bl=0.9, bd=0.9, maxit=150, nitmon=0, tol=5e-5, data=None, io_manager=None):
    u"""
    Calculates a robust estimation of a covariance matrix, user-supplied
    weight function plus derivatives.

    For full information please refer to the NAG Library document for
    g02hl

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02hlf.html

    Parameters
    ----------
    ucv : callable (u, ud, w, wd) = ucv(t, data=None)
        `ucv` must return the values of the functions u and w and their
        derivatives for a given value of its argument.

        Parameters
        ~~~~~~~~~~
        t : float
            The argument for which the functions u and w must be
            evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        u : float
            The value of the u function at the point `t`.

        ud : float
            The value of the derivative of the u function at the point
            `t`.

        w : float
            The value of the w function at the point `t`.

        wd : float
            The value of the derivative of the w function at the point
            `t`.

    indm : int
        Indicates which form of the function v will be used.

        `indm` = 1

          v = 1.

        `indm` != 1

          v = u.

    x : float, array-like, shape (n, m)
        `x`[i-1,j-1] must contain the ith observation on the jth
        variable, for j = 1,2,...,m, for i = 1,2,...,n.

    a : float, array-like, shape (m*(m+1)/2)
        An initial estimate of the lower triangular real matrix A. Only
        the lower triangular elements must be given and these should be
        stored row-wise in the array.

        The diagonal elements must be != 0, and in practice will usually
        be > 0.

        If the magnitudes of the columns of X are of the same order, the
        identity matrix will often provide a suitable initial value for
        A.

        If the columns of X are of different magnitudes, the diagonal
        elements of the initial value of A should be approximately
        inversely proportional to the magnitude of the columns of X.

    theta : float, array-like, shape (m)
        An initial estimate of the location parameter, theta_j, for j =
        1,2,...,m.

        In many cases an initial estimate of theta_j = 0, for j =
        1,2,...,m, will be adequate.

        Alternatively medians may be used as given by
        ``univar.robust_1var_median``.

    bl : float, optional
        The magnitude of the bound for the off-diagonal elements of S_k,
        BL.

    bd : float, optional
        The magnitude of the bound for the diagonal elements of S_k, BD.

    maxit : int, optional
        The maximum number of iterations that will be used during the
        calculation of A.

    nitmon : int, optional
        Indicates the amount of information on the iteration that is
        printed.

        `nitmon` > 0

          The value of A, theta and delta (see [Accuracy]) will be
          printed at the first and every `nitmon` iterations.

        `nitmon` <= 0

          No iteration monitoring is printed.

        When printing occurs the output is directed to the file object
        associated with the advisory I/O unit (see
        :class:`~naginterfaces.base.utils.FileObjManager`).

    tol : float, optional
        The relative precision for the final estimates of the covariance
        matrix. Iteration will stop when maximum delta (see [Accuracy])
        is less than `tol`.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    cov : float, ndarray, shape (m*(m+1)/2)
        Contains a robust estimate of the covariance matrix, C. The
        upper triangular part of the matrix C is stored packed by
        columns (lower triangular stored by rows), C_{ij} is returned in
        `cov`[(j*(j-1)/2+i)-1], i <= j.

    a : float, ndarray, shape (m*(m+1)/2)
        The lower triangular elements of the inverse of the matrix A,
        stored row-wise.

    wt : float, ndarray, shape (n)
        `wt`[i-1] contains the weights, wt_i = u(||z_i||_2), for i =
        1,2,...,n.

    theta : float, ndarray, shape (m)
        Contains the robust estimate of the location parameter, theta_j,
        for j = 1,2,...,m.

    nit : int
        The number of iterations performed.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>* and m = *<value>*.

            Constraint: n >= m.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 2)
            On entry, `bd` = *<value>*.

            Constraint: `bd` > 0.0.

        (`errno` 2)
            On entry, `bl` = *<value>*.

            Constraint: `bl` > 0.0.

        (`errno` 2)
            On entry, `maxit` = *<value>*.

            Constraint: `maxit` > 0.

        (`errno` 2)
            On entry, `tol` = *<value>*.

            Constraint: `tol` > 0.0.

        (`errno` 2)
            On entry, i = *<value>* and the ith diagonal element of A is
            0.

            Constraint: all diagonal elements of A must be non-zero.

        (`errno` 3)
            On entry, a variable has a constant value, i.e., all
            elements in column *<value>* of `x` are identical.

        (`errno` 4)
            w value returned by `ucv` < 0.0: w(*<value>*) = *<value>*.

            Constraint: `w` >= 0.0.

        (`errno` 4)
            u value returned by `ucv` < 0.0: u(*<value>*) = *<value>*.

            Constraint: `u` >= 0.0.

        (`errno` 5)
            Iterations to calculate weights failed to converge.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 6)
            The sum D_2 is zero. Try either a larger initial estimate of
            A or make u and w less strict.

        (`errno` 6)
            The sum D_3 is zero. Try either a larger initial estimate of
            A or make u and w less strict.

        (`errno` 6)
            The sum D_1 is zero. Try either a larger initial estimate of
            A or make u and w less strict.
    """

    from ..base.correg import robustm_corr_user_deriv as b_robustm_corr_user_deriv

    def cb_ucv(t, data=None):
        """Wrapping user's ucv. Called as base ucv."""
        if data is not None:
            _u_ucv, _ud_ucv, _w_ucv, _wd_ucv = ucv(
                t,
                data,
            )
        else:
            _u_ucv, _ud_ucv, _w_ucv, _wd_ucv = ucv(
                t,
            )
        _u_ucv_base = _u_ucv
        _ud_ucv_base = _ud_ucv
        _w_ucv_base = _w_ucv
        _wd_ucv_base = _wd_ucv
        return (
            _u_ucv_base,
            _ud_ucv_base,
            _w_ucv_base,
            _wd_ucv_base,
        )

    fun_name = 'naginterfaces.library.correg.robustm_corr_user_deriv'

    _b_utils._check_callable(ucv, {'fun_name': fun_name, 'entity': 'ucv'})
    _ucv = cb_ucv
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _cov = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py*(_m_py+1)//2,),
    )
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _wt = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _theta = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'theta'},
        theta,
        copy=True,
    )
    try:
        _return_scalar = b_robustm_corr_user_deriv(
            _ucv,
            indm,
            _n,
            _m,
            _x,
            _cov,
            _a,
            _wt,
            _theta,
            bl,
            bd,
            maxit,
            nitmon,
            tol,
            data,
            io_manager,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _a_py = _a
    _theta_py = _theta
    _cov_py = _cov
    _wt_py = _wt
    _nit_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['cov', 'a', 'wt', 'theta', 'nit'],
    )(
        _cov_py,
        _a_py,
        _wt_py,
        _theta_py,
        _nit_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def robustm_corr_user(ucv, indm, x, a, theta, bl=0.9, bd=0.9, maxit=150, nitmon=0, tol=5e-5, data=None, io_manager=None):
    u"""
    Calculates a robust estimation of a covariance matrix, user-supplied
    weight function.

    For full information please refer to the NAG Library document for
    g02hm

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02hmf.html

    Parameters
    ----------
    ucv : callable (u, w) = ucv(t, data=None)
        `ucv` must return the values of the functions u and w for a
        given value of its argument.

        Parameters
        ~~~~~~~~~~
        t : float
            The argument for which the functions u and w must be
            evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        u : float
            The value of the u function at the point `t`.

        w : float
            The value of the w function at the point `t`.

    indm : int
        Indicates which form of the function v will be used.

        `indm` = 1

          v = 1.

        `indm` != 1

          v = u.

    x : float, array-like, shape (n, m)
        `x`[i-1,j-1] must contain the ith observation on the jth
        variable, for j = 1,2,...,m, for i = 1,2,...,n.

    a : float, array-like, shape (m*(m+1)/2)
        An initial estimate of the lower triangular real matrix A. Only
        the lower triangular elements must be given and these should be
        stored row-wise in the array.

        The diagonal elements must be != 0, and in practice will usually
        be > 0.

        If the magnitudes of the columns of X are of the same order, the
        identity matrix will often provide a suitable initial value for
        A.

        If the columns of X are of different magnitudes, the diagonal
        elements of the initial value of A should be approximately
        inversely proportional to the magnitude of the columns of X.

    theta : float, array-like, shape (m)
        An initial estimate of the location parameter, theta_j, for j =
        1,2,...,m.

        In many cases an initial estimate of theta_j = 0, for j =
        1,2,...,m, will be adequate.

        Alternatively medians may be used as given by
        ``univar.robust_1var_median``.

    bl : float, optional
        The magnitude of the bound for the off-diagonal elements of S_k,
        BL.

    bd : float, optional
        The magnitude of the bound for the diagonal elements of S_k, BD.

    maxit : int, optional
        The maximum number of iterations that will be used during the
        calculation of A.

    nitmon : int, optional
        Indicates the amount of information on the iteration that is
        printed.

        `nitmon` > 0

          The value of A, theta and delta (see [Accuracy]) will be
          printed at the first and every `nitmon` iterations.

        `nitmon` <= 0

          No iteration monitoring is printed.

    tol : float, optional
        The relative precision for the final estimate of the covariance
        matrix. Iteration will stop when maximum delta (see [Accuracy])
        is less than `tol`.

    data : arbitrary, optional
        User-communication data for callback functions.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    cov : float, ndarray, shape (m*(m+1)/2)
        A robust estimate of the covariance matrix, C. The upper
        triangular part of the matrix C is stored packed by columns
        (lower triangular stored by rows), that is C_{ij} is returned in
        `cov`[j*(j-1)/2+i-1], i <= j.

    a : float, ndarray, shape (m*(m+1)/2)
        The lower triangular elements of the inverse of the matrix A,
        stored row-wise.

    wt : float, ndarray, shape (n)
        `wt`[i-1] contains the weights, wt_i = u(||z_i||_2), for i =
        1,2,...,n.

    theta : float, ndarray, shape (m)
        Contains the robust estimate of the location parameter, theta_j,
        for j = 1,2,...,m.

    nit : int
        The number of iterations performed.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>* and m = *<value>*.

            Constraint: n >= m.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 2)
            On entry, `bd` = *<value>*.

            Constraint: `bd` > 0.0.

        (`errno` 2)
            On entry, `bl` = *<value>*.

            Constraint: `bl` > 0.0.

        (`errno` 2)
            On entry, `maxit` = *<value>*.

            Constraint: `maxit` > 0.

        (`errno` 2)
            On entry, `tol` = *<value>*.

            Constraint: `tol` > 0.0.

        (`errno` 2)
            On entry, i = *<value>* and the ith diagonal element of A is
            0.

            Constraint: all diagonal elements of A must be non-zero.

        (`errno` 3)
            On entry, a variable has a constant value, i.e., all
            elements in column *<value>* of `x` are identical.

        (`errno` 4)
            w value returned by `ucv` < 0.0: w(*<value>*) = *<value>*.

            Constraint: `w` >= 0.0.

        (`errno` 4)
            u value returned by `ucv` < 0.0: u(*<value>*) = *<value>*.

            Constraint: `u` >= 0.0.

        (`errno` 5)
            Iterations to calculate weights failed to converge.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 6)
            The sum D_2 is zero. Try either a larger initial estimate of
            A or make u and w less strict.

        (`errno` 6)
            The sum D_1 is zero. Try either a larger initial estimate of
            A or make u and w less strict.
    """

    from ..base.correg import robustm_corr_user as b_robustm_corr_user

    def cb_ucv(t, data=None):
        """Wrapping user's ucv. Called as base ucv."""
        if data is not None:
            _u_ucv, _w_ucv = ucv(
                t,
                data,
            )
        else:
            _u_ucv, _w_ucv = ucv(
                t,
            )
        _u_ucv_base = _u_ucv
        _w_ucv_base = _w_ucv
        return (
            _u_ucv_base,
            _w_ucv_base,
        )

    fun_name = 'naginterfaces.library.correg.robustm_corr_user'

    _b_utils._check_callable(ucv, {'fun_name': fun_name, 'entity': 'ucv'})
    _ucv = cb_ucv
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _cov = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py*(_m_py+1)//2,),
    )
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _wt = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _theta = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'theta'},
        theta,
        copy=True,
    )
    try:
        _return_scalar = b_robustm_corr_user(
            _ucv,
            indm,
            _n,
            _m,
            _x,
            _cov,
            _a,
            _wt,
            _theta,
            bl,
            bd,
            maxit,
            nitmon,
            tol,
            data,
            io_manager,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _a_py = _a
    _theta_py = _theta
    _cov_py = _cov
    _wt_py = _wt
    _nit_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['cov', 'a', 'wt', 'theta', 'nit'],
    )(
        _cov_py,
        _a_py,
        _wt_py,
        _theta_py,
        _nit_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

@_b_utils._withdrawal(
    'naginterfaces.library.correg.mixeff_reml',
    mkwdrn=None,
    advice=[
        'Please use lmm_init followed by lmm_fit instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def mixeff_reml(dat, levels, yvid, cwid, fvid, fint, rvid, nvpr, vpr, rint, svid, gamma, lb, maxit=-1, tol=0.0, io_manager=None):
    u"""
    Linear mixed effects regression using Restricted Maximum Likelihood
    (REML).

    Note: ``mixeff_reml`` is deprecated.
    Please use ``lmm_init`` followed by ``lmm_fit`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    g02ja

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02jaf.html

    Parameters
    ----------
    dat : float, array-like, shape (n, ncol)
        Array containing all of the data. For the ith observation:

        - `dat`[i-1,`yvid`-1] holds the dependent variable, y;

        - if `cwid` != 0, `dat`[i-1,`cwid`-1] holds the case weights;

        - if `svid` != 0, `dat`[i-1,`svid`-1] holds the subject
          variable.

        The remaining columns hold the values of the independent
        variables.

    levels : int, array-like, shape (ncol)
        `levels`[i-1] contains the number of levels associated with the
        ith variable of the data matrix `dat`. If this variable is
        continuous or binary then `levels`[i-1] should be 1; if the
        variable is discrete then `levels`[i-1] is the number of levels
        associated with it and `dat`[j-1,i-1] is assumed to take the
        values 1 to `levels`[i-1], for j = 1,2,...,n.

    yvid : int
        The column of `dat` holding the dependent, y, variable.

    cwid : int
        The column of `dat` holding the case weights.

        If `cwid` = 0, no weights are used.

    fvid : int, array-like, shape (nfv)
        The columns of the data matrix `dat` holding the fixed
        independent variables with `fvid`[i-1] holding the column number
        corresponding to the ith fixed variable.

    fint : int
        Flag indicating whether a fixed intercept is included (`fint` =
        1).

    rvid : int, array-like, shape (nrv)
        The columns of the data matrix `dat` holding the random
        independent variables with `rvid`[i-1] holding the column number
        corresponding to the ith random variable.

    nvpr : int
        If `rint` = 1 and `svid` != 0, `nvpr` is the number of variance
        components being estimated-2, (g-1), else `nvpr` = g.

        If nrv = 0, `nvpr` is not referenced.

    vpr : int, array-like, shape (nrv)
        `vpr`[i-1] holds a flag indicating the variance of the ith
        random variable. The variance of the ith random variable is
        sigma_j^2, where j = `vpr`[i-1]+1 if `rint` = 1 and `svid` != 0
        and j = `vpr`[i-1] otherwise. Random variables with the same
        value of j are assumed to be taken from the same distribution.

    rint : int
        Flag indicating whether a random intercept is included (`rint` =
        1).

        If `svid` = 0, `rint` is not referenced.

    svid : int
        The column of `dat` holding the subject variable.

        If `svid` = 0, no subject variable is used.

        Specifying a subject variable is equivalent to specifying the
        interaction between that variable and all of the random-effects.

        Letting the notation Z_1*Z_S denote the interaction between
        variables Z_1 and Z_S, fitting a model with `rint` = 0,
        random-effects Z_1+Z_2 and subject variable Z_S is equivalent to
        fitting a model with random-effects Z_1*Z_S+Z_2*Z_S and no
        subject variable.

        If `rint` = 1 the model is equivalent to fitting
        Z_S+Z_1*Z_S+Z_2*Z_S and no subject variable.

    gamma : float, array-like, shape (`nvpr`+2)
        Holds the initial values of the variance components, gamma_0,
        with `gamma`[i-1] the initial value for sigma_i^2/sigma_R^2, for
        i = 1,2,...,g. If `rint` = 1 and `svid` != 0, g = `nvpr`+1, else
        g = `nvpr`.

        If `gamma`[0] = -1.0, the remaining elements of `gamma` are
        ignored and the initial values for the variance components are
        estimated from the data using MIVQUE0.

    lb : int
        The size of the array `b`.

    maxit : int, optional
        The maximum number of iterations.

        If `maxit` < 0, the default value of 100 is used.

        If `maxit` = 0, the parameter estimates (beta,nu) and
        corresponding standard errors are calculated based on the value
        of gamma_0 supplied in `gamma`.

    tol : float, optional
        The tolerance used to assess convergence.

        If `tol` <= 0.0, the default value of epsilon^0.7 is used, where
        epsilon is the machine precision.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    gamma : float, ndarray, shape (`nvpr`+2)
        `gamma`[i-1], for i = 1,2,...,g, holds the final estimate of
        sigma_i^2 and `gamma`[g] holds the final estimate for sigma_R^2.

    nff : int
        The number of fixed effects estimated (i.e., the number of
        columns, p, in the design matrix X).

    nrf : int
        The number of random effects estimated (i.e., the number of
        columns, q, in the design matrix Z).

    df : int
        The degrees of freedom.

    reml : float
        -2l_R(gamma^) where l_R is the log of the restricted maximum
        likelihood calculated at gamma^, the estimated variance
        components returned in `gamma`.

    b : float, ndarray, shape (`lb`)
        The parameter estimates, (beta,nu), with the first `nff`
        elements of `b` containing the fixed effect parameter estimates,
        beta and the next `nrf` elements of `b` containing the random
        effect parameter estimates, nu.

        Fixed effects

        If `fint` = 1, `b`[0] contains the estimate of the fixed
        intercept.

        Let L_i denote the number of levels associated with the ith
        fixed variable, that is L_i = `levels`[ `fvid`[i-1] -1].

        Define

        - if `fint` = 1, F_1 = 2 else if `fint` = 0, F_1 = 1;

        - F_{i+1} = F_i+max(L_i-1,1), i >= 1.

        Then for i = 1,2,...,nfv:

        - if L_i > 1, `b`[ F_i+j-2 -1] contains the parameter estimate
          for the jth level of the ith fixed variable, for j =
          2,3,...,L_i;

        - if L_i <= 1, `b`[F_i-1] contains the parameter estimate for
          the ith fixed variable.

        Random effects

        Redefining L_i to denote the number of levels associated with
        the ith random variable, that is L_i = `levels`[ `rvid`[i-1]
        -1].

        Define

        - if `rint` = 1, R_1 = 2 else if `rint` = 0, R_1 = 1;

          R_{i+1} = R_i+L_i, i >= 1.

        Then for i = 1,2,...,nrv:

        - if `svid` = 0,

          - if L_i > 1, `b`[ `nff` + R_i +j-1 -1] contains the parameter
            estimate for the jth level of the ith random variable, for j
            = 1,2,...,L_i;

          - if L_i <= 1, `b`[ `nff` + R_i -1] contains the parameter
            estimate for the ith random variable;

        - if `svid` != 0,

          - let L_S denote the number of levels associated with the
            subject variable, that is L_S = `levels`[ `svid` -1];

          - if L_i > 1, `b`[ `nff` + (s-1) L_S + R_i + j - 1 -1]
            contains the parameter estimate for the interaction between
            the sth level of the subject variable and the jth level of
            the ith random variable, for j = 1,2,...,L_i, for s =
            1,2,...,L_S;

          - if L_i <= 1, `b`[ `nff` + (s-1) L_S + R_i -1] contains the
            parameter estimate for the interaction between the sth level
            of the subject variable and the ith random variable, for s =
            1,2,...,L_S;

          - if `rint` = 1, `b`[`nff`] contains the estimate of the
            random intercept.

    se : float, ndarray, shape (`lb`)
        The standard errors of the parameter estimates given in `b`.

    warn : int
        Is set to 1 if a variance component was estimated to be a
        negative value during the fitting process. Otherwise `warn` is
        set to 0.

        If `warn` = 1, the negative estimate is set to zero and the
        estimation process allowed to continue.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: number of observations with nonzero weights must
            be greater than one.

        (`errno` 1)
            On entry, ncol = *<value>*.

            Constraint: ncol >= 1.

        (`errno` 1)
            On entry, nfv = *<value>* and ncol = *<value>*.

            Constraint: 0 <= nfv < ncol.

        (`errno` 1)
            On entry, nrv = *<value>* and ncol = *<value>*.

            Constraint: 0 <= nrv < ncol and nrv+`rint` > 0.

        (`errno` 1)
            On entry, `nvpr` = *<value>* and nrv = *<value>*.

            Constraint: 0 <= `nvpr` <= nrv and (nrv != 0 or `nvpr` >=
            1).

        (`errno` 1)
            On entry, `yvid` = *<value>* and ncol = *<value>*.

            Constraint: 1 <= `yvid` <= ncol.

        (`errno` 1)
            On entry, `svid` = *<value>* and ncol = *<value>*.

            Constraint: 0 <= `svid` <= ncol.

        (`errno` 1)
            On entry, `cwid` = *<value>* and ncol = *<value>*.

            Constraint: 0 <= `cwid` <= ncol and any supplied weights
            must be >= 0.0.

        (`errno` 1)
            On entry, `fint` = *<value>*.

            Constraint: `fint` = 0 or 1.

        (`errno` 1)
            On entry, `rint` = *<value>*.

            Constraint: `rint` = 0 or 1.

        (`errno` 1)
            On entry, `lb` too small: `lb` = *<value>*.

        (`errno` 2)
            On entry, `levels`[i] < 1, for at least one i.

        (`errno` 2)
            On entry, ncol = *<value>*.

            Constraint: 1 <= `fvid`[i] <= ncol, for all i.

        (`errno` 2)
            On entry, ncol = *<value>*.

            Constraint: 1 <= `rvid`[i] <= ncol, for all i.

        (`errno` 2)
            On entry, `nvpr` = *<value>*.

            Constraint: 1 <= `vpr`[i] <= `nvpr`, for all i.

        (`errno` 2)
            On entry, invalid data: categorical variable with value
            greater than that specified in `levels`.

        (`errno` 2)
            On entry, `gamma`[i] < 0.0, for at least one i.

        (`errno` 3)
            Degrees of freedom < 1: `df` = *<value>*.

        (`errno` 4)
            Routine failed to converge to specified tolerance: `tol` =
            *<value>*.

        (`errno` 4)
            Routine failed to converge in `maxit` iterations: `maxit` =
            *<value>*.
    """

    from ..base.correg import mixeff_reml as b_mixeff_reml

    fun_name = 'naginterfaces.library.correg.mixeff_reml'

    _dat = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'dat'},
        dat,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_dat, {'fun_name': fun_name, 'entity': 'dat'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _ncol_py = _b_utils._EngineFloat64ArrayType.get_shape(_dat, {'fun_name': fun_name, 'entity': 'dat'}, exp_rank=2)[1]
    _ncol = _b_utils._EngineIntScalarType(
        _ncol_py, {'fun_name': fun_name, 'entity': '_ncol'},
    ).to_ctypes()
    _levels = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'levels'},
        levels,
        copy=False,
    )
    _fvid = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'fvid'},
        fvid,
        copy=False,
    )
    _nfv_py = _b_utils._EngineIntArrayType.get_shape(_fvid, {'fun_name': fun_name, 'entity': 'fvid'}, exp_rank=1)[0]
    _nfv = _b_utils._EngineIntScalarType(
        _nfv_py, {'fun_name': fun_name, 'entity': '_nfv'},
    ).to_ctypes()
    _rvid = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'rvid'},
        rvid,
        copy=False,
    )
    _nrv_py = _b_utils._EngineIntArrayType.get_shape(_rvid, {'fun_name': fun_name, 'entity': 'rvid'}, exp_rank=1)[0]
    _nrv = _b_utils._EngineIntScalarType(
        _nrv_py, {'fun_name': fun_name, 'entity': '_nrv'},
    ).to_ctypes()
    _vpr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vpr'},
        vpr,
        copy=False,
    )
    _gamma = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'gamma'},
        gamma,
        copy=True,
    )
    _lb = _b_utils._EngineIntScalarType(
        lb, {'fun_name': fun_name, 'entity': 'lb'},
    ).to_ctypes()
    _lb_py = _b_utils._EngineIntScalarType.to_py(_lb)
    _b = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_lb_py,),
    )
    _se = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_lb_py,),
    )
    _return_scalars = b_mixeff_reml(
        _n,
        _ncol,
        _dat,
        _levels,
        yvid,
        cwid,
        _nfv,
        _fvid,
        fint,
        _nrv,
        _rvid,
        nvpr,
        _vpr,
        rint,
        svid,
        _gamma,
        _lb,
        _b,
        _se,
        maxit,
        tol,
        io_manager,
    )
    _gamma_py = _gamma
    _nff_py = _return_scalars.nff
    _nrf_py = _return_scalars.nrf
    _df_py = _return_scalars.df
    _reml_py = _return_scalars.reml
    _b_py = _b
    _se_py = _se
    _warn_py = _return_scalars.warn
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['gamma', 'nff', 'nrf', 'df', 'reml', 'b', 'se', 'warn'],
    )(
        _gamma_py,
        _nff_py,
        _nrf_py,
        _df_py,
        _reml_py,
        _b_py,
        _se_py,
        _warn_py,
    )
    return _return_data

@_b_utils._withdrawal(
    'naginterfaces.library.correg.mixeff_ml',
    mkwdrn=None,
    advice=[
        'Please use lmm_init followed by lmm_fit instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def mixeff_ml(dat, levels, yvid, cwid, fvid, fint, rvid, nvpr, vpr, rint, svid, gamma, lb, maxit=-1, tol=0.0, io_manager=None):
    u"""
    Linear mixed effects regression using Maximum Likelihood (ML).

    Note: ``mixeff_ml`` is deprecated.
    Please use ``lmm_init`` followed by ``lmm_fit`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    g02jb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02jbf.html

    Parameters
    ----------
    dat : float, array-like, shape (n, ncol)
        Array containing all of the data. For the ith observation:

        - `dat`[i-1,`yvid`-1] holds the dependent variable, y;

        - if `cwid` != 0, `dat`[i-1,`cwid`-1] holds the case weights;

        - if `svid` != 0, `dat`[i-1,`svid`-1] holds the subject
          variable.

        The remaining columns hold the values of the independent
        variables.

    levels : int, array-like, shape (ncol)
        `levels`[i-1] contains the number of levels associated with the
        ith variable of the data matrix `dat`. If this variable is
        continuous or binary then `levels`[i-1] should be 1; if the
        variable is discrete then `levels`[i-1] is the number of levels
        associated with it and `dat`[j-1,i-1] is assumed to take the
        values 1 to `levels`[i-1], for j = 1,2,...,n.

    yvid : int
        The column of `dat` holding the dependent, y, variable.

    cwid : int
        The column of `dat` holding the case weights.

        If `cwid` = 0, no weights are used.

    fvid : int, array-like, shape (nfv)
        The columns of the data matrix `dat` holding the fixed
        independent variables with `fvid`[i-1] holding the column number
        corresponding to the ith fixed variable.

    fint : int
        Flag indicating whether a fixed intercept is included (`fint` =
        1).

    rvid : int, array-like, shape (nrv)
        The columns of the data matrix `dat` holding the random
        independent variables with `rvid`[i-1] holding the column number
        corresponding to the ith random variable.

    nvpr : int
        If `rint` = 1 and `svid` != 0, `nvpr` is the number of variance
        components being estimated-2, (g-1), else `nvpr` = g.

        If nrv = 0, `nvpr` is not referenced.

    vpr : int, array-like, shape (nrv)
        `vpr`[i-1] holds a flag indicating the variance of the ith
        random variable. The variance of the ith random variable is
        sigma_j^2, where j = `vpr`[i-1]+1 if `rint` = 1 and `svid` != 0
        and j = `vpr`[i-1] otherwise. Random variables with the same
        value of j are assumed to be taken from the same distribution.

    rint : int
        Flag indicating whether a random intercept is included (`rint` =
        1).

        If `svid` = 0, `rint` is not referenced.

    svid : int
        The column of `dat` holding the subject variable.

        If `svid` = 0, no subject variable is used.

        Specifying a subject variable is equivalent to specifying the
        interaction between that variable and all of the random-effects.

        Letting the notation Z_1*Z_S denote the interaction between
        variables Z_1 and Z_S, fitting a model with `rint` = 0,
        random-effects Z_1+Z_2 and subject variable Z_S is equivalent to
        fitting a model with random-effects Z_1*Z_S+Z_2*Z_S and no
        subject variable.

        If `rint` = 1 the model is equivalent to fitting
        Z_S+Z_1*Z_S+Z_2*Z_S and no subject variable.

    gamma : float, array-like, shape (`nvpr`+2)
        Holds the initial values of the variance components, gamma_0,
        with `gamma`[i-1] the initial value for sigma_i^2/sigma_R^2, for
        i = 1,2,...,g. If `rint` = 1 and `svid` != 0, g = `nvpr`+1, else
        g = `nvpr`.

        If `gamma`[0] = -1.0, the remaining elements of `gamma` are
        ignored and the initial values for the variance components are
        estimated from the data using MIVQUE0.

    lb : int
        The size of the array `b`.

    maxit : int, optional
        The maximum number of iterations.

        If `maxit` < 0, the default value of 100 is used.

        If `maxit` = 0, the parameter estimates (beta,nu) and
        corresponding standard errors are calculated based on the value
        of gamma_0 supplied in `gamma`.

    tol : float, optional
        The tolerance used to assess convergence.

        If `tol` <= 0.0, the default value of epsilon^0.7 is used, where
        epsilon is the machine precision.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    gamma : float, ndarray, shape (`nvpr`+2)
        `gamma`[i-1], for i = 1,2,...,g, holds the final estimate of
        sigma_i^2 and `gamma`[g] holds the final estimate for sigma_R^2.

    nff : int
        The number of fixed effects estimated (i.e., the number of
        columns, p, in the design matrix X).

    nrf : int
        The number of random effects estimated (i.e., the number of
        columns, q, in the design matrix Z).

    df : int
        The degrees of freedom.

    ml : float
        -2l_R(gamma^) where l_R is the log of the maximum likelihood
        calculated at gamma^, the estimated variance components returned
        in `gamma`.

    b : float, ndarray, shape (`lb`)
        The parameter estimates, (beta,nu), with the first `nff`
        elements of `b` containing the fixed effect parameter estimates,
        beta and the next `nrf` elements of `b` containing the random
        effect parameter estimates, nu.

        Fixed effects

        If `fint` = 1, `b`[0] contains the estimate of the fixed
        intercept.

        Let L_i denote the number of levels associated with the ith
        fixed variable, that is L_i = `levels`[ `fvid`[i-1] -1].

        Define

        - if `fint` = 1, F_1 = 2 else if `fint` = 0, F_1 = 1;

        - F_{i+1} = F_i+max(L_i-1,1), i >= 1.

        Then for i = 1,2,...,nfv:

        - if L_i > 1, `b`[ F_i+j-2 -1] contains the parameter estimate
          for the jth level of the ith fixed variable, for j =
          2,3,...,L_i;

        - if L_i <= 1, `b`[F_i-1] contains the parameter estimate for
          the ith fixed variable.

        Random effects

        Redefining L_i to denote the number of levels associated with
        the ith random variable, that is L_i = `levels`[ `rvid`[i-1]
        -1].

        Define

        - if `rint` = 1, R_1 = 2 else if `rint` = 0, R_1 = 1;

          R_{i+1} = R_i+L_i, i >= 1.

        Then for i = 1,2,...,nrv:

        - if `svid` = 0,

          - if L_i > 1, `b`[ `nff` + R_i +j-1 -1] contains the parameter
            estimate for the jth level of the ith random variable, for j
            = 1,2,...,L_i;

          - if L_i <= 1, `b`[ `nff` + R_i -1] contains the parameter
            estimate for the ith random variable;

        - if `svid` != 0,

          - let L_S denote the number of levels associated with the
            subject variable, that is L_S = `levels`[ `svid` -1];

          - if L_i > 1, `b`[ `nff` + (s-1) L_S + R_i + j - 1 -1]
            contains the parameter estimate for the interaction between
            the sth level of the subject variable and the jth level of
            the ith random variable, for j = 1,2,...,L_i, for s =
            1,2,...,L_S;

          - if L_i <= 1, `b`[ `nff` + (s-1) L_S + R_i -1] contains the
            parameter estimate for the interaction between the sth level
            of the subject variable and the ith random variable, for s =
            1,2,...,L_S;

          - if `rint` = 1, `b`[`nff`] contains the estimate of the
            random intercept.

    se : float, ndarray, shape (`lb`)
        The standard errors of the parameter estimates given in `b`.

    warn : int
        Is set to 1 if a variance component was estimated to be a
        negative value during the fitting process. Otherwise `warn` is
        set to 0.

        If `warn` = 1, the negative estimate is set to zero and the
        estimation process allowed to continue.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: number of observations with nonzero weights must
            be greater than one.

        (`errno` 1)
            On entry, ncol = *<value>*.

            Constraint: ncol >= 1.

        (`errno` 1)
            On entry, nfv = *<value>* and ncol = *<value>*.

            Constraint: 0 <= nfv < ncol.

        (`errno` 1)
            On entry, nrv = *<value>* and ncol = *<value>*.

            Constraint: 0 <= nrv < ncol and nrv+`rint` > 0.

        (`errno` 1)
            On entry, `nvpr` = *<value>* and nrv = *<value>*.

            Constraint: 0 <= `nvpr` <= nrv and (nrv <= 0 or `nvpr` >=
            1).

        (`errno` 1)
            On entry, `yvid` = *<value>* and ncol = *<value>*.

            Constraint: 1 <= `yvid` <= ncol.

        (`errno` 1)
            On entry, `svid` = *<value>* and ncol = *<value>*.

            Constraint: 0 <= `svid` <= ncol.

        (`errno` 1)
            On entry, `cwid` = *<value>* and ncol = *<value>*.

            Constraint: 0 <= `cwid` <= ncol and any supplied weights
            must be >= 0.0.

        (`errno` 1)
            On entry, `fint` = *<value>*.

            Constraint: `fint` = 0 or 1.

        (`errno` 1)
            On entry, `rint` = *<value>*.

            Constraint: `rint` = 0 or 1.

        (`errno` 1)
            On entry, `lb` too small: `lb` = *<value>*.

        (`errno` 2)
            On entry, `levels`[i] < 1, for at least one i.

        (`errno` 2)
            On entry, ncol = *<value>*.

            Constraint: 1 <= `fvid`[i] <= ncol, for all i.

        (`errno` 2)
            On entry, ncol = *<value>*.

            Constraint: 1 <= `rvid`[i] <= ncol, for all i.

        (`errno` 2)
            On entry, `nvpr` = *<value>*.

            Constraint: 1 <= `vpr`[i] <= `nvpr`, for all i.

        (`errno` 2)
            On entry, invalid data: categorical variable with value
            greater than that specified in `levels`.

        (`errno` 2)
            On entry, `gamma`[i] < 0, for at least one i.

        (`errno` 3)
            Degrees of freedom < 1: `df` = *<value>*.

        (`errno` 4)
            Routine failed to converge to specified tolerance: `tol` =
            *<value>*.

        (`errno` 4)
            Routine failed to converge in `maxit` iterations: `maxit` =
            *<value>*.
    """

    from ..base.correg import mixeff_ml as b_mixeff_ml

    fun_name = 'naginterfaces.library.correg.mixeff_ml'

    _dat = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'dat'},
        dat,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_dat, {'fun_name': fun_name, 'entity': 'dat'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _ncol_py = _b_utils._EngineFloat64ArrayType.get_shape(_dat, {'fun_name': fun_name, 'entity': 'dat'}, exp_rank=2)[1]
    _ncol = _b_utils._EngineIntScalarType(
        _ncol_py, {'fun_name': fun_name, 'entity': '_ncol'},
    ).to_ctypes()
    _levels = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'levels'},
        levels,
        copy=False,
    )
    _fvid = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'fvid'},
        fvid,
        copy=False,
    )
    _nfv_py = _b_utils._EngineIntArrayType.get_shape(_fvid, {'fun_name': fun_name, 'entity': 'fvid'}, exp_rank=1)[0]
    _nfv = _b_utils._EngineIntScalarType(
        _nfv_py, {'fun_name': fun_name, 'entity': '_nfv'},
    ).to_ctypes()
    _rvid = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'rvid'},
        rvid,
        copy=False,
    )
    _nrv_py = _b_utils._EngineIntArrayType.get_shape(_rvid, {'fun_name': fun_name, 'entity': 'rvid'}, exp_rank=1)[0]
    _nrv = _b_utils._EngineIntScalarType(
        _nrv_py, {'fun_name': fun_name, 'entity': '_nrv'},
    ).to_ctypes()
    _vpr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vpr'},
        vpr,
        copy=False,
    )
    _gamma = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'gamma'},
        gamma,
        copy=True,
    )
    _lb = _b_utils._EngineIntScalarType(
        lb, {'fun_name': fun_name, 'entity': 'lb'},
    ).to_ctypes()
    _lb_py = _b_utils._EngineIntScalarType.to_py(_lb)
    _b = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_lb_py,),
    )
    _se = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_lb_py,),
    )
    _return_scalars = b_mixeff_ml(
        _n,
        _ncol,
        _dat,
        _levels,
        yvid,
        cwid,
        _nfv,
        _fvid,
        fint,
        _nrv,
        _rvid,
        nvpr,
        _vpr,
        rint,
        svid,
        _gamma,
        _lb,
        _b,
        _se,
        maxit,
        tol,
        io_manager,
    )
    _gamma_py = _gamma
    _nff_py = _return_scalars.nff
    _nrf_py = _return_scalars.nrf
    _df_py = _return_scalars.df
    _ml_py = _return_scalars.ml
    _b_py = _b
    _se_py = _se
    _warn_py = _return_scalars.warn
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['gamma', 'nff', 'nrf', 'df', 'ml', 'b', 'se', 'warn'],
    )(
        _gamma_py,
        _nff_py,
        _nrf_py,
        _df_py,
        _ml_py,
        _b_py,
        _se_py,
        _warn_py,
    )
    return _return_data

@_b_utils._withdrawal(
    'naginterfaces.library.correg.mixeff_hier_init',
    mkwdrn=None,
    advice=[
        'Please use lmm_init instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def mixeff_hier_init(dat, levels, y, fixed, rndm, wt=None):
    u"""
    Hierarchical mixed effects regression, initialization function for
    ``mixeff_hier_reml`` and ``mixeff_hier_ml``.

    Note: ``mixeff_hier_init`` is deprecated.
    Please use ``lmm_init`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    g02jc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02jcf.html

    Parameters
    ----------
    dat : float, array-like, shape (n, ncol)
        A matrix of data, with `dat`[i-1,j-1] holding the ith
        observation on the jth variable. The two design matrices X and Z
        are constructed from `dat` and the information given in `fixed`
        (for X) and `rndm` (for Z).

    levels : int, array-like, shape (ncol)
        `levels`[i-1] contains the number of levels associated with the
        ith variable held in `dat`.

        If the ith variable is continuous or binary, then `levels`[i-1]
        must be set to 1.

        Otherwise the ith variable is assumed to take an integer value
        between 1 and `levels`[i-1], (i.e., the ith variable is discrete
        with `levels`[i-1] levels).

    y : float, array-like, shape (n)
        y, the vector of observations on the dependent variable.

    fixed : int, array-like, shape (lfixed)
        Defines the structure of the fixed effects design matrix, X.

        `fixed`[0]

          The number of variables, N_F, to include as fixed effects (not
          including the intercept if present).

        `fixed`[1]

          The fixed intercept flag which must contain 1 if a fixed
          intercept is to be included and 0 otherwise.

        `fixed`[2+i-1]

          The column of `dat` holding the ith fixed variable, for i =
          1,2,...,`fixed`[0].

        See [Construction of the fixed effects design matrix, XX] for
        more details on the construction of X.

    rndm : int, array-like, shape (:, nrndm)
        `rndm`[i-1,j-1] defines the structure of the random effects
        design matrix, Z. The bth column of `rndm` defines a block of
        columns in the design matrix Z.

        `rndm`[0,b-1]

          The number of variables, N_R_b, to include as random effects
          in the bth block (not including the random intercept if
          present).

        `rndm`[1,b-1]

          The random intercept flag which must contain 1 if block b
          includes a random intercept and 0 otherwise.

        `rndm`[2+i-1,b-1]

          The column of `dat` holding the ith random variable in the bth
          block, for i = 1,2,...,`rndm`[0,b-1].

        `rndm`[3+N_R_b-1,b-1]

          The number of subject variables, N_S_b, for the bth block. The
          subject variables define the nesting structure for this block.

        `rndm`[3+N_R_b+i-1,b-1]

          The column of `dat` holding the ith subject variable in the
          bth block, for i = 1,2,...,`rndm`[3+N_R_b-1,b-1].

        See [Construction of random effects design matrix, ZZ] for more
        details on the construction of Z.

    wt : None or float, array-like, shape (:), optional
        Note: the required length for this argument is determined as
        follows: if `wt` is not None: n; otherwise: 0.

        Optionally, the weights to be used in the weighted regression.

        If `wt`[i-1] = 0.0, the ith observation is not included in the
        model, in which case the effective number of observations is the
        number of observations with nonzero weights.

        If weights are not provided then `wt` must be set to None and
        the effective number of observations is n.

    Returns
    -------
    nff : int
        p, the number of fixed effects estimated, i.e., the number of
        columns in the design matrix X.

    nlsv : int
        The number of levels for the overall subject variable (see
        [Construction of random effects design matrix, ZZ] for a
        description of what this means). If there is no overall subject
        variable, `nlsv` = 1.

    nrf : int
        The number of random effects estimated in each of the overall
        subject blocks. The number of columns in the design matrix Z is
        given by q = `nrf`*`nlsv`.

    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 3)
            On entry, ncol = *<value>*.

            Constraint: ncol >= 0.

        (`errno` 4)
            On entry, variable j of observation i is less than 1 or
            greater than `levels`[j-1]: i = *<value>*, j = *<value>*,
            value = *<value>*, `levels`[j-1] = *<value>*.

        (`errno` 6)
            On entry, `levels`[*<value>*] = *<value>*.

            Constraint: `levels`[i-1] >= 1.

        (`errno` 8)
            On entry, `wt`[*<value>*] = *<value>*.

            Constraint: `wt`[i-1] >= 0.0.

        (`errno` 9)
            On entry, number of fixed parameters, *<value>* is less than
            zero.

        (`errno` 10)
            On entry, lfixed = *<value>*.

            Constraint: lfixed >= *<value>*.

        (`errno` 11)
            On entry, nrndm = *<value>*.

            Constraint: nrndm > 0.

        (`errno` 12)
            On entry, number of random parameters for random statement i
            is less than 0: i = *<value>*, number of parameters =
            *<value>*.

        (`errno` 102)
            On entry, more fixed factors than observations, n =
            *<value>*.

            Constraint: n >= *<value>*.

        (`errno` 108)
            On entry, no observations due to zero weights.

        (`errno` 109)
            On entry, invalid value for fixed intercept flag: value =
            *<value>*.

        (`errno` 112)
            On entry, invalid value for random intercept flag for random
            statement i: i = *<value>*, value = *<value>*.

        (`errno` 209)
            On entry, index of fixed variable j is less than 1 or
            greater than ncol: j = *<value>*, index = *<value>* and ncol
            = *<value>*.

        (`errno` 212)
            On entry, must be at least one parameter, or an intercept in
            each random statement i: i = *<value>*.

        (`errno` 312)
            On entry, index of random variable j in random statement i
            is less than 1 or greater than ncol: i = *<value>*, j =
            *<value>*, index = *<value>* and ncol = *<value>*.

        (`errno` 412)
            On entry, number of subject parameters for random statement
            i is less than 0: i = *<value>*, number of parameters =
            *<value>*.

        (`errno` 512)
            On entry, nesting variable j in random statement i has one
            level: j = *<value>*, i = *<value>*.
    """

    from ..base.correg import mixeff_hier_init as b_mixeff_hier_init

    fun_name = 'naginterfaces.library.correg.mixeff_hier_init'

    _dat = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'dat'},
        dat,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_dat, {'fun_name': fun_name, 'entity': 'dat'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _ncol_py = _b_utils._EngineFloat64ArrayType.get_shape(_dat, {'fun_name': fun_name, 'entity': 'dat'}, exp_rank=2)[1]
    _ncol = _b_utils._EngineIntScalarType(
        _ncol_py, {'fun_name': fun_name, 'entity': '_ncol'},
    ).to_ctypes()
    _levels = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'levels'},
        levels,
        copy=False,
    )
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _wt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'wt'},
        wt,
        copy=False,
        allow_none=True,
    )
    _fixed = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'fixed'},
        fixed,
        copy=False,
    )
    _lfixed_py = _b_utils._EngineIntArrayType.get_shape(_fixed, {'fun_name': fun_name, 'entity': 'fixed'}, exp_rank=1)[0]
    _lfixed = _b_utils._EngineIntScalarType(
        _lfixed_py, {'fun_name': fun_name, 'entity': '_lfixed'},
    ).to_ctypes()
    _rndm = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'rndm'},
        rndm,
        copy=False,
    )
    _nrndm_py = _b_utils._EngineIntArrayType.get_shape(_rndm, {'fun_name': fun_name, 'entity': 'rndm'}, exp_rank=2)[1]
    _nrndm = _b_utils._EngineIntScalarType(
        _nrndm_py, {'fun_name': fun_name, 'entity': '_nrndm'},
    ).to_ctypes()
    _return_scalars = b_mixeff_hier_init(
        _n,
        _ncol,
        _dat,
        _levels,
        _y,
        _wt,
        _fixed,
        _lfixed,
        _nrndm,
        _rndm,
    )
    _nff_py = _return_scalars.nff
    _nlsv_py = _return_scalars.nlsv
    _nrf_py = _return_scalars.nrf
    _comm_py = _return_scalars.comm
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['nff', 'nlsv', 'nrf', 'comm'],
    )(
        _nff_py,
        _nlsv_py,
        _nrf_py,
        _comm_py,
    )
    return _return_data

@_b_utils._withdrawal(
    'naginterfaces.library.correg.mixeff_hier_reml',
    mkwdrn=None,
    advice=[
        'Please use lmm_fit instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def mixeff_hier_reml(vpr, gamma, comm, iopt=None, ropt=None, io_manager=None):
    u"""
    Hierarchical mixed effects regression using Restricted Maximum
    Likelihood (REML).

    Note: ``mixeff_hier_reml`` is deprecated.
    Please use ``lmm_fit`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    g02jd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02jdf.html

    Parameters
    ----------
    vpr : int, array-like, shape (lvpr)
        A vector of flags indicating the mapping between the random
        variables specified in rndm and the variance components,
        sigma_i^2. See [Further Comments] for more details.

    gamma : float, array-like, shape (nvpr+1)
        Holds the initial values of the variance components, gamma_0,
        with `gamma`[i-1] the initial value for sigma_i^2/sigma_R^2, for
        i = 1,2,...,nvpr.

        If `gamma`[0] = -1.0, the remaining elements of `gamma` are
        ignored and the initial values for the variance components are
        estimated from the data using MIVQUE0.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``mixeff_hier_init``.

    iopt : None or int, array-like, shape (liopt), optional
        Options passed to the optimization function.

        By default ``mixeff_hier_reml`` fits the specified model using a
        modified Newton optimization algorithm as implemented in
        ``opt.bounds_mod_deriv2_comp``. In some cases, where the
        calculation of the derivatives is computationally expensive it
        may be more efficient to use a sequential QP algorithm.

        The sequential QP algorithm as implemented in ``opt.nlp1_solve``
        can be chosen by setting `iopt`[4] = 1.

        If liopt < 5 or `iopt`[4] != 1, then the modified Newton
        algorithm will be used.

        Different options are available depending on the optimization
        function used.

        In all cases, using a value of -1 will cause the default value
        to be used.

        In addition only the first liopt values of `iopt` are used, so
        for example, if only the first element of `iopt` needs changing
        and default values for all other options are sufficient liopt
        can be set to 1.

        The following table lists the association between elements of
        `iopt` and arguments in the optimizer when the modified Newton
        algorithm is being used.

        [table omitted]

        If requested, monitoring information is displayed in a similar
        format to that given by the modified Newton optimizer.

        The following table lists the association between elements of
        `iopt` and options in the optimizer when the sequential QP
        algorithm is being used.

        [table omitted]

        If liopt <= 0, default values are used for all options and
        `iopt` is not referenced.

    ropt : None or float, array-like, shape (lropt), optional
        Options passed to the optimization function.

        Different options are available depending on the optimization
        function used.

        In all cases, using a value of -1.0 will cause the default value
        to be used.

        In addition only the first lropt values of `ropt` are used, so
        for example, if only the first element of `ropt` needs changing
        and default values for all other options are sufficient lropt
        can be set to 1.

        The following table lists the association between elements of
        `ropt` and arguments in the optimizer when the modified Newton
        algorithm is being used.

        [table omitted]

        The following table lists the association between elements of
        `ropt` and options in the optimizer when the sequential QP
        algorithm is being used.

        [table omitted]

        where eps is the machine precision returned by
        ``machine.precision`` and zz_{ii} denotes the i diagonal element
        of Z^TZ.

        If lropt <= 0, then default values are used for all options and
        `ropt` may be set to None.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    gamma : float, ndarray, shape (nvpr+1)
        `gamma`[i-1], for i = 1,2,...,nvpr, holds the final estimate of
        sigma_i^2 and `gamma`[nvpr] holds the final estimate for
        sigma_R^2.

    effn : int
        Effective number of observations. If there are no weights (i.e.,
        weight = 'U'), or all weights are nonzero, `effn` = n.

    rnkx : int
        The rank of the design matrix, X, for the fixed effects.

    ncov : int
        Number of variance components not estimated to be zero. If none
        of the variance components are estimated to be zero, then `ncov`
        = nvpr.

    lnlike : float
        -2l_R(gamma^) where l_R is the log of the restricted maximum
        likelihood calculated at gamma^, the estimated variance
        components returned in `gamma`.

    estid : int, ndarray, shape (:, lb)
        An array describing the parameter estimates returned in `b`. The
        first nlsv*nrf columns of `estid` describe the parameter
        estimates for the random effects and the last nff columns the
        parameter estimates for the fixed effects.

    b : float, ndarray, shape (nff+nrf*nlsv)
        The parameter estimates, with the first nrf*nlsv elements of `b`
        containing the parameter estimates for the random effects, nu,
        and the remaining nff elements containing the parameter
        estimates for the fixed effects, beta. The order of these
        estimates are described by the `estid` argument.

    se : float, ndarray, shape (nff+nrf*nlsv)
        The standard errors of the parameter estimates given in `b`.

    czz : float, ndarray, shape (nrf, :)
        If nlsv = 1, then `czz` holds the lower triangular portion of
        the matrix (1/sigma^2)(Z^TR^^-1Z+G^^-1), where R^ and G^ are the
        estimates of R and G respectively. If nlsv > 1, then `czz` holds
        this matrix in compressed form, with the first nrf columns
        holding the part of the matrix corresponding to the first level
        of the overall subject variable, the next nrf columns the part
        corresponding to the second level of the overall subject
        variable etc.

    cxx : float, ndarray, shape (nff, :)
        `cxx` holds the lower triangular portion of the matrix
        (1/sigma^2)X^TV^^-1X, where V^ is the estimated value of V.

    cxz : float, ndarray, shape (nff, :)
        If nlsv = 1, then `cxz` holds the matrix
        (1/sigma^2)(X^TV^^-1Z)G^, where V^ and G^ are the estimates of V
        and G respectively. If nlsv > 1, then `cxz` holds this matrix in
        compressed form, with the first nrf columns holding the part of
        the matrix corresponding to the first level of the overall
        subject variable, the next nrf columns the part corresponding to
        the second level of the overall subject variable etc.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, lvpr = *<value>*.

            Constraint: lvpr >= *<value>*.

        (`errno` 2)
            On entry, `vpr`[*<value>*] = *<value>* and nvpr = *<value>*.

            Constraint: 1 <= `vpr`[i] <= nvpr.

        (`errno` 3)
            On entry, nvpr = *<value>*.

            Constraint: 1 <= nvpr <= *<value>*.

        (`errno` 4)
            On entry, `gamma`[*<value>*] = *<value>*.

            Constraint: `gamma`[0] = -1.0 or `gamma`[i-1] >= 0.0.

        (`errno` 9)
            On entry, lb = *<value>*.

            Constraint: lb >= *<value>*.

        (`errno` 11)
            On entry, ldid = *<value>*.

            Constraint: ldid >= *<value>*.

        (`errno` 21)
            On entry, `comm`['icomm'] has not been initialized
            correctly.

        (`errno` 32)
            On entry, at least one value of i, for i = 1,2,...,nvpr,
            does not appear in `vpr`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 101)
            Optimal solution found, but requested accuracy not achieved.

        (`errno` 102)
            Too many major iterations.

        (`errno` 103)
            Current point cannot be improved upon.

        (`errno` 104)
            At least one negative estimate for `gamma` was obtained. All
            negative estimates have been set to zero.
    """

    from ..base.correg import mixeff_hier_reml as b_mixeff_hier_reml

    fun_name = 'naginterfaces.library.correg.mixeff_hier_reml'

    _vpr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vpr'},
        vpr,
        copy=False,
    )
    _lvpr_py = _b_utils._EngineIntArrayType.get_shape(_vpr, {'fun_name': fun_name, 'entity': 'vpr'}, exp_rank=1)[0]
    _lvpr = _b_utils._EngineIntScalarType(
        _lvpr_py, {'fun_name': fun_name, 'entity': '_lvpr'},
    ).to_ctypes()
    _gamma = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'gamma'},
        gamma,
        copy=True,
    )
    _nvpr = _b_utils._EngineIntScalarType(
        _b_utils._EngineFloat64ArrayType.get_shape(_gamma, {'fun_name': fun_name, 'entity': 'gamma'}, exp_rank=1)[0]-1, {'fun_name': fun_name, 'entity': '_nvpr'},
    ).to_ctypes()
    _b_utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('rcomm', 'icomm'),
    )
    _vl_g02jcf_nff_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (5,), validate=False)
    _vl_g02jcf_nrf_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (6,), validate=False)
    _vl_g02jcf_nlsv_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (7,), validate=False)
    _lb = _b_utils._EngineIntScalarType(
        _vl_g02jcf_nff_py+_vl_g02jcf_nrf_py*_vl_g02jcf_nlsv_py, {'fun_name': fun_name, 'entity': '_lb'},
    ).to_ctypes()
    _vl_vlicomm_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (0,), validate=False)
    if _vl_vlicomm_py >= 5:
        _ldid = _b_utils._EngineIntScalarType(
            _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (4,), validate=False), {'fun_name': fun_name, 'entity': '_ldid'},
        ).to_ctypes()
    else:
        _ldid = 0
        _ldid_py = _ldid
    _lb_py = _b_utils._EngineIntScalarType.to_py(_lb)
    if _vl_vlicomm_py >= 5:
        _estid_dim1 = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (4,), validate=False)
    else:
        _estid_dim1 = 0
    exp_shape_estid = (_estid_dim1, _lb_py)
    _estid = _b_utils._EngineIntArrayType.empty_ndarray(
        exp_shape_estid,
        sorder=_b_utils._EngineData.default_sorder,
    )
    _b = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_g02jcf_nff_py+_vl_g02jcf_nrf_py*_vl_g02jcf_nlsv_py,),
    )
    _se = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_g02jcf_nff_py+_vl_g02jcf_nrf_py*_vl_g02jcf_nlsv_py,),
    )
    if _vl_vlicomm_py >= 8:
        _czz_dim2 = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (6,), validate=False)*_b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (7,), validate=False)
    else:
        _czz_dim2 = 0
    exp_shape_czz = (_vl_g02jcf_nrf_py, _czz_dim2)
    _czz = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_czz,
        sorder=_b_utils._EngineData.default_sorder,
    )
    if _vl_vlicomm_py >= 6:
        _cxx_dim2 = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (5,), validate=False)
    else:
        _cxx_dim2 = 0
    exp_shape_cxx = (_vl_g02jcf_nff_py, _cxx_dim2)
    _cxx = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_cxx,
        sorder=_b_utils._EngineData.default_sorder,
    )
    if _vl_vlicomm_py >= 8:
        _cxz_dim2 = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (6,), validate=False)*_b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (7,), validate=False)
    else:
        _cxz_dim2 = 0
    exp_shape_cxz = (_vl_g02jcf_nff_py, _cxz_dim2)
    _cxz = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_cxz,
        sorder=_b_utils._EngineData.default_sorder,
    )
    _iopt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'iopt'},
        iopt,
        copy=False,
        allow_none=True,
    )
    _liopt_py = _b_utils._EngineIntArrayType.get_shape(_iopt, {'fun_name': fun_name, 'entity': 'iopt'}, exp_rank=1, allow_none=True)[0]
    _liopt = _b_utils._EngineIntScalarType(
        _liopt_py, {'fun_name': fun_name, 'entity': '_liopt'},
    ).to_ctypes()
    _ropt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ropt'},
        ropt,
        copy=False,
        allow_none=True,
    )
    _lropt_py = _b_utils._EngineFloat64ArrayType.get_shape(_ropt, {'fun_name': fun_name, 'entity': 'ropt'}, exp_rank=1, allow_none=True)[0]
    _lropt = _b_utils._EngineIntScalarType(
        _lropt_py, {'fun_name': fun_name, 'entity': '_lropt'},
    ).to_ctypes()
    try:
        _return_scalars = b_mixeff_hier_reml(
            _lvpr,
            _vpr,
            _nvpr,
            _gamma,
            _lb,
            _estid,
            _ldid,
            _b,
            _se,
            _czz,
            _cxx,
            _cxz,
            comm,
            _iopt,
            _liopt,
            _ropt,
            _lropt,
            io_manager,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _gamma_py = _gamma
    _effn_py = _return_scalars.effn
    _rnkx_py = _return_scalars.rnkx
    _ncov_py = _return_scalars.ncov
    _lnlike_py = _return_scalars.lnlike
    _estid_py = _estid
    _b_py = _b
    _se_py = _se
    _czz_py = _czz
    _cxx_py = _cxx
    _cxz_py = _cxz
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['gamma', 'effn', 'rnkx', 'ncov', 'lnlike', 'estid', 'b', 'se', 'czz', 'cxx', 'cxz'],
    )(
        _gamma_py,
        _effn_py,
        _rnkx_py,
        _ncov_py,
        _lnlike_py,
        _estid_py,
        _b_py,
        _se_py,
        _czz_py,
        _cxx_py,
        _cxz_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

@_b_utils._withdrawal(
    'naginterfaces.library.correg.mixeff_hier_ml',
    mkwdrn=None,
    advice=[
        'Please use lmm_fit instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def mixeff_hier_ml(vpr, nvpr, gamma, comm, iopt=None, ropt=None, io_manager=None):
    u"""
    Hierarchical mixed effects regression using Maximum Likelihood (ML).

    Note: ``mixeff_hier_ml`` is deprecated.
    Please use ``lmm_fit`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    g02je

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02jef.html

    Parameters
    ----------
    vpr : int, array-like, shape (lvpr)
        A vector of flags indicating the mapping between the random
        variables specified in rndm and the variance components,
        sigma_i^2. See [Further Comments] for more details.

    nvpr : int
        g, the number of variance components being estimated (excluding
        the overall variance, sigma_R^2).

    gamma : float, array-like, shape (`nvpr`+1)
        Holds the initial values of the variance components, gamma_0,
        with `gamma`[i-1] the initial value for sigma_i^2/sigma_R^2, for
        i = 1,2,...,`nvpr`.

        If `gamma`[0] = -1.0, the remaining elements of `gamma` are
        ignored and the initial values for the variance components are
        estimated from the data using MIVQUE0.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``mixeff_hier_init``.

    iopt : None or int, array-like, shape (liopt), optional
        Options passed to the optimization function.

        By default ``mixeff_hier_ml`` fits the specified model using a
        modified Newton optimization algorithm as implemented in
        ``opt.bounds_mod_deriv2_comp``. In some cases, where the
        calculation of the derivatives is computationally expensive it
        may be more efficient to use a sequential QP algorithm.

        The sequential QP algorithm as implemented in ``opt.nlp1_solve``
        can be chosen by setting `iopt`[4] = 1.

        If liopt < 5 or `iopt`[4] != 1, then the modified Newton
        algorithm will be used.

        Different options are available depending on the optimization
        function used.

        In all cases, using a value of -1 will cause the default value
        to be used.

        In addition only the first liopt values of `iopt` are used, so
        for example, if only the first element of `iopt` needs changing
        and default values for all other options are sufficient liopt
        can be set to 1.

        The following table lists the association between elements of
        `iopt` and arguments in the optimizer when the modified Newton
        algorithm is being used.

        [table omitted]

        If requested, monitoring information is displayed in a similar
        format to that given by the modified Newton optimizer.

        The following table lists the association between elements of
        `iopt` and options in the optimizer when the sequential QP
        algorithm is being used.

        [table omitted]

        If liopt <= 0, default values are used for all options and
        `iopt` is not referenced.

    ropt : None or float, array-like, shape (lropt), optional
        Options passed to the optimization function.

        Different options are available depending on the optimization
        function used.

        In all cases, using a value of -1.0 will cause the default value
        to be used.

        In addition only the first lropt values of `ropt` are used, so
        for example, if only the first element of `ropt` needs changing
        and default values for all other options are sufficient lropt
        can be set to 1.

        The following table lists the association between elements of
        `ropt` and arguments in the optimizer when the modified Newton
        algorithm is being used.

        [table omitted]

        The following table lists the association between elements of
        `ropt` and options in the optimizer when the sequential QP
        algorithm is being used.

        [table omitted]

        where eps is the machine precision returned by
        ``machine.precision`` and zz_{ii} denotes the i diagonal element
        of Z^TZ.

        If lropt <= 0, then default values are used for all options and
        `ropt` may be set to None.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    gamma : float, ndarray, shape (`nvpr`+1)
        `gamma`[i-1], for i = 1,2,...,`nvpr`, holds the final estimate
        of sigma_i^2 and `gamma`[`nvpr`] holds the final estimate for
        sigma_R^2.

    effn : int
        Effective number of observations. If there are no weights (i.e.,
        weight = 'U'), or all weights are nonzero, `effn` = n.

    rnkx : int
        The rank of the design matrix, X, for the fixed effects.

    ncov : int
        Number of variance components not estimated to be zero. If none
        of the variance components are estimated to be zero, then `ncov`
        = `nvpr`.

    lnlike : float
        -2l_R(gamma^) where l_R is the log of the maximum likelihood
        calculated at gamma^, the estimated variance components returned
        in `gamma`.

    estid : int, ndarray, shape (:, lb)
        An array describing the parameter estimates returned in `b`. The
        first nlsv*nrf columns of `estid` describe the parameter
        estimates for the random effects and the last nff columns the
        parameter estimates for the fixed effects.

    b : float, ndarray, shape (nff+nrf*nlsv)
        The parameter estimates, with the first nrf*nlsv elements of `b`
        containing the parameter estimates for the random effects, nu,
        and the remaining nff elements containing the parameter
        estimates for the fixed effects, beta. The order of these
        estimates are described by the `estid` argument.

    se : float, ndarray, shape (nff+nrf*nlsv)
        The standard errors of the parameter estimates given in `b`.

    czz : float, ndarray, shape (nrf, :)
        If nlsv = 1, then `czz` holds the lower triangular portion of
        the matrix (1/sigma^2)(Z^TR^^-1Z+G^^-1), where R^ and G^ are the
        estimates of R and G respectively. If nlsv > 1, then `czz` holds
        this matrix in compressed form, with the first nrf columns
        holding the part of the matrix corresponding to the first level
        of the overall subject variable, the next nrf columns the part
        corresponding to the second level of the overall subject
        variable etc.

    cxx : float, ndarray, shape (nff, :)
        `cxx` holds the lower triangular portion of the matrix
        (1/sigma^2)X^TV^^-1X, where V^ is the estimated value of V.

    cxz : float, ndarray, shape (nff, :)
        If nlsv = 1, then `cxz` holds the matrix
        (1/sigma^2)(X^TV^^-1Z)G^, where V^ and G^ are the estimates of V
        and G respectively. If nlsv > 1, then `cxz` holds this matrix in
        compressed form, with the first nrf columns holding the part of
        the matrix corresponding to the first level of the overall
        subject variable, the next nrf columns the part corresponding to
        the second level of the overall subject variable etc.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, lvpr = *<value>*.

            Constraint: lvpr >= *<value>*.

        (`errno` 2)
            On entry, `vpr`[*<value>*] = *<value>* and `nvpr` =
            *<value>*.

            Constraint: 1 <= `vpr`[i] <= `nvpr`.

        (`errno` 3)
            On entry, `nvpr` = *<value>*.

            Constraint: 1 <= `nvpr` <= *<value>*.

        (`errno` 4)
            On entry, `gamma`[*<value>*] = *<value>*.

            Constraint: `gamma`[0] = -1.0 or `gamma`[i-1] >= 0.0.

        (`errno` 9)
            On entry, lb = *<value>*.

            Constraint: lb >= *<value>*.

        (`errno` 11)
            On entry, ldid = *<value>*.

            Constraint: ldid >= *<value>*.

        (`errno` 21)
            On entry, `comm`['icomm'] has not been initialized
            correctly.

        (`errno` 32)
            On entry, at least one value of i, for i = 1,2,...,`nvpr`,
            does not appear in `vpr`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 101)
            Optimal solution found, but requested accuracy not achieved.

        (`errno` 102)
            Too many major iterations.

        (`errno` 103)
            Current point cannot be improved upon.

        (`errno` 104)
            At least one negative estimate for `gamma` was obtained. All
            negative estimates have been set to zero.
    """

    from ..base.correg import mixeff_hier_ml as b_mixeff_hier_ml

    fun_name = 'naginterfaces.library.correg.mixeff_hier_ml'

    _vpr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vpr'},
        vpr,
        copy=False,
    )
    _lvpr_py = _b_utils._EngineIntArrayType.get_shape(_vpr, {'fun_name': fun_name, 'entity': 'vpr'}, exp_rank=1)[0]
    _lvpr = _b_utils._EngineIntScalarType(
        _lvpr_py, {'fun_name': fun_name, 'entity': '_lvpr'},
    ).to_ctypes()
    _gamma = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'gamma'},
        gamma,
        copy=True,
    )
    _b_utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('rcomm', 'icomm'),
    )
    _vl_g02jcf_nff_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (5,), validate=False)
    _vl_g02jcf_nrf_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (6,), validate=False)
    _vl_g02jcf_nlsv_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (7,), validate=False)
    _lb = _b_utils._EngineIntScalarType(
        _vl_g02jcf_nff_py+_vl_g02jcf_nrf_py*_vl_g02jcf_nlsv_py, {'fun_name': fun_name, 'entity': '_lb'},
    ).to_ctypes()
    _vl_vlicomm_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (0,), validate=False)
    if _vl_vlicomm_py >= 5:
        _ldid = _b_utils._EngineIntScalarType(
            _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (4,), validate=False), {'fun_name': fun_name, 'entity': '_ldid'},
        ).to_ctypes()
    else:
        _ldid = 0
        _ldid_py = _ldid
    _lb_py = _b_utils._EngineIntScalarType.to_py(_lb)
    if _vl_vlicomm_py >= 5:
        _estid_dim1 = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (4,), validate=False)
    else:
        _estid_dim1 = 0
    exp_shape_estid = (_estid_dim1, _lb_py)
    _estid = _b_utils._EngineIntArrayType.empty_ndarray(
        exp_shape_estid,
        sorder=_b_utils._EngineData.default_sorder,
    )
    _b = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_g02jcf_nff_py+_vl_g02jcf_nrf_py*_vl_g02jcf_nlsv_py,),
    )
    _se = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_g02jcf_nff_py+_vl_g02jcf_nrf_py*_vl_g02jcf_nlsv_py,),
    )
    if _vl_vlicomm_py >= 8:
        _czz_dim2 = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (6,), validate=False)*_b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (7,), validate=False)
    else:
        _czz_dim2 = 0
    exp_shape_czz = (_vl_g02jcf_nrf_py, _czz_dim2)
    _czz = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_czz,
        sorder=_b_utils._EngineData.default_sorder,
    )
    if _vl_vlicomm_py >= 6:
        _cxx_dim2 = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (5,), validate=False)
    else:
        _cxx_dim2 = 0
    exp_shape_cxx = (_vl_g02jcf_nff_py, _cxx_dim2)
    _cxx = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_cxx,
        sorder=_b_utils._EngineData.default_sorder,
    )
    if _vl_vlicomm_py >= 8:
        _cxz_dim2 = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (6,), validate=False)*_b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (7,), validate=False)
    else:
        _cxz_dim2 = 0
    exp_shape_cxz = (_vl_g02jcf_nff_py, _cxz_dim2)
    _cxz = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_cxz,
        sorder=_b_utils._EngineData.default_sorder,
    )
    _iopt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'iopt'},
        iopt,
        copy=False,
        allow_none=True,
    )
    _liopt_py = _b_utils._EngineIntArrayType.get_shape(_iopt, {'fun_name': fun_name, 'entity': 'iopt'}, exp_rank=1, allow_none=True)[0]
    _liopt = _b_utils._EngineIntScalarType(
        _liopt_py, {'fun_name': fun_name, 'entity': '_liopt'},
    ).to_ctypes()
    _ropt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ropt'},
        ropt,
        copy=False,
        allow_none=True,
    )
    _lropt_py = _b_utils._EngineFloat64ArrayType.get_shape(_ropt, {'fun_name': fun_name, 'entity': 'ropt'}, exp_rank=1, allow_none=True)[0]
    _lropt = _b_utils._EngineIntScalarType(
        _lropt_py, {'fun_name': fun_name, 'entity': '_lropt'},
    ).to_ctypes()
    try:
        _return_scalars = b_mixeff_hier_ml(
            _lvpr,
            _vpr,
            nvpr,
            _gamma,
            _lb,
            _estid,
            _ldid,
            _b,
            _se,
            _czz,
            _cxx,
            _cxz,
            comm,
            _iopt,
            _liopt,
            _ropt,
            _lropt,
            io_manager,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _gamma_py = _gamma
    _effn_py = _return_scalars.effn
    _rnkx_py = _return_scalars.rnkx
    _ncov_py = _return_scalars.ncov
    _lnlike_py = _return_scalars.lnlike
    _estid_py = _estid
    _b_py = _b
    _se_py = _se
    _czz_py = _czz
    _cxx_py = _cxx
    _cxz_py = _cxz
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['gamma', 'effn', 'rnkx', 'ncov', 'lnlike', 'estid', 'b', 'se', 'czz', 'cxx', 'cxz'],
    )(
        _gamma_py,
        _effn_py,
        _rnkx_py,
        _ncov_py,
        _lnlike_py,
        _estid_py,
        _b_py,
        _se_py,
        _czz_py,
        _cxx_py,
        _cxz_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def lmm_init(hlmm, hddesc, hfixed, y, dat, hrndm=None, wt=None):
    u"""
    Linear mixed effects regression, initialization function for
    ``lmm_fit``.

    Note: this function uses optional algorithmic parameters, see also:
    ``blgm.optset``, ``blgm.optget``, ``lmm_fit``.

    For full information please refer to the NAG Library document for
    g02jf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02jff.html

    Parameters
    ----------
    hlmm : Handle, modified in place
        On entry: must be set to a null Handle or, alternatively, an
        existing G22 handle may be supplied in which case ``lmm_init``
        will destroy the supplied G22 handle as if ``blgm.handle_free``
        had been called.

        On exit: holds a G22 handle to the internal data structure
        containing a description of the model. You must not change the
        G22 handle other than through the functions in submodule
        ``correg`` or submodule ``blgm``.

    hddesc : Handle
        A G22 handle to the internal data structure containing a
        description of the data matrix, D, as returned in hddesc by
        ``blgm.lm_describe_data``.

    hfixed : Handle
        A G22 handle to the internal data structure containing a
        description of the fixed part of the model M_f as returned in
        hform by ``blgm.lm_formula``.

        If `hfixed` is a null Handle then the model is assumed to not
        have a fixed part.

    y : float, array-like, shape (n)
        y, the vector of observations on the dependent variable.

    dat : float, array-like, shape (:, :)
        The data matrix, D. By default, D_{ij}, the ith value for the
        jth variable, for j = 1,2,...,m_d, for i = 1,2,...,n, should be
        supplied in `dat`[i-1,j-1].

        If the option 'Storage Order', described in
        ``blgm.lm_describe_data``, is set to 'VAROBS', D_{ij} should be
        supplied in `dat`[j-1,i-1].

        If either y_i, w_i or D_{ij}, for a variable j used in the
        model, is NaN (Not A Number) then that value is treated as
        missing and the whole observation is excluded from the analysis.

    hrndm : None or Handle, list, shape (nrndm), optional
        A series of G22 handles to internal data structures containing a
        description of the random part of the model M_r as returned in
        hform by ``blgm.lm_formula``.

    wt : None or float, array-like, shape (n), optional
        Optionally, the diagonal elements of the weight matrix W_c.

        If `wt`[i-1] = 0.0, the ith observation is not included in the
        model and the effective number of observations is the number of
        observations with nonzero weights.

        If weights are not provided then `wt` must be set to None, and
        the effective number of observations is n.

    Returns
    -------
    fnlsv : int
        The number of levels for the overall subject variable in M_f. If
        there is no overall subject variable, `fnlsv` = 1.

    nff : int
        The number of fixed effects estimated in each of the `fnlsv`
        subject blocks. The number of columns, p, in the design matrix X
        is given by p = `nff`*`fnlsv`.

    rnlsv : int
        The number of levels for the overall subject variable in M_r. If
        there is no overall subject variable, `rnlsv` = 1.

    nrf : int
        The number of random effects estimated in each of the `rnlsv`
        subject blocks. The number of columns, q, in the design matrix Z
        is given by q = `nrf`*`rnlsv`.

    nvpr : int
        g, the number of variance components being estimated (excluding
        the overall variance, sigma_R^2). This is defined by the number
        of terms in the random part of the model, M_r (see [Algorithmic
        Details] for details).

    comm : dict, communication object
        Communication structure.

    Other Parameters
    ----------------
    'Gamma Lower Bound' : float
        Default = sqrt(machine precision)/100

        A lower bound for the elements of gamma*, where gamma* =
        gamma/sigma_R^2.

    'Gamma Upper Bound' : float
        Default = 10^20

        An upper bound for the elements of gamma*, where gamma* =
        gamma/sigma_R^2.

    'Initial Distance' : float
        Default = 100000.0

        The initial distance from the solution.

        - When 'Solver' = 'E04LB', ``lmm_fit`` passes 'Initial Distance'
          to the solver as stepmx.

        - When 'Solver' = 'E04UC', this option is ignored.

    'Initial Value Strategy' : int
        Default = special

        Controls how ``lmm_fit`` will choose the initial values for the
        variance components, gamma, if not supplied.

        'Initial Value Strategy' = 0
          The MIVQUE0 estimates of the variance components based on the
          likelihood specified by 'Likelihood' are used.

        'Initial Value Strategy' = 1
          The MIVQUE0 estimates based on the maximum likelihood are
          used, irrespective of the value of 'Likelihood'.

        See Rao for a description of the minimum variance quadratic
        unbiased estimators (MIVQUE0).

        By default, for small problems, 'Initial Value Strategy' = 0 and
        for large problems 'Initial Value Strategy' = 1.

    'Likelihood' : str
        Default = 'REML'

        'Likelihood' defines whether ``lmm_fit`` will use the restricted
        maximum likelihood (REML) or the maximum likelihood (ML) when
        fitting the model.

    'Linear Minimization Accuracy' : float
        Default = 0.9

        The accuracy of the linear minimizations.

        - When 'Solver' = 'E04LB', ``lmm_fit`` passes 'Linear
          Minimization Accuracy' to the solver as eta.

        - When 'Solver' = 'E04UC', this option is ignored.

    'Line Search Tolerance' : float
        Default = 0.9

        The line search tolerance.

        - When 'Solver' = 'E04LB', this option is ignored.

        - When 'Solver' = 'E04UC', ``lmm_fit`` passes 'Line Search
          Tolerance' to the solver as 'Line Search Tolerance'.

    'List' : valueless
        Option 'List' enables printing of each option specification as
        it is supplied. 'NoList' suppresses this printing.

    'NoList' : valueless
        Default

        Option 'List' enables printing of each option specification as
        it is supplied. 'NoList' suppresses this printing.

    'Major Iteration Limit' : int
        Default = special

        The number of major iterations.

        - When 'Solver' = 'E04LB', ``lmm_fit`` passes 'Major Iteration
          Limit' to the solver as maxcal. In this case, the default
          value used is 1000.

        - When 'Solver' = 'E04UC', ``lmm_fit`` passes 'Major Iteration
          Limit' to the solver as 'Major Iteration Limit'. In this case,
          the default value used is max(50,3*g), where g is the number
          of variance components being estimated (excluding the overall
          variance, sigma_R^2).

    'Major Print Level' : int
        Default = special

        The frequency that monitoring information is output to 'Unit
        Number'.

        - When 'Solver' = 'E04LB', ``lmm_fit`` passes 'Major Print
          Level' to the solver as iprint. In this case, the default
          value used is -1 and hence no monitoring information will be
          output.

        - When 'Solver' = 'E04UC', ``lmm_fit`` passes 'Major Print
          Level' to the solver as 'Major Print Level'. In this case, the
          default value used is 0 and hence no monitoring information
          will be output.

    'Maximum Number of Threads' : int
        Default = special

        Controls the maximum number of threads used by ``lmm_fit`` in a
        multithreaded library.
        By default, the maximum number of available threads are used.

        In a library that is not multithreaded, this option has no
        effect.

    'Minor Iteration Limit' : int
        Default = max(50,3*g)

        The number of minor iterations.

        - When 'Solver' = 'E04LB', this option is ignored.

        - When 'Solver' = 'E04UC', ``lmm_fit`` passes 'Minor Iteration
          Limit' to the solver as 'Minor Iteration Limit'. In this case,
          the default value used is max(50,3*g), where g is the number
          of variance components being estimated (excluding the overall
          variance, sigma_R^2).

    'Minor Print Level' : int
        Default = 0

        The frequency that additional monitoring information is output
        to 'Unit Number'.

        - When 'Solver' = 'E04LB', this option is ignored.

        - When 'Solver' = 'E04UC', ``lmm_fit`` passes 'Minor Print
          Level' to the solver as 'Minor Print Level'. The default value
          of 0 means that no additional monitoring information will be
          output.

    'Optimality Tolerance' : float
        Default = machine precision^0.72

        The optimality tolerance.

        - When 'Solver' = 'E04LB', this option is ignored.

        - When 'Solver' = 'E04UC', ``lmm_fit`` passes 'Optimality
          Tolerance' to the solver as 'Optimality Tolerance'.

    'Parallelisation Strategy' : int
        Default = special

        If 'Maximum Number of Threads' > 0 then 'Parallelisation
        Strategy' controls how ``lmm_fit`` is parallelised in a
        multithreaded library.

        'Parallelisation Strategy' = 1
          ``lmm_fit`` will attempt to parallelise operations involving
          Z, even if `rnlsv` = 1.

        'Parallelisation Strategy' = 2
          ``lmm_fit`` will only attempt to parallelise operations
          involving Z, if `rnlsv` > 1.

        By default, 'Parallelisation Strategy' = 1, however, for some
        models / datasets, this may be slower than using
        'Parallelisation Strategy' = 2 when `rnlsv` = 1.

        In a library that is not multithreaded, this option has no
        effect.

    'Solution Accuracy' : float
        Default = 0.0

        The accuracy to which the solution is required.

        - When 'Solver' = 'E04LB', ``lmm_fit`` passes 'Solution
          Accuracy' to the solver as xtol.

        - When 'Solver' = 'E04UC', this option is ignored.

    'Solver' : str
        Default = special

        Controls which solver ``lmm_fit`` will use when fitting the
        model.
        By default, 'Solver' = 'E04LB' is used for small problems and
        'Solver' = 'E04UC', otherwise.

        If 'Solver' = 'E04LB', then the solver used is the one
        implemented in ``opt.bounds_mod_deriv2_comp`` and if 'Solver' =
        'E04UC', then the solver used is the one implemented in
        ``opt.nlp1_solve``.

    'Sweep Tolerance' : float
        Default = special

        The sweep tolerance used by ``lmm_fit`` when performing the
        sweep operation Wolfinger, Tobias and Sall.
        The default value used is 'Sweep Tolerance' =
        max(epsilon,epsilon*(max_i((Z^T)_{ii}))), where epsilon =
        sqrt(machine precision).

    'Unit Number' : int
        Default = advisory message unit number

        The monitoring unit number to which ``lmm_fit`` will send any
        monitoring information.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, `hlmm` is not a null Handle or a recognised G22
            handle.

        (`errno` 21)
            `hddesc` has not been initialized or is corrupt.

        (`errno` 22)
            `hddesc` is not a G22 handle as generated by
            ``blgm.lm_describe_data``.

        (`errno` 31)
            `hfixed` has not been initialized or is corrupt.

        (`errno` 32)
            `hfixed` is not a G22 handle as generated by
            ``blgm.lm_formula``.

        (`errno` 33)
            A variable name used when creating `hfixed` is not present
            in `hddesc`.

            Variable name: *<value>*.

        (`errno` 41)
            On entry, nrndm = *<value>*.

            Constraint: nrndm >= 0.

        (`errno` 51)
            i = *<value>*.

            `hrndm`[i-1] has not been initialized or is corrupt.

        (`errno` 52)
            i = *<value>*.

            `hrndm`[i-1] is not a G22 handle as generated by
            ``blgm.lm_formula``.

        (`errno` 53)
            No model has been specified.

        (`errno` 54)
            A variable name used when creating `hrndm` is not present in
            `hddesc`.

            Variable name: *<value>*.

        (`errno` 61)
            On entry, lwt = *<value>* and n = *<value>*.

            Constraint: lwt = 0 or n.

        (`errno` 71)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 72)
            On entry, n = *<value>* and n_d = *<value>*.

            Constraint: n <= n_d.

        (`errno` 73)
            On entry, no observations due to zero weights or missing
            values.

        (`errno` 91)
            On entry, i = *<value>* and `wt`[i-1] = *<value>*.

            Constraint: `wt`[i-1] >= 0.0.

        (`errno` 101)
            On entry, column j of the data matrix, D, is not consistent
            with information supplied in `hddesc`, j = *<value>*.

        (`errno` 111)
            On entry, n = *<value>* and lddat = *<value>*.

            Constraint: lddat >= n.

        (`errno` 112)
            On entry, m_d = *<value>* and lddat = *<value>*.

            Constraint: lddat >= m_d.

        (`errno` 121)
            On entry, m_d = *<value>* and sddat = *<value>*.

            Constraint: sddat >= m_d.

        (`errno` 122)
            On entry, n = *<value>* and sddat = *<value>*.

            Constraint: sddat >= n.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 34)
            The fixed part of the model contains categorical variables,
            but no intercept or main effects terms have been requested.

        (`errno` 102)
            Column j of the data matrix, D, required rounding more than
            expected when being treated as a categorical variable, j =
            *<value>*.

    See Also
    --------
    :meth:`naginterfaces.library.examples.correg.lmm_init_combine_ex.main`
    """

    from ..base.correg import lmm_init as b_lmm_init

    fun_name = 'naginterfaces.library.correg.lmm_init'

    _hrndm = hrndm
    _nrndm = _b_utils._EngineIntScalarType(
        0 if hrndm is None else _b_utils._EngineCptrArrayType.get_shape(_hrndm, {'fun_name': fun_name, 'entity': 'hrndm'}, exp_rank=1, allow_none=True)[0], {'fun_name': fun_name, 'entity': '_nrndm'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_y, {'fun_name': fun_name, 'entity': 'y'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _wt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'wt'},
        wt,
        copy=False,
        allow_none=True,
    )
    _dat = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'dat'},
        dat,
        copy=False,
    )
    try:
        _return_scalars = b_lmm_init(
            hlmm,
            hddesc,
            hfixed,
            _nrndm,
            _hrndm,
            _n,
            _y,
            _wt,
            _dat,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _fnlsv_py = _return_scalars.fnlsv
    _nff_py = _return_scalars.nff
    _rnlsv_py = _return_scalars.rnlsv
    _nrf_py = _return_scalars.nrf
    _nvpr_py = _return_scalars.nvpr
    _comm_py = _return_scalars.comm
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['fnlsv', 'nff', 'rnlsv', 'nrf', 'nvpr', 'comm'],
    )(
        _fnlsv_py,
        _nff_py,
        _rnlsv_py,
        _nrf_py,
        _nvpr_py,
        _comm_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def lmm_init_combine(hlmm, xcomm, ycomm):
    u"""
    Linear mixed effects regression, initialization function for
    ``lmm_init_combine`` and ``lmm_fit``.

    For full information please refer to the NAG Library document for
    g02jg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02jgf.html

    Parameters
    ----------
    hlmm : Handle
        If the two sets of communication arrays were generated using
        ``lmm_init``, then a G22 handle as generated by one of the calls
        to ``lmm_init``.

        If the two sets of communication arrays were generated using
        ``mixeff_hier_init``, then this argument is not referenced and
        need not be set.

    xcomm : dict, communication object, modified in place
        Communication structure for D_x.

        This argument must have been initialized by a prior call to
        ``lmm_init`` or ``mixeff_hier_init``.

    ycomm : dict, communication object
        Communication structure for D_y.

        This argument must have been initialized by a prior call to
        ``lmm_init`` or ``mixeff_hier_init``.

    Raises
    ------
    NagValueError
        (`errno` 11)
            `hlmm` has not been initialized or is corrupt.

        (`errno` 12)
            `hlmm` is not a G22 handle as generated by ``lmm_init``.

        (`errno` 41)
            On entry, `xcomm`['icomm'] has not been initialized
            correctly.

        (`errno` 42)
            On entry, the information stored in `xcomm`['icomm'] and
            `ycomm`['icomm'] is not compatible.

        (`errno` 71)
            On entry, `ycomm`['icomm'] has not been initialized
            correctly.

    See Also
    --------
    :meth:`naginterfaces.library.examples.correg.lmm_init_combine_ex.main`
    """

    from ..base.correg import lmm_init_combine as b_lmm_init_combine

    fun_name = 'naginterfaces.library.correg.lmm_init_combine'

    b_lmm_init_combine(
        hlmm,
        xcomm,
        ycomm,
    )
    _tmp_xlrcomm_py = _b_utils._EngineFloat64ArrayType.get_shape(xcomm['rcomm'], {'fun_name': fun_name, 'entity': 'xcomm[\'rcomm\']'}, exp_rank=1)[0]
    _tmp_xlicomm_py = _b_utils._EngineFloat64ArrayType.get_shape(xcomm['icomm'], {'fun_name': fun_name, 'entity': 'xcomm[\'icomm\']'}, exp_rank=1)[0]
    _xlrcomm_py = _b_utils._EngineIntArrayType.get_element(xcomm['icomm'], {'fun_name': fun_name, 'entity': 'xcomm[\'icomm\']'}, (1,), validate=False)
    _xlicomm_py = _b_utils._EngineIntArrayType.get_element(xcomm['icomm'], {'fun_name': fun_name, 'entity': 'xcomm[\'icomm\']'}, (0,), validate=False)
    if (_tmp_xlrcomm_py < _xlrcomm_py) or (_tmp_xlicomm_py < _xlicomm_py):
        _xicomm = _b_utils._EngineIntArrayType.empty_ctypes(_xlicomm_py)
        _xrcomm = _b_utils._EngineFloat64ArrayType.empty_ctypes(_xlrcomm_py)
        _xicomm[0:_tmp_xlicomm_py] = xcomm['icomm'][0:_tmp_xlicomm_py]
        _xrcomm[0:_tmp_xlrcomm_py] = xcomm['rcomm'][0:_tmp_xlrcomm_py]
        xcomm['icomm'] = _xicomm
        xcomm['rcomm'] = _xrcomm
        b_lmm_init_combine(
            hlmm,
            xcomm,
            ycomm,
        )

def lmm_fit(hlmm, gamma, comm, wantc=False, io_manager=None):
    u"""
    Linear mixed effects regression using either Restricted Maximum
    Likelihood (REML) or Maximum Likelihood (ML).

    For full information please refer to the NAG Library document for
    g02jh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02jhf.html

    Parameters
    ----------
    hlmm : Handle
        A G22 handle to the internal data structure containing a
        description of the required model as returned in hlmm by
        ``lmm_init``.

    gamma : float, array-like, shape (nvpr+1)
        Holds the initial values of the variance components, gamma_0,
        with `gamma`[i-1] the initial value for sigma_i^2/sigma_R^2, for
        i = 1,2,...,nvpr.

        If `gamma`[0] = -1.0, the remaining elements of `gamma` are
        ignored and the initial values for the variance components are
        estimated from the data using MIVQUE0.

    comm : dict, communication object, modified in place
        Communication structure.

        This argument must have been initialized by a prior call to
        ``lmm_init``.

    wantc : bool, optional
        Flag indicating whether the arrays `czz`, `cxx` and `cxz` are
        required.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    gamma : float, ndarray, shape (nvpr+1)
        `gamma`[i-1], for i = 1,2,...,nvpr, holds the final estimate of
        sigma_i^2 and `gamma`[nvpr] holds the final estimate for
        sigma_R^2.

        Labels for the variance components can be obtained using
        ``blgm.lm_submodel`` as demonstrated in [Example].

    effn : int
        Effective number of observations. If there are no weights (i.e.,
        weight = 'U'), or all weights are nonzero, `effn` = n.

    rnkx : int
        The rank of the design matrix, X, for the fixed effects.

    ncov : int
        Number of variance components not estimated to be zero. If none
        of the variance components are estimated to be zero, `ncov` =
        nvpr.

    lnlike : float
        -2l_R(gamma^) where l_R is the log of the restricted maximum
        likelihood calculated at gamma^, the estimated variance
        components returned in `gamma`.

    b : float, ndarray, shape (fnlsv*nff+rnlsv*nrf)
        The parameter estimates, with the first nrf*rnlsv elements of
        `b` containing the parameter estimates for the random effects,
        nu, and the remaining nff*fnlsv elements containing the
        parameter estimates for the fixed effects, beta.

        Labels for the parameter estimates can be obtained using
        ``blgm.lm_submodel`` as demonstrated in [Example].

    se : float, ndarray, shape (fnlsv*nff+rnlsv*nrf)
        The standard errors of the parameter estimates given in `b`.

    czz : float, ndarray, shape (:, :)
        If rnlsv = 1, `czz` holds the lower triangular portion of the
        matrix (1/sigma^2)(Z^TR^^-1Z+G^^-1), where R^ and G^ are the
        estimates of R and G respectively.

        If rnlsv > 1, then `czz` holds this matrix in compressed form,
        with the first nrf columns holding the part of the matrix
        corresponding to the first level of the overall random subject
        variable, the next nrf columns holding the part of the matrix
        corresponding to the second level of the overall random subject
        variable etc.

        If ldczz = 0, `czz` is not referenced.

    cxx : float, ndarray, shape (:, :)
        If fnlsv = 1, `cxx` holds the lower triangular portion of the
        matrix (1/sigma^2)(X^TV^^-1X), where V^ is the estimated value
        of V.

        If fnlsv > 1, then `cxx` holds this matrix in compressed form,
        with the first nff columns holding the part of the matrix
        corresponding to the first level of the overall fixed subject
        variable, the next nff columns holding the part of the matrix
        corresponding to the second level of the overall fixed subject
        variable, etc.

        If ldcxx = 0, `cxx` is not referenced.

    cxz : float, ndarray, shape (:, :)
        `cxz` holds the matrix (1/sigma^2)(X^TV^^-1Z)G^, where V^ and G^
        are the estimates of V and G respectively.

        If ldcxz = 0, `cxz` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` 11)
            `hlmm` has not been initialized or is corrupt.

        (`errno` 12)
            `hlmm` is not a G22 handle as generated by ``lmm_init``.

        (`errno` 21)
            On entry, nvpr = *<value>*.

            Constraint: nvpr >= *<value>*.

        (`errno` 31)
            On entry, `gamma`[*<value>*] = *<value>*.

            Constraint: `gamma`[0] = -1.0 or `gamma`[i-1] >= 0.0.

        (`errno` 81)
            On entry, lb = *<value>*.

            Constraint: lb >= *<value>*.

        (`errno` 171)
            On entry, the communication arrays, `comm`['icomm'] and
            `comm`['rcomm'], have not been initialized correctly.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1001)
            Optimal solution found, but requested accuracy not achieved.

        (`errno` 1002)
            Too many major iterations.

        (`errno` 1003)
            Current point cannot be improved upon.

        (`errno` 1004)
            At least one negative estimate for `gamma` was obtained. All
            negative estimates have been set to zero.

    See Also
    --------
    :meth:`naginterfaces.library.examples.correg.lmm_init_combine_ex.main`
    """

    from ..base.correg import lmm_fit as b_lmm_fit

    fun_name = 'naginterfaces.library.correg.lmm_fit'

    _gamma = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'gamma'},
        gamma,
        copy=True,
    )
    _nvpr = _b_utils._EngineIntScalarType(
        _b_utils._EngineFloat64ArrayType.get_shape(_gamma, {'fun_name': fun_name, 'entity': 'gamma'}, exp_rank=1)[0]-1, {'fun_name': fun_name, 'entity': '_nvpr'},
    ).to_ctypes()
    _b_utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('rcomm', 'icomm'),
    )
    _vl_g02jff_fnlsv_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (23,), validate=False)
    _vl_g02jff_nff_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (5,), validate=False)
    _vl_g02jff_rnlsv_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (7,), validate=False)
    _vl_g02jff_nrf_py = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (6,), validate=False)
    _lb = _b_utils._EngineIntScalarType(
        _vl_g02jff_fnlsv_py*_vl_g02jff_nff_py+_vl_g02jff_rnlsv_py*_vl_g02jff_nrf_py, {'fun_name': fun_name, 'entity': '_lb'},
    ).to_ctypes()
    _b = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_g02jff_fnlsv_py*_vl_g02jff_nff_py+_vl_g02jff_rnlsv_py*_vl_g02jff_nrf_py,),
    )
    _se = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_g02jff_fnlsv_py*_vl_g02jff_nff_py+_vl_g02jff_rnlsv_py*_vl_g02jff_nrf_py,),
    )
    _wantc = _b_utils._EngineLogicalScalarType(
        wantc, {'fun_name': fun_name, 'entity': 'wantc'},
    ).to_ctypes()
    _wantc_py = _b_utils._EngineLogicalScalarType.to_py(_wantc)
    if _wantc_py:
        _czz_dim1 = _vl_g02jff_nrf_py
    else:
        _czz_dim1 = 0
    _czz_dim2 = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (7,), validate=False)*_b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (6,), validate=False)
    exp_shape_czz = (_czz_dim1, _czz_dim2)
    _czz = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_czz,
        sorder=_b_utils._EngineData.default_sorder,
    )
    if _wantc_py:
        _cxx_dim1 = _vl_g02jff_nff_py
    else:
        _cxx_dim1 = 0
    _cxx_dim2 = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (23,), validate=False)*_b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (5,), validate=False)
    exp_shape_cxx = (_cxx_dim1, _cxx_dim2)
    _cxx = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_cxx,
        sorder=_b_utils._EngineData.default_sorder,
    )
    if _wantc_py:
        _cxz_dim1 = _vl_g02jff_fnlsv_py*_vl_g02jff_nff_py
    else:
        _cxz_dim1 = 0
    _cxz_dim2 = _b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (7,), validate=False)*_b_utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (6,), validate=False)
    exp_shape_cxz = (_cxz_dim1, _cxz_dim2)
    _cxz = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_cxz,
        sorder=_b_utils._EngineData.default_sorder,
    )
    try:
        _return_scalars = b_lmm_fit(
            hlmm,
            _nvpr,
            _gamma,
            _lb,
            _b,
            _se,
            _czz,
            _cxx,
            _cxz,
            comm,
            io_manager,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _gamma_py = _gamma
    _effn_py = _return_scalars.effn
    _rnkx_py = _return_scalars.rnkx
    _ncov_py = _return_scalars.ncov
    _lnlike_py = _return_scalars.lnlike
    _b_py = _b
    _se_py = _se
    _czz_py = _czz
    _cxx_py = _cxx
    _cxz_py = _cxz
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['gamma', 'effn', 'rnkx', 'ncov', 'lnlike', 'b', 'se', 'czz', 'cxx', 'cxz'],
    )(
        _gamma_py,
        _effn_py,
        _rnkx_py,
        _ncov_py,
        _lnlike_py,
        _b_py,
        _se_py,
        _czz_py,
        _cxx_py,
        _cxz_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def ridge_opt(x, isx, y, h, opt, niter, tol, orig, optloo, tau=0.0):
    u"""
    Ridge regression, optimizing a ridge regression parameter.

    For full information please refer to the NAG Library document for
    g02ka

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02kaf.html

    Parameters
    ----------
    x : float, array-like, shape (n, m)
        The values of independent variables in the data matrix X.

    isx : int, array-like, shape (m)
        Indicates which m independent variables are included in the
        model.

        `isx`[j-1] = 1

          The jth variable in `x` will be included in the model.

        `isx`[j-1] = 0

          Variable j is excluded.

    y : float, array-like, shape (n)
        The n values of the dependent variable y.

    h : float
        An initial value for the ridge regression parameter h; used as a
        starting point for the optimization.

    opt : int
        The measure of prediction error used to optimize the ridge
        regression parameter h. The value of `opt` must be set equal to
        one of:

        `opt` = 1

          Generalized cross-validation (GCV);

        `opt` = 2

          Unbiased estimate of variance (UEV)

        `opt` = 3

          Future prediction error (FPE)

        `opt` = 4

          Bayesian information criteron (BIC).

    niter : int
        The maximum number of iterations allowed to optimize the ridge
        regression parameter h.

    tol : float
        Iterations of the ridge regression parameter h will halt when
        consecutive values of h lie within `tol`.

    orig : int
        If `orig` = 1, the parameter estimates b are calculated for the
        original data; otherwise `orig` = 2 and the parameter estimates
        b~ are calculated for the standardized data.

    optloo : int
        If `optloo` = 2, the leave-one-out cross-validation estimate of
        prediction error is calculated; otherwise no such estimate is
        calculated and `optloo` = 1.

    tau : float, optional
        Singular values less than `tau` of the SVD of the data matrix X
        will be set equal to zero.

    Returns
    -------
    h : float
        This argument is the optimized value of the ridge regression
        parameter h.

    niter : int
        The number of iterations used to optimize the ridge regression
        parameter h within `tol`.

    nep : float
        The number of effective parameters, gamma, in the model.

    b : float, ndarray, shape (ip+1)
        Contains the intercept and parameter estimates for the fitted
        ridge regression model in the order indicated by `isx`. The
        first element of `b` contains the estimate for the intercept;
        `b`[j] contains the parameter estimate for the jth independent
        variable in the model, for j = 1,2,...,ip.

    vif : float, ndarray, shape (ip)
        The variance inflation factors in the order indicated by `isx`.
        For the jth independent variable in the model, `vif`[j-1] is the
        value of v_j, for j = 1,2,...,ip.

    res : float, ndarray, shape (n)
        `res`[i-1] is the value of the ith residual for the fitted ridge
        regression model, for i = 1,2,...,n.

    rss : float
        The sum of squares of residual values.

    df : int
        The degrees of freedom for the residual sum of squares `rss`.

    perr : float, ndarray, shape (5)
        The first four elements contain, in this order, the measures of
        prediction error: GCV, UEV, FPE and BIC.

        If `optloo` = 2, `perr`[4] is the LOOCV estimate of prediction
        error; otherwise `perr`[4] is not referenced.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n > 1.

        (`errno` 1)
            On entry, `tau` = *<value>*.

            Constraint: `tau` >= 0.0.

        (`errno` 1)
            On entry, `opt` = *<value>*.

            Constraint: `opt` = 1, 2, 3 or 4.

        (`errno` 1)
            On entry, `h` = *<value>*.

            Constraint: `h` > 0.0.

        (`errno` 1)
            On entry, `optloo` = *<value>*.

            Constraint: `optloo` = 1 or 2.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` > 0.0.

        (`errno` 1)
            On entry, `niter` = *<value>*.

            Constraint: `niter` >= 1.

        (`errno` 1)
            On entry, `orig` = *<value>*.

            Constraint: `orig` = 1 or 2.

        (`errno` 2)
            On entry, m = *<value>* and n = *<value>*.

            Constraint: m <= n.

        (`errno` 2)
            On entry, ip = *<value>*; m = *<value>*.

            Constraint: 1 <= ip <= m.

        (`errno` 2)
            On entry, `isx`[*<value>*] = *<value>*.

            Constraint: `isx`[j-1] = 0 or 1.

        (`errno` 2)
            On entry, ip = *<value>*.

            Constraint: sum(`isx`) = ip.

        (`errno` 3)
            SVD failed to converge.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` -1)
            Maximum number of iterations used.

    See Also
    --------
    :meth:`naginterfaces.library.examples.correg.ridge_opt_ex.main`
    """

    from ..base.correg import ridge_opt as b_ridge_opt

    fun_name = 'naginterfaces.library.correg.ridge_opt'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _isx = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'isx'},
        isx,
        copy=False,
    )
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _vl_ip_py = 0
    for _vl_i_py in range(1, _m_py+1):
        if _b_utils._EngineIntArrayType.get_element(_isx, {'fun_name': fun_name, 'entity': 'isx'}, (_vl_i_py-1,)) == 1:
            _vl_ip_py = _vl_ip_py + 1
    _b = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_ip_py+1,),
    )
    _vif = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_ip_py,),
    )
    _res = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _perr = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (5,),
    )
    try:
        _return_scalars = b_ridge_opt(
            _n,
            _m,
            _x,
            _isx,
            tau,
            _y,
            h,
            opt,
            niter,
            tol,
            orig,
            _b,
            _vif,
            _res,
            optloo,
            _perr,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _h_py = _return_scalars.h
    _niter_py = _return_scalars.niter
    _nep_py = _return_scalars.nep
    _b_py = _b
    _vif_py = _vif
    _res_py = _res
    _rss_py = _return_scalars.rss
    _df_py = _return_scalars.df
    _perr_py = _perr
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['h', 'niter', 'nep', 'b', 'vif', 'res', 'rss', 'df', 'perr'],
    )(
        _h_py,
        _niter_py,
        _nep_py,
        _b_py,
        _vif_py,
        _res_py,
        _rss_py,
        _df_py,
        _perr_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def ridge(x, isx, y, h, wantb, wantvf, pec=None):
    u"""
    Ridge regression using a number of supplied ridge regression
    parameters.

    For full information please refer to the NAG Library document for
    g02kb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02kbf.html

    Parameters
    ----------
    x : float, array-like, shape (n, m)
        The values of independent variables in the data matrix X.

    isx : int, array-like, shape (m)
        Indicates which m independent variables are included in the
        model.

        `isx`[j-1] = 1

          The jth variable in `x` will be included in the model.

        `isx`[j-1] = 0

          Variable j is excluded.

    y : float, array-like, shape (n)
        The n values of the dependent variable y.

    h : float, array-like, shape (lh)
        `h`[j-1] is the value of the jth ridge parameter h.

    wantb : int
        Defines the options for parameter estimates.

        `wantb` = 0

          Parameter estimates are not calculated and `b` is not
          referenced.

        `wantb` = 1

          Parameter estimates b are calculated for the original data.

        `wantb` = 2

          Parameter estimates b~ are calculated for the standardized
          data.

    wantvf : int
        Defines the options for variance inflation factors.

        `wantvf` = 0

          Variance inflation factors are not calculated and the array
          `vf` is not referenced.

        `wantvf` = 1

          Variance inflation factors are calculated.

    pec : None or str, length 1, array-like, shape (lpec), optional
        If `pec` is not None, `pec`[j-1] defines the jth prediction
        error, for j = 1,2,...,lpec; otherwise `pec` is not referenced.

        `pec`[j-1] = 'B'

          Bayesian information criterion (BIC).

        `pec`[j-1] = 'F'

          Future prediction error (FPE).

        `pec`[j-1] = 'G'

          Generalized cross-validation (GCV).

        `pec`[j-1] = 'L'

          Leave-one-out cross-validation (LOOCV).

        `pec`[j-1] = 'U'

          Unbiased estimate of variance (UEV).

    Returns
    -------
    nep : float, ndarray, shape (lh)
        `nep`[j-1] is the number of effective parameters, gamma, in the
        jth model, for j = 1,2,...,lh.

    b : float, ndarray, shape (:, :)
        If `wantb` != 0, this argument contains the intercept and
        parameter estimates for the fitted ridge regression model in the
        order indicated by `isx`. `b`[0,j-1], for j = 1,2,...,lh,
        contains the estimate for the intercept; `b`[i,j-1] contains the
        parameter estimate for the ith independent variable in the model
        fitted with ridge parameter `h`[j-1], for i = 1,2,...,ip.

    vf : float, ndarray, shape (:, :)
        If `wantvf` = 1, the variance inflation factors. For the ith
        independent variable in a model fitted with ridge parameter
        `h`[j-1], `vf`[i-1,j-1] is the value of v_i, for i = 1,2,...,ip.

    pe : None or float, ndarray, shape (:, :)
        If `pec` is None on entry, this argument is None; otherwise
        `pe`[i-1,j-1] contains the prediction error of criterion
        `pec`[i-1] for the model fitted with ridge parameter `h`[j-1],
        for j = 1,2,...,lh, for i = 1,2,...,lpec.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 1)
            On entry, m = *<value>* and n = *<value>*.

            Constraint: m <= n.

        (`errno` 1)
            On entry, `h`[j-1] < 0 for at least one j.

            Constraint: `h`[j-1] <= 0.0, for all j.

        (`errno` 1)
            On entry, lh = *<value>*.

            Constraint: lh > 0.

        (`errno` 1)
            On entry, `wantb` = *<value>*.

            Constraint: `wantb` = 0, 1 or 2.

        (`errno` 1)
            On entry, `wantvf` = *<value>*.

            Constraint: `wantvf` = 0 or 1.

        (`errno` 1)
            On entry, `pec`[j-1] is invalid for at least one j.

            Constraint: if `pec` is not None, `pec`[j-1] = 'B', 'F',
            'G', 'L' or 'U', for all j.

        (`errno` 2)
            On entry, `isx`[j-1] != 0 or 1 for at least one j.

            Constraint: `isx`[j-1] = 0 or 1, for all j.

        (`errno` 2)
            On entry, ip is not equal to the sum of elements in `isx`.

            Constraint: exactly ip elements of `isx` must be equal to 1.

        (`errno` 2)
            On entry, ldb = *<value>* and ip = *<value>*.

            Constraint: if `wantb` != 0, ldb >= ip+1.

        (`errno` 2)
            On entry, ldvf = *<value>* and ip = *<value>*.

            Constraint: if `wantvf` != 0, ldvf >= ip.

        (`errno` 3)
            On entry, `wantb` = 0 and `wantvf` = 0.

            Constraint: `wantb` = 0, `wantvf` = 1.
    """

    from ..base.correg import ridge as b_ridge

    fun_name = 'naginterfaces.library.correg.ridge'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _isx = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'isx'},
        isx,
        copy=False,
    )
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _h = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'h'},
        h,
        copy=False,
    )
    _lh_py = _b_utils._EngineFloat64ArrayType.get_shape(_h, {'fun_name': fun_name, 'entity': 'h'}, exp_rank=1)[0]
    _lh = _b_utils._EngineIntScalarType(
        _lh_py, {'fun_name': fun_name, 'entity': '_lh'},
    ).to_ctypes()
    _nep = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_lh_py,),
    )
    _vl_ip_py = 0
    for _vl_i_py in range(1, _m_py+1):
        if _b_utils._EngineIntArrayType.get_element(_isx, {'fun_name': fun_name, 'entity': 'isx'}, (_vl_i_py-1,)) == 1:
            _vl_ip_py = _vl_ip_py + 1
    _wantb = _b_utils._EngineIntScalarType(
        wantb, {'fun_name': fun_name, 'entity': 'wantb'},
    ).to_ctypes()
    _wantb_py = _b_utils._EngineIntScalarType.to_py(_wantb)
    if _wantb_py != 0:
        _b_dim1 = _vl_ip_py+1
    else:
        _b_dim1 = 1
    if _wantb_py != 0:
        _b_dim2 = _lh_py
    else:
        _b_dim2 = 0
    exp_shape_b = (_b_dim1, _b_dim2)
    _b = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_b,
        sorder=_m_sorder,
    )
    _wantvf = _b_utils._EngineIntScalarType(
        wantvf, {'fun_name': fun_name, 'entity': 'wantvf'},
    ).to_ctypes()
    _wantvf_py = _b_utils._EngineIntScalarType.to_py(_wantvf)
    if _wantvf_py != 0:
        _vf_dim1 = _vl_ip_py
    else:
        _vf_dim1 = 1
    if _wantvf_py != 0:
        _vf_dim2 = _lh_py
    else:
        _vf_dim2 = 0
    exp_shape_vf = (_vf_dim1, _vf_dim2)
    _vf = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_vf,
        sorder=_m_sorder,
    )
    _pec = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'pec'},
        pec,
        copy=False,
        allow_none=True,
    )
    _lpec = _b_utils._EngineIntScalarType(
        0 if pec is None else _b_utils._EngineCharArrayType.get_shape(_pec, {'fun_name': fun_name, 'entity': 'pec'}, exp_rank=1, allow_none=True)[0], {'fun_name': fun_name, 'entity': '_lpec'},
    ).to_ctypes()
    _lpec_py = _b_utils._EngineIntScalarType.to_py(_lpec)
    if _lpec_py > 0:
        if _lpec_py > 0:
            _pe_dim1 = _lpec_py
        else:
            _pe_dim1 = 1
        if _lpec_py > 0:
            _pe_dim2 = _lh_py
        else:
            _pe_dim2 = 0
        exp_shape_pe = (_pe_dim1, _pe_dim2)
        _pe = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            exp_shape_pe,
            sorder=_m_sorder,
        )
    else:
        _pe = None
    b_ridge(
        _n,
        _m,
        _x,
        _isx,
        _y,
        _lh,
        _h,
        _nep,
        _wantb,
        _b,
        _wantvf,
        _vf,
        _lpec,
        _pec,
        _pe,
    )
    _nep_py = _nep
    _b_py = _b
    _vf_py = _vf
    if _lpec_py > 0:
        _pe_py = _pe
    else:
        _pe_py = None
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['nep', 'b', 'vf', 'pe'],
    )(
        _nep_py,
        _b_py,
        _vf_py,
        _pe_py,
    )
    return _return_data

def pls_svd(x, isx, y, iscale, xstd, ystd, maxfac, io_manager=None):
    u"""
    Partial least squares (PLS) regression using singular value
    decomposition.

    For full information please refer to the NAG Library document for
    g02la

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02laf.html

    Parameters
    ----------
    x : float, array-like, shape (n, mx)
        `x`[i-1,j-1] must contain the ith observation on the jth
        predictor variable, for j = 1,2,...,mx, for i = 1,2,...,n.

    isx : int, array-like, shape (mx)
        Indicates which predictor variables are to be included in the
        model.

        `isx`[j-1] = 1

          The jth predictor variable (with variates in the jth column of
          X) is included in the model.

        `isx`[j-1] = 0

          Otherwise.

    y : float, array-like, shape (n, my)
        `y`[i-1,j-1] must contain the ith observation for the jth
        response variable, for j = 1,2,...,my, for i = 1,2,...,n.

    iscale : int
        Indicates how predictor variables are scaled.

        `iscale` = 1

          Data are scaled by the standard deviation of variables.

        `iscale` = 2

          Data are scaled by user-supplied scalings.

        `iscale` = -1

          No scaling.

    xstd : float, array-like, shape (ip)
        If `iscale` = 2, `xstd`[j-1] must contain the user-supplied
        scaling for the jth predictor variable in the model, for j =
        1,2,...,ip. Otherwise `xstd` need not be set.

    ystd : float, array-like, shape (my)
        If `iscale` = 2, `ystd`[j-1] must contain the user-supplied
        scaling for the jth response variable in the model, for j =
        1,2,...,my. Otherwise `ystd` need not be set.

    maxfac : int
        k, the number of latent variables to calculate.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    xbar : float, ndarray, shape (ip)
        Mean values of predictor variables in the model.

    ybar : float, ndarray, shape (my)
        The mean value of each response variable.

    xstd : float, ndarray, shape (ip)
        If `iscale` = 1, standard deviations of predictor variables in
        the model. Otherwise this argument is not changed.

    ystd : float, ndarray, shape (my)
        If `iscale` = 1, the standard deviation of each response
        variable. Otherwise this argument is not changed.

    xres : float, ndarray, shape (n, ip)
        The predictor variables' residual matrix X_k.

    yres : float, ndarray, shape (n, my)
        The residuals for each response variable, Y_k.

    w : float, ndarray, shape (ip, `maxfac`)
        The jth column of W contains the x-weights w_j, for j =
        1,2,...,`maxfac`.

    p : float, ndarray, shape (ip, `maxfac`)
        The jth column of P contains the x-loadings p_j, for j =
        1,2,...,`maxfac`.

    t : float, ndarray, shape (n, `maxfac`)
        The jth column of T contains the x-scores t_j, for j =
        1,2,...,`maxfac`.

    c : float, ndarray, shape (my, `maxfac`)
        The jth column of C contains the y-loadings c_j, for j =
        1,2,...,`maxfac`.

    u : float, ndarray, shape (n, `maxfac`)
        The jth column of U contains the y-scores u_j, for j =
        1,2,...,`maxfac`.

    xcv : float, ndarray, shape (`maxfac`)
        `xcv`[j-1] contains the cumulative percentage of variance in the
        predictor variables explained by the first j factors, for j =
        1,2,...,`maxfac`.

    ycv : float, ndarray, shape (`maxfac`, my)
        `ycv`[i-1,j-1] is the cumulative percentage of variance of the
        jth response variable explained by the first i factors, for j =
        1,2,...,my, for i = 1,2,...,`maxfac`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n > 1.

        (`errno` 1)
            On entry, mx = *<value>*.

            Constraint: mx > 1.

        (`errno` 1)
            On entry, `isx`[*<value>*] is invalid.

            Constraint: `isx`[j-1] = 0 or 1, for all j.

        (`errno` 1)
            On entry, my = *<value>*.

            Constraint: my >= 1.

        (`errno` 1)
            On entry, `iscale` = *<value>*.

            Constraint: `iscale` = -1 or 1.

        (`errno` 2)
            On entry, ip = *<value>* and mx = *<value>*.

            Constraint: 1 < ip <= mx.

        (`errno` 2)
            On entry, `maxfac` = *<value>* and ip = *<value>*.

            Constraint: 1 <= `maxfac` <= ip.

        (`errno` 3)
            On entry, ip = *<value>* and sum(`isx`) = *<value>*.

            Constraint: the sum of elements in `isx` must equal ip.

    See Also
    --------
    :meth:`naginterfaces.library.examples.correg.pls_ex.main`
    """

    from ..base.correg import pls_svd as b_pls_svd

    fun_name = 'naginterfaces.library.correg.pls_svd'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'), (_y, 'y')),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _mx_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _mx = _b_utils._EngineIntScalarType(
        _mx_py, {'fun_name': fun_name, 'entity': '_mx'},
    ).to_ctypes()
    _isx = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'isx'},
        isx,
        copy=False,
    )
    _my_py = _b_utils._EngineFloat64ArrayType.get_shape(_y, {'fun_name': fun_name, 'entity': 'y'}, exp_rank=2)[1]
    _my = _b_utils._EngineIntScalarType(
        _my_py, {'fun_name': fun_name, 'entity': '_my'},
    ).to_ctypes()
    _vl_ip_py = 0
    for _vl_i_py in range(1, _mx_py+1):
        if _b_utils._EngineIntArrayType.get_element(_isx, {'fun_name': fun_name, 'entity': 'isx'}, (_vl_i_py-1,)) == 1:
            _vl_ip_py = _vl_ip_py + 1
    _xbar = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_ip_py,),
    )
    _ybar = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_my_py,),
    )
    _xstd = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'xstd'},
        xstd,
        copy=True,
    )
    _ystd = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ystd'},
        ystd,
        copy=True,
    )
    _xres = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _vl_ip_py),
        sorder=_m_sorder,
    )
    _yres = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _my_py),
        sorder=_m_sorder,
    )
    _maxfac = _b_utils._EngineIntScalarType(
        maxfac, {'fun_name': fun_name, 'entity': 'maxfac'},
    ).to_ctypes()
    _maxfac_py = _b_utils._EngineIntScalarType.to_py(_maxfac)
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_ip_py, _maxfac_py),
        sorder=_m_sorder,
    )
    _p = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_ip_py, _maxfac_py),
        sorder=_m_sorder,
    )
    _t = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _maxfac_py),
        sorder=_m_sorder,
    )
    _c = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_my_py, _maxfac_py),
        sorder=_m_sorder,
    )
    _u = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _maxfac_py),
        sorder=_m_sorder,
    )
    _xcv = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_maxfac_py,),
    )
    _ycv = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_maxfac_py, _my_py),
        sorder=_m_sorder,
    )
    b_pls_svd(
        _n,
        _mx,
        _x,
        _isx,
        _my,
        _y,
        _xbar,
        _ybar,
        iscale,
        _xstd,
        _ystd,
        _maxfac,
        _xres,
        _yres,
        _w,
        _p,
        _t,
        _c,
        _u,
        _xcv,
        _ycv,
        io_manager,
    )
    _xstd_py = _xstd
    _ystd_py = _ystd
    _xbar_py = _xbar
    _ybar_py = _ybar
    _xres_py = _xres
    _yres_py = _yres
    _w_py = _w
    _p_py = _p
    _t_py = _t
    _c_py = _c
    _u_py = _u
    _xcv_py = _xcv
    _ycv_py = _ycv
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['xbar', 'ybar', 'xstd', 'ystd', 'xres', 'yres', 'w', 'p', 't', 'c', 'u', 'xcv', 'ycv'],
    )(
        _xbar_py,
        _ybar_py,
        _xstd_py,
        _ystd_py,
        _xres_py,
        _yres_py,
        _w_py,
        _p_py,
        _t_py,
        _c_py,
        _u_py,
        _xcv_py,
        _ycv_py,
    )
    return _return_data

def pls_wold(x, isx, y, iscale, xstd, ystd, maxfac, maxit=200, tau=1.0e-4, io_manager=None):
    u"""
    Partial least squares (PLS) regression using Wold's iterative
    method.

    For full information please refer to the NAG Library document for
    g02lb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02lbf.html

    Parameters
    ----------
    x : float, array-like, shape (n, mx)
        `x`[i-1,j-1] must contain the ith observation on the jth
        predictor variable, for j = 1,2,...,mx, for i = 1,2,...,n.

    isx : int, array-like, shape (mx)
        Indicates which predictor variables are to be included in the
        model.

        `isx`[j-1] = 1

          The jth predictor variable (with variates in the jth column of
          X) is included in the model.

        `isx`[j-1] = 0

          Otherwise.

    y : float, array-like, shape (n, my)
        `y`[i-1,j-1] must contain the ith observation for the jth
        response variable, for j = 1,2,...,my, for i = 1,2,...,n.

    iscale : int
        Indicates how predictor variables are scaled.

        `iscale` = 1

          Data are scaled by the standard deviation of variables.

        `iscale` = 2

          Data are scaled by user-supplied scalings.

        `iscale` = -1

          No scaling.

    xstd : float, array-like, shape (ip)
        If `iscale` = 2, `xstd`[j-1] must contain the user-supplied
        scaling for the jth predictor variable in the model, for j =
        1,2,...,ip. Otherwise `xstd` need not be set.

    ystd : float, array-like, shape (my)
        If `iscale` = 2, `ystd`[j-1] must contain the user-supplied
        scaling for the jth response variable in the model, for j =
        1,2,...,my. Otherwise `ystd` need not be set.

    maxfac : int
        k, the number of latent variables to calculate.

    maxit : int, optional
        If my = 1, `maxit` is not referenced; otherwise the maximum
        number of iterations used to calculate the x-weights.

    tau : float, optional
        If my = 1, `tau` is not referenced; otherwise the iterative
        procedure used to calculate the x-weights will halt if the
        Euclidean distance between two subsequent estimates is less than
        or equal to `tau`.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    xbar : float, ndarray, shape (ip)
        Mean values of predictor variables in the model.

    ybar : float, ndarray, shape (my)
        The mean value of each response variable.

    xstd : float, ndarray, shape (ip)
        If `iscale` = 1, standard deviations of predictor variables in
        the model. Otherwise this argument is not changed.

    ystd : float, ndarray, shape (my)
        If `iscale` = 1, the standard deviation of each response
        variable. Otherwise this argument is not changed.

    xres : float, ndarray, shape (n, ip)
        The predictor variables' residual matrix X_k.

    yres : float, ndarray, shape (n, my)
        The residuals for each response variable, Y_k.

    w : float, ndarray, shape (ip, `maxfac`)
        The jth column of W contains the x-weights w_j, for j =
        1,2,...,`maxfac`.

    p : float, ndarray, shape (ip, `maxfac`)
        The jth column of P contains the x-loadings p_j, for j =
        1,2,...,`maxfac`.

    t : float, ndarray, shape (n, `maxfac`)
        The jth column of T contains the x-scores t_j, for j =
        1,2,...,`maxfac`.

    c : float, ndarray, shape (my, `maxfac`)
        The jth column of C contains the y-loadings c_j, for j =
        1,2,...,`maxfac`.

    u : float, ndarray, shape (n, `maxfac`)
        The jth column of U contains the y-scores u_j, for j =
        1,2,...,`maxfac`.

    xcv : float, ndarray, shape (`maxfac`)
        `xcv`[j-1] contains the cumulative percentage of variance in the
        predictor variables explained by the first j factors, for j =
        1,2,...,`maxfac`.

    ycv : float, ndarray, shape (`maxfac`, my)
        `ycv`[i-1,j-1] is the cumulative percentage of variance of the
        jth response variable explained by the first i factors, for j =
        1,2,...,my, for i = 1,2,...,`maxfac`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n > 1.

        (`errno` 1)
            On entry, mx = *<value>*.

            Constraint: mx > 1.

        (`errno` 1)
            On entry, `isx`[*<value>*] is invalid.

            Constraint: `isx`[j-1] = 0 or 1, for all j.

        (`errno` 1)
            On entry, my = *<value>*.

            Constraint: my >= 1.

        (`errno` 1)
            On entry, `iscale` = *<value>*.

            Constraint: `iscale` = -1 or 1.

        (`errno` 2)
            On entry, ip = *<value>* and mx = *<value>*.

            Constraint: 1 < ip <= mx.

        (`errno` 2)
            On entry, `maxfac` = *<value>* and ip = *<value>*.

            Constraint: 1 <= `maxfac` <= ip.

        (`errno` 2)
            On entry, my = *<value>* and `maxit` = *<value>*.

            Constraint: if my > 1, `maxit` > 1.

        (`errno` 2)
            On entry, `tau` = *<value>*.

            Constraint: if my > 1, `tau` > 0.0.

        (`errno` 3)
            On entry, ip = *<value>* and sum(`isx`) = *<value>*.

            Constraint: the sum of elements in `isx` must equal ip.
    """

    from ..base.correg import pls_wold as b_pls_wold

    fun_name = 'naginterfaces.library.correg.pls_wold'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'), (_y, 'y')),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _mx_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _mx = _b_utils._EngineIntScalarType(
        _mx_py, {'fun_name': fun_name, 'entity': '_mx'},
    ).to_ctypes()
    _isx = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'isx'},
        isx,
        copy=False,
    )
    _my_py = _b_utils._EngineFloat64ArrayType.get_shape(_y, {'fun_name': fun_name, 'entity': 'y'}, exp_rank=2)[1]
    _my = _b_utils._EngineIntScalarType(
        _my_py, {'fun_name': fun_name, 'entity': '_my'},
    ).to_ctypes()
    _vl_ip_py = 0
    for _vl_i_py in range(1, _mx_py+1):
        if _b_utils._EngineIntArrayType.get_element(_isx, {'fun_name': fun_name, 'entity': 'isx'}, (_vl_i_py-1,)) == 1:
            _vl_ip_py = _vl_ip_py + 1
    _xbar = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_ip_py,),
    )
    _ybar = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_my_py,),
    )
    _xstd = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'xstd'},
        xstd,
        copy=True,
    )
    _ystd = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ystd'},
        ystd,
        copy=True,
    )
    _xres = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _vl_ip_py),
        sorder=_m_sorder,
    )
    _yres = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _my_py),
        sorder=_m_sorder,
    )
    _maxfac = _b_utils._EngineIntScalarType(
        maxfac, {'fun_name': fun_name, 'entity': 'maxfac'},
    ).to_ctypes()
    _maxfac_py = _b_utils._EngineIntScalarType.to_py(_maxfac)
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_ip_py, _maxfac_py),
        sorder=_m_sorder,
    )
    _p = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_ip_py, _maxfac_py),
        sorder=_m_sorder,
    )
    _t = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _maxfac_py),
        sorder=_m_sorder,
    )
    _c = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_my_py, _maxfac_py),
        sorder=_m_sorder,
    )
    _u = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _maxfac_py),
        sorder=_m_sorder,
    )
    _xcv = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_maxfac_py,),
    )
    _ycv = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_maxfac_py, _my_py),
        sorder=_m_sorder,
    )
    b_pls_wold(
        _n,
        _mx,
        _x,
        _isx,
        _my,
        _y,
        _xbar,
        _ybar,
        iscale,
        _xstd,
        _ystd,
        _maxfac,
        maxit,
        tau,
        _xres,
        _yres,
        _w,
        _p,
        _t,
        _c,
        _u,
        _xcv,
        _ycv,
        io_manager,
    )
    _xstd_py = _xstd
    _ystd_py = _ystd
    _xbar_py = _xbar
    _ybar_py = _ybar
    _xres_py = _xres
    _yres_py = _yres
    _w_py = _w
    _p_py = _p
    _t_py = _t
    _c_py = _c
    _u_py = _u
    _xcv_py = _xcv
    _ycv_py = _ycv
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['xbar', 'ybar', 'xstd', 'ystd', 'xres', 'yres', 'w', 'p', 't', 'c', 'u', 'xcv', 'ycv'],
    )(
        _xbar_py,
        _ybar_py,
        _xstd_py,
        _ystd_py,
        _xres_py,
        _yres_py,
        _w_py,
        _p_py,
        _t_py,
        _c_py,
        _u_py,
        _xcv_py,
        _ycv_py,
    )
    return _return_data

def pls_fit(nfact, p, c, w, rcond, orig, xbar, ybar, iscale, xstd, ystd, vipopt, ycv):
    u"""
    PLS parameter estimates following partial least squares regression
    by ``pls_svd`` or ``pls_wold``.

    For full information please refer to the NAG Library document for
    g02lc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02lcf.html

    Parameters
    ----------
    nfact : int
        l, the number of factors to include in the calculation of
        parameter estimates.

    p : float, array-like, shape (ip, maxfac)
        x-loadings as returned from ``pls_svd`` and ``pls_wold``.

    c : float, array-like, shape (my, maxfac)
        y-loadings as returned from ``pls_svd`` and ``pls_wold``.

    w : float, array-like, shape (ip, maxfac)
        x-weights as returned from ``pls_svd`` and ``pls_wold``.

    rcond : float
        Singular values of P^TW less than `rcond` times the maximum
        singular value are treated as zero when calculating parameter
        estimates. If `rcond` is negative, a value of 0.005 is used.

    orig : int
        Indicates how parameter estimates are calculated.

        `orig` = -1

          Parameter estimates for the centred, and possibly, scaled
          data.

        `orig` = 1

          Parameter estimates for the original data.

    xbar : float, array-like, shape (ip)
        If `orig` = 1, mean values of predictor variables in the model;
        otherwise `xbar` is not referenced.

    ybar : float, array-like, shape (my)
        If `orig` = 1, mean value of each response variable in the
        model; otherwise `ybar` is not referenced.

    iscale : int
        If `orig` = 1, `iscale` must take the value supplied to either
        ``pls_svd`` or ``pls_wold``; otherwise `iscale` is not
        referenced.

    xstd : float, array-like, shape (ip)
        If `orig` = 1 and `iscale` != -1, the scalings of predictor
        variables in the model as returned from either ``pls_svd`` or
        ``pls_wold``; otherwise `xstd` is not referenced.

    ystd : float, array-like, shape (my)
        If `orig` = 1 and `iscale` != -1, the scalings of response
        variables as returned from either ``pls_svd`` or ``pls_wold``;
        otherwise `ystd` is not referenced.

    vipopt : int
        A flag that determines variable influence on projections (VIP)
        options.

        `vipopt` = 0

          VIP are not calculated.

        `vipopt` = 1

          VIP are calculated for predictor variables using the mean
          explained variance in responses.

        `vipopt` = my

          VIP are calculated for predictor variables for each response
          variable in the model.

        Note that setting `vipopt` = my when my = 1 gives the same
        result as setting `vipopt` = 1 directly.

    ycv : float, array-like, shape (:, my)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `vipopt`!=0: `nfact`; otherwise: 0.

        If `vipopt` != 0, `ycv`[i-1,j-1] is the cumulative percentage of
        variance of the jth response variable explained by the first i
        factors, for j = 1,2,...,my, for i = 1,2,...,`nfact`; otherwise
        `ycv` is not referenced.

    Returns
    -------
    b : float, ndarray, shape (ip, my)
        `b`[i-1,j-1] contains the parameter estimate for the ith
        predictor variable in the model for the jth response variable,
        for j = 1,2,...,my, for i = 1,2,...,ip.

    ob : float, ndarray, shape (:, my)
        If `orig` = 1, `ob`[0,j-1] contains the intercept value for the
        jth response variable, and `ob`[i,j-1] contains the parameter
        estimate on the original scale for the ith predictor variable in
        the model, for j = 1,2,...,my, for i = 1,2,...,ip. Otherwise
        this argument is not referenced.

    vip : float, ndarray, shape (:, `vipopt`)
        If `vipopt` = 1, `vip`[i-1,0] contains the VIP statistic for the
        ith predictor variable in the model for all response variables,
        for i = 1,2,...,ip.

        If `vipopt` = my, `vip`[i-1,j-1] contains the VIP statistic for
        the ith predictor variable in the model for the jth response
        variable, for j = 1,2,...,my, for i = 1,2,...,ip.

        Otherwise this argument is not referenced.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, ip = *<value>*.

            Constraint: ip > 1.

        (`errno` 1)
            On entry, my = *<value>*.

            Constraint: my >= 1.

        (`errno` 1)
            On entry, `orig` = *<value>*.

            Constraint: `orig` = -1 or 1.

        (`errno` 1)
            On entry, `iscale` = *<value>*.

            Constraint: if `orig` = 1, `iscale` = -1 or 1.

        (`errno` 1)
            On entry, `vipopt` = *<value>* and my = *<value>*.

            Constraint: `vipopt` = 0, 1 or my.

        (`errno` 2)
            On entry, maxfac = *<value>* and ip = *<value>*.

            Constraint: 1 <= maxfac <= ip.

        (`errno` 2)
            On entry, `nfact` = *<value>* and maxfac = *<value>*.

            Constraint: 1 <= `nfact` <= maxfac.

        (`errno` 2)
            On entry, ldob = *<value>* and ip = *<value>*.

            Constraint: if `orig` = 1, ldob >= ip+1.

        (`errno` 2)
            On entry, ldycv = *<value>* and `nfact` = *<value>*.

            Constraint: if `vipopt` != 0, ldycv >= `nfact`.

        (`errno` 2)
            On entry, ldvip = *<value>* and ip = *<value>*.

            Constraint: if `vipopt` != 0, ldvip >= ip.

    See Also
    --------
    :meth:`naginterfaces.library.examples.correg.pls_ex.main`
    """

    from ..base.correg import pls_fit as b_pls_fit

    fun_name = 'naginterfaces.library.correg.pls_fit'

    _p = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'p'},
        p,
        copy=False,
    )
    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=False,
    )
    _w = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'w'},
        w,
        copy=False,
    )
    _ycv = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ycv'},
        ycv,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_p, 'p'), (_c, 'c'), (_w, 'w'), (_ycv, 'ycv')),
    )
    _ip_py = _b_utils._EngineFloat64ArrayType.get_shape(_p, {'fun_name': fun_name, 'entity': 'p'}, exp_rank=2)[0]
    _ip = _b_utils._EngineIntScalarType(
        _ip_py, {'fun_name': fun_name, 'entity': '_ip'},
    ).to_ctypes()
    _my_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[0]
    _my = _b_utils._EngineIntScalarType(
        _my_py, {'fun_name': fun_name, 'entity': '_my'},
    ).to_ctypes()
    _maxfac_py = _b_utils._EngineFloat64ArrayType.get_shape(_p, {'fun_name': fun_name, 'entity': 'p'}, exp_rank=2)[1]
    _maxfac = _b_utils._EngineIntScalarType(
        _maxfac_py, {'fun_name': fun_name, 'entity': '_maxfac'},
    ).to_ctypes()
    _b = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_ip_py, _my_py),
        sorder=_m_sorder,
    )
    _xbar = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'xbar'},
        xbar,
        copy=False,
    )
    _ybar = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ybar'},
        ybar,
        copy=False,
    )
    _xstd = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'xstd'},
        xstd,
        copy=False,
    )
    _ystd = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ystd'},
        ystd,
        copy=False,
    )
    _orig = _b_utils._EngineIntScalarType(
        orig, {'fun_name': fun_name, 'entity': 'orig'},
    ).to_ctypes()
    _orig_py = _b_utils._EngineIntScalarType.to_py(_orig)
    if _orig_py == 1:
        _ob_dim1 = _ip_py+1
    else:
        _ob_dim1 = 1
    exp_shape_ob = (_ob_dim1, _my_py)
    _ob = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_ob,
        sorder=_m_sorder,
    )
    _vipopt = _b_utils._EngineIntScalarType(
        vipopt, {'fun_name': fun_name, 'entity': 'vipopt'},
    ).to_ctypes()
    _vipopt_py = _b_utils._EngineIntScalarType.to_py(_vipopt)
    if _vipopt_py != 0:
        _vip_dim1 = _ip_py
    else:
        _vip_dim1 = 0
    exp_shape_vip = (_vip_dim1, _vipopt_py)
    _vip = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_vip,
        sorder=_m_sorder,
    )
    b_pls_fit(
        _ip,
        _my,
        _maxfac,
        nfact,
        _p,
        _c,
        _w,
        rcond,
        _b,
        _orig,
        _xbar,
        _ybar,
        iscale,
        _xstd,
        _ystd,
        _ob,
        _vipopt,
        _ycv,
        _vip,
    )
    _b_py = _b
    _ob_py = _ob
    _vip_py = _vip
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['b', 'ob', 'vip'],
    )(
        _b_py,
        _ob_py,
        _vip_py,
    )
    return _return_data

def pls_pred(orig, xbar, ybar, iscale, xstd, ystd, b, isz, z):
    u"""
    PLS predictions based on parameter estimates from ``pls_fit``.

    For full information please refer to the NAG Library document for
    g02ld

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02ldf.html

    Parameters
    ----------
    orig : int
        Indicates how parameter estimates are supplied.

        `orig` = 1

          Parameter estimates are for the original data.

        `orig` = -1

          Parameter estimates are for the centred, and possibly scaled,
          data.

    xbar : float, array-like, shape (ip)
        If `orig` = -1, `xbar` must contain mean values of predictor
        variables in the model; otherwise `xbar` is not referenced.

    ybar : float, array-like, shape (my)
        If `orig` = -1, `ybar` must contain the mean value of each
        response variable in the model; otherwise `ybar` is not
        referenced.

    iscale : int
        If `orig` = -1, `iscale` must take the value supplied to either
        ``pls_svd`` or ``pls_wold``; otherwise `iscale` is not
        referenced.

    xstd : float, array-like, shape (ip)
        If `orig` = -1 and `iscale` != -1, `xstd` must contain the
        scalings of predictor variables in the model as returned from
        either ``pls_svd`` or ``pls_wold``; otherwise `xstd` is not
        referenced.

    ystd : float, array-like, shape (my)
        If `orig` = -1 and `iscale` != -1, `ystd` must contain the
        scalings of response variables as returned from either
        ``pls_svd`` or ``pls_wold``; otherwise `ystd` is not referenced.

    b : float, array-like, shape (:, my)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `orig`=-1: ip; if `orig`=1: 1+ip;
        otherwise: 0.

        If `orig` = -1, `b` must contain the parameter estimate for the
        centred, and possibly scaled, data as returned by ``pls_fit``;
        otherwise `b` must contain the parameter estimates for the
        original data as returned by ``pls_fit``.

    isz : int, array-like, shape (mz)
        Indicates which predictor variables are to be included in the
        model. Predictor variables included from `z` must be in the same
        order as those included in the fitted model.

        If `isz`[j-1] = 1, the jth predictor variable is included in the
        model, for j = 1,2,...,mz, otherwise `isz`[j-1] = 0.

    z : float, array-like, shape (n, mz)
        `z`[i-1,j-1] contains the ith observation on the jth available
        predictor variable, for j = 1,2,...,mz, for i = 1,2,...,n.

    Returns
    -------
    yhat : float, ndarray, shape (n, my)
        `yhat`[i-1,j-1] contains the ith predicted value of the jth
        y-variable in the model.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, ip = *<value>*.

            Constraint: ip > 1.

        (`errno` 1)
            On entry, my = *<value>*.

            Constraint: my >= 1.

        (`errno` 1)
            On entry, `orig` = *<value>*.

            Constraint: `orig` = -1 or 1.

        (`errno` 1)
            On entry, `iscale` = *<value>*.

            Constraint: if `orig` = -1, `iscale` = -1, 1 or 2.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 1)
            On entry, `isz`[*<value>*] = *<value>*.

            Constraint: `isz`[j-1] = 0 or 1.

        (`errno` 2)
            On entry, ldb = *<value>* and ip+1 = *<value>*.

            Constraint: if `orig` = 1, ldb >= 1+ip.

        (`errno` 2)
            On entry, ldb = *<value>* and ip = *<value>*.

            Constraint: if `orig` = -1, ldb >= ip.

        (`errno` 2)
            On entry, mz = *<value>* and ip = *<value>*.

            Constraint: mz >= ip.

        (`errno` 3)
            On entry, the number of elements of `isz` equal to 1 is not
            ip.

    See Also
    --------
    :meth:`naginterfaces.library.examples.correg.pls_ex.main`
    """

    from ..base.correg import pls_pred as b_pls_pred

    fun_name = 'naginterfaces.library.correg.pls_pred'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
    )
    _z = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'z'},
        z,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_b, 'b'), (_z, 'z')),
    )
    _xbar = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'xbar'},
        xbar,
        copy=False,
    )
    _ip_py = _b_utils._EngineFloat64ArrayType.get_shape(_xbar, {'fun_name': fun_name, 'entity': 'xbar'}, exp_rank=1)[0]
    _ip = _b_utils._EngineIntScalarType(
        _ip_py, {'fun_name': fun_name, 'entity': '_ip'},
    ).to_ctypes()
    _ybar = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ybar'},
        ybar,
        copy=False,
    )
    _my_py = _b_utils._EngineFloat64ArrayType.get_shape(_ybar, {'fun_name': fun_name, 'entity': 'ybar'}, exp_rank=1)[0]
    _my = _b_utils._EngineIntScalarType(
        _my_py, {'fun_name': fun_name, 'entity': '_my'},
    ).to_ctypes()
    _xstd = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'xstd'},
        xstd,
        copy=False,
    )
    _ystd = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ystd'},
        ystd,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _isz = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'isz'},
        isz,
        copy=False,
    )
    _mz_py = _b_utils._EngineIntArrayType.get_shape(_isz, {'fun_name': fun_name, 'entity': 'isz'}, exp_rank=1)[0]
    _mz = _b_utils._EngineIntScalarType(
        _mz_py, {'fun_name': fun_name, 'entity': '_mz'},
    ).to_ctypes()
    _yhat = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _my_py),
        sorder=_m_sorder,
    )
    b_pls_pred(
        _ip,
        _my,
        orig,
        _xbar,
        _ybar,
        iscale,
        _xstd,
        _ystd,
        _b,
        _n,
        _mz,
        _isz,
        _z,
        _yhat,
    )
    _yhat_py = _yhat
    return _yhat_py

def lars(mtype, d, y, pred=3, prey=1, isx=None, mnstep=None, ropt=None, io_manager=None):
    u"""
    Least angle regression (LARS), least absolute shrinkage and
    selection operator (LASSO) and forward stagewise regression.

    For full information please refer to the NAG Library document for
    g02ma

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02maf.html

    Parameters
    ----------
    mtype : int
        Indicates the type of model to fit.

        `mtype` = 1

          LARS is performed.

        `mtype` = 2

          Forward linear stagewise regression is performed.

        `mtype` = 3

          LASSO model is fit.

        `mtype` = 4

          A positive LASSO model is fit.

    d : float, array-like, shape (n, m)
        D, the data, which along with `pred` and `isx`, defines the
        design matrix X. The ith observation for the jth variable must
        be supplied in `d`[i-1,j-1], for j = 1,2,...,m, for i =
        1,2,...,n.

    y : float, array-like, shape (n)
        y, the observations on the dependent variable.

    pred : int, optional
        Indicates the type of data preprocessing to perform on the
        independent variables supplied in `d` to comply with the
        standardized form of the design matrix.

        `pred` = 0

          No preprocessing is performed.

        `pred` = 1

          Each of the independent variables, x_j, for j = 1,2,...,p, are
          mean centred prior to fitting the model. The means of the
          independent variables, x_, are returned in `b`, with x__j =
          `b`[j-1,`nstep`+1], for j = 1,2,...,p.

        `pred` = 2

          Each independent variable is normalized, with the jth variable
          scaled by 1/sqrt(x_j^Tx_j). The scaling factor used by
          variable j is returned in `b`[j-1,`nstep`].

        `pred` = 3

          As `pred` = 1 or 2, all of the independent variables are mean
          centred prior to being normalized.

    prey : int, optional
        Indicates the type of data preprocessing to perform on the
        dependent variable supplied in `y`.

        `prey` = 0

          No preprocessing is performed, this is equivalent to setting
          alpha = 0.

        `prey` = 1

          The dependent variable, y, is mean centred prior to fitting
          the model, so alpha = y_. Which is equivalent to fitting a
          non-penalized intercept to the model and the degrees of
          freedom etc. are adjusted accordingly.

        The value of alpha used is returned in `fitsum`[0,`nstep`].

    isx : None or int, array-like, shape (lisx), optional
        Indicates which independent variables from `d` will be included
        in the design matrix, X.

        If `isx` is None, all variables are included in the design
        matrix.

        Otherwise `isx`[j-1] must be set as follows, for j = 1,2,...,m:

        `isx`[j-1] = 1

          To indicate that the jth variable, as supplied in `d`, is
          included in the design matrix;

        `isx`[j-1] = 0

          To indicated that the jth variable, as supplied in `d`, is not
          included in the design matrix;

        and p = sum(`isx`[j-1]).

    mnstep : int, optional
        Note: if this argument is None then a default value will be
        used, determined as follows: if `mtype` = 1: `m`; otherwise:
        200*`m`.

        The maximum number of steps to carry out in the model fitting
        process.

        If `mtype` = 1, i.e., a LARS is being performed, the maximum
        number of steps the algorithm will take is min(p,n) if `prey` =
        0, otherwise min(p,n-1).

        If `mtype` = 2, i.e., a forward linear stagewise regression is
        being performed, the maximum number of steps the algorithm will
        take is likely to be several orders of magnitude more and is no
        longer bound by p or n.

        If `mtype` = 3 or 4, i.e., a LASSO or positive LASSO model is
        being fit, the maximum number of steps the algorithm will take
        lies somewhere between that of the LARS and forward linear
        stagewise regression, again it is no longer bound by p or n.

    ropt : None or float, array-like, shape (lropt), optional
        Options to control various aspects of the LARS algorithm.

        The default value will be used for `ropt`[i-1] if the length of
        `ropt` is less than i, therefore to use the default values for
        all options `ropt` need not be set and may be None.

        The default value will also be used if an invalid value is
        supplied for a particular argument, for example, setting
        `ropt`[i-1] = -1 will use the default value for argument i.

        `ropt`[0]

          The minimum step size that will be taken.

          Default is 100*eps is used, where eps is the machine precision
          returned by ``machine.precision``.

        `ropt`[1]

          General tolerance, used amongst other things, for comparing
          correlations.

          Default is `ropt`[0].

        `ropt`[2]

          If set to 1, parameter estimates are rescaled before being
          returned.

          If set to 0, no rescaling is performed.

          This argument has no effect when `pred` = 0 or 1.

          Default is for the parameter estimates to be rescaled.

        `ropt`[3]

          If set to 1, it is assumed that the model contains an
          intercept during the model fitting process and when
          calculating the degrees of freedom.

          If set to 0, no intercept is assumed.

          This has no effect on the amount of preprocessing performed on
          `y`.

          Default is to treat the model as having an intercept when
          `prey` = 1 and as not having an intercept when `prey` = 0.

        `ropt`[4]

          As implemented, the LARS algorithm can either work directly
          with y and X, or it can work with the cross-product matrices,
          X^Ty and X^TX. In most cases it is more efficient to work with
          the cross-product matrices. This flag allows you direct
          control over which method is used, however, the default value
          will usually be the best choice.

          If `ropt`[4] = 1, y and X are worked with directly.

          If `ropt`[4] = 0, the cross-product matrices are used.

          Default is 1 when p >= 500 and n < p and 0 otherwise.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    ip : int
        p, number of parameter estimates.

        If `isx` is None, p = m, i.e., the number of variables in `d`.

        Otherwise p is the number of nonzero values in `isx`.

    nstep : int
        K, the actual number of steps carried out in the model fitting
        process.

    b : float, ndarray, shape (`ip`, `nstep`+2)
        beta the parameter estimates, with `b`[j-1,k-1] = beta_{kj}, the
        parameter estimate for the jth variable, j = 1,2,...,p at the
        kth step of the model fitting process, k = 1,2,...,`nstep`.

        By default, when `pred` = 2 or 3 the parameter estimates are
        rescaled prior to being returned.

        If the parameter estimates are required on the normalized scale,
        then this can be overridden via `ropt`.

    fitsum : float, ndarray, shape (6, `mnstep`+1)
        Summaries of the model fitting process. When k =
        1,2,...,`nstep`,

        `fitsum`[0,k-1]

          ||beta_k||_1, the sum of the absolute values of the parameter
          estimates for the kth step of the modelling fitting process.
          If `pred` = 2 or 3, the scaled parameter estimates are used in
          the summation.

        `fitsum`[1,k-1]

          RSS_k, the residual sums of squares for the kth step, where
          RSS_k = ||y-X^T beta_k||^2.

        `fitsum`[2,k-1]

          nu_k, approximate degrees of freedom for the kth step.

        `fitsum`[3,k-1]

          C_p^(k), a C_p-type statistic for the kth step, where C_p^(k)
          = (RSS_k)/(sigma^2)-n+2*nu_k.

        `fitsum`[4,k-1]

          C^_k, correlation between the residual at step k-1 and the
          most correlated variable not yet in the active set A, where
          the residual at step 0 is y.

        `fitsum`[5,k-1]

          gamma^_k, the step size used at step k.

        In addition

        `fitsum`[0,`nstep`]

          alpha, with alpha = y_ if `prey` = 1 and 0 otherwise.

        `fitsum`[1,`nstep`]

          RSS_0, the residual sums of squares for the null model, where
          RSS_0 = y^Ty when `prey` = 0 and RSS_0 = (y-y_)^T(y-y_)
          otherwise.

        `fitsum`[2,`nstep`]

          nu_0, the degrees of freedom for the null model, where nu_0 =
          0 if `prey` = 0 and nu_0 = 1 otherwise.

        `fitsum`[3,`nstep`]

          C_p^(0), a C_p-type statistic for the null model, where
          C_p^(0) = (RSS_0)/(sigma^2)-n+2*nu_0.

        `fitsum`[4,`nstep`]

          sigma^2, where sigma^2 = (n-RSS_K)/(nu_K) and K = `nstep`.

        Although the C_p statistics described above are returned when
        `errno` = 112 they may not be meaningful due to the estimate
        sigma^2 not being based on the saturated model.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, `mtype` = *<value>*.

            Constraint: `mtype` = 1, 2, 3 or 4.

        (`errno` 21)
            On entry, `pred` = *<value>*.

            Constraint: `pred` = 0, 1, 2 or 3.

        (`errno` 31)
            On entry, `prey` = *<value>*.

            Constraint: `prey` = 0 or 1.

        (`errno` 41)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 51)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 81)
            On entry, `isx`[*<value>*] = *<value>*.

            Constraint: `isx`[i] = 0 or 1, for all i.

        (`errno` 82)
            On entry, all values of `isx` are zero.

            Constraint: at least one value of `isx` must be nonzero.

        (`errno` 91)
            On entry, lisx = *<value>* and m = *<value>*.

            Constraint: lisx = 0 or m.

        (`errno` 111)
            On entry, `mnstep` = *<value>*.

            Constraint: `mnstep` >= 1.

        (`errno` 151)
            On entry, ldb = *<value>* and m = *<value>*.

            Constraint: if lisx = 0 then ldb >= m.

        (`errno` 152)
            On entry, ldb = *<value>* and p = *<value>*.

            Constraint: if lisx = m then ldb >= p.

        (`errno` 181)
            On entry, lropt = *<value>*.

            Constraint: 0 <= lropt <= 5.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 112)
            Fitting process did not finish in `mnstep` steps. Try
            increasing the size of `mnstep` and supplying larger output
            arrays.

            All output is returned as documented, up to step `mnstep`,
            however, sigma and the C_p statistics may not be meaningful.

        (`errno` 161)
            sigma^2 is approximately zero and hence the C_p-type
            criterion cannot be calculated. All other output is returned
            as documented.

        (`errno` 162)
            nu_K = n, therefore sigma has been set to a large value.
            Output is returned as documented.

        (`errno` 163)
            Degenerate model, no variables added and `nstep` = 0. Output
            is returned as documented.

    See Also
    --------
    :meth:`naginterfaces.library.examples.correg.lars_ex.main`
    """

    from ..base.correg import lars as b_lars

    fun_name = 'naginterfaces.library.correg.lars'

    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_d, 'd'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _isx = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'isx'},
        isx,
        copy=False,
        allow_none=True,
    )
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _mtype = _b_utils._EngineIntScalarType(
        mtype, {'fun_name': fun_name, 'entity': 'mtype'},
    ).to_ctypes()
    _mtype_py = _b_utils._EngineIntScalarType.to_py(_mtype)
    if _mtype_py == 1:
        _mnstep = _m_py
    else:
        _mnstep = 200*_m_py
    _mnstep = _mnstep if mnstep is None else mnstep
    if isx is None:
        _vl_p_py = _m_py
    else:
        _vl_p_py = 0
        for _vl_j_py in range(1, _m_py+1):
            if _b_utils._EngineIntArrayType.get_element(_isx, {'fun_name': fun_name, 'entity': 'isx'}, (_vl_j_py-1,)) == 1:
                _vl_p_py = _vl_p_py + 1
    _mnstep = _b_utils._EngineIntScalarType(
        _mnstep, {'fun_name': fun_name, 'entity': '_mnstep'},
    ).to_ctypes()
    _mnstep_py = _b_utils._EngineIntScalarType.to_py(_mnstep)
    _b = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_p_py, _mnstep_py+2),
        sorder=_m_sorder,
    )
    _fitsum = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (6, _mnstep_py+1),
        sorder=_m_sorder,
    )
    _ropt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ropt'},
        ropt,
        copy=False,
        allow_none=True,
    )
    _lropt_py = _b_utils._EngineFloat64ArrayType.get_shape(_ropt, {'fun_name': fun_name, 'entity': 'ropt'}, exp_rank=1, allow_none=True)[0]
    _lropt = _b_utils._EngineIntScalarType(
        _lropt_py, {'fun_name': fun_name, 'entity': '_lropt'},
    ).to_ctypes()
    try:
        _return_scalars = b_lars(
            _mtype,
            pred,
            prey,
            _n,
            _m,
            _d,
            _isx,
            _y,
            _mnstep,
            _b,
            _fitsum,
            _ropt,
            _lropt,
            io_manager,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _ip_py = _return_scalars.ip
    _nstep_py = _return_scalars.nstep
    _b_py = _b[:_ip_py, :_nstep_py+2]
    _fitsum_py = _fitsum[:, :_nstep_py+1]
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['ip', 'nstep', 'b', 'fitsum'],
    )(
        _ip_py,
        _nstep_py,
        _b_py,
        _fitsum_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def lars_xtx(mtype, n, dtd, dty, yty, pred=2, intcpt=1, isx=None, mnstep=None, ropt=None, io_manager=None):
    u"""
    Least Angle Regression (LARS), Least Absolute Shrinkage and
    Selection Operator (LASSO) and forward stagewise regression using
    the cross-products matrix.

    For full information please refer to the NAG Library document for
    g02mb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02mbf.html

    Parameters
    ----------
    mtype : int
        Indicates the type of model to fit.

        `mtype` = 1

          LARS is performed.

        `mtype` = 2

          Forward linear stagewise regression is performed.

        `mtype` = 3

          LASSO model is fit.

        `mtype` = 4

          A positive LASSO model is fit.

    n : int
        n, the number of observations.

    dtd : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `dtd`.shape[0]=1: 1; if
        `dtd`.shape[1]=1: m*(m+1)/2; otherwise: m.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `dtd`.shape[0]=1: m*(m+1)/2;
        if `dtd`.shape[1]=1: 1; otherwise: m.

        D^TD, the cross-product matrix, which along with `isx`, defines
        the design matrix cross-product X^TX.

        If the cross-product matrix is packed into a single row or
        column of `dtd`, `dtd`[0,i*(i-1)/2+j-1] or
        `dtd`[i*(i-1)/2+j-1,0] must contain the cross-product of the ith
        and jth variable, for j = 1,2,...,m, for i = 1,2,...,m. That is
        the cross-product stacked by columns as returned by ``ssqmat``,
        for example.

        Otherwise `dtd`[i-1,j-1] must contain the cross-product of the
        ith and jth variable, for j = 1,2,...,m, for i = 1,2,...,m.

        It should be noted that, even though D^TD is symmetric, the full
        matrix must be supplied.

        The matrix specified in `dtd` must be a valid cross-products
        matrix.

    dty : float, array-like, shape (m)
        D^Ty, the cross-product between the dependent variable, y, and
        the independent variables D.

    yty : float
        y^Ty, the sums of squares of the dependent variable.

    pred : int, optional
        Indicates the type of preprocessing to perform on the
        cross-products involving the independent variables, i.e., those
        supplied in `dtd` and `dty`.

        `pred` = 0

          No preprocessing is performed.

        `pred` = 2

          Each independent variable is normalized, with the jth variable
          scaled by 1/sqrt(x_j^Tx_j). The scaling factor used by
          variable j is returned in `b`[j-1,`nstep`].

    intcpt : int, optional
        Indicates the type of data preprocessing that was perform on the
        dependent variable, y, prior to calling this function.

        `intcpt` = 0

          No preprocessing was performed.

        `intcpt` = 1

          The dependent variable, y, was mean centred.

    isx : None or int, array-like, shape (lisx), optional
        Indicates which independent variables from `dtd` will be
        included in the design matrix, X.

        If `isx` is None, all variables are included in the design
        matrix.

        Otherwise `isx`[j-1] must be set as follows, for j = 1,2,...,m:

        `isx`[j-1] = 1

          To indicate that the jth variable, as supplied in `dtd`, is
          included in the design matrix;

        `isx`[j-1] = 0

          To indicate that the jth variable, as supplied in `dtd`, is
          not included in the design matrix;

        and p = sum(`isx`[j-1]).

    mnstep : int, optional
        Note: if this argument is None then a default value will be
        used, determined as follows: if `mtype` = 1: `m`; otherwise:
        200*`m`.

        The maximum number of steps to carry out in the model fitting
        process.

        If `mtype` = 1, i.e., a LARS is being performed, the maximum
        number of steps the algorithm will take is min(p,n) if `intcpt`
        = 0, otherwise min(p,n-1).

        If `mtype` = 2, i.e., a forward linear stagewise regression is
        being performed, the maximum number of steps the algorithm will
        take is likely to be several orders of magnitude more and is no
        longer bound by p or n.

        If `mtype` = 3 or 4, i.e., a LASSO or positive LASSO model is
        being fit, the maximum number of steps the algorithm will take
        lies somewhere between that of the LARS and forward linear
        stagewise regression, again it is no longer bound by p or n.

    ropt : None or float, array-like, shape (lropt), optional
        Options to control various aspects of the LARS algorithm.

        The default value will be used for `ropt`[i-1] if lropt < i,
        therefore setting lropt = 0 will use the default values for all
        options and `ropt` need not be set and may be None.

        The default value will also be used if an invalid value is
        supplied for a particular argument, for example, setting
        `ropt`[i-1] = -1 will use the default value for argument i.

        `ropt`[0]

          The minimum step size that will be taken.

          Default is 100*eps is used, where eps is the machine precision
          returned by ``machine.precision``.

        `ropt`[1]

          General tolerance, used amongst other things, for comparing
          correlations.

          Default is `ropt`[0].

        `ropt`[2]

          If set to 1, parameter estimates are rescaled before being
          returned. If set to 0, no rescaling is performed. This
          argument has no effect when `pred` = 0.

          Default is for the parameter estimates to be rescaled.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    ip : int
        p, number of parameter estimates.

        If `isx` is None, p = m, i.e., the number of variables in `dtd`.

        Otherwise p is the number of nonzero values in `isx`.

    nstep : int
        K, the actual number of steps carried out in the model fitting
        process.

    b : float, ndarray, shape (`ip`, `nstep`+1)
        beta the parameter estimates, with `b`[j-1,k-1] = beta_{kj}, the
        parameter estimate for the jth variable, j = 1,2,...,p at the
        kth step of the model fitting process, k = 1,2,...,`nstep`.

        By default, when `pred` = 2 the parameter estimates are rescaled
        prior to being returned.

        If the parameter estimates are required on the normalized scale,
        then this can be overridden via `ropt`.

        The values held in the remaining part of `b` depend on the type
        of preprocessing performed.

        [table omitted]

        for j = 1,2,...p.

    fitsum : float, ndarray, shape (6, `mnstep`+1)
        Summaries of the model fitting process. When k = 1,2,...,`nstep`

        `fitsum`[0,k-1]

          ||beta_k||_1, the sum of the absolute values of the parameter
          estimates for the kth step of the modelling fitting process.
          If `pred` = 2, the scaled parameter estimates are used in the
          summation.

        `fitsum`[1,k-1]

          RSS_k, the residual sums of squares for the kth step, where
          RSS_k = ||y-X^T beta_k||^2.

        `fitsum`[2,k-1]

          nu_k, approximate degrees of freedom for the kth step.

        `fitsum`[3,k-1]

          C_p^(k), a C_p-type statistic for the kth step, where C_p^(k)
          = (RSS_k)/(sigma^2)-n+2*nu_k.

        `fitsum`[4,k-1]

          C^_k, correlation between the residual at step k-1 and the
          most correlated variable not yet in the active set A, where
          the residual at step 0 is y.

        `fitsum`[5,k-1]

          gamma^_k, the step size used at step k.

        In addition

        `fitsum`[0,`nstep`]

          0.

        `fitsum`[1,`nstep`]

          RSS_0, the residual sums of squares for the null model, where
          RSS_0 = y^Ty.

        `fitsum`[2,`nstep`]

          nu_0, the degrees of freedom for the null model, where nu_0 =
          0 if `intcpt` = 0 and nu_0 = 1 otherwise.

        `fitsum`[3,`nstep`]

          C_p^(0), a C_p-type statistic for the null model, where
          C_p^(0) = (RSS_0)/(sigma^2)-n+2*nu_0.

        `fitsum`[4,`nstep`]

          sigma^2, where sigma^2 = (n-RSS_K)/(nu_K) and K = `nstep`.

        Although the C_p statistics described above are returned when
        `errno` = 122 they may not be meaningful due to the estimate
        sigma^2 not being based on the saturated model.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, `mtype` = *<value>*.

            Constraint: `mtype` = 1, 2, 3 or 4.

        (`errno` 21)
            On entry, `pred` = *<value>*.

            Constraint: `pred` = 0 or 2.

        (`errno` 31)
            On entry, `intcpt` = *<value>*.

            Constraint: `intcpt` = 0 or 1.

        (`errno` 41)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 51)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 61)
            The cross-product matrix supplied in `dtd` is not symmetric.

        (`errno` 62)
            On entry, `dtd`[0,*<value>*] = *<value>*.

            Constraint: diagonal elements of D^TD must be positive.

        (`errno` 62)
            On entry, i = *<value>* and `dtd`[i-1,i-1] = *<value>*.

            Constraint: diagonal elements of D^TD must be positive.

        (`errno` 81)
            On entry, `isx`[*<value>*] = *<value>*.

            Constraint: `isx`[i] = 0 or 1, for all i.

        (`errno` 82)
            On entry, all values of `isx` are zero.

            Constraint: at least one value of `isx` must be nonzero.

        (`errno` 91)
            On entry, lisx = *<value>* and m = *<value>*.

            Constraint: lisx = 0 or m.

        (`errno` 111)
            On entry, `yty` = *<value>*.

            Constraint: `yty` > 0.0.

        (`errno` 112)
            A negative value for the residual sums of squares was
            obtained. Check the values of `dtd`, `dty` and `yty`.

        (`errno` 121)
            On entry, `mnstep` = *<value>*.

            Constraint: `mnstep` >= 1.

        (`errno` 191)
            On entry, lropt = *<value>*.

            Constraint: 0 <= lropt <= 3.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 122)
            Fitting process did not finished in `mnstep` steps. Try
            increasing the size of `mnstep` and supplying larger output
            arrays.

            All output is returned as documented, up to step `mnstep`,
            however, sigma and the C_p statistics may not be meaningful.

        (`errno` 171)
            sigma^2 is approximately zero and hence the C_p-type
            criterion cannot be calculated. All other output is returned
            as documented.

        (`errno` 172)
            nu_K = n, therefore sigma has been set to a large value.
            Output is returned as documented.

        (`errno` 173)
            Degenerate model, no variables added and `nstep` = 0. Output
            is returned as documented.

    See Also
    --------
    :meth:`naginterfaces.library.examples.correg.lars_param_ex.main`
    """

    from ..base.correg import lars_xtx as b_lars_xtx

    fun_name = 'naginterfaces.library.correg.lars_xtx'

    _dtd = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'dtd'},
        dtd,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_dtd, 'dtd'),),
    )
    _dty = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'dty'},
        dty,
        copy=False,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_dty, {'fun_name': fun_name, 'entity': 'dty'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _isx = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'isx'},
        isx,
        copy=False,
        allow_none=True,
    )
    _mtype = _b_utils._EngineIntScalarType(
        mtype, {'fun_name': fun_name, 'entity': 'mtype'},
    ).to_ctypes()
    _mtype_py = _b_utils._EngineIntScalarType.to_py(_mtype)
    if _mtype_py == 1:
        _mnstep = _m_py
    else:
        _mnstep = 200*_m_py
    _mnstep = _mnstep if mnstep is None else mnstep
    if isx is None:
        _vl_p_py = _m_py
    else:
        _vl_p_py = 0
        for _vl_j_py in range(1, _m_py+1):
            if _b_utils._EngineIntArrayType.get_element(_isx, {'fun_name': fun_name, 'entity': 'isx'}, (_vl_j_py-1,)) == 1:
                _vl_p_py = _vl_p_py + 1
    _mnstep = _b_utils._EngineIntScalarType(
        _mnstep, {'fun_name': fun_name, 'entity': '_mnstep'},
    ).to_ctypes()
    _mnstep_py = _b_utils._EngineIntScalarType.to_py(_mnstep)
    _b = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_vl_p_py, _mnstep_py+1),
        sorder=_m_sorder,
    )
    _fitsum = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (6, _mnstep_py+1),
        sorder=_m_sorder,
    )
    _ropt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ropt'},
        ropt,
        copy=False,
        allow_none=True,
    )
    _lropt_py = _b_utils._EngineFloat64ArrayType.get_shape(_ropt, {'fun_name': fun_name, 'entity': 'ropt'}, exp_rank=1, allow_none=True)[0]
    _lropt = _b_utils._EngineIntScalarType(
        _lropt_py, {'fun_name': fun_name, 'entity': '_lropt'},
    ).to_ctypes()
    try:
        _return_scalars = b_lars_xtx(
            _mtype,
            pred,
            intcpt,
            n,
            _m,
            _dtd,
            _isx,
            _dty,
            yty,
            _mnstep,
            _b,
            _fitsum,
            _ropt,
            _lropt,
            io_manager,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _ip_py = _return_scalars.ip
    _nstep_py = _return_scalars.nstep
    _b_py = _b[:_ip_py, :_nstep_py+1]
    _fitsum_py = _fitsum[:, :_nstep_py+1]
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['ip', 'nstep', 'b', 'fitsum'],
    )(
        _ip_py,
        _nstep_py,
        _b_py,
        _fitsum_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def lars_param(b, fitsum, ktype, nk):
    u"""
    Calculates additional parameter estimates following Least Angle
    Regression (LARS), Least Absolute Shrinkage and Selection Operator
    (LASSO) or forward stagewise regression.

    For full information please refer to the NAG Library document for
    g02mc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02mcf.html

    Parameters
    ----------
    b : float, array-like, shape (ip, nstep+1)
        beta the parameter estimates, as returned by ``lars`` and
        ``lars_xtx``, with `b`[j-1,k-1] = beta_{kj}, the parameter
        estimate for the jth variable, for j = 1,2,...,p, at the kth
        step of the model fitting process.

    fitsum : float, array-like, shape (6, nstep+1)
        Summaries of the model fitting process, as returned by ``lars``
        and ``lars_xtx``.

    ktype : int
        Indicates what target values are held in `nk`.

        `ktype` = 1

          `nk` holds (fractional) LARS step numbers.

        `ktype` = 2

          `nk` holds values for L_1 norm of the (scaled) parameters.

        `ktype` = 3

          `nk` holds ratios with respect to the largest (scaled) L_1
          norm.

        `ktype` = 4

          `nk` holds values for the L_1 norm of the (unscaled)
          parameters.

        `ktype` = 5

          `nk` holds ratios with respect to the largest (unscaled) L_1
          norm.

        If ``lars`` was called with pred = 0 or 1 or ``lars_xtx`` was
        called with pred = 0 then the model fitting routine did not
        rescale the independent variables, X, prior to fitting the model
        and therefore there is no difference between `ktype` = 2 or 3
        and `ktype` = 4 or 5.

    nk : float, array-like, shape (lnk)
        Target values used for predicting the new set of parameter
        estimates.

    Returns
    -------
    nb : float, ndarray, shape (ip, lnk)
        beta~ the predicted parameter estimates, with `b`[j-1,i-1] =
        beta~_{ij}, the parameter estimate for variable j, j = 1,2,...,p
        at the point in the fitting process associated with `nk`[i-1], i
        = 1,2,...,lnk.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, nstep = *<value>*.

            Constraint: nstep >= 0.

        (`errno` 21)
            On entry, ip = *<value>*.

            Constraint: ip >= 1.

        (`errno` 31)
            `b` has been corrupted since the last call to ``lars`` or
            ``lars_xtx``.

        (`errno` 51)
            `fitsum` has been corrupted since the last call to ``lars``
            or ``lars_xtx``.

        (`errno` 61)
            On entry, `ktype` = *<value>*.

            Constraint: `ktype` = 1, 2, 3, 4 or 5.

        (`errno` 71)
            On entry, `ktype` = 1, `nk`[*<value>*] = *<value>* and nstep
            = *<value>*

            Constraint: 0 <= `nk`[i] <= nstep, for all i.

        (`errno` 72)
            On entry, `ktype` = 2, `nk`[*<value>*] = *<value>*, nstep =
            *<value>* and `fitsum`[0,nstep-1] = *<value>*.

            Constraint: 0 <= `nk`[i] <= `fitsum`[0,nstep-1], for all i.

        (`errno` 73)
            On entry, `ktype` = 3 or 5, `nk`[*<value>*] = *<value>*.

            Constraint: 0 <= `nk`[i] <= 1, for all i.

        (`errno` 74)
            On entry, `ktype` = 4, `nk`[*<value>*] = *<value>* and
            ||beta_K||_1 = *<value>*

            Constraint: 0 <= `nk`[i] <= ||beta_K||_1, for all i.

        (`errno` 81)
            On entry, lnk = *<value>*.

            Constraint: lnk >= 1.

    See Also
    --------
    :meth:`naginterfaces.library.examples.correg.lars_param_ex.main`
    """

    from ..base.correg import lars_param as b_lars_param

    fun_name = 'naginterfaces.library.correg.lars_param'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
    )
    _fitsum = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'fitsum'},
        fitsum,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_b, 'b'), (_fitsum, 'fitsum')),
    )
    _nstep = _b_utils._EngineIntScalarType(
        _b_utils._EngineFloat64ArrayType.get_shape(_fitsum, {'fun_name': fun_name, 'entity': 'fitsum'}, exp_rank=2)[1]-1, {'fun_name': fun_name, 'entity': '_nstep'},
    ).to_ctypes()
    _ip_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _ip = _b_utils._EngineIntScalarType(
        _ip_py, {'fun_name': fun_name, 'entity': '_ip'},
    ).to_ctypes()
    _nk = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'nk'},
        nk,
        copy=False,
    )
    _lnk_py = _b_utils._EngineFloat64ArrayType.get_shape(_nk, {'fun_name': fun_name, 'entity': 'nk'}, exp_rank=1)[0]
    _lnk = _b_utils._EngineIntScalarType(
        _lnk_py, {'fun_name': fun_name, 'entity': '_lnk'},
    ).to_ctypes()
    _nb = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_ip_py, _lnk_py),
        sorder=_m_sorder,
    )
    b_lars_param(
        _nstep,
        _ip,
        _b,
        _fitsum,
        ktype,
        _nk,
        _lnk,
        _nb,
    )
    _nb_py = _nb
    return _nb_py

def quantile_linreg_easy(x, y, tau, io_manager=None):
    u"""
    Linear quantile regression, simple interface, independent,
    identically distributed (IID) errors.

    For full information please refer to the NAG Library document for
    g02qf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02qff.html

    Parameters
    ----------
    x : float, array-like, shape (n, m)
        X, the design matrix, with the ith value for the jth variate
        supplied in `x`[i-1,j-1], for j = 1,2,...,m, for i = 1,2,...,n.

    y : float, array-like, shape (n)
        y, the observations on the dependent variable.

    tau : float, array-like, shape (ntau)
        The vector of quantiles of interest. A separate model is fitted
        to each quantile.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    df : float
        The degrees of freedom given by n-k, where n is the number of
        observations and k is the rank of the cross-product matrix X^TX.

    b : float, ndarray, shape (m, ntau)
        beta^, the estimates of the parameters of the regression model,
        with `b`[j-1,l-1] containing the coefficient for the variable in
        column j of `x`, estimated for tau = `tau`[l-1].

    bl : float, ndarray, shape (m, ntau)
        beta^_L, the lower limit of a 95% confidence interval for beta^,
        with `bl`[j-1,l-1] holding the lower limit associated with
        `b`[j-1,l-1].

    bu : float, ndarray, shape (m, ntau)
        beta^_U, the upper limit of a 95% confidence interval for beta^,
        with `bu`[j-1,l-1] holding the upper limit associated with
        `b`[j-1,l-1].

    info : int, ndarray, shape (ntau)
        `info`[l] holds additional information concerning the model
        fitting and confidence limit calculations when tau = `tau`[l].

        [table omitted]

        It is possible for multiple warnings to be applicable to a
        single model.

        In these cases the value returned in `info` is the sum of the
        corresponding individual nonzero warning codes.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 21)
            On entry, m = *<value>* and n = *<value>*.

            Constraint: 1 <= m < n.

        (`errno` 51)
            On entry, ntau = *<value>*.

            Constraint: ntau >= 1.

        (`errno` 61)
            On entry, `tau`[*<value>*] = *<value>*.

            Constraint: sqrt(epsilon) < `tau`[l-1] < 1-sqrt(epsilon)
            where epsilon is the machine precision returned by
            ``machine.precision``, for all ntau.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 111)
            A potential problem occurred whilst fitting the model(s).

            Additional information has been returned in `info`.
    """

    from ..base.correg import quantile_linreg_easy as b_quantile_linreg_easy

    fun_name = 'naginterfaces.library.correg.quantile_linreg_easy'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x, 'x'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
    )
    _ntau_py = _b_utils._EngineFloat64ArrayType.get_shape(_tau, {'fun_name': fun_name, 'entity': 'tau'}, exp_rank=1)[0]
    _ntau = _b_utils._EngineIntScalarType(
        _ntau_py, {'fun_name': fun_name, 'entity': '_ntau'},
    ).to_ctypes()
    _b = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _ntau_py),
        sorder=_m_sorder,
    )
    _bl = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _ntau_py),
        sorder=_m_sorder,
    )
    _bu = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py, _ntau_py),
        sorder=_m_sorder,
    )
    _info = _b_utils._EngineIntArrayType.empty_ndarray(
        (_ntau_py,),
    )
    try:
        _return_scalar = b_quantile_linreg_easy(
            _n,
            _m,
            _x,
            _y,
            _ntau,
            _tau,
            _b,
            _bl,
            _bu,
            _info,
            io_manager,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _df_py = _return_scalar
    _b_py = _b
    _bl_py = _bl
    _bu_py = _bu
    _info_py = _info
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['df', 'b', 'bl', 'bu', 'info'],
    )(
        _df_py,
        _b_py,
        _bl_py,
        _bu_py,
        _info_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def quantile_linreg(sorder, dat, isx, y, tau, intcpt='Y', wt=None, b=None, comm=None, statecomm=None, io_manager=None):
    u"""
    Linear quantile regression, comprehensive interface.

    Note: this function uses optional algorithmic parameters, see also:
    ``optset``, ``optget``.

    For full information please refer to the NAG Library document for
    g02qg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02qgf.html

    Parameters
    ----------
    sorder : int
        Determines the storage order of variates supplied in `dat`.

    dat : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `sorder`=1: n; otherwise: m.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `sorder`=1: m; otherwise: n.

        The ith value for the jth variate, for j = 1,2,...,m, for i =
        1,2,...,n, must be supplied in

        - `dat`[i-1,j-1] if `sorder` = 1, and

        - `dat`[j-1,i-1] if `sorder` = 2.

        The design matrix X is constructed from `dat`, `isx` and
        `intcpt`.

    isx : int, array-like, shape (m)
        Indicates which independent variables are to be included in the
        model.

        `isx`[j-1] = 0

          The jth variate, supplied in `dat`, is not included in the
          regression model.

        `isx`[j-1] = 1

          The jth variate, supplied in `dat`, is included in the
          regression model.

    y : float, array-like, shape (n)
        y, the observations on the dependent variable.

    tau : float, array-like, shape (ntau)
        The vector of quantiles of interest. A separate model is fitted
        to each quantile.

    intcpt : str, length 1, optional
        Indicates whether an intercept will be included in the model.
        The intercept is included by adding a column of ones as the
        first column in the design matrix, X.

        `intcpt` = 'Y'

          An intercept will be included in the model.

        `intcpt` = 'N'

          An intercept will not be included in the model.

    wt : None or float, array-like, shape (:), optional
        Note: the required length for this argument is determined as
        follows: if `wt` is not None: n; otherwise: 0.

        If not None, `wt` must contain the diagonal elements of the
        weight matrix W.

        If weights are not provided then `wt` must be set to None.

        When

        'Drop Zero Weights' = 'YES'

          If `wt`[i-1] = 0.0, the ith observation is not included in the
          model, in which case the effective number of observations, n,
          is the number of observations with nonzero weights. If 'Return
          Residuals' = 'YES', the values of `res` will be set to zero
          for observations with zero weights.

        'Drop Zero Weights' = 'NO'

          All observations are included in the model and the effective
          number of observations is n, i.e., n = n.

    b : None or float, array-like, shape (ip, ntau), optional
        If 'Calculate Initial Values' = 'NO', `b`[i-1,l-1] must hold an
        initial estimates for beta^_i, for l = 1,2,...,ntau, for i =
        1,2,...,ip. If 'Calculate Initial Values' = 'YES', `b` need not
        be set.

    comm : None or dict, communication object, optional
        Communication structure.

        If not None, this argument must have been initialized by a prior
        call to ``optset``.

    statecomm : None or dict, RNG communication object, optional,
    modified in place
        RNG communication structure.

        If 'Interval Method' = 'BOOTSTRAP XY', this argument must have
        been initialized by a prior call to ``rand.init_repeat`` or
        ``rand.init_nonrepeat``.

    io_manager : FileObjManager, optional
        Manager for I/O in this routine.

    Returns
    -------
    df : float
        The degrees of freedom given by n-k, where n is the effective
        number of observations and k is the rank of the cross-product
        matrix X^TX.

    b : float, ndarray, shape (ip, ntau)
        `b`[i-1,l-1], for i = 1,2,...,ip, contains the estimates of the
        parameters of the regression model, beta^, estimated for tau =
        `tau`[l-1].

        If `intcpt` = 'Y', `b`[0,l-1] will contain the estimate
        corresponding to the intercept and `b`[i,l-1] will contain the
        coefficient of the jth variate contained in `dat`, where
        `isx`[j-1] is the ith nonzero value in the array `isx`.

        If `intcpt` = 'N', `b`[i-1,l-1] will contain the coefficient of
        the jth variate contained in `dat`, where `isx`[j-1] is the ith
        nonzero value in the array `isx`.

    bl : None or float, ndarray, shape (ip, :)
        If 'Interval Method' != 'NONE', `bl`[i-1,l-1] contains the lower
        limit of an (100*alpha)% confidence interval for `b`[i-1,l-1],
        for l = 1,2,...,ntau, for i = 1,2,...,ip.

        If 'Interval Method' = 'NONE', `bl` is not referenced.

        The method used for calculating the interval is controlled by
        the options 'Interval Method' and 'Bootstrap Interval Method'.

        The size of the interval, alpha, is controlled by the option
        'Significance Level'.

    bu : None or float, ndarray, shape (ip, :)
        If 'Interval Method' != 'NONE', `bu`[i-1,l-1] contains the upper
        limit of an (100*alpha)% confidence interval for `b`[i-1,l-1],
        for l = 1,2,...,ntau, for i = 1,2,...,ip.

        If 'Interval Method' = 'NONE', `bu` is not referenced.

        The method used for calculating the interval is controlled by
        the options 'Interval Method' and 'Bootstrap Interval Method'.

        The size of the interval, alpha is controlled by the option
        'Significance Level'.

    ch : None or float, ndarray, shape (ip, ip, :)
        Depending on the supplied options, `ch` will either not be
        referenced, hold an estimate of the upper triangular part of the
        covariance matrix, Sigma, or an estimate of the upper triangular
        parts of n*J_n and n^-1*H_n^-1.

        If 'Interval Method' = 'NONE' or 'Matrix Returned' = 'NONE',
        `ch` is not referenced.

        If 'Interval Method' = 'BOOTSTRAP XY' or 'IID' and 'Matrix
        Returned' = 'H INVERSE', `ch` is not referenced.

        Otherwise, for i,j = 1,2,…,ip,j >= i and l = 1,2,…,ntau:

        - If 'Matrix Returned' = 'COVARIANCE', `ch`[i-1,j-1,l-1] holds
          an estimate of the covariance between `b`[i-1,l-1] and
          `b`[j-1,l-1].

        - If 'Matrix Returned' = 'H INVERSE', `ch`[i-1,j-1,0] holds an
          estimate of the (i,j)th element of n*J_n and `ch`[i-1,j-1,l]
          holds an estimate of the (i,j)th element of n^-1*H_n^-1, for
          tau = `tau`[l-1].

        The method used for calculating Sigma and H_n^-1 is controlled
        by the option 'Interval Method'.

    res : None or float, ndarray, shape (n, :)
        If 'Return Residuals' = 'YES', `res`[i-1,l-1] holds the
        (weighted) residuals, r_i, for tau = `tau`[l-1], for l =
        1,2,...,ntau, for i = 1,2,...,n.

        If `wt` is not None and 'Drop Zero Weights' = 'YES', the value
        of `res` will be set to zero for observations with zero weights.

        If 'Return Residuals' = 'NO', `res` is not referenced and can be
        set to None.

    info : int, ndarray, shape (ntau)
        `info`[i] holds additional information concerning the model
        fitting and confidence limit calculations when tau = `tau`[i].

        [table omitted]

        It is possible for multiple warnings to be applicable to a
        single model.

        In these cases the value returned in `info` is the sum of the
        corresponding individual nonzero warning codes.

    Other Parameters
    ----------------
    'Band Width Alpha' : float
        Default = 1.0

        A multiplier used to construct the parameter alpha_b used when
        calculating the Sheather--Hall bandwidth (see [Description]),
        with alpha_b = (1-alpha)*'Band Width Alpha'.
        Here, alpha is the 'Significance Level'.

    'Band Width Method' : str
        Default = 'SHEATHER HALL'

        The method used to calculate the bandwidth used in the
        calculation of the asymptotic covariance matrix Sigma and H^-1
        if 'Interval Method' = 'HKS', 'KERNEL' or 'IID' (see
        [Description]).

    'Big' : float
        Default = 10.0^20

        This argument should be set to something larger than the biggest
        value supplied in `dat` and `y`.

    'Bootstrap Interval Method' : str
        Default = 'QUANTILE'

        If 'Interval Method' = 'BOOTSTRAP XY', 'Bootstrap Interval
        Method' controls how the confidence intervals are calculated
        from the bootstrap estimates.

        'Bootstrap Interval Method' = 'T'
          t intervals are calculated. That is, the covariance matrix,
          Sigma = {sigma_{ij}:i,j = 1,2,...,p} is calculated from the
          bootstrap estimates and the limits calculated as
          beta_i+/-t_(n-p,(1+alpha)/2)sigma_{ii} where
          t_(n-p,(1+alpha)/2) is the (1+alpha)/2 percentage point from a
          Student's t distribution on n-p degrees of freedom, n is the
          effective number of observations and alpha is given by the
          option 'Significance Level'.

        'Bootstrap Interval Method' = 'QUANTILE'
          Quantile intervals are calculated. That is, the upper and
          lower limits are taken as the (1+alpha)/2 and (1-alpha)/2
          quantiles of the bootstrap estimates, as calculated using
          ``stat.quantiles``.

    'Bootstrap Iterations' : int
        Default = 100

        The number of bootstrap samples used to calculate the confidence
        limits and covariance matrix (if requested) when 'Interval
        Method' = 'BOOTSTRAP XY'.

    'Bootstrap Monitoring' : str
        Default = 'NO'

        If 'Bootstrap Monitoring' = 'YES' and 'Interval Method' =
        'BOOTSTRAP XY', the parameter estimates for each of the
        bootstrap samples are displayed.
        This information is sent to the unit number specified by 'Unit
        Number'.

    'Calculate Initial Values' : str
        Default = 'YES'

        If 'Calculate Initial Values' = 'YES' then the initial values
        for the regression parameters, beta, are calculated from the
        data.
        Otherwise they must be supplied in `b`.

    'Defaults' : valueless
        This special keyword is used to reset all options to their
        default values.

    'Drop Zero Weights' : str
        Default = 'YES'

        If a weighted regression is being performed and 'Drop Zero
        Weights' = 'YES' then observations with zero weight are dropped
        from the analysis.
        Otherwise such observations are included.

    'Epsilon' : float
        Default = sqrt(epsilon)

        epsilon_u, the tolerance used when calculating the covariance
        matrix and the initial values for u and v.
        For additional details see [Calculation of Covariance Matrix]
        and [Additional information] respectively.

    'Interval Method' : str
        Default = 'IID'

        The value of 'Interval Method' controls whether confidence
        limits are returned in `bl` and `bu` and how these limits are
        calculated.
        This argument also controls how the matrices returned in `ch`
        are calculated.

        'Interval Method' = 'NONE'
          No limits are calculated and `bl`, `bu` and `ch` are not
          referenced.

        'Interval Method' = 'KERNEL'
          The Powell Sandwich method with a Gaussian kernel is used.

        'Interval Method' = 'HKS'
          The Hendricks--Koenker Sandwich is used.

        'Interval Method' = 'IID'
          The errors are assumed to be identical, and independently
          distributed.

        'Interval Method' = 'BOOTSTRAP XY'
          A bootstrap method is used, where sampling is done on the pair
          (y_i,x_i). The number of bootstrap samples is controlled by
          the argument 'Bootstrap Iterations' and the type of interval
          constructed from the bootstrap samples is controlled by
          'Bootstrap Interval Method'.

    'Iteration Limit' : int
        Default = 100

        The maximum number of iterations to be performed by the interior
        point optimization algorithm.

    'Matrix Returned' : str
        Default = 'NONE'

        The value of 'Matrix Returned' controls the type of matrices
        returned in `ch`.
        If 'Interval Method' = 'NONE', this argument is ignored and `ch`
        is not referenced.
        Otherwise:

        'Matrix Returned' = 'NONE'
          No matrices are returned and `ch` is not referenced.

        'Matrix Returned' = 'COVARIANCE'
          The covariance matrices are returned.

        'Matrix Returned' = 'H INVERSE'
          If 'Interval Method' = 'KERNEL' or 'HKS', the matrices J and
          H^-1 are returned. Otherwise no matrices are returned and `ch`
          is not referenced.

        The matrices returned are calculated as described in
        [Description], with the algorithm used specified by 'Interval
        Method'.
        In the case of 'Interval Method' = 'BOOTSTRAP XY' the covariance
        matrix is calculated directly from the bootstrap estimates.

    'Monitoring' : str
        Default = 'NO'

        If 'Monitoring' = 'YES' then the duality gap is displayed at
        each iteration of the interior point optimization algorithm.
        In addition, the final estimates for beta are also displayed.

        The monitoring information is sent to the unit number specified
        by 'Unit Number'.

    'QR Tolerance' : float
        Default = epsilon^0.9

        The tolerance used to calculate the rank, k, of the p*p
        cross-product matrix, X^TX.
        Letting Q be the orthogonal matrix obtained from a QR
        decomposition of X^TX, then the rank is calculated by comparing
        Q_{ii} with Q_{11}*'QR Tolerance'.

        If the cross-product matrix is rank deficient, the parameter
        estimates for the p-k columns with the smallest values of Q_{ii}
        are set to zero, along with the corresponding entries in `bl`,
        `bu` and `ch`, if returned.
        This is equivalent to dropping these variables from the model.
        Details on the QR decomposition used can be found in
        ``lapackeig.dgeqp3``.

    'Return Residuals' : str
        Default = 'NO'

        If 'Return Residuals' = 'YES', the residuals are returned in
        `res`.
        Otherwise `res` is not referenced.

    'Sigma' : float
        Default = 0.99995

        The scaling factor used when calculating the affine scaling step
        size (see equation [equation]).

    'Significance Level' : float
        Default = 0.95

        alpha, the size of the confidence interval whose limits are
        returned in `bl` and `bu`.

    'Tolerance' : float
        Default = sqrt(epsilon)

        Convergence tolerance.
        The optimization is deemed to have converged if the duality gap
        is less than 'Tolerance' (see [Update and convergence]).

    'Unit Number' : int
        Default = advisory message unit number

        The unit number to which any monitoring information is sent.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, `sorder` = *<value>*.

            Constraint: `sorder` = 1 or 2.

        (`errno` 21)
            On entry, `intcpt` = *<value>*.

            Constraint: `intcpt` = 'N' or 'Y'.

        (`errno` 41)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 51)
            On entry, m = *<value>*.

            Constraint: m >= 0.

        (`errno` 81)
            On entry, `isx`[*<value>*] = *<value>*.

            Constraint: `isx`[i] = 0 or 1, for all i.

        (`errno` 91)
            On entry, ip = *<value>* and n = *<value>*.

            Constraint: 1 <= ip < n.

        (`errno` 92)
            On entry, ip is not consistent with `isx` or `intcpt`: ip =
            *<value>*, expected value = *<value>*.

        (`errno` 111)
            On entry, `wt`[*<value>*] = *<value>*.

            Constraint: `wt`[i] >= 0.0, for all i.

        (`errno` 112)
            On entry, effective number of observations = *<value>*.

            Constraint: effective number of observations >= *<value>*.

        (`errno` 121)
            On entry, ntau = *<value>*.

            Constraint: ntau >= 1.

        (`errno` 131)
            On entry, `tau`[*<value>*] = *<value>*.

            Constraint: sqrt(epsilon) < `tau`[l-1] < 1-sqrt(epsilon)
            where epsilon is the machine precision returned by
            ``machine.precision``, for all ntau.

        (`errno` 201)
            On entry, either the option arrays have not been initialized
            or they have been corrupted.

        (`errno` 221)
            On entry, `statecomm`['state'] vector has been corrupted or
            not initialized.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 231)
            A potential problem occurred whilst fitting the model(s).

            Additional information has been returned in `info`.

    See Also
    --------
    :meth:`naginterfaces.library.examples.correg.quantile_linreg_ex.main`
    """

    from ..base.correg import quantile_linreg as b_quantile_linreg

    fun_name = 'naginterfaces.library.correg.quantile_linreg'

    _dat = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'dat'},
        dat,
        copy=False,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        allow_none=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_dat, 'dat'), (_b, 'b')),
    )
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_y, {'fun_name': fun_name, 'entity': 'y'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _isx = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'isx'},
        isx,
        copy=False,
    )
    _m_py = _b_utils._EngineIntArrayType.get_shape(_isx, {'fun_name': fun_name, 'entity': 'isx'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _intcpt = _b_utils._EngineCharScalarType(
        intcpt, {'fun_name': fun_name, 'entity': 'intcpt'}, exp_chrlen=1,
    ).to_ctypes()
    _intcpt_py = _b_utils._EngineCharScalarType.to_py(_intcpt)
    if _intcpt_py in ('Y', 'y'):
        _ip = 1
        _ip_py = _ip
    else:
        _ip = 0
        _ip_py = _ip
    for _vl_i_py in range(0, _m_py-1+1):
        if _b_utils._EngineIntArrayType.get_element(_isx, {'fun_name': fun_name, 'entity': 'isx'}, (_vl_i_py,)) > 0:
            _ip = _ip+1
    _ip_py = _b_utils._EngineIntScalarType.to_py(_ip)
    _wt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'wt'},
        wt,
        copy=False,
        allow_none=True,
    )
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
    )
    _ntau_py = _b_utils._EngineFloat64ArrayType.get_shape(_tau, {'fun_name': fun_name, 'entity': 'tau'}, exp_rank=1)[0]
    _ntau = _b_utils._EngineIntScalarType(
        _ntau_py, {'fun_name': fun_name, 'entity': '_ntau'},
    ).to_ctypes()
    _b_utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('iopts', 'opts'),
        allow_none=True,
    )
    _b_utils._check_comm(
        statecomm, {'fun_name': fun_name, 'entity': 'statecomm'},
        req_keys=('state',),
        allow_none=True,
    )
    if comm is None:
        _init_py = 0
    else:
        _init_py = 1
    _sdres, _rip, _tdch, _liopts, _lopts, _lstate, _calcib = _b_utils._g02zkbn(_init_py, 'G02QGF', _ip, _ntau, comm, statecomm)
    _sdres_py = _b_utils._EngineIntScalarType.to_py(_sdres)
    _rip_py = _b_utils._EngineIntScalarType.to_py(_rip)
    _tdch_py = _b_utils._EngineIntScalarType.to_py(_tdch)
    _liopts_py = _b_utils._EngineIntScalarType.to_py(_liopts)
    _lopts_py = _b_utils._EngineIntScalarType.to_py(_lopts)
    _lstate_py = _b_utils._EngineIntScalarType.to_py(_lstate)
    _calcib_py = _b_utils._EngineIntScalarType.to_py(_calcib)
    _vl_g02zkb_calcib_py = _calcib_py
    if _vl_g02zkb_calcib_py == 0:
        if b is None:
            raise _b_utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'b'},
                'must not be None when option \'Calculate Initial Values\' = \'NO\'',
            )
    if b is None:
        _b = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            (_ip_py, _ntau_py),
            sorder=_m_sorder,
        )
    _vl_g02zkb_rip_py = _rip_py
    if _vl_g02zkb_rip_py > 0:
        if _vl_g02zkb_rip_py > 0:
            _bl_dim2 = _ntau_py
        else:
            _bl_dim2 = 0
        exp_shape_bl = (_ip_py, _bl_dim2)
        _bl = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            exp_shape_bl,
            sorder=_m_sorder,
        )
    else:
        _bl = None
    if _vl_g02zkb_rip_py > 0:
        if _vl_g02zkb_rip_py > 0:
            _bu_dim2 = _ntau_py
        else:
            _bu_dim2 = 0
        exp_shape_bu = (_ip_py, _bu_dim2)
        _bu = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            exp_shape_bu,
            sorder=_m_sorder,
        )
    else:
        _bu = None
    _vl_g02zkb_tdch_py = _tdch_py
    if _vl_g02zkb_rip_py > 0 and _vl_g02zkb_tdch_py > 0:
        if _vl_g02zkb_rip_py > 0 and _vl_g02zkb_tdch_py > 0:
            _ch_dim3 = _ntau_py
        else:
            _ch_dim3 = _ntau_py+1
        exp_shape_ch = (_ip_py, _ip_py, _ch_dim3)
        _ch = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            exp_shape_ch,
            sorder=_m_sorder,
        )
    else:
        _ch = None
    _vl_g02zkb_sdres_py = _sdres_py
    if _vl_g02zkb_sdres_py > 0:
        if _vl_g02zkb_sdres_py > 0:
            _res_dim2 = _ntau_py
        else:
            _res_dim2 = 0
        exp_shape_res = (_n_py, _res_dim2)
        _res = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            exp_shape_res,
            sorder=_m_sorder,
        )
    else:
        _res = None
    _info = _b_utils._EngineIntArrayType.empty_ndarray(
        (_ntau_py,),
    )
    try:
        _return_scalar = b_quantile_linreg(
            sorder,
            intcpt,
            _n,
            _m,
            _dat,
            _isx,
            _ip,
            _y,
            _wt,
            _ntau,
            _tau,
            _b,
            _bl,
            _bu,
            _ch,
            _res,
            comm,
            statecomm,
            _info,
            io_manager,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _b_py = _b
    _df_py = _return_scalar
    if _vl_g02zkb_rip_py > 0:
        _bl_py = _bl
    else:
        _bl_py = None
    if _vl_g02zkb_rip_py > 0:
        _bu_py = _bu
    else:
        _bu_py = None
    _vl_g02zkb_tdch_py = _tdch_py
    if _vl_g02zkb_rip_py > 0 and _vl_g02zkb_tdch_py > 0:
        _ch_py = _ch
    else:
        _ch_py = None
    _vl_g02zkb_sdres_py = _sdres_py
    if _vl_g02zkb_sdres_py > 0:
        _res_py = _res
    else:
        _res_py = None
    _info_py = _info
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['df', 'b', 'bl', 'bu', 'ch', 'res', 'info'],
    )(
        _df_py,
        _b_py,
        _bl_py,
        _bu_py,
        _ch_py,
        _res_py,
        _info_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def optset(optstr, comm):
    u"""
    Option setting function for ``quantile_linreg``.

    For full information please refer to the NAG Library document for
    g02zk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02zkf.html

    Parameters
    ----------
    optstr : str
        A string identifying the option to be set.

        Initialize = function name

          Initialize the option arrays `comm`['iopts'] and
          `comm`['opts'] for use with function function name, where
          function name is the short name of the problem solving
          function you wish to use.

        Defaults

          Resets all options to their default values.

        option = optval

          See [Optional Parameters] for details of valid values for
          option and optval. The equals sign (=) delimiter must be used
          to separate the option from its optval value.

        `optstr` is case insensitive.

        Each token in the option and optval component must be separated
        by at least one space.

    comm : dict, communication object, modified in place
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, the option supplied in `optstr` was not
            recognized: `optstr` = *<value>*.

        (`errno` 12)
            On entry, the expected delimiter '=' was not found in
            `optstr`: `optstr` = *<value>*.

        (`errno` 13)
            On entry, could not convert the specified optval to an
            integer: `optstr` = *<value>*.

        (`errno` 13)
            On entry, could not convert the specified optval to a real:
            `optstr` = *<value>*.

        (`errno` 14)
            On entry, attempting to initialize the option arrays but
            specified function name was not valid: name = *<value>*.

        (`errno` 15)
            On entry, the optval supplied for the integer option is not
            valid.

            `optstr` = *<value>*.

        (`errno` 16)
            On entry, the optval supplied for the real option is not
            valid.

            `optstr` = *<value>*.

        (`errno` 17)
            On entry, the optval supplied for the character option is
            not valid.

            `optstr` = *<value>*.

        (`errno` 21)
            On entry, either the option arrays have not been initialized
            or they have been corrupted.

    See Also
    --------
    :meth:`naginterfaces.library.examples.correg.quantile_linreg_ex.main`
    """

    from ..base.correg import optset as b_optset

    b_optset(
        optstr,
        comm,
    )

def optget(optstr, comm):
    u"""
    Option getting function for ``quantile_linreg``.

    For full information please refer to the NAG Library document for
    g02zl

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g02/g02zlf.html

    Parameters
    ----------
    optstr : str
        A string identifying the option whose current value is required.
        See [Optional Parameters] for information on valid options. In
        addition, the following is a valid option:

        Identify

          ``optget`` returns in cvalue the function name supplied to
          ``optset`` when the option arrays `comm`['iopts'] and
          `comm`['opts'] were initialized.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``optset`` or ``quantile_linreg``.

    Returns
    -------
    optvalue : dict
        The option-value ``dict``, with the following keys:

        ``'value'`` : float, int or str
            The value of the requested option.

        ``'annotation'`` : None or str
            Possible additional information about the option value.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, the option in `optstr` was not recognized:
            `optstr` = *<value>*.

        (`errno` 61)
            On entry, either the option arrays have not been initialized
            or they have been corrupted.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 41)
            On entry, `optstr` indicates a character option, but cvalue
            is too short to hold the stored value. The returned value
            will be truncated.
    """

    from ..base import _nag_g_consts
    from ..base.correg import optget as b_optget

    try:
        _return_scalars = b_optget(
            optstr,
            comm,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    fun_name = 'naginterfaces.library.correg.optget'

    _optype_py = _return_scalars.optype
    _ivalue_py = _return_scalars.ivalue
    _rvalue_py = _return_scalars.rvalue
    _cvalue_py = _return_scalars.cvalue
    if _optype_py == 1:
        _optvalue_py = {}
        _optvalue_py['value'] = _ivalue_py
        _optvalue_py['annotation'] = None
    elif _optype_py == 2:
        _optvalue_py = {}
        _optvalue_py['value'] = _rvalue_py
        _optvalue_py['annotation'] = None
    elif _optype_py == 3:
        _optvalue_py = {}
        _optvalue_py['value'] = _cvalue_py
        _optvalue_py['annotation'] = None
    elif _optype_py == 6:
        _optvalue_py = {}
        if _ivalue_py == _nag_g_consts.QLR_ID:
            _optvalue_py = {}
            _optvalue_py['value'] = 'quantile_linreg'
        elif _ivalue_py == _nag_g_consts.JC_MNUM:
            _optvalue_py = {}
            _optvalue_py['value'] = 'mixeff_hier_init'
        else:
            raise _b_utils.NagException(
                {'fun_name': fun_name},
                'inconceivable option configuration',
            )
        _optvalue_py['annotation'] = None
    else:
        raise _b_utils.NagException(
            {'fun_name': fun_name},
            'inconceivable option configuration',
        )
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['optvalue'],
    )(
        _optvalue_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _optvalue_py
