"""
Deprecated wrappers for the NAG C Library (Mark 26.0) f08 Chapter.

f08 - Least Squares and Eigenvalue Problems (LAPACK).

https://www.nag.com/numeric/CL/nagdoc_cl26/html/F08/f08conts.html

NAG Copyright 2017.
"""
from ctypes import c_double
from ctypes import POINTER

from .util import get_library

from .util import Nag_OrderType
from .util import Nag_TransType
from .util import Integer
from .util import NagError
from .util import Nag_SideType
from .util import Complex
from .util import Nag_JobType
from .util import Nag_UploType
from .util import Nag_RangeType
from .util import Nag_VectType
from .util import Nag_ComputeZType
from .util import Nag_ComputeEigVecsType
from .util import Nag_EigValRankType
from .util import Nag_ComputeUType
from .util import Nag_ComputeVTType
from .util import Nag_Preprocess
from .util import Nag_LeftVecsType
from .util import Nag_RightVecsType
from .util import Nag_ZeroCols
from .util import Nag_Perturb
from .util import Nag_MatrixType
from .util import Nag_ComputeSingularVecsType
from .util import Nag_BalanceType
from .util import Nag_RCondType
from .util import Nag_SortEigValsType
from .util import NAG_F08PAC_SELECT
from .util import NAG_F08PBC_SELECT
from .util import Nag_EigValsSourceType
from .util import Nag_InitVeenumtype
from .util import Nag_Boolean
from .util import NAG_F08PNC_SELECT
from .util import NAG_F08PPC_SELECT
from .util import Nag_ComputeQType
from .util import Nag_SignType
from .util import Nag_HowManyType
from .util import Nag_SignsType
from .util import Nag_ComputeType
from .util import Nag_ComputeVType
from .util import NAG_F08XAC_SELCTG
from .util import NAG_F08XBC_SELCTG
from .util import NAG_F08XCC_SELCTG
from .util import NAG_F08XNC_SELCTG
from .util import NAG_F08XPC_SELCTG
from .util import NAG_F08XQC_SELCTG


_library_cl = get_library()

f08aac_ctypes = _library_cl.f08aac
f08aac_ctypes.restype = None
f08aac_ctypes.argtypes = [Nag_OrderType, Nag_TransType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f08aac(order, trans, m, n, nrhs, a, pda, b, pdb, fail):
    """
    Solves a real linear least problem of full rank.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08aac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08aac', a, 'a')
    _b = nag_double_type_check_and_cast('f08aac', b, 'b')
    f08aac_ctypes(order, trans, m, n, nrhs, _a, pda, _b, pdb, fail)

f08abc_ctypes = _library_cl.f08abc
f08abc_ctypes.restype = None
f08abc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f08abc(order, m, n, nb, a, pda, t, pdt, fail):
    """
    Performs a QR factorization of real general rectangular matrix, with
    explicit blocking.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08abc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, t
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, t
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08abc', a, 'a')
    _t = nag_double_type_check_and_cast('f08abc', t, 't')
    f08abc_ctypes(order, m, n, nb, _a, pda, _t, pdt, fail)

f08acc_ctypes = _library_cl.f08acc
f08acc_ctypes.restype = None
f08acc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_TransType, Integer, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f08acc(order, side, trans, m, n, k, nb, v, pdv, t, pdt, c, pdc, fail):
    """
    Applies the orthogonal transformation determined by f08abc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08acc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        v, t, c
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        v, t, c
    """
    from .util import nag_double_type_check_and_cast
    _v = nag_double_type_check_and_cast('f08acc', v, 'v')
    _t = nag_double_type_check_and_cast('f08acc', t, 't')
    _c = nag_double_type_check_and_cast('f08acc', c, 'c')
    f08acc_ctypes(order, side, trans, m, n, k, nb, _v, pdv, _t, pdt, _c, pdc, fail)

f08aec_ctypes = _library_cl.f08aec
f08aec_ctypes.restype = None
f08aec_ctypes.argtypes = [Nag_OrderType, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f08aec(order, m, n, a, pda, tau, fail):
    """
    Performs a QR factorization of real general rectangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08aec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, tau
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08aec', a, 'a')
    _tau = nag_double_type_check_and_cast('f08aec', tau, 'tau')
    f08aec_ctypes(order, m, n, _a, pda, _tau, fail)

f08afc_ctypes = _library_cl.f08afc
f08afc_ctypes.restype = None
f08afc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f08afc(order, m, n, k, a, pda, tau, fail):
    """
    Forms all or part of orthogonal Q from QR factorization determined by
    f08aec, f08bec or f08bfc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08afc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, tau
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08afc', a, 'a')
    _tau = nag_double_type_check_and_cast('f08afc', tau, 'tau')
    f08afc_ctypes(order, m, n, k, _a, pda, _tau, fail)

f08agc_ctypes = _library_cl.f08agc
f08agc_ctypes.restype = None
f08agc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_TransType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f08agc(order, side, trans, m, n, k, a, pda, tau, c, pdc, fail):
    """
    Applies an orthogonal transformation determined by f08aec, f08bec or
    f08bfc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08agc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau, c
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, tau, c
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08agc', a, 'a')
    _tau = nag_double_type_check_and_cast('f08agc', tau, 'tau')
    _c = nag_double_type_check_and_cast('f08agc', c, 'c')
    f08agc_ctypes(order, side, trans, m, n, k, _a, pda, _tau, _c, pdc, fail)

f08ahc_ctypes = _library_cl.f08ahc
f08ahc_ctypes.restype = None
f08ahc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f08ahc(order, m, n, a, pda, tau, fail):
    """
    Performs a LQ factorization of real general rectangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08ahc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, tau
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08ahc', a, 'a')
    _tau = nag_double_type_check_and_cast('f08ahc', tau, 'tau')
    f08ahc_ctypes(order, m, n, _a, pda, _tau, fail)

f08ajc_ctypes = _library_cl.f08ajc
f08ajc_ctypes.restype = None
f08ajc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f08ajc(order, m, n, k, a, pda, tau, fail):
    """
    Forms all or part of orthogonal Q from LQ factorization determined by
    f08ahc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08ajc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, tau
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08ajc', a, 'a')
    _tau = nag_double_type_check_and_cast('f08ajc', tau, 'tau')
    f08ajc_ctypes(order, m, n, k, _a, pda, _tau, fail)

f08akc_ctypes = _library_cl.f08akc
f08akc_ctypes.restype = None
f08akc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_TransType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f08akc(order, side, trans, m, n, k, a, pda, tau, c, pdc, fail):
    """
    Applies the orthogonal transformation determined by f08ahc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08akc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau, c
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, tau, c
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08akc', a, 'a')
    _tau = nag_double_type_check_and_cast('f08akc', tau, 'tau')
    _c = nag_double_type_check_and_cast('f08akc', c, 'c')
    f08akc_ctypes(order, side, trans, m, n, k, _a, pda, _tau, _c, pdc, fail)

f08anc_ctypes = _library_cl.f08anc
f08anc_ctypes.restype = None
f08anc_ctypes.argtypes = [Nag_OrderType, Nag_TransType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f08anc(order, trans, m, n, nrhs, a, pda, b, pdb, fail):
    """
    Solves a complex linear least problem of full rank.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08anc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08anc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08anc', b, 'b')
    f08anc_ctypes(order, trans, m, n, nrhs, _a, pda, _b, pdb, fail)

f08apc_ctypes = _library_cl.f08apc
f08apc_ctypes.restype = None
f08apc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f08apc(order, m, n, nb, a, pda, t, pdt, fail):
    """
    Performs a QR factorization of complex general rectangular matrix using
    recursive algorithm.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08apc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, t
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, t
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08apc', a, 'a')
    _t = nag_complex_type_check_and_cast('f08apc', t, 't')
    f08apc_ctypes(order, m, n, nb, _a, pda, _t, pdt, fail)

f08aqc_ctypes = _library_cl.f08aqc
f08aqc_ctypes.restype = None
f08aqc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_TransType, Integer, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f08aqc(order, side, trans, m, n, k, nb, v, pdv, t, pdt, c, pdc, fail):
    """
    Applies the unitary transformation determined by f08apc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08aqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        v, t, c
    TypeError will be raised when the following arguments are not of data type
    Complex:
        v, t, c
    """
    from .util import nag_complex_type_check_and_cast
    _v = nag_complex_type_check_and_cast('f08aqc', v, 'v')
    _t = nag_complex_type_check_and_cast('f08aqc', t, 't')
    _c = nag_complex_type_check_and_cast('f08aqc', c, 'c')
    f08aqc_ctypes(order, side, trans, m, n, k, nb, _v, pdv, _t, pdt, _c, pdc, fail)

f08asc_ctypes = _library_cl.f08asc
f08asc_ctypes.restype = None
f08asc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(NagError)]


def f08asc(order, m, n, a, pda, tau, fail):
    """
    Performs a QR factorization of complex general rectangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08asc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, tau
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08asc', a, 'a')
    _tau = nag_complex_type_check_and_cast('f08asc', tau, 'tau')
    f08asc_ctypes(order, m, n, _a, pda, _tau, fail)

f08atc_ctypes = _library_cl.f08atc
f08atc_ctypes.restype = None
f08atc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(NagError)]


def f08atc(order, m, n, k, a, pda, tau, fail):
    """
    Forms all or part of unitary Q from QR factorization determined by
    f08asc, f08bsc or f08btc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08atc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, tau
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08atc', a, 'a')
    _tau = nag_complex_type_check_and_cast('f08atc', tau, 'tau')
    f08atc_ctypes(order, m, n, k, _a, pda, _tau, fail)

f08auc_ctypes = _library_cl.f08auc
f08auc_ctypes.restype = None
f08auc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_TransType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(Complex), Integer, POINTER(NagError)]


def f08auc(order, side, trans, m, n, k, a, pda, tau, c, pdc, fail):
    """
    Applies a unitary transformation determined by f08asc, f08bsc or f08btc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08auc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau, c
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, tau, c
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08auc', a, 'a')
    _tau = nag_complex_type_check_and_cast('f08auc', tau, 'tau')
    _c = nag_complex_type_check_and_cast('f08auc', c, 'c')
    f08auc_ctypes(order, side, trans, m, n, k, _a, pda, _tau, _c, pdc, fail)

f08avc_ctypes = _library_cl.f08avc
f08avc_ctypes.restype = None
f08avc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(NagError)]


def f08avc(order, m, n, a, pda, tau, fail):
    """
    Performs a LQ factorization of complex general rectangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08avc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, tau
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08avc', a, 'a')
    _tau = nag_complex_type_check_and_cast('f08avc', tau, 'tau')
    f08avc_ctypes(order, m, n, _a, pda, _tau, fail)

f08awc_ctypes = _library_cl.f08awc
f08awc_ctypes.restype = None
f08awc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(NagError)]


def f08awc(order, m, n, k, a, pda, tau, fail):
    """
    Forms all or part of unitary Q from LQ factorization determined by
    f08avc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08awc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, tau
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08awc', a, 'a')
    _tau = nag_complex_type_check_and_cast('f08awc', tau, 'tau')
    f08awc_ctypes(order, m, n, k, _a, pda, _tau, fail)

f08axc_ctypes = _library_cl.f08axc
f08axc_ctypes.restype = None
f08axc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_TransType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(Complex), Integer, POINTER(NagError)]


def f08axc(order, side, trans, m, n, k, a, pda, tau, c, pdc, fail):
    """
    Applies the unitary transformation determined by f08avc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08axc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau, c
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, tau, c
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08axc', a, 'a')
    _tau = nag_complex_type_check_and_cast('f08axc', tau, 'tau')
    _c = nag_complex_type_check_and_cast('f08axc', c, 'c')
    f08axc_ctypes(order, side, trans, m, n, k, _a, pda, _tau, _c, pdc, fail)

f08bac_ctypes = _library_cl.f08bac
f08bac_ctypes.restype = None
f08bac_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(Integer), c_double, POINTER(Integer), POINTER(NagError)]


def f08bac(order, m, n, nrhs, a, pda, b, pdb, jpvt, rcond, rank, fail):
    """
    Computes the minimum-norm solution to a real linear least squares
    problem.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08bac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, jpvt, rank
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        jpvt, rank
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08bac', a, 'a')
    _b = nag_double_type_check_and_cast('f08bac', b, 'b')
    _jpvt = nag_integer_type_check_and_cast('f08bac', jpvt, 'jpvt')
    _rank = nag_integer_type_check_and_cast('f08bac', rank, 'rank')
    f08bac_ctypes(order, m, n, nrhs, _a, pda, _b, pdb, _jpvt, rcond, _rank, fail)

f08bbc_ctypes = _library_cl.f08bbc
f08bbc_ctypes.restype = None
f08bbc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f08bbc(order, m, n, l, nb, a, pda, b, pdb, t, pdt, fail):
    """
    QR factorization of real general triangular-pentagonal matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08bbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, t
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, t
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08bbc', a, 'a')
    _b = nag_double_type_check_and_cast('f08bbc', b, 'b')
    _t = nag_double_type_check_and_cast('f08bbc', t, 't')
    f08bbc_ctypes(order, m, n, l, nb, _a, pda, _b, pdb, _t, pdt, fail)

f08bcc_ctypes = _library_cl.f08bcc
f08bcc_ctypes.restype = None
f08bcc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_TransType, Integer, Integer, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f08bcc(order, side, trans, m, n, k, l, nb, v, pdv, t, pdt, a, pda, b, pdb, fail):
    """
    Applies the orthogonal transformation determined by f08bbc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08bcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        v, t, a, b
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        v, t, a, b
    """
    from .util import nag_double_type_check_and_cast
    _v = nag_double_type_check_and_cast('f08bcc', v, 'v')
    _t = nag_double_type_check_and_cast('f08bcc', t, 't')
    _a = nag_double_type_check_and_cast('f08bcc', a, 'a')
    _b = nag_double_type_check_and_cast('f08bcc', b, 'b')
    f08bcc_ctypes(order, side, trans, m, n, k, l, nb, _v, pdv, _t, pdt, _a, pda, _b, pdb, fail)

f08bec_ctypes = _library_cl.f08bec
f08bec_ctypes.restype = None
f08bec_ctypes.argtypes = [Nag_OrderType, Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def f08bec(order, m, n, a, pda, jpvt, tau, fail):
    """
    QR factorization, with column pivoting, of real general rectangular
    matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08bec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, jpvt, tau
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, tau
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        jpvt
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08bec', a, 'a')
    _jpvt = nag_integer_type_check_and_cast('f08bec', jpvt, 'jpvt')
    _tau = nag_double_type_check_and_cast('f08bec', tau, 'tau')
    f08bec_ctypes(order, m, n, _a, pda, _jpvt, _tau, fail)

f08bfc_ctypes = _library_cl.f08bfc
f08bfc_ctypes.restype = None
f08bfc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def f08bfc(order, m, n, a, pda, jpvt, tau, fail):
    """
    QR factorization, with column pivoting, using BLAS-3, of real general
    rectangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08bfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, jpvt, tau
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, tau
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        jpvt
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08bfc', a, 'a')
    _jpvt = nag_integer_type_check_and_cast('f08bfc', jpvt, 'jpvt')
    _tau = nag_double_type_check_and_cast('f08bfc', tau, 'tau')
    f08bfc_ctypes(order, m, n, _a, pda, _jpvt, _tau, fail)

f08bhc_ctypes = _library_cl.f08bhc
f08bhc_ctypes.restype = None
f08bhc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f08bhc(order, m, n, a, pda, tau, fail):
    """
    Reduces a real upper trapezoidal matrix to upper triangular form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08bhc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, tau
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08bhc', a, 'a')
    _tau = nag_double_type_check_and_cast('f08bhc', tau, 'tau')
    f08bhc_ctypes(order, m, n, _a, pda, _tau, fail)

f08bkc_ctypes = _library_cl.f08bkc
f08bkc_ctypes.restype = None
f08bkc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_TransType, Integer, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f08bkc(order, side, trans, m, n, k, l, a, pda, tau, c, pdc, fail):
    """
    Applies the orthogonal transformation determined by f08bhc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08bkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau, c
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, tau, c
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08bkc', a, 'a')
    _tau = nag_double_type_check_and_cast('f08bkc', tau, 'tau')
    _c = nag_double_type_check_and_cast('f08bkc', c, 'c')
    f08bkc_ctypes(order, side, trans, m, n, k, l, _a, pda, _tau, _c, pdc, fail)

f08bnc_ctypes = _library_cl.f08bnc
f08bnc_ctypes.restype = None
f08bnc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Integer), c_double, POINTER(Integer), POINTER(NagError)]


def f08bnc(order, m, n, nrhs, a, pda, b, pdb, jpvt, rcond, rank, fail):
    """
    Computes the minimum-norm solution to a complex linear least squares
    problem.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08bnc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, jpvt, rank
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        jpvt, rank
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08bnc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08bnc', b, 'b')
    _jpvt = nag_integer_type_check_and_cast('f08bnc', jpvt, 'jpvt')
    _rank = nag_integer_type_check_and_cast('f08bnc', rank, 'rank')
    f08bnc_ctypes(order, m, n, nrhs, _a, pda, _b, pdb, _jpvt, rcond, _rank, fail)

f08bpc_ctypes = _library_cl.f08bpc
f08bpc_ctypes.restype = None
f08bpc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f08bpc(order, m, n, l, nb, a, pda, b, pdb, t, pdt, fail):
    """
    QR factorization of complex triangular-pentagonal matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08bpc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, t
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, t
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08bpc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08bpc', b, 'b')
    _t = nag_complex_type_check_and_cast('f08bpc', t, 't')
    f08bpc_ctypes(order, m, n, l, nb, _a, pda, _b, pdb, _t, pdt, fail)

f08bqc_ctypes = _library_cl.f08bqc
f08bqc_ctypes.restype = None
f08bqc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_TransType, Integer, Integer, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f08bqc(order, side, trans, m, n, k, l, nb, v, pdv, t, pdt, a, pda, b, pdb, fail):
    """
    Applies the unitary transformation determined by f08bpc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08bqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        v, t, a, b
    TypeError will be raised when the following arguments are not of data type
    Complex:
        v, t, a, b
    """
    from .util import nag_complex_type_check_and_cast
    _v = nag_complex_type_check_and_cast('f08bqc', v, 'v')
    _t = nag_complex_type_check_and_cast('f08bqc', t, 't')
    _a = nag_complex_type_check_and_cast('f08bqc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08bqc', b, 'b')
    f08bqc_ctypes(order, side, trans, m, n, k, l, nb, _v, pdv, _t, pdt, _a, pda, _b, pdb, fail)

f08bsc_ctypes = _library_cl.f08bsc
f08bsc_ctypes.restype = None
f08bsc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, POINTER(Complex), Integer, POINTER(Integer), POINTER(Complex), POINTER(NagError)]


def f08bsc(order, m, n, a, pda, jpvt, tau, fail):
    """
    QR factorization, with column pivoting, of complex general rectangular
    matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08bsc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, jpvt, tau
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        jpvt
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, tau
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08bsc', a, 'a')
    _jpvt = nag_integer_type_check_and_cast('f08bsc', jpvt, 'jpvt')
    _tau = nag_complex_type_check_and_cast('f08bsc', tau, 'tau')
    f08bsc_ctypes(order, m, n, _a, pda, _jpvt, _tau, fail)

f08btc_ctypes = _library_cl.f08btc
f08btc_ctypes.restype = None
f08btc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, POINTER(Complex), Integer, POINTER(Integer), POINTER(Complex), POINTER(NagError)]


def f08btc(order, m, n, a, pda, jpvt, tau, fail):
    """
    QR factorization, with column pivoting, using BLAS-3, of complex general
    rectangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08btc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, jpvt, tau
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        jpvt
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, tau
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08btc', a, 'a')
    _jpvt = nag_integer_type_check_and_cast('f08btc', jpvt, 'jpvt')
    _tau = nag_complex_type_check_and_cast('f08btc', tau, 'tau')
    f08btc_ctypes(order, m, n, _a, pda, _jpvt, _tau, fail)

f08bvc_ctypes = _library_cl.f08bvc
f08bvc_ctypes.restype = None
f08bvc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(NagError)]


def f08bvc(order, m, n, a, pda, tau, fail):
    """
    Reduces a complex upper trapezoidal matrix to upper triangular form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08bvc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, tau
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08bvc', a, 'a')
    _tau = nag_complex_type_check_and_cast('f08bvc', tau, 'tau')
    f08bvc_ctypes(order, m, n, _a, pda, _tau, fail)

f08bxc_ctypes = _library_cl.f08bxc
f08bxc_ctypes.restype = None
f08bxc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_TransType, Integer, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(Complex), Integer, POINTER(NagError)]


def f08bxc(order, side, trans, m, n, k, l, a, pda, tau, c, pdc, fail):
    """
    Applies the unitary transformation determined by f08bvc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08bxc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau, c
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, tau, c
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08bxc', a, 'a')
    _tau = nag_complex_type_check_and_cast('f08bxc', tau, 'tau')
    _c = nag_complex_type_check_and_cast('f08bxc', c, 'c')
    f08bxc_ctypes(order, side, trans, m, n, k, l, _a, pda, _tau, _c, pdc, fail)

f08cec_ctypes = _library_cl.f08cec
f08cec_ctypes.restype = None
f08cec_ctypes.argtypes = [Nag_OrderType, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f08cec(order, m, n, a, pda, tau, fail):
    """
    QL factorization of real general rectangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08cec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, tau
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08cec', a, 'a')
    _tau = nag_double_type_check_and_cast('f08cec', tau, 'tau')
    f08cec_ctypes(order, m, n, _a, pda, _tau, fail)

f08cfc_ctypes = _library_cl.f08cfc
f08cfc_ctypes.restype = None
f08cfc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f08cfc(order, m, n, k, a, pda, tau, fail):
    """
    Form all or part of orthogonal Q from QL factorization determined by
    f08cec.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08cfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, tau
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08cfc', a, 'a')
    _tau = nag_double_type_check_and_cast('f08cfc', tau, 'tau')
    f08cfc_ctypes(order, m, n, k, _a, pda, _tau, fail)

f08cgc_ctypes = _library_cl.f08cgc
f08cgc_ctypes.restype = None
f08cgc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_TransType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f08cgc(order, side, trans, m, n, k, a, pda, tau, c, pdc, fail):
    """
    Applies the orthogonal transformation determined by f08cec.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08cgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau, c
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, tau, c
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08cgc', a, 'a')
    _tau = nag_double_type_check_and_cast('f08cgc', tau, 'tau')
    _c = nag_double_type_check_and_cast('f08cgc', c, 'c')
    f08cgc_ctypes(order, side, trans, m, n, k, _a, pda, _tau, _c, pdc, fail)

f08chc_ctypes = _library_cl.f08chc
f08chc_ctypes.restype = None
f08chc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f08chc(order, m, n, a, pda, tau, fail):
    """
    RQ factorization of real general rectangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08chc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, tau
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08chc', a, 'a')
    _tau = nag_double_type_check_and_cast('f08chc', tau, 'tau')
    f08chc_ctypes(order, m, n, _a, pda, _tau, fail)

f08cjc_ctypes = _library_cl.f08cjc
f08cjc_ctypes.restype = None
f08cjc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f08cjc(order, m, n, k, a, pda, tau, fail):
    """
    Form all or part of orthogonal Q from RQ factorization determined by
    f08chc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08cjc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, tau
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08cjc', a, 'a')
    _tau = nag_double_type_check_and_cast('f08cjc', tau, 'tau')
    f08cjc_ctypes(order, m, n, k, _a, pda, _tau, fail)

f08ckc_ctypes = _library_cl.f08ckc
f08ckc_ctypes.restype = None
f08ckc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_TransType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f08ckc(order, side, trans, m, n, k, a, pda, tau, c, pdc, fail):
    """
    Applies the orthogonal transformation determined by f08chc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08ckc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau, c
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, tau, c
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08ckc', a, 'a')
    _tau = nag_double_type_check_and_cast('f08ckc', tau, 'tau')
    _c = nag_double_type_check_and_cast('f08ckc', c, 'c')
    f08ckc_ctypes(order, side, trans, m, n, k, _a, pda, _tau, _c, pdc, fail)

f08csc_ctypes = _library_cl.f08csc
f08csc_ctypes.restype = None
f08csc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(NagError)]


def f08csc(order, m, n, a, pda, tau, fail):
    """
    QL factorization of complex general rectangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08csc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, tau
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08csc', a, 'a')
    _tau = nag_complex_type_check_and_cast('f08csc', tau, 'tau')
    f08csc_ctypes(order, m, n, _a, pda, _tau, fail)

f08ctc_ctypes = _library_cl.f08ctc
f08ctc_ctypes.restype = None
f08ctc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(NagError)]


def f08ctc(order, m, n, k, a, pda, tau, fail):
    """
    Form all or part of unitary Q from QL factorization determined by
    f08csc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08ctc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, tau
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08ctc', a, 'a')
    _tau = nag_complex_type_check_and_cast('f08ctc', tau, 'tau')
    f08ctc_ctypes(order, m, n, k, _a, pda, _tau, fail)

f08cuc_ctypes = _library_cl.f08cuc
f08cuc_ctypes.restype = None
f08cuc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_TransType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(Complex), Integer, POINTER(NagError)]


def f08cuc(order, side, trans, m, n, k, a, pda, tau, c, pdc, fail):
    """
    Applies the unitary transformation determined by f08csc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08cuc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau, c
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, tau, c
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08cuc', a, 'a')
    _tau = nag_complex_type_check_and_cast('f08cuc', tau, 'tau')
    _c = nag_complex_type_check_and_cast('f08cuc', c, 'c')
    f08cuc_ctypes(order, side, trans, m, n, k, _a, pda, _tau, _c, pdc, fail)

f08cvc_ctypes = _library_cl.f08cvc
f08cvc_ctypes.restype = None
f08cvc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(NagError)]


def f08cvc(order, m, n, a, pda, tau, fail):
    """
    RQ factorization of complex general rectangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08cvc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, tau
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08cvc', a, 'a')
    _tau = nag_complex_type_check_and_cast('f08cvc', tau, 'tau')
    f08cvc_ctypes(order, m, n, _a, pda, _tau, fail)

f08cwc_ctypes = _library_cl.f08cwc
f08cwc_ctypes.restype = None
f08cwc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(NagError)]


def f08cwc(order, m, n, k, a, pda, tau, fail):
    """
    Form all or part of unitary Q from RQ factorization determined by
    f08cvc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08cwc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, tau
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08cwc', a, 'a')
    _tau = nag_complex_type_check_and_cast('f08cwc', tau, 'tau')
    f08cwc_ctypes(order, m, n, k, _a, pda, _tau, fail)

f08cxc_ctypes = _library_cl.f08cxc
f08cxc_ctypes.restype = None
f08cxc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_TransType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(Complex), Integer, POINTER(NagError)]


def f08cxc(order, side, trans, m, n, k, a, pda, tau, c, pdc, fail):
    """
    Applies the unitary transformation determined by f08cvc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08cxc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau, c
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, tau, c
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08cxc', a, 'a')
    _tau = nag_complex_type_check_and_cast('f08cxc', tau, 'tau')
    _c = nag_complex_type_check_and_cast('f08cxc', c, 'c')
    f08cxc_ctypes(order, side, trans, m, n, k, _a, pda, _tau, _c, pdc, fail)

f08fac_ctypes = _library_cl.f08fac
f08fac_ctypes.restype = None
f08fac_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_UploType, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f08fac(order, job, uplo, n, a, pda, w, fail):
    """
    Computes all eigenvalues and, optionally, eigenvectors of a real
    symmetric matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08fac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, w
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, w
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08fac', a, 'a')
    _w = nag_double_type_check_and_cast('f08fac', w, 'w')
    f08fac_ctypes(order, job, uplo, n, _a, pda, _w, fail)

f08fbc_ctypes = _library_cl.f08fbc
f08fbc_ctypes.restype = None
f08fbc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_RangeType, Nag_UploType, Integer, POINTER(c_double), Integer, c_double, c_double, Integer, Integer, c_double, POINTER(Integer), POINTER(c_double), POINTER(c_double), Integer, POINTER(Integer), POINTER(NagError)]


def f08fbc(order, job, _range, uplo, n, a, pda, vl, vu, il, iu, abstol, m, w, z, pdz, jfail, fail):
    """
    Computes selected eigenvalues and, optionally, eigenvectors of a real
    symmetric matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08fbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, m, w, z, jfail
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, w, z
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m, jfail
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08fbc', a, 'a')
    _m = nag_integer_type_check_and_cast('f08fbc', m, 'm')
    _w = nag_double_type_check_and_cast('f08fbc', w, 'w')
    _z = nag_double_type_check_and_cast('f08fbc', z, 'z')
    _jfail = nag_integer_type_check_and_cast('f08fbc', jfail, 'jfail')
    f08fbc_ctypes(order, job, _range, uplo, n, _a, pda, vl, vu, il, iu, abstol, _m, _w, _z, pdz, _jfail, fail)

f08fcc_ctypes = _library_cl.f08fcc
f08fcc_ctypes.restype = None
f08fcc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_UploType, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f08fcc(order, job, uplo, n, a, pda, w, fail):
    """
    Computes all eigenvalues and, optionally, all eigenvectors of real
    symmetric matrix (divide-and-conquer).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08fcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, w
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, w
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08fcc', a, 'a')
    _w = nag_double_type_check_and_cast('f08fcc', w, 'w')
    f08fcc_ctypes(order, job, uplo, n, _a, pda, _w, fail)

f08fdc_ctypes = _library_cl.f08fdc
f08fdc_ctypes.restype = None
f08fdc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_RangeType, Nag_UploType, Integer, POINTER(c_double), Integer, c_double, c_double, Integer, Integer, c_double, POINTER(Integer), POINTER(c_double), POINTER(c_double), Integer, POINTER(Integer), POINTER(NagError)]


def f08fdc(order, job, _range, uplo, n, a, pda, vl, vu, il, iu, abstol, m, w, z, pdz, isuppz, fail):
    """
    Computes selected eigenvalues and, optionally, eigenvectors of a real
    symmetric matrix (Relatively Robust Representations).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08fdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, m, w, z, isuppz
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, w, z
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m, isuppz
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08fdc', a, 'a')
    _m = nag_integer_type_check_and_cast('f08fdc', m, 'm')
    _w = nag_double_type_check_and_cast('f08fdc', w, 'w')
    _z = nag_double_type_check_and_cast('f08fdc', z, 'z')
    _isuppz = nag_integer_type_check_and_cast('f08fdc', isuppz, 'isuppz')
    f08fdc_ctypes(order, job, _range, uplo, n, _a, pda, vl, vu, il, iu, abstol, _m, _w, _z, pdz, _isuppz, fail)

f08fec_ctypes = _library_cl.f08fec
f08fec_ctypes.restype = None
f08fec_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f08fec(order, uplo, n, a, pda, d, e, tau, fail):
    """
    Orthogonal reduction of real symmetric matrix to symmetric tridiagonal
    form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08fec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, d, e, tau
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, d, e, tau
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08fec', a, 'a')
    _d = nag_double_type_check_and_cast('f08fec', d, 'd')
    _e = nag_double_type_check_and_cast('f08fec', e, 'e')
    _tau = nag_double_type_check_and_cast('f08fec', tau, 'tau')
    f08fec_ctypes(order, uplo, n, _a, pda, _d, _e, _tau, fail)

f08ffc_ctypes = _library_cl.f08ffc
f08ffc_ctypes.restype = None
f08ffc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f08ffc(order, uplo, n, a, pda, tau, fail):
    """
    Generate orthogonal transformation matrix from reduction to tridiagonal
    form determined by f08fec.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08ffc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, tau
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08ffc', a, 'a')
    _tau = nag_double_type_check_and_cast('f08ffc', tau, 'tau')
    f08ffc_ctypes(order, uplo, n, _a, pda, _tau, fail)

f08fgc_ctypes = _library_cl.f08fgc
f08fgc_ctypes.restype = None
f08fgc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_UploType, Nag_TransType, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f08fgc(order, side, uplo, trans, m, n, a, pda, tau, c, pdc, fail):
    """
    Applies the orthogonal transformation determined by f08fec.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08fgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau, c
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, tau, c
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08fgc', a, 'a')
    _tau = nag_double_type_check_and_cast('f08fgc', tau, 'tau')
    _c = nag_double_type_check_and_cast('f08fgc', c, 'c')
    f08fgc_ctypes(order, side, uplo, trans, m, n, _a, pda, _tau, _c, pdc, fail)

f08flc_ctypes = _library_cl.f08flc
f08flc_ctypes.restype = None
f08flc_ctypes.argtypes = [Nag_JobType, Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f08flc(job, m, n, d, sep, fail):
    """
    Computes the reciprocal condition numbers for the eigenvectors of a real
    symmetric or complex Hermitian matrix or for the left or right singular
    vectors of a general matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08flc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        d, sep
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, sep
    """
    from .util import nag_double_type_check_and_cast
    _d = nag_double_type_check_and_cast('f08flc', d, 'd')
    _sep = nag_double_type_check_and_cast('f08flc', sep, 'sep')
    f08flc_ctypes(job, m, n, _d, _sep, fail)

f08fnc_ctypes = _library_cl.f08fnc
f08fnc_ctypes.restype = None
f08fnc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_UploType, Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(NagError)]


def f08fnc(order, job, uplo, n, a, pda, w, fail):
    """
    Computes all eigenvalues and, optionally, eigenvectors of a complex
    Hermitian matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08fnc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, w
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        w
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08fnc', a, 'a')
    _w = nag_double_type_check_and_cast('f08fnc', w, 'w')
    f08fnc_ctypes(order, job, uplo, n, _a, pda, _w, fail)

f08fpc_ctypes = _library_cl.f08fpc
f08fpc_ctypes.restype = None
f08fpc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_RangeType, Nag_UploType, Integer, POINTER(Complex), Integer, c_double, c_double, Integer, Integer, c_double, POINTER(Integer), POINTER(c_double), POINTER(Complex), Integer, POINTER(Integer), POINTER(NagError)]


def f08fpc(order, job, _range, uplo, n, a, pda, vl, vu, il, iu, abstol, m, w, z, pdz, jfail, fail):
    """
    Computes selected eigenvalues and, optionally, eigenvectors of a complex
    Hermitian matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08fpc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, m, w, z, jfail
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        w
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m, jfail
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, z
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08fpc', a, 'a')
    _m = nag_integer_type_check_and_cast('f08fpc', m, 'm')
    _w = nag_double_type_check_and_cast('f08fpc', w, 'w')
    _z = nag_complex_type_check_and_cast('f08fpc', z, 'z')
    _jfail = nag_integer_type_check_and_cast('f08fpc', jfail, 'jfail')
    f08fpc_ctypes(order, job, _range, uplo, n, _a, pda, vl, vu, il, iu, abstol, _m, _w, _z, pdz, _jfail, fail)

f08fqc_ctypes = _library_cl.f08fqc
f08fqc_ctypes.restype = None
f08fqc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_UploType, Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(NagError)]


def f08fqc(order, job, uplo, n, a, pda, w, fail):
    """
    Computes all eigenvalues and, optionally, all eigenvectors of complex
    Hermitian matrix (divide-and-conquer).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08fqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, w
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        w
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08fqc', a, 'a')
    _w = nag_double_type_check_and_cast('f08fqc', w, 'w')
    f08fqc_ctypes(order, job, uplo, n, _a, pda, _w, fail)

f08frc_ctypes = _library_cl.f08frc
f08frc_ctypes.restype = None
f08frc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_RangeType, Nag_UploType, Integer, POINTER(Complex), Integer, c_double, c_double, Integer, Integer, c_double, POINTER(Integer), POINTER(c_double), POINTER(Complex), Integer, POINTER(Integer), POINTER(NagError)]


def f08frc(order, job, _range, uplo, n, a, pda, vl, vu, il, iu, abstol, m, w, z, pdz, isuppz, fail):
    """
    Computes selected eigenvalues and, optionally, eigenvectors of a complex
    Hermitian matrix (Relatively Robust Representations).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08frc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, m, w, z, isuppz
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        w
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m, isuppz
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, z
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08frc', a, 'a')
    _m = nag_integer_type_check_and_cast('f08frc', m, 'm')
    _w = nag_double_type_check_and_cast('f08frc', w, 'w')
    _z = nag_complex_type_check_and_cast('f08frc', z, 'z')
    _isuppz = nag_integer_type_check_and_cast('f08frc', isuppz, 'isuppz')
    f08frc_ctypes(order, job, _range, uplo, n, _a, pda, vl, vu, il, iu, abstol, _m, _w, _z, pdz, _isuppz, fail)

f08fsc_ctypes = _library_cl.f08fsc
f08fsc_ctypes.restype = None
f08fsc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(c_double), POINTER(Complex), POINTER(NagError)]


def f08fsc(order, uplo, n, a, pda, d, e, tau, fail):
    """
    Unitary reduction of complex Hermitian matrix to real symmetric
    tridiagonal form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08fsc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, d, e, tau
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, e
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, tau
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08fsc', a, 'a')
    _d = nag_double_type_check_and_cast('f08fsc', d, 'd')
    _e = nag_double_type_check_and_cast('f08fsc', e, 'e')
    _tau = nag_complex_type_check_and_cast('f08fsc', tau, 'tau')
    f08fsc_ctypes(order, uplo, n, _a, pda, _d, _e, _tau, fail)

f08ftc_ctypes = _library_cl.f08ftc
f08ftc_ctypes.restype = None
f08ftc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(NagError)]


def f08ftc(order, uplo, n, a, pda, tau, fail):
    """
    Generate unitary transformation matrix from reduction to tridiagonal
    form determined by f08fsc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08ftc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, tau
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08ftc', a, 'a')
    _tau = nag_complex_type_check_and_cast('f08ftc', tau, 'tau')
    f08ftc_ctypes(order, uplo, n, _a, pda, _tau, fail)

f08fuc_ctypes = _library_cl.f08fuc
f08fuc_ctypes.restype = None
f08fuc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_UploType, Nag_TransType, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(Complex), Integer, POINTER(NagError)]


def f08fuc(order, side, uplo, trans, m, n, a, pda, tau, c, pdc, fail):
    """
    Applies the unitary transformation matrix determined by f08fsc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08fuc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau, c
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, tau, c
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08fuc', a, 'a')
    _tau = nag_complex_type_check_and_cast('f08fuc', tau, 'tau')
    _c = nag_complex_type_check_and_cast('f08fuc', c, 'c')
    f08fuc_ctypes(order, side, uplo, trans, m, n, _a, pda, _tau, _c, pdc, fail)

f08gac_ctypes = _library_cl.f08gac
f08gac_ctypes.restype = None
f08gac_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_UploType, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f08gac(order, job, uplo, n, ap, w, z, pdz, fail):
    """
    Computes all eigenvalues and, optionally, eigenvectors of a real
    symmetric matrix, packed storage.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08gac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, w, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ap, w, z
    """
    from .util import nag_double_type_check_and_cast
    _ap = nag_double_type_check_and_cast('f08gac', ap, 'ap')
    _w = nag_double_type_check_and_cast('f08gac', w, 'w')
    _z = nag_double_type_check_and_cast('f08gac', z, 'z')
    f08gac_ctypes(order, job, uplo, n, _ap, _w, _z, pdz, fail)

f08gbc_ctypes = _library_cl.f08gbc
f08gbc_ctypes.restype = None
f08gbc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_RangeType, Nag_UploType, Integer, POINTER(c_double), c_double, c_double, Integer, Integer, c_double, POINTER(Integer), POINTER(c_double), POINTER(c_double), Integer, POINTER(Integer), POINTER(NagError)]


def f08gbc(order, job, _range, uplo, n, ap, vl, vu, il, iu, abstol, m, w, z, pdz, jfail, fail):
    """
    Computes selected eigenvalues and, optionally, eigenvectors of a real
    symmetric matrix, packed storage.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08gbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, m, w, z, jfail
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ap, w, z
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m, jfail
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _ap = nag_double_type_check_and_cast('f08gbc', ap, 'ap')
    _m = nag_integer_type_check_and_cast('f08gbc', m, 'm')
    _w = nag_double_type_check_and_cast('f08gbc', w, 'w')
    _z = nag_double_type_check_and_cast('f08gbc', z, 'z')
    _jfail = nag_integer_type_check_and_cast('f08gbc', jfail, 'jfail')
    f08gbc_ctypes(order, job, _range, uplo, n, _ap, vl, vu, il, iu, abstol, _m, _w, _z, pdz, _jfail, fail)

f08gcc_ctypes = _library_cl.f08gcc
f08gcc_ctypes.restype = None
f08gcc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_UploType, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f08gcc(order, job, uplo, n, ap, w, z, pdz, fail):
    """
    Computes all eigenvalues and, optionally, all eigenvectors of real
    symmetric matrix, packed storage (divide-and-conquer or Pal--Walker--
    Kahan variant of the QL or QR algorithm).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08gcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, w, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ap, w, z
    """
    from .util import nag_double_type_check_and_cast
    _ap = nag_double_type_check_and_cast('f08gcc', ap, 'ap')
    _w = nag_double_type_check_and_cast('f08gcc', w, 'w')
    _z = nag_double_type_check_and_cast('f08gcc', z, 'z')
    f08gcc_ctypes(order, job, uplo, n, _ap, _w, _z, pdz, fail)

f08gec_ctypes = _library_cl.f08gec
f08gec_ctypes.restype = None
f08gec_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f08gec(order, uplo, n, ap, d, e, tau, fail):
    """
    Orthogonal reduction of real symmetric matrix to symmetric tridiagonal
    form, packed storage.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08gec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, d, e, tau
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ap, d, e, tau
    """
    from .util import nag_double_type_check_and_cast
    _ap = nag_double_type_check_and_cast('f08gec', ap, 'ap')
    _d = nag_double_type_check_and_cast('f08gec', d, 'd')
    _e = nag_double_type_check_and_cast('f08gec', e, 'e')
    _tau = nag_double_type_check_and_cast('f08gec', tau, 'tau')
    f08gec_ctypes(order, uplo, n, _ap, _d, _e, _tau, fail)

f08gfc_ctypes = _library_cl.f08gfc
f08gfc_ctypes.restype = None
f08gfc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f08gfc(order, uplo, n, ap, tau, q, pdq, fail):
    """
    Generate orthogonal transformation matrix from reduction to tridiagonal
    form determined by f08gec.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08gfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, tau, q
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ap, tau, q
    """
    from .util import nag_double_type_check_and_cast
    _ap = nag_double_type_check_and_cast('f08gfc', ap, 'ap')
    _tau = nag_double_type_check_and_cast('f08gfc', tau, 'tau')
    _q = nag_double_type_check_and_cast('f08gfc', q, 'q')
    f08gfc_ctypes(order, uplo, n, _ap, _tau, _q, pdq, fail)

f08ggc_ctypes = _library_cl.f08ggc
f08ggc_ctypes.restype = None
f08ggc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_UploType, Nag_TransType, Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f08ggc(order, side, uplo, trans, m, n, ap, tau, c, pdc, fail):
    """
    Applies the orthogonal transformation determined by f08gec.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08ggc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, tau, c
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ap, tau, c
    """
    from .util import nag_double_type_check_and_cast
    _ap = nag_double_type_check_and_cast('f08ggc', ap, 'ap')
    _tau = nag_double_type_check_and_cast('f08ggc', tau, 'tau')
    _c = nag_double_type_check_and_cast('f08ggc', c, 'c')
    f08ggc_ctypes(order, side, uplo, trans, m, n, _ap, _tau, _c, pdc, fail)

f08gnc_ctypes = _library_cl.f08gnc
f08gnc_ctypes.restype = None
f08gnc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_UploType, Integer, POINTER(Complex), POINTER(c_double), POINTER(Complex), Integer, POINTER(NagError)]


def f08gnc(order, job, uplo, n, ap, w, z, pdz, fail):
    """
    Computes all eigenvalues and, optionally, eigenvectors of a complex
    Hermitian matrix, packed storage.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08gnc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, w, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        w
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ap, z
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _ap = nag_complex_type_check_and_cast('f08gnc', ap, 'ap')
    _w = nag_double_type_check_and_cast('f08gnc', w, 'w')
    _z = nag_complex_type_check_and_cast('f08gnc', z, 'z')
    f08gnc_ctypes(order, job, uplo, n, _ap, _w, _z, pdz, fail)

f08gpc_ctypes = _library_cl.f08gpc
f08gpc_ctypes.restype = None
f08gpc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_RangeType, Nag_UploType, Integer, POINTER(Complex), c_double, c_double, Integer, Integer, c_double, POINTER(Integer), POINTER(c_double), POINTER(Complex), Integer, POINTER(Integer), POINTER(NagError)]


def f08gpc(order, job, _range, uplo, n, ap, vl, vu, il, iu, abstol, m, w, z, pdz, jfail, fail):
    """
    Computes selected eigenvalues and, optionally, eigenvectors of a complex
    Hermitian matrix, packed storage.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08gpc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, m, w, z, jfail
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        w
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m, jfail
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ap, z
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _ap = nag_complex_type_check_and_cast('f08gpc', ap, 'ap')
    _m = nag_integer_type_check_and_cast('f08gpc', m, 'm')
    _w = nag_double_type_check_and_cast('f08gpc', w, 'w')
    _z = nag_complex_type_check_and_cast('f08gpc', z, 'z')
    _jfail = nag_integer_type_check_and_cast('f08gpc', jfail, 'jfail')
    f08gpc_ctypes(order, job, _range, uplo, n, _ap, vl, vu, il, iu, abstol, _m, _w, _z, pdz, _jfail, fail)

f08gqc_ctypes = _library_cl.f08gqc
f08gqc_ctypes.restype = None
f08gqc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_UploType, Integer, POINTER(Complex), POINTER(c_double), POINTER(Complex), Integer, POINTER(NagError)]


def f08gqc(order, job, uplo, n, ap, w, z, pdz, fail):
    """
    Computes all eigenvalues and, optionally, all eigenvectors of complex
    Hermitian matrix, packed storage (divide-and-conquer or Pal--Walker--
    Kahan variant of the QL or QR algorithm).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08gqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, w, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        w
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ap, z
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _ap = nag_complex_type_check_and_cast('f08gqc', ap, 'ap')
    _w = nag_double_type_check_and_cast('f08gqc', w, 'w')
    _z = nag_complex_type_check_and_cast('f08gqc', z, 'z')
    f08gqc_ctypes(order, job, uplo, n, _ap, _w, _z, pdz, fail)

f08gsc_ctypes = _library_cl.f08gsc
f08gsc_ctypes.restype = None
f08gsc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, POINTER(Complex), POINTER(c_double), POINTER(c_double), POINTER(Complex), POINTER(NagError)]


def f08gsc(order, uplo, n, ap, d, e, tau, fail):
    """
    Performs a unitary reduction of complex Hermitian matrix to real
    symmetric tridiagonal form, packed storage.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08gsc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, d, e, tau
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, e
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ap, tau
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _ap = nag_complex_type_check_and_cast('f08gsc', ap, 'ap')
    _d = nag_double_type_check_and_cast('f08gsc', d, 'd')
    _e = nag_double_type_check_and_cast('f08gsc', e, 'e')
    _tau = nag_complex_type_check_and_cast('f08gsc', tau, 'tau')
    f08gsc_ctypes(order, uplo, n, _ap, _d, _e, _tau, fail)

f08gtc_ctypes = _library_cl.f08gtc
f08gtc_ctypes.restype = None
f08gtc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, POINTER(Complex), POINTER(Complex), POINTER(Complex), Integer, POINTER(NagError)]


def f08gtc(order, uplo, n, ap, tau, q, pdq, fail):
    """
    Generates a unitary transformation matrix from reduction to tridiagonal
    form determined by f08gsc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08gtc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, tau, q
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ap, tau, q
    """
    from .util import nag_complex_type_check_and_cast
    _ap = nag_complex_type_check_and_cast('f08gtc', ap, 'ap')
    _tau = nag_complex_type_check_and_cast('f08gtc', tau, 'tau')
    _q = nag_complex_type_check_and_cast('f08gtc', q, 'q')
    f08gtc_ctypes(order, uplo, n, _ap, _tau, _q, pdq, fail)

f08guc_ctypes = _library_cl.f08guc
f08guc_ctypes.restype = None
f08guc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_UploType, Nag_TransType, Integer, Integer, POINTER(Complex), POINTER(Complex), POINTER(Complex), Integer, POINTER(NagError)]


def f08guc(order, side, uplo, trans, m, n, ap, tau, c, pdc, fail):
    """
    Applies the unitary transformation matrix determined by f08gsc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08guc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, tau, c
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ap, tau, c
    """
    from .util import nag_complex_type_check_and_cast
    _ap = nag_complex_type_check_and_cast('f08guc', ap, 'ap')
    _tau = nag_complex_type_check_and_cast('f08guc', tau, 'tau')
    _c = nag_complex_type_check_and_cast('f08guc', c, 'c')
    f08guc_ctypes(order, side, uplo, trans, m, n, _ap, _tau, _c, pdc, fail)

f08hac_ctypes = _library_cl.f08hac
f08hac_ctypes.restype = None
f08hac_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_UploType, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f08hac(order, job, uplo, n, kd, ab, pdab, w, z, pdz, fail):
    """
    Computes all eigenvalues and, optionally, eigenvectors of a real
    symmetric band matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08hac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, w, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ab, w, z
    """
    from .util import nag_double_type_check_and_cast
    _ab = nag_double_type_check_and_cast('f08hac', ab, 'ab')
    _w = nag_double_type_check_and_cast('f08hac', w, 'w')
    _z = nag_double_type_check_and_cast('f08hac', z, 'z')
    f08hac_ctypes(order, job, uplo, n, kd, _ab, pdab, _w, _z, pdz, fail)

f08hbc_ctypes = _library_cl.f08hbc
f08hbc_ctypes.restype = None
f08hbc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_RangeType, Nag_UploType, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, c_double, c_double, Integer, Integer, c_double, POINTER(Integer), POINTER(c_double), POINTER(c_double), Integer, POINTER(Integer), POINTER(NagError)]


def f08hbc(order, job, _range, uplo, n, kd, ab, pdab, q, pdq, vl, vu, il, iu, abstol, m, w, z, pdz, jfail, fail):
    """
    Computes selected eigenvalues and, optionally, eigenvectors of a real
    symmetric band matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08hbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, q, m, w, z, jfail
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ab, q, w, z
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m, jfail
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _ab = nag_double_type_check_and_cast('f08hbc', ab, 'ab')
    _q = nag_double_type_check_and_cast('f08hbc', q, 'q')
    _m = nag_integer_type_check_and_cast('f08hbc', m, 'm')
    _w = nag_double_type_check_and_cast('f08hbc', w, 'w')
    _z = nag_double_type_check_and_cast('f08hbc', z, 'z')
    _jfail = nag_integer_type_check_and_cast('f08hbc', jfail, 'jfail')
    f08hbc_ctypes(order, job, _range, uplo, n, kd, _ab, pdab, _q, pdq, vl, vu, il, iu, abstol, _m, _w, _z, pdz, _jfail, fail)

f08hcc_ctypes = _library_cl.f08hcc
f08hcc_ctypes.restype = None
f08hcc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_UploType, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f08hcc(order, job, uplo, n, kd, ab, pdab, w, z, pdz, fail):
    """
    Computes all eigenvalues and, optionally, all eigenvectors of real
    symmetric band matrix (divide-and-conquer or Pal--Walker--Kahan variant
    of the QL or QR algorithm).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08hcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, w, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ab, w, z
    """
    from .util import nag_double_type_check_and_cast
    _ab = nag_double_type_check_and_cast('f08hcc', ab, 'ab')
    _w = nag_double_type_check_and_cast('f08hcc', w, 'w')
    _z = nag_double_type_check_and_cast('f08hcc', z, 'z')
    f08hcc_ctypes(order, job, uplo, n, kd, _ab, pdab, _w, _z, pdz, fail)

f08hec_ctypes = _library_cl.f08hec
f08hec_ctypes.restype = None
f08hec_ctypes.argtypes = [Nag_OrderType, Nag_VectType, Nag_UploType, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f08hec(order, vect, uplo, n, kd, ab, pdab, d, e, q, pdq, fail):
    """
    Performs an orthogonal reduction of real symmetric band matrix to
    symmetric tridiagonal form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08hec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, d, e, q
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ab, d, e, q
    """
    from .util import nag_double_type_check_and_cast
    _ab = nag_double_type_check_and_cast('f08hec', ab, 'ab')
    _d = nag_double_type_check_and_cast('f08hec', d, 'd')
    _e = nag_double_type_check_and_cast('f08hec', e, 'e')
    _q = nag_double_type_check_and_cast('f08hec', q, 'q')
    f08hec_ctypes(order, vect, uplo, n, kd, _ab, pdab, _d, _e, _q, pdq, fail)

f08hnc_ctypes = _library_cl.f08hnc
f08hnc_ctypes.restype = None
f08hnc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_UploType, Integer, Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(Complex), Integer, POINTER(NagError)]


def f08hnc(order, job, uplo, n, kd, ab, pdab, w, z, pdz, fail):
    """
    Computes all eigenvalues and, optionally, eigenvectors of a complex
    Hermitian band matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08hnc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, w, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        w
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ab, z
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _ab = nag_complex_type_check_and_cast('f08hnc', ab, 'ab')
    _w = nag_double_type_check_and_cast('f08hnc', w, 'w')
    _z = nag_complex_type_check_and_cast('f08hnc', z, 'z')
    f08hnc_ctypes(order, job, uplo, n, kd, _ab, pdab, _w, _z, pdz, fail)

f08hpc_ctypes = _library_cl.f08hpc
f08hpc_ctypes.restype = None
f08hpc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_RangeType, Nag_UploType, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, c_double, c_double, Integer, Integer, c_double, POINTER(Integer), POINTER(c_double), POINTER(Complex), Integer, POINTER(Integer), POINTER(NagError)]


def f08hpc(order, job, _range, uplo, n, kd, ab, pdab, q, pdq, vl, vu, il, iu, abstol, m, w, z, pdz, jfail, fail):
    """
    Computes selected eigenvalues and, optionally, eigenvectors of a complex
    Hermitian band matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08hpc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, q, m, w, z, jfail
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        w
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m, jfail
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ab, q, z
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _ab = nag_complex_type_check_and_cast('f08hpc', ab, 'ab')
    _q = nag_complex_type_check_and_cast('f08hpc', q, 'q')
    _m = nag_integer_type_check_and_cast('f08hpc', m, 'm')
    _w = nag_double_type_check_and_cast('f08hpc', w, 'w')
    _z = nag_complex_type_check_and_cast('f08hpc', z, 'z')
    _jfail = nag_integer_type_check_and_cast('f08hpc', jfail, 'jfail')
    f08hpc_ctypes(order, job, _range, uplo, n, kd, _ab, pdab, _q, pdq, vl, vu, il, iu, abstol, _m, _w, _z, pdz, _jfail, fail)

f08hqc_ctypes = _library_cl.f08hqc
f08hqc_ctypes.restype = None
f08hqc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_UploType, Integer, Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(Complex), Integer, POINTER(NagError)]


def f08hqc(order, job, uplo, n, kd, ab, pdab, w, z, pdz, fail):
    """
    Computes all eigenvalues and, optionally, all eigenvectors of complex
    Hermitian band matrix (divide-and-conquer).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08hqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, w, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        w
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ab, z
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _ab = nag_complex_type_check_and_cast('f08hqc', ab, 'ab')
    _w = nag_double_type_check_and_cast('f08hqc', w, 'w')
    _z = nag_complex_type_check_and_cast('f08hqc', z, 'z')
    f08hqc_ctypes(order, job, uplo, n, kd, _ab, pdab, _w, _z, pdz, fail)

f08hsc_ctypes = _library_cl.f08hsc
f08hsc_ctypes.restype = None
f08hsc_ctypes.argtypes = [Nag_OrderType, Nag_VectType, Nag_UploType, Integer, Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(c_double), POINTER(Complex), Integer, POINTER(NagError)]


def f08hsc(order, vect, uplo, n, kd, ab, pdab, d, e, q, pdq, fail):
    """
    Performs a unitary reduction of complex Hermitian band matrix to real
    symmetric tridiagonal form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08hsc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, d, e, q
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, e
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ab, q
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _ab = nag_complex_type_check_and_cast('f08hsc', ab, 'ab')
    _d = nag_double_type_check_and_cast('f08hsc', d, 'd')
    _e = nag_double_type_check_and_cast('f08hsc', e, 'e')
    _q = nag_complex_type_check_and_cast('f08hsc', q, 'q')
    f08hsc_ctypes(order, vect, uplo, n, kd, _ab, pdab, _d, _e, _q, pdq, fail)

f08jac_ctypes = _library_cl.f08jac
f08jac_ctypes.restype = None
f08jac_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f08jac(order, job, n, d, e, z, pdz, fail):
    """
    Computes all eigenvalues and, optionally, eigenvectors of a real
    symmetric tridiagonal matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08jac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        d, e, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, e, z
    """
    from .util import nag_double_type_check_and_cast
    _d = nag_double_type_check_and_cast('f08jac', d, 'd')
    _e = nag_double_type_check_and_cast('f08jac', e, 'e')
    _z = nag_double_type_check_and_cast('f08jac', z, 'z')
    f08jac_ctypes(order, job, n, _d, _e, _z, pdz, fail)

f08jbc_ctypes = _library_cl.f08jbc
f08jbc_ctypes.restype = None
f08jbc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_RangeType, Integer, POINTER(c_double), POINTER(c_double), c_double, c_double, Integer, Integer, c_double, POINTER(Integer), POINTER(c_double), POINTER(c_double), Integer, POINTER(Integer), POINTER(NagError)]


def f08jbc(order, job, _range, n, d, e, vl, vu, il, iu, abstol, m, w, z, pdz, jfail, fail):
    """
    Computes selected eigenvalues and, optionally, eigenvectors of a real
    symmetric tridiagonal matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08jbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        d, e, m, w, z, jfail
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, e, w, z
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m, jfail
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _d = nag_double_type_check_and_cast('f08jbc', d, 'd')
    _e = nag_double_type_check_and_cast('f08jbc', e, 'e')
    _m = nag_integer_type_check_and_cast('f08jbc', m, 'm')
    _w = nag_double_type_check_and_cast('f08jbc', w, 'w')
    _z = nag_double_type_check_and_cast('f08jbc', z, 'z')
    _jfail = nag_integer_type_check_and_cast('f08jbc', jfail, 'jfail')
    f08jbc_ctypes(order, job, _range, n, _d, _e, vl, vu, il, iu, abstol, _m, _w, _z, pdz, _jfail, fail)

f08jcc_ctypes = _library_cl.f08jcc
f08jcc_ctypes.restype = None
f08jcc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f08jcc(order, job, n, d, e, z, pdz, fail):
    """
    Computes all eigenvalues and, optionally, all eigenvectors of real
    symmetric tridiagonal matrix (divide-and-conquer).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08jcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        d, e, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, e, z
    """
    from .util import nag_double_type_check_and_cast
    _d = nag_double_type_check_and_cast('f08jcc', d, 'd')
    _e = nag_double_type_check_and_cast('f08jcc', e, 'e')
    _z = nag_double_type_check_and_cast('f08jcc', z, 'z')
    f08jcc_ctypes(order, job, n, _d, _e, _z, pdz, fail)

f08jdc_ctypes = _library_cl.f08jdc
f08jdc_ctypes.restype = None
f08jdc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_RangeType, Integer, POINTER(c_double), POINTER(c_double), c_double, c_double, Integer, Integer, c_double, POINTER(Integer), POINTER(c_double), POINTER(c_double), Integer, POINTER(Integer), POINTER(NagError)]


def f08jdc(order, job, _range, n, d, e, vl, vu, il, iu, abstol, m, w, z, pdz, isuppz, fail):
    """
    Computes selected eigenvalues and, optionally, eigenvectors of a real
    symmetric tridiagonal matrix (Relatively Robust Representations).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08jdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        d, e, m, w, z, isuppz
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, e, w, z
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m, isuppz
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _d = nag_double_type_check_and_cast('f08jdc', d, 'd')
    _e = nag_double_type_check_and_cast('f08jdc', e, 'e')
    _m = nag_integer_type_check_and_cast('f08jdc', m, 'm')
    _w = nag_double_type_check_and_cast('f08jdc', w, 'w')
    _z = nag_double_type_check_and_cast('f08jdc', z, 'z')
    _isuppz = nag_integer_type_check_and_cast('f08jdc', isuppz, 'isuppz')
    f08jdc_ctypes(order, job, _range, n, _d, _e, vl, vu, il, iu, abstol, _m, _w, _z, pdz, _isuppz, fail)

f08jec_ctypes = _library_cl.f08jec
f08jec_ctypes.restype = None
f08jec_ctypes.argtypes = [Nag_OrderType, Nag_ComputeZType, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f08jec(order, compz, n, d, e, z, pdz, fail):
    """
    Computes all eigenvalues and eigenvectors of real symmetric tridiagonal
    matrix, reduced from real symmetric matrix using the implicit QL or QR
    algorithm.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08jec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        d, e, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, e, z
    """
    from .util import nag_double_type_check_and_cast
    _d = nag_double_type_check_and_cast('f08jec', d, 'd')
    _e = nag_double_type_check_and_cast('f08jec', e, 'e')
    _z = nag_double_type_check_and_cast('f08jec', z, 'z')
    f08jec_ctypes(order, compz, n, _d, _e, _z, pdz, fail)

f08jfc_ctypes = _library_cl.f08jfc
f08jfc_ctypes.restype = None
f08jfc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f08jfc(n, d, e, fail):
    """
    Computes all eigenvalues of real symmetric tridiagonal matrix, root-free
    variant of the QL or QR algorithm.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08jfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        d, e
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, e
    """
    from .util import nag_double_type_check_and_cast
    _d = nag_double_type_check_and_cast('f08jfc', d, 'd')
    _e = nag_double_type_check_and_cast('f08jfc', e, 'e')
    f08jfc_ctypes(n, _d, _e, fail)

f08jgc_ctypes = _library_cl.f08jgc
f08jgc_ctypes.restype = None
f08jgc_ctypes.argtypes = [Nag_OrderType, Nag_ComputeZType, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f08jgc(order, compz, n, d, e, z, pdz, fail):
    """
    Computes all eigenvalues and eigenvectors of real symmetric positive
    definite tridiagonal matrix, reduced from real symmetric positive
    definite matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08jgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        d, e, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, e, z
    """
    from .util import nag_double_type_check_and_cast
    _d = nag_double_type_check_and_cast('f08jgc', d, 'd')
    _e = nag_double_type_check_and_cast('f08jgc', e, 'e')
    _z = nag_double_type_check_and_cast('f08jgc', z, 'z')
    f08jgc_ctypes(order, compz, n, _d, _e, _z, pdz, fail)

f08jhc_ctypes = _library_cl.f08jhc
f08jhc_ctypes.restype = None
f08jhc_ctypes.argtypes = [Nag_OrderType, Nag_ComputeEigVecsType, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f08jhc(order, compz, n, d, e, z, pdz, fail):
    """
    Computes all eigenvalues and, optionally, eigenvectors of a real
    symmetric tridiagonal matrix or a matrix reduced to this form (divide-
    and-conquer).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08jhc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        d, e, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, e, z
    """
    from .util import nag_double_type_check_and_cast
    _d = nag_double_type_check_and_cast('f08jhc', d, 'd')
    _e = nag_double_type_check_and_cast('f08jhc', e, 'e')
    _z = nag_double_type_check_and_cast('f08jhc', z, 'z')
    f08jhc_ctypes(order, compz, n, _d, _e, _z, pdz, fail)

f08jjc_ctypes = _library_cl.f08jjc
f08jjc_ctypes.restype = None
f08jjc_ctypes.argtypes = [Nag_RangeType, Nag_EigValRankType, Integer, c_double, c_double, Integer, Integer, c_double, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(NagError)]


def f08jjc(_range, rank, n, vl, vu, il, iu, abstol, d, e, m, nsplit, w, iblock, isplit, fail):
    """
    Computes selected eigenvalues of real symmetric tridiagonal matrix by
    bisection.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08jjc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        d, e, m, nsplit, w, iblock, isplit
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, e, w
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m, nsplit, iblock, isplit
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _d = nag_double_type_check_and_cast('f08jjc', d, 'd')
    _e = nag_double_type_check_and_cast('f08jjc', e, 'e')
    _m = nag_integer_type_check_and_cast('f08jjc', m, 'm')
    _nsplit = nag_integer_type_check_and_cast('f08jjc', nsplit, 'nsplit')
    _w = nag_double_type_check_and_cast('f08jjc', w, 'w')
    _iblock = nag_integer_type_check_and_cast('f08jjc', iblock, 'iblock')
    _isplit = nag_integer_type_check_and_cast('f08jjc', isplit, 'isplit')
    f08jjc_ctypes(_range, rank, n, vl, vu, il, iu, abstol, _d, _e, _m, _nsplit, _w, _iblock, _isplit, fail)

f08jkc_ctypes = _library_cl.f08jkc
f08jkc_ctypes.restype = None
f08jkc_ctypes.argtypes = [Nag_OrderType, Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(c_double), Integer, POINTER(Integer), POINTER(NagError)]


def f08jkc(order, n, d, e, m, w, iblock, isplit, z, pdz, ifailv, fail):
    """
    Computes selected eigenvectors of real symmetric tridiagonal matrix by
    inverse iteration, storing eigenvectors in real array.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08jkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        d, e, w, iblock, isplit, z, ifailv
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, e, w, z
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iblock, isplit, ifailv
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _d = nag_double_type_check_and_cast('f08jkc', d, 'd')
    _e = nag_double_type_check_and_cast('f08jkc', e, 'e')
    _w = nag_double_type_check_and_cast('f08jkc', w, 'w')
    _iblock = nag_integer_type_check_and_cast('f08jkc', iblock, 'iblock')
    _isplit = nag_integer_type_check_and_cast('f08jkc', isplit, 'isplit')
    _z = nag_double_type_check_and_cast('f08jkc', z, 'z')
    _ifailv = nag_integer_type_check_and_cast('f08jkc', ifailv, 'ifailv')
    f08jkc_ctypes(order, n, _d, _e, m, _w, _iblock, _isplit, _z, pdz, _ifailv, fail)

f08jlc_ctypes = _library_cl.f08jlc
f08jlc_ctypes.restype = None
f08jlc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_RangeType, Integer, POINTER(c_double), POINTER(c_double), c_double, c_double, Integer, Integer, POINTER(Integer), POINTER(c_double), POINTER(c_double), Integer, POINTER(Integer), POINTER(NagError)]


def f08jlc(order, job, _range, n, d, e, vl, vu, il, iu, m, w, z, pdz, isuppz, fail):
    """
    Computes all eigenvalues and, optionally, eigenvectors of a real
    symmetric tridiagonal matrix or a symmetric matrix reduced to this form
    (Relatively Robust Representations).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08jlc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        d, e, m, w, z, isuppz
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, e, w, z
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m, isuppz
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _d = nag_double_type_check_and_cast('f08jlc', d, 'd')
    _e = nag_double_type_check_and_cast('f08jlc', e, 'e')
    _m = nag_integer_type_check_and_cast('f08jlc', m, 'm')
    _w = nag_double_type_check_and_cast('f08jlc', w, 'w')
    _z = nag_double_type_check_and_cast('f08jlc', z, 'z')
    _isuppz = nag_integer_type_check_and_cast('f08jlc', isuppz, 'isuppz')
    f08jlc_ctypes(order, job, _range, n, _d, _e, vl, vu, il, iu, _m, _w, _z, pdz, _isuppz, fail)

f08jsc_ctypes = _library_cl.f08jsc
f08jsc_ctypes.restype = None
f08jsc_ctypes.argtypes = [Nag_OrderType, Nag_ComputeZType, Integer, POINTER(c_double), POINTER(c_double), POINTER(Complex), Integer, POINTER(NagError)]


def f08jsc(order, compz, n, d, e, z, pdz, fail):
    """
    Computes all eigenvalues and eigenvectors of real symmetric tridiagonal
    matrix, reduced from complex Hermitian matrix, using the implicit QL or
    QR algorithm.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08jsc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        d, e, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, e
    TypeError will be raised when the following arguments are not of data type
    Complex:
        z
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _d = nag_double_type_check_and_cast('f08jsc', d, 'd')
    _e = nag_double_type_check_and_cast('f08jsc', e, 'e')
    _z = nag_complex_type_check_and_cast('f08jsc', z, 'z')
    f08jsc_ctypes(order, compz, n, _d, _e, _z, pdz, fail)

f08juc_ctypes = _library_cl.f08juc
f08juc_ctypes.restype = None
f08juc_ctypes.argtypes = [Nag_OrderType, Nag_ComputeZType, Integer, POINTER(c_double), POINTER(c_double), POINTER(Complex), Integer, POINTER(NagError)]


def f08juc(order, compz, n, d, e, z, pdz, fail):
    """
    Computes all eigenvalues and eigenvectors of real symmetric positive
    definite tridiagonal matrix, reduced from complex Hermitian positive
    definite matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08juc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        d, e, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, e
    TypeError will be raised when the following arguments are not of data type
    Complex:
        z
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _d = nag_double_type_check_and_cast('f08juc', d, 'd')
    _e = nag_double_type_check_and_cast('f08juc', e, 'e')
    _z = nag_complex_type_check_and_cast('f08juc', z, 'z')
    f08juc_ctypes(order, compz, n, _d, _e, _z, pdz, fail)

f08jvc_ctypes = _library_cl.f08jvc
f08jvc_ctypes.restype = None
f08jvc_ctypes.argtypes = [Nag_OrderType, Nag_ComputeEigVecsType, Integer, POINTER(c_double), POINTER(c_double), POINTER(Complex), Integer, POINTER(NagError)]


def f08jvc(order, compz, n, d, e, z, pdz, fail):
    """
    Computes all eigenvalues and, optionally, eigenvectors of a real
    symmetric tridiagonal matrix or a complex Hermitian matrix reduced to
    this form (divide-and-conquer).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08jvc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        d, e, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, e
    TypeError will be raised when the following arguments are not of data type
    Complex:
        z
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _d = nag_double_type_check_and_cast('f08jvc', d, 'd')
    _e = nag_double_type_check_and_cast('f08jvc', e, 'e')
    _z = nag_complex_type_check_and_cast('f08jvc', z, 'z')
    f08jvc_ctypes(order, compz, n, _d, _e, _z, pdz, fail)

f08jxc_ctypes = _library_cl.f08jxc
f08jxc_ctypes.restype = None
f08jxc_ctypes.argtypes = [Nag_OrderType, Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(Complex), Integer, POINTER(Integer), POINTER(NagError)]


def f08jxc(order, n, d, e, m, w, iblock, isplit, z, pdz, ifailv, fail):
    """
    Computes selected eigenvectors of real symmetric tridiagonal matrix by
    inverse iteration, storing eigenvectors in complex array.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08jxc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        d, e, w, iblock, isplit, z, ifailv
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, e, w
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iblock, isplit, ifailv
    TypeError will be raised when the following arguments are not of data type
    Complex:
        z
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _d = nag_double_type_check_and_cast('f08jxc', d, 'd')
    _e = nag_double_type_check_and_cast('f08jxc', e, 'e')
    _w = nag_double_type_check_and_cast('f08jxc', w, 'w')
    _iblock = nag_integer_type_check_and_cast('f08jxc', iblock, 'iblock')
    _isplit = nag_integer_type_check_and_cast('f08jxc', isplit, 'isplit')
    _z = nag_complex_type_check_and_cast('f08jxc', z, 'z')
    _ifailv = nag_integer_type_check_and_cast('f08jxc', ifailv, 'ifailv')
    f08jxc_ctypes(order, n, _d, _e, m, _w, _iblock, _isplit, _z, pdz, _ifailv, fail)

f08jyc_ctypes = _library_cl.f08jyc
f08jyc_ctypes.restype = None
f08jyc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_RangeType, Integer, POINTER(c_double), POINTER(c_double), c_double, c_double, Integer, Integer, POINTER(Integer), POINTER(c_double), POINTER(Complex), Integer, POINTER(Integer), POINTER(NagError)]


def f08jyc(order, job, _range, n, d, e, vl, vu, il, iu, m, w, z, pdz, isuppz, fail):
    """
    Computes all eigenvalues and, optionally, eigenvectors of a real
    symmetric tridiagonal matrix or a complex Hermitian matrix reduced to
    this form (Relatively Robust Representations).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08jyc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        d, e, m, w, z, isuppz
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, e, w
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m, isuppz
    TypeError will be raised when the following arguments are not of data type
    Complex:
        z
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _d = nag_double_type_check_and_cast('f08jyc', d, 'd')
    _e = nag_double_type_check_and_cast('f08jyc', e, 'e')
    _m = nag_integer_type_check_and_cast('f08jyc', m, 'm')
    _w = nag_double_type_check_and_cast('f08jyc', w, 'w')
    _z = nag_complex_type_check_and_cast('f08jyc', z, 'z')
    _isuppz = nag_integer_type_check_and_cast('f08jyc', isuppz, 'isuppz')
    f08jyc_ctypes(order, job, _range, n, _d, _e, vl, vu, il, iu, _m, _w, _z, pdz, _isuppz, fail)

f08kac_ctypes = _library_cl.f08kac
f08kac_ctypes.restype = None
f08kac_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), c_double, POINTER(Integer), POINTER(NagError)]


def f08kac(order, m, n, nrhs, a, pda, b, pdb, s, rcond, rank, fail):
    """
    Computes the minimum-norm solution to a real linear least squares
    problem using singular value decomposition.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08kac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, s, rank
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, s
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        rank
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08kac', a, 'a')
    _b = nag_double_type_check_and_cast('f08kac', b, 'b')
    _s = nag_double_type_check_and_cast('f08kac', s, 's')
    _rank = nag_integer_type_check_and_cast('f08kac', rank, 'rank')
    f08kac_ctypes(order, m, n, nrhs, _a, pda, _b, pdb, _s, rcond, _rank, fail)

f08kbc_ctypes = _library_cl.f08kbc
f08kbc_ctypes.restype = None
f08kbc_ctypes.argtypes = [Nag_OrderType, Nag_ComputeUType, Nag_ComputeVTType, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f08kbc(order, jobu, jobvt, m, n, a, pda, s, u, pdu, vt, pdvt, work, fail):
    """
    Computes the singular value decomposition of a real matrix, optionally
    computing the left and/or right singular vectors.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08kbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, s, u, vt, work
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, s, u, vt, work
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08kbc', a, 'a')
    _s = nag_double_type_check_and_cast('f08kbc', s, 's')
    _u = nag_double_type_check_and_cast('f08kbc', u, 'u')
    _vt = nag_double_type_check_and_cast('f08kbc', vt, 'vt')
    _work = nag_double_type_check_and_cast('f08kbc', work, 'work')
    f08kbc_ctypes(order, jobu, jobvt, m, n, _a, pda, _s, _u, pdu, _vt, pdvt, _work, fail)

f08kcc_ctypes = _library_cl.f08kcc
f08kcc_ctypes.restype = None
f08kcc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), c_double, POINTER(Integer), POINTER(NagError)]


def f08kcc(order, m, n, nrhs, a, pda, b, pdb, s, rcond, rank, fail):
    """
    Computes the minimum-norm solution to a real linear least squares
    problem using singular value decomposition (divide-and-conquer).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08kcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, s, rank
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, s
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        rank
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08kcc', a, 'a')
    _b = nag_double_type_check_and_cast('f08kcc', b, 'b')
    _s = nag_double_type_check_and_cast('f08kcc', s, 's')
    _rank = nag_integer_type_check_and_cast('f08kcc', rank, 'rank')
    f08kcc_ctypes(order, m, n, nrhs, _a, pda, _b, pdb, _s, rcond, _rank, fail)

f08kdc_ctypes = _library_cl.f08kdc
f08kdc_ctypes.restype = None
f08kdc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f08kdc(order, job, m, n, a, pda, s, u, pdu, vt, pdvt, fail):
    """
    Computes the singular value decomposition of a real matrix, optionally
    computing the left and/or right singular vectors (divide-and-conquer).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08kdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, s, u, vt
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, s, u, vt
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08kdc', a, 'a')
    _s = nag_double_type_check_and_cast('f08kdc', s, 's')
    _u = nag_double_type_check_and_cast('f08kdc', u, 'u')
    _vt = nag_double_type_check_and_cast('f08kdc', vt, 'vt')
    f08kdc_ctypes(order, job, m, n, _a, pda, _s, _u, pdu, _vt, pdvt, fail)

f08kec_ctypes = _library_cl.f08kec
f08kec_ctypes.restype = None
f08kec_ctypes.argtypes = [Nag_OrderType, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f08kec(order, m, n, a, pda, d, e, tauq, taup, fail):
    """
    Performs an orthogonal reduction of real general rectangular matrix to
    bidiagonal form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08kec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, d, e, tauq, taup
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, d, e, tauq, taup
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08kec', a, 'a')
    _d = nag_double_type_check_and_cast('f08kec', d, 'd')
    _e = nag_double_type_check_and_cast('f08kec', e, 'e')
    _tauq = nag_double_type_check_and_cast('f08kec', tauq, 'tauq')
    _taup = nag_double_type_check_and_cast('f08kec', taup, 'taup')
    f08kec_ctypes(order, m, n, _a, pda, _d, _e, _tauq, _taup, fail)

f08kfc_ctypes = _library_cl.f08kfc
f08kfc_ctypes.restype = None
f08kfc_ctypes.argtypes = [Nag_OrderType, Nag_VectType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f08kfc(order, vect, m, n, k, a, pda, tau, fail):
    """
    Generates an orthogonal transformation matrices from reduction to
    bidiagonal form determined by f08kec.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08kfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, tau
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08kfc', a, 'a')
    _tau = nag_double_type_check_and_cast('f08kfc', tau, 'tau')
    f08kfc_ctypes(order, vect, m, n, k, _a, pda, _tau, fail)

f08kgc_ctypes = _library_cl.f08kgc
f08kgc_ctypes.restype = None
f08kgc_ctypes.argtypes = [Nag_OrderType, Nag_VectType, Nag_SideType, Nag_TransType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f08kgc(order, vect, side, trans, m, n, k, a, pda, tau, c, pdc, fail):
    """
    Applies the orthogonal transformations from reduction to bidiagonal form
    determined by f08kec.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08kgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau, c
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, tau, c
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08kgc', a, 'a')
    _tau = nag_double_type_check_and_cast('f08kgc', tau, 'tau')
    _c = nag_double_type_check_and_cast('f08kgc', c, 'c')
    f08kgc_ctypes(order, vect, side, trans, m, n, k, _a, pda, _tau, _c, pdc, fail)

f08khc_ctypes = _library_cl.f08khc
f08khc_ctypes.restype = None
f08khc_ctypes.argtypes = [Nag_OrderType, Nag_Preprocess, Nag_LeftVecsType, Nag_RightVecsType, Nag_ZeroCols, Nag_TransType, Nag_Perturb, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def f08khc(order, joba, jobu, jobv, jobr, jobt, jobp, m, n, a, pda, sva, u, pdu, v, pdv, work, iwork, fail):
    """
    Computes the singular value decomposition of a real matrix, optionally
    computing the left and/or right singular vectors (preconditioned
    Jacobi).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08khc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, sva, u, v, work, iwork
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, sva, u, v, work
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iwork
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08khc', a, 'a')
    _sva = nag_double_type_check_and_cast('f08khc', sva, 'sva')
    _u = nag_double_type_check_and_cast('f08khc', u, 'u')
    _v = nag_double_type_check_and_cast('f08khc', v, 'v')
    _work = nag_double_type_check_and_cast('f08khc', work, 'work')
    _iwork = nag_integer_type_check_and_cast('f08khc', iwork, 'iwork')
    f08khc_ctypes(order, joba, jobu, jobv, jobr, jobt, jobp, m, n, _a, pda, _sva, _u, pdu, _v, pdv, _work, _iwork, fail)

f08kjc_ctypes = _library_cl.f08kjc
f08kjc_ctypes.restype = None
f08kjc_ctypes.argtypes = [Nag_OrderType, Nag_MatrixType, Nag_LeftVecsType, Nag_RightVecsType, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f08kjc(order, joba, jobu, jobv, m, n, a, pda, sva, mv, v, pdv, work, lwork, fail):
    """
    Computes the singular value decomposition of a real matrix, optionally
    computing the left and/or right singular vectors (fast Jacobi).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08kjc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, sva, v, work
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, sva, v, work
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08kjc', a, 'a')
    _sva = nag_double_type_check_and_cast('f08kjc', sva, 'sva')
    _v = nag_double_type_check_and_cast('f08kjc', v, 'v')
    _work = nag_double_type_check_and_cast('f08kjc', work, 'work')
    f08kjc_ctypes(order, joba, jobu, jobv, m, n, _a, pda, _sva, mv, _v, pdv, _work, lwork, fail)

f08knc_ctypes = _library_cl.f08knc
f08knc_ctypes.restype = None
f08knc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(c_double), c_double, POINTER(Integer), POINTER(NagError)]


def f08knc(order, m, n, nrhs, a, pda, b, pdb, s, rcond, rank, fail):
    """
    Computes the minimum-norm solution to a complex linear least squares
    problem using singular value decomposition.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08knc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, s, rank
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        s
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        rank
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08knc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08knc', b, 'b')
    _s = nag_double_type_check_and_cast('f08knc', s, 's')
    _rank = nag_integer_type_check_and_cast('f08knc', rank, 'rank')
    f08knc_ctypes(order, m, n, nrhs, _a, pda, _b, pdb, _s, rcond, _rank, fail)

f08kpc_ctypes = _library_cl.f08kpc
f08kpc_ctypes.restype = None
f08kpc_ctypes.argtypes = [Nag_OrderType, Nag_ComputeUType, Nag_ComputeVTType, Integer, Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(NagError)]


def f08kpc(order, jobu, jobvt, m, n, a, pda, s, u, pdu, vt, pdvt, rwork, fail):
    """
    Computes the singular value decomposition of a complex matrix,
    optionally computing the left and/or right singular vectors.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08kpc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, s, u, vt, rwork
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        s, rwork
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, u, vt
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08kpc', a, 'a')
    _s = nag_double_type_check_and_cast('f08kpc', s, 's')
    _u = nag_complex_type_check_and_cast('f08kpc', u, 'u')
    _vt = nag_complex_type_check_and_cast('f08kpc', vt, 'vt')
    _rwork = nag_double_type_check_and_cast('f08kpc', rwork, 'rwork')
    f08kpc_ctypes(order, jobu, jobvt, m, n, _a, pda, _s, _u, pdu, _vt, pdvt, _rwork, fail)

f08kqc_ctypes = _library_cl.f08kqc
f08kqc_ctypes.restype = None
f08kqc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(c_double), c_double, POINTER(Integer), POINTER(NagError)]


def f08kqc(order, m, n, nrhs, a, pda, b, pdb, s, rcond, rank, fail):
    """
    Computes the minimum-norm solution to a complex linear least squares
    problem using singular value decomposition (divide-and-conquer).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08kqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, s, rank
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        s
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        rank
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08kqc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08kqc', b, 'b')
    _s = nag_double_type_check_and_cast('f08kqc', s, 's')
    _rank = nag_integer_type_check_and_cast('f08kqc', rank, 'rank')
    f08kqc_ctypes(order, m, n, nrhs, _a, pda, _b, pdb, _s, rcond, _rank, fail)

f08krc_ctypes = _library_cl.f08krc
f08krc_ctypes.restype = None
f08krc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Integer, Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f08krc(order, job, m, n, a, pda, s, u, pdu, vt, pdvt, fail):
    """
    Computes the singular value decomposition of a complex matrix,
    optionally computing the left and/or right singular vectors (divide-and-
    conquer).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08krc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, s, u, vt
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        s
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, u, vt
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08krc', a, 'a')
    _s = nag_double_type_check_and_cast('f08krc', s, 's')
    _u = nag_complex_type_check_and_cast('f08krc', u, 'u')
    _vt = nag_complex_type_check_and_cast('f08krc', vt, 'vt')
    f08krc_ctypes(order, job, m, n, _a, pda, _s, _u, pdu, _vt, pdvt, fail)

f08ksc_ctypes = _library_cl.f08ksc
f08ksc_ctypes.restype = None
f08ksc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(c_double), POINTER(Complex), POINTER(Complex), POINTER(NagError)]


def f08ksc(order, m, n, a, pda, d, e, tauq, taup, fail):
    """
    Performs a unitary reduction of complex general rectangular matrix to
    bidiagonal form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08ksc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, d, e, tauq, taup
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, e
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, tauq, taup
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08ksc', a, 'a')
    _d = nag_double_type_check_and_cast('f08ksc', d, 'd')
    _e = nag_double_type_check_and_cast('f08ksc', e, 'e')
    _tauq = nag_complex_type_check_and_cast('f08ksc', tauq, 'tauq')
    _taup = nag_complex_type_check_and_cast('f08ksc', taup, 'taup')
    f08ksc_ctypes(order, m, n, _a, pda, _d, _e, _tauq, _taup, fail)

f08ktc_ctypes = _library_cl.f08ktc
f08ktc_ctypes.restype = None
f08ktc_ctypes.argtypes = [Nag_OrderType, Nag_VectType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(NagError)]


def f08ktc(order, vect, m, n, k, a, pda, tau, fail):
    """
    Generates unitary transformation matrices from the reduction to
    bidiagonal form determined by f08ksc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08ktc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, tau
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08ktc', a, 'a')
    _tau = nag_complex_type_check_and_cast('f08ktc', tau, 'tau')
    f08ktc_ctypes(order, vect, m, n, k, _a, pda, _tau, fail)

f08kuc_ctypes = _library_cl.f08kuc
f08kuc_ctypes.restype = None
f08kuc_ctypes.argtypes = [Nag_OrderType, Nag_VectType, Nag_SideType, Nag_TransType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(Complex), Integer, POINTER(NagError)]


def f08kuc(order, vect, side, trans, m, n, k, a, pda, tau, c, pdc, fail):
    """
    Applies the unitary transformations from reduction to bidiagonal form
    determined by f08ksc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08kuc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau, c
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, tau, c
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08kuc', a, 'a')
    _tau = nag_complex_type_check_and_cast('f08kuc', tau, 'tau')
    _c = nag_complex_type_check_and_cast('f08kuc', c, 'c')
    f08kuc_ctypes(order, vect, side, trans, m, n, k, _a, pda, _tau, _c, pdc, fail)

f08lec_ctypes = _library_cl.f08lec
f08lec_ctypes.restype = None
f08lec_ctypes.argtypes = [Nag_OrderType, Nag_VectType, Integer, Integer, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f08lec(order, vect, m, n, ncc, kl, ku, ab, pdab, d, e, q, pdq, pt, pdpt, c, pdc, fail):
    """
    Performs a reduction of real rectangular band matrix to upper bidiagonal
    form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08lec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, d, e, q, pt, c
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ab, d, e, q, pt, c
    """
    from .util import nag_double_type_check_and_cast
    _ab = nag_double_type_check_and_cast('f08lec', ab, 'ab')
    _d = nag_double_type_check_and_cast('f08lec', d, 'd')
    _e = nag_double_type_check_and_cast('f08lec', e, 'e')
    _q = nag_double_type_check_and_cast('f08lec', q, 'q')
    _pt = nag_double_type_check_and_cast('f08lec', pt, 'pt')
    _c = nag_double_type_check_and_cast('f08lec', c, 'c')
    f08lec_ctypes(order, vect, m, n, ncc, kl, ku, _ab, pdab, _d, _e, _q, pdq, _pt, pdpt, _c, pdc, fail)

f08lsc_ctypes = _library_cl.f08lsc
f08lsc_ctypes.restype = None
f08lsc_ctypes.argtypes = [Nag_OrderType, Nag_VectType, Integer, Integer, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(c_double), POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f08lsc(order, vect, m, n, ncc, kl, ku, ab, pdab, d, e, q, pdq, pt, pdpt, c, pdc, fail):
    """
    Reduction of complex rectangular band matrix to upper bidiagonal form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08lsc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, d, e, q, pt, c
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, e
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ab, q, pt, c
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _ab = nag_complex_type_check_and_cast('f08lsc', ab, 'ab')
    _d = nag_double_type_check_and_cast('f08lsc', d, 'd')
    _e = nag_double_type_check_and_cast('f08lsc', e, 'e')
    _q = nag_complex_type_check_and_cast('f08lsc', q, 'q')
    _pt = nag_complex_type_check_and_cast('f08lsc', pt, 'pt')
    _c = nag_complex_type_check_and_cast('f08lsc', c, 'c')
    f08lsc_ctypes(order, vect, m, n, ncc, kl, ku, _ab, pdab, _d, _e, _q, pdq, _pt, pdpt, _c, pdc, fail)

f08mdc_ctypes = _library_cl.f08mdc
f08mdc_ctypes.restype = None
f08mdc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Nag_ComputeSingularVecsType, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def f08mdc(order, uplo, compq, n, d, e, u, pdu, vt, pdvt, q, iq, fail):
    """
    Computes the singular value decomposition of a real bidiagonal matrix,
    optionally computing the singular vectors (divide-and-conquer).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08mdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        d, e, u, vt, q, iq
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, e, u, vt, q
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iq
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _d = nag_double_type_check_and_cast('f08mdc', d, 'd')
    _e = nag_double_type_check_and_cast('f08mdc', e, 'e')
    _u = nag_double_type_check_and_cast('f08mdc', u, 'u')
    _vt = nag_double_type_check_and_cast('f08mdc', vt, 'vt')
    _q = nag_double_type_check_and_cast('f08mdc', q, 'q')
    _iq = nag_integer_type_check_and_cast('f08mdc', iq, 'iq')
    f08mdc_ctypes(order, uplo, compq, n, _d, _e, _u, pdu, _vt, pdvt, _q, _iq, fail)

f08mec_ctypes = _library_cl.f08mec
f08mec_ctypes.restype = None
f08mec_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, Integer, Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f08mec(order, uplo, n, ncvt, nru, ncc, d, e, vt, pdvt, u, pdu, c, pdc, fail):
    """
    Performs an SVD of real bidiagonal matrix reduced from real general
    matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08mec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        d, e, vt, u, c
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, e, vt, u, c
    """
    from .util import nag_double_type_check_and_cast
    _d = nag_double_type_check_and_cast('f08mec', d, 'd')
    _e = nag_double_type_check_and_cast('f08mec', e, 'e')
    _vt = nag_double_type_check_and_cast('f08mec', vt, 'vt')
    _u = nag_double_type_check_and_cast('f08mec', u, 'u')
    _c = nag_double_type_check_and_cast('f08mec', c, 'c')
    f08mec_ctypes(order, uplo, n, ncvt, nru, ncc, _d, _e, _vt, pdvt, _u, pdu, _c, pdc, fail)

f08msc_ctypes = _library_cl.f08msc
f08msc_ctypes.restype = None
f08msc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, Integer, Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f08msc(order, uplo, n, ncvt, nru, ncc, d, e, vt, pdvt, u, pdu, c, pdc, fail):
    """
    Performs an SVD of real bidiagonal matrix reduced from complex general
    matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08msc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        d, e, vt, u, c
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, e
    TypeError will be raised when the following arguments are not of data type
    Complex:
        vt, u, c
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _d = nag_double_type_check_and_cast('f08msc', d, 'd')
    _e = nag_double_type_check_and_cast('f08msc', e, 'e')
    _vt = nag_complex_type_check_and_cast('f08msc', vt, 'vt')
    _u = nag_complex_type_check_and_cast('f08msc', u, 'u')
    _c = nag_complex_type_check_and_cast('f08msc', c, 'c')
    f08msc_ctypes(order, uplo, n, ncvt, nru, ncc, _d, _e, _vt, pdvt, _u, pdu, _c, pdc, fail)

f08nac_ctypes = _library_cl.f08nac
f08nac_ctypes.restype = None
f08nac_ctypes.argtypes = [Nag_OrderType, Nag_LeftVecsType, Nag_RightVecsType, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f08nac(order, jobvl, jobvr, n, a, pda, wr, wi, vl, pdvl, vr, pdvr, fail):
    """
    Computes all eigenvalues and, optionally, left and/or right eigenvectors
    of a real nonsymmetric matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08nac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, wr, wi, vl, vr
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, wr, wi, vl, vr
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08nac', a, 'a')
    _wr = nag_double_type_check_and_cast('f08nac', wr, 'wr')
    _wi = nag_double_type_check_and_cast('f08nac', wi, 'wi')
    _vl = nag_double_type_check_and_cast('f08nac', vl, 'vl')
    _vr = nag_double_type_check_and_cast('f08nac', vr, 'vr')
    f08nac_ctypes(order, jobvl, jobvr, n, _a, pda, _wr, _wi, _vl, pdvl, _vr, pdvr, fail)

f08nbc_ctypes = _library_cl.f08nbc
f08nbc_ctypes.restype = None
f08nbc_ctypes.argtypes = [Nag_OrderType, Nag_BalanceType, Nag_LeftVecsType, Nag_RightVecsType, Nag_RCondType, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f08nbc(order, balanc, jobvl, jobvr, sense, n, a, pda, wr, wi, vl, pdvl, vr, pdvr, ilo, ihi, scale, abnrm, rconde, rcondv, fail):
    """
    Computes all eigenvalues and, optionally, left and/or right eigenvectors
    of a real nonsymmetric matrix; also, optionally, the balancing
    transformation, the reciprocal condition numbers for the eigenvalues and
    for the right eigenvectors.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08nbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, wr, wi, vl, vr, ilo, ihi, scale, abnrm, rconde, rcondv
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, wr, wi, vl, vr, scale, abnrm, rconde, rcondv
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ilo, ihi
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08nbc', a, 'a')
    _wr = nag_double_type_check_and_cast('f08nbc', wr, 'wr')
    _wi = nag_double_type_check_and_cast('f08nbc', wi, 'wi')
    _vl = nag_double_type_check_and_cast('f08nbc', vl, 'vl')
    _vr = nag_double_type_check_and_cast('f08nbc', vr, 'vr')
    _ilo = nag_integer_type_check_and_cast('f08nbc', ilo, 'ilo')
    _ihi = nag_integer_type_check_and_cast('f08nbc', ihi, 'ihi')
    _scale = nag_double_type_check_and_cast('f08nbc', scale, 'scale')
    _abnrm = nag_double_type_check_and_cast('f08nbc', abnrm, 'abnrm')
    _rconde = nag_double_type_check_and_cast('f08nbc', rconde, 'rconde')
    _rcondv = nag_double_type_check_and_cast('f08nbc', rcondv, 'rcondv')
    f08nbc_ctypes(order, balanc, jobvl, jobvr, sense, n, _a, pda, _wr, _wi, _vl, pdvl, _vr, pdvr, _ilo, _ihi, _scale, _abnrm, _rconde, _rcondv, fail)

f08nec_ctypes = _library_cl.f08nec
f08nec_ctypes.restype = None
f08nec_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f08nec(order, n, ilo, ihi, a, pda, tau, fail):
    """
    Performs an orthogonal reduction of real general matrix to upper
    Hessenberg form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08nec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, tau
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08nec', a, 'a')
    _tau = nag_double_type_check_and_cast('f08nec', tau, 'tau')
    f08nec_ctypes(order, n, ilo, ihi, _a, pda, _tau, fail)

f08nfc_ctypes = _library_cl.f08nfc
f08nfc_ctypes.restype = None
f08nfc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f08nfc(order, n, ilo, ihi, a, pda, tau, fail):
    """
    Generates an orthogonal transformation matrix from reduction to
    Hessenberg form determined by f08nec.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08nfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, tau
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08nfc', a, 'a')
    _tau = nag_double_type_check_and_cast('f08nfc', tau, 'tau')
    f08nfc_ctypes(order, n, ilo, ihi, _a, pda, _tau, fail)

f08ngc_ctypes = _library_cl.f08ngc
f08ngc_ctypes.restype = None
f08ngc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_TransType, Integer, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f08ngc(order, side, trans, m, n, ilo, ihi, a, pda, tau, c, pdc, fail):
    """
    Applies the orthogonal transformation matrix from reduction to
    Hessenberg form determined by f08nec.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08ngc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau, c
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, tau, c
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08ngc', a, 'a')
    _tau = nag_double_type_check_and_cast('f08ngc', tau, 'tau')
    _c = nag_double_type_check_and_cast('f08ngc', c, 'c')
    f08ngc_ctypes(order, side, trans, m, n, ilo, ihi, _a, pda, _tau, _c, pdc, fail)

f08nhc_ctypes = _library_cl.f08nhc
f08nhc_ctypes.restype = None
f08nhc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def f08nhc(order, job, n, a, pda, ilo, ihi, scale, fail):
    """
    Balances a real general matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08nhc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, ilo, ihi, scale
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, scale
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ilo, ihi
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08nhc', a, 'a')
    _ilo = nag_integer_type_check_and_cast('f08nhc', ilo, 'ilo')
    _ihi = nag_integer_type_check_and_cast('f08nhc', ihi, 'ihi')
    _scale = nag_double_type_check_and_cast('f08nhc', scale, 'scale')
    f08nhc_ctypes(order, job, n, _a, pda, _ilo, _ihi, _scale, fail)

f08njc_ctypes = _library_cl.f08njc
f08njc_ctypes.restype = None
f08njc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_SideType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f08njc(order, job, side, n, ilo, ihi, scale, m, v, pdv, fail):
    """
    Transforms eigenvectors of real balanced matrix to those of original
    matrix supplied to f08nhc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08njc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        scale, v
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        scale, v
    """
    from .util import nag_double_type_check_and_cast
    _scale = nag_double_type_check_and_cast('f08njc', scale, 'scale')
    _v = nag_double_type_check_and_cast('f08njc', v, 'v')
    f08njc_ctypes(order, job, side, n, ilo, ihi, _scale, m, _v, pdv, fail)

f08nnc_ctypes = _library_cl.f08nnc
f08nnc_ctypes.restype = None
f08nnc_ctypes.argtypes = [Nag_OrderType, Nag_LeftVecsType, Nag_RightVecsType, Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f08nnc(order, jobvl, jobvr, n, a, pda, w, vl, pdvl, vr, pdvr, fail):
    """
    Computes all eigenvalues and, optionally, left and/or right eigenvectors
    of a complex nonsymmetric matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08nnc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, w, vl, vr
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, w, vl, vr
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08nnc', a, 'a')
    _w = nag_complex_type_check_and_cast('f08nnc', w, 'w')
    _vl = nag_complex_type_check_and_cast('f08nnc', vl, 'vl')
    _vr = nag_complex_type_check_and_cast('f08nnc', vr, 'vr')
    f08nnc_ctypes(order, jobvl, jobvr, n, _a, pda, _w, _vl, pdvl, _vr, pdvr, fail)

f08npc_ctypes = _library_cl.f08npc
f08npc_ctypes.restype = None
f08npc_ctypes.argtypes = [Nag_OrderType, Nag_BalanceType, Nag_LeftVecsType, Nag_RightVecsType, Nag_RCondType, Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f08npc(order, balanc, jobvl, jobvr, sense, n, a, pda, w, vl, pdvl, vr, pdvr, ilo, ihi, scale, abnrm, rconde, rcondv, fail):
    """
    Computes all eigenvalues and, optionally, left and/or right eigenvectors
    of a complex nonsymmetric matrix; also, optionally, the balancing
    transformation, the reciprocal condition numbers for the eigenvalues and
    for the right eigenvectors.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08npc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, w, vl, vr, ilo, ihi, scale, abnrm, rconde, rcondv
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        scale, abnrm, rconde, rcondv
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ilo, ihi
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, w, vl, vr
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08npc', a, 'a')
    _w = nag_complex_type_check_and_cast('f08npc', w, 'w')
    _vl = nag_complex_type_check_and_cast('f08npc', vl, 'vl')
    _vr = nag_complex_type_check_and_cast('f08npc', vr, 'vr')
    _ilo = nag_integer_type_check_and_cast('f08npc', ilo, 'ilo')
    _ihi = nag_integer_type_check_and_cast('f08npc', ihi, 'ihi')
    _scale = nag_double_type_check_and_cast('f08npc', scale, 'scale')
    _abnrm = nag_double_type_check_and_cast('f08npc', abnrm, 'abnrm')
    _rconde = nag_double_type_check_and_cast('f08npc', rconde, 'rconde')
    _rcondv = nag_double_type_check_and_cast('f08npc', rcondv, 'rcondv')
    f08npc_ctypes(order, balanc, jobvl, jobvr, sense, n, _a, pda, _w, _vl, pdvl, _vr, pdvr, _ilo, _ihi, _scale, _abnrm, _rconde, _rcondv, fail)

f08nsc_ctypes = _library_cl.f08nsc
f08nsc_ctypes.restype = None
f08nsc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(NagError)]


def f08nsc(order, n, ilo, ihi, a, pda, tau, fail):
    """
    Performs a unitary reduction of complex general matrix to upper
    Hessenberg form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08nsc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, tau
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08nsc', a, 'a')
    _tau = nag_complex_type_check_and_cast('f08nsc', tau, 'tau')
    f08nsc_ctypes(order, n, ilo, ihi, _a, pda, _tau, fail)

f08ntc_ctypes = _library_cl.f08ntc
f08ntc_ctypes.restype = None
f08ntc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(NagError)]


def f08ntc(order, n, ilo, ihi, a, pda, tau, fail):
    """
    Generates a unitary transformation matrix from reduction to Hessenberg
    form determined by f08nsc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08ntc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, tau
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08ntc', a, 'a')
    _tau = nag_complex_type_check_and_cast('f08ntc', tau, 'tau')
    f08ntc_ctypes(order, n, ilo, ihi, _a, pda, _tau, fail)

f08nuc_ctypes = _library_cl.f08nuc
f08nuc_ctypes.restype = None
f08nuc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_TransType, Integer, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(Complex), Integer, POINTER(NagError)]


def f08nuc(order, side, trans, m, n, ilo, ihi, a, pda, tau, c, pdc, fail):
    """
    Applies the unitary transformation matrix from reduction to Hessenberg
    form determined by f08nsc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08nuc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, tau, c
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, tau, c
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08nuc', a, 'a')
    _tau = nag_complex_type_check_and_cast('f08nuc', tau, 'tau')
    _c = nag_complex_type_check_and_cast('f08nuc', c, 'c')
    f08nuc_ctypes(order, side, trans, m, n, ilo, ihi, _a, pda, _tau, _c, pdc, fail)

f08nvc_ctypes = _library_cl.f08nvc
f08nvc_ctypes.restype = None
f08nvc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Integer, POINTER(Complex), Integer, POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def f08nvc(order, job, n, a, pda, ilo, ihi, scale, fail):
    """
    Balances a complex general matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08nvc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, ilo, ihi, scale
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        scale
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ilo, ihi
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08nvc', a, 'a')
    _ilo = nag_integer_type_check_and_cast('f08nvc', ilo, 'ilo')
    _ihi = nag_integer_type_check_and_cast('f08nvc', ihi, 'ihi')
    _scale = nag_double_type_check_and_cast('f08nvc', scale, 'scale')
    f08nvc_ctypes(order, job, n, _a, pda, _ilo, _ihi, _scale, fail)

f08nwc_ctypes = _library_cl.f08nwc
f08nwc_ctypes.restype = None
f08nwc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_SideType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f08nwc(order, job, side, n, ilo, ihi, scale, m, v, pdv, fail):
    """
    Transforms eigenvectors of complex balanced matrix to those of original
    matrix supplied to f08nvc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08nwc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        scale, v
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        scale
    TypeError will be raised when the following arguments are not of data type
    Complex:
        v
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _scale = nag_double_type_check_and_cast('f08nwc', scale, 'scale')
    _v = nag_complex_type_check_and_cast('f08nwc', v, 'v')
    f08nwc_ctypes(order, job, side, n, ilo, ihi, _scale, m, _v, pdv, fail)

f08pac_ctypes = _library_cl.f08pac
f08pac_ctypes.restype = None
f08pac_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_SortEigValsType, NAG_F08PAC_SELECT, Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f08pac(order, jobvs, sort, select, n, a, pda, sdim, wr, wi, vs, pdvs, fail):
    """
    Computes for real square nonsymmetric matrix, the eigenvalues, the real
    Schur form, and, optionally, the matrix of Schur vectors.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08pac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, sdim, wr, wi, vs
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, wr, wi, vs
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        sdim
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08pac', a, 'a')
    _sdim = nag_integer_type_check_and_cast('f08pac', sdim, 'sdim')
    _wr = nag_double_type_check_and_cast('f08pac', wr, 'wr')
    _wi = nag_double_type_check_and_cast('f08pac', wi, 'wi')
    _vs = nag_double_type_check_and_cast('f08pac', vs, 'vs')
    f08pac_ctypes(order, jobvs, sort, select, n, _a, pda, _sdim, _wr, _wi, _vs, pdvs, fail)

f08pbc_ctypes = _library_cl.f08pbc
f08pbc_ctypes.restype = None
f08pbc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_SortEigValsType, NAG_F08PBC_SELECT, Nag_RCondType, Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f08pbc(order, jobvs, sort, select, sense, n, a, pda, sdim, wr, wi, vs, pdvs, rconde, rcondv, fail):
    """
    Computes for real square nonsymmetric matrix, the eigenvalues, the real
    Schur form, and, optionally, the matrix of Schur vectors; also,
    optionally, computes reciprocal condition numbers for selected
    eigenvalues.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08pbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, sdim, wr, wi, vs, rconde, rcondv
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, wr, wi, vs, rconde, rcondv
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        sdim
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08pbc', a, 'a')
    _sdim = nag_integer_type_check_and_cast('f08pbc', sdim, 'sdim')
    _wr = nag_double_type_check_and_cast('f08pbc', wr, 'wr')
    _wi = nag_double_type_check_and_cast('f08pbc', wi, 'wi')
    _vs = nag_double_type_check_and_cast('f08pbc', vs, 'vs')
    _rconde = nag_double_type_check_and_cast('f08pbc', rconde, 'rconde')
    _rcondv = nag_double_type_check_and_cast('f08pbc', rcondv, 'rcondv')
    f08pbc_ctypes(order, jobvs, sort, select, sense, n, _a, pda, _sdim, _wr, _wi, _vs, pdvs, _rconde, _rcondv, fail)

f08pec_ctypes = _library_cl.f08pec
f08pec_ctypes.restype = None
f08pec_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_ComputeZType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f08pec(order, job, compz, n, ilo, ihi, h, pdh, wr, wi, z, pdz, fail):
    """
    Computes the eigenvalues and Schur factorization of real upper
    Hessenberg matrix reduced from real general matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08pec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        h, wr, wi, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        h, wr, wi, z
    """
    from .util import nag_double_type_check_and_cast
    _h = nag_double_type_check_and_cast('f08pec', h, 'h')
    _wr = nag_double_type_check_and_cast('f08pec', wr, 'wr')
    _wi = nag_double_type_check_and_cast('f08pec', wi, 'wi')
    _z = nag_double_type_check_and_cast('f08pec', z, 'z')
    f08pec_ctypes(order, job, compz, n, ilo, ihi, _h, pdh, _wr, _wi, _z, pdz, fail)

f08pkc_ctypes = _library_cl.f08pkc
f08pkc_ctypes.restype = None
f08pkc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_EigValsSourceType, Nag_InitVeenumtype, POINTER(Nag_Boolean), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, Integer, POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(NagError)]


def f08pkc(order, side, eigsource, initv, select, n, h, pdh, wr, wi, vl, pdvl, vr, pdvr, mm, m, ifaill, ifailr, fail):
    """
    Computes selected right and/or left eigenvectors of real upper
    Hessenberg matrix by inverse iteration.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08pkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        select, h, wr, wi, vl, vr, m, ifaill, ifailr
    TypeError will be raised when the following arguments are not of data type
    Nag_Boolean:
        select
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        h, wr, wi, vl, vr
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m, ifaill, ifailr
    """
    from .util import nag_enum_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _select = nag_enum_type_check_and_cast('f08pkc', select, 'select', Nag_Boolean)
    _h = nag_double_type_check_and_cast('f08pkc', h, 'h')
    _wr = nag_double_type_check_and_cast('f08pkc', wr, 'wr')
    _wi = nag_double_type_check_and_cast('f08pkc', wi, 'wi')
    _vl = nag_double_type_check_and_cast('f08pkc', vl, 'vl')
    _vr = nag_double_type_check_and_cast('f08pkc', vr, 'vr')
    _m = nag_integer_type_check_and_cast('f08pkc', m, 'm')
    _ifaill = nag_integer_type_check_and_cast('f08pkc', ifaill, 'ifaill')
    _ifailr = nag_integer_type_check_and_cast('f08pkc', ifailr, 'ifailr')
    f08pkc_ctypes(order, side, eigsource, initv, _select, n, _h, pdh, _wr, _wi, _vl, pdvl, _vr, pdvr, mm, _m, _ifaill, _ifailr, fail)

f08pnc_ctypes = _library_cl.f08pnc
f08pnc_ctypes.restype = None
f08pnc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_SortEigValsType, NAG_F08PNC_SELECT, Integer, POINTER(Complex), Integer, POINTER(Integer), POINTER(Complex), POINTER(Complex), Integer, POINTER(NagError)]


def f08pnc(order, jobvs, sort, select, n, a, pda, sdim, w, vs, pdvs, fail):
    """
    Computes for complex square nonsymmetric matrix, the eigenvalues, the
    Schur form, and, optionally, the matrix of Schur vectors.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08pnc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, sdim, w, vs
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        sdim
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, w, vs
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08pnc', a, 'a')
    _sdim = nag_integer_type_check_and_cast('f08pnc', sdim, 'sdim')
    _w = nag_complex_type_check_and_cast('f08pnc', w, 'w')
    _vs = nag_complex_type_check_and_cast('f08pnc', vs, 'vs')
    f08pnc_ctypes(order, jobvs, sort, select, n, _a, pda, _sdim, _w, _vs, pdvs, fail)

f08ppc_ctypes = _library_cl.f08ppc
f08ppc_ctypes.restype = None
f08ppc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_SortEigValsType, NAG_F08PPC_SELECT, Nag_RCondType, Integer, POINTER(Complex), Integer, POINTER(Integer), POINTER(Complex), POINTER(Complex), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f08ppc(order, jobvs, sort, select, sense, n, a, pda, sdim, w, vs, pdvs, rconde, rcondv, fail):
    """
    Computes for real square nonsymmetric matrix, the eigenvalues, the Schur
    form, and, optionally, the matrix of Schur vectors; also computes a
    reciprocal condition number for the average of the selected eigenvalues
    and for the right invariant subspace corresponding to these eigenvalues.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08ppc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, sdim, w, vs, rconde, rcondv
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rconde, rcondv
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        sdim
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, w, vs
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08ppc', a, 'a')
    _sdim = nag_integer_type_check_and_cast('f08ppc', sdim, 'sdim')
    _w = nag_complex_type_check_and_cast('f08ppc', w, 'w')
    _vs = nag_complex_type_check_and_cast('f08ppc', vs, 'vs')
    _rconde = nag_double_type_check_and_cast('f08ppc', rconde, 'rconde')
    _rcondv = nag_double_type_check_and_cast('f08ppc', rcondv, 'rcondv')
    f08ppc_ctypes(order, jobvs, sort, select, sense, n, _a, pda, _sdim, _w, _vs, pdvs, _rconde, _rcondv, fail)

f08psc_ctypes = _library_cl.f08psc
f08psc_ctypes.restype = None
f08psc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_ComputeZType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(Complex), Integer, POINTER(NagError)]


def f08psc(order, job, compz, n, ilo, ihi, h, pdh, w, z, pdz, fail):
    """
    Computes the eigenvalues and Schur factorization of complex upper
    Hessenberg matrix reduced from complex general matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08psc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        h, w, z
    TypeError will be raised when the following arguments are not of data type
    Complex:
        h, w, z
    """
    from .util import nag_complex_type_check_and_cast
    _h = nag_complex_type_check_and_cast('f08psc', h, 'h')
    _w = nag_complex_type_check_and_cast('f08psc', w, 'w')
    _z = nag_complex_type_check_and_cast('f08psc', z, 'z')
    f08psc_ctypes(order, job, compz, n, ilo, ihi, _h, pdh, _w, _z, pdz, fail)

f08pxc_ctypes = _library_cl.f08pxc
f08pxc_ctypes.restype = None
f08pxc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_EigValsSourceType, Nag_InitVeenumtype, POINTER(Nag_Boolean), Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(Complex), Integer, POINTER(Complex), Integer, Integer, POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(NagError)]


def f08pxc(order, side, eigsource, initv, select, n, h, pdh, w, vl, pdvl, vr, pdvr, mm, m, ifaill, ifailr, fail):
    """
    Computes selected right and/or left eigenvectors of complex upper
    Hessenberg matrix by inverse iteration.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08pxc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        select, h, w, vl, vr, m, ifaill, ifailr
    TypeError will be raised when the following arguments are not of data type
    Nag_Boolean:
        select
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m, ifaill, ifailr
    TypeError will be raised when the following arguments are not of data type
    Complex:
        h, w, vl, vr
    """
    from .util import nag_enum_type_check_and_cast
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _select = nag_enum_type_check_and_cast('f08pxc', select, 'select', Nag_Boolean)
    _h = nag_complex_type_check_and_cast('f08pxc', h, 'h')
    _w = nag_complex_type_check_and_cast('f08pxc', w, 'w')
    _vl = nag_complex_type_check_and_cast('f08pxc', vl, 'vl')
    _vr = nag_complex_type_check_and_cast('f08pxc', vr, 'vr')
    _m = nag_integer_type_check_and_cast('f08pxc', m, 'm')
    _ifaill = nag_integer_type_check_and_cast('f08pxc', ifaill, 'ifaill')
    _ifailr = nag_integer_type_check_and_cast('f08pxc', ifailr, 'ifailr')
    f08pxc_ctypes(order, side, eigsource, initv, _select, n, _h, pdh, _w, _vl, pdvl, _vr, pdvr, mm, _m, _ifaill, _ifailr, fail)

f08qfc_ctypes = _library_cl.f08qfc
f08qfc_ctypes.restype = None
f08qfc_ctypes.argtypes = [Nag_OrderType, Nag_ComputeQType, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(Integer), POINTER(NagError)]


def f08qfc(order, compq, n, t, pdt, q, pdq, ifst, ilst, fail):
    """
    Reorders a Schur factorization of real matrix using orthogonal
    similarity transformation.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08qfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        t, q, ifst, ilst
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        t, q
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ifst, ilst
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _t = nag_double_type_check_and_cast('f08qfc', t, 't')
    _q = nag_double_type_check_and_cast('f08qfc', q, 'q')
    _ifst = nag_integer_type_check_and_cast('f08qfc', ifst, 'ifst')
    _ilst = nag_integer_type_check_and_cast('f08qfc', ilst, 'ilst')
    f08qfc_ctypes(order, compq, n, _t, pdt, _q, pdq, _ifst, _ilst, fail)

f08qgc_ctypes = _library_cl.f08qgc
f08qgc_ctypes.restype = None
f08qgc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_ComputeQType, POINTER(Nag_Boolean), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f08qgc(order, job, compq, select, n, t, pdt, q, pdq, wr, wi, m, s, sep, fail):
    """
    Reorders a Schur factorization of real matrix, form orthonormal basis of
    right invariant subspace for selected eigenvalues, with estimates of
    sensitivities.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08qgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        select, t, q, wr, wi, m, s, sep
    TypeError will be raised when the following arguments are not of data type
    Nag_Boolean:
        select
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        t, q, wr, wi, s, sep
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m
    """
    from .util import nag_enum_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _select = nag_enum_type_check_and_cast('f08qgc', select, 'select', Nag_Boolean)
    _t = nag_double_type_check_and_cast('f08qgc', t, 't')
    _q = nag_double_type_check_and_cast('f08qgc', q, 'q')
    _wr = nag_double_type_check_and_cast('f08qgc', wr, 'wr')
    _wi = nag_double_type_check_and_cast('f08qgc', wi, 'wi')
    _m = nag_integer_type_check_and_cast('f08qgc', m, 'm')
    _s = nag_double_type_check_and_cast('f08qgc', s, 's')
    _sep = nag_double_type_check_and_cast('f08qgc', sep, 'sep')
    f08qgc_ctypes(order, job, compq, _select, n, _t, pdt, _q, pdq, _wr, _wi, _m, _s, _sep, fail)

f08qhc_ctypes = _library_cl.f08qhc
f08qhc_ctypes.restype = None
f08qhc_ctypes.argtypes = [Nag_OrderType, Nag_TransType, Nag_TransType, Nag_SignType, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f08qhc(order, trana, tranb, sign, m, n, a, pda, b, pdb, c, pdc, scale, fail):
    """
    Solves the real Sylvester matrix equation AX+XB = C, A and B are upper
    quasi-triangular or transposes.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08qhc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, c, scale
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, c, scale
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08qhc', a, 'a')
    _b = nag_double_type_check_and_cast('f08qhc', b, 'b')
    _c = nag_double_type_check_and_cast('f08qhc', c, 'c')
    _scale = nag_double_type_check_and_cast('f08qhc', scale, 'scale')
    f08qhc_ctypes(order, trana, tranb, sign, m, n, _a, pda, _b, pdb, _c, pdc, _scale, fail)

f08qkc_ctypes = _library_cl.f08qkc
f08qkc_ctypes.restype = None
f08qkc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_HowManyType, POINTER(Nag_Boolean), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, Integer, POINTER(Integer), POINTER(NagError)]


def f08qkc(order, side, howmany, select, n, t, pdt, vl, pdvl, vr, pdvr, mm, m, fail):
    """
    Computes left and right eigenvectors of real upper quasi-triangular
    matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08qkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        select, t, vl, vr, m
    TypeError will be raised when the following arguments are not of data type
    Nag_Boolean:
        select
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        t, vl, vr
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m
    """
    from .util import nag_enum_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _select = nag_enum_type_check_and_cast('f08qkc', select, 'select', Nag_Boolean)
    _t = nag_double_type_check_and_cast('f08qkc', t, 't')
    _vl = nag_double_type_check_and_cast('f08qkc', vl, 'vl')
    _vr = nag_double_type_check_and_cast('f08qkc', vr, 'vr')
    _m = nag_integer_type_check_and_cast('f08qkc', m, 'm')
    f08qkc_ctypes(order, side, howmany, _select, n, _t, pdt, _vl, pdvl, _vr, pdvr, mm, _m, fail)

f08qlc_ctypes = _library_cl.f08qlc
f08qlc_ctypes.restype = None
f08qlc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_HowManyType, POINTER(Nag_Boolean), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(Integer), POINTER(NagError)]


def f08qlc(order, job, howmany, select, n, t, pdt, vl, pdvl, vr, pdvr, s, sep, mm, m, fail):
    """
    Computes estimates of sensitivities of selected eigenvalues and
    eigenvectors of real upper quasi-triangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08qlc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        select, t, vl, vr, s, sep, m
    TypeError will be raised when the following arguments are not of data type
    Nag_Boolean:
        select
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        t, vl, vr, s, sep
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m
    """
    from .util import nag_enum_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _select = nag_enum_type_check_and_cast('f08qlc', select, 'select', Nag_Boolean)
    _t = nag_double_type_check_and_cast('f08qlc', t, 't')
    _vl = nag_double_type_check_and_cast('f08qlc', vl, 'vl')
    _vr = nag_double_type_check_and_cast('f08qlc', vr, 'vr')
    _s = nag_double_type_check_and_cast('f08qlc', s, 's')
    _sep = nag_double_type_check_and_cast('f08qlc', sep, 'sep')
    _m = nag_integer_type_check_and_cast('f08qlc', m, 'm')
    f08qlc_ctypes(order, job, howmany, _select, n, _t, pdt, _vl, pdvl, _vr, pdvr, _s, _sep, mm, _m, fail)

f08qtc_ctypes = _library_cl.f08qtc
f08qtc_ctypes.restype = None
f08qtc_ctypes.argtypes = [Nag_OrderType, Nag_ComputeQType, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, Integer, Integer, POINTER(NagError)]


def f08qtc(order, compq, n, t, pdt, q, pdq, ifst, ilst, fail):
    """
    Reorders a Schur factorization of complex matrix using unitary
    similarity transformation.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08qtc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        t, q
    TypeError will be raised when the following arguments are not of data type
    Complex:
        t, q
    """
    from .util import nag_complex_type_check_and_cast
    _t = nag_complex_type_check_and_cast('f08qtc', t, 't')
    _q = nag_complex_type_check_and_cast('f08qtc', q, 'q')
    f08qtc_ctypes(order, compq, n, _t, pdt, _q, pdq, ifst, ilst, fail)

f08quc_ctypes = _library_cl.f08quc
f08quc_ctypes.restype = None
f08quc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_ComputeQType, POINTER(Nag_Boolean), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f08quc(order, job, compq, select, n, t, pdt, q, pdq, w, m, s, sep, fail):
    """
    Reorders a Schur factorization of complex matrix, form orthonormal basis
    of right invariant subspace for selected eigenvalues, with estimates of
    sensitivities.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08quc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        select, t, q, w, m, s, sep
    TypeError will be raised when the following arguments are not of data type
    Nag_Boolean:
        select
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        s, sep
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m
    TypeError will be raised when the following arguments are not of data type
    Complex:
        t, q, w
    """
    from .util import nag_enum_type_check_and_cast
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _select = nag_enum_type_check_and_cast('f08quc', select, 'select', Nag_Boolean)
    _t = nag_complex_type_check_and_cast('f08quc', t, 't')
    _q = nag_complex_type_check_and_cast('f08quc', q, 'q')
    _w = nag_complex_type_check_and_cast('f08quc', w, 'w')
    _m = nag_integer_type_check_and_cast('f08quc', m, 'm')
    _s = nag_double_type_check_and_cast('f08quc', s, 's')
    _sep = nag_double_type_check_and_cast('f08quc', sep, 'sep')
    f08quc_ctypes(order, job, compq, _select, n, _t, pdt, _q, pdq, _w, _m, _s, _sep, fail)

f08qvc_ctypes = _library_cl.f08qvc
f08qvc_ctypes.restype = None
f08qvc_ctypes.argtypes = [Nag_OrderType, Nag_TransType, Nag_TransType, Nag_SignType, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(NagError)]


def f08qvc(order, trana, tranb, sign, m, n, a, pda, b, pdb, c, pdc, scal, fail):
    """
    Solves the complex Sylvester matrix equation AX+XB = C, A and B are
    upper triangular or conjugate-transposes.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08qvc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, c, scal
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        scal
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, c
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08qvc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08qvc', b, 'b')
    _c = nag_complex_type_check_and_cast('f08qvc', c, 'c')
    _scal = nag_double_type_check_and_cast('f08qvc', scal, 'scal')
    f08qvc_ctypes(order, trana, tranb, sign, m, n, _a, pda, _b, pdb, _c, pdc, _scal, fail)

f08qxc_ctypes = _library_cl.f08qxc
f08qxc_ctypes.restype = None
f08qxc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_HowManyType, POINTER(Nag_Boolean), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, Integer, POINTER(Integer), POINTER(NagError)]


def f08qxc(order, side, howmany, select, n, t, pdt, vl, pdvl, vr, pdvr, mm, m, fail):
    """
    Computes left and right eigenvectors of complex upper triangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08qxc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        select, t, vl, vr, m
    TypeError will be raised when the following arguments are not of data type
    Nag_Boolean:
        select
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m
    TypeError will be raised when the following arguments are not of data type
    Complex:
        t, vl, vr
    """
    from .util import nag_enum_type_check_and_cast
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _select = nag_enum_type_check_and_cast('f08qxc', select, 'select', Nag_Boolean)
    _t = nag_complex_type_check_and_cast('f08qxc', t, 't')
    _vl = nag_complex_type_check_and_cast('f08qxc', vl, 'vl')
    _vr = nag_complex_type_check_and_cast('f08qxc', vr, 'vr')
    _m = nag_integer_type_check_and_cast('f08qxc', m, 'm')
    f08qxc_ctypes(order, side, howmany, _select, n, _t, pdt, _vl, pdvl, _vr, pdvr, mm, _m, fail)

f08qyc_ctypes = _library_cl.f08qyc
f08qyc_ctypes.restype = None
f08qyc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_HowManyType, POINTER(Nag_Boolean), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(Integer), POINTER(NagError)]


def f08qyc(order, job, howmany, select, n, t, pdt, vl, pdvl, vr, pdvr, s, sep, mm, m, fail):
    """
    Computes estimates of sensitivities of selected eigenvalues and
    eigenvectors of complex upper triangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08qyc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        select, t, vl, vr, s, sep, m
    TypeError will be raised when the following arguments are not of data type
    Nag_Boolean:
        select
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        s, sep
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m
    TypeError will be raised when the following arguments are not of data type
    Complex:
        t, vl, vr
    """
    from .util import nag_enum_type_check_and_cast
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _select = nag_enum_type_check_and_cast('f08qyc', select, 'select', Nag_Boolean)
    _t = nag_complex_type_check_and_cast('f08qyc', t, 't')
    _vl = nag_complex_type_check_and_cast('f08qyc', vl, 'vl')
    _vr = nag_complex_type_check_and_cast('f08qyc', vr, 'vr')
    _s = nag_double_type_check_and_cast('f08qyc', s, 's')
    _sep = nag_double_type_check_and_cast('f08qyc', sep, 'sep')
    _m = nag_integer_type_check_and_cast('f08qyc', m, 'm')
    f08qyc_ctypes(order, job, howmany, _select, n, _t, pdt, _vl, pdvl, _vr, pdvr, _s, _sep, mm, _m, fail)

f08rac_ctypes = _library_cl.f08rac
f08rac_ctypes.restype = None
f08rac_ctypes.argtypes = [Nag_OrderType, Nag_ComputeUType, Nag_ComputeUType, Nag_ComputeVTType, Nag_ComputeVTType, Nag_SignsType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f08rac(order, jobu1, jobu2, jobv1t, jobv2t, signs, m, p, q, x11, pdx11, x12, pdx12, x21, pdx21, x22, pdx22, theta, u1, pdu1, u2, pdu2, v1t, pdv1t, v2t, pdv2t, fail):
    """
    Computes the CS decomposition of an orthogonal matrix partitioned into
    four real submatrices.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08rac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x11, x12, x21, x22, theta, u1, u2, v1t, v2t
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x11, x12, x21, x22, theta, u1, u2, v1t, v2t
    """
    from .util import nag_double_type_check_and_cast
    _x11 = nag_double_type_check_and_cast('f08rac', x11, 'x11')
    _x12 = nag_double_type_check_and_cast('f08rac', x12, 'x12')
    _x21 = nag_double_type_check_and_cast('f08rac', x21, 'x21')
    _x22 = nag_double_type_check_and_cast('f08rac', x22, 'x22')
    _theta = nag_double_type_check_and_cast('f08rac', theta, 'theta')
    _u1 = nag_double_type_check_and_cast('f08rac', u1, 'u1')
    _u2 = nag_double_type_check_and_cast('f08rac', u2, 'u2')
    _v1t = nag_double_type_check_and_cast('f08rac', v1t, 'v1t')
    _v2t = nag_double_type_check_and_cast('f08rac', v2t, 'v2t')
    f08rac_ctypes(order, jobu1, jobu2, jobv1t, jobv2t, signs, m, p, q, _x11, pdx11, _x12, pdx12, _x21, pdx21, _x22, pdx22, _theta, _u1, pdu1, _u2, pdu2, _v1t, pdv1t, _v2t, pdv2t, fail)

f08rnc_ctypes = _library_cl.f08rnc
f08rnc_ctypes.restype = None
f08rnc_ctypes.argtypes = [Nag_OrderType, Nag_ComputeUType, Nag_ComputeUType, Nag_ComputeVTType, Nag_ComputeVTType, Nag_SignsType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f08rnc(order, jobu1, jobu2, jobv1t, jobv2t, signs, m, p, q, x11, pdx11, x12, pdx12, x21, pdx21, x22, pdx22, theta, u1, pdu1, u2, pdu2, v1t, pdv1t, v2t, pdv2t, fail):
    """
    Computes the CS decomposition of an unitary matrix partitioned into four
    complex submatrices.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08rnc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x11, x12, x21, x22, theta, u1, u2, v1t, v2t
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        theta
    TypeError will be raised when the following arguments are not of data type
    Complex:
        x11, x12, x21, x22, u1, u2, v1t, v2t
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _x11 = nag_complex_type_check_and_cast('f08rnc', x11, 'x11')
    _x12 = nag_complex_type_check_and_cast('f08rnc', x12, 'x12')
    _x21 = nag_complex_type_check_and_cast('f08rnc', x21, 'x21')
    _x22 = nag_complex_type_check_and_cast('f08rnc', x22, 'x22')
    _theta = nag_double_type_check_and_cast('f08rnc', theta, 'theta')
    _u1 = nag_complex_type_check_and_cast('f08rnc', u1, 'u1')
    _u2 = nag_complex_type_check_and_cast('f08rnc', u2, 'u2')
    _v1t = nag_complex_type_check_and_cast('f08rnc', v1t, 'v1t')
    _v2t = nag_complex_type_check_and_cast('f08rnc', v2t, 'v2t')
    f08rnc_ctypes(order, jobu1, jobu2, jobv1t, jobv2t, signs, m, p, q, _x11, pdx11, _x12, pdx12, _x21, pdx21, _x22, pdx22, _theta, _u1, pdu1, _u2, pdu2, _v1t, pdv1t, _v2t, pdv2t, fail)

f08sac_ctypes = _library_cl.f08sac
f08sac_ctypes.restype = None
f08sac_ctypes.argtypes = [Nag_OrderType, Integer, Nag_JobType, Nag_UploType, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f08sac(order, itype, job, uplo, n, a, pda, b, pdb, w, fail):
    """
    Computes all the eigenvalues, and optionally, the eigenvectors of a real
    generalized symmetric-definite eigenproblem.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08sac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, w
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, w
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08sac', a, 'a')
    _b = nag_double_type_check_and_cast('f08sac', b, 'b')
    _w = nag_double_type_check_and_cast('f08sac', w, 'w')
    f08sac_ctypes(order, itype, job, uplo, n, _a, pda, _b, pdb, _w, fail)

f08sbc_ctypes = _library_cl.f08sbc
f08sbc_ctypes.restype = None
f08sbc_ctypes.argtypes = [Nag_OrderType, Integer, Nag_JobType, Nag_RangeType, Nag_UploType, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, c_double, c_double, Integer, Integer, c_double, POINTER(Integer), POINTER(c_double), POINTER(c_double), Integer, POINTER(Integer), POINTER(NagError)]


def f08sbc(order, itype, job, _range, uplo, n, a, pda, b, pdb, vl, vu, il, iu, abstol, m, w, z, pdz, jfail, fail):
    """
    Computes selected eigenvalues, and optionally, the eigenvectors of a
    real generalized symmetric-definite eigenproblem.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08sbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, m, w, z, jfail
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, w, z
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m, jfail
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08sbc', a, 'a')
    _b = nag_double_type_check_and_cast('f08sbc', b, 'b')
    _m = nag_integer_type_check_and_cast('f08sbc', m, 'm')
    _w = nag_double_type_check_and_cast('f08sbc', w, 'w')
    _z = nag_double_type_check_and_cast('f08sbc', z, 'z')
    _jfail = nag_integer_type_check_and_cast('f08sbc', jfail, 'jfail')
    f08sbc_ctypes(order, itype, job, _range, uplo, n, _a, pda, _b, pdb, vl, vu, il, iu, abstol, _m, _w, _z, pdz, _jfail, fail)

f08scc_ctypes = _library_cl.f08scc
f08scc_ctypes.restype = None
f08scc_ctypes.argtypes = [Nag_OrderType, Integer, Nag_JobType, Nag_UploType, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f08scc(order, itype, job, uplo, n, a, pda, b, pdb, w, fail):
    """
    Computes all the eigenvalues, and optionally, the eigenvectors of a real
    generalized symmetric-definite eigenproblem (divide-and-conquer).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08scc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, w
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, w
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08scc', a, 'a')
    _b = nag_double_type_check_and_cast('f08scc', b, 'b')
    _w = nag_double_type_check_and_cast('f08scc', w, 'w')
    f08scc_ctypes(order, itype, job, uplo, n, _a, pda, _b, pdb, _w, fail)

f08sec_ctypes = _library_cl.f08sec
f08sec_ctypes.restype = None
f08sec_ctypes.argtypes = [Nag_OrderType, Nag_ComputeType, Nag_UploType, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f08sec(order, comptype, uplo, n, a, pda, b, pdb, fail):
    """
    Performs a reduction to standard form of real symmetric-definite
    generalized eigenproblem Ax = lambdaBx, ABx = lambdax or BAx = lambdax,
    B factorized by f07fdc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08sec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08sec', a, 'a')
    _b = nag_double_type_check_and_cast('f08sec', b, 'b')
    f08sec_ctypes(order, comptype, uplo, n, _a, pda, _b, pdb, fail)

f08snc_ctypes = _library_cl.f08snc
f08snc_ctypes.restype = None
f08snc_ctypes.argtypes = [Nag_OrderType, Integer, Nag_JobType, Nag_UploType, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(NagError)]


def f08snc(order, itype, job, uplo, n, a, pda, b, pdb, w, fail):
    """
    Computes all the eigenvalues, and optionally, the eigenvectors of a
    complex generalized Hermitian-definite eigenproblem.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08snc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, w
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        w
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08snc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08snc', b, 'b')
    _w = nag_double_type_check_and_cast('f08snc', w, 'w')
    f08snc_ctypes(order, itype, job, uplo, n, _a, pda, _b, pdb, _w, fail)

f08spc_ctypes = _library_cl.f08spc
f08spc_ctypes.restype = None
f08spc_ctypes.argtypes = [Nag_OrderType, Integer, Nag_JobType, Nag_RangeType, Nag_UploType, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, c_double, c_double, Integer, Integer, c_double, POINTER(Integer), POINTER(c_double), POINTER(Complex), Integer, POINTER(Integer), POINTER(NagError)]


def f08spc(order, itype, job, _range, uplo, n, a, pda, b, pdb, vl, vu, il, iu, abstol, m, w, z, pdz, jfail, fail):
    """
    Computes selected eigenvalues, and optionally, the eigenvectors of a
    complex generalized Hermitian-definite eigenproblem.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08spc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, m, w, z, jfail
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        w
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m, jfail
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, z
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08spc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08spc', b, 'b')
    _m = nag_integer_type_check_and_cast('f08spc', m, 'm')
    _w = nag_double_type_check_and_cast('f08spc', w, 'w')
    _z = nag_complex_type_check_and_cast('f08spc', z, 'z')
    _jfail = nag_integer_type_check_and_cast('f08spc', jfail, 'jfail')
    f08spc_ctypes(order, itype, job, _range, uplo, n, _a, pda, _b, pdb, vl, vu, il, iu, abstol, _m, _w, _z, pdz, _jfail, fail)

f08sqc_ctypes = _library_cl.f08sqc
f08sqc_ctypes.restype = None
f08sqc_ctypes.argtypes = [Nag_OrderType, Integer, Nag_JobType, Nag_UploType, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(NagError)]


def f08sqc(order, itype, job, uplo, n, a, pda, b, pdb, w, fail):
    """
    Computes all the eigenvalues, and optionally, the eigenvectors of a
    complex generalized Hermitian-definite eigenproblem (divide-and-
    conquer).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08sqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, w
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        w
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08sqc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08sqc', b, 'b')
    _w = nag_double_type_check_and_cast('f08sqc', w, 'w')
    f08sqc_ctypes(order, itype, job, uplo, n, _a, pda, _b, pdb, _w, fail)

f08ssc_ctypes = _library_cl.f08ssc
f08ssc_ctypes.restype = None
f08ssc_ctypes.argtypes = [Nag_OrderType, Nag_ComputeType, Nag_UploType, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f08ssc(order, comptype, uplo, n, a, pda, b, pdb, fail):
    """
    Performs a reduction to standard form of complex Hermitian-definite
    generalized eigenproblem Ax = lambdaBx, ABx = lambdax or BAx = lambdax,
    B factorized by f07frc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08ssc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08ssc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08ssc', b, 'b')
    f08ssc_ctypes(order, comptype, uplo, n, _a, pda, _b, pdb, fail)

f08tac_ctypes = _library_cl.f08tac
f08tac_ctypes.restype = None
f08tac_ctypes.argtypes = [Nag_OrderType, Integer, Nag_JobType, Nag_UploType, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f08tac(order, itype, job, uplo, n, ap, bp, w, z, pdz, fail):
    """
    Computes all the eigenvalues, and optionally, the eigenvectors of a real
    generalized symmetric-definite eigenproblem, packed storage.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08tac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, bp, w, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ap, bp, w, z
    """
    from .util import nag_double_type_check_and_cast
    _ap = nag_double_type_check_and_cast('f08tac', ap, 'ap')
    _bp = nag_double_type_check_and_cast('f08tac', bp, 'bp')
    _w = nag_double_type_check_and_cast('f08tac', w, 'w')
    _z = nag_double_type_check_and_cast('f08tac', z, 'z')
    f08tac_ctypes(order, itype, job, uplo, n, _ap, _bp, _w, _z, pdz, fail)

f08tbc_ctypes = _library_cl.f08tbc
f08tbc_ctypes.restype = None
f08tbc_ctypes.argtypes = [Nag_OrderType, Integer, Nag_JobType, Nag_RangeType, Nag_UploType, Integer, POINTER(c_double), POINTER(c_double), c_double, c_double, Integer, Integer, c_double, POINTER(Integer), POINTER(c_double), POINTER(c_double), Integer, POINTER(Integer), POINTER(NagError)]


def f08tbc(order, itype, job, _range, uplo, n, ap, bp, vl, vu, il, iu, abstol, m, w, z, pdz, jfail, fail):
    """
    Computes selected eigenvalues, and optionally, the eigenvectors of a
    real generalized symmetric-definite eigenproblem, packed storage.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08tbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, bp, m, w, z, jfail
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ap, bp, w, z
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m, jfail
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _ap = nag_double_type_check_and_cast('f08tbc', ap, 'ap')
    _bp = nag_double_type_check_and_cast('f08tbc', bp, 'bp')
    _m = nag_integer_type_check_and_cast('f08tbc', m, 'm')
    _w = nag_double_type_check_and_cast('f08tbc', w, 'w')
    _z = nag_double_type_check_and_cast('f08tbc', z, 'z')
    _jfail = nag_integer_type_check_and_cast('f08tbc', jfail, 'jfail')
    f08tbc_ctypes(order, itype, job, _range, uplo, n, _ap, _bp, vl, vu, il, iu, abstol, _m, _w, _z, pdz, _jfail, fail)

f08tcc_ctypes = _library_cl.f08tcc
f08tcc_ctypes.restype = None
f08tcc_ctypes.argtypes = [Nag_OrderType, Integer, Nag_JobType, Nag_UploType, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f08tcc(order, itype, job, uplo, n, ap, bp, w, z, pdz, fail):
    """
    Computes all the eigenvalues, and optionally, the eigenvectors of a real
    generalized symmetric-definite eigenproblem, packed storage (divide-and-
    conquer).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08tcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, bp, w, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ap, bp, w, z
    """
    from .util import nag_double_type_check_and_cast
    _ap = nag_double_type_check_and_cast('f08tcc', ap, 'ap')
    _bp = nag_double_type_check_and_cast('f08tcc', bp, 'bp')
    _w = nag_double_type_check_and_cast('f08tcc', w, 'w')
    _z = nag_double_type_check_and_cast('f08tcc', z, 'z')
    f08tcc_ctypes(order, itype, job, uplo, n, _ap, _bp, _w, _z, pdz, fail)

f08tec_ctypes = _library_cl.f08tec
f08tec_ctypes.restype = None
f08tec_ctypes.argtypes = [Nag_OrderType, Nag_ComputeType, Nag_UploType, Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f08tec(order, comptype, uplo, n, ap, bp, fail):
    """
    Performs a reduction to standard form of real symmetric-definite
    generalized eigenproblem Ax = lambdaBx, ABx = lambdax or BAx = lambdax,
    packed storage, B factorized by f07gdc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08tec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, bp
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ap, bp
    """
    from .util import nag_double_type_check_and_cast
    _ap = nag_double_type_check_and_cast('f08tec', ap, 'ap')
    _bp = nag_double_type_check_and_cast('f08tec', bp, 'bp')
    f08tec_ctypes(order, comptype, uplo, n, _ap, _bp, fail)

f08tnc_ctypes = _library_cl.f08tnc
f08tnc_ctypes.restype = None
f08tnc_ctypes.argtypes = [Nag_OrderType, Integer, Nag_JobType, Nag_UploType, Integer, POINTER(Complex), POINTER(Complex), POINTER(c_double), POINTER(Complex), Integer, POINTER(NagError)]


def f08tnc(order, itype, job, uplo, n, ap, bp, w, z, pdz, fail):
    """
    Computes all the eigenvalues, and optionally, the eigenvectors of a
    complex generalized Hermitian-definite eigenproblem, packed storage.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08tnc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, bp, w, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        w
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ap, bp, z
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _ap = nag_complex_type_check_and_cast('f08tnc', ap, 'ap')
    _bp = nag_complex_type_check_and_cast('f08tnc', bp, 'bp')
    _w = nag_double_type_check_and_cast('f08tnc', w, 'w')
    _z = nag_complex_type_check_and_cast('f08tnc', z, 'z')
    f08tnc_ctypes(order, itype, job, uplo, n, _ap, _bp, _w, _z, pdz, fail)

f08tpc_ctypes = _library_cl.f08tpc
f08tpc_ctypes.restype = None
f08tpc_ctypes.argtypes = [Nag_OrderType, Integer, Nag_JobType, Nag_RangeType, Nag_UploType, Integer, POINTER(Complex), POINTER(Complex), c_double, c_double, Integer, Integer, c_double, POINTER(Integer), POINTER(c_double), POINTER(Complex), Integer, POINTER(Integer), POINTER(NagError)]


def f08tpc(order, itype, job, _range, uplo, n, ap, bp, vl, vu, il, iu, abstol, m, w, z, pdz, jfail, fail):
    """
    Computes selected eigenvalues, and optionally, the eigenvectors of a
    complex generalized Hermitian-definite eigenproblem, packed storage.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08tpc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, bp, m, w, z, jfail
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        w
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m, jfail
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ap, bp, z
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _ap = nag_complex_type_check_and_cast('f08tpc', ap, 'ap')
    _bp = nag_complex_type_check_and_cast('f08tpc', bp, 'bp')
    _m = nag_integer_type_check_and_cast('f08tpc', m, 'm')
    _w = nag_double_type_check_and_cast('f08tpc', w, 'w')
    _z = nag_complex_type_check_and_cast('f08tpc', z, 'z')
    _jfail = nag_integer_type_check_and_cast('f08tpc', jfail, 'jfail')
    f08tpc_ctypes(order, itype, job, _range, uplo, n, _ap, _bp, vl, vu, il, iu, abstol, _m, _w, _z, pdz, _jfail, fail)

f08tqc_ctypes = _library_cl.f08tqc
f08tqc_ctypes.restype = None
f08tqc_ctypes.argtypes = [Nag_OrderType, Integer, Nag_JobType, Nag_UploType, Integer, POINTER(Complex), POINTER(Complex), POINTER(c_double), POINTER(Complex), Integer, POINTER(NagError)]


def f08tqc(order, itype, job, uplo, n, ap, bp, w, z, pdz, fail):
    """
    Computes selected eigenvalues, and optionally, the eigenvectors of a
    complex generalized Hermitian-definite eigenproblem, packed storage
    (divide-and-conquer).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08tqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, bp, w, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        w
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ap, bp, z
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _ap = nag_complex_type_check_and_cast('f08tqc', ap, 'ap')
    _bp = nag_complex_type_check_and_cast('f08tqc', bp, 'bp')
    _w = nag_double_type_check_and_cast('f08tqc', w, 'w')
    _z = nag_complex_type_check_and_cast('f08tqc', z, 'z')
    f08tqc_ctypes(order, itype, job, uplo, n, _ap, _bp, _w, _z, pdz, fail)

f08tsc_ctypes = _library_cl.f08tsc
f08tsc_ctypes.restype = None
f08tsc_ctypes.argtypes = [Nag_OrderType, Nag_ComputeType, Nag_UploType, Integer, POINTER(Complex), POINTER(Complex), POINTER(NagError)]


def f08tsc(order, comptype, uplo, n, ap, bp, fail):
    """
    Performs a reduction to standard form of complex Hermitian-definite
    generalized eigenproblem Ax = lambdaBx, ABx = lambdax or BAx = lambdax,
    packed storage, B factorized by f07grc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08tsc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, bp
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ap, bp
    """
    from .util import nag_complex_type_check_and_cast
    _ap = nag_complex_type_check_and_cast('f08tsc', ap, 'ap')
    _bp = nag_complex_type_check_and_cast('f08tsc', bp, 'bp')
    f08tsc_ctypes(order, comptype, uplo, n, _ap, _bp, fail)

f08uac_ctypes = _library_cl.f08uac
f08uac_ctypes.restype = None
f08uac_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_UploType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f08uac(order, job, uplo, n, ka, kb, ab, pdab, bb, pdbb, w, z, pdz, fail):
    """
    Computes all the eigenvalues, and optionally, the eigenvectors of a real
    banded generalized symmetric-definite eigenproblem.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08uac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, bb, w, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ab, bb, w, z
    """
    from .util import nag_double_type_check_and_cast
    _ab = nag_double_type_check_and_cast('f08uac', ab, 'ab')
    _bb = nag_double_type_check_and_cast('f08uac', bb, 'bb')
    _w = nag_double_type_check_and_cast('f08uac', w, 'w')
    _z = nag_double_type_check_and_cast('f08uac', z, 'z')
    f08uac_ctypes(order, job, uplo, n, ka, kb, _ab, pdab, _bb, pdbb, _w, _z, pdz, fail)

f08ubc_ctypes = _library_cl.f08ubc
f08ubc_ctypes.restype = None
f08ubc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_RangeType, Nag_UploType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, c_double, c_double, Integer, Integer, c_double, POINTER(Integer), POINTER(c_double), POINTER(c_double), Integer, POINTER(Integer), POINTER(NagError)]


def f08ubc(order, job, _range, uplo, n, ka, kb, ab, pdab, bb, pdbb, q, pdq, vl, vu, il, iu, abstol, m, w, z, pdz, jfail, fail):
    """
    Computes selected eigenvalues, and optionally, the eigenvectors of a
    real banded generalized symmetric-definite eigenproblem.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08ubc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, bb, q, m, w, z, jfail
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ab, bb, q, w, z
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m, jfail
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _ab = nag_double_type_check_and_cast('f08ubc', ab, 'ab')
    _bb = nag_double_type_check_and_cast('f08ubc', bb, 'bb')
    _q = nag_double_type_check_and_cast('f08ubc', q, 'q')
    _m = nag_integer_type_check_and_cast('f08ubc', m, 'm')
    _w = nag_double_type_check_and_cast('f08ubc', w, 'w')
    _z = nag_double_type_check_and_cast('f08ubc', z, 'z')
    _jfail = nag_integer_type_check_and_cast('f08ubc', jfail, 'jfail')
    f08ubc_ctypes(order, job, _range, uplo, n, ka, kb, _ab, pdab, _bb, pdbb, _q, pdq, vl, vu, il, iu, abstol, _m, _w, _z, pdz, _jfail, fail)

f08ucc_ctypes = _library_cl.f08ucc
f08ucc_ctypes.restype = None
f08ucc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_UploType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f08ucc(order, job, uplo, n, ka, kb, ab, pdab, bb, pdbb, w, z, pdz, fail):
    """
    Computes all the eigenvalues, and optionally, the eigenvectors of a real
    banded generalized symmetric-definite eigenproblem (divide-and-conquer).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08ucc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, bb, w, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ab, bb, w, z
    """
    from .util import nag_double_type_check_and_cast
    _ab = nag_double_type_check_and_cast('f08ucc', ab, 'ab')
    _bb = nag_double_type_check_and_cast('f08ucc', bb, 'bb')
    _w = nag_double_type_check_and_cast('f08ucc', w, 'w')
    _z = nag_double_type_check_and_cast('f08ucc', z, 'z')
    f08ucc_ctypes(order, job, uplo, n, ka, kb, _ab, pdab, _bb, pdbb, _w, _z, pdz, fail)

f08uec_ctypes = _library_cl.f08uec
f08uec_ctypes.restype = None
f08uec_ctypes.argtypes = [Nag_OrderType, Nag_VectType, Nag_UploType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f08uec(order, vect, uplo, n, ka, kb, ab, pdab, bb, pdbb, x, pdx, fail):
    """
    Performs a reduction of real symmetric-definite banded generalized
    eigenproblem Ax = lambdaBx to standard form Cy = lambday, such that C
    has the same bandwidth as A.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08uec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, bb, x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ab, bb, x
    """
    from .util import nag_double_type_check_and_cast
    _ab = nag_double_type_check_and_cast('f08uec', ab, 'ab')
    _bb = nag_double_type_check_and_cast('f08uec', bb, 'bb')
    _x = nag_double_type_check_and_cast('f08uec', x, 'x')
    f08uec_ctypes(order, vect, uplo, n, ka, kb, _ab, pdab, _bb, pdbb, _x, pdx, fail)

f08ufc_ctypes = _library_cl.f08ufc
f08ufc_ctypes.restype = None
f08ufc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f08ufc(order, uplo, n, kb, bb, pdbb, fail):
    """
    Computes a split Cholesky factorization of real symmetric positive
    definite band matrix A.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08ufc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        bb
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        bb
    """
    from .util import nag_double_type_check_and_cast
    _bb = nag_double_type_check_and_cast('f08ufc', bb, 'bb')
    f08ufc_ctypes(order, uplo, n, kb, _bb, pdbb, fail)

f08unc_ctypes = _library_cl.f08unc
f08unc_ctypes.restype = None
f08unc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_UploType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(Complex), Integer, POINTER(NagError)]


def f08unc(order, job, uplo, n, ka, kb, ab, pdab, bb, pdbb, w, z, pdz, fail):
    """
    Computes all the eigenvalues, and optionally, the eigenvectors of a
    complex banded generalized Hermitian-definite eigenproblem.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08unc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, bb, w, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        w
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ab, bb, z
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _ab = nag_complex_type_check_and_cast('f08unc', ab, 'ab')
    _bb = nag_complex_type_check_and_cast('f08unc', bb, 'bb')
    _w = nag_double_type_check_and_cast('f08unc', w, 'w')
    _z = nag_complex_type_check_and_cast('f08unc', z, 'z')
    f08unc_ctypes(order, job, uplo, n, ka, kb, _ab, pdab, _bb, pdbb, _w, _z, pdz, fail)

f08upc_ctypes = _library_cl.f08upc
f08upc_ctypes.restype = None
f08upc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_RangeType, Nag_UploType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, c_double, c_double, Integer, Integer, c_double, POINTER(Integer), POINTER(c_double), POINTER(Complex), Integer, POINTER(Integer), POINTER(NagError)]


def f08upc(order, job, _range, uplo, n, ka, kb, ab, pdab, bb, pdbb, q, pdq, vl, vu, il, iu, abstol, m, w, z, pdz, jfail, fail):
    """
    Computes selected eigenvalues, and optionally, the eigenvectors of a
    complex banded generalized Hermitian-definite eigenproblem.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08upc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, bb, q, m, w, z, jfail
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        w
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m, jfail
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ab, bb, q, z
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _ab = nag_complex_type_check_and_cast('f08upc', ab, 'ab')
    _bb = nag_complex_type_check_and_cast('f08upc', bb, 'bb')
    _q = nag_complex_type_check_and_cast('f08upc', q, 'q')
    _m = nag_integer_type_check_and_cast('f08upc', m, 'm')
    _w = nag_double_type_check_and_cast('f08upc', w, 'w')
    _z = nag_complex_type_check_and_cast('f08upc', z, 'z')
    _jfail = nag_integer_type_check_and_cast('f08upc', jfail, 'jfail')
    f08upc_ctypes(order, job, _range, uplo, n, ka, kb, _ab, pdab, _bb, pdbb, _q, pdq, vl, vu, il, iu, abstol, _m, _w, _z, pdz, _jfail, fail)

f08uqc_ctypes = _library_cl.f08uqc
f08uqc_ctypes.restype = None
f08uqc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_UploType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(Complex), Integer, POINTER(NagError)]


def f08uqc(order, job, uplo, n, ka, kb, ab, pdab, bb, pdbb, w, z, pdz, fail):
    """
    Computes all the eigenvalues, and optionally, the eigenvectors of a
    complex banded generalized Hermitian-definite eigenproblem (divide-and-
    conquer).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08uqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, bb, w, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        w
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ab, bb, z
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _ab = nag_complex_type_check_and_cast('f08uqc', ab, 'ab')
    _bb = nag_complex_type_check_and_cast('f08uqc', bb, 'bb')
    _w = nag_double_type_check_and_cast('f08uqc', w, 'w')
    _z = nag_complex_type_check_and_cast('f08uqc', z, 'z')
    f08uqc_ctypes(order, job, uplo, n, ka, kb, _ab, pdab, _bb, pdbb, _w, _z, pdz, fail)

f08usc_ctypes = _library_cl.f08usc
f08usc_ctypes.restype = None
f08usc_ctypes.argtypes = [Nag_OrderType, Nag_VectType, Nag_UploType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f08usc(order, vect, uplo, n, ka, kb, ab, pdab, bb, pdbb, x, pdx, fail):
    """
    Performs a reduction of complex Hermitian-definite banded generalized
    eigenproblem Ax = lambdaBx to standard form Cy = lambday, such that C
    has the same bandwidth as A.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08usc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, bb, x
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ab, bb, x
    """
    from .util import nag_complex_type_check_and_cast
    _ab = nag_complex_type_check_and_cast('f08usc', ab, 'ab')
    _bb = nag_complex_type_check_and_cast('f08usc', bb, 'bb')
    _x = nag_complex_type_check_and_cast('f08usc', x, 'x')
    f08usc_ctypes(order, vect, uplo, n, ka, kb, _ab, pdab, _bb, pdbb, _x, pdx, fail)

f08utc_ctypes = _library_cl.f08utc
f08utc_ctypes.restype = None
f08utc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f08utc(order, uplo, n, kb, bb, pdbb, fail):
    """
    Computes a split Cholesky factorization of complex Hermitian positive
    definite band matrix A.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08utc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        bb
    TypeError will be raised when the following arguments are not of data type
    Complex:
        bb
    """
    from .util import nag_complex_type_check_and_cast
    _bb = nag_complex_type_check_and_cast('f08utc', bb, 'bb')
    f08utc_ctypes(order, uplo, n, kb, _bb, pdbb, fail)

f08vac_ctypes = _library_cl.f08vac
f08vac_ctypes.restype = None
f08vac_ctypes.argtypes = [Nag_OrderType, Nag_ComputeUType, Nag_ComputeVType, Nag_ComputeQType, Integer, Integer, Integer, POINTER(Integer), POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(NagError)]


def f08vac(order, jobu, jobv, jobq, m, n, p, k, l, a, pda, b, pdb, alpha, beta, u, pdu, v, pdv, q, pdq, iwork, fail):
    """
    Computes the generalized singular value decomposition of a real matrix
    pair.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08vac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        k, l, a, b, alpha, beta, u, v, q, iwork
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, alpha, beta, u, v, q
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        k, l, iwork
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _k = nag_integer_type_check_and_cast('f08vac', k, 'k')
    _l = nag_integer_type_check_and_cast('f08vac', l, 'l')
    _a = nag_double_type_check_and_cast('f08vac', a, 'a')
    _b = nag_double_type_check_and_cast('f08vac', b, 'b')
    _alpha = nag_double_type_check_and_cast('f08vac', alpha, 'alpha')
    _beta = nag_double_type_check_and_cast('f08vac', beta, 'beta')
    _u = nag_double_type_check_and_cast('f08vac', u, 'u')
    _v = nag_double_type_check_and_cast('f08vac', v, 'v')
    _q = nag_double_type_check_and_cast('f08vac', q, 'q')
    _iwork = nag_integer_type_check_and_cast('f08vac', iwork, 'iwork')
    f08vac_ctypes(order, jobu, jobv, jobq, m, n, p, _k, _l, _a, pda, _b, pdb, _alpha, _beta, _u, pdu, _v, pdv, _q, pdq, _iwork, fail)

f08vcc_ctypes = _library_cl.f08vcc
f08vcc_ctypes.restype = None
f08vcc_ctypes.argtypes = [Nag_OrderType,
                          Nag_ComputeUType,
                          Nag_ComputeVType,
                          Nag_ComputeQType,
                          Integer,
                          Integer,
                          Integer,
                          POINTER(Integer),
                          POINTER(Integer),
                          POINTER(c_double),
                          Integer,
                          POINTER(c_double),
                          Integer,
                          POINTER(c_double),
                          POINTER(c_double),
                          POINTER(c_double),
                          Integer,
                          POINTER(c_double),
                          Integer,
                          POINTER(c_double),
                          Integer,
                          POINTER(Integer),
                          POINTER(NagError)]


def f08vcc(order, jobu, jobv, jobq, m, n, p, k, l, a, pda, b, pdb, alpha,
           beta, u, pdu, v, pdv, q, pdq, iwork, fail):
    """
    Computes, using BLAS-3, the generalized singular value decomposition
    of a real matrix pair.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08vcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, alpha, beta, k, l, q, u, v, iwork
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, alpha, beta, q, u, v
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        k, l, iwork
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08vcc', a, 'a')
    _b = nag_double_type_check_and_cast('f08vcc', b, 'b')
    _alpha = nag_double_type_check_and_cast('f08vcc', alpha, 'alpha')
    _beta = nag_double_type_check_and_cast('f08vcc', beta, 'beta')
    _k = nag_integer_type_check_and_cast('f08vcc', k, 'k')
    _l = nag_integer_type_check_and_cast('f08vcc', l, 'l')
    _u = nag_double_type_check_and_cast('f08vcc', u, 'u')
    _v = nag_double_type_check_and_cast('f08vcc', v, 'v')
    _q = nag_double_type_check_and_cast('f08vcc', q, 'q')
    _iwork = nag_integer_type_check_and_cast('f08vcc', iwork, 'iwork')
    f08vcc_ctypes(order, jobu, jobv, jobq, m, n, p, _k, _l, _a, pda, _b, pdb,
                  _alpha, _beta, _u, pdu, _v, pdv, _q, pdq, _iwork, fail)

f08vec_ctypes = _library_cl.f08vec
f08vec_ctypes.restype = None
f08vec_ctypes.argtypes = [Nag_OrderType, Nag_ComputeUType, Nag_ComputeVType, Nag_ComputeQType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, c_double, c_double, POINTER(Integer), POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f08vec(order, jobu, jobv, jobq, m, p, n, a, pda, b, pdb, tola, tolb, k, l, u, pdu, v, pdv, q, pdq, fail):
    """
    Produces orthogonal matrices that simultaneously reduce the m by n
    matrix A and the p by n matrix B to upper triangular form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08vec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, k, l, u, v, q
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, u, v, q
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        k, l
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08vec', a, 'a')
    _b = nag_double_type_check_and_cast('f08vec', b, 'b')
    _k = nag_integer_type_check_and_cast('f08vec', k, 'k')
    _l = nag_integer_type_check_and_cast('f08vec', l, 'l')
    _u = nag_double_type_check_and_cast('f08vec', u, 'u')
    _v = nag_double_type_check_and_cast('f08vec', v, 'v')
    _q = nag_double_type_check_and_cast('f08vec', q, 'q')
    f08vec_ctypes(order, jobu, jobv, jobq, m, p, n, _a, pda, _b, pdb, tola, tolb, _k, _l, _u, pdu, _v, pdv, _q, pdq, fail)

f08vgc_ctypes = _library_cl.f08vgc
f08vgc_ctypes.restype = None
f08vgc_ctypes.argtypes = [Nag_OrderType,
                          Nag_ComputeUType,
                          Nag_ComputeVType,
                          Nag_ComputeQType,
                          Integer,
                          Integer,
                          Integer,
                          POINTER(c_double),
                          Integer,
                          POINTER(c_double),
                          Integer,
                          c_double,
                          c_double,
                          POINTER(Integer),
                          POINTER(Integer),
                          POINTER(c_double),
                          Integer,
                          POINTER(c_double),
                          Integer,
                          POINTER(c_double),
                          Integer,
                          POINTER(NagError)]


def f08vgc(order, jobu, jobv, jobq, m, p, n, a, pda, b, pdb, tola, tolb, k, l,
           u, pdu, v, pdv, q, pdq, fail):
    """
    Produces orthogonal matrices, using BLAS-3, that simultaneously reduce
    the m by n matrix A and the p by n matrix B to upper triangular form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08vgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, k, l, q, u, v
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, q, u, v
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        k, l
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08vgc', a, 'a')
    _b = nag_double_type_check_and_cast('f08vgc', b, 'b')
    _k = nag_integer_type_check_and_cast('f08vgc', k, 'k')
    _l = nag_integer_type_check_and_cast('f08vgc', l, 'l')
    _u = nag_double_type_check_and_cast('f08vgc', u, 'u')
    _v = nag_double_type_check_and_cast('f08vgc', v, 'v')
    _q = nag_double_type_check_and_cast('f08vgc', q, 'q')
    f08vgc_ctypes(order, jobu, jobv, jobq, m, p, n, _a, pda, _b, pdb, tola,
                  tolb, _k, _l, _u, pdu, _v, pdv, _q, pdq, fail)

f08vnc_ctypes = _library_cl.f08vnc
f08vnc_ctypes.restype = None
f08vnc_ctypes.argtypes = [Nag_OrderType, Nag_ComputeUType, Nag_ComputeVType, Nag_ComputeQType, Integer, Integer, Integer, POINTER(Integer), POINTER(Integer), POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(c_double), POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Integer), POINTER(NagError)]


def f08vnc(order, jobu, jobv, jobq, m, n, p, k, l, a, pda, b, pdb, alpha, beta, u, pdu, v, pdv, q, pdq, iwork, fail):
    """
    Computes the generalized singular value decomposition of a complex
    matrix pair.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08vnc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        k, l, a, b, alpha, beta, u, v, q, iwork
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        alpha, beta
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        k, l, iwork
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, u, v, q
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _k = nag_integer_type_check_and_cast('f08vnc', k, 'k')
    _l = nag_integer_type_check_and_cast('f08vnc', l, 'l')
    _a = nag_complex_type_check_and_cast('f08vnc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08vnc', b, 'b')
    _alpha = nag_double_type_check_and_cast('f08vnc', alpha, 'alpha')
    _beta = nag_double_type_check_and_cast('f08vnc', beta, 'beta')
    _u = nag_complex_type_check_and_cast('f08vnc', u, 'u')
    _v = nag_complex_type_check_and_cast('f08vnc', v, 'v')
    _q = nag_complex_type_check_and_cast('f08vnc', q, 'q')
    _iwork = nag_integer_type_check_and_cast('f08vnc', iwork, 'iwork')
    f08vnc_ctypes(order, jobu, jobv, jobq, m, n, p, _k, _l, _a, pda, _b, pdb, _alpha, _beta, _u, pdu, _v, pdv, _q, pdq, _iwork, fail)

f08vqc_ctypes = _library_cl.f08vqc
f08vqc_ctypes.restype = None
f08vqc_ctypes.argtypes = [Nag_OrderType,
                          Nag_ComputeUType,
                          Nag_ComputeVType,
                          Nag_ComputeQType,
                          Integer,
                          Integer,
                          Integer,
                          POINTER(Integer),
                          POINTER(Integer),
                          POINTER(Complex),
                          Integer,
                          POINTER(Complex),
                          Integer,
                          POINTER(c_double),
                          POINTER(c_double),
                          POINTER(Complex),
                          Integer,
                          POINTER(Complex),
                          Integer,
                          POINTER(Complex),
                          Integer,
                          POINTER(Integer),
                          POINTER(NagError)]


def f08vqc(order, jobu, jobv, jobq, m, n, p, k, l, a, pda, b, pdb, alpha,
           beta, u, pdu, v, pdv, q, pdq, iwork, fail):
    """
    Computes, using BLAS-3, the generalized singular value decomposition
    of a complex matrix pair.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08vqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, alpha, beta, k, l, q, u, v, iwork
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        alpha, beta
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        k, l, iwork
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, q, u, v
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08vqc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08vqc', b, 'b')
    _alpha = nag_double_type_check_and_cast('f08vqc', alpha, 'alpha')
    _beta = nag_double_type_check_and_cast('f08vqc', beta, 'beta')
    _k = nag_integer_type_check_and_cast('f08vqc', k, 'k')
    _l = nag_integer_type_check_and_cast('f08vqc', l, 'l')
    _u = nag_complex_type_check_and_cast('f08vqc', u, 'u')
    _v = nag_complex_type_check_and_cast('f08vqc', v, 'v')
    _q = nag_complex_type_check_and_cast('f08vqc', q, 'q')
    _iwork = nag_integer_type_check_and_cast('f08vqc', iwork, 'iwork')
    f08vqc_ctypes(order, jobu, jobv, jobq, m, n, p, _k, _l, _a, pda, _b, pdb,
                  _alpha, _beta, _u, pdu, _v, pdv, _q, pdq, _iwork, fail)

f08vsc_ctypes = _library_cl.f08vsc
f08vsc_ctypes.restype = None
f08vsc_ctypes.argtypes = [Nag_OrderType, Nag_ComputeUType, Nag_ComputeVType, Nag_ComputeQType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, c_double, c_double, POINTER(Integer), POINTER(Integer), POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f08vsc(order, jobu, jobv, jobq, m, p, n, a, pda, b, pdb, tola, tolb, k, l, u, pdu, v, pdv, q, pdq, fail):
    """
    Produces unitary matrices that simultaneously reduce the complex ,m by
    n, matrix A and the complex, p by n, matrix B to upper triangular form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08vsc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, k, l, u, v, q
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        k, l
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, u, v, q
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08vsc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08vsc', b, 'b')
    _k = nag_integer_type_check_and_cast('f08vsc', k, 'k')
    _l = nag_integer_type_check_and_cast('f08vsc', l, 'l')
    _u = nag_complex_type_check_and_cast('f08vsc', u, 'u')
    _v = nag_complex_type_check_and_cast('f08vsc', v, 'v')
    _q = nag_complex_type_check_and_cast('f08vsc', q, 'q')
    f08vsc_ctypes(order, jobu, jobv, jobq, m, p, n, _a, pda, _b, pdb, tola, tolb, _k, _l, _u, pdu, _v, pdv, _q, pdq, fail)

f08vuc_ctypes = _library_cl.f08vuc
f08vuc_ctypes.restype = None
f08vuc_ctypes.argtypes = [Nag_OrderType,
                          Nag_ComputeUType,
                          Nag_ComputeVType,
                          Nag_ComputeQType,
                          Integer,
                          Integer,
                          Integer,
                          POINTER(Complex),
                          Integer,
                          POINTER(Complex),
                          Integer,
                          c_double,
                          c_double,
                          POINTER(Integer),
                          POINTER(Integer),
                          POINTER(Complex),
                          Integer,
                          POINTER(Complex),
                          Integer,
                          POINTER(Complex),
                          Integer,
                          POINTER(NagError)]


def f08vuc(order, jobu, jobv, jobq, m, p, n, a, pda, b, pdb, tola, tolb, k, l,
           u, pdu, v, pdv, q, pdq, fail):
    """
    Produces unitary matrices, using BLAS-3, that simultaneously reduce
    the complex m by n matrix A and the complex p by n matrix B to
    upper triangular form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08vuc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, k, l, q, u, v
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        k, l
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, q, u, v
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08vuc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08vuc', b, 'b')
    _k = nag_integer_type_check_and_cast('f08vuc', k, 'k')
    _l = nag_integer_type_check_and_cast('f08vuc', l, 'l')
    _u = nag_complex_type_check_and_cast('f08vuc', u, 'u')
    _v = nag_complex_type_check_and_cast('f08vuc', v, 'v')
    _q = nag_complex_type_check_and_cast('f08vuc', q, 'q')
    f08vuc_ctypes(order, jobu, jobv, jobq, m, p, n, _a, pda, _b, pdb, tola,
                  tolb, _k, _l, _u, pdu, _v, pdv, _q, pdq, fail)

f08wac_ctypes = _library_cl.f08wac
f08wac_ctypes.restype = None
f08wac_ctypes.argtypes = [Nag_OrderType, Nag_LeftVecsType, Nag_RightVecsType, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f08wac(order, jobvl, jobvr, n, a, pda, b, pdb, alphar, alphai, beta, vl, pdvl, vr, pdvr, fail):
    """
    Computes, for a real nonsymmetric matrix pair, the generalized
    eigenvalues, and optionally, the left and/or right generalized
    eigenvectors.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08wac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, alphar, alphai, beta, vl, vr
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, alphar, alphai, beta, vl, vr
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08wac', a, 'a')
    _b = nag_double_type_check_and_cast('f08wac', b, 'b')
    _alphar = nag_double_type_check_and_cast('f08wac', alphar, 'alphar')
    _alphai = nag_double_type_check_and_cast('f08wac', alphai, 'alphai')
    _beta = nag_double_type_check_and_cast('f08wac', beta, 'beta')
    _vl = nag_double_type_check_and_cast('f08wac', vl, 'vl')
    _vr = nag_double_type_check_and_cast('f08wac', vr, 'vr')
    f08wac_ctypes(order, jobvl, jobvr, n, _a, pda, _b, pdb, _alphar, _alphai, _beta, _vl, pdvl, _vr, pdvr, fail)

f08wbc_ctypes = _library_cl.f08wbc
f08wbc_ctypes.restype = None
f08wbc_ctypes.argtypes = [Nag_OrderType, Nag_BalanceType, Nag_LeftVecsType, Nag_RightVecsType, Nag_RCondType, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f08wbc(order, balanc, jobvl, jobvr, sense, n, a, pda, b, pdb, alphar, alphai, beta, vl, pdvl, vr, pdvr, ilo, ihi, lscale, rscale, abnrm, bbnrm, rconde, rcondv, fail):
    """
    Computes, for a real nonsymmetric matrix pair, the generalized
    eigenvalues, and optionally, the left and/or right generalized
    eigenvectors; also, optionally, the balancing transformation, the
    reciprocal condition numbers for the eigenvalues and for the right
    eigenvectors.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08wbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, alphar, alphai, beta, vl, vr, ilo, ihi, lscale, rscale, abnrm, bbnrm, rconde, rcondv
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, alphar, alphai, beta, vl, vr, lscale, rscale, abnrm, bbnrm, rconde, rcondv
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ilo, ihi
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08wbc', a, 'a')
    _b = nag_double_type_check_and_cast('f08wbc', b, 'b')
    _alphar = nag_double_type_check_and_cast('f08wbc', alphar, 'alphar')
    _alphai = nag_double_type_check_and_cast('f08wbc', alphai, 'alphai')
    _beta = nag_double_type_check_and_cast('f08wbc', beta, 'beta')
    _vl = nag_double_type_check_and_cast('f08wbc', vl, 'vl')
    _vr = nag_double_type_check_and_cast('f08wbc', vr, 'vr')
    _ilo = nag_integer_type_check_and_cast('f08wbc', ilo, 'ilo')
    _ihi = nag_integer_type_check_and_cast('f08wbc', ihi, 'ihi')
    _lscale = nag_double_type_check_and_cast('f08wbc', lscale, 'lscale')
    _rscale = nag_double_type_check_and_cast('f08wbc', rscale, 'rscale')
    _abnrm = nag_double_type_check_and_cast('f08wbc', abnrm, 'abnrm')
    _bbnrm = nag_double_type_check_and_cast('f08wbc', bbnrm, 'bbnrm')
    _rconde = nag_double_type_check_and_cast('f08wbc', rconde, 'rconde')
    _rcondv = nag_double_type_check_and_cast('f08wbc', rcondv, 'rcondv')
    f08wbc_ctypes(order, balanc, jobvl, jobvr, sense, n, _a, pda, _b, pdb, _alphar, _alphai, _beta, _vl, pdvl, _vr, pdvr, _ilo, _ihi, _lscale, _rscale, _abnrm, _bbnrm, _rconde, _rcondv, fail)

f08wcc_ctypes = _library_cl.f08wcc
f08wcc_ctypes.restype = None
f08wcc_ctypes.argtypes = [Nag_OrderType,
                          Nag_LeftVecsType,
                          Nag_RightVecsType,
                          Integer,
                          POINTER(c_double),
                          Integer,
                          POINTER(c_double),
                          Integer,
                          POINTER(c_double),
                          POINTER(c_double),
                          POINTER(c_double),
                          POINTER(c_double),
                          Integer,
                          POINTER(c_double),
                          Integer,
                          POINTER(NagError)]


def f08wcc(order, jobvl, jobvr, n, a, pda, b, pdb, alphar, alphai, beta, vl,
           pdvl, vr, pdvr, fail):
    """
    Computes, for a real nonsymmetric matrix pair, using BLAS-3, the generalized eigenvalues,
    and optionally, the left and/or right generalized eigenvectors.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08wcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, alphar, alphai, beta, vl, vr
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, alphar, alphai, beta, vl, vr
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08wcc', a, 'a')
    _b = nag_double_type_check_and_cast('f08wcc', b, 'b')
    _alphar = nag_double_type_check_and_cast('f08wcc', alphar, 'alphar')
    _alphai = nag_double_type_check_and_cast('f08wcc', alphai, 'alphai')
    _beta = nag_double_type_check_and_cast('f08wcc', beta, 'beta')
    _vl = nag_double_type_check_and_cast('f08wcc', vl, 'vl')
    _vr = nag_double_type_check_and_cast('f08wcc', vr, 'vr')
    f08wcc_ctypes(order, jobvl, jobvr, n, _a, pda, _b, pdb, _alphar, _alphai,
                  _beta, _vl, pdvl, _vr, pdvr, fail)

f08wec_ctypes = _library_cl.f08wec
f08wec_ctypes.restype = None
f08wec_ctypes.argtypes = [Nag_OrderType, Nag_ComputeQType, Nag_ComputeZType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f08wec(order, compq, compz, n, ilo, ihi, a, pda, b, pdb, q, pdq, z, pdz, fail):
    """
    Performs an orthogonal reduction of a pair of real general matrices to
    generalized upper Hessenberg form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08wec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, q, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, q, z
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08wec', a, 'a')
    _b = nag_double_type_check_and_cast('f08wec', b, 'b')
    _q = nag_double_type_check_and_cast('f08wec', q, 'q')
    _z = nag_double_type_check_and_cast('f08wec', z, 'z')
    f08wec_ctypes(order, compq, compz, n, ilo, ihi, _a, pda, _b, pdb, _q, pdq, _z, pdz, fail)

f08wfc_ctypes = _library_cl.f08wfc
f08wfc_ctypes.restype = None
f08wfc_ctypes.argtypes = [Nag_OrderType,
                          Nag_ComputeQType,
                          Nag_ComputeZType,
                          Integer,
                          Integer,
                          Integer,
                          POINTER(c_double),
                          Integer,
                          POINTER(c_double),
                          Integer,
                          POINTER(c_double),
                          Integer,
                          POINTER(c_double),
                          Integer,
                          NagError]


def f08wfc(order, compq, compz, n, ilo, ihi, a, pda, b, pdb, q, pdq, z, pdz,
           fail):
    """
    Performs, using BLAS-3, an orthogonal reduction of a pair of
    real general matrices to generalized upper Hessenberg form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08wfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, q, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, q, z
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08wfc', a, 'a')
    _b = nag_double_type_check_and_cast('f08wfc', b, 'b')
    _q = nag_double_type_check_and_cast('f08wfc', q, 'q')
    _z = nag_double_type_check_and_cast('f08wfc', z, 'z')
    f08wfc_ctypes(order, compq, compz, n, ilo, ihi, _a, pda, _b, pdb, _q, pdq,
                  _z, pdz, fail)

f08whc_ctypes = _library_cl.f08whc
f08whc_ctypes.restype = None
f08whc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f08whc(order, job, n, a, pda, b, pdb, ilo, ihi, lscale, rscale, fail):
    """
    Balances a pair of real, square, matrices.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08whc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, ilo, ihi, lscale, rscale
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, lscale, rscale
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ilo, ihi
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08whc', a, 'a')
    _b = nag_double_type_check_and_cast('f08whc', b, 'b')
    _ilo = nag_integer_type_check_and_cast('f08whc', ilo, 'ilo')
    _ihi = nag_integer_type_check_and_cast('f08whc', ihi, 'ihi')
    _lscale = nag_double_type_check_and_cast('f08whc', lscale, 'lscale')
    _rscale = nag_double_type_check_and_cast('f08whc', rscale, 'rscale')
    f08whc_ctypes(order, job, n, _a, pda, _b, pdb, _ilo, _ihi, _lscale, _rscale, fail)

f08wjc_ctypes = _library_cl.f08wjc
f08wjc_ctypes.restype = None
f08wjc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_SideType, Integer, Integer, Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f08wjc(order, job, side, n, ilo, ihi, lscale, rscale, m, v, pdv, fail):
    """
    Transforms eigenvectors of a pair of real balanced matrices to those of
    original matrix pair supplied to f08whc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08wjc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        lscale, rscale, v
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        lscale, rscale, v
    """
    from .util import nag_double_type_check_and_cast
    _lscale = nag_double_type_check_and_cast('f08wjc', lscale, 'lscale')
    _rscale = nag_double_type_check_and_cast('f08wjc', rscale, 'rscale')
    _v = nag_double_type_check_and_cast('f08wjc', v, 'v')
    f08wjc_ctypes(order, job, side, n, ilo, ihi, _lscale, _rscale, m, _v, pdv, fail)

f08wnc_ctypes = _library_cl.f08wnc
f08wnc_ctypes.restype = None
f08wnc_ctypes.argtypes = [Nag_OrderType, Nag_LeftVecsType, Nag_RightVecsType, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(Complex), POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f08wnc(order, jobvl, jobvr, n, a, pda, b, pdb, alpha, beta, vl, pdvl, vr, pdvr, fail):
    """
    Computes, for a complex nonsymmetric matrix pair, the generalized
    eigenvalues, and optionally, the left and/or right generalized
    eigenvectors.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08wnc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, alpha, beta, vl, vr
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, alpha, beta, vl, vr
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08wnc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08wnc', b, 'b')
    _alpha = nag_complex_type_check_and_cast('f08wnc', alpha, 'alpha')
    _beta = nag_complex_type_check_and_cast('f08wnc', beta, 'beta')
    _vl = nag_complex_type_check_and_cast('f08wnc', vl, 'vl')
    _vr = nag_complex_type_check_and_cast('f08wnc', vr, 'vr')
    f08wnc_ctypes(order, jobvl, jobvr, n, _a, pda, _b, pdb, _alpha, _beta, _vl, pdvl, _vr, pdvr, fail)

f08wpc_ctypes = _library_cl.f08wpc
f08wpc_ctypes.restype = None
f08wpc_ctypes.argtypes = [Nag_OrderType, Nag_BalanceType, Nag_LeftVecsType, Nag_RightVecsType, Nag_RCondType, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(Complex), POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f08wpc(order, balanc, jobvl, jobvr, sense, n, a, pda, b, pdb, alpha, beta, vl, pdvl, vr, pdvr, ilo, ihi, lscale, rscale, abnrm, bbnrm, rconde, rcondv, fail):
    """
    Computes, for a complex nonsymmetric matrix pair, the generalized
    eigenvalues, and optionally, the left and/or right generalized
    eigenvectors; also, optionally, the balancing transformation, the
    reciprocal condition numbers for the eigenvalues and for the right
    eigenvectors.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08wpc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, alpha, beta, vl, vr, ilo, ihi, lscale, rscale, abnrm, bbnrm, rconde, rcondv
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        lscale, rscale, abnrm, bbnrm, rconde, rcondv
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ilo, ihi
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, alpha, beta, vl, vr
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08wpc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08wpc', b, 'b')
    _alpha = nag_complex_type_check_and_cast('f08wpc', alpha, 'alpha')
    _beta = nag_complex_type_check_and_cast('f08wpc', beta, 'beta')
    _vl = nag_complex_type_check_and_cast('f08wpc', vl, 'vl')
    _vr = nag_complex_type_check_and_cast('f08wpc', vr, 'vr')
    _ilo = nag_integer_type_check_and_cast('f08wpc', ilo, 'ilo')
    _ihi = nag_integer_type_check_and_cast('f08wpc', ihi, 'ihi')
    _lscale = nag_double_type_check_and_cast('f08wpc', lscale, 'lscale')
    _rscale = nag_double_type_check_and_cast('f08wpc', rscale, 'rscale')
    _abnrm = nag_double_type_check_and_cast('f08wpc', abnrm, 'abnrm')
    _bbnrm = nag_double_type_check_and_cast('f08wpc', bbnrm, 'bbnrm')
    _rconde = nag_double_type_check_and_cast('f08wpc', rconde, 'rconde')
    _rcondv = nag_double_type_check_and_cast('f08wpc', rcondv, 'rcondv')
    f08wpc_ctypes(order, balanc, jobvl, jobvr, sense, n, _a, pda, _b, pdb, _alpha, _beta, _vl, pdvl, _vr, pdvr, _ilo, _ihi, _lscale, _rscale, _abnrm, _bbnrm, _rconde, _rcondv, fail)

f08wqc_ctypes = _library_cl.f08wqc
f08wqc_ctypes.restype = None
f08wqc_ctypes.argtypes = [Nag_OrderType,
                          Nag_LeftVecsType,
                          Nag_RightVecsType,
                          Integer,
                          POINTER(Complex),
                          Integer,
                          POINTER(Complex),
                          Integer,
                          POINTER(Complex),
                          POINTER(Complex),
                          POINTER(Complex),
                          Integer,
                          POINTER(Complex),
                          Integer,
                          POINTER(NagError)]


def f08wqc(order, jobvl, jobvr, n, a, pda, b, pdb, alpha, beta, vl, pdvl, vr,
           pdvr, fail):
    """
    Computes, for a complex nonsymmetric matrix pair, using BLAS-3, the generalized eigenvalues,
    and optionally, the left and/or right generalized eigenvectors.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08wqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, alpha, beta, vl, vr
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, alpha, beta, vl, vr
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08wqc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08wqc', b, 'b')
    _alpha = nag_complex_type_check_and_cast('f08wqc', alpha, 'alpha')
    _beta = nag_complex_type_check_and_cast('f08wqc', beta, 'beta')
    _vl = nag_complex_type_check_and_cast('f08wqc', vl, 'vl')
    _vr = nag_complex_type_check_and_cast('f08wqc', vr, 'vr')
    f08wqc_ctypes(order, jobvl, jobvr, n, _a, pda, _b, pdb, _alpha, _beta,
                  _vl, pdvl, _vr, pdvr, fail)

f08wsc_ctypes = _library_cl.f08wsc
f08wsc_ctypes.restype = None
f08wsc_ctypes.argtypes = [Nag_OrderType, Nag_ComputeQType, Nag_ComputeZType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f08wsc(order, compq, compz, n, ilo, ihi, a, pda, b, pdb, q, pdq, z, pdz, fail):
    """
    Performs a unitary reduction of a pair of complex general matrices to
    generalized upper Hessenberg form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08wsc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, q, z
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, q, z
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08wsc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08wsc', b, 'b')
    _q = nag_complex_type_check_and_cast('f08wsc', q, 'q')
    _z = nag_complex_type_check_and_cast('f08wsc', z, 'z')
    f08wsc_ctypes(order, compq, compz, n, ilo, ihi, _a, pda, _b, pdb, _q, pdq, _z, pdz, fail)

f08wtc_ctypes = _library_cl.f08wtc
f08wtc_ctypes.restype = None
f08wtc_ctypes.argtypes = [Nag_OrderType,
                          Nag_ComputeQType,
                          Nag_ComputeZType,
                          Integer,
                          Integer,
                          Integer,
                          POINTER(Complex),
                          Integer,
                          POINTER(Complex),
                          Integer,
                          POINTER(Complex),
                          Integer,
                          POINTER(Complex),
                          Integer,
                          POINTER(NagError)]


def f08wtc(order, compq, compz, n, ilo, ihi, a, pda, b, pdb, q, pdq, z, pdz,
           fail):
    """
    Performs, using BLAS-3, a unitary reduction of a pair of complex general matrices
    to generalized upper Hessenberg form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08wtc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, q, z
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, q, z
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08wtc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08wtc', b, 'b')
    _q = nag_complex_type_check_and_cast('f08wtc', q, 'q')
    _z = nag_complex_type_check_and_cast('f08wtc', z, 'z')
    f08wtc_ctypes(order, compq, compz, n, ilo, ihi, _a, pda, _b, pdb, _q, pdq,
                  _z, pdz, fail)

f08wvc_ctypes = _library_cl.f08wvc
f08wvc_ctypes.restype = None
f08wvc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f08wvc(order, job, n, a, pda, b, pdb, ilo, ihi, lscale, rscale, fail):
    """
    Balances a pair of complex, square, matrices.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08wvc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, ilo, ihi, lscale, rscale
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        lscale, rscale
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ilo, ihi
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08wvc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08wvc', b, 'b')
    _ilo = nag_integer_type_check_and_cast('f08wvc', ilo, 'ilo')
    _ihi = nag_integer_type_check_and_cast('f08wvc', ihi, 'ihi')
    _lscale = nag_double_type_check_and_cast('f08wvc', lscale, 'lscale')
    _rscale = nag_double_type_check_and_cast('f08wvc', rscale, 'rscale')
    f08wvc_ctypes(order, job, n, _a, pda, _b, pdb, _ilo, _ihi, _lscale, _rscale, fail)

f08wwc_ctypes = _library_cl.f08wwc
f08wwc_ctypes.restype = None
f08wwc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_SideType, Integer, Integer, Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f08wwc(order, job, side, n, ilo, ihi, lscale, rscale, m, v, pdv, fail):
    """
    Transforms eigenvectors of a pair of complex balanced matrices to those
    of original matrix pair supplied to f08wvc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08wwc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        lscale, rscale, v
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        lscale, rscale
    TypeError will be raised when the following arguments are not of data type
    Complex:
        v
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _lscale = nag_double_type_check_and_cast('f08wwc', lscale, 'lscale')
    _rscale = nag_double_type_check_and_cast('f08wwc', rscale, 'rscale')
    _v = nag_complex_type_check_and_cast('f08wwc', v, 'v')
    f08wwc_ctypes(order, job, side, n, ilo, ihi, _lscale, _rscale, m, _v, pdv, fail)

f08xac_ctypes = _library_cl.f08xac
f08xac_ctypes.restype = None
f08xac_ctypes.argtypes = [Nag_OrderType, Nag_LeftVecsType, Nag_RightVecsType, Nag_SortEigValsType, NAG_F08XAC_SELCTG, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f08xac(order, jobvsl, jobvsr, sort, selctg, n, a, pda, b, pdb, sdim, alphar, alphai, beta, vsl, pdvsl, vsr, pdvsr, fail):
    """
    Computes, for a real nonsymmetric matrix pair, the generalized
    eigenvalues, the generalized real Schur form and, optionally, the left
    and/or right matrices of Schur vectors.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08xac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, sdim, alphar, alphai, beta, vsl, vsr
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, alphar, alphai, beta, vsl, vsr
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        sdim
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08xac', a, 'a')
    _b = nag_double_type_check_and_cast('f08xac', b, 'b')
    _sdim = nag_integer_type_check_and_cast('f08xac', sdim, 'sdim')
    _alphar = nag_double_type_check_and_cast('f08xac', alphar, 'alphar')
    _alphai = nag_double_type_check_and_cast('f08xac', alphai, 'alphai')
    _beta = nag_double_type_check_and_cast('f08xac', beta, 'beta')
    _vsl = nag_double_type_check_and_cast('f08xac', vsl, 'vsl')
    _vsr = nag_double_type_check_and_cast('f08xac', vsr, 'vsr')
    f08xac_ctypes(order, jobvsl, jobvsr, sort, selctg, n, _a, pda, _b, pdb, _sdim, _alphar, _alphai, _beta, _vsl, pdvsl, _vsr, pdvsr, fail)

f08xbc_ctypes = _library_cl.f08xbc
f08xbc_ctypes.restype = None
f08xbc_ctypes.argtypes = [Nag_OrderType, Nag_LeftVecsType, Nag_RightVecsType, Nag_SortEigValsType, NAG_F08XBC_SELCTG, Nag_RCondType, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f08xbc(order, jobvsl, jobvsr, sort, selctg, sense, n, a, pda, b, pdb, sdim, alphar, alphai, beta, vsl, pdvsl, vsr, pdvsr, rconde, rcondv, fail):
    """
    Computes, for a real nonsymmetric matrix pair, the generalized
    eigenvalues, the generalized real Schur form and, optionally, the left
    and/or right matrices of Schur vectors; also, optionally, computes
    reciprocal condition numbers for selected eigenvalues.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08xbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, sdim, alphar, alphai, beta, vsl, vsr, rconde, rcondv
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, alphar, alphai, beta, vsl, vsr, rconde, rcondv
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        sdim
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08xbc', a, 'a')
    _b = nag_double_type_check_and_cast('f08xbc', b, 'b')
    _sdim = nag_integer_type_check_and_cast('f08xbc', sdim, 'sdim')
    _alphar = nag_double_type_check_and_cast('f08xbc', alphar, 'alphar')
    _alphai = nag_double_type_check_and_cast('f08xbc', alphai, 'alphai')
    _beta = nag_double_type_check_and_cast('f08xbc', beta, 'beta')
    _vsl = nag_double_type_check_and_cast('f08xbc', vsl, 'vsl')
    _vsr = nag_double_type_check_and_cast('f08xbc', vsr, 'vsr')
    _rconde = nag_double_type_check_and_cast('f08xbc', rconde, 'rconde')
    _rcondv = nag_double_type_check_and_cast('f08xbc', rcondv, 'rcondv')
    f08xbc_ctypes(order, jobvsl, jobvsr, sort, selctg, sense, n, _a, pda, _b, pdb, _sdim, _alphar, _alphai, _beta, _vsl, pdvsl, _vsr, pdvsr, _rconde, _rcondv, fail)

f08xcc_ctypes = _library_cl.f08xcc
f08xcc_ctypes.restype = None
f08xcc_ctypes.argtypes = [Nag_OrderType,
                          Nag_LeftVecsType,
                          Nag_RightVecsType,
                          Nag_SortEigValsType,
                          NAG_F08XCC_SELCTG,
                          Integer,
                          POINTER(c_double),
                          Integer,
                          POINTER(c_double),
                          Integer,
                          POINTER(Integer),
                          POINTER(c_double),
                          POINTER(c_double),
                          POINTER(c_double),
                          POINTER(c_double),
                          Integer,
                          POINTER(c_double),
                          Integer,
                          POINTER(NagError)]


def f08xcc(order, jobvsl, jobvsr, sort, selctg, n, a, pda, b, pdb, sdim,
           alphar, alphai, beta, vsl, pdvsl, vsr, pdvsr, fail):
    """
    Computes, for a real nonsymmetric matrix pair, using BLAS-3, the generalized eigenvalues,
    the generalized real Schur form and, optionally, the left and/or right matrices
    of Schur vectors.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08xcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, sdim, alphar, alphai, beta, rsl, rsr
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, alphar, alphai, beta, rsl, rsr
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        sdim
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast

    _a = nag_double_type_check_and_cast('f08xcc', a, 'a')
    _b = nag_double_type_check_and_cast('f08xcc', b, 'b')
    _sdim = nag_integer_type_check_and_cast('f08xcc', sdim, 'sdim')
    _alphar = nag_double_type_check_and_cast('f08xcc', alphar, 'alphar')
    _alphai = nag_double_type_check_and_cast('f08xcc', alphai, 'alphai')
    _beta = nag_double_type_check_and_cast('f08xcc', beta, 'beta')
    _vsl = nag_double_type_check_and_cast('f08xcc', vsl, 'vsl')
    _vsr = nag_double_type_check_and_cast('f08xcc', vsr, 'vsr')

    f08xcc_ctypes(order, jobvsl, jobvsr, sort, selctg, n, _a, pda, _b, pdb,
                  _sdim, _alphar, _alphai, _beta, _vsl, pdvsl, _vsr, pdvsr,
                  fail)

f08xec_ctypes = _library_cl.f08xec
f08xec_ctypes.restype = None
f08xec_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_ComputeQType, Nag_ComputeZType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f08xec(order, job, compq, compz, n, ilo, ihi, a, pda, b, pdb, alphar, alphai, beta, q, pdq, z, pdz, fail):
    """
    Computes eigenvalues and generalized Schur factorization of real
    generalized upper Hessenberg form reduced from a pair of real general
    matrices.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08xec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, alphar, alphai, beta, q, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, alphar, alphai, beta, q, z
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08xec', a, 'a')
    _b = nag_double_type_check_and_cast('f08xec', b, 'b')
    _alphar = nag_double_type_check_and_cast('f08xec', alphar, 'alphar')
    _alphai = nag_double_type_check_and_cast('f08xec', alphai, 'alphai')
    _beta = nag_double_type_check_and_cast('f08xec', beta, 'beta')
    _q = nag_double_type_check_and_cast('f08xec', q, 'q')
    _z = nag_double_type_check_and_cast('f08xec', z, 'z')
    f08xec_ctypes(order, job, compq, compz, n, ilo, ihi, _a, pda, _b, pdb, _alphar, _alphai, _beta, _q, pdq, _z, pdz, fail)

f08xnc_ctypes = _library_cl.f08xnc
f08xnc_ctypes.restype = None
f08xnc_ctypes.argtypes = [Nag_OrderType, Nag_LeftVecsType, Nag_RightVecsType, Nag_SortEigValsType, NAG_F08XNC_SELCTG, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Integer), POINTER(Complex), POINTER(Complex), POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f08xnc(order, jobvsl, jobvsr, sort, selctg, n, a, pda, b, pdb, sdim, alpha, beta, vsl, pdvsl, vsr, pdvsr, fail):
    """
    Computes, for a complex nonsymmetric matrix pair, the generalized
    eigenvalues, the generalized complex Schur form and, optionally, the
    left and/or right matrices of Schur vectors.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08xnc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, sdim, alpha, beta, vsl, vsr
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        sdim
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, alpha, beta, vsl, vsr
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08xnc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08xnc', b, 'b')
    _sdim = nag_integer_type_check_and_cast('f08xnc', sdim, 'sdim')
    _alpha = nag_complex_type_check_and_cast('f08xnc', alpha, 'alpha')
    _beta = nag_complex_type_check_and_cast('f08xnc', beta, 'beta')
    _vsl = nag_complex_type_check_and_cast('f08xnc', vsl, 'vsl')
    _vsr = nag_complex_type_check_and_cast('f08xnc', vsr, 'vsr')
    f08xnc_ctypes(order, jobvsl, jobvsr, sort, selctg, n, _a, pda, _b, pdb, _sdim, _alpha, _beta, _vsl, pdvsl, _vsr, pdvsr, fail)

f08xpc_ctypes = _library_cl.f08xpc
f08xpc_ctypes.restype = None
f08xpc_ctypes.argtypes = [Nag_OrderType, Nag_LeftVecsType, Nag_RightVecsType, Nag_SortEigValsType, NAG_F08XPC_SELCTG, Nag_RCondType, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Integer), POINTER(Complex), POINTER(Complex), POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f08xpc(order, jobvsl, jobvsr, sort, selctg, sense, n, a, pda, b, pdb, sdim, alpha, beta, vsl, pdvsl, vsr, pdvsr, rconde, rcondv, fail):
    """
    Computes, for a complex nonsymmetric matrix pair, the generalized
    eigenvalues, the generalized complex Schur form and, optionally, the
    left and/or right matrices of Schur vectors; also, optionally, computes
    reciprocal condition numbers for selected eigenvalues.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08xpc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, sdim, alpha, beta, vsl, vsr, rconde, rcondv
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rconde, rcondv
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        sdim
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, alpha, beta, vsl, vsr
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08xpc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08xpc', b, 'b')
    _sdim = nag_integer_type_check_and_cast('f08xpc', sdim, 'sdim')
    _alpha = nag_complex_type_check_and_cast('f08xpc', alpha, 'alpha')
    _beta = nag_complex_type_check_and_cast('f08xpc', beta, 'beta')
    _vsl = nag_complex_type_check_and_cast('f08xpc', vsl, 'vsl')
    _vsr = nag_complex_type_check_and_cast('f08xpc', vsr, 'vsr')
    _rconde = nag_double_type_check_and_cast('f08xpc', rconde, 'rconde')
    _rcondv = nag_double_type_check_and_cast('f08xpc', rcondv, 'rcondv')
    f08xpc_ctypes(order, jobvsl, jobvsr, sort, selctg, sense, n, _a, pda, _b, pdb, _sdim, _alpha, _beta, _vsl, pdvsl, _vsr, pdvsr, _rconde, _rcondv, fail)

f08xqc_ctypes = _library_cl.f08xqc
f08xqc_ctypes.restype = None
f08xqc_ctypes.argtypes = [Nag_OrderType,
                          Nag_LeftVecsType,
                          Nag_RightVecsType,
                          Nag_SortEigValsType,
                          NAG_F08XQC_SELCTG,
                          Integer,
                          POINTER(Complex),
                          Integer,
                          POINTER(Complex),
                          Integer,
                          POINTER(Integer),
                          POINTER(Complex),
                          POINTER(Complex),
                          POINTER(Complex),
                          Integer,
                          POINTER(Complex),
                          Integer,
                          POINTER(NagError)]


def f08xqc(order, jobvsl, jobvsr, sort, selctg, n, a, pda, b, pdb, sdim,
           alpha, beta, vsl, pdvsl, vsr, pdvsr, fail):
    """
    Computes, for a complex nonsymmetric matrix pair, using BLAS-3, the generalized eigenvalues,
    the generalized complex Schur form and, optionally, the left and/or right matrices
    of Schur vectors.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08xqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, sdim, alpha, beta, vsl, vsr
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        sdim
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, alpha, beta, vsl, vsr
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08xqc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08xqc', b, 'b')
    _sdim = nag_integer_type_check_and_cast('f08xqc', sdim, 'sdim')
    _alpha = nag_complex_type_check_and_cast('f08xqc', alpha, 'alpha')
    _beta = nag_complex_type_check_and_cast('f08xqc', beta, 'beta')
    _vsl = nag_complex_type_check_and_cast('f08xqc', vsl, 'vsl')
    _vsr = nag_complex_type_check_and_cast('f08xqc', vsr, 'vsr')
    f08xqc_ctypes(order, jobvsl, jobvsr, sort, selctg, n, _a, pda, _b, pdb,
                  _sdim, _alpha, _beta, _vsl, pdvsl, _vsr, pdvsr, fail)

f08xsc_ctypes = _library_cl.f08xsc
f08xsc_ctypes.restype = None
f08xsc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_ComputeQType, Nag_ComputeZType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(Complex), POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f08xsc(order, job, compq, compz, n, ilo, ihi, a, pda, b, pdb, alpha, beta, q, pdq, z, pdz, fail):
    """
    Eigenvalues and generalized Schur factorization of complex generalized
    upper Hessenberg form reduced from a pair of complex, square, matrices.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08xsc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, alpha, beta, q, z
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, alpha, beta, q, z
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08xsc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08xsc', b, 'b')
    _alpha = nag_complex_type_check_and_cast('f08xsc', alpha, 'alpha')
    _beta = nag_complex_type_check_and_cast('f08xsc', beta, 'beta')
    _q = nag_complex_type_check_and_cast('f08xsc', q, 'q')
    _z = nag_complex_type_check_and_cast('f08xsc', z, 'z')
    f08xsc_ctypes(order, job, compq, compz, n, ilo, ihi, _a, pda, _b, pdb, _alpha, _beta, _q, pdq, _z, pdz, fail)

f08yec_ctypes = _library_cl.f08yec
f08yec_ctypes.restype = None
f08yec_ctypes.argtypes = [Nag_OrderType, Nag_ComputeUType, Nag_ComputeVType, Nag_ComputeQType, Integer, Integer, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, c_double, c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(NagError)]


def f08yec(order, jobu, jobv, jobq, m, p, n, k, l, a, pda, b, pdb, tola, tolb, alpha, beta, u, pdu, v, pdv, q, pdq, ncycle, fail):
    """
    Computes the generalized singular value decomposition of a real upper
    triangular (or trapezoidal) matrix pair.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08yec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, alpha, beta, u, v, q, ncycle
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, alpha, beta, u, v, q
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ncycle
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08yec', a, 'a')
    _b = nag_double_type_check_and_cast('f08yec', b, 'b')
    _alpha = nag_double_type_check_and_cast('f08yec', alpha, 'alpha')
    _beta = nag_double_type_check_and_cast('f08yec', beta, 'beta')
    _u = nag_double_type_check_and_cast('f08yec', u, 'u')
    _v = nag_double_type_check_and_cast('f08yec', v, 'v')
    _q = nag_double_type_check_and_cast('f08yec', q, 'q')
    _ncycle = nag_integer_type_check_and_cast('f08yec', ncycle, 'ncycle')
    f08yec_ctypes(order, jobu, jobv, jobq, m, p, n, k, l, _a, pda, _b, pdb, tola, tolb, _alpha, _beta, _u, pdu, _v, pdv, _q, pdq, _ncycle, fail)

f08yfc_ctypes = _library_cl.f08yfc
f08yfc_ctypes.restype = None
f08yfc_ctypes.argtypes = [Nag_OrderType, Nag_Boolean, Nag_Boolean, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(Integer), POINTER(NagError)]


def f08yfc(order, wantq, wantz, n, a, pda, b, pdb, q, pdq, z, pdz, ifst, ilst, fail):
    """
    Reorders the generalized real Schur decomposition of a real matrix pair
    using an orthogonal equivalence transformation.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08yfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, q, z, ifst, ilst
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, q, z
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ifst, ilst
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08yfc', a, 'a')
    _b = nag_double_type_check_and_cast('f08yfc', b, 'b')
    _q = nag_double_type_check_and_cast('f08yfc', q, 'q')
    _z = nag_double_type_check_and_cast('f08yfc', z, 'z')
    _ifst = nag_integer_type_check_and_cast('f08yfc', ifst, 'ifst')
    _ilst = nag_integer_type_check_and_cast('f08yfc', ilst, 'ilst')
    f08yfc_ctypes(order, wantq, wantz, n, _a, pda, _b, pdb, _q, pdq, _z, pdz, _ifst, _ilst, fail)

f08ygc_ctypes = _library_cl.f08ygc
f08ygc_ctypes.restype = None
f08ygc_ctypes.argtypes = [Nag_OrderType, Integer, Nag_Boolean, Nag_Boolean, POINTER(Nag_Boolean), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f08ygc(order, ijob, wantq, wantz, select, n, a, pda, b, pdb, alphar, alphai, beta, q, pdq, z, pdz, m, pl, pr, dif, fail):
    """
    Reorders the generalized real Schur decomposition of a real matrix pair
    using an orthogonal equivalence transformation, computes the generalized
    eigenvalues of the reordered pair and, optionally, computes the
    estimates of reciprocal condition numbers for eigenvalues and
    eigenspaces.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08ygc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        select, a, b, alphar, alphai, beta, q, z, m, pl, pr, dif
    TypeError will be raised when the following arguments are not of data type
    Nag_Boolean:
        select
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, alphar, alphai, beta, q, z, pl, pr, dif
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m
    """
    from .util import nag_enum_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _select = nag_enum_type_check_and_cast('f08ygc', select, 'select', Nag_Boolean)
    _a = nag_double_type_check_and_cast('f08ygc', a, 'a')
    _b = nag_double_type_check_and_cast('f08ygc', b, 'b')
    _alphar = nag_double_type_check_and_cast('f08ygc', alphar, 'alphar')
    _alphai = nag_double_type_check_and_cast('f08ygc', alphai, 'alphai')
    _beta = nag_double_type_check_and_cast('f08ygc', beta, 'beta')
    _q = nag_double_type_check_and_cast('f08ygc', q, 'q')
    _z = nag_double_type_check_and_cast('f08ygc', z, 'z')
    _m = nag_integer_type_check_and_cast('f08ygc', m, 'm')
    _pl = nag_double_type_check_and_cast('f08ygc', pl, 'pl')
    _pr = nag_double_type_check_and_cast('f08ygc', pr, 'pr')
    _dif = nag_double_type_check_and_cast('f08ygc', dif, 'dif')
    f08ygc_ctypes(order, ijob, wantq, wantz, _select, n, _a, pda, _b, pdb, _alphar, _alphai, _beta, _q, pdq, _z, pdz, _m, _pl, _pr, _dif, fail)

f08yhc_ctypes = _library_cl.f08yhc
f08yhc_ctypes.restype = None
f08yhc_ctypes.argtypes = [Nag_OrderType, Nag_TransType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f08yhc(order, trans, ijob, m, n, a, pda, b, pdb, c, pdc, d, pdd, e, pde, f, pdf, scale, dif, fail):
    """
    Solves the real-valued, generalized, quasi-trangular, Sylvester
    equation.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08yhc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, c, d, e, f, scale, dif
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, c, d, e, f, scale, dif
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08yhc', a, 'a')
    _b = nag_double_type_check_and_cast('f08yhc', b, 'b')
    _c = nag_double_type_check_and_cast('f08yhc', c, 'c')
    _d = nag_double_type_check_and_cast('f08yhc', d, 'd')
    _e = nag_double_type_check_and_cast('f08yhc', e, 'e')
    _f = nag_double_type_check_and_cast('f08yhc', f, 'f')
    _scale = nag_double_type_check_and_cast('f08yhc', scale, 'scale')
    _dif = nag_double_type_check_and_cast('f08yhc', dif, 'dif')
    f08yhc_ctypes(order, trans, ijob, m, n, _a, pda, _b, pdb, _c, pdc, _d, pdd, _e, pde, _f, pdf, _scale, _dif, fail)

f08ykc_ctypes = _library_cl.f08ykc
f08ykc_ctypes.restype = None
f08ykc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_HowManyType, POINTER(Nag_Boolean), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, Integer, POINTER(Integer), POINTER(NagError)]


def f08ykc(order, side, howmany, select, n, a, pda, b, pdb, vl, pdvl, vr, pdvr, mm, m, fail):
    """
    Computes right and left generalized eigenvectors of the matrix pair
    (A,B)) which is assumed to be in generalized upper Schur form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08ykc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        select, a, b, vl, vr, m
    TypeError will be raised when the following arguments are not of data type
    Nag_Boolean:
        select
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, vl, vr
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m
    """
    from .util import nag_enum_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _select = nag_enum_type_check_and_cast('f08ykc', select, 'select', Nag_Boolean)
    _a = nag_double_type_check_and_cast('f08ykc', a, 'a')
    _b = nag_double_type_check_and_cast('f08ykc', b, 'b')
    _vl = nag_double_type_check_and_cast('f08ykc', vl, 'vl')
    _vr = nag_double_type_check_and_cast('f08ykc', vr, 'vr')
    _m = nag_integer_type_check_and_cast('f08ykc', m, 'm')
    f08ykc_ctypes(order, side, howmany, _select, n, _a, pda, _b, pdb, _vl, pdvl, _vr, pdvr, mm, _m, fail)

f08ylc_ctypes = _library_cl.f08ylc
f08ylc_ctypes.restype = None
f08ylc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_HowManyType, POINTER(Nag_Boolean), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(Integer), POINTER(NagError)]


def f08ylc(order, job, howmny, select, n, a, pda, b, pdb, vl, pdvl, vr, pdvr, s, dif, mm, m, fail):
    """
    Estimates reciprocal condition numbers for specified eigenvalues and/or
    eigenvectors of a real matrix pair in generalized real Schur canonical
    form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08ylc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        select, a, b, vl, vr, s, dif, m
    TypeError will be raised when the following arguments are not of data type
    Nag_Boolean:
        select
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, vl, vr, s, dif
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m
    """
    from .util import nag_enum_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _select = nag_enum_type_check_and_cast('f08ylc', select, 'select', Nag_Boolean)
    _a = nag_double_type_check_and_cast('f08ylc', a, 'a')
    _b = nag_double_type_check_and_cast('f08ylc', b, 'b')
    _vl = nag_double_type_check_and_cast('f08ylc', vl, 'vl')
    _vr = nag_double_type_check_and_cast('f08ylc', vr, 'vr')
    _s = nag_double_type_check_and_cast('f08ylc', s, 's')
    _dif = nag_double_type_check_and_cast('f08ylc', dif, 'dif')
    _m = nag_integer_type_check_and_cast('f08ylc', m, 'm')
    f08ylc_ctypes(order, job, howmny, _select, n, _a, pda, _b, pdb, _vl, pdvl, _vr, pdvr, _s, _dif, mm, _m, fail)

f08ysc_ctypes = _library_cl.f08ysc
f08ysc_ctypes.restype = None
f08ysc_ctypes.argtypes = [Nag_OrderType, Nag_ComputeUType, Nag_ComputeVType, Nag_ComputeQType, Integer, Integer, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, c_double, c_double, POINTER(c_double), POINTER(c_double), POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Integer), POINTER(NagError)]


def f08ysc(order, jobu, jobv, jobq, m, p, n, k, l, a, pda, b, pdb, tola, tolb, alpha, beta, u, pdu, v, pdv, q, pdq, ncycle, fail):
    """
    Computes the generalized singular value decomposition of a complex upper
    triangular (or trapezoidal) matrix pair.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08ysc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, alpha, beta, u, v, q, ncycle
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        alpha, beta
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ncycle
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, u, v, q
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08ysc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08ysc', b, 'b')
    _alpha = nag_double_type_check_and_cast('f08ysc', alpha, 'alpha')
    _beta = nag_double_type_check_and_cast('f08ysc', beta, 'beta')
    _u = nag_complex_type_check_and_cast('f08ysc', u, 'u')
    _v = nag_complex_type_check_and_cast('f08ysc', v, 'v')
    _q = nag_complex_type_check_and_cast('f08ysc', q, 'q')
    _ncycle = nag_integer_type_check_and_cast('f08ysc', ncycle, 'ncycle')
    f08ysc_ctypes(order, jobu, jobv, jobq, m, p, n, k, l, _a, pda, _b, pdb, tola, tolb, _alpha, _beta, _u, pdu, _v, pdv, _q, pdq, _ncycle, fail)

f08ytc_ctypes = _library_cl.f08ytc
f08ytc_ctypes.restype = None
f08ytc_ctypes.argtypes = [Nag_OrderType, Nag_Boolean, Nag_Boolean, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, Integer, POINTER(Integer), POINTER(NagError)]


def f08ytc(order, wantq, wantz, n, a, pda, b, pdb, q, pdq, z, pdz, ifst, ilst, fail):
    """
    Reorders the generalized Schur decomposition of a complex matrix pair
    using an unitary equivalence transformation.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08ytc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, q, z, ilst
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ilst
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, q, z
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08ytc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08ytc', b, 'b')
    _q = nag_complex_type_check_and_cast('f08ytc', q, 'q')
    _z = nag_complex_type_check_and_cast('f08ytc', z, 'z')
    _ilst = nag_integer_type_check_and_cast('f08ytc', ilst, 'ilst')
    f08ytc_ctypes(order, wantq, wantz, n, _a, pda, _b, pdb, _q, pdq, _z, pdz, ifst, _ilst, fail)

f08yuc_ctypes = _library_cl.f08yuc
f08yuc_ctypes.restype = None
f08yuc_ctypes.argtypes = [Nag_OrderType, Integer, Nag_Boolean, Nag_Boolean, POINTER(Nag_Boolean), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(Complex), POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f08yuc(order, ijob, wantq, wantz, select, n, a, pda, b, pdb, alpha, beta, q, pdq, z, pdz, m, pl, pr, dif, fail):
    """
    Reorders the generalized Schur decomposition of a complex matrix pair
    using an unitary equivalence transformation, computes the generalized
    eigenvalues of the reordered pair and, optionally, computes the
    estimates of reciprocal condition numbers for eigenvalues and
    eigenspaces.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08yuc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        select, a, b, alpha, beta, q, z, m, pl, pr, dif
    TypeError will be raised when the following arguments are not of data type
    Nag_Boolean:
        select
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        pl, pr, dif
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, alpha, beta, q, z
    """
    from .util import nag_enum_type_check_and_cast
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _select = nag_enum_type_check_and_cast('f08yuc', select, 'select', Nag_Boolean)
    _a = nag_complex_type_check_and_cast('f08yuc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08yuc', b, 'b')
    _alpha = nag_complex_type_check_and_cast('f08yuc', alpha, 'alpha')
    _beta = nag_complex_type_check_and_cast('f08yuc', beta, 'beta')
    _q = nag_complex_type_check_and_cast('f08yuc', q, 'q')
    _z = nag_complex_type_check_and_cast('f08yuc', z, 'z')
    _m = nag_integer_type_check_and_cast('f08yuc', m, 'm')
    _pl = nag_double_type_check_and_cast('f08yuc', pl, 'pl')
    _pr = nag_double_type_check_and_cast('f08yuc', pr, 'pr')
    _dif = nag_double_type_check_and_cast('f08yuc', dif, 'dif')
    f08yuc_ctypes(order, ijob, wantq, wantz, _select, n, _a, pda, _b, pdb, _alpha, _beta, _q, pdq, _z, pdz, _m, _pl, _pr, _dif, fail)

f08yvc_ctypes = _library_cl.f08yvc
f08yvc_ctypes.restype = None
f08yvc_ctypes.argtypes = [Nag_OrderType, Nag_TransType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f08yvc(order, trans, ijob, m, n, a, pda, b, pdb, c, pdc, d, pdd, e, pde, f, pdf, scale, dif, fail):
    """
    Solves the complex generalized Sylvester equation.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08yvc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, c, d, e, f, scale, dif
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        scale, dif
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, c, d, e, f
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08yvc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08yvc', b, 'b')
    _c = nag_complex_type_check_and_cast('f08yvc', c, 'c')
    _d = nag_complex_type_check_and_cast('f08yvc', d, 'd')
    _e = nag_complex_type_check_and_cast('f08yvc', e, 'e')
    _f = nag_complex_type_check_and_cast('f08yvc', f, 'f')
    _scale = nag_double_type_check_and_cast('f08yvc', scale, 'scale')
    _dif = nag_double_type_check_and_cast('f08yvc', dif, 'dif')
    f08yvc_ctypes(order, trans, ijob, m, n, _a, pda, _b, pdb, _c, pdc, _d, pdd, _e, pde, _f, pdf, _scale, _dif, fail)

f08yxc_ctypes = _library_cl.f08yxc
f08yxc_ctypes.restype = None
f08yxc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_HowManyType, POINTER(Nag_Boolean), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, Integer, POINTER(Integer), POINTER(NagError)]


def f08yxc(order, side, howmany, select, n, a, pda, b, pdb, vl, pdvl, vr, pdvr, mm, m, fail):
    """
    Computes left and right eigenvectors of a pair of complex upper
    triangular matrices.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08yxc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        select, a, b, vl, vr, m
    TypeError will be raised when the following arguments are not of data type
    Nag_Boolean:
        select
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, vl, vr
    """
    from .util import nag_enum_type_check_and_cast
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _select = nag_enum_type_check_and_cast('f08yxc', select, 'select', Nag_Boolean)
    _a = nag_complex_type_check_and_cast('f08yxc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08yxc', b, 'b')
    _vl = nag_complex_type_check_and_cast('f08yxc', vl, 'vl')
    _vr = nag_complex_type_check_and_cast('f08yxc', vr, 'vr')
    _m = nag_integer_type_check_and_cast('f08yxc', m, 'm')
    f08yxc_ctypes(order, side, howmany, _select, n, _a, pda, _b, pdb, _vl, pdvl, _vr, pdvr, mm, _m, fail)

f08yyc_ctypes = _library_cl.f08yyc
f08yyc_ctypes.restype = None
f08yyc_ctypes.argtypes = [Nag_OrderType, Nag_JobType, Nag_HowManyType, POINTER(Nag_Boolean), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(Integer), POINTER(NagError)]


def f08yyc(order, job, howmany, select, n, a, pda, b, pdb, vl, pdvl, vr, pdvr, s, dif, mm, m, fail):
    """
    Estimates reciprocal condition numbers for specified eigenvalues and/or
    eigenvectors of a complex matrix pair in generalized Schur canonical
    form.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08yyc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        select, a, b, vl, vr, s, dif, m
    TypeError will be raised when the following arguments are not of data type
    Nag_Boolean:
        select
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        s, dif
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, vl, vr
    """
    from .util import nag_enum_type_check_and_cast
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _select = nag_enum_type_check_and_cast('f08yyc', select, 'select', Nag_Boolean)
    _a = nag_complex_type_check_and_cast('f08yyc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08yyc', b, 'b')
    _vl = nag_complex_type_check_and_cast('f08yyc', vl, 'vl')
    _vr = nag_complex_type_check_and_cast('f08yyc', vr, 'vr')
    _s = nag_double_type_check_and_cast('f08yyc', s, 's')
    _dif = nag_double_type_check_and_cast('f08yyc', dif, 'dif')
    _m = nag_integer_type_check_and_cast('f08yyc', m, 'm')
    f08yyc_ctypes(order, job, howmany, _select, n, _a, pda, _b, pdb, _vl, pdvl, _vr, pdvr, _s, _dif, mm, _m, fail)

f08zac_ctypes = _library_cl.f08zac
f08zac_ctypes.restype = None
f08zac_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f08zac(order, m, n, p, a, pda, b, pdb, c, d, x, fail):
    """
    Solves the real linear equality-constrained least squares (LSE) problem.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08zac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, c, d, x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, c, d, x
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08zac', a, 'a')
    _b = nag_double_type_check_and_cast('f08zac', b, 'b')
    _c = nag_double_type_check_and_cast('f08zac', c, 'c')
    _d = nag_double_type_check_and_cast('f08zac', d, 'd')
    _x = nag_double_type_check_and_cast('f08zac', x, 'x')
    f08zac_ctypes(order, m, n, p, _a, pda, _b, pdb, _c, _d, _x, fail)

f08zbc_ctypes = _library_cl.f08zbc
f08zbc_ctypes.restype = None
f08zbc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f08zbc(order, m, n, p, a, pda, b, pdb, d, x, y, fail):
    """
    Solves a real general Gauss--Markov linear model (GLM) problem.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08zbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, d, x, y
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, d, x, y
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08zbc', a, 'a')
    _b = nag_double_type_check_and_cast('f08zbc', b, 'b')
    _d = nag_double_type_check_and_cast('f08zbc', d, 'd')
    _x = nag_double_type_check_and_cast('f08zbc', x, 'x')
    _y = nag_double_type_check_and_cast('f08zbc', y, 'y')
    f08zbc_ctypes(order, m, n, p, _a, pda, _b, pdb, _d, _x, _y, fail)

f08zec_ctypes = _library_cl.f08zec
f08zec_ctypes.restype = None
f08zec_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f08zec(order, n, m, p, a, pda, taua, b, pdb, taub, fail):
    """
    Computes a generalized QR factorization of a real matrix pair.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08zec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, taua, b, taub
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, taua, b, taub
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08zec', a, 'a')
    _taua = nag_double_type_check_and_cast('f08zec', taua, 'taua')
    _b = nag_double_type_check_and_cast('f08zec', b, 'b')
    _taub = nag_double_type_check_and_cast('f08zec', taub, 'taub')
    f08zec_ctypes(order, n, m, p, _a, pda, _taua, _b, pdb, _taub, fail)

f08zfc_ctypes = _library_cl.f08zfc
f08zfc_ctypes.restype = None
f08zfc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f08zfc(order, m, p, n, a, pda, taua, b, pdb, taub, fail):
    """
    Computes a generalized RQ factorization of a real matrix pair.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08zfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, taua, b, taub
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, taua, b, taub
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f08zfc', a, 'a')
    _taua = nag_double_type_check_and_cast('f08zfc', taua, 'taua')
    _b = nag_double_type_check_and_cast('f08zfc', b, 'b')
    _taub = nag_double_type_check_and_cast('f08zfc', taub, 'taub')
    f08zfc_ctypes(order, m, p, n, _a, pda, _taua, _b, pdb, _taub, fail)

f08znc_ctypes = _library_cl.f08znc
f08znc_ctypes.restype = None
f08znc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(Complex), POINTER(Complex), POINTER(NagError)]


def f08znc(order, m, n, p, a, pda, b, pdb, c, d, x, fail):
    """
    Solves the complex linear equality-constrained least squares (LSE)
    problem.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08znc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, c, d, x
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, c, d, x
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08znc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08znc', b, 'b')
    _c = nag_complex_type_check_and_cast('f08znc', c, 'c')
    _d = nag_complex_type_check_and_cast('f08znc', d, 'd')
    _x = nag_complex_type_check_and_cast('f08znc', x, 'x')
    f08znc_ctypes(order, m, n, p, _a, pda, _b, pdb, _c, _d, _x, fail)

f08zpc_ctypes = _library_cl.f08zpc
f08zpc_ctypes.restype = None
f08zpc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(Complex), POINTER(Complex), POINTER(NagError)]


def f08zpc(order, m, n, p, a, pda, b, pdb, d, x, y, fail):
    """
    Solves a complex general Gauss--Markov linear model (GLM) problem.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08zpc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, d, x, y
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, d, x, y
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08zpc', a, 'a')
    _b = nag_complex_type_check_and_cast('f08zpc', b, 'b')
    _d = nag_complex_type_check_and_cast('f08zpc', d, 'd')
    _x = nag_complex_type_check_and_cast('f08zpc', x, 'x')
    _y = nag_complex_type_check_and_cast('f08zpc', y, 'y')
    f08zpc_ctypes(order, m, n, p, _a, pda, _b, pdb, _d, _x, _y, fail)

f08zsc_ctypes = _library_cl.f08zsc
f08zsc_ctypes.restype = None
f08zsc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(Complex), Integer, POINTER(Complex), POINTER(NagError)]


def f08zsc(order, n, m, p, a, pda, taua, b, pdb, taub, fail):
    """
    Computes a generalized QR factorization of a complex matrix pair.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08zsc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, taua, b, taub
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, taua, b, taub
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08zsc', a, 'a')
    _taua = nag_complex_type_check_and_cast('f08zsc', taua, 'taua')
    _b = nag_complex_type_check_and_cast('f08zsc', b, 'b')
    _taub = nag_complex_type_check_and_cast('f08zsc', taub, 'taub')
    f08zsc_ctypes(order, n, m, p, _a, pda, _taua, _b, pdb, _taub, fail)

f08ztc_ctypes = _library_cl.f08ztc
f08ztc_ctypes.restype = None
f08ztc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(Complex), Integer, POINTER(Complex), POINTER(NagError)]


def f08ztc(order, m, p, n, a, pda, taua, b, pdb, taub, fail):
    """
    Computes a generalized RQ factorization of a complex matrix pair.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f08/f08ztc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, taua, b, taub
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, taua, b, taub
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f08ztc', a, 'a')
    _taua = nag_complex_type_check_and_cast('f08ztc', taua, 'taua')
    _b = nag_complex_type_check_and_cast('f08ztc', b, 'b')
    _taub = nag_complex_type_check_and_cast('f08ztc', taub, 'taub')
    f08ztc_ctypes(order, m, p, n, _a, pda, _taua, _b, pdb, _taub, fail)

nag_dgels = f08aac
nag_dgeqrt = f08abc
nag_dgemqrt = f08acc
nag_dgeqrf = f08aec
nag_dorgqr = f08afc
nag_dormqr = f08agc
nag_dgelqf = f08ahc
nag_dorglq = f08ajc
nag_dormlq = f08akc
nag_zgels = f08anc
nag_zgeqrt = f08apc
nag_zgemqrt = f08aqc
nag_zgeqrf = f08asc
nag_zungqr = f08atc
nag_zunmqr = f08auc
nag_zgelqf = f08avc
nag_zunglq = f08awc
nag_zunmlq = f08axc
nag_dgelsy = f08bac
nag_dtpqrt = f08bbc
nag_dtpmqrt = f08bcc
nag_dgeqpf = f08bec
nag_dgeqp3 = f08bfc
nag_dtzrzf = f08bhc
nag_dormrz = f08bkc
nag_zgelsy = f08bnc
nag_ztpqrt = f08bpc
nag_ztpmqrt = f08bqc
nag_zgeqpf = f08bsc
nag_zgeqp3 = f08btc
nag_ztzrzf = f08bvc
nag_zunmrz = f08bxc
nag_dgeqlf = f08cec
nag_dorgql = f08cfc
nag_dormql = f08cgc
nag_dgerqf = f08chc
nag_dorgrq = f08cjc
nag_dormrq = f08ckc
nag_zgeqlf = f08csc
nag_zungql = f08ctc
nag_zunmql = f08cuc
nag_zgerqf = f08cvc
nag_zungrq = f08cwc
nag_zunmrq = f08cxc
nag_dsyev = f08fac
nag_dsyevx = f08fbc
nag_dsyevd = f08fcc
nag_dsyevr = f08fdc
nag_dsytrd = f08fec
nag_dorgtr = f08ffc
nag_dormtr = f08fgc
nag_ddisna = f08flc
nag_zheev = f08fnc
nag_zheevx = f08fpc
nag_zheevd = f08fqc
nag_zheevr = f08frc
nag_zhetrd = f08fsc
nag_zungtr = f08ftc
nag_zunmtr = f08fuc
nag_dspev = f08gac
nag_dspevx = f08gbc
nag_dspevd = f08gcc
nag_dsptrd = f08gec
nag_dopgtr = f08gfc
nag_dopmtr = f08ggc
nag_zhpev = f08gnc
nag_zhpevx = f08gpc
nag_zhpevd = f08gqc
nag_zhptrd = f08gsc
nag_zupgtr = f08gtc
nag_zupmtr = f08guc
nag_dsbev = f08hac
nag_dsbevx = f08hbc
nag_dsbevd = f08hcc
nag_dsbtrd = f08hec
nag_zhbev = f08hnc
nag_zhbevx = f08hpc
nag_zhbevd = f08hqc
nag_zhbtrd = f08hsc
nag_dstev = f08jac
nag_dstevx = f08jbc
nag_dstevd = f08jcc
nag_dstevr = f08jdc
nag_dsteqr = f08jec
nag_dsterf = f08jfc
nag_dpteqr = f08jgc
nag_dstedc = f08jhc
nag_dstebz = f08jjc
nag_dstein = f08jkc
nag_dstegr = f08jlc
nag_zsteqr = f08jsc
nag_zpteqr = f08juc
nag_zstedc = f08jvc
nag_zstein = f08jxc
nag_zstegr = f08jyc
nag_dgelss = f08kac
nag_dgesvd = f08kbc
nag_dgelsd = f08kcc
nag_dgesdd = f08kdc
nag_dgebrd = f08kec
nag_dorgbr = f08kfc
nag_dormbr = f08kgc
nag_dgejsv = f08khc
nag_dgesvj = f08kjc
nag_zgelss = f08knc
nag_zgesvd = f08kpc
nag_zgelsd = f08kqc
nag_zgesdd = f08krc
nag_zgebrd = f08ksc
nag_zungbr = f08ktc
nag_zunmbr = f08kuc
nag_dgbbrd = f08lec
nag_zgbbrd = f08lsc
nag_dbdsdc = f08mdc
nag_dbdsqr = f08mec
nag_zbdsqr = f08msc
nag_dgeev = f08nac
nag_dgeevx = f08nbc
nag_dgehrd = f08nec
nag_dorghr = f08nfc
nag_dormhr = f08ngc
nag_dgebal = f08nhc
nag_dgebak = f08njc
nag_zgeev = f08nnc
nag_zgeevx = f08npc
nag_zgehrd = f08nsc
nag_zunghr = f08ntc
nag_zunmhr = f08nuc
nag_zgebal = f08nvc
nag_zgebak = f08nwc
nag_dgees = f08pac
nag_dgeesx = f08pbc
nag_dhseqr = f08pec
nag_dhsein = f08pkc
nag_zgees = f08pnc
nag_zgeesx = f08ppc
nag_zhseqr = f08psc
nag_zhsein = f08pxc
nag_dtrexc = f08qfc
nag_dtrsen = f08qgc
nag_dtrsyl = f08qhc
nag_dtrevc = f08qkc
nag_dtrsna = f08qlc
nag_ztrexc = f08qtc
nag_ztrsen = f08quc
nag_ztrsyl = f08qvc
nag_ztrevc = f08qxc
nag_ztrsna = f08qyc
nag_dorcsd = f08rac
nag_zuncsd = f08rnc
nag_dsygv = f08sac
nag_dsygvx = f08sbc
nag_dsygvd = f08scc
nag_dsygst = f08sec
nag_zhegv = f08snc
nag_zhegvx = f08spc
nag_zhegvd = f08sqc
nag_zhegst = f08ssc
nag_dspgv = f08tac
nag_dspgvx = f08tbc
nag_dspgvd = f08tcc
nag_dspgst = f08tec
nag_zhpgv = f08tnc
nag_zhpgvx = f08tpc
nag_zhpgvd = f08tqc
nag_zhpgst = f08tsc
nag_dsbgv = f08uac
nag_dsbgvx = f08ubc
nag_dsbgvd = f08ucc
nag_dsbgst = f08uec
nag_dpbstf = f08ufc
nag_zhbgv = f08unc
nag_zhbgvx = f08upc
nag_zhbgvd = f08uqc
nag_zhbgst = f08usc
nag_zpbstf = f08utc
nag_dggsvd = f08vac
nag_dggsvd3 = f08vcc
nag_dggsvp = f08vec
nag_dggsvp3 = f08vgc
nag_zggsvd = f08vnc
nag_zggsvd3 = f08vqc
nag_zggsvp = f08vsc
nag_zggsvp3 = f08vuc
nag_dggev = f08wac
nag_dggevx = f08wbc
nag_dggev3 = f08wcc
nag_dgghrd = f08wec
nag_dgghd3 = f08wfc
nag_dggbal = f08whc
nag_dggbak = f08wjc
nag_zggev = f08wnc
nag_zggevx = f08wpc
nag_zggev3 = f08wqc
nag_zgghrd = f08wsc
nag_zgghd3 = f08wtc
nag_zggbal = f08wvc
nag_zggbak = f08wwc
nag_dgges = f08xac
nag_dggesx = f08xbc
nag_dgges3 = f08xcc
nag_dhgeqz = f08xec
nag_zgges = f08xnc
nag_zggesx = f08xpc
nag_zgges3 = f08xqc
nag_zhgeqz = f08xsc
nag_dtgsja = f08yec
nag_dtgexc = f08yfc
nag_dtgsen = f08ygc
nag_dtgsyl = f08yhc
nag_dtgevc = f08ykc
nag_dtgsna = f08ylc
nag_ztgsja = f08ysc
nag_ztgexc = f08ytc
nag_ztgsen = f08yuc
nag_ztgsyl = f08yvc
nag_ztgevc = f08yxc
nag_ztgsna = f08yyc
nag_dgglse = f08zac
nag_dggglm = f08zbc
nag_dggqrf = f08zec
nag_dggrqf = f08zfc
nag_zgglse = f08znc
nag_zggglm = f08zpc
nag_zggqrf = f08zsc
nag_zggrqf = f08ztc
__all__ = ['f08aac', 'f08abc', 'f08acc', 'f08aec', 'f08afc', 'f08agc',
           'f08ahc', 'f08ajc', 'f08akc', 'f08anc', 'f08apc', 'f08aqc',
           'f08asc', 'f08atc', 'f08auc', 'f08avc', 'f08awc', 'f08axc',
           'f08bac', 'f08bbc', 'f08bcc', 'f08bec', 'f08bfc', 'f08bhc',
           'f08bkc', 'f08bnc', 'f08bpc', 'f08bqc', 'f08bsc', 'f08btc',
           'f08bvc', 'f08bxc', 'f08cec', 'f08cfc', 'f08cgc', 'f08chc',
           'f08cjc', 'f08ckc', 'f08csc', 'f08ctc', 'f08cuc', 'f08cvc',
           'f08cwc', 'f08cxc', 'f08fac', 'f08fbc', 'f08fcc', 'f08fdc',
           'f08fec', 'f08ffc', 'f08fgc', 'f08flc', 'f08fnc', 'f08fpc',
           'f08fqc', 'f08frc', 'f08fsc', 'f08ftc', 'f08fuc', 'f08gac',
           'f08gbc', 'f08gcc', 'f08gec', 'f08gfc', 'f08ggc', 'f08gnc',
           'f08gpc', 'f08gqc', 'f08gsc', 'f08gtc', 'f08guc', 'f08hac',
           'f08hbc', 'f08hcc', 'f08hec', 'f08hnc', 'f08hpc', 'f08hqc',
           'f08hsc', 'f08jac', 'f08jbc', 'f08jcc', 'f08jdc', 'f08jec',
           'f08jfc', 'f08jgc', 'f08jhc', 'f08jjc', 'f08jkc', 'f08jlc',
           'f08jsc', 'f08juc', 'f08jvc', 'f08jxc', 'f08jyc', 'f08kac',
           'f08kbc', 'f08kcc', 'f08kdc', 'f08kec', 'f08kfc', 'f08kgc',
           'f08khc', 'f08kjc', 'f08knc', 'f08kpc', 'f08kqc', 'f08krc',
           'f08ksc', 'f08ktc', 'f08kuc', 'f08lec', 'f08lsc', 'f08mdc',
           'f08mec', 'f08msc', 'f08nac', 'f08nbc', 'f08nec', 'f08nfc',
           'f08ngc', 'f08nhc', 'f08njc', 'f08nnc', 'f08npc', 'f08nsc',
           'f08ntc', 'f08nuc', 'f08nvc', 'f08nwc', 'f08pac', 'f08pbc',
           'f08pec', 'f08pkc', 'f08pnc', 'f08ppc', 'f08psc', 'f08pxc',
           'f08qfc', 'f08qgc', 'f08qhc', 'f08qkc', 'f08qlc', 'f08qtc',
           'f08quc', 'f08qvc', 'f08qxc', 'f08qyc', 'f08rac', 'f08rnc',
           'f08sac', 'f08sbc', 'f08scc', 'f08sec', 'f08snc', 'f08spc',
           'f08sqc', 'f08ssc', 'f08tac', 'f08tbc', 'f08tcc', 'f08tec',
           'f08tnc', 'f08tpc', 'f08tqc', 'f08tsc', 'f08uac', 'f08ubc',
           'f08ucc', 'f08uec', 'f08ufc', 'f08unc', 'f08upc', 'f08uqc',
           'f08usc', 'f08utc', 'f08vac', 'f08vcc', 'f08vec', 'f08vgc',
           'f08vnc', 'f08vqc', 'f08vsc', 'f08vuc',
           'f08wac', 'f08wbc', 'f08wcc', 'f08wec', 'f08wfc', 'f08whc',
           'f08wjc',
           'f08wnc',
           'f08wpc', 'f08wqc', 'f08wsc', 'f08wtc', 'f08wvc', 'f08wwc',
           'f08xac',
           'f08xbc', 'f08xcc',
           'f08xec', 'f08xnc', 'f08xpc', 'f08xqc', 'f08xsc', 'f08yec',
           'f08yfc',
           'f08ygc', 'f08yhc', 'f08ykc', 'f08ylc', 'f08ysc', 'f08ytc',
           'f08yuc', 'f08yvc', 'f08yxc', 'f08yyc', 'f08zac', 'f08zbc',
           'f08zec', 'f08zfc', 'f08znc', 'f08zpc', 'f08zsc', 'f08ztc',
           'nag_dbdsdc', 'nag_dbdsqr', 'nag_ddisna', 'nag_dgbbrd',
           'nag_dgebak', 'nag_dgebal', 'nag_dgebrd', 'nag_dgees',
           'nag_dgeesx', 'nag_dgeev', 'nag_dgeevx', 'nag_dgehrd',
           'nag_dgejsv', 'nag_dgelqf', 'nag_dgels', 'nag_dgelsd',
           'nag_dgelss', 'nag_dgelsy', 'nag_dgemqrt', 'nag_dgeqlf',
           'nag_dgeqp3', 'nag_dgeqpf', 'nag_dgeqrf', 'nag_dgeqrt',
           'nag_dgerqf', 'nag_dgesdd', 'nag_dgesvd', 'nag_dgesvj',
           'nag_dggbak', 'nag_dggbal', 'nag_dgges', 'nag_dggesx',
           'nag_dggev', 'nag_dggevx', 'nag_dggglm', 'nag_dgghrd',
           'nag_dgglse', 'nag_dggqrf', 'nag_dggrqf', 'nag_dggsvd',
           'nag_dggsvp', 'nag_dhgeqz', 'nag_dhsein', 'nag_dhseqr',
           'nag_dopgtr', 'nag_dopmtr', 'nag_dorcsd', 'nag_dorgbr',
           'nag_dorghr', 'nag_dorglq', 'nag_dorgql', 'nag_dorgqr',
           'nag_dorgrq', 'nag_dorgtr', 'nag_dormbr', 'nag_dormhr',
           'nag_dormlq', 'nag_dormql', 'nag_dormqr', 'nag_dormrq',
           'nag_dormrz', 'nag_dormtr', 'nag_dpbstf', 'nag_dpteqr',
           'nag_dsbev', 'nag_dsbevd', 'nag_dsbevx', 'nag_dsbgst',
           'nag_dsbgv', 'nag_dsbgvd', 'nag_dsbgvx', 'nag_dsbtrd',
           'nag_dspev', 'nag_dspevd', 'nag_dspevx', 'nag_dspgst',
           'nag_dspgv', 'nag_dspgvd', 'nag_dspgvx', 'nag_dsptrd',
           'nag_dstebz', 'nag_dstedc', 'nag_dstegr', 'nag_dstein',
           'nag_dsteqr', 'nag_dsterf', 'nag_dstev', 'nag_dstevd',
           'nag_dstevr', 'nag_dstevx', 'nag_dsyev', 'nag_dsyevd',
           'nag_dsyevr', 'nag_dsyevx', 'nag_dsygst', 'nag_dsygv',
           'nag_dsygvd', 'nag_dsygvx', 'nag_dsytrd', 'nag_dtgevc',
           'nag_dtgexc', 'nag_dtgsen', 'nag_dtgsja', 'nag_dtgsna',
           'nag_dtgsyl', 'nag_dtpmqrt', 'nag_dtpqrt', 'nag_dtrevc',
           'nag_dtrexc', 'nag_dtrsen', 'nag_dtrsna', 'nag_dtrsyl',
           'nag_dtzrzf', 'nag_zbdsqr', 'nag_zgbbrd', 'nag_zgebak',
           'nag_zgebal', 'nag_zgebrd', 'nag_zgees', 'nag_zgeesx',
           'nag_zgeev', 'nag_zgeevx', 'nag_zgehrd', 'nag_zgelqf',
           'nag_zgels', 'nag_zgelsd', 'nag_zgelss', 'nag_zgelsy',
           'nag_zgemqrt', 'nag_zgeqlf', 'nag_zgeqp3', 'nag_zgeqpf',
           'nag_zgeqrf', 'nag_zgeqrt', 'nag_zgerqf', 'nag_zgesdd',
           'nag_zgesvd', 'nag_zggbak', 'nag_zggbal', 'nag_zgges',
           'nag_zggesx', 'nag_zggev', 'nag_zggevx', 'nag_zggglm',
           'nag_zgghrd', 'nag_zgglse', 'nag_zggqrf', 'nag_zggrqf',
           'nag_zggsvd', 'nag_zggsvp', 'nag_zhbev', 'nag_zhbevd',
           'nag_zhbevx', 'nag_zhbgst', 'nag_zhbgv', 'nag_zhbgvd',
           'nag_zhbgvx', 'nag_zhbtrd', 'nag_zheev', 'nag_zheevd',
           'nag_zheevr', 'nag_zheevx', 'nag_zhegst', 'nag_zhegv',
           'nag_zhegvd', 'nag_zhegvx', 'nag_zhetrd', 'nag_zhgeqz',
           'nag_zhpev', 'nag_zhpevd', 'nag_zhpevx', 'nag_zhpgst',
           'nag_zhpgv', 'nag_zhpgvd', 'nag_zhpgvx', 'nag_zhptrd',
           'nag_zhsein', 'nag_zhseqr', 'nag_zpbstf', 'nag_zpteqr',
           'nag_zstedc', 'nag_zstegr', 'nag_zstein', 'nag_zsteqr',
           'nag_ztgevc', 'nag_ztgexc', 'nag_ztgsen', 'nag_ztgsja',
           'nag_ztgsna', 'nag_ztgsyl', 'nag_ztpmqrt', 'nag_ztpqrt',
           'nag_ztrevc', 'nag_ztrexc', 'nag_ztrsen', 'nag_ztrsna',
           'nag_ztrsyl', 'nag_ztzrzf', 'nag_zuncsd', 'nag_zungbr',
           'nag_zunghr', 'nag_zunglq', 'nag_zungql', 'nag_zungqr',
           'nag_zungrq', 'nag_zungtr', 'nag_zunmbr', 'nag_zunmhr',
           'nag_zunmlq', 'nag_zunmql', 'nag_zunmqr', 'nag_zunmrq',
           'nag_zunmrz', 'nag_zunmtr', 'nag_zupgtr', 'nag_zupmtr',
           'nag_dggsvd3', 'nag_zggsvd3', 'nag_dggsvp3', 'nag_zggsvp3',
           'nag_dggev3', 'nag_zggev3', 'nag_zgghd3', 'nag_zgges3']
